Package: antedep
Title: Antedependence Models for Longitudinal Data
Version: 0.2.0
Authors@R: c(
    person("Chenyang", "Li", email = "chenyang-li@uiowa.edu", 
           role = c("aut", "cre")),
    person("Dale", "Zimmerman", email = "dale-zimmerman@uiowa.edu", 
           role = c("aut", "ctb"))
    )
Author: Chenyang Li [aut, cre],
  Dale Zimmerman [aut, ctb]
Maintainer: Chenyang Li <chenyang-li@uiowa.edu>
Description: Fitting, simulation, and inference for antedependence models for
    longitudinal data, as described in Zimmerman and Nunez-Anton (2009,
    ISBN:9781420011074). Supports integer-valued antedependence (INAD) models
    for count data with thinning operators (binomial, Poisson, negative binomial)
    and flexible innovation distributions (Poisson, Bell, negative binomial),
    categorical antedependence models for discrete-state longitudinal outcomes,
    and Gaussian antedependence (AD) models for continuous data. Implements
    maximum likelihood estimation via time-separable optimization and block
    coordinate descent, with confidence intervals based on Louis' identity and
    profile likelihood.
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
URL: https://tanchyking.github.io/antedep/,
        https://github.com/TanchyKing/antedep
BugReports: https://github.com/TanchyKing/antedep/issues
Imports: nloptr (>= 1.2.0), stats, ggplot2, MASS
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
Depends: R (>= 3.5)
LazyData: true
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-09 05:30:44 UTC; Tanch
Repository: CRAN
Date/Publication: 2026-05-09 14:30:02 UTC
Built: R 4.5.2; ; 2026-05-09 16:58:30 UTC; unix

Package: antedep
Title: Antedependence Models for Longitudinal Data
Version: 0.2.0
Authors@R: c(
    person("Chenyang", "Li", email = "chenyang-li@uiowa.edu", 
           role = c("aut", "cre")),
    person("Dale", "Zimmerman", email = "dale-zimmerman@uiowa.edu", 
           role = c("aut", "ctb"))
    )
Author: Chenyang Li [aut, cre],
  Dale Zimmerman [aut, ctb]
Maintainer: Chenyang Li <chenyang-li@uiowa.edu>
Description: Fitting, simulation, and inference for antedependence models for
    longitudinal data, as described in Zimmerman and Nunez-Anton (2009,
    ISBN:9781420011074). Supports integer-valued antedependence (INAD) models
    for count data with thinning operators (binomial, Poisson, negative binomial)
    and flexible innovation distributions (Poisson, Bell, negative binomial),
    categorical antedependence models for discrete-state longitudinal outcomes,
    and Gaussian antedependence (AD) models for continuous data. Implements
    maximum likelihood estimation via time-separable optimization and block
    coordinate descent, with confidence intervals based on Louis' identity and
    profile likelihood.
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
URL: https://tanchyking.github.io/antedep/,
        https://github.com/TanchyKing/antedep
BugReports: https://github.com/TanchyKing/antedep/issues
Imports: nloptr (>= 1.2.0), stats, ggplot2, MASS
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
Depends: R (>= 3.5)
LazyData: true
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-09 05:30:44 UTC; Tanch
Repository: CRAN
Date/Publication: 2026-05-09 14:30:02 UTC
Built: R 4.6.0; ; 2026-05-09 18:53:58 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(as.ts,cat_sim)
S3method(as.ts,gau_sim)
S3method(as.ts,inad_sim)
S3method(coef,cat_fit)
S3method(coef,gau_fit)
S3method(coef,inad_fit)
S3method(confint,cat_fit)
S3method(confint,gau_fit)
S3method(confint,inad_fit)
S3method(deviance,cat_fit)
S3method(deviance,gau_fit)
S3method(deviance,inad_fit)
S3method(fitted,gau_fit)
S3method(logLik,cat_fit)
S3method(logLik,gau_fit)
S3method(logLik,inad_fit)
S3method(nobs,cat_fit)
S3method(nobs,gau_fit)
S3method(nobs,inad_fit)
S3method(plot,cat_fit)
S3method(plot,gau_fit)
S3method(plot,inad_fit)
S3method(plot,partial_corr)
S3method(print,bic_order_inad)
S3method(print,cat_ci)
S3method(print,cat_fit)
S3method(print,cat_lrt)
S3method(print,gau_bic_order)
S3method(print,gau_ci)
S3method(print,gau_contrast_test)
S3method(print,gau_fit)
S3method(print,gau_homogeneity_test)
S3method(print,gau_mean_test)
S3method(print,gau_order_test)
S3method(print,homogeneity_tests_inad)
S3method(print,inad_ci)
S3method(print,inad_fit)
S3method(print,partial_corr)
S3method(print,stationarity_tests_gau)
S3method(print,stationarity_tests_inad)
S3method(print,test_homogeneity_inad)
S3method(print,test_order_inad)
S3method(print,test_stationarity_gau)
S3method(print,test_stationarity_inad)
S3method(residuals,gau_fit)
S3method(vcov,gau_fit)
S3method(summary,cat_ci)
S3method(summary,cat_fit)
S3method(summary,gau_ci)
S3method(summary,gau_fit)
S3method(summary,inad_ci)
S3method(summary,inad_fit)
S3method(summary,partial_corr)
export(aic_cat)
export(aic_gau)
export(aic_inad)
export(bic_cat)
export(bic_gau)
export(bic_inad)
export(bic_order_cat)
export(bic_order_gau)
export(bic_order_inad)
export(ci_cat)
export(ci_gau)
export(ci_inad)
export(dbell)
export(em_cat)
export(em_gau)
export(em_inad)
export(fit_cat)
export(fit_gau)
export(fit_inad)
export(logL_cat)
export(logL_gau)
export(logL_inad)
export(logL_inad_i)
export(partial_corr)
export(pbell)
export(plot_prism)
export(plot_profile)
export(qbell)
export(rbell)
export(run_homogeneity_tests_inad)
export(run_order_tests_cat)
export(run_stationarity_tests_cat)
export(run_stationarity_tests_gau)
export(run_stationarity_tests_inad)
export(simulate_cat)
export(simulate_gau)
export(simulate_inad)
export(test_contrast_gau)
export(test_homogeneity_cat)
export(test_homogeneity_gau)
export(test_homogeneity_inad)
export(test_one_sample_gau)
export(test_order_cat)
export(test_order_gau)
export(test_order_inad)
export(test_stationarity_cat)
export(test_stationarity_gau)
export(test_stationarity_inad)
export(test_timeinvariance_cat)
export(test_two_sample_gau)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_errorbar)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(stats,as.ts)
importFrom(stats,complete.cases)
importFrom(stats,confint)
importFrom(stats,deviance)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,dbinom)
importFrom(stats,dnbinom)
importFrom(stats,dnorm)
importFrom(stats,dpois)
importFrom(stats,fitted)
importFrom(stats,logLik)
importFrom(stats,lm)
importFrom(stats,nobs)
importFrom(stats,optim)
importFrom(stats,pchisq)
importFrom(stats,qchisq)
importFrom(stats,qnorm)
importFrom(stats,residuals)
importFrom(stats,rmultinom)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,uniroot)
importFrom(stats,var)
importFrom(stats,vcov)

# antedep

`antedep` fits antedependence models for longitudinal data:

- Gaussian AD models for continuous outcomes
- INAD models for count outcomes
- Categorical AD (CAT) models for discrete state outcomes

## Installation

```r
# install.packages("remotes")
remotes::install_github("TanchyKing/antedep")
```

## Production-Readiness Matrix

| Model | Data type | Complete-data fit/logLik | Missing-data fit/logLik | Notes |
|---|---|---|---|---|
| AD | Continuous | Ready | Ready (`fit_gau`, `logL_gau`) | Missing-data fit uses EM or observed-data likelihood modes |
| INAD | Counts | Ready | Ready (`fit_inad`, `logL_inad`) | Missing-data fit supports `na_action = "marginalize"` |
| CAT | Categorical states | Ready | Ready (`fit_cat`, `logL_cat`) | Missing-data fit supports orders 0, 1, 2 |

## Quick Start

Included datasets:

- `bolus_inad` (morphine bolus counts)
- `cattle_growth` (book companion cattle growth data; continuous)
- `cochlear_implant` (book companion speech recognition/cochlear data; continuous)
- `labor_force_cat` (labor-force categorical table expanded to subject sequences)
- `race_100km` (100km race split times; continuous)

### AD example (continuous)

```r
library(antedep)
set.seed(1)

y <- simulate_gau(n_subjects = 80, n_time = 5, order = 1)
fit <- fit_gau(y, order = 1)
fit$log_l
```

### Missing-data workflow

```r
library(antedep)
set.seed(1)

y <- simulate_inad(n_subjects = 60, n_time = 5, order = 1)
y[sample(length(y), 20)] <- NA

# Fit observed-data likelihood under MAR
fit_miss <- fit_inad(y, order = 1, na_action = "marginalize")
fit_miss$log_l
```

### CAT missing-data workflow

```r
library(antedep)
set.seed(1)

y_cat <- simulate_cat(n_subjects = 80, n_time = 5, order = 1, n_categories = 3)
y_cat[sample(length(y_cat), 30)] <- NA

# Observed-data likelihood (orders 0/1/2)
fit_cat_marg <- fit_cat(y_cat, order = 1, na_action = "marginalize")

# EM (orders 0/1) via explicit entry point or fit_cat dispatch
fit_cat_em1 <- em_cat(y_cat, order = 1, max_iter = 80)
fit_cat_em2 <- fit_cat(y_cat, order = 1, na_action = "em", em_max_iter = 80)
```

If EM becomes unstable or converges slowly, try increasing `epsilon`, increasing
`max_iter`, and using `safeguard = TRUE`.

## Known Limitations

- EM entry points: `em_gau` (Gaussian), `em_inad` (INAD), and `em_cat` (CAT, orders 0/1) are available; for CAT order 2 with missing data, use `fit_cat(na_action = "marginalize")`.
- For CAT models, `fit_cat()` supports both observed-data likelihood (`na_action = "marginalize"`) and EM (`na_action = "em"` for orders 0/1).
- Missing-data confidence intervals are not yet implemented (`ci_gau`, `ci_inad`, `ci_cat` require complete-data fits).
- AD missing-data EM for order 2 is currently available with simplified updates and should be treated as provisional.
- AD missing-data LRT/mean/covariance tests remain complete-data only.
- CAT missing-data stationarity/time-invariance tests (`lrt_stationarity_cat`, `lrt_timeinvariance_cat`, `run_stationarity_tests_cat`) remain complete-data only.
- INAD missing-data LRTs and CAT missing-data order/homogeneity LRTs are supported via observed-data likelihood (`na_action = "marginalize"`).
- Current model order support is up to order 2 for AD, INAD, and CAT.
- CAT missing-data marginalization currently supports orders 0, 1, and 2.

## Vignette

Source vignette: `vignettes/antedep-intro.Rmd`.
Rendered site article: `docs/articles/antedep-intro.html`.
Integrated function reference: `docs/reference/index.html`.

## Function Reference Site (pkgdown)

- Local build: `Rscript -e "pkgdown::build_site(preview = FALSE)"`
- CI deployment: `.github/workflows/pkgdown.yml` (GitHub Pages via Actions)

## Local Check Guide

- See `LOCAL_CHECKS.md` for tarball-mode vs directory-mode checks and expected NOTE messages.

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(antedep)
set.seed(123)

## ----eval=FALSE---------------------------------------------------------------
# data("race_100km")
# dim(race_100km$y)
# colMeans(race_100km$y)

## ----warning=FALSE, message=FALSE, fig.width=10, fig.height=4.5, out.width='100%'----
data("bolus_inad")
plot_profile(
  bolus_inad$y,
  blocks = bolus_inad$blocks,
  block_labels = c("Group 1", "Group 2"),
  title = "Bolus Longitudinal Profiles"
)

## ----eval=FALSE---------------------------------------------------------------
# # More detailed dependence diagnostic
# plot_prism(bolus_inad$y)

## ----warning=FALSE, message=FALSE---------------------------------------------
y_gau <- simulate_gau(n_subjects = 20, n_time = 4, order = 1, phi = 0.5)
fit_gau1 <- fit_gau(y_gau, order = 1)
fit_gau2 <- fit_gau(y_gau, order = 2)

c(order1_logLik = fit_gau1$log_l, order2_logLik = fit_gau2$log_l)
bic_order_gau(y_gau, max_order = 2)$table

## -----------------------------------------------------------------------------
# Example AD order test
test_order_gau(y_gau, p = 0, q = 1)$p_value

## -----------------------------------------------------------------------------
y_gau_miss <- y_gau
y_gau_miss[sample(length(y_gau_miss), 8)] <- NA

fit_gau_em <- fit_gau(y_gau_miss, order = 1, na_action = "em", em_max_iter = 10)
fit_gau_cc <- fit_gau(y_gau_miss, order = 1, na_action = "complete")

c(em_logLik = fit_gau_em$log_l, complete_case_logLik = fit_gau_cc$log_l)
fit_gau_em$pct_missing

## -----------------------------------------------------------------------------
# One-sample and two-sample mean-profile tests
mu0 <- colMeans(y_gau)
blocks_gau <- rep(1:2, each = nrow(y_gau) / 2)

test_one_sample_gau(y_gau, mu0 = mu0, p = 1)$p_value
test_two_sample_gau(y_gau, blocks = blocks_gau, p = 1)$p_value

## -----------------------------------------------------------------------------
# Wald contrast test: H0 that adjacent means are equal
C <- diff(diag(ncol(y_gau)))
test_contrast_gau(y_gau, C = C, p = 1)$p_value

## -----------------------------------------------------------------------------
# Covariance homogeneity across groups
test_homogeneity_gau(y_gau, blocks = blocks_gau, p = 1)$p_value

## -----------------------------------------------------------------------------
y_inad <- simulate_inad(
  n_subjects = 20,
  n_time = 4,
  order = 1,
  thinning = "binom",
  innovation = "pois",
  alpha = 0.4,
  theta = 3
)

fit_inad1 <- fit_inad(y_inad, order = 1, thinning = "binom", innovation = "pois")
fit_inad1$log_l
bic_order_inad(y_inad, max_order = 2, thinning = "binom", innovation = "pois")$table

## -----------------------------------------------------------------------------
y_inad_miss <- y_inad
y_inad_miss[sample(length(y_inad_miss), 10)] <- NA

fit_inad_miss <- fit_inad(
  y_inad_miss,
  order = 1,
  thinning = "binom",
  innovation = "pois",
  na_action = "marginalize",
  max_iter = 10
)

fit_inad_miss$log_l
fit_inad_miss$pct_missing

## -----------------------------------------------------------------------------
# Missing-data LRTs (observed-data likelihood)
blocks_inad <- rep(1:2, each = nrow(y_inad_miss) / 2)
test_order_inad(y_inad_miss, order_null = 0, order_alt = 1, blocks = blocks_inad)$p_value
test_homogeneity_inad(y_inad_miss, blocks = blocks_inad, order = 1, test = "mean")$p_value

## -----------------------------------------------------------------------------
y_cat <- simulate_cat(n_subjects = 20, n_time = 4, order = 1, n_categories = 3)
fit_cat1 <- fit_cat(y_cat, order = 1)
fit_cat1$log_l

# Wald CIs (complete data)
ci_cat1 <- ci_cat(fit_cat1, parameters = "marginal")
head(ci_cat1$marginal, 3)

## -----------------------------------------------------------------------------
# Order tests for CAT
run_order_tests_cat(y_cat, max_order = 2)$table

## -----------------------------------------------------------------------------
y_cat_miss <- y_cat
y_cat_miss[sample(length(y_cat_miss), 12)] <- NA

fit_cat_miss <- fit_cat(y_cat_miss, order = 1, na_action = "marginalize")
fit_cat_miss$log_l
fit_cat_miss$settings$na_action_effective

## -----------------------------------------------------------------------------
# EM alternative for CAT missing data (orders 0/1)
fit_cat_miss_em <- fit_cat(y_cat_miss, order = 1, na_action = "em", em_max_iter = 10)
fit_cat_miss_em$log_l

## -----------------------------------------------------------------------------
# Missing-data order/homogeneity tests are supported
blocks_cat <- rep(1:2, each = nrow(y_cat_miss) / 2)
test_order_cat(y_cat_miss, order_null = 0, order_alt = 1)$p_value
test_homogeneity_cat(y_cat_miss, blocks = blocks_cat, order = 1)$p_value

## ----eval=FALSE---------------------------------------------------------------
# # INAD homogeneity/stationarity tools
# blocks <- rep(1:2, each = nrow(y_inad) / 2)
# run_homogeneity_tests_inad(y_inad, blocks = blocks, order = 1)
# run_stationarity_tests_inad(y_inad, order = 1, blocks = blocks)
# 
# # CAT stationarity/time-invariance tools
# test_timeinvariance_cat(y_cat, order = 1)
# test_stationarity_cat(y_cat, order = 1)

## ----eval=FALSE---------------------------------------------------------------
# Rscript scripts/check-package.R --as-cran --no-manual

#' Confidence intervals for fitted INAD models
#'
#' Computes confidence intervals for selected parameters from a fitted INAD model.
#' For the fixed effect case, Wald intervals for time varying alpha and theta are
#' computed via Louis identity for supported thinning-innovation combinations.
#' For block effects tau, profile likelihood intervals are computed by fixing one
#' component of tau and re maximizing the log likelihood over nuisance parameters.
#' For negative binomial innovations, Wald intervals for the innovation size
#' parameter are computed using a one dimensional observed information approximation
#' per time point, holding other parameters fixed at their fitted values.
#'
#' @param y Integer matrix with \code{n_subjects} rows and \code{n_time} columns.
#' @param fit A fitted model object returned by \code{\link{fit_inad}}.
#' @param blocks Optional integer vector of length \code{n_subjects}. Required for
#'   block effect intervals. If provided, should match \code{fit$settings$blocks}.
#' @param level Confidence level between 0 and 1.
#' @param idx_time Optional integer vector of time indices for which to compute
#'   intervals. Default is all time points.
#' @param ridge Nonnegative ridge value added to the observed information matrix
#'   used for Louis based Wald intervals.
#' @param profile_maxeval Maximum number of function evaluations used in the
#'   profile likelihood refits.
#' @param profile_xtol_rel Relative tolerance used in the profile likelihood refits.
#'
#' @return An object of class \code{inad_ci}, a list with elements
#'   \code{settings}, \code{level}, \code{alpha}, \code{theta}, \code{nb_inno_size},
#'   and \code{tau}. Each non NULL interval element is a data frame with columns
#'   \code{param}, \code{est}, \code{lower}, \code{upper}, and possibly \code{se}
#'   and \code{width}.
#'
#' @examples
#' \donttest{
#' data("bolus_inad", package = "antedep")
#' y <- bolus_inad$y
#' blocks <- bolus_inad$blocks
#' fit <- fit_inad(y, order = 1, thinning = "binom", innovation = "bell", blocks = blocks)
#' ci <- ci_inad(y, fit, blocks = blocks)
#' ci$alpha
#' ci$theta
#' ci$tau
#' }
#'
#' @export
ci_inad <- function(
        y,
        fit,
        blocks = NULL,
        level = 0.95,
        idx_time = NULL,
        ridge = 0,
        profile_maxeval = 2500,
        profile_xtol_rel = 1e-6
) {
    if (!is.matrix(y)) y <- as.matrix(y)
    if (anyNA(y)) {
        stop(
            "ci_inad currently supports complete data only. Missing-data INAD confidence intervals are not implemented yet.",
            call. = FALSE
        )
    }
    if (any(!is.finite(y)) || any(y < 0) || any(y != floor(y))) stop("y must be a nonnegative integer matrix")

    if (is.null(fit$settings$order)) stop("fit$settings$order is missing")
    if (is.null(fit$settings$thinning)) stop("fit$settings$thinning is missing")
    if (is.null(fit$settings$innovation)) stop("fit$settings$innovation is missing")
    if (is.null(fit$theta)) stop("fit$theta is missing")
    if (is.null(fit$log_l) || !is.finite(fit$log_l)) stop("fit$log_l must be finite")
    if (!is.null(fit$settings$na_action) && identical(fit$settings$na_action, "marginalize")) {
        stop(
            "ci_inad currently supports complete-data fits only. Missing-data INAD confidence intervals are not implemented yet.",
            call. = FALSE
        )
    }

    ord <- fit$settings$order
    thinning <- fit$settings$thinning
    innovation <- fit$settings$innovation

    n <- nrow(y)
    N <- ncol(y)

    if (is.null(idx_time)) idx_time <- seq_len(N)
    idx_time <- as.integer(idx_time)
    if (any(idx_time < 1) || any(idx_time > N)) stop("idx_time out of range")

    has_blocks <- !is.null(blocks) && !is.null(fit$tau) && length(fit$tau) > 0L
    if (has_blocks) {
        blocks <- as.integer(blocks)
        if (length(blocks) != n) stop("blocks must have length nrow(y)")
        if (any(blocks < 1)) stop("blocks must be positive integers starting at 1")
    }

    out <- list(settings = fit$settings, level = level)

    # Compute alpha and theta CIs using Louis identity for supported combinations
    if (has_blocks && ord %in% c(1, 2)) {
        tmp <- .ci_alpha_theta_louis_fe(
            y = y,
            fit = fit,
            blocks = blocks,
            level = level,
            idx_time = idx_time,
            ridge = ridge,
            thinning = thinning,
            innovation = innovation
        )
        out$alpha <- tmp$alpha
        out$theta <- tmp$theta
    } else {
        out$alpha <- NULL
        out$theta <- NULL
    }

    if (innovation == "nbinom") {
        out$nb_inno_size <- .ci_nb_inno_size_obsinfo_1d(
            y = y,
            fit = fit,
            blocks = if (has_blocks) blocks else NULL,
            level = level,
            idx_time = idx_time
        )
    } else {
        out$nb_inno_size <- NULL
    }

    if (has_blocks) {
        out$tau <- .ci_tau_profile_inad(
            y = y,
            fit = fit,
            blocks = blocks,
            level = level,
            maxeval = profile_maxeval,
            xtol_rel = profile_xtol_rel
        )
    } else {
        out$tau <- NULL
    }

    class(out) <- "inad_ci"
    out
}

#' Print method for INAD confidence intervals
#'
#' @param x An object of class \code{inad_ci}.
#' @param ... Unused.
#'
#' @return The input object, invisibly.
#'
#' @export
print.inad_ci <- function(x, ...) {
    cat("CI level:", x$level, "\n")
    if (!is.null(x$alpha)) cat("alpha rows:", nrow(x$alpha), "\n")
    if (!is.null(x$theta)) cat("theta rows:", nrow(x$theta), "\n")
    if (!is.null(x$nb_inno_size)) cat("nb_inno_size rows:", nrow(x$nb_inno_size), "\n")
    if (!is.null(x$tau)) cat("tau rows:", nrow(x$tau), "\n")
    invisible(x)
}

#' Summary method for inad_ci objects
#'
#' @param object An \code{inad_ci} object.
#' @param ... Unused.
#'
#' @return A data frame stacking available confidence intervals with columns
#'   \code{param}, \code{component}, \code{est}, \code{se}, \code{lower},
#'   \code{upper}, and \code{level}. Returns \code{NULL} if no intervals are available.
#' @export
summary.inad_ci <- function(object, ...) {
    parts <- list()

    add_part <- function(df, component) {
        if (is.null(df) || !is.data.frame(df) || nrow(df) == 0L) return(NULL)
        df$component <- component
        keep <- intersect(
            c("param", "component", "est", "se", "lower", "upper", "level"),
            names(df)
        )
        df[, keep, drop = FALSE]
    }

    parts[[length(parts) + 1L]] <- add_part(object$alpha, "alpha")
    parts[[length(parts) + 1L]] <- add_part(object$theta, "theta")
    parts[[length(parts) + 1L]] <- add_part(object$nb_inno_size, "nb_inno_size")
    parts[[length(parts) + 1L]] <- add_part(object$tau, "tau")
    parts <- Filter(Negate(is.null), parts)

    if (length(parts) == 0L) return(NULL)
    do.call(rbind, parts)
}

#' @keywords internal
.lse <- function(v) {
    m <- max(v)
    m + log(sum(exp(v - m)))
}

#' Posterior computations for all thinning-innovation combinations
#'
#' Computes conditional expectations and variances needed for Louis' identity.
#' See Appendix C of the paper for the derivation.
#'
#' @keywords internal
.poster_general <- function(m, n, alpha, lambda, thinning, innovation, nb_inno_size = NULL) {
    alpha <- max(alpha, 1e-8)
    lambda <- max(lambda, 1e-8)

    # Compute posterior weights
    k <- 0:n
    u <- n - k  # innovation values

    # Thinning log-probabilities
    if (thinning == "binom") {
        alpha <- min(alpha, 1 - 1e-8)
        lthin <- stats::dbinom(k, size = m, prob = alpha, log = TRUE)
    } else if (thinning == "pois") {
        lthin <- stats::dpois(k, lambda = alpha * m, log = TRUE)
    } else {  # nbinom
        p_nb <- 1 / (1 + alpha)
        lthin <- stats::dnbinom(k, size = m, prob = p_nb, log = TRUE)
    }

    # Innovation log-probabilities
    if (innovation == "pois") {
        linno <- stats::dpois(u, lambda = lambda, log = TRUE)
    } else if (innovation == "bell") {
        linno <- sapply(u, function(z) log(dbell(z, theta = lambda)))
    } else {  # nbinom
        if (is.null(nb_inno_size)) nb_inno_size <- 1
        linno <- stats::dnbinom(u, size = nb_inno_size, mu = lambda, log = TRUE)
    }

    lw <- lthin + linno
    lZ <- .lse(lw)
    w <- exp(lw - lZ)

    # Score functions for thinning parameter alpha
    # See Appendix A.1 for the conditional distributions
    if (thinning == "binom") {
        # Binomial thinning: k | Y ~ Bin(Y, alpha)
        # Score: k/alpha - (m-k)/(1-alpha)
        score_alpha <- k / alpha - (m - k) / (1 - alpha)
        # Second derivative (negative Hessian): k/alpha^2 + (m-k)/(1-alpha)^2
        h_alpha <- k / (alpha^2) + (m - k) / ((1 - alpha)^2)
    } else if (thinning == "pois") {
        # Poisson thinning: k | Y ~ Pois(alpha * Y)
        # Score: k/alpha - m
        score_alpha <- k / alpha - m
        # Second derivative: k/alpha^2
        h_alpha <- k / (alpha^2)
    } else {  # nbinom
        # NB thinning: k | Y ~ NB(Y, 1/(1+alpha))
        # p = 1/(1+alpha), so dp/dalpha = -1/(1+alpha)^2 = -p^2
        # Score and Hessian for NB(size=m, prob=p) w.r.t. alpha
        p <- 1 / (1 + alpha)
        q <- 1 - p
        score_alpha <- -m * p + k * (p^2 / q)
        h_alpha <- -m * p^2 + k * (p^3 * (2 - p) / (q^2))
    }

    # Score functions for innovation parameter lambda (theta)
    if (innovation == "pois") {
        # Poisson: score = u/lambda - 1
        score_lambda <- u / lambda - 1
        h_lambda <- u / (lambda^2)
    } else if (innovation == "bell") {
        # Bell: score = u/lambda - exp(lambda)
        score_lambda <- u / lambda - exp(lambda)
        h_lambda <- u / (lambda^2) + exp(lambda)
    } else {  # nbinom
        # NB(size=r, mu=lambda): score = u/lambda - (r+u)/(r+lambda)
        r <- if (is.null(nb_inno_size)) 1 else nb_inno_size
        score_lambda <- u / lambda - (r + u) / (r + lambda)
        h_lambda <- u / (lambda^2) - (r + u) / ((r + lambda)^2)
    }

    # Compute expectations
    E_sA <- sum(w * score_alpha)
    E_hA <- sum(w * h_alpha)
    E_sL <- sum(w * score_lambda)
    E_hL <- sum(w * h_lambda)

    E_sA2 <- sum(w * (score_alpha^2))
    E_sL2 <- sum(w * (score_lambda^2))
    E_sAsL <- sum(w * (score_alpha * score_lambda))

    Var_sA <- max(0, E_sA2 - E_sA^2)
    Var_sL <- max(0, E_sL2 - E_sL^2)
    Cov_sA_sL <- E_sAsL - E_sA * E_sL

    c(
        E_hA = E_hA,
        Var_sA = Var_sA,
        E_hL = E_hL,
        Var_sL = Var_sL,
        Cov_sA_sL = Cov_sA_sL
    )
}

#' Louis' identity observed information at time i
#'
#' @keywords internal
.louis_info_i_fe <- function(y, i, alpha_hat, theta_hat, tau_hat, blocks,
                              thinning, innovation, nb_inno_size = NULL) {
    n <- nrow(y)

    if (i == 1) {
        # At i=1, only innovation contributes
        Itt <- 0
        for (s in 1:n) {
            b <- blocks[s]
            z <- y[s, 1]
            if (innovation == "bell") {
                mu <- .bell_mean_from_theta(theta_hat) + tau_hat[b]
                la <- max(.bell_theta_from_mean(mu), 1e-8)
            } else {
                la <- max(theta_hat + tau_hat[b], 1e-8)
            }

            if (innovation == "pois") {
                Itt <- Itt + z / (la^2)
            } else if (innovation == "bell") {
                Itt <- Itt + z / (la^2) + exp(la)
            } else {  # nbinom
                r <- if (is.null(nb_inno_size)) 1 else nb_inno_size[1]
                Itt <- Itt + z / (la^2) - (r + z) / ((r + la)^2)
            }
        }
        mat <- matrix(Itt, 1, 1)
        dimnames(mat) <- list("theta[1]", "theta[1]")
        return(mat)
    }

    m <- y[, i - 1]
    nvec <- y[, i]

    a <- max(alpha_hat, 1e-8)
    th <- max(theta_hat, 1e-8)

    E_hA_sum <- 0
    Var_sA_sum <- 0
    E_hL_theta <- 0
    Var_sL_theta <- 0
    Cov_theta <- 0

    nb_sz <- if (innovation == "nbinom" && !is.null(nb_inno_size)) nb_inno_size[i] else NULL

    for (s in 1:n) {
        b <- blocks[s]
        if (innovation == "bell") {
            mu <- .bell_mean_from_theta(th) + tau_hat[b]
            la <- max(.bell_theta_from_mean(mu), 1e-8)
        } else {
            la <- max(th + tau_hat[b], 1e-8)
        }
        acc <- .poster_general(m = m[s], n = nvec[s], alpha = a, lambda = la,
                               thinning = thinning, innovation = innovation,
                               nb_inno_size = nb_sz)

        E_hA_sum <- E_hA_sum + acc["E_hA"]
        Var_sA_sum <- Var_sA_sum + acc["Var_sA"]
        E_hL_theta <- E_hL_theta + acc["E_hL"]
        Var_sL_theta <- Var_sL_theta + acc["Var_sL"]
        Cov_theta <- Cov_theta + acc["Cov_sA_sL"]
    }

    # Louis' identity: I_obs = E[B|Y] - Var[S|Y]
    Iaa <- E_hA_sum - Var_sA_sum
    Itt <- E_hL_theta - Var_sL_theta
    Iat <- -Cov_theta

    mat <- matrix(c(Iaa, Iat, Iat, Itt), 2, 2, byrow = TRUE)
    par_names <- c(paste0("alpha[", i, "]"), paste0("theta[", i, "]"))
    dimnames(mat) <- list(par_names, par_names)
    mat
}

#' Wald CI at time i using Louis' identity
#'
#' @keywords internal
.ci_wald_i_louis_fe <- function(y, i, alpha_hat, theta_hat, tau_hat, blocks,
                                 thinning, innovation, nb_inno_size = NULL,
                                 level = 0.95, ridge = 0) {
    Iobs <- .louis_info_i_fe(y, i, alpha_hat, theta_hat, tau_hat, blocks,
                              thinning, innovation, nb_inno_size)
    if (ridge > 0) Iobs <- Iobs + diag(ridge, nrow(Iobs))

    V <- tryCatch(solve(Iobs), error = function(e) NULL)
    if (is.null(V)) stop("Singular information at i = ", i, ".")

    z <- qnorm(1 - (1 - level) / 2)

    if (i == 1) {
        est <- c(theta_hat)
        nms <- c("theta[1]")
    } else {
        est <- c(alpha_hat, theta_hat)
        nms <- c(paste0("alpha[", i, "]"), paste0("theta[", i, "]"))
    }

    se <- sqrt(pmax(diag(V), 0))
    L <- est - z * se
    U <- est + z * se

    if (i != 1) {
        L[1] <- max(L[1], 0)
        U[1] <- max(U[1], 0)
    }

    data.frame(
        param = nms,
        est = est,
        se = se,
        lower = L,
        upper = U,
        width = U - L,
        level = level,
        row.names = NULL
    )
}

#' Compute alpha and theta CIs using Louis' identity (general version)
#'
#' @keywords internal
.ci_alpha_theta_louis_fe <- function(y, fit, blocks, level, idx_time, ridge,
                                      thinning, innovation) {
    alpha_rows <- list()
    theta_rows <- list()

    nb_inno_size <- fit$nb_inno_size

    for (i in idx_time) {
        if (i == 1) {
            tmp <- .ci_wald_i_louis_fe(
                y = y,
                i = 1,
                alpha_hat = NA,
                theta_hat = fit$theta[1],
                tau_hat = fit$tau,
                blocks = blocks,
                thinning = thinning,
                innovation = innovation,
                nb_inno_size = nb_inno_size,
                level = level,
                ridge = ridge
            )
            theta_rows[[length(theta_rows) + 1L]] <- tmp
        } else {
            alpha_i <- if (is.matrix(fit$alpha)) fit$alpha[i, 1] else fit$alpha[i]
            tmp <- .ci_wald_i_louis_fe(
                y = y,
                i = i,
                alpha_hat = alpha_i,
                theta_hat = fit$theta[i],
                tau_hat = fit$tau,
                blocks = blocks,
                thinning = thinning,
                innovation = innovation,
                nb_inno_size = nb_inno_size,
                level = level,
                ridge = ridge
            )
            alpha_rows[[length(alpha_rows) + 1L]] <- tmp[tmp$param == paste0("alpha[", i, "]"), , drop = FALSE]
            theta_rows[[length(theta_rows) + 1L]] <- tmp[tmp$param == paste0("theta[", i, "]"), , drop = FALSE]
        }
    }

    list(
        alpha = if (length(alpha_rows) > 0) do.call(rbind, alpha_rows) else NULL,
        theta = if (length(theta_rows) > 0) do.call(rbind, theta_rows) else NULL
    )
}

#' @keywords internal
.ci_nb_inno_size_obsinfo_1d <- function(y, fit, blocks, level, idx_time) {
    eps_pos <- 1e-10
    z <- qnorm(1 - (1 - level) / 2)

    if (is.null(fit$nb_inno_size)) return(NULL)

    nb <- as.numeric(fit$nb_inno_size)
    if (length(nb) == 1L) nb <- rep(nb, ncol(y))
    if (length(nb) != ncol(y)) stop("fit$nb_inno_size must be length 1 or ncol(y)")

    ord <- fit$settings$order
    thinning <- fit$settings$thinning
    innovation <- fit$settings$innovation

    if (innovation != "nbinom") return(NULL)

    res <- vector("list", length(idx_time))

    for (jj in seq_along(idx_time)) {
        i <- idx_time[jj]
        s0 <- max(nb[i], eps_pos)

        ll_at <- function(sz) {
            sz <- max(sz, eps_pos)
            nb_try <- nb
            nb_try[i] <- sz

            ll <- tryCatch(
                logL_inad(
                    y = y,
                    order = ord,
                    thinning = thinning,
                    innovation = innovation,
                    alpha = fit$alpha,
                    theta = fit$theta,
                    nb_inno_size = nb_try,
                    blocks = blocks,
                    tau = if (!is.null(blocks)) fit$tau else NULL
                ),
                error = function(e) NA_real_
            )
            ll
        }

        h0 <- max(1e-4, 1e-3 * s0)
        f0 <- ll_at(s0)
        d2 <- NA_real_

        for (hfac in c(1, 0.1, 0.01)) {
            h <- h0 * hfac
            fp <- ll_at(s0 + h)
            fm <- ll_at(max(eps_pos, s0 - h))
            if (is.finite(f0) && is.finite(fp) && is.finite(fm)) {
                d2 <- (fp - 2 * f0 + fm) / (h^2)
                break
            }
        }

        if (is.na(d2)) {
            res[[jj]] <- data.frame(
                param = paste0("nb_inno_size[", i, "]"),
                est = s0,
                se = NA_real_,
                lower = NA_real_,
                upper = NA_real_,
                width = NA_real_,
                level = level,
                row.names = NULL
            )
            next
        }
        I <- max(0, -d2)

        if (I <= 0 || !is.finite(I)) {
            se <- NA_real_
            L <- NA_real_
            U <- NA_real_
        } else {
            se <- sqrt(1 / I)
            L <- max(0, s0 - z * se)
            U <- max(0, s0 + z * se)
        }

        res[[jj]] <- data.frame(
            param = paste0("nb_inno_size[", i, "]"),
            est = s0,
            se = se,
            lower = L,
            upper = U,
            width = if (is.finite(L) && is.finite(U)) (U - L) else NA_real_,
            level = level,
            row.names = NULL
        )
    }

    do.call(rbind, res)
}

#' @keywords internal
.ci_tau_profile_inad <- function(
        y,
        fit,
        blocks,
        level,
        maxeval,
        xtol_rel,
        expand = 2,
        max_bracket_iter = 50
) {
    if (!requireNamespace("nloptr", quietly = TRUE)) stop("nloptr is required for tau profiling")

    if (is.null(fit$tau) || length(fit$tau) < 2) stop("fit$tau must exist with at least two blocks")
    if (is.null(fit$alpha)) stop("fit$alpha is missing for profiling refits")
    if (is.null(fit$theta)) stop("fit$theta is missing for profiling refits")

    n <- nrow(y)
    N <- ncol(y)
    B <- max(blocks)

    if (length(fit$tau) != B) stop("length(fit$tau) must equal max(blocks)")
    if (length(fit$theta) != N) stop("length(fit$theta) must equal ncol(y)")

    ord <- fit$settings$order
    thinning <- fit$settings$thinning
    innovation <- fit$settings$innovation

    eps_pos <- 1e-10
    eps_alpha <- 1e-10
    alpha_ub <- 1 - eps_alpha
    alpha_share_tol <- 1e-6

    # If order-1 alpha is supplied as time-constant, profile tau under the same
    # shared-alpha nuisance structure (rather than a fully time-varying alpha).
    alpha_profile_shared <- FALSE
    if (ord == 1 && N >= 2 && length(fit$alpha) == N) {
        a <- as.numeric(fit$alpha)
        if (all(is.finite(a[2:N])) && max(abs(a[2:N] - a[2])) <= alpha_share_tol) {
            alpha_profile_shared <- TRUE
        }
    }

    lambda_ok <- function(theta_vec, tau_vec) {
        mu <- if (innovation == "bell") {
            outer(rep(1, n), .bell_mean_from_theta(theta_vec)) + matrix(tau_vec[blocks], n, N)
        } else {
            outer(rep(1, n), theta_vec) + matrix(tau_vec[blocks], n, N)
        }
        all(is.finite(mu)) && all(mu > eps_pos)
    }

    pack_par <- function(alpha, theta, tau, nb_inno_size, ord, B, N, innovation,
                         tau_fix_idx, alpha_profile_shared = FALSE) {
        out <- numeric(0)
        tau_free_idx <- 2:B
        if (length(tau_fix_idx) > 0) tau_free_idx <- setdiff(tau_free_idx, tau_fix_idx)
        if (length(tau_free_idx) > 0) out <- c(out, tau[tau_free_idx])

        if (ord == 1) {
            if (N >= 2) {
                if (alpha_profile_shared) {
                    out <- c(out, alpha[2])
                } else {
                    out <- c(out, alpha[2:N])
                }
            }
        }
        if (ord == 2) {
            if (N >= 2) out <- c(out, alpha[2:N, 1])
            if (N >= 3) out <- c(out, alpha[3:N, 2])
        }

        out <- c(out, theta)

        # nb_inno_size is NOT included: held fixed at its full-model MLE during
        # profile refits (Variant 1 / constrained-fit paradigm).
        out
    }

    unpack_par <- function(par, alpha0, theta0, tau0, nb0, ord, B, N, innovation,
                           tau_fix_idx, tau_fix_val, alpha_profile_shared = FALSE) {
        k <- 0

        tau <- tau0
        tau[1] <- 0
        tau_free_idx <- 2:B
        if (length(tau_fix_idx) > 0) tau_free_idx <- setdiff(tau_free_idx, tau_fix_idx)
        if (length(tau_free_idx) > 0) {
            tau[tau_free_idx] <- par[(k + 1):(k + length(tau_free_idx))]
            k <- k + length(tau_free_idx)
        }
        if (length(tau_fix_idx) > 0) {
            tau[tau_fix_idx] <- tau_fix_val
            tau[1] <- 0
        }

        alpha <- alpha0
        if (ord == 1) {
            if (N >= 2) {
                if (alpha_profile_shared) {
                    alpha[2:N] <- par[k + 1]
                    k <- k + 1
                } else {
                    alpha[2:N] <- par[(k + 1):(k + (N - 1))]
                    k <- k + (N - 1)
                }
            }
            alpha[1] <- 0
            alpha <- pmin(pmax(alpha, 0), alpha_ub)
            alpha[1] <- 0
        }
        if (ord == 2) {
            if (N >= 2) {
                alpha[2:N, 1] <- par[(k + 1):(k + (N - 1))]
                k <- k + (N - 1)
            }
            if (N >= 3) {
                alpha[3:N, 2] <- par[(k + 1):(k + (N - 2))]
                k <- k + (N - 2)
            }
            alpha[1, 1] <- 0
            alpha[1, 2] <- 0
            if (N >= 2) alpha[2, 2] <- 0
            alpha <- pmin(pmax(alpha, 0), alpha_ub)
            alpha[1, 1] <- 0
            alpha[1, 2] <- 0
            if (N >= 2) alpha[2, 2] <- 0
        }

        theta <- par[(k + 1):(k + N)]
        k <- k + N
        theta <- pmax(theta, eps_pos)

        # nb_inno_size is held fixed at its full-model MLE (nb0) throughout the
        # profile refit.  Re-optimising it as a nuisance parameter (Variant 2)
        # would widen the CI inconsistently with the constrained-fit paradigm
        # used everywhere else in the package and can yield intervals that
        # contradict the LRT p-value.
        nb_inno_size <- nb0

        list(alpha = alpha, theta = theta, tau = tau, nb_inno_size = nb_inno_size)
    }

    refit_tau_fixed <- function(tau_fix_idx, tau_fix_val) {
        alpha0 <- fit$alpha
        theta0 <- pmax(as.numeric(fit$theta), eps_pos)

        tau0 <- as.numeric(fit$tau)
        tau0[1] <- 0

        nb0 <- fit$nb_inno_size
        if (innovation == "nbinom") {
            if (is.null(nb0)) nb0 <- rep(1, N)
            nb0 <- as.numeric(nb0)
            if (length(nb0) == 1L) nb0 <- rep(nb0, N)
            if (length(nb0) != N) stop("fit$nb_inno_size must be length 1 or ncol(y)")
            nb0 <- pmax(nb0, eps_pos)
        } else {
            nb0 <- NULL
        }

        par0 <- pack_par(alpha0, theta0, tau0, nb0, ord, B, N, innovation,
                         tau_fix_idx, alpha_profile_shared = alpha_profile_shared)

        obj <- function(par) {
            cur <- unpack_par(
                par = par,
                alpha0 = alpha0,
                theta0 = theta0,
                tau0 = tau0,
                nb0 = nb0,
                ord = ord,
                B = B,
                N = N,
                innovation = innovation,
                tau_fix_idx = tau_fix_idx,
                tau_fix_val = tau_fix_val,
                alpha_profile_shared = alpha_profile_shared
            )
            if (!lambda_ok(cur$theta, cur$tau)) return(1e8)

            ll <- tryCatch(
                logL_inad(
                    y = y,
                    order = ord,
                    thinning = thinning,
                    innovation = innovation,
                    alpha = cur$alpha,
                    theta = cur$theta,
                    nb_inno_size = cur$nb_inno_size,
                    blocks = blocks,
                    tau = cur$tau
                ),
                error = function(e) NA_real_
            )
            if (!is.finite(ll)) return(1e8)
            -ll
        }

        opt <- nloptr::nloptr(
            x0 = par0,
            eval_f = obj,
            opts = list(
                algorithm = "NLOPT_LN_BOBYQA",
                xtol_rel = xtol_rel,
                maxeval = maxeval
            )
        )

        -opt$objective
    }

    q_chi_1 <- qchisq(level, df = 1)
    prof_target <- function(b, tval) {
        ll_prof <- refit_tau_fixed(tau_fix_idx = b, tau_fix_val = tval)
        2 * (fit$log_l - ll_prof) - q_chi_1
    }

    bracket_one_side <- function(b, direction, step0) {
        tau_mle <- fit$tau[b]
        lower_bound <- if (innovation == "bell") {
            -min(.bell_mean_from_theta(fit$theta))
        } else {
            -min(fit$theta)
        }

        x1 <- tau_mle
        step <- step0 * direction

        for (k in 1:max_bracket_iter) {
            x2 <- x1 + step
            # Enforce only the physical lower bound (innovation mean must be
            # positive); no artificial upper cap so the search is free to expand
            # as far as the profile likelihood requires.
            x2 <- max(lower_bound, x2)
            f2 <- prof_target(b, x2)
            if (is.finite(f2) && f2 >= 0) return(sort(c(x1, x2)))
            if (x2 == lower_bound) break   # hit hard physical constraint
            x1 <- x2
            step <- step * expand
        }

        NULL
    }

    tau_ci <- vector("list", length = B - 1)
    for (b in 2:B) {
        tau_mle <- fit$tau[b]
        lower_bound <- if (innovation == "bell") {
            -min(.bell_mean_from_theta(fit$theta))
        } else {
            -min(fit$theta)
        }
        # Initial step: 20 % of |tau_mle| (min 0.1).  With expand = 2 and
        # max_bracket_iter = 50 the bracket can reach tau_mle ± step0*(2^50-1),
        # far beyond any realistic CI bound.
        step0 <- max(0.1, abs(tau_mle) * 0.2)
        if (!is.finite(step0) || step0 <= 0) step0 <- 0.1

        f_mle <- prof_target(b, tau_mle)
        if (!is.finite(f_mle) || f_mle > 1e-6) stop("Profile target at MLE not near zero for tau[", b, "]")

        left_int <- bracket_one_side(b, direction = -1, step0 = step0)
        right_int <- bracket_one_side(b, direction = 1, step0 = step0)

        left <- NA_real_
        right <- NA_real_

        if (!is.null(left_int)) left <- uniroot(function(x) prof_target(b, x), interval = left_int)$root
        if (!is.null(right_int)) right <- uniroot(function(x) prof_target(b, x), interval = right_int)$root

        # Approximate SE from observed profile curvature at tau_mle.
        # Fall back to CI-width-based approximation if curvature is unavailable.
        se <- NA_real_
        h <- max(1e-4, min(0.1, abs(tau_mle) * 0.05))
        if (tau_mle - h > lower_bound + 1e-10) {
            ll_m <- refit_tau_fixed(tau_fix_idx = b, tau_fix_val = tau_mle - h)
            ll_0 <- fit$log_l - (f_mle + q_chi_1) / 2
            ll_p <- refit_tau_fixed(tau_fix_idx = b, tau_fix_val = tau_mle + h)
            if (is.finite(ll_m) && is.finite(ll_0) && is.finite(ll_p)) {
                d2 <- (ll_p - 2 * ll_0 + ll_m) / (h^2)
                info <- -d2
                if (is.finite(info) && info > 0) se <- sqrt(1 / info)
            }
        }
        if (!is.finite(se) && is.finite(left) && is.finite(right)) {
            z <- qnorm(1 - (1 - level) / 2)
            if (is.finite(z) && z > 0) se <- (right - left) / (2 * z)
        }

        tau_ci[[b - 1]] <- data.frame(
            param = paste0("tau[", b, "]"),
            est = tau_mle,
            se = se,
            lower = left,
            upper = right,
            width = if (is.finite(left) && is.finite(right)) right - left else NA_real_,
            level = level,
            row.names = NULL
        )
    }

    do.call(rbind, tau_ci)
}

test_that("ci_inad behaves correctly on bolus_inad", {
    skip_on_cran()
    skip_if_not_installed("nloptr")
    data(bolus_inad, package = "antedep")

    y <- bolus_inad$y
    blocks <- if (!is.null(bolus_inad$blocks)) bolus_inad$blocks else bolus_inad$bolus

    fit <- fit_inad(
        y = y,
        order = 1,
        thinning = "nbinom",
        innovation = "bell",
        blocks = blocks,
        max_iter = 25,
        tol = 1e-6,
        verbose = FALSE
    )

    ci <- ci_inad(
        y = y,
        fit = fit,
        blocks = blocks,
        level = 0.95,
        profile_maxeval = 1200,
        profile_xtol_rel = 1e-6
    )

    expect_s3_class(ci, "inad_ci")

    ## alpha and theta: either NULL or valid data.frame
    if (!is.null(ci$alpha)) {
        expect_true(is.data.frame(ci$alpha))
        expect_true(all(c("param", "est", "se", "lower", "upper", "width", "level") %in% names(ci$alpha)))
        expect_true(all(ci$alpha$lower >= 0))
        expect_true(all(ci$alpha$lower <= ci$alpha$upper))
    }

    if (!is.null(ci$theta)) {
        expect_true(is.data.frame(ci$theta))
        expect_true(all(c("param", "est", "se", "lower", "upper", "width", "level") %in% names(ci$theta)))
        expect_true(all(ci$theta$lower <= ci$theta$upper))
    }

    ## tau: profile CI, may be NULL if blocks degenerate
    if (!is.null(ci$tau)) {
        expect_true(is.data.frame(ci$tau))
        expect_true(all(c("param", "est", "se", "lower", "upper", "width", "level") %in% names(ci$tau)))
        finite_bounds <- is.finite(ci$tau$lower) & is.finite(ci$tau$upper)
        if (any(finite_bounds)) {
            expect_true(all(ci$tau$lower[finite_bounds] <= ci$tau$upper[finite_bounds]))
            expect_true(all(ci$tau$lower[finite_bounds] <= ci$tau$est[finite_bounds]))
            expect_true(all(ci$tau$est[finite_bounds] <= ci$tau$upper[finite_bounds]))
        }
    }
})

test_that("ci_inad never errors for fitted models", {
    skip_on_cran()
    data(bolus_inad, package = "antedep")

    fit <- fit_inad(
        y = bolus_inad$y,
        order = 1,
        thinning = "nbinom",
        innovation = "bell",
        blocks = bolus_inad$bolus
    )

    expect_silent(
        ci_inad(y = bolus_inad$y, fit = fit, blocks = bolus_inad$bolus)
    )
})

# test_order_cat.R - Likelihood ratio tests for order in categorical AD models

.cat_fit_uses_missing_likelihood <- function(fit) {
  na_effective <- fit$settings$na_action_effective
  na_raw <- fit$settings$na_action
  identical(na_effective, "marginalize") ||
    identical(na_raw, "marginalize") ||
    is.null(fit$cell_counts)
}

.stop_cat_missing_inference <- function(fn_name) {
  stop(
    paste0(
      fn_name,
      " currently supports complete data only. Missing-data CAT likelihood-ratio inference is not implemented yet."
    ),
    call. = FALSE
  )
}

#' Likelihood ratio test for antedependence order (categorical AD data)
#'
#' Tests whether a higher-order AD model provides significantly better fit
#' than a lower-order model for categorical longitudinal data.
#'
#' @param y Integer matrix with n_subjects rows and n_time columns. Each entry
#'   should be a category code from 1 to c. Can be NULL if both fit_null and
#'   fit_alt are provided.
#' @param order_null Order under the null hypothesis (default 0).
#' @param order_alt Order under the alternative hypothesis (default 1).
#'   Must be greater than order_null.
#' @param blocks Optional integer vector of length n_subjects specifying group
#'   membership.
#' @param homogeneous Logical. If TRUE (default), parameters are shared across
#'   all groups.
#' @param n_categories Number of categories. If NULL, inferred from data.
#' @param fit_null Optional pre-fitted model under null hypothesis (class "cat_fit").
#'   If provided, y is not required for fitting under H0.
#' @param fit_alt Optional pre-fitted model under alternative hypothesis.
#'   If provided, y is not required for fitting under H1.
#' @param test Type of test statistic. One of \code{"lrt"} (default),
#'   \code{"score"}, \code{"mlrt"}, or \code{"wald"}.
#'
#' @return A list of class \code{"cat_lrt"} containing:
#' \describe{
#'   \item{method}{Inference method used: one of \code{"lrt"}, \code{"score"},
#'     \code{"mlrt"}, or \code{"wald"}.}
#'   \item{lrt_stat}{Likelihood ratio test statistic}
#'   \item{df}{Degrees of freedom}
#'   \item{p_value}{P-value from chi-square distribution}
#'   \item{fit_null}{Fitted model under H0}
#'   \item{fit_alt}{Fitted model under H1}
#'   \item{order_null}{Order under null}
#'   \item{order_alt}{Order under alternative}
#'   \item{table}{Summary data frame}
#' }
#'
#' @details
#' The likelihood ratio test statistic is:
#' \deqn{\lambda = -2[\ell_0 - \ell_1]}
#' where \eqn{\ell_0} and \eqn{\ell_1} are the maximized log-likelihoods under
#' the null and alternative hypotheses.
#'
#' Under H0, \eqn{\lambda} follows a chi-square distribution with degrees of
#' freedom equal to the difference in the number of free parameters.
#'
#' For testing AD(p) vs AD(p+1), the degrees of freedom are:
#' \deqn{df = (c-1)^2 \times c^p \times (n - p - 1)}
#' where c is the number of categories and n is the number of time points.
#'
#' If \code{y} contains missing values and models are fit internally, this
#' function defaults to \code{na_action = "marginalize"} for fitting.
#' Score- and Wald-based variants currently require complete data.
#'
#' @examples
#' \donttest{
#' # Simulate AD(1) data
#' set.seed(123)
#' y <- simulate_cat(200, 6, order = 1, n_categories = 2)
#'
#' # Test AD(0) vs AD(1)
#' test_01 <- test_order_cat(y, order_null = 0, order_alt = 1)
#' print(test_01$table)
#'
#' # Test AD(1) vs AD(2)
#' test_12 <- test_order_cat(y, order_null = 1, order_alt = 2)
#' print(test_12$table)
#'
#' # Using pre-fitted models
#' fit0 <- fit_cat(y, order = 0)
#' fit1 <- fit_cat(y, order = 1)
#' test_prefitted <- test_order_cat(fit_null = fit0, fit_alt = fit1)
#' }
#'
#' @references
#' Xie, Y. and Zimmerman, D. L. (2013). Antedependence models for nonstationary
#' categorical longitudinal data with ignorable missingness: likelihood-based
#' inference. \emph{Statistics in Medicine}, 32, 3274-3289.
#'
#' @seealso \code{\link{fit_cat}}, \code{\link{bic_order_cat}}
#'
#' @export
test_order_cat <- function(y = NULL, order_null = 0, order_alt = 1,
                          blocks = NULL, homogeneous = TRUE, n_categories = NULL,
                          fit_null = NULL, fit_alt = NULL,
                          test = c("lrt", "score", "mlrt", "wald")) {
  test <- match.arg(test)
  
 # Validate that we have either y or pre-fitted models
  if (is.null(y) && (is.null(fit_null) || is.null(fit_alt))) {
    stop("Either y must be provided, or both fit_null and fit_alt must be provided")
  }
  use_missing <- !is.null(y) && anyNA(y)
  na_action_fit <- if (use_missing) "marginalize" else "fail"
  
  # Validate orders
  if (order_alt <= order_null) {
    stop("order_alt must be greater than order_null")
  }
  if (order_null < 0) {
    stop("order_null must be non-negative")
  }
  if (order_alt > 2) {
    stop("order_alt > 2 not currently supported")
  }
  
  # Fit models if not provided
  if (is.null(fit_null)) {
    fit_null <- fit_cat(y, order = order_null, blocks = blocks,
                        homogeneous = homogeneous, n_categories = n_categories,
                        na_action = na_action_fit)
  } else {
    if (!inherits(fit_null, "cat_fit")) {
      stop("fit_null must be a cat_fit object")
    }
    order_null <- fit_null$settings$order
  }
  
  if (is.null(fit_alt)) {
    fit_alt <- fit_cat(y, order = order_alt, blocks = blocks,
                       homogeneous = homogeneous, n_categories = n_categories,
                       na_action = na_action_fit)
  } else {
    if (!inherits(fit_alt, "cat_fit")) {
      stop("fit_alt must be a cat_fit object")
    }
    order_alt <- fit_alt$settings$order
  }
  
  # Verify order relationship from fitted models
  if (fit_alt$settings$order <= fit_null$settings$order) {
    stop("Alternative model order must be greater than null model order")
  }
  
  # Extract info
  log_l_null <- fit_null$log_l
  log_l_alt <- fit_alt$log_l
  n_params_null <- fit_null$n_params
  n_params_alt <- fit_alt$n_params
  
  # Compute LRT statistic
  lrt_stat_raw <- -2 * (log_l_null - log_l_alt)
  
  # Degrees of freedom
  df <- n_params_alt - n_params_null

  # Select test statistic
  stat_value <- lrt_stat_raw
  e_hat_mlrt <- NA_real_
  if (identical(test, "score")) {
    if (is.null(y)) {
      stop("y must be provided when test = 'score'")
    }
    if (anyNA(y) || .cat_fit_uses_missing_likelihood(fit_null) || .cat_fit_uses_missing_likelihood(fit_alt)) {
      .stop_cat_missing_inference("test_order_cat(test = 'score')")
    }

    c <- fit_null$settings$n_categories
    pops <- .cat_split_populations(y, blocks = blocks, fit = fit_null)
    stat_value <- 0
    for (pop in pops) {
      stat_value <- stat_value + .cat_score_order_single(
        y = pop$y,
        p = fit_null$settings$order,
        q = fit_alt$settings$order,
        c = c,
        marginal = pop$marginal,
        transition = pop$transition
      )
    }
  } else if (identical(test, "mlrt")) {
    e_hat_mlrt <- .cat_mlrt_order_e(fit_null = fit_null, fit_alt = fit_alt)
    if (!is.finite(e_hat_mlrt) || e_hat_mlrt <= 0) {
      stop("Modified LRT scaling factor could not be computed (non-positive or non-finite)")
    }
    stat_value <- lrt_stat_raw * df / e_hat_mlrt
  } else if (identical(test, "wald")) {
    if (is.null(y)) {
      stop("y must be provided when test = 'wald'")
    }
    if (anyNA(y) || .cat_fit_uses_missing_likelihood(fit_null) || .cat_fit_uses_missing_likelihood(fit_alt)) {
      .stop_cat_missing_inference("test_order_cat(test = 'wald')")
    }
    if (!identical(fit_null$settings$homogeneous, fit_alt$settings$homogeneous)) {
      stop("fit_null and fit_alt must both be homogeneous or both heterogeneous for Wald order test")
    }

    c <- fit_null$settings$n_categories
    pops_null <- .cat_split_populations(y, blocks = blocks, fit = fit_null)
    pops_alt <- .cat_split_populations(y, blocks = blocks, fit = fit_alt)
    if (length(pops_null) != length(pops_alt)) {
      stop("Population structures in fit_null and fit_alt do not match for Wald test")
    }

    stat_value <- 0
    for (i in seq_along(pops_null)) {
      stat_value <- stat_value + .cat_wald_order_single(
        y = pops_null[[i]]$y,
        p = fit_null$settings$order,
        q = fit_alt$settings$order,
        c = c,
        marginal_null = pops_null[[i]]$marginal,
        transition_null = pops_null[[i]]$transition,
        marginal_alt = pops_alt[[i]]$marginal,
        transition_alt = pops_alt[[i]]$transition
      )
    }
  }
  
  # P-value from chi-square
  p_value <- stats::pchisq(stat_value, df = df, lower.tail = FALSE)
  
  # Build summary table
  table_df <- data.frame(
    model = c("H0 (null)", "H1 (alt)"),
    order = c(fit_null$settings$order, fit_alt$settings$order),
    log_l = c(log_l_null, log_l_alt),
    n_params = c(n_params_null, n_params_alt),
    aic = c(fit_null$aic, fit_alt$aic),
    bic = c(fit_null$bic, fit_alt$bic)
  )
  
  # Assemble output
  out <- list(
    method = test,
    lrt_stat = stat_value,
    statistic = stat_value,
    lrt_stat_raw = lrt_stat_raw,
    e_hat_mlrt = e_hat_mlrt,
    df = df,
    p_value = p_value,
    test = test,
    fit_null = fit_null,
    fit_alt = fit_alt,
    order_null = fit_null$settings$order,
    order_alt = fit_alt$settings$order,
    table = table_df
  )
  
  class(out) <- "cat_lrt"
  out
}


#' Print method for cat_lrt objects
#'
#' @param x A cat_lrt object
#' @param ... Additional arguments (ignored)
#' @return Invisibly returns \code{x}.
#' @export
print.cat_lrt <- function(x, ...) {
  test_name <- x$test
  if (is.null(test_name)) {
    test_name <- "lrt"
  }
  test_label <- switch(
    test_name,
    lrt = "Likelihood Ratio",
    score = "Score (Pearson)",
    mlrt = "Modified LRT",
    wald = "Wald",
    "Likelihood Ratio"
  )
  cat(test_label, "Test for Categorical AD\n")
  cat("===============================\n\n")

  if (!is.null(x$order_null) && !is.null(x$order_alt)) {
    cat("H0: AD(", x$order_null, ")\n", sep = "")
    cat("H1: AD(", x$order_alt, ")\n\n", sep = "")
  } else if (!is.null(x$order)) {
    cat("Order:", x$order, "\n\n")
  }
  
  cat("Test statistic:", round(x$lrt_stat, 4), "\n")
  cat("Degrees of freedom:", x$df, "\n")
  cat("P-value:", format.pval(x$p_value, digits = 4), "\n\n")
  
  if (!is.null(x$table)) {
    cat("Model comparison:\n")
    print(x$table, row.names = FALSE)
  }
  
  invisible(x)
}


#' Run all pairwise order tests
#'
#' Performs sequential likelihood ratio tests for AD orders 0 vs 1, 1 vs 2, etc.
#'
#' @param y Integer matrix of categorical data (n_subjects x n_time).
#' @param max_order Maximum order to test. Default is 2.
#' @param blocks Optional block membership vector.
#' @param homogeneous Whether to use homogeneous parameters across blocks.
#' @param n_categories Number of categories (inferred if NULL).
#' @param test Type of test statistic for each pairwise comparison. One of
#'   \code{"lrt"} (default), \code{"score"}, \code{"mlrt"}, or \code{"wald"}.
#'   Passed to \code{\link{test_order_cat}}.
#'
#' @return A list containing:
#'   \item{tests}{List of test_order_cat results for each comparison}
#'   \item{table}{Summary data frame with all comparisons}
#'   \item{fits}{List of all fitted models}
#'   \item{selected_order}{Recommended order based on sequential testing at alpha=0.05}
#'
#' @details
#' This function performs forward selection: starting from order 0, it tests
#' whether increasing the order provides significant improvement. The selected
#' order is the highest order where the test was significant (at alpha = 0.05).
#'
#' @examples
#' \donttest{
#' y <- simulate_cat(200, 6, order = 1, n_categories = 2)
#' result <- run_order_tests_cat(y, max_order = 2)
#' print(result$table)
#' cat("Selected order:", result$selected_order, "\n")
#' }
#'
#' @export
run_order_tests_cat <- function(y, max_order = 2, blocks = NULL,
                                 homogeneous = TRUE, n_categories = NULL,
                                 test = c("lrt", "score", "mlrt", "wald")) {
  test <- match.arg(test)
  use_missing <- anyNA(y)
  
  # Validate data
  validated <- .validate_y_cat(y, n_categories, allow_na = use_missing)
  y <- validated$y
  n_categories <- validated$n_categories
  n_time <- ncol(y)
  
  # Adjust max_order if needed
  max_possible <- n_time - 1
  if (max_order > max_possible) {
    warning("max_order reduced to ", max_possible, " (n_time - 1)")
    max_order <- max_possible
  }
  if (max_order > 2) {
    warning("max_order > 2 not supported; setting to 2")
    max_order <- 2
  }
  
  # Fit all models first
  orders <- 0:max_order
  fits <- vector("list", length(orders))
  names(fits) <- paste0("order_", orders)
  
  for (i in seq_along(orders)) {
    fits[[i]] <- fit_cat(y, order = orders[i], blocks = blocks,
                         homogeneous = homogeneous, n_categories = n_categories,
                         na_action = if (use_missing) "marginalize" else "fail")
  }
  
  # Run pairwise tests
  n_tests <- max_order
  tests <- vector("list", n_tests)
  names(tests) <- paste0("order_", 0:(max_order-1), "_vs_", 1:max_order)
  
  test_results <- data.frame(
    comparison = character(n_tests),
    order_null = integer(n_tests),
    order_alt = integer(n_tests),
    method = character(n_tests),
    lrt_stat = numeric(n_tests),
    df = integer(n_tests),
    p_value = numeric(n_tests),
    significant = logical(n_tests),
    stringsAsFactors = FALSE
  )
  
  for (i in seq_len(n_tests)) {
    p_null <- orders[i]
    p_alt <- orders[i + 1]

    tests[[i]] <- test_order_cat(
      y = if (identical(test, "lrt")) NULL else y,
      blocks = if (identical(test, "lrt")) NULL else blocks,
      homogeneous = homogeneous,
      n_categories = n_categories,
      fit_null = fits[[i]],
      fit_alt = fits[[i + 1]],
      test = test
    )
    
    test_results$comparison[i] <- paste0("AD(", p_null, ") vs AD(", p_alt, ")")
    test_results$order_null[i] <- p_null
    test_results$order_alt[i] <- p_alt
    test_results$method[i] <- tests[[i]]$method
    test_results$lrt_stat[i] <- tests[[i]]$lrt_stat
    test_results$df[i] <- tests[[i]]$df
    test_results$p_value[i] <- tests[[i]]$p_value
    test_results$significant[i] <- tests[[i]]$p_value < 0.05
  }
  
  # Determine selected order via forward selection
  # Start at order 0, increase while tests are significant
  selected_order <- 0
  for (i in seq_len(n_tests)) {
    if (test_results$significant[i]) {
      selected_order <- test_results$order_alt[i]
    } else {
      break
    }
  }
  
  list(
    method = test,
    tests = tests,
    table = test_results,
    fits = fits,
    selected_order = selected_order
  )
}

# test-lrt_cat.R - Tests for CAT hypothesis testing functions

# ============================================================
# Test test_order_cat
# ============================================================
test_that("test_order_cat works for AD(0) vs AD(1)", {
  skip_on_cran()
  set.seed(123)
  # Simulate AD(1) data - should reject AD(0)
  marg <- list(t1 = c(0.6, 0.4))
  trans <- list(
    t2 = matrix(c(0.8, 0.2, 0.3, 0.7), 2, byrow = TRUE),
    t3 = matrix(c(0.8, 0.2, 0.3, 0.7), 2, byrow = TRUE),
    t4 = matrix(c(0.8, 0.2, 0.3, 0.7), 2, byrow = TRUE)
  )
  y <- simulate_cat(100, 4, order = 1, n_categories = 2,
                    marginal = marg, transition = trans)

  test <- test_order_cat(y, order_null = 0, order_alt = 1)
  
  expect_s3_class(test, "cat_lrt")
  expect_true(test$lrt_stat >= 0)
  expect_true(test$df > 0)
  expect_true(test$p_value >= 0 && test$p_value <= 1)
  expect_equal(test$order_null, 0)
  expect_equal(test$order_alt, 1)
  
  # For data with true AD(1), should reject AD(0)
  expect_true(test$p_value < 0.05)
})


test_that("test_order_cat works for AD(1) vs AD(2)", {
  skip_on_cran()
  set.seed(456)
  # Simulate AD(1) data - should NOT reject AD(1) in favor of AD(2)
  y <- simulate_cat(80, 5, order = 1, n_categories = 2)

  test <- test_order_cat(y, order_null = 1, order_alt = 2)
  
  expect_s3_class(test, "cat_lrt")
  expect_true(test$lrt_stat >= 0)
  expect_equal(test$order_null, 1)
  expect_equal(test$order_alt, 2)
  
  # For data with true AD(1), should usually NOT reject AD(1)
  # (but with random data this isn't guaranteed, so just check it runs)
})

test_that("test_order_cat supports score and mlrt options", {
  skip_on_cran()
  set.seed(460)
  y <- simulate_cat(100, 5, order = 1, n_categories = 2)

  score_test <- test_order_cat(y, order_null = 0, order_alt = 1, test = "score")
  expect_s3_class(score_test, "cat_lrt")
  expect_equal(score_test$test, "score")
  expect_true(score_test$lrt_stat >= 0)
  expect_true(score_test$p_value >= 0 && score_test$p_value <= 1)

  mlrt_test <- test_order_cat(y, order_null = 0, order_alt = 1, test = "mlrt")
  expect_s3_class(mlrt_test, "cat_lrt")
  expect_equal(mlrt_test$test, "mlrt")
  expect_true(is.finite(mlrt_test$e_hat_mlrt))
  expect_true(mlrt_test$e_hat_mlrt > 0)
  expect_true(mlrt_test$lrt_stat >= 0)
  expect_true(mlrt_test$p_value >= 0 && mlrt_test$p_value <= 1)

  lrt_test <- test_order_cat(y, order_null = 0, order_alt = 1, test = "lrt")
  expect_false(isTRUE(all.equal(mlrt_test$lrt_stat, lrt_test$lrt_stat, tolerance = 1e-12)))
})

test_that("test_order_cat supports wald option", {
  skip_on_cran()
  set.seed(461)
  y <- simulate_cat(100, 5, order = 1, n_categories = 2)

  wald_test <- test_order_cat(y, order_null = 0, order_alt = 1, test = "wald")
  expect_s3_class(wald_test, "cat_lrt")
  expect_equal(wald_test$test, "wald")
  expect_true(is.finite(wald_test$lrt_stat))
  expect_true(wald_test$lrt_stat >= 0)
  expect_true(wald_test$p_value >= 0 && wald_test$p_value <= 1)
})


test_that("test_order_cat works with pre-fitted models", {
  skip_on_cran()
  set.seed(789)
  y <- simulate_cat(60, 4, order = 1, n_categories = 2)
  
  fit0 <- fit_cat(y, order = 0)
  fit1 <- fit_cat(y, order = 1)
  
  # Test with pre-fitted models
  test <- test_order_cat(fit_null = fit0, fit_alt = fit1)
  
  expect_s3_class(test, "cat_lrt")
  expect_equal(test$order_null, 0)
  expect_equal(test$order_alt, 1)
  
  # Test with y + one pre-fitted model
  test2 <- test_order_cat(y, order_null = 0, fit_alt = fit1)
  expect_equal(test$lrt_stat, test2$lrt_stat, tolerance = 1e-10)
})


test_that("test_order_cat validates inputs correctly", {
  skip_on_cran()
  y <- simulate_cat(50, 4, order = 1, n_categories = 2)
  
  # Error: order_alt <= order_null
  expect_error(test_order_cat(y, order_null = 1, order_alt = 0))
  expect_error(test_order_cat(y, order_null = 1, order_alt = 1))
  
  # Error: negative order
  expect_error(test_order_cat(y, order_null = -1, order_alt = 1))
  
  # Error: order too high
  expect_error(test_order_cat(y, order_null = 2, order_alt = 3))
  
  # Error: no data and no pre-fitted models
  expect_error(test_order_cat(order_null = 0, order_alt = 1))
})


test_that("test_order_cat df calculation is correct", {
  skip_on_cran()
  set.seed(111)
  y <- simulate_cat(100, 5, order = 1, n_categories = 2)
  
  # AD(0) vs AD(1) with c=2, n=5
  # df = (c-1) * c^0 * (n-1) = 1 * 1 * 4 = 4
  test_01 <- test_order_cat(y, order_null = 0, order_alt = 1)
  expect_equal(test_01$df, 4)
  
  # AD(1) vs AD(2) with c=2, n=5
  # df = (c-1) * c^1 * (n-2) = 1 * 2 * 3 = 6
  test_12 <- test_order_cat(y, order_null = 1, order_alt = 2)
  expect_equal(test_12$df, 6)
})


test_that("run_order_tests_cat works correctly", {
  skip_on_cran()
  set.seed(222)
  # Simulate AD(1) data
  marg <- list(t1 = c(0.6, 0.4))
  trans <- list(
    t2 = matrix(c(0.85, 0.15, 0.25, 0.75), 2, byrow = TRUE),
    t3 = matrix(c(0.85, 0.15, 0.25, 0.75), 2, byrow = TRUE),
    t4 = matrix(c(0.85, 0.15, 0.25, 0.75), 2, byrow = TRUE),
    t5 = matrix(c(0.85, 0.15, 0.25, 0.75), 2, byrow = TRUE)
  )
  y <- simulate_cat(120, 5, order = 1, n_categories = 2,
                    marginal = marg, transition = trans)

  result <- run_order_tests_cat(y, max_order = 2)
  
  expect_true(is.list(result))
  expect_true("tests" %in% names(result))
  expect_true("table" %in% names(result))
  expect_true("fits" %in% names(result))
  expect_true("selected_order" %in% names(result))
  
  expect_equal(nrow(result$table), 2)  # Two tests: 0v1, 1v2
  expect_true(result$selected_order %in% 0:2)
  
  # For true AD(1), should select order 1
  # (0v1 significant, 1v2 not significant)
  expect_equal(result$selected_order, 1)
})


# ============================================================
# Test test_homogeneity_cat
# ============================================================
test_that("test_homogeneity_cat detects heterogeneity", {
  skip_on_cran()
  set.seed(333)
  # Create two groups with different transition probabilities
  marg1 <- list(t1 = c(0.8, 0.2))
  marg2 <- list(t1 = c(0.3, 0.7))
  trans1 <- list(
    t2 = matrix(c(0.9, 0.1, 0.1, 0.9), 2, byrow = TRUE),
    t3 = matrix(c(0.9, 0.1, 0.1, 0.9), 2, byrow = TRUE)
  )
  trans2 <- list(
    t2 = matrix(c(0.4, 0.6, 0.6, 0.4), 2, byrow = TRUE),
    t3 = matrix(c(0.4, 0.6, 0.6, 0.4), 2, byrow = TRUE)
  )
  
  y1 <- simulate_cat(80, 3, order = 1, n_categories = 2,
                     marginal = marg1, transition = trans1)
  y2 <- simulate_cat(80, 3, order = 1, n_categories = 2,
                     marginal = marg2, transition = trans2)
  y <- rbind(y1, y2)
  blocks <- c(rep(1, 80), rep(2, 80))
  
  test <- test_homogeneity_cat(y, blocks, order = 1)
  
  expect_s3_class(test, "cat_lrt")
  expect_true(test$lrt_stat >= 0)
  expect_true(test$df > 0)
  expect_equal(test$n_groups, 2)
  
  # Should reject homogeneity
  expect_true(test$p_value < 0.05)
})


test_that("test_homogeneity_cat accepts homogeneous data", {
  skip_on_cran()
  set.seed(444)
  # Create two groups with SAME parameters
  y1 <- simulate_cat(50, 3, order = 1, n_categories = 2)
  y2 <- simulate_cat(50, 3, order = 1, n_categories = 2)
  y <- rbind(y1, y2)
  blocks <- c(rep(1, 50), rep(2, 50))
  
  test <- test_homogeneity_cat(y, blocks, order = 1)
  
  expect_s3_class(test, "cat_lrt")
  
  # Should NOT reject homogeneity (p > 0.05 usually, but not guaranteed)
  # Just check that test runs and gives reasonable output
  expect_true(test$p_value >= 0 && test$p_value <= 1)
})

test_that("test_homogeneity_cat supports score option", {
  skip_on_cran()
  set.seed(445)
  y1 <- simulate_cat(50, 4, order = 1, n_categories = 2)
  y2 <- simulate_cat(50, 4, order = 1, n_categories = 2)
  y <- rbind(y1, y2)
  blocks <- c(rep(1, 50), rep(2, 50))

  score_test <- test_homogeneity_cat(y, blocks, order = 1, test = "score")
  expect_s3_class(score_test, "cat_lrt")
  expect_equal(score_test$test, "score")
  expect_true(score_test$lrt_stat >= 0)
  expect_true(score_test$p_value >= 0 && score_test$p_value <= 1)
})

test_that("test_homogeneity_cat supports mlrt option", {
  skip_on_cran()
  old_opt <- options(antedep.cat_mlrt_nsim = 20L, antedep.cat_mlrt_seed = 101L)
  on.exit(options(old_opt), add = TRUE)

  set.seed(446)
  y1 <- simulate_cat(50, 4, order = 1, n_categories = 2)
  y2 <- simulate_cat(50, 4, order = 1, n_categories = 2)
  y <- rbind(y1, y2)
  blocks <- c(rep(1, 50), rep(2, 50))

  mlrt_test <- test_homogeneity_cat(y, blocks, order = 1, test = "mlrt")
  expect_s3_class(mlrt_test, "cat_lrt")
  expect_equal(mlrt_test$test, "mlrt")
  expect_true(is.finite(mlrt_test$e_hat_mlrt))
  expect_true(mlrt_test$e_hat_mlrt > 0)
  expect_true(mlrt_test$lrt_stat >= 0)
})


test_that("test_homogeneity_cat works with pre-fitted models", {
  skip_on_cran()
  set.seed(555)
  y1 <- simulate_cat(50, 3, order = 1, n_categories = 2)
  y2 <- simulate_cat(50, 3, order = 1, n_categories = 2)
  y <- rbind(y1, y2)
  blocks <- c(rep(1, 50), rep(2, 50))
  
  fit_homo <- fit_cat(y, order = 1, blocks = blocks, homogeneous = TRUE)
  fit_hetero <- fit_cat(y, order = 1, blocks = blocks, homogeneous = FALSE)
  
  test <- test_homogeneity_cat(fit_null = fit_homo, fit_alt = fit_hetero)
  
  expect_s3_class(test, "cat_lrt")
  expect_equal(test$n_groups, 2)
})


test_that("test_homogeneity_cat validates inputs", {
  skip_on_cran()
  y <- simulate_cat(50, 3, order = 1, n_categories = 2)
  
  # Error: no blocks provided
  expect_error(test_homogeneity_cat(y, blocks = NULL))
  
  # Error: no y and no pre-fitted models
  expect_error(test_homogeneity_cat(blocks = c(1, 1, 2, 2)))
})


# ============================================================
# Test test_timeinvariance_cat
# ============================================================
test_that("test_timeinvariance_cat works", {
  skip_on_cran()
  set.seed(666)
  # Simulate with time-invariant transitions (default)
  y <- simulate_cat(80, 5, order = 1, n_categories = 2)
  
  test <- test_timeinvariance_cat(y, order = 1)
  
  expect_s3_class(test, "cat_lrt")
  expect_true(test$lrt_stat >= 0)
  expect_true(test$df > 0)
  expect_equal(test$order, 1)
  
  # For time-invariant data, should NOT reject time-invariance
  # (but this is statistical, so just check it runs)
  expect_true(test$p_value >= 0 && test$p_value <= 1)
})


test_that("test_timeinvariance_cat detects time-varying transitions", {
  skip_on_cran()
  set.seed(777)
  # Simulate with time-VARYING transitions
  marg <- list(t1 = c(0.5, 0.5))
  trans_varying <- list(
    t2 = matrix(c(0.9, 0.1, 0.1, 0.9), 2, byrow = TRUE),
    t3 = matrix(c(0.5, 0.5, 0.5, 0.5), 2, byrow = TRUE),
    t4 = matrix(c(0.2, 0.8, 0.8, 0.2), 2, byrow = TRUE),
    t5 = matrix(c(0.7, 0.3, 0.3, 0.7), 2, byrow = TRUE)
  )
  
  y <- simulate_cat(200, 5, order = 1, n_categories = 2,
                    marginal = marg, transition = trans_varying)

  test <- test_timeinvariance_cat(y, order = 1)
  
  expect_s3_class(test, "cat_lrt")
  
  # Should reject time-invariance
  expect_true(test$p_value < 0.05)
})

test_that("test_timeinvariance_cat supports score option", {
  skip_on_cran()
  set.seed(778)
  y <- simulate_cat(100, 5, order = 1, n_categories = 2)
  score_test <- test_timeinvariance_cat(y, order = 1, test = "score")
  expect_s3_class(score_test, "cat_lrt")
  expect_equal(score_test$test, "score")
  expect_true(score_test$lrt_stat >= 0)
})

test_that("test_timeinvariance_cat supports mlrt option", {
  skip_on_cran()
  old_opt <- options(antedep.cat_mlrt_nsim = 20L, antedep.cat_mlrt_seed = 202L)
  on.exit(options(old_opt), add = TRUE)

  set.seed(779)
  y <- simulate_cat(100, 5, order = 1, n_categories = 2)
  mlrt_test <- test_timeinvariance_cat(y, order = 1, test = "mlrt")
  expect_s3_class(mlrt_test, "cat_lrt")
  expect_equal(mlrt_test$test, "mlrt")
  expect_true(is.finite(mlrt_test$e_hat_mlrt))
  expect_true(mlrt_test$e_hat_mlrt > 0)
  expect_true(mlrt_test$lrt_stat >= 0)
})


test_that("test_timeinvariance_cat validates inputs", {
  skip_on_cran()
  y <- simulate_cat(50, 4, order = 1, n_categories = 2)
  
  # Error: order < 1
  expect_error(test_timeinvariance_cat(y, order = 0))
})


# ============================================================
# Test print methods
# ============================================================
test_that("print.cat_lrt works", {
  skip_on_cran()
  set.seed(888)
  y <- simulate_cat(100, 4, order = 1, n_categories = 2)
  
  test <- test_order_cat(y, order_null = 0, order_alt = 1)
  
  # Just verify it doesn't error
  expect_output(print(test), "Likelihood Ratio Test")
})

test_that("test_stationarity_cat supports score option (order 1)", {
  skip_on_cran()
  set.seed(889)
  y <- simulate_cat(100, 5, order = 1, n_categories = 2)
  score_test <- test_stationarity_cat(y, order = 1, test = "score")
  expect_s3_class(score_test, "cat_lrt")
  expect_equal(score_test$test, "score")
  expect_true(score_test$lrt_stat >= 0)
  expect_true(score_test$p_value >= 0 && score_test$p_value <= 1)
})

test_that("test_stationarity_cat supports score option (order 2)", {
  skip_on_cran()
  set.seed(890)
  y <- simulate_cat(100, 6, order = 2, n_categories = 2)
  score_test <- NULL
  expect_warning(
    score_test <- test_stationarity_cat(y, order = 2, test = "score"),
    "marginal-constancy plus time-invariant transitions"
  )
  expect_s3_class(score_test, "cat_lrt")
  expect_equal(score_test$test, "score")
  expect_true(score_test$lrt_stat >= 0)
  expect_true(score_test$p_value >= 0 && score_test$p_value <= 1)
})

test_that("test_stationarity_cat blocks lrt/mlrt at order 2", {
  skip_on_cran()
  set.seed(892)
  y <- simulate_cat(80, 6, order = 2, n_categories = 2)
  expect_error(
    test_stationarity_cat(y, order = 2, test = "lrt"),
    "not supported for order >= 2"
  )
  expect_error(
    test_stationarity_cat(y, order = 2, test = "mlrt"),
    "not supported for order >= 2"
  )
})

test_that("test_stationarity_cat supports mlrt option (order 1)", {
  skip_on_cran()
  old_opt <- options(antedep.cat_mlrt_nsim = 20L, antedep.cat_mlrt_seed = 303L)
  on.exit(options(old_opt), add = TRUE)

  set.seed(891)
  y <- simulate_cat(80, 5, order = 1, n_categories = 2)
  mlrt_test <- test_stationarity_cat(y, order = 1, test = "mlrt")
  expect_s3_class(mlrt_test, "cat_lrt")
  expect_equal(mlrt_test$test, "mlrt")
  expect_true(is.finite(mlrt_test$e_hat_mlrt))
  expect_true(mlrt_test$e_hat_mlrt > 0)
  expect_true(mlrt_test$lrt_stat >= 0)
  expect_true(mlrt_test$p_value >= 0 && mlrt_test$p_value <= 1)
})

#' Confidence intervals for fitted Gaussian AD models
#'
#' Computes approximate Wald confidence intervals for selected parameters from a
#' fitted Gaussian AD model.
#'
#' @param fit A fitted model object returned by \code{\link{fit_gau}}.
#' @param level Confidence level between 0 and 1.
#' @param parameters Which parameters to include: \code{"all"} (default),
#'   \code{"mu"}, \code{"phi"}, or \code{"sigma"}.
#'
#' @return An object of class \code{gau_ci}, a list with elements
#'   \code{settings}, \code{level}, \code{mu}, \code{phi}, and \code{sigma}.
#'   Each non-NULL element is a data frame with columns \code{param},
#'   \code{est}, \code{se}, \code{lower}, \code{upper}, and \code{level}.
#'
#' @details
#' This helper currently supports complete-data Gaussian AD fits.
#'
#' Standard errors are based on large-sample approximations:
#' \itemize{
#'   \item \eqn{SE(\hat{\mu}_t) \approx \hat{\sigma}_t / \sqrt{n}}
#'   \item \eqn{SE(\hat{\sigma}_t) \approx \hat{\sigma}_t / \sqrt{2n}}
#'   \item \eqn{SE(\hat{\phi}) \approx \sqrt{(1-\hat{\phi}^2)/n}} for free \eqn{\phi} entries
#' }
#'
#' @examples
#' \donttest{
#' y <- simulate_gau(n_subjects = 80, n_time = 6, order = 1, phi = 0.4)
#' fit <- fit_gau(y, order = 1)
#' ci <- ci_gau(fit)
#' ci$mu
#' ci$phi
#' ci$sigma
#' }
#'
#' @seealso \code{\link{fit_gau}}, \code{\link{ci_cat}}, \code{\link{ci_inad}}
#' @export
ci_gau <- function(fit, level = 0.95, parameters = "all") {
    if (is.null(fit$settings$order)) stop("fit$settings$order is missing")
    if (is.null(fit$settings$n_subjects)) stop("fit$settings$n_subjects is missing")
    if (is.null(fit$log_l) || !is.finite(fit$log_l)) stop("fit$log_l must be finite")
    if (level <= 0 || level >= 1) stop("level must be between 0 and 1")

    if (!is.null(fit$n_missing) && fit$n_missing > 0) {
        stop(
            "ci_gau currently supports complete-data fits only. Missing-data Gaussian AD confidence intervals are not implemented yet.",
            call. = FALSE
        )
    }
    if (!is.null(fit$settings$na_action) && identical(fit$settings$na_action, "em")) {
        stop(
            "ci_gau currently supports complete-data fits only. Missing-data Gaussian AD confidence intervals are not implemented yet.",
            call. = FALSE
        )
    }

    parameters <- match.arg(parameters, c("all", "mu", "phi", "sigma"))
    z <- stats::qnorm(1 - (1 - level) / 2)

    n <- fit$settings$n_subjects
    ord <- fit$settings$order
    N <- if (!is.null(fit$sigma)) length(as.numeric(fit$sigma)) else length(as.numeric(fit$mu))

    out <- list(settings = fit$settings, level = level, mu = NULL, phi = NULL, sigma = NULL)

    if (parameters %in% c("all", "mu") && !is.null(fit$mu)) {
        mu_hat <- as.numeric(fit$mu)
        sigma_ref <- if (!is.null(fit$sigma)) as.numeric(fit$sigma) else rep(stats::sd(mu_hat), length(mu_hat))
        se_mu <- sigma_ref / sqrt(n)
        out$mu <- data.frame(
            param = paste0("mu[", seq_along(mu_hat), "]"),
            est = mu_hat,
            se = se_mu,
            lower = mu_hat - z * se_mu,
            upper = mu_hat + z * se_mu,
            level = level,
            row.names = NULL
        )
    }

    if (parameters %in% c("all", "sigma") && !is.null(fit$sigma)) {
        sigma_hat <- as.numeric(fit$sigma)
        se_sigma <- sigma_hat / sqrt(2 * n)
        out$sigma <- data.frame(
            param = paste0("sigma[", seq_along(sigma_hat), "]"),
            est = sigma_hat,
            se = se_sigma,
            lower = pmax(0, sigma_hat - z * se_sigma),
            upper = sigma_hat + z * se_sigma,
            level = level,
            row.names = NULL
        )
    }

    if (parameters %in% c("all", "phi")) {
        phi_rows <- list()

        if (ord == 1 && !is.null(fit$phi)) {
            phi_hat <- as.numeric(fit$phi)
            if (length(phi_hat) != N) stop("fit$phi has unexpected length for order 1")

            if (N >= 2) for (t in 2:N) {
                est <- phi_hat[t]
                se <- sqrt(max(1e-12, 1 - est^2) / n)
                phi_rows[[length(phi_rows) + 1L]] <- data.frame(
                    param = paste0("phi1[", t, "]"),
                    est = est,
                    se = se,
                    lower = est - z * se,
                    upper = est + z * se,
                    level = level,
                    row.names = NULL
                )
            }
        }

        if (ord == 2 && !is.null(fit$phi)) {
            phi_hat <- as.matrix(fit$phi)
            if (nrow(phi_hat) != 2 || ncol(phi_hat) != N) stop("fit$phi has unexpected dimensions for order 2")

            if (N >= 2) for (t in 2:N) {
                est <- phi_hat[1, t]
                se <- sqrt(max(1e-12, 1 - est^2) / n)
                phi_rows[[length(phi_rows) + 1L]] <- data.frame(
                    param = paste0("phi1[", t, "]"),
                    est = est,
                    se = se,
                    lower = est - z * se,
                    upper = est + z * se,
                    level = level,
                    row.names = NULL
                )
            }

            if (N >= 3) for (t in 3:N) {
                est <- phi_hat[2, t]
                se <- sqrt(max(1e-12, 1 - est^2) / n)
                phi_rows[[length(phi_rows) + 1L]] <- data.frame(
                    param = paste0("phi2[", t, "]"),
                    est = est,
                    se = se,
                    lower = est - z * se,
                    upper = est + z * se,
                    level = level,
                    row.names = NULL
                )
            }
        }

        if (length(phi_rows) > 0) out$phi <- do.call(rbind, phi_rows)
    }

    class(out) <- "gau_ci"
    out
}

#' Print method for AD confidence intervals
#'
#' @param x An object of class \code{gau_ci}.
#' @param ... Unused.
#'
#' @return The input object, invisibly.
#' @export
print.gau_ci <- function(x, ...) {
    cat("CI level:", x$level, "\n")
    if (!is.null(x$mu)) cat("mu rows:", nrow(x$mu), "\n")
    if (!is.null(x$phi)) cat("phi rows:", nrow(x$phi), "\n")
    if (!is.null(x$sigma)) cat("sigma rows:", nrow(x$sigma), "\n")
    invisible(x)
}

#' Summary method for gau_ci objects
#'
#' @param object A \code{gau_ci} object.
#' @param ... Unused.
#'
#' @return A data frame stacking available confidence intervals with columns
#'   \code{param}, \code{component}, \code{est}, \code{se}, \code{lower},
#'   \code{upper}, and \code{level}. Returns \code{NULL} if no intervals are available.
#' @export
summary.gau_ci <- function(object, ...) {
    parts <- list()

    add_part <- function(df, component) {
        if (is.null(df) || !is.data.frame(df) || nrow(df) == 0L) return(NULL)
        df$component <- component
        keep <- intersect(
            c("param", "component", "est", "se", "lower", "upper", "level"),
            names(df)
        )
        df[, keep, drop = FALSE]
    }

    parts[[length(parts) + 1L]] <- add_part(object$mu, "mu")
    parts[[length(parts) + 1L]] <- add_part(object$phi, "phi")
    parts[[length(parts) + 1L]] <- add_part(object$sigma, "sigma")
    parts <- Filter(Negate(is.null), parts)

    if (length(parts) == 0L) return(NULL)
    do.call(rbind, parts)
}

test_that("ci_gau returns Wald intervals for complete-data Gaussian AD fit", {
  skip_on_cran()
  set.seed(510)
  y <- simulate_gau(
    n_subjects = 90,
    n_time = 6,
    order = 1,
    phi = c(0, rep(0.4, 5))
  )
  fit <- fit_gau(y, order = 1)

  ci <- ci_gau(fit, level = 0.95, parameters = "all")

  expect_s3_class(ci, "gau_ci")
  expect_true(is.data.frame(ci$mu))
  expect_true(is.data.frame(ci$phi))
  expect_true(is.data.frame(ci$sigma))
  expect_equal(nrow(ci$mu), ncol(y))
  expect_equal(nrow(ci$sigma), ncol(y))
  expect_equal(nrow(ci$phi), ncol(y) - 1)
})

test_that("ci_gau rejects missing-data Gaussian AD fits", {
  skip_on_cran()
  set.seed(511)
  y <- simulate_gau(
    n_subjects = 60,
    n_time = 5,
    order = 1,
    phi = c(0, rep(0.35, 4))
  )
  y[sample(length(y), 12)] <- NA
  fit_em <- fit_gau(y, order = 1, na_action = "em", em_max_iter = 20)

  expect_error(
    ci_gau(fit_em),
    "complete-data fits only",
    fixed = FALSE
  )
})

# fit_cat.R - Maximum likelihood estimation for categorical antedependence models

#' Fit categorical antedependence model by maximum likelihood
#'
#' Computes maximum likelihood estimates for the parameters of an AD(p) model
#' for categorical longitudinal data. The model is parameterized by transition
#' probabilities, and MLEs are obtained in closed form.
#'
#' @param y Integer matrix with n_subjects rows and n_time columns. Each entry
#'   should be a category code from 1 to c, where c is the number of categories.
#' @param order Antedependence order p. Must be 0, 1, or 2. Default is 1.
#' @param blocks Optional integer vector of length n_subjects specifying group
#'   membership. If NULL, all subjects are treated as one group.
#' @param homogeneous Logical. If TRUE (default), parameters are shared across
#'   all groups (blocks are ignored for estimation). If FALSE, separate
#'   transition probabilities are estimated for each group.
#' @param n_categories Number of categories. If NULL (default), inferred from
#'   the maximum value in y.
#' @param na_action Handling of missing values in \code{y}. One of
#'   \code{"fail"} (default, error if any missing), \code{"complete"}
#'   (drop subjects with any missing values), or \code{"marginalize"}
#'   (maximize observed-data likelihood by integrating over missing outcomes),
#'   or \code{"em"} (use \code{\link{em_cat}} for orders 0 and 1).
#' @param em_max_iter Maximum EM iterations used when \code{na_action = "em"}.
#' @param em_tol EM convergence tolerance used when \code{na_action = "em"}.
#' @param em_epsilon Numerical smoothing constant used when \code{na_action = "em"}.
#' @param em_safeguard Logical; if \code{TRUE}, use step-halving safeguard in
#'   \code{\link{em_cat}} when \code{na_action = "em"}.
#' @param em_verbose Logical; print EM progress when \code{na_action = "em"}.
#'
#' @return A list of class \code{"cat_fit"} containing:
#'   \item{marginal}{List of marginal/joint probabilities for initial time points}
#'   \item{transition}{List of transition probability arrays for k = p+1 to n}
#'   \item{log_l}{Log-likelihood at MLE}
#'   \item{aic}{Akaike Information Criterion}
#'   \item{bic}{Bayesian Information Criterion}
#'   \item{n_params}{Number of free parameters}
#'   \item{cell_counts}{List of cell counts: observed counts for closed-form fits
#'   (\code{na_action = "fail"/"complete"}), expected counts from the final E-step
#'   for EM fits (\code{na_action = "em"}), and \code{NULL} for
#'   \code{na_action = "marginalize"}}
#'   \item{convergence}{Optimizer convergence code (0 for closed-form solutions)}
#'   \item{settings}{List of model settings}
#'
#' @details
#' For AD(p), the model decomposes as:
#' \deqn{P(Y_1, \ldots, Y_n) = P(Y_1, \ldots, Y_p) \times \prod_{k=p+1}^{n} P(Y_k | Y_{k-p}, \ldots, Y_{k-1})}
#'
#' MLEs are computed as empirical proportions:
#' \itemize{
#'   \item Marginal/joint probabilities: count / N
#'   \item Transition probabilities: conditional count / marginal count
#' }
#'
#' Empty cells receive probability 0 (if denominator is also 0).
#'
#' When \code{na_action = "em"}, \code{fit_cat()} dispatches to
#' \code{\link{em_cat}}. In that case, \code{em_safeguard} controls step-halving
#' protection against likelihood-decreasing updates, and returned
#' \code{log_l}/AIC/BIC/\code{cell_counts} are synchronized via a final E-step
#' under the returned parameters.
#' For \code{order = 2}, \code{na_action = "em"} is not available and errors
#' explicitly; use \code{na_action = "marginalize"}.
#'
#' @examples
#' \donttest{
#' # Simulate binary AD(1) data
#' set.seed(123)
#' y <- simulate_cat(n_subjects = 100, n_time = 5, order = 1, n_categories = 2)
#'
#' # Fit model
#' fit <- fit_cat(y, order = 1)
#' print(fit)
#'
#' # Compare orders
#' fit0 <- fit_cat(y, order = 0)
#' fit1 <- fit_cat(y, order = 1)
#' fit2 <- fit_cat(y, order = 2)
#' c(AIC_0 = fit0$aic, AIC_1 = fit1$aic, AIC_2 = fit2$aic)
#'
#' # EM fit with missing data
#' y_miss <- y
#' y_miss[sample(length(y_miss), size = round(0.15 * length(y_miss)))] <- NA
#' fit_em <- fit_cat(
#'   y_miss,
#'   order = 1,
#'   na_action = "em",
#'   em_max_iter = 80,
#'   em_tol = 1e-6
#' )
#' fit_em$settings$n_iter
#' fit_em$settings$cell_counts_type
#' }
#'
#' @references
#' Xie, Y. and Zimmerman, D. L. (2013). Antedependence models for nonstationary
#' categorical longitudinal data with ignorable missingness: likelihood-based
#' inference. \emph{Statistics in Medicine}, 32, 3274-3289.
#'
#' @export
fit_cat <- function(y, order = 1, blocks = NULL, homogeneous = TRUE, 
                    n_categories = NULL,
                    na_action = c("fail", "complete", "marginalize", "em"),
                    em_max_iter = 100, em_tol = 1e-6, em_epsilon = 1e-8,
                    em_safeguard = TRUE, em_verbose = FALSE) {
  na_action <- match.arg(na_action)
  blocks_input <- blocks
  

  # Validate inputs
  validated <- .validate_y_cat(y, n_categories, allow_na = (na_action != "fail"))
  y <- validated$y
  c <- validated$n_categories
  
  n_subjects <- nrow(y)
  n_time <- ncol(y)
  p <- as.integer(order)
  
  # Validate order
  if (p < 0) stop("order must be non-negative")
  if (p > 2) stop("order > 2 not currently supported")
  if (p >= n_time) stop("order must be less than number of time points")
  if (na_action == "marginalize" && p > 2) {
    stop("na_action = 'marginalize' currently supports order 0, 1, and 2")
  }
  
  # Validate and process blocks
  block_info <- .normalize_blocks(blocks, n_subjects)
  blocks <- block_info$blocks_id
  block_levels <- block_info$block_levels

  if (na_action == "em") {
    if (p == 2L) {
      stop("na_action = 'em' currently supports order 0 and 1; use na_action = 'marginalize' for order 2")
    }
    return(em_cat(
      y = y,
      order = p,
      blocks = blocks_input,
      homogeneous = homogeneous,
      n_categories = c,
      max_iter = em_max_iter,
      tol = em_tol,
      epsilon = em_epsilon,
      safeguard = em_safeguard,
      verbose = em_verbose
    ))
  }
  
  # Handle complete-case filtering if requested
  if (na_action == "complete") {
    keep <- stats::complete.cases(y)
    y <- y[keep, , drop = FALSE]
    if (!is.null(blocks_input)) {
      block_info <- .normalize_blocks(blocks_input[keep], nrow(y))
      blocks <- block_info$blocks_id
      block_levels <- block_info$block_levels
    } else {
      blocks <- rep(1L, nrow(y))
      block_levels <- "1"
    }
    if (nrow(y) == 0) {
      stop("No complete subjects remain after na_action = 'complete'")
    }
  }
  
  n_subjects <- nrow(y)
  effective_na_action <- na_action
  if (na_action == "marginalize" && !any(is.na(y))) {
    # Degenerate to complete-data closed-form fit when no missing values remain.
    effective_na_action <- "complete"
  }
  
  n_blocks <- max(blocks)
  
  # Determine number of populations to estimate
  if (homogeneous) {
    n_pops <- 1
  } else {
    n_pops <- n_blocks
  }
  
  # Count parameters
  n_params <- .count_params_cat(p, c, n_time, n_blocks, homogeneous)
  
  convergence <- 0L
  
  # Initialize storage for parameters
  if (n_pops == 1) {
    if (effective_na_action == "marginalize") {
      result <- .fit_cat_single_pop_marginalize(y, p, c, n_time)
      convergence <- as.integer(result$convergence)
    } else {
      result <- .fit_cat_single_pop(y, p, c, n_time, subject_mask = NULL)
    }
    marginal <- result$marginal
    transition <- result$transition
    cell_counts <- result$cell_counts
    log_l <- result$log_l
  } else {
    # Fit separately for each block
    marginal <- vector("list", n_pops)
    transition <- vector("list", n_pops)
    cell_counts <- vector("list", n_pops)
    log_l <- 0
    conv_codes <- integer(n_pops)
    
    for (g in seq_len(n_pops)) {
      mask <- (blocks == g)
      y_g <- y[mask, , drop = FALSE]
      if (effective_na_action == "marginalize") {
        result_g <- .fit_cat_single_pop_marginalize(y_g, p, c, n_time)
        conv_codes[g] <- as.integer(result_g$convergence)
      } else {
        result_g <- .fit_cat_single_pop(y, p, c, n_time, subject_mask = mask)
        conv_codes[g] <- 0L
      }
      marginal[[g]] <- result_g$marginal
      transition[[g]] <- result_g$transition
      # Preserve list length even when cell_counts is NULL under marginalization.
      cell_counts[g] <- list(result_g$cell_counts)
      log_l <- log_l + result_g$log_l
    }
    names(marginal) <- paste0("block_", seq_len(n_pops))
    names(transition) <- paste0("block_", seq_len(n_pops))
    names(cell_counts) <- paste0("block_", seq_len(n_pops))
    convergence <- as.integer(max(conv_codes))
  }
  
  # Compute AIC and BIC
  aic <- -2 * log_l + 2 * n_params
  bic <- -2 * log_l + n_params * log(n_subjects)
  
  # Assemble output
  out <- list(
    marginal = marginal,
    transition = transition,
    log_l = log_l,
    aic = aic,
    bic = bic,
    n_params = n_params,
    n_obs = sum(!is.na(y)),
    n_missing = sum(is.na(y)),
    pct_missing = mean(is.na(y)) * 100,
    cell_counts = cell_counts,
    convergence = convergence,
    settings = list(
      order = p,
      n_categories = c,
      n_time = n_time,
      n_subjects = n_subjects,
      blocks = if (n_blocks > 1) blocks else NULL,
      block_levels = block_levels,
      homogeneous = homogeneous,
      n_blocks = n_blocks,
      na_action = na_action,
      na_action_effective = effective_na_action,
      cell_counts_type = if (identical(effective_na_action, "marginalize")) "none" else "observed"
    )
  )
  
  class(out) <- "cat_fit"
  out
}


#' Fit CAT model with missing data via observed-data likelihood optimization
#'
#' @param y Data matrix for one population (may contain NA)
#' @param p Order
#' @param c Number of categories
#' @param n_time Number of time points
#'
#' @return List with marginal, transition, cell_counts, log_l, convergence
#'
#' @keywords internal
.fit_cat_single_pop_marginalize <- function(y, p, c, n_time) {
  # Initialize from complete cases if available; otherwise uniform probabilities.
  y_complete <- y[stats::complete.cases(y), , drop = FALSE]
  if (nrow(y_complete) > 0) {
    init_fit <- .fit_cat_single_pop(y_complete, p, c, n_time, subject_mask = NULL)
    init_marginal <- init_fit$marginal
    init_transition <- init_fit$transition
  } else {
    init <- .uniform_cat_params(p, c, n_time)
    init_marginal <- init$marginal
    init_transition <- init$transition
  }
  
  theta_start <- .pack_cat_params(init_marginal, init_transition, p, c, n_time)
  
  objective <- function(theta) {
    params <- .unpack_cat_params(theta, p, c, n_time)
    ll <- logL_cat(
      y = y,
      order = p,
      marginal = params$marginal,
      transition = params$transition,
      n_categories = c,
      na_action = "marginalize"
    )
    if (!is.finite(ll)) return(1e50)
    -as.numeric(ll)
  }
  
  if (length(theta_start) == 0) {
    # Degenerate single-category case
    params <- .unpack_cat_params(theta_start, p, c, n_time)
    ll <- -objective(theta_start)
    return(list(
      marginal = params$marginal,
      transition = params$transition,
      cell_counts = NULL,
      log_l = ll,
      convergence = 0L
    ))
  }
  
  opt <- stats::optim(
    par = theta_start,
    fn = objective,
    method = "BFGS",
    control = list(maxit = 2000, reltol = 1e-10)
  )
  
  # Fallback if BFGS does not converge cleanly.
  if (opt$convergence != 0L || !is.finite(opt$value)) {
    opt_nm <- stats::optim(
      par = opt$par,
      fn = objective,
      method = "Nelder-Mead",
      control = list(maxit = 4000, reltol = 1e-10)
    )
    if (is.finite(opt_nm$value) && (!is.finite(opt$value) || opt_nm$value < opt$value)) {
      opt <- opt_nm
    }
  }
  
  params <- .unpack_cat_params(opt$par, p, c, n_time)
  
  list(
    marginal = params$marginal,
    transition = params$transition,
    cell_counts = NULL,
    log_l = -opt$value,
    convergence = as.integer(opt$convergence)
  )
}


#' Convert probability row to unconstrained logits
#'
#' @param prob Probability vector of length c
#' @param c Number of categories
#' @param eps Lower bound for numerical stability
#'
#' @return Numeric vector of length c-1
#'
#' @keywords internal
.cat_prob_to_theta <- function(prob, c, eps = 1e-8) {
  if (c <= 1) return(numeric(0))
  prob <- as.numeric(prob)
  if (length(prob) != c) {
    stop("Probability row has wrong length in CAT parameter packing")
  }
  prob <- pmax(prob, eps)
  prob <- prob / sum(prob)
  log(prob[seq_len(c - 1)] / prob[c])
}


#' Convert unconstrained logits to probability row
#'
#' @param theta_row Numeric vector of length c-1
#' @param c Number of categories
#'
#' @return Probability vector of length c
#'
#' @keywords internal
.cat_theta_to_prob <- function(theta_row, c) {
  if (c <= 1) return(1)
  if (length(theta_row) != (c - 1)) {
    stop("Theta row has wrong length in CAT parameter unpacking")
  }
  z <- c(theta_row, 0)
  z <- z - max(z)
  ez <- exp(z)
  ez / sum(ez)
}


#' Pack CAT parameters into unconstrained vector
#'
#' @param marginal Marginal parameter list
#' @param transition Transition parameter list
#' @param p Order
#' @param c Number of categories
#' @param n_time Number of time points
#'
#' @return Numeric parameter vector
#'
#' @keywords internal
.pack_cat_params <- function(marginal, transition, p, c, n_time) {
  theta <- numeric(0)
  
  if (p == 0) {
    for (k in seq_len(n_time)) {
      theta <- c(theta, .cat_prob_to_theta(marginal[[k]], c))
    }
    return(theta)
  }
  
  theta <- c(theta, .cat_prob_to_theta(marginal[["t1"]], c))
  
  if (p == 1) {
    for (k in 2:n_time) {
      trans_k <- transition[[paste0("t", k)]]
      for (from in seq_len(c)) {
        theta <- c(theta, .cat_prob_to_theta(trans_k[from, ], c))
      }
    }
    return(theta)
  }
  
  # p == 2
  trans_t2 <- marginal[["t2_given_1to1"]]
  for (from in seq_len(c)) {
    theta <- c(theta, .cat_prob_to_theta(trans_t2[from, ], c))
  }
  for (k in 3:n_time) {
    trans_k <- transition[[paste0("t", k)]]
    for (from1 in seq_len(c)) {
      for (from2 in seq_len(c)) {
        theta <- c(theta, .cat_prob_to_theta(trans_k[from1, from2, ], c))
      }
    }
  }
  
  theta
}


#' Unpack unconstrained CAT parameter vector
#'
#' @param theta Numeric parameter vector
#' @param p Order
#' @param c Number of categories
#' @param n_time Number of time points
#'
#' @return List with marginal and transition parameter lists
#'
#' @keywords internal
.unpack_cat_params <- function(theta, p, c, n_time) {
  idx <- 1L
  
  next_row <- function() {
    if (c <= 1) return(1)
    end_idx <- idx + c - 2
    if (end_idx > length(theta)) {
      stop("Theta vector ended early while unpacking CAT parameters")
    }
    row <- .cat_theta_to_prob(theta[idx:end_idx], c)
    idx <<- end_idx + 1L
    row
  }
  
  marginal <- list()
  transition <- list()
  
  if (p == 0) {
    for (k in seq_len(n_time)) {
      marginal[[k]] <- next_row()
      names(marginal[[k]]) <- paste0("cat_", seq_len(c))
    }
    names(marginal) <- paste0("t", seq_len(n_time))
  } else if (p == 1) {
    marginal[["t1"]] <- next_row()
    names(marginal[["t1"]]) <- paste0("cat_", seq_len(c))
    
    for (k in 2:n_time) {
      trans_k <- matrix(0, nrow = c, ncol = c)
      for (from in seq_len(c)) {
        trans_k[from, ] <- next_row()
      }
      transition[[paste0("t", k)]] <- trans_k
    }
  } else if (p == 2) {
    marginal[["t1"]] <- next_row()
    names(marginal[["t1"]]) <- paste0("cat_", seq_len(c))
    
    trans_t2 <- matrix(0, nrow = c, ncol = c)
    for (from in seq_len(c)) {
      trans_t2[from, ] <- next_row()
    }
    marginal[["t2_given_1to1"]] <- trans_t2
    
    for (k in 3:n_time) {
      trans_k <- array(0, dim = c(c, c, c))
      for (from1 in seq_len(c)) {
        for (from2 in seq_len(c)) {
          trans_k[from1, from2, ] <- next_row()
        }
      }
      transition[[paste0("t", k)]] <- trans_k
    }
  } else {
    stop("Only orders 0, 1, and 2 are supported")
  }
  
  if (idx <= length(theta)) {
    stop("Theta vector has extra elements after CAT parameter unpacking")
  }
  
  list(marginal = marginal, transition = transition)
}


#' Build uniform CAT parameter values
#'
#' @param p Order
#' @param c Number of categories
#' @param n_time Number of time points
#'
#' @return List with marginal and transition
#'
#' @keywords internal
.uniform_cat_params <- function(p, c, n_time) {
  probs <- rep(1 / c, c)
  marginal <- list()
  transition <- list()
  
  if (p == 0) {
    marginal <- lapply(seq_len(n_time), function(k) {
      out <- probs
      names(out) <- paste0("cat_", seq_len(c))
      out
    })
    names(marginal) <- paste0("t", seq_len(n_time))
    return(list(marginal = marginal, transition = transition))
  }
  
  marginal[["t1"]] <- stats::setNames(probs, paste0("cat_", seq_len(c)))
  
  if (p == 1) {
    for (k in 2:n_time) {
      transition[[paste0("t", k)]] <- matrix(rep(probs, each = c), nrow = c, byrow = TRUE)
    }
  } else if (p == 2) {
    marginal[["t2_given_1to1"]] <- matrix(rep(probs, each = c), nrow = c, byrow = TRUE)
    for (k in 3:n_time) {
      transition[[paste0("t", k)]] <- array(
        rep(probs, each = c * c),
        dim = c(c, c, c)
      )
    }
  }
  
  list(marginal = marginal, transition = transition)
}


#' Fit CAT model for a single population
#'
#' @param y Data matrix
#' @param p Order
#' @param c Number of categories
#' @param n_time Number of time points
#' @param subject_mask Logical vector for subject selection (NULL = all)
#'
#' @return List with marginal, transition, cell_counts, log_l
#'
#' @keywords internal
.fit_cat_single_pop <- function(y, p, c, n_time, subject_mask = NULL) {
  
  # Apply mask
  if (!is.null(subject_mask)) {
    y_sub <- y[subject_mask, , drop = FALSE]
  } else {
    y_sub <- y
  }
  N <- nrow(y_sub)
  
  # Storage
  marginal <- list()
  transition <- list()
  cell_counts <- list()
  log_l <- 0
  
  if (p == 0) {
    # Independence model: just estimate marginal at each time point
    for (k in seq_len(n_time)) {
      counts_k <- .count_cells_table_cat(y_sub, k, c, subject_mask = NULL)
      probs_k <- counts_k / N
      
      marginal[[k]] <- as.numeric(probs_k)
      names(marginal[[k]]) <- paste0("cat_", 1:c)
      cell_counts[[paste0("t", k)]] <- counts_k
      
      # Log-likelihood contribution
      log_l <- log_l + .loglik_contribution(counts_k, probs_k)
    }
    names(marginal) <- paste0("t", seq_len(n_time))
    
  } else {
    # AD(p) model with p >= 1
    
    # Handle initial time points (k = 1 to p)
    # For these, we need the full joint distribution P(Y_1, ..., Y_k)
    
    for (k in seq_len(min(p, n_time))) {
      time_indices <- seq_len(k)
      counts_k <- .count_cells_table_cat(y_sub, time_indices, c, subject_mask = NULL)
      
      if (k == 1) {
        # Marginal of Y_1
        probs_k <- counts_k / N
        marginal[["t1"]] <- as.numeric(probs_k)
        names(marginal[["t1"]]) <- paste0("cat_", 1:c)
      } else {
        # P(Y_k | Y_1, ..., Y_{k-1})
        # counts_k has dimensions c^k
        # Need to compute conditional probabilities
        probs_k <- .counts_to_transition_probs(counts_k)
        marginal[[paste0("t", k, "_given_1to", k-1)]] <- probs_k
      }
      
      cell_counts[[paste0("t1_to_t", k)]] <- counts_k
      
      # Log-likelihood contribution for this time point
      # For k=1: sum N_y1 * log(pi_y1)
      # For k>1: sum N_{y1...yk} * log(pi_{yk|y1...y_{k-1}})
      if (k == 1) {
        log_l <- log_l + .loglik_contribution(counts_k, probs_k)
      } else {
        # Need joint counts and transition probs
        # LL contribution = sum over all (y1,...,yk) of N * log(pi_{yk|y1...y_{k-1}})
        log_l <- log_l + .loglik_contribution(counts_k, probs_k)
      }
    }
    
    # Handle time points k = p+1 to n (only condition on last p values)
    for (k in (p + 1):n_time) {
      # Count (Y_{k-p}, ..., Y_{k-1}, Y_k) combinations
      time_indices <- (k - p):k
      counts_k <- .count_cells_table_cat(y_sub, time_indices, c, subject_mask = NULL)
      
      # Compute transition probabilities P(Y_k | Y_{k-p}, ..., Y_{k-1})
      probs_k <- .counts_to_transition_probs(counts_k)
      
      transition[[paste0("t", k)]] <- probs_k
      cell_counts[[paste0("t", k-p, "_to_t", k)]] <- counts_k
      
      # Log-likelihood contribution
      log_l <- log_l + .loglik_contribution(counts_k, probs_k)
    }
  }
  
  list(
    marginal = marginal,
    transition = transition,
    cell_counts = cell_counts,
    log_l = log_l
  )
}

# test-logL_cat.R - Tests for categorical AD log-likelihood

test_that("logL_cat validates inputs correctly", {
  skip_on_cran()
  set.seed(123)
  y <- matrix(sample(1:3, 50, replace = TRUE), nrow = 10, ncol = 5)
  
  # Order must be non-negative
  expect_error(logL_cat(y, order = -1, marginal = list(), transition = list()),
               "non-negative")
  
  # Order must be less than n_time
  expect_error(logL_cat(y, order = 5, marginal = list(), transition = list()),
               "less than")
})


test_that("logL_cat missing-data modes behave correctly", {
  skip_on_cran()
  set.seed(124)
  y <- simulate_cat(50, 5, order = 1, n_categories = 2)
  fit <- fit_cat(y, order = 1)
  
  y_miss <- y
  y_miss[1, 3] <- NA
  y_miss[2, 5] <- NA
  
  expect_error(
    logL_cat(y_miss, order = 1, marginal = fit$marginal, transition = fit$transition,
             na_action = "fail"),
    "Missing data"
  )
  
  keep <- stats::complete.cases(y_miss)
  ll_complete_mode <- logL_cat(
    y_miss, order = 1, marginal = fit$marginal, transition = fit$transition,
    na_action = "complete"
  )
  ll_complete_ref <- logL_cat(
    y_miss[keep, , drop = FALSE], order = 1,
    marginal = fit$marginal, transition = fit$transition
  )
  expect_equal(as.numeric(ll_complete_mode), as.numeric(ll_complete_ref), tolerance = 1e-10)
  
  ll_no_missing <- logL_cat(y, order = 1, marginal = fit$marginal, transition = fit$transition)
  ll_no_missing_marg <- logL_cat(
    y, order = 1, marginal = fit$marginal, transition = fit$transition,
    na_action = "marginalize"
  )
  expect_equal(as.numeric(ll_no_missing_marg), as.numeric(ll_no_missing), tolerance = 1e-10)
})


test_that("logL_cat marginalize is finite for MAR-style missingness", {
  skip_on_cran()
  set.seed(125)
  y <- simulate_cat(60, 5, order = 1, n_categories = 2)
  
  # Monotone missingness (dropout)
  y_monotone <- y
  dropout_time <- sample(2:5, nrow(y_monotone), replace = TRUE)
  dropout_flag <- runif(nrow(y_monotone)) < 0.35
  for (i in seq_len(nrow(y_monotone))) {
    if (dropout_flag[i]) {
      y_monotone[i, dropout_time[i]:ncol(y_monotone)] <- NA
    }
  }
  
  fit_monotone <- fit_cat(y_monotone, order = 1, na_action = "marginalize")
  ll_monotone <- logL_cat(
    y_monotone, order = 1, marginal = fit_monotone$marginal,
    transition = fit_monotone$transition, na_action = "marginalize"
  )
  expect_true(is.finite(fit_monotone$log_l))
  expect_true(is.finite(ll_monotone))
  
  # Intermittent missingness
  y_intermittent <- y
  mask <- matrix(runif(length(y_intermittent)) < 0.15, nrow = nrow(y_intermittent))
  y_intermittent[mask] <- NA
  
  fit_intermittent <- fit_cat(y_intermittent, order = 1, na_action = "marginalize")
  ll_intermittent <- logL_cat(
    y_intermittent, order = 1, marginal = fit_intermittent$marginal,
    transition = fit_intermittent$transition, na_action = "marginalize"
  )
  expect_true(is.finite(fit_intermittent$log_l))
  expect_true(is.finite(ll_intermittent))
})


test_that("logL_cat computes correct log-likelihood for order 0", {
  skip_on_cran()
  set.seed(234)
  n_subjects <- 50
  n_time <- 4
  c <- 2
  
  # Create uniform marginals
  marginal <- lapply(1:n_time, function(k) c(0.5, 0.5))
  names(marginal) <- 1:n_time
  
  # Simulate uniform data
  y <- matrix(sample(1:c, n_subjects * n_time, replace = TRUE), 
              nrow = n_subjects, ncol = n_time)
  
  # Compute log-likelihood
  ll <- logL_cat(y, order = 0, marginal = marginal, n_categories = c)
  
  # Manual computation: each observation has prob 0.5
  ll_manual <- n_subjects * n_time * log(0.5)
  
  expect_equal(as.numeric(ll), ll_manual)
})


test_that("logL_cat computes correct log-likelihood for order 1", {
  skip_on_cran()
  set.seed(345)
  n_subjects <- 100
  n_time <- 3
  c <- 2
  
  # Known marginal
  marginal <- list(t1 = c(0.6, 0.4))
  
  # Known transitions
  transition <- list(
    t2 = matrix(c(0.8, 0.2,
                  0.3, 0.7), nrow = 2, byrow = TRUE),
    t3 = matrix(c(0.8, 0.2,
                  0.3, 0.7), nrow = 2, byrow = TRUE)
  )
  
  # Simulate data from this model
  y <- simulate_cat(n_subjects, n_time, order = 1, n_categories = c,
                    marginal = marginal, transition = transition)
  
  # Compute log-likelihood at true parameters
  ll_true <- logL_cat(y, order = 1, marginal = marginal, 
                      transition = transition, n_categories = c)
  
  # Compute at wrong parameters (uniform)
  marginal_wrong <- list(t1 = c(0.5, 0.5))
  transition_wrong <- list(
    t2 = matrix(c(0.5, 0.5, 0.5, 0.5), nrow = 2, byrow = TRUE),
    t3 = matrix(c(0.5, 0.5, 0.5, 0.5), nrow = 2, byrow = TRUE)
  )
  ll_wrong <- logL_cat(y, order = 1, marginal = marginal_wrong, 
                       transition = transition_wrong, n_categories = c)
  
  # Log-likelihood should be higher at true parameters
  expect_true(as.numeric(ll_true) > as.numeric(ll_wrong))
})


test_that("logL_cat matches fit_cat log-likelihood", {
  skip_on_cran()
  set.seed(456)
  n_subjects <- 80
  n_time <- 5
  
  # Simulate data
  y <- simulate_cat(n_subjects, n_time, order = 1, n_categories = 2)
  
  # Fit model
  fit <- fit_cat(y, order = 1)
  
  # Compute log-likelihood at fitted parameters
  ll_check <- logL_cat(y, order = 1, 
                       marginal = fit$marginal, 
                       transition = fit$transition,
                       n_categories = 2)
  
  expect_equal(as.numeric(ll_check), as.numeric(fit$log_l), tolerance = 1e-10)
})


test_that("logL_cat works for order 2", {
  skip_on_cran()
  set.seed(567)
  n_subjects <- 100
  n_time <- 5
  c <- 2
  
  # Simulate order 2 data
  y <- simulate_cat(n_subjects, n_time, order = 2, n_categories = c)
  
  # Fit order 2 model
  fit <- fit_cat(y, order = 2)
  
  # Check that log-likelihood matches
  ll_check <- logL_cat(y, order = 2,
                       marginal = fit$marginal,
                       transition = fit$transition,
                       n_categories = c)
  
  expect_equal(as.numeric(ll_check), as.numeric(fit$log_l), tolerance = 1e-10)
})


test_that("logL_cat handles more than 2 categories", {
  skip_on_cran()
  set.seed(678)
  n_subjects <- 80
  n_time <- 4
  c <- 3
  
  # Simulate with 3 categories
  y <- simulate_cat(n_subjects, n_time, order = 1, n_categories = c)
  
  # Fit and verify
  fit <- fit_cat(y, order = 1, n_categories = c)
  
  ll_check <- logL_cat(y, order = 1,
                       marginal = fit$marginal,
                       transition = fit$transition,
                       n_categories = c)
  
  expect_equal(as.numeric(ll_check), as.numeric(fit$log_l), tolerance = 1e-10)
})


test_that("logL_cat works with heterogeneous blocks", {
  skip_on_cran()
  set.seed(789)
  n1 <- 40
  n2 <- 40
  n_time <- 4
  c <- 2
  
  # Simulate two groups with different parameters
  marg1 <- list(t1 = c(0.7, 0.3))
  trans1 <- list(
    t2 = matrix(c(0.9, 0.1, 0.2, 0.8), 2, byrow = TRUE),
    t3 = matrix(c(0.9, 0.1, 0.2, 0.8), 2, byrow = TRUE),
    t4 = matrix(c(0.9, 0.1, 0.2, 0.8), 2, byrow = TRUE)
  )
  
  marg2 <- list(t1 = c(0.4, 0.6))
  trans2 <- list(
    t2 = matrix(c(0.5, 0.5, 0.5, 0.5), 2, byrow = TRUE),
    t3 = matrix(c(0.5, 0.5, 0.5, 0.5), 2, byrow = TRUE),
    t4 = matrix(c(0.5, 0.5, 0.5, 0.5), 2, byrow = TRUE)
  )
  
  y1 <- simulate_cat(n1, n_time, order = 1, n_categories = c,
                     marginal = marg1, transition = trans1)
  y2 <- simulate_cat(n2, n_time, order = 1, n_categories = c,
                     marginal = marg2, transition = trans2)
  y <- rbind(y1, y2)
  blocks <- c(rep(1, n1), rep(2, n2))
  
  # Fit heterogeneous model
  fit <- fit_cat(y, order = 1, blocks = blocks, homogeneous = FALSE)
  
  # Check log-likelihood
  ll_check <- logL_cat(y, order = 1,
                       marginal = fit$marginal,
                       transition = fit$transition,
                       blocks = blocks,
                       homogeneous = FALSE,
                       n_categories = c)
  
  expect_equal(as.numeric(ll_check), as.numeric(fit$log_l), tolerance = 1e-10)
})


test_that("logL_cat is additive across independent subjects", {
  skip_on_cran()
  set.seed(901)
  n_time <- 4
  c <- 2
  
  # Create two subjects
  y <- matrix(sample(1:c, 2 * n_time, replace = TRUE), nrow = 2, ncol = n_time)
  
  # Fit model to get valid parameters
  fit <- fit_cat(y, order = 1, n_categories = c)
  
  # Full log-likelihood
  ll_full <- logL_cat(y, order = 1, marginal = fit$marginal,
                      transition = fit$transition, n_categories = c)
  
  # Individual log-likelihoods
  ll_1 <- logL_cat(y[1, , drop = FALSE], order = 1, marginal = fit$marginal,
                   transition = fit$transition, n_categories = c)
  ll_2 <- logL_cat(y[2, , drop = FALSE], order = 1, marginal = fit$marginal,
                   transition = fit$transition, n_categories = c)
  
  expect_equal(as.numeric(ll_full), as.numeric(ll_1) + as.numeric(ll_2), tolerance = 1e-10)
})


test_that("logL_cat handles n_categories override", {
  skip_on_cran()
  set.seed(112)
  n_subjects <- 50
  n_time <- 4
  
  # Generate data with only categories 1 and 2 observed
  y <- matrix(sample(1:2, n_subjects * n_time, replace = TRUE), 
              nrow = n_subjects, ncol = n_time)
  
  # Fit with 3 categories (even though category 3 not observed)
  fit <- fit_cat(y, order = 1, n_categories = 3)
  
  # Verify log-likelihood works with the override
  ll <- logL_cat(y, order = 1, marginal = fit$marginal,
                 transition = fit$transition, n_categories = 3)
  
  expect_true(is.finite(as.numeric(ll)))
  expect_equal(as.numeric(ll), as.numeric(fit$log_l), tolerance = 1e-10)
})


test_that("logL_cat likelihood increases with correct model", {
  skip_on_cran()
  set.seed(223)
  n_subjects <- 60
  n_time <- 5
  c <- 2
  
  # Strong dependence structure
  marginal_true <- list(t1 = c(0.5, 0.5))
  transition_true <- list(
    t2 = matrix(c(0.95, 0.05, 0.05, 0.95), 2, byrow = TRUE),
    t3 = matrix(c(0.95, 0.05, 0.05, 0.95), 2, byrow = TRUE),
    t4 = matrix(c(0.95, 0.05, 0.05, 0.95), 2, byrow = TRUE),
    t5 = matrix(c(0.95, 0.05, 0.05, 0.95), 2, byrow = TRUE)
  )
  
  # Simulate from true model
  y <- simulate_cat(n_subjects, n_time, order = 1, n_categories = c,
                    marginal = marginal_true, transition = transition_true)
  
  # Log-likelihood at true parameters
  ll_true <- logL_cat(y, order = 1, marginal = marginal_true,
                      transition = transition_true, n_categories = c)
  
  # Log-likelihood at MLE (should be >= true)
  fit <- fit_cat(y, order = 1, n_categories = c)
  ll_mle <- fit$log_l
  
  # MLE should be at least as good as true parameters
  expect_true(as.numeric(ll_mle) >= as.numeric(ll_true) - 1e-6)  # Small tolerance for numerical issues
})

# test-em-cat.R - Tests for EM estimation in CAT models

test_that("em_cat returns closed-form fit when no missing data", {
  skip_on_cran()
  set.seed(1301)
  y <- simulate_cat(n_subjects = 50, n_time = 4, order = 1, n_categories = 2)

  fit_em <- em_cat(y, order = 1)
  fit_cf <- fit_cat(y, order = 1)

  expect_s3_class(fit_em, "cat_fit")
  expect_equal(as.numeric(fit_em$log_l), as.numeric(fit_cf$log_l), tolerance = 1e-10)
  expect_equal(fit_em$settings$method, "closed_form")
})


test_that("em_cat fits missing-data order 0 model", {
  skip_on_cran()
  set.seed(1302)
  y <- simulate_cat(n_subjects = 40, n_time = 5, order = 0, n_categories = 3)
  y[sample(length(y), 25)] <- NA

  fit <- em_cat(y, order = 0, max_iter = 20, tol = 1e-7)

  expect_s3_class(fit, "cat_fit")
  expect_true(is.finite(fit$log_l))
  expect_equal(fit$settings$order, 0)
  expect_true(all(vapply(fit$marginal, function(x) abs(sum(x) - 1) < 1e-8, logical(1))))
  expect_false(is.null(fit$cell_counts))
  expect_equal(names(fit$cell_counts), paste0("t", 1:ncol(y)))
})


test_that("em_cat handles non-sequential block IDs and stores block levels", {
  skip_on_cran()
  set.seed(1303)
  y <- simulate_cat(n_subjects = 40, n_time = 4, order = 1, n_categories = 2)
  y[sample(length(y), 20)] <- NA
  blocks <- rep(c(2, 5, 2), length.out = nrow(y))

  fit <- em_cat(y, order = 1, blocks = blocks, homogeneous = FALSE, max_iter = 15)

  expect_equal(fit$settings$n_blocks, 2)
  expect_equal(sort(fit$settings$block_levels), c("2", "5"))
  expect_equal(length(fit$marginal), 2)
  expect_equal(length(fit$transition), 2)
  expect_equal(names(fit$cell_counts), c("block_1", "block_2"))
  expect_true("t1_to_t1" %in% names(fit$cell_counts$block_1))
})


test_that("em_cat returns fit_cat-style order 1 cell_counts usable by ci_cat", {
  skip_on_cran()
  set.seed(1307)
  y <- simulate_cat(n_subjects = 50, n_time = 4, order = 1, n_categories = 2)
  y[sample(length(y), 25)] <- NA

  fit <- em_cat(y, order = 1, max_iter = 15, tol = 1e-6)

  expected_names <- c("t1_to_t1", paste0("t", 1:3, "_to_t", 2:4))
  expect_equal(names(fit$cell_counts), expected_names)
  expect_true(is.matrix(fit$cell_counts$t1_to_t2))

  ci <- ci_cat(fit)
  expect_s3_class(ci, "cat_ci")
})


test_that("em_cat order 1 does not report immediate convergence on first iteration", {
  skip_on_cran()
  set.seed(1304)
  y <- simulate_cat(n_subjects = 50, n_time = 4, order = 1, n_categories = 2)
  y[sample(length(y), 15)] <- NA

  fit <- em_cat(y, order = 1, max_iter = 20, tol = 1e-6)

  expect_true(fit$settings$n_iter > 1)
})


test_that("em_cat transition probabilities are valid for order 1", {
  skip_on_cran()
  set.seed(1308)
  y <- simulate_cat(n_subjects = 40, n_time = 4, order = 1, n_categories = 3)
  y[sample(length(y), 20)] <- NA

  fit <- em_cat(y, order = 1, max_iter = 15)

  for (t in 2:ncol(y)) {
    trans_t <- fit$transition[[paste0("t", t)]]
    expect_true(all(trans_t >= 0))
    expect_equal(rowSums(trans_t), rep(1, nrow(trans_t)), tolerance = 1e-8)
  }
})


test_that("em_cat homogeneous and one-block heterogeneous fits agree", {
  skip_on_cran()
  set.seed(1309)
  y <- simulate_cat(n_subjects = 50, n_time = 4, order = 1, n_categories = 2)
  y[sample(length(y), 25)] <- NA
  blocks_one <- rep(5, nrow(y))

  fit_h <- em_cat(y, order = 1, homogeneous = TRUE, max_iter = 15)
  fit_het_one <- em_cat(y, order = 1, blocks = blocks_one, homogeneous = FALSE, max_iter = 15)

  expect_equal(as.numeric(fit_h$log_l), as.numeric(fit_het_one$log_l), tolerance = 1e-8)
  expect_equal(as.numeric(fit_h$marginal$t1), as.numeric(fit_het_one$marginal$t1), tolerance = 1e-8)
})


test_that("em_cat accepts safeguard argument", {
  skip_on_cran()
  set.seed(1310)
  y <- simulate_cat(n_subjects = 40, n_time = 4, order = 1, n_categories = 2)
  y[sample(length(y), 18)] <- NA

  fit_safe <- em_cat(y, order = 1, safeguard = TRUE, max_iter = 15)
  fit_no_safe <- em_cat(y, order = 1, safeguard = FALSE, max_iter = 15)

  expect_s3_class(fit_safe, "cat_fit")
  expect_s3_class(fit_no_safe, "cat_fit")
})


test_that("em_cat order 2 reports not implemented", {
  skip_on_cran()
  set.seed(1305)
  y <- simulate_cat(n_subjects = 20, n_time = 4, order = 2, n_categories = 2)
  y[sample(length(y), 5)] <- NA

  expect_error(
    em_cat(y, order = 2),
    "not yet implemented"
  )
})


test_that("em_cat final log-likelihood is consistent when max_iter is reached", {
  skip_on_cran()
  set.seed(1306)
  y <- simulate_cat(n_subjects = 50, n_time = 4, order = 1, n_categories = 2)
  y[sample(length(y), 30)] <- NA

  expect_warning(
    fit <- em_cat(y, order = 1, max_iter = 1, tol = 1e-12),
    "did not converge"
  )

  ll_check <- logL_cat(
    y = y,
    order = 1,
    marginal = fit$marginal,
    transition = fit$transition,
    n_categories = 2,
    na_action = "marginalize"
  )

  expect_equal(as.numeric(fit$log_l), as.numeric(ll_check), tolerance = 1e-8)
  expect_equal(
    as.numeric(fit$aic),
    as.numeric(-2 * fit$log_l + 2 * fit$n_params),
    tolerance = 1e-8
  )
  expect_equal(
    as.numeric(fit$bic),
    as.numeric(-2 * fit$log_l + log(nrow(y)) * fit$n_params),
    tolerance = 1e-8
  )
})

#' EM algorithm for INAD model estimation
#'
#' Fits INAD models using the Expectation-Maximization algorithm.
#' This is an alternative to direct likelihood optimization.
#'
#' For Gaussian and CAT EM entry points, see \code{\link{em_gau}} and
#' \code{\link{em_cat}}. For CAT specifically, \code{fit_cat()} supports
#' \code{na_action = "em"} for orders 0/1 and \code{na_action = "marginalize"}
#' for order 2 missing-data fits.
#'
#' @param y Integer matrix with n_subjects rows and n_time columns.
#' @param order Model order (1 or 2). Order 0 does not require EM.
#' @param thinning Thinning operator: "binom", "pois", or "nbinom".
#' @param innovation Innovation distribution: "pois", "bell", or "nbinom".
#' @param blocks Optional integer vector of length n_subjects for block effects.
#' @param max_iter Maximum number of EM iterations.
#' @param tol Convergence tolerance for log-likelihood change.
#' @param alpha_init Optional initial values for alpha parameters.
#' @param theta_init Optional initial values for theta parameters.
#' @param tau_init Optional initial values for tau parameters.
#' @param nb_inno_size Size parameter for negative binomial innovation (if used).
#' @param safeguard Logical; if TRUE, use step-halving when likelihood decreases.
#' @param verbose Logical; if TRUE, print iteration progress.
#'
#' @return A list with class "inad_fit" containing estimated parameters.
#'
#' @examples
#' set.seed(1)
#' y <- simulate_inad(
#'   n_subjects = 50,
#'   n_time = 5,
#'   order = 1,
#'   thinning = "binom",
#'   innovation = "pois",
#'   alpha = 0.25,
#'   theta = 2
#' )
#' fit <- em_inad(y, order = 1, thinning = "binom", innovation = "pois", max_iter = 20, tol = 1e-6)
#' fit$log_l
#' @seealso \code{\link{em_gau}}, \code{\link{em_cat}}, \code{\link{fit_inad}},
#'   \code{\link{fit_cat}}
#' @export
em_inad <- function(y, order = 1, thinning = "binom", innovation = "pois",
                    blocks = NULL, max_iter = 200, tol = 1e-7,
                    alpha_init = NULL, theta_init = NULL, tau_init = NULL,
                    nb_inno_size = NULL, safeguard = TRUE, verbose = FALSE) {
  if (!is.matrix(y)) y <- as.matrix(y)
  if (any(y < 0) || any(y != floor(y))) stop("y must contain non-negative integers")
  n_subjects <- nrow(y); n_time <- ncol(y)
  if (!order %in% c(1, 2)) stop("EM algorithm supports order 1 or 2.")
  if (order >= n_time) stop("order must be less than n_time")
  thinning <- match.arg(thinning, c("binom", "pois", "nbinom"))
  innovation <- match.arg(innovation, c("pois", "bell", "nbinom"))
  block_info <- .normalize_blocks(blocks, n_subjects)
  blocks <- block_info$blocks_id
  n_blocks <- block_info$n_blocks
  
  if (!(thinning == "binom" && innovation == "pois")) {
    warning("EM is optimized for binom-pois. Other combinations use approximations.")
  }
  
  init <- .em_init(y, order, blocks, alpha_init, theta_init, tau_init, innovation = innovation)
  alpha <- init$alpha; theta <- init$theta; tau <- init$tau
  
  loglik_old <- .em_ll(y, order, thinning, innovation, alpha, theta, tau, blocks, nb_inno_size)
  loglik_trace <- loglik_old; converged <- FALSE; iter <- 0; loglik_new <- loglik_old
  
  if (verbose) cat(sprintf("EM iter 0: logL = %.6f\n", loglik_old))
  
  for (iter in 1:max_iter) {
    E_vals <- .em_e(y, order, thinning, innovation, alpha, theta, tau, blocks, nb_inno_size)
    updates <- .em_m(y, order, thinning, innovation, E_vals, blocks, nb_inno_size)
    alpha_new <- updates$alpha; theta_new <- updates$theta; tau_new <- updates$tau
    loglik_new <- .em_ll(y, order, thinning, innovation, alpha_new, theta_new, tau_new, blocks, nb_inno_size)
    
    if (safeguard && loglik_new + 1e-10 < loglik_old) {
      lam <- 0.5
      while (lam > 1/1024) {
        alpha_try <- (1 - lam) * alpha + lam * alpha_new
        theta_try <- (1 - lam) * theta + lam * theta_new
        tau_try <- (1 - lam) * tau + lam * tau_new
        loglik_try <- .em_ll(y, order, thinning, innovation, alpha_try, theta_try, tau_try, blocks, nb_inno_size)
        if (loglik_try + 1e-10 >= loglik_old) {
          alpha_new <- alpha_try; theta_new <- theta_try; tau_new <- tau_try; loglik_new <- loglik_try; break
        }
        lam <- lam / 2
      }
    }
    
    loglik_trace <- c(loglik_trace, loglik_new)
    if (verbose) cat(sprintf("EM iter %d: logL = %.6f\n", iter, loglik_new))
    
    if (abs(loglik_new - loglik_old) / (1 + abs(loglik_old)) < tol) {
      converged <- TRUE; alpha <- alpha_new; theta <- theta_new; tau <- tau_new; break
    }
    alpha <- alpha_new; theta <- theta_new; tau <- tau_new; loglik_old <- loglik_new
  }
  
  result <- list(
    alpha = alpha, theta = theta, tau = tau,
    log_l = loglik_new, iterations = iter, converged = converged, loglik_trace = loglik_trace,
    settings = list(order = order, thinning = thinning, innovation = innovation,
                    blocks = if (n_blocks > 1L) blocks else NULL,
                    block_levels = block_info$block_levels,
                    n_subjects = n_subjects, n_time = n_time, n_blocks = n_blocks, method = "em")
  )
  class(result) <- "inad_fit"
  result
}

#' @keywords internal
.em_init <- function(y, order, blocks, alpha_init, theta_init, tau_init, innovation = "pois") {
  n_time <- ncol(y); n_blocks <- length(unique(blocks))
  theta <- if (is.null(theta_init)) {
    mu0 <- pmax(colMeans(y), 1e-6)
    if (innovation == "bell") {
      th0 <- .bell_theta_from_mean(mu0)
      th0[!is.finite(th0)] <- 1
      pmax(th0, 1e-6)
    } else {
      mu0
    }
  } else pmax(theta_init, 1e-6)
  if (is.null(alpha_init)) {
    if (order == 1) {
      alpha <- numeric(n_time); alpha[1] <- 0
      for (i in 2:n_time) alpha[i] <- pmin(pmax(mean(y[,i]) / (2 * max(mean(y[,i-1]), 1e-8)), 1e-6), 0.99)
    } else {
      alpha1 <- alpha2 <- numeric(n_time); alpha1[1:2] <- alpha2[1:2] <- 0
      for (i in 3:n_time) {
        alpha1[i] <- pmin(pmax(mean(y[,i]) / (3 * max(mean(y[,i-1]), 1e-8)), 0.1), 0.9)
        alpha2[i] <- pmin(pmax(mean(y[,i]) / (3 * max(mean(y[,i-2]), 1e-8)), 0.1), 0.9)
      }
      alpha <- cbind(alpha1, alpha2)
    }
  } else alpha <- alpha_init
  tau <- if (is.null(tau_init)) rep(0, n_blocks) else { tau_init[1] <- 0; tau_init }
  list(alpha = alpha, theta = theta, tau = tau)
}

#' @keywords internal
.em_e <- function(y, order, thinning, innovation, alpha, theta, tau, blocks, nb_inno_size) {
  n <- nrow(y); N <- ncol(y)
  alpha_list <- .unpack_alpha(order, alpha, N); a1 <- alpha_list$a1; a2 <- alpha_list$a2
  if (order == 1) {
    E_Z <- E_W <- matrix(0, n, N); E_Z[,1] <- 0; E_W[,1] <- y[,1]
    for (i in 2:N) for (s in 1:n) {
      lam <- .inad_effective_innovation_param(theta[i], tau[blocks[s]], innovation)
      y_i <- y[s,i]; y_i1 <- y[s,i-1]
      if (y_i1 == 0) { E_Z[s,i] <- 0; E_W[s,i] <- y_i } else {
        z_vals <- 0:min(y_i1, y_i); w_vals <- y_i - z_vals
        log_pz <- log(pmax(.thin_vec(z_vals, y_i1, a1[i], thinning), 1e-300))
        log_pw <- log(pmax(.innov_vec(w_vals, lam, innovation, nb_inno_size), 1e-300))
        post <- exp(log_pz + log_pw - .log_sum_exp(log_pz + log_pw))
        E_Z[s,i] <- sum(z_vals * post); E_W[s,i] <- y_i - E_Z[s,i]
      }
    }
    list(E_Z = E_Z, E_W = E_W)
  } else {
    E_Z1 <- E_Z2 <- E_W <- matrix(0, n, N); E_W[,1] <- y[,1]
    for (s in 1:n) {
      lam <- .inad_effective_innovation_param(theta[2], tau[blocks[s]], innovation)
      y_2 <- y[s,2]; y_1 <- y[s,1]
      if (y_1 > 0) {
        z_vals <- 0:min(y_1, y_2); w_vals <- y_2 - z_vals
        log_pz <- log(pmax(.thin_vec(z_vals, y_1, a1[2], thinning), 1e-300))
        log_pw <- log(pmax(.innov_vec(w_vals, lam, innovation, nb_inno_size), 1e-300))
        post <- exp(log_pz + log_pw - .log_sum_exp(log_pz + log_pw))
        E_Z1[s,2] <- sum(z_vals * post)
      }
      E_W[s,2] <- y[s,2] - E_Z1[s,2]
    }
    for (i in 3:N) for (s in 1:n) {
      lam <- .inad_effective_innovation_param(theta[i], tau[blocks[s]], innovation)
      y_i <- y[s,i]; y_i1 <- y[s,i-1]; y_i2 <- y[s,i-2]
      if (y_i1 == 0 && y_i2 == 0) { E_W[s,i] <- y_i; next }
      grid <- expand.grid(z1 = 0:min(y_i1, y_i), z2 = 0:min(y_i2, y_i))
      grid <- grid[grid$z1 + grid$z2 <= y_i, ]
      if (nrow(grid) == 0) { E_W[s,i] <- y_i; next }
      log_pz1 <- log(pmax(.thin_vec(grid$z1, y_i1, a1[i], thinning), 1e-300))
      log_pz2 <- log(pmax(.thin_vec(grid$z2, y_i2, a2[i], thinning), 1e-300))
      log_pw <- log(pmax(.innov_vec(y_i - grid$z1 - grid$z2, lam, innovation, nb_inno_size), 1e-300))
      post <- exp(log_pz1 + log_pz2 + log_pw - .log_sum_exp(log_pz1 + log_pz2 + log_pw))
      E_Z1[s,i] <- sum(grid$z1 * post); E_Z2[s,i] <- sum(grid$z2 * post)
      E_W[s,i] <- y_i - E_Z1[s,i] - E_Z2[s,i]
    }
    list(E_Z1 = E_Z1, E_Z2 = E_Z2, E_W = E_W)
  }
}

#' @keywords internal
.em_m <- function(y, order, thinning, innovation, E_vals, blocks, nb_inno_size) {
  n <- nrow(y); N <- ncol(y); B <- length(unique(blocks))
  if (order == 1) {
    E_Z <- E_vals$E_Z; E_W <- E_vals$E_W
    alpha <- numeric(N); alpha[1] <- 0
    for (i in 2:N) alpha[i] <- pmax(pmin(sum(E_Z[,i]) / max(sum(y[,i-1]), 1), 0.9999), 1e-12)
    theta <- numeric(N); tau <- rep(0, B)
    for (k in 1:10) {
      if (innovation == "bell") {
        for (i in 1:N) {
          mu_i <- pmax(mean(E_W[, i] - tau[blocks]), 1e-12)
          th_i <- .bell_theta_from_mean(mu_i)
          theta[i] <- if (is.finite(th_i)) pmax(th_i, 1e-12) else 1e-12
        }
        mu_theta <- .bell_mean_from_theta(theta)
        if (B > 1) {
          for (b in 2:B) {
            idx <- which(blocks == b)
            if (length(idx) > 0) {
              tau[b] <- mean(colMeans(E_W[idx, , drop = FALSE]) - mu_theta)
            }
          }
        }
      } else {
        for (i in 1:N) theta[i] <- pmax(mean(E_W[,i] - tau[blocks]), 1e-12)
        if (B > 1) for (b in 2:B) { idx <- which(blocks == b); if (length(idx) > 0) tau[b] <- mean(rowMeans(E_W[idx,,drop=FALSE]) - mean(theta)) }
      }
    }
    list(alpha = alpha, theta = theta, tau = tau)
  } else {
    E_Z1 <- E_vals$E_Z1; E_Z2 <- E_vals$E_Z2; E_W <- E_vals$E_W
    alpha1 <- alpha2 <- numeric(N); alpha1[1:2] <- alpha2[1:2] <- 0
    alpha1[2] <- pmax(pmin(sum(E_Z1[,2]) / max(sum(y[,1]), 1), 0.9999), 1e-12)
    for (i in 3:N) {
      alpha1[i] <- pmax(pmin(sum(E_Z1[,i]) / max(sum(y[,i-1]), 1), 0.9999), 1e-12)
      alpha2[i] <- pmax(pmin(sum(E_Z2[,i]) / max(sum(y[,i-2]), 1), 0.9999), 1e-12)
    }
    theta <- numeric(N); tau <- rep(0, B)
    for (k in 1:10) {
      if (innovation == "bell") {
        for (i in 1:N) {
          mu_i <- pmax(mean(E_W[, i] - tau[blocks]), 1e-12)
          th_i <- .bell_theta_from_mean(mu_i)
          theta[i] <- if (is.finite(th_i)) pmax(th_i, 1e-12) else 1e-12
        }
        mu_theta <- .bell_mean_from_theta(theta)
        if (B > 1) {
          for (b in 2:B) {
            idx <- which(blocks == b)
            if (length(idx) > 0) {
              tau[b] <- mean(colMeans(E_W[idx, , drop = FALSE]) - mu_theta)
            }
          }
        }
      } else {
        for (i in 1:N) theta[i] <- pmax(mean(E_W[,i] - tau[blocks]), 1e-12)
        if (B > 1) for (b in 2:B) { idx <- which(blocks == b); if (length(idx) > 0) tau[b] <- mean(rowMeans(E_W[idx,,drop=FALSE]) - mean(theta)) }
      }
    }
    list(alpha = cbind(alpha1, alpha2), theta = theta, tau = tau)
  }
}

#' @keywords internal
.em_ll <- function(y, order, thinning, innovation, alpha, theta, tau, blocks, nb_inno_size) {
  tryCatch(logL_inad(y = y, order = order, thinning = thinning, innovation = innovation,
                     alpha = alpha, theta = theta, blocks = blocks, tau = tau, nb_inno_size = nb_inno_size),
           error = function(e) -Inf)
}

#' Print method for INAD model fits
#'
#' @param x An object of class \code{inad_fit}.
#' @param digits Number of digits to print.
#' @param ... Unused.
#'
#' @return The input object, invisibly.
#' @export
print.inad_fit <- function(x, digits = 4, ...) {
  cat("\nINAD Model Fit")
  if (!is.null(x$settings$method) && x$settings$method == "em") cat(" (EM Algorithm)")
  cat("\n", paste(rep("=", 40), collapse = ""), "\n\n")
  cat("Order:", x$settings$order, " Thinning:", x$settings$thinning, " Innovation:", x$settings$innovation, "\n")
  cat("Log-likelihood:", round(x$log_l, digits), "\n")
  if (!is.null(x$aic)) cat("AIC:", round(x$aic, digits), "\n")
  if (!is.null(x$bic)) cat("BIC:", round(x$bic, digits), "\n")
  if (!is.null(x$n_params)) cat("Parameters:", x$n_params, "\n")
  if (!is.null(x$n_missing)) cat("Missing values:", x$n_missing, "\n")
  if (!is.null(x$convergence)) cat("Convergence code:", x$convergence, "\n")
  if (!is.null(x$iterations)) cat("Iterations:", x$iterations, " Converged:", x$converged, "\n")
  invisible(x)
}

test_that("em_inad preserves original block labels in settings", {
  skip_on_cran()
  set.seed(77)
  y <- matrix(rpois(20 * 4, lambda = 2), nrow = 20, ncol = 4)
  blocks <- rep(c("grp_5", "grp_2"), each = 10)

  fit <- em_inad(
    y,
    order = 1,
    thinning = "binom",
    innovation = "pois",
    blocks = blocks,
    max_iter = 3,
    tol = 1e-5,
    verbose = FALSE
  )

  expect_s3_class(fit, "inad_fit")
  expect_equal(sort(fit$settings$block_levels), c("grp_2", "grp_5"))
  expect_equal(length(fit$tau), 2)
})

# test_homogeneity_cat.R - Likelihood ratio test for homogeneity in categorical AD

#' Likelihood ratio test for homogeneity across groups (categorical AD data)
#'
#' Tests whether multiple groups share the same transition probability parameters
#' in a categorical antedependence model.
#'
#' @param y Integer matrix with n_subjects rows and n_time columns. Each entry
#'   should be a category code from 1 to c. Can be NULL if both fit_null and
#'   fit_alt are provided.
#' @param blocks Integer vector of length n_subjects specifying group membership.
#'   Required unless pre-fitted models are provided.
#' @param order Antedependence order p. Default is 1.
#' @param n_categories Number of categories. If NULL, inferred from data.
#' @param fit_null Optional pre-fitted homogeneous model (class "cat_fit" with
#'   homogeneous = TRUE). If provided, y is not required for fitting under H0.
#' @param fit_alt Optional pre-fitted heterogeneous model (class "cat_fit" with
#'   homogeneous = FALSE). If provided, y is not required for fitting under H1.
#' @param test Type of test statistic. One of \code{"lrt"} (default),
#'   \code{"score"}, or \code{"mlrt"}.
#'
#' @return A list of class \code{"cat_lrt"} containing:
#' \describe{
#'   \item{method}{Inference method used: one of \code{"lrt"}, \code{"score"},
#'     \code{"mlrt"}, or \code{"wald"}.}
#'   \item{lrt_stat}{Likelihood ratio test statistic}
#'   \item{df}{Degrees of freedom}
#'   \item{p_value}{P-value from chi-square distribution}
#'   \item{fit_null}{Fitted homogeneous model (H0)}
#'   \item{fit_alt}{Fitted heterogeneous model (H1)}
#'   \item{n_groups}{Number of groups}
#'   \item{table}{Summary data frame}
#' }
#'
#' @details
#' The null hypothesis is that all G groups share the same transition probability
#' parameters:
#' \deqn{H_0: \pi^{(1)} = \pi^{(2)} = \ldots = \pi^{(G)}}
#'
#' The alternative hypothesis allows each group to have its own parameters.
#'
#' The degrees of freedom are:
#' \deqn{df = (G-1) \times k}
#' where G is the number of groups and k is the number of free parameters per
#' population.
#'
#' @examples
#' \donttest{
#' # Simulate data with different transition probabilities for two groups
#' set.seed(123)
#' marg1 <- list(t1 = c(0.7, 0.3))
#' marg2 <- list(t1 = c(0.4, 0.6))
#' trans1 <- list(t2 = matrix(c(0.9, 0.1, 0.2, 0.8), 2, byrow = TRUE),
#'                t3 = matrix(c(0.9, 0.1, 0.2, 0.8), 2, byrow = TRUE))
#' trans2 <- list(t2 = matrix(c(0.5, 0.5, 0.5, 0.5), 2, byrow = TRUE),
#'                t3 = matrix(c(0.5, 0.5, 0.5, 0.5), 2, byrow = TRUE))
#'
#' y1 <- simulate_cat(100, 3, order = 1, n_categories = 2,
#'                    marginal = marg1, transition = trans1)
#' y2 <- simulate_cat(100, 3, order = 1, n_categories = 2,
#'                    marginal = marg2, transition = trans2)
#' y <- rbind(y1, y2)
#' blocks <- c(rep(1, 100), rep(2, 100))
#'
#' # Test homogeneity
#' test <- test_homogeneity_cat(y, blocks, order = 1)
#' print(test)
#' }
#'
#' @references
#' Xie, Y. and Zimmerman, D. L. (2013). Antedependence models for nonstationary
#' categorical longitudinal data with ignorable missingness: likelihood-based
#' inference. \emph{Statistics in Medicine}, 32, 3274-3289.
#'
#' @seealso \code{\link{fit_cat}}, \code{\link{test_order_cat}}
#'
#' @export
test_homogeneity_cat <- function(y = NULL, blocks = NULL, order = 1,
                                 n_categories = NULL,
                                 fit_null = NULL, fit_alt = NULL,
                                 test = c("lrt", "score", "mlrt")) {
  test <- match.arg(test)
  
  # Validate that we have either y+blocks or pre-fitted models
  if (is.null(y) && (is.null(fit_null) || is.null(fit_alt))) {
    stop("Either y and blocks must be provided, or both fit_null and fit_alt must be provided")
  }
  use_missing <- !is.null(y) && anyNA(y)
  na_action_fit <- if (use_missing) "marginalize" else "fail"
  
  if (!is.null(y) && is.null(blocks)) {
    stop("blocks must be provided when y is provided")
  }
  
  # Fit models if not provided
  if (is.null(fit_null)) {
    fit_null <- fit_cat(y, order = order, blocks = blocks,
                        homogeneous = TRUE, n_categories = n_categories,
                        na_action = na_action_fit)
  } else {
    if (!inherits(fit_null, "cat_fit")) {
      stop("fit_null must be a cat_fit object")
    }
    if (!fit_null$settings$homogeneous) {
      warning("fit_null should be a homogeneous model (homogeneous = TRUE)")
    }
  }
  
  if (is.null(fit_alt)) {
    fit_alt <- fit_cat(y, order = order, blocks = blocks,
                       homogeneous = FALSE, n_categories = n_categories,
                       na_action = na_action_fit)
  } else {
    if (!inherits(fit_alt, "cat_fit")) {
      stop("fit_alt must be a cat_fit object")
    }
    if (fit_alt$settings$homogeneous) {
      warning("fit_alt should be a heterogeneous model (homogeneous = FALSE)")
    }
  }
  
  # Verify they have same order
  if (fit_null$settings$order != fit_alt$settings$order) {
    stop("fit_null and fit_alt must have the same order")
  }
  
  # Extract info
  log_l_null <- fit_null$log_l
  log_l_alt <- fit_alt$log_l
  n_params_null <- fit_null$n_params
  n_params_alt <- fit_alt$n_params
  n_groups <- fit_alt$settings$n_blocks
  
  # Compute LRT statistic
  lrt_stat_raw <- -2 * (log_l_null - log_l_alt)
  
  # Degrees of freedom = (G-1) * k, where k is params per population
  df <- n_params_alt - n_params_null
  
  # Select test statistic
  stat_value <- lrt_stat_raw
  e_hat_mlrt <- NA_real_
  if (identical(test, "score")) {
    if (is.null(y)) {
      stop("y and blocks must be provided when test = 'score'")
    }
    if (anyNA(y) || .cat_fit_uses_missing_likelihood(fit_null) || .cat_fit_uses_missing_likelihood(fit_alt)) {
      .stop_cat_missing_inference("test_homogeneity_cat(test = 'score')")
    }
    stat_value <- .cat_score_homogeneity(
      y = y,
      blocks = blocks,
      p = fit_null$settings$order,
      c = fit_null$settings$n_categories,
      fit_null = fit_null
    )
  } else if (identical(test, "mlrt")) {
    p <- fit_null$settings$order
    c <- fit_null$settings$n_categories
    n_subjects <- fit_null$settings$n_subjects
    n_time <- fit_null$settings$n_time
    blocks_eff <- .cat_resolve_blocks(blocks, fit_alt)

    if (length(blocks_eff) != n_subjects) {
      stop("blocks length does not match n_subjects for modified homogeneity test")
    }

    simulate_fn <- function() {
      simulate_cat(
        n_subjects = n_subjects,
        n_time = n_time,
        order = p,
        n_categories = c,
        marginal = fit_null$marginal,
        transition = fit_null$transition,
        blocks = blocks_eff,
        homogeneous = TRUE
      )
    }
    lrt_raw_fn <- function(y_b) {
      fit0_b <- fit_cat(
        y_b,
        order = p,
        blocks = blocks_eff,
        homogeneous = TRUE,
        n_categories = c,
        na_action = "fail"
      )
      fit1_b <- fit_cat(
        y_b,
        order = p,
        blocks = blocks_eff,
        homogeneous = FALSE,
        n_categories = c,
        na_action = "fail"
      )
      -2 * (fit0_b$log_l - fit1_b$log_l)
    }

    e_hat_mlrt <- .cat_mlrt_expected_lrt_boot(simulate_fn, lrt_raw_fn)
    if (is.finite(e_hat_mlrt) && e_hat_mlrt > 0) {
      stat_value <- lrt_stat_raw * df / e_hat_mlrt
    } else {
      warning(
        "Modified LRT scaling factor could not be estimated; returning unmodified LRT statistic.",
        call. = FALSE
      )
    }
  }

  # P-value from chi-square
  p_value <- stats::pchisq(stat_value, df = df, lower.tail = FALSE)
  
  # Build summary table
  table_df <- data.frame(
    model = c("Homogeneous (H0)", "Heterogeneous (H1)"),
    log_l = c(log_l_null, log_l_alt),
    n_params = c(n_params_null, n_params_alt),
    aic = c(fit_null$aic, fit_alt$aic),
    bic = c(fit_null$bic, fit_alt$bic)
  )
  
  # Assemble output
  out <- list(
    method = test,
    lrt_stat = stat_value,
    statistic = stat_value,
    lrt_stat_raw = lrt_stat_raw,
    e_hat_mlrt = e_hat_mlrt,
    df = df,
    p_value = p_value,
    test = test,
    fit_null = fit_null,
    fit_alt = fit_alt,
    n_groups = n_groups,
    order = fit_null$settings$order,
    table = table_df
  )
  
  class(out) <- "cat_lrt"
  out
}


#' Likelihood ratio test for time-invariance (categorical data)
#'
#' Tests whether transition probabilities are constant over time in a
#' categorical antedependence model.
#'
#' @param y Integer matrix with n_subjects rows and n_time columns. Each entry
#'   should be a category code from 1 to c.
#' @param order Antedependence order p. Default is 1.
#' @param blocks Optional integer vector of length n_subjects specifying group
#'   membership.
#' @param homogeneous Logical. If TRUE (default), parameters are shared across
#'   all groups.
#' @param n_categories Number of categories. If NULL, inferred from data.
#' @param test Type of test statistic. One of \code{"lrt"} (default),
#'   \code{"score"}, or \code{"mlrt"}.
#'
#' @return A list of class \code{"cat_lrt"} containing:
#' \describe{
#'   \item{method}{Inference method used: one of \code{"lrt"}, \code{"score"},
#'     \code{"mlrt"}, or \code{"wald"}.}
#'   \item{lrt_stat}{Likelihood ratio test statistic}
#'   \item{df}{Degrees of freedom}
#'   \item{p_value}{P-value from chi-square distribution}
#'   \item{fit_null}{Fitted time-invariant model (H0)}
#'   \item{fit_alt}{Fitted time-varying model (H1)}
#'   \item{table}{Summary data frame}
#' }
#'
#' @details
#' The null hypothesis is that all transition probabilities (for k > p) are
#' equal across time:
#' \deqn{H_0: \pi_{y_k | y_{k-p}, \ldots, y_{k-1}} \text{ is constant for } k = p+1, \ldots, n}
#'
#' This reduces (n-p) separate transition matrices/arrays to a single one.
#'
#' The degrees of freedom are:
#' \deqn{df = (c-1) \times c^p \times (n - p - 1)}
#'
#' This function currently supports complete data only. If \code{y} contains
#' missing values, use model-fitting functions (for example \code{fit_cat})
#' directly with missing-data handling instead of this test wrapper.
#'
#' @examples
#' \donttest{
#' # Simulate data with time-invariant transitions
#' set.seed(123)
#' y <- simulate_cat(200, 6, order = 1, n_categories = 2)
#'
#' # Test time-invariance
#' test <- test_timeinvariance_cat(y, order = 1)
#' print(test)
#' }
#'
#' @references
#' Xie, Y. and Zimmerman, D. L. (2013). Antedependence models for nonstationary
#' categorical longitudinal data with ignorable missingness: likelihood-based
#' inference. \emph{Statistics in Medicine}, 32, 3274-3289.
#'
#' @seealso \code{\link{fit_cat}}, \code{\link{test_order_cat}}
#'
#' @export
test_timeinvariance_cat <- function(y, order = 1, blocks = NULL,
                                    homogeneous = TRUE, n_categories = NULL,
                                    test = c("lrt", "score", "mlrt")) {
  test <- match.arg(test)
  if (anyNA(y)) {
    .stop_cat_missing_inference("test_timeinvariance_cat")
  }
  
  # Validate data
  validated <- .validate_y_cat(y, n_categories)
  y <- validated$y
  c <- validated$n_categories
  n_time <- ncol(y)
  n_subjects <- nrow(y)
  p <- as.integer(order)
  
  if (p < 1) {
    stop("Time-invariance test requires order >= 1")
  }
  if (p >= n_time) {
    stop("order must be less than n_time")
  }
  
  # Fit unconstrained (time-varying) model - this is the alternative
  fit_alt <- fit_cat(y, order = p, blocks = blocks,
                     homogeneous = homogeneous, n_categories = c)
  
  # Fit constrained (time-invariant) model
  # This requires pooling all time points p+1 to n
  fit_null <- .fit_cat_timeinvariant(y, p, c, blocks, homogeneous)
  
  # Compute LRT statistic
  lrt_stat_raw <- -2 * (fit_null$log_l - fit_alt$log_l)
  
  # Degrees of freedom
  # Time-varying has (n-p) separate transition matrices, each with (c-1)*c^p params
  # Time-invariant has 1 transition matrix with (c-1)*c^p params
  # df = (n-p-1) * (c-1) * c^p
  n_trans_reduced <- (n_time - p - 1) * (c - 1) * c^p
  
  # Also need to account for marginal if p >= 1
  # For now, we only test time-invariance of transitions (not marginal)
  df <- n_trans_reduced
  
  # Adjust for heterogeneous case
  if (!homogeneous && !is.null(blocks)) {
    n_blocks <- length(unique(blocks))
    df <- df * n_blocks
  }
  
  # Select test statistic
  stat_value <- lrt_stat_raw
  e_hat_mlrt <- NA_real_
  if (identical(test, "score")) {
    pops <- .cat_split_populations(y, blocks = blocks, fit = fit_null)
    stat_value <- 0
    for (pop in pops) {
      pooled_transition <- pop$transition[["pooled"]]
      stat_value <- stat_value + .cat_score_timeinvariance_single(
        y = pop$y,
        p = p,
        c = c,
        pooled_transition = pooled_transition
      )
    }
  } else if (identical(test, "mlrt")) {
    n_subjects <- fit_alt$settings$n_subjects
    blocks_eff <- .cat_resolve_blocks(blocks, fit_alt)
    sim_params <- .cat_timeinvariant_sim_params(fit_null)

    simulate_fn <- function() {
      simulate_cat(
        n_subjects = n_subjects,
        n_time = n_time,
        order = p,
        n_categories = c,
        marginal = sim_params$marginal,
        transition = sim_params$transition,
        blocks = blocks_eff,
        homogeneous = homogeneous
      )
    }
    lrt_raw_fn <- function(y_b) {
      fit1_b <- fit_cat(
        y_b,
        order = p,
        blocks = blocks_eff,
        homogeneous = homogeneous,
        n_categories = c,
        na_action = "fail"
      )
      fit0_b <- .fit_cat_timeinvariant(
        y_b,
        p = p,
        c = c,
        blocks = blocks_eff,
        homogeneous = homogeneous
      )
      -2 * (fit0_b$log_l - fit1_b$log_l)
    }

    e_hat_mlrt <- .cat_mlrt_expected_lrt_boot(simulate_fn, lrt_raw_fn)
    if (is.finite(e_hat_mlrt) && e_hat_mlrt > 0) {
      stat_value <- lrt_stat_raw * df / e_hat_mlrt
    } else {
      warning(
        "Modified LRT scaling factor could not be estimated; returning unmodified LRT statistic.",
        call. = FALSE
      )
    }
  }

  # P-value from chi-square
  p_value <- stats::pchisq(stat_value, df = df, lower.tail = FALSE)
  
  # Build summary table
  table_df <- data.frame(
    model = c("Time-invariant (H0)", "Time-varying (H1)"),
    log_l = c(fit_null$log_l, fit_alt$log_l),
    n_params = c(fit_null$n_params, fit_alt$n_params),
    aic = c(fit_null$aic, fit_alt$aic),
    bic = c(fit_null$bic, fit_alt$bic)
  )
  
  # Assemble output
  out <- list(
    method = test,
    lrt_stat = stat_value,
    statistic = stat_value,
    lrt_stat_raw = lrt_stat_raw,
    e_hat_mlrt = e_hat_mlrt,
    df = df,
    p_value = p_value,
    test = test,
    fit_null = fit_null,
    fit_alt = fit_alt,
    order = p,
    table = table_df
  )
  
  class(out) <- "cat_lrt"
  out
}


#' Fit time-invariant categorical AD model
#'
#' Internal function to fit a model where transition probabilities are
#' constant across time (for k > p).
#'
#' @param y Data matrix
#' @param p Order
#' @param c Number of categories
#' @param blocks Block vector (or NULL)
#' @param homogeneous Whether to pool across blocks
#'
#' @return A cat_fit-like object with time-invariant transitions
#'
#' @keywords internal
.fit_cat_timeinvariant <- function(y, p, c, blocks = NULL, homogeneous = TRUE) {
  n_subjects <- nrow(y)
  n_time <- ncol(y)
  
  # Process blocks
  if (is.null(blocks)) {
    blocks <- rep(1L, n_subjects)
  }
  n_blocks <- max(blocks)
  
  if (homogeneous) {
    n_pops <- 1
  } else {
    n_pops <- n_blocks
  }
  
  # For each population, pool transition counts across all time points
  if (n_pops == 1) {
    result <- .fit_cat_timeinvariant_single(y, p, c, n_time, subject_mask = NULL)
    marginal <- result$marginal
    transition <- result$transition
    log_l <- result$log_l
    n_params <- result$n_params
  } else {
    marginal <- vector("list", n_pops)
    transition <- vector("list", n_pops)
    log_l <- 0
    n_params <- 0
    
    for (g in seq_len(n_pops)) {
      mask <- (blocks == g)
      result_g <- .fit_cat_timeinvariant_single(y, p, c, n_time, subject_mask = mask)
      marginal[[g]] <- result_g$marginal
      transition[[g]] <- result_g$transition
      log_l <- log_l + result_g$log_l
      n_params <- n_params + result_g$n_params
    }
    names(marginal) <- paste0("block_", seq_len(n_pops))
    names(transition) <- paste0("block_", seq_len(n_pops))
  }
  
  # Compute AIC and BIC
  aic <- -2 * log_l + 2 * n_params
  bic <- -2 * log_l + n_params * log(n_subjects)
  
  out <- list(
    marginal = marginal,
    transition = transition,
    log_l = log_l,
    aic = aic,
    bic = bic,
    n_params = n_params,
    convergence = 0L,
    settings = list(
      order = p,
      n_categories = c,
      n_time = n_time,
      n_subjects = n_subjects,
      blocks = if (n_blocks > 1) blocks else NULL,
      homogeneous = homogeneous,
      n_blocks = n_blocks,
      time_invariant = TRUE
    )
  )
  
  class(out) <- "cat_fit"
  out
}


#' Fit time-invariant model for single population
#'
#' @param y Data matrix
#' @param p Order
#' @param c Number of categories
#' @param n_time Number of time points
#' @param subject_mask Logical mask for subjects (NULL = all)
#'
#' @return List with marginal, transition, log_l, n_params
#'
#' @keywords internal
.fit_cat_timeinvariant_single <- function(y, p, c, n_time, subject_mask = NULL) {
  
  # Apply mask
  if (!is.null(subject_mask)) {
    y_sub <- y[subject_mask, , drop = FALSE]
  } else {
    y_sub <- y
  }
  N <- nrow(y_sub)
  
  # Storage
  marginal <- list()
  log_l <- 0
  
  # Handle initial time points (same as regular fit)
  for (k in seq_len(min(p, n_time))) {
    time_indices <- seq_len(k)
    counts_k <- .count_cells_table_cat(y_sub, time_indices, c, subject_mask = NULL)
    
    if (k == 1) {
      probs_k <- counts_k / N
      marginal[["t1"]] <- as.numeric(probs_k)
      names(marginal[["t1"]]) <- paste0("cat_", 1:c)
    } else {
      probs_k <- .counts_to_transition_probs(counts_k)
      marginal[[paste0("t", k, "_given_1to", k - 1)]] <- probs_k
    }
    
    log_l <- log_l + .loglik_contribution(counts_k, probs_k)
  }
  
  # Pool transition counts across all time points k = p+1 to n
  # Count all (Y_{k-p}, ..., Y_{k-1}, Y_k) patterns together
  pooled_counts <- array(0L, dim = rep(c, p + 1))
  
  for (k in (p + 1):n_time) {
    time_indices <- (k - p):k
    counts_k <- .count_cells_table_cat(y_sub, time_indices, c, subject_mask = NULL)
    pooled_counts <- pooled_counts + counts_k
  }
  
  # Compute pooled transition probabilities
  pooled_probs <- .counts_to_transition_probs(pooled_counts)
  
  # Store single transition matrix
  transition <- list(pooled = pooled_probs)
  
  # Compute log-likelihood contribution from transitions
  # Need to evaluate at each time point using the pooled probs
  for (k in (p + 1):n_time) {
    time_indices <- (k - p):k
    counts_k <- .count_cells_table_cat(y_sub, time_indices, c, subject_mask = NULL)
    log_l <- log_l + .loglik_contribution(counts_k, pooled_probs)
  }
  
  # Count parameters
  # Marginal: (c-1) for t1, plus (c-1)*c^{k-1} for each k=2,...,p
  # Transition: (c-1)*c^p (just one matrix, not n-p)
  n_params_marginal <- 0
  for (k in seq_len(min(p, n_time))) {
    if (k == 1) {
      n_params_marginal <- n_params_marginal + (c - 1)
    } else {
      n_params_marginal <- n_params_marginal + (c - 1) * c^(k - 1)
    }
  }
  n_params_trans <- (c - 1) * c^p
  n_params <- n_params_marginal + n_params_trans
  
  list(
    marginal = marginal,
    transition = transition,
    log_l = log_l,
    n_params = n_params
  )
}

# em_cat.R - EM estimation for categorical antedependence models

#' EM algorithm for categorical AD model estimation
#'
#' Fits categorical antedependence models with missing outcomes using the
#' Expectation-Maximization (EM) algorithm for orders 0 and 1.
#'
#' @param y Integer matrix with n_subjects rows and n_time columns. Values are
#'   category codes in \code{1, ..., n_categories}; \code{NA} is allowed.
#' @param order Antedependence order. Supported values are \code{0} and
#'   \code{1}. Order \code{2} is not yet implemented in \code{em_cat()}.
#' @param blocks Optional block/group vector of length \code{n_subjects}. Any
#'   coding is accepted (e.g., non-sequential integers or factor levels).
#' @param homogeneous Logical. If \code{TRUE}, a single parameter set is fitted
#'   across blocks. If \code{FALSE}, separate parameters are fitted by block.
#' @param n_categories Number of categories. If \code{NULL}, inferred from
#'   observed data.
#' @param max_iter Maximum number of EM iterations.
#' @param tol Convergence tolerance on absolute log-likelihood change.
#' @param epsilon Small positive constant used for smoothing and numerical
#'   stability.
#' @param safeguard Logical; if \code{TRUE}, apply step-halving when an M-step
#'   update decreases observed-data log-likelihood.
#' @param verbose Logical; if \code{TRUE}, print EM progress.
#'
#' @return A \code{cat_fit} object with fields matching \code{\link{fit_cat}}.
#'   In EM mode, \code{cell_counts} stores expected counts from the final
#'   E-step, with \code{settings$cell_counts_type = "expected"}.
#'
#' @details
#' For complete data (no missing values), this function defers to
#' \code{\link{fit_cat}} with closed-form MLEs.
#'
#' For missing data and orders 0/1, each EM iteration computes expected
#' sufficient statistics with a forward-backward E-step, then updates
#' probabilities by normalized expected counts in the M-step.
#' If \code{safeguard = TRUE}, a step-halving line search is applied to the
#' M-step update whenever the observed-data likelihood decreases.
#'
#' A final E-step is run before returning so that \code{log_l}/AIC/BIC and
#' expected cell counts correspond exactly to the returned parameter values.
#'
#' @examples
#' set.seed(1)
#' y <- simulate_cat(n_subjects = 40, n_time = 5, order = 1, n_categories = 3)
#' y[sample(length(y), 10)] <- NA
#' fit <- em_cat(y, order = 1, n_categories = 3, max_iter = 20, tol = 1e-5)
#' fit$settings$na_action
#'
#' @seealso \code{\link{fit_cat}}, \code{\link{logL_cat}}
#' @export
em_cat <- function(y, order = 1, blocks = NULL, homogeneous = TRUE,
                   n_categories = NULL, max_iter = 100, tol = 1e-6,
                   epsilon = 1e-8, safeguard = TRUE, verbose = FALSE) {
  validated <- .validate_y_cat(y, n_categories = n_categories, allow_na = TRUE)
  y <- validated$y
  c <- validated$n_categories

  n_subjects <- nrow(y)
  n_time <- ncol(y)
  p <- as.integer(order)

  if (!p %in% c(0L, 1L, 2L)) stop("order must be 0, 1, or 2")
  if (p >= n_time) stop("order must be less than number of time points")
  if (max_iter < 1) stop("max_iter must be >= 1")
  if (!is.finite(tol) || tol <= 0) stop("tol must be a positive finite number")
  if (!is.finite(epsilon) || epsilon <= 0) stop("epsilon must be a positive finite number")
  if (!is.logical(safeguard) || length(safeguard) != 1L || is.na(safeguard)) {
    stop("safeguard must be TRUE or FALSE")
  }

  if (p == 2L) {
    stop(
      paste0(
        "em_cat with order=2 is not yet implemented.\n",
        "Use fit_cat(..., na_action = 'marginalize') for order 2 with missing data."
      )
    )
  }

  block_info <- .normalize_blocks_em_cat(blocks, n_subjects)
  blocks_id <- block_info$blocks_id
  block_levels <- block_info$block_levels
  n_blocks <- block_info$n_blocks
  homogeneous_effective <- isTRUE(homogeneous) || n_blocks == 1L

  # Complete data: defer to closed-form MLEs.
  if (!any(is.na(y))) {
    if (verbose) message("No missing data; using closed-form MLEs via fit_cat().")
    fit <- fit_cat(
      y = y,
      order = p,
      blocks = blocks_id,
      homogeneous = homogeneous,
      n_categories = c,
      na_action = "complete"
    )
    fit$settings$method <- "closed_form"
    fit$settings$block_levels <- block_levels
    return(fit)
  }

  init <- .initialize_parameters_cat_em(
    y = y, order = p, blocks_id = blocks_id, homogeneous = homogeneous_effective,
    c = c, n_time = n_time, epsilon = epsilon, verbose = verbose
  )
  marginal <- init$marginal
  transition <- init$transition

  max_iter <- as.integer(max_iter)
  converged <- FALSE
  iter <- 0L
  logL_prev <- -Inf
  logL_current <- -Inf
  delta_logL <- Inf

  for (iter in seq_len(max_iter)) {
    e_result <- .e_step_cat_em(
      y = y, order = p, marginal = marginal, transition = transition,
      blocks_id = blocks_id, homogeneous = homogeneous_effective, c = c
    )

    if (isTRUE(e_result$failed)) {
      stop(sprintf("EM iteration %d: %s", iter, e_result$message))
    }

    logL_current <- e_result$log_likelihood
    delta_logL <- logL_current - logL_prev

    if (verbose) {
      message(sprintf("Iteration %d: logL = %.6f, change = %.6f", iter, logL_current, delta_logL))
    }

    # Convergence check is done before M-step.
    if (iter > 1L && abs(delta_logL) < tol) {
      converged <- TRUE
      break
    }

    if (delta_logL < -1e-6) {
      warning(
        sprintf(
          "Likelihood decreased at iteration %d (%.6f -> %.6f); possible numerical issue.",
          iter, logL_prev, logL_current
        )
      )
    }

    logL_prev <- logL_current

    params_new <- .m_step_cat_em(
      counts = e_result$counts, order = p, c = c, n_time = n_time,
      n_blocks = n_blocks, homogeneous = homogeneous_effective, epsilon = epsilon
    )

    if (isTRUE(safeguard)) {
      safeguarded <- .safeguard_update_cat_em(
        y = y,
        order = p,
        old_marginal = marginal,
        old_transition = transition,
        new_marginal = params_new$marginal,
        new_transition = params_new$transition,
        blocks_id = blocks_id,
        homogeneous = homogeneous_effective,
        c = c,
        logL_old = logL_current,
        n_time = n_time,
        n_blocks = n_blocks
      )
      marginal <- safeguarded$marginal
      transition <- safeguarded$transition
      if (verbose && safeguarded$step < 1) {
        message(sprintf("Iteration %d: safeguard step-halving accepted (step = %.6f).", iter, safeguarded$step))
      }
      if (!safeguarded$accepted) {
        warning(sprintf("Iteration %d: safeguard rejected M-step update; keeping previous parameters.", iter))
      }
    } else {
      marginal <- params_new$marginal
      transition <- params_new$transition
    }
  }

  # Final E-step ensures returned fit metrics/cell counts correspond exactly
  # to returned parameter values, including max_iter exits.
  e_final <- .e_step_cat_em(
    y = y, order = p, marginal = marginal, transition = transition,
    blocks_id = blocks_id, homogeneous = homogeneous_effective, c = c
  )
  if (isTRUE(e_final$failed)) {
    stop(sprintf("Final likelihood evaluation failed: %s", e_final$message))
  }
  logL_current <- e_final$log_likelihood
  delta_logL <- logL_current - logL_prev
  cell_counts <- .expected_counts_to_cell_counts_cat_em(
    counts = e_final$counts,
    order = p,
    n_time = n_time,
    n_blocks = n_blocks,
    homogeneous = homogeneous_effective
  )

  if (!converged && iter == max_iter && abs(delta_logL) >= tol) {
    warning(sprintf("EM did not converge after %d iterations (change = %.6f).", max_iter, delta_logL))
  }

  n_params <- .count_params_cat(
    order = p,
    n_categories = c,
      n_time = n_time,
      n_blocks = n_blocks,
      homogeneous = homogeneous_effective
  )

  aic <- -2 * logL_current + 2 * n_params
  bic <- -2 * logL_current + log(n_subjects) * n_params

  out <- list(
    marginal = marginal,
    transition = transition,
    log_l = as.numeric(logL_current),
    aic = as.numeric(aic),
    bic = as.numeric(bic),
    n_params = as.integer(n_params),
    cell_counts = cell_counts,
    convergence = if (converged) 0L else 1L,
    settings = list(
      order = p,
      n_categories = c,
      n_time = n_time,
      n_subjects = n_subjects,
      blocks = if (n_blocks > 1) blocks_id else NULL,
      homogeneous = homogeneous,
      homogeneous_effective = homogeneous_effective,
      n_blocks = n_blocks,
      na_action = "em",
      na_action_effective = "em",
      method = "em",
      max_iter = max_iter,
      tol = tol,
      epsilon = epsilon,
      safeguard = safeguard,
      n_iter = as.integer(iter),
      converged = converged,
      delta_log_l = as.numeric(delta_logL),
      block_levels = block_levels,
      cell_counts_type = "expected"
    )
  )

  class(out) <- "cat_fit"
  out
}


#' @keywords internal
.normalize_blocks_em_cat <- function(blocks, n_subjects) {
  if (is.null(blocks)) {
    return(list(blocks_id = rep(1L, n_subjects), block_levels = "1", n_blocks = 1L))
  }
  if (length(blocks) != n_subjects) {
    stop("blocks must have length equal to number of subjects (", n_subjects, ")")
  }
  if (any(is.na(blocks))) {
    stop("blocks must not contain missing values")
  }

  block_fac <- factor(blocks)
  list(
    blocks_id = as.integer(block_fac),
    block_levels = levels(block_fac),
    n_blocks = nlevels(block_fac)
  )
}


#' @keywords internal
.initialize_parameters_cat_em <- function(y, order, blocks_id, homogeneous,
                                          c, n_time, epsilon, verbose) {
  complete_subjects <- stats::complete.cases(y)
  n_complete <- sum(complete_subjects)
  n_blocks <- max(blocks_id)

  if (homogeneous || n_blocks == 1L) {
    if (n_complete >= 10L) {
      if (verbose) message("Initializing from complete cases.")
      return(.initialize_from_complete_cat_em(y[complete_subjects, , drop = FALSE], order, c, n_time, epsilon))
    }
    if (verbose) message("Initializing from random positive probabilities.")
    return(.initialize_uniform_cat_em(order, c, n_time))
  }

  marginal <- vector("list", n_blocks)
  transition <- vector("list", n_blocks)

  for (b in seq_len(n_blocks)) {
    idx_block <- which(blocks_id == b)
    idx_complete <- idx_block[complete_subjects[idx_block]]

    if (length(idx_complete) >= 5L) {
      params_b <- .initialize_from_complete_cat_em(y[idx_complete, , drop = FALSE], order, c, n_time, epsilon)
    } else {
      params_b <- .initialize_uniform_cat_em(order, c, n_time)
    }
    marginal[[b]] <- params_b$marginal
    transition[[b]] <- params_b$transition
  }

  names(marginal) <- paste0("block_", seq_len(n_blocks))
  names(transition) <- paste0("block_", seq_len(n_blocks))
  list(marginal = marginal, transition = transition)
}


#' @keywords internal
.initialize_from_complete_cat_em <- function(y_complete, order, c, n_time, epsilon) {
  if (order == 0L) {
    marginal <- list()
    for (t in seq_len(n_time)) {
      counts_t <- table(factor(y_complete[, t], levels = seq_len(c)))
      probs_t <- (as.numeric(counts_t) + epsilon) / (sum(counts_t) + c * epsilon)
      names(probs_t) <- paste0("cat_", seq_len(c))
      marginal[[paste0("t", t)]] <- probs_t
    }
    return(list(marginal = marginal, transition = list()))
  }

  counts_t1 <- table(factor(y_complete[, 1], levels = seq_len(c)))
  pi1 <- (as.numeric(counts_t1) + epsilon) / (sum(counts_t1) + c * epsilon)
  names(pi1) <- paste0("cat_", seq_len(c))
  marginal <- list(t1 = pi1)

  transition <- list()
  for (t in 2:n_time) {
    counts_t <- table(
      factor(y_complete[, t - 1], levels = seq_len(c)),
      factor(y_complete[, t], levels = seq_len(c))
    )
    counts_t <- counts_t + epsilon
    trans_t <- sweep(counts_t, 1, rowSums(counts_t), "/")
    transition[[paste0("t", t)]] <- unname(trans_t)
  }

  list(marginal = marginal, transition = transition)
}


#' @keywords internal
.initialize_uniform_cat_em <- function(order, c, n_time) {
  if (order == 0L) {
    marginal <- list()
    for (t in seq_len(n_time)) {
      probs <- rep(1 / c, c)
      names(probs) <- paste0("cat_", seq_len(c))
      marginal[[paste0("t", t)]] <- probs
    }
    return(list(marginal = marginal, transition = list()))
  }

  pi1 <- rep(1 / c, c)
  names(pi1) <- paste0("cat_", seq_len(c))
  marginal <- list(t1 = pi1)

  transition <- list()
  for (t in 2:n_time) {
    trans_t <- matrix(stats::rexp(c * c, rate = 1), nrow = c, ncol = c)
    trans_t <- sweep(trans_t, 1, rowSums(trans_t), "/")
    transition[[paste0("t", t)]] <- trans_t
  }

  list(marginal = marginal, transition = transition)
}


#' @keywords internal
.forward_backward_cat_order0 <- function(y_i, marginal, c) {
  n_time <- length(y_i)
  gamma <- matrix(0, nrow = n_time, ncol = c)
  scale <- numeric(n_time)
  log_likelihood <- 0

  for (t in seq_len(n_time)) {
    marg_t <- marginal[[paste0("t", t)]]
    if (is.null(marg_t)) marg_t <- marginal[[t]]
    if (is.null(marg_t) || length(marg_t) != c) {
      return(list(failed = TRUE, log_likelihood = -Inf, message = sprintf("Invalid marginal at t=%d", t)))
    }
    marg_t <- as.numeric(marg_t)
    marg_t <- marg_t / sum(marg_t)

    if (is.na(y_i[t])) {
      gamma[t, ] <- marg_t
      scale[t] <- 1
    } else {
      y_obs <- as.integer(y_i[t])
      if (y_obs < 1L || y_obs > c) {
        return(list(failed = TRUE, log_likelihood = -Inf, message = sprintf("Observed value out of range at t=%d", t)))
      }
      gamma[t, y_obs] <- 1
      scale[t] <- marg_t[y_obs]
    }

    if (!is.finite(scale[t]) || scale[t] <= 0) {
      return(list(failed = TRUE, log_likelihood = -Inf, message = sprintf("Zero probability at t=%d", t)))
    }

    log_likelihood <- log_likelihood + log(scale[t])
  }

  list(
    gamma = gamma,
    xi = array(0, dim = c(0, c, c)),
    scale = scale,
    log_likelihood = log_likelihood,
    failed = FALSE
  )
}


#' @keywords internal
.forward_backward_cat_order1 <- function(y_i, marginal, transition, c) {
  n_time <- length(y_i)
  alpha <- matrix(0, nrow = n_time, ncol = c)
  beta <- matrix(0, nrow = n_time, ncol = c)
  scale <- numeric(n_time)
  emission <- matrix(1, nrow = n_time, ncol = c)

  for (t in seq_len(n_time)) {
    if (!is.na(y_i[t])) {
      y_obs <- as.integer(y_i[t])
      if (y_obs < 1L || y_obs > c) {
        return(list(failed = TRUE, log_likelihood = -Inf, message = sprintf("Observed value out of range at t=%d", t)))
      }
      emission[t, ] <- 0
      emission[t, y_obs] <- 1
    }
  }

  pi1 <- as.numeric(marginal$t1)
  if (length(pi1) != c) {
    return(list(failed = TRUE, log_likelihood = -Inf, message = "Invalid t1 marginal"))
  }
  pi1 <- pi1 / sum(pi1)

  alpha[1, ] <- pi1 * emission[1, ]
  scale[1] <- sum(alpha[1, ])
  if (!is.finite(scale[1]) || scale[1] <= 0) {
    return(list(failed = TRUE, log_likelihood = -Inf, message = "Zero probability at t=1"))
  }
  alpha[1, ] <- alpha[1, ] / scale[1]

  if (n_time >= 2L) {
    for (t in 2:n_time) {
      trans_t <- transition[[paste0("t", t)]]
      if (is.null(trans_t) || any(dim(trans_t) != c(c, c))) {
        return(list(failed = TRUE, log_likelihood = -Inf, message = sprintf("Invalid transition at t=%d", t)))
      }

      alpha[t, ] <- as.numeric(alpha[t - 1, ] %*% trans_t) * emission[t, ]
      scale[t] <- sum(alpha[t, ])
      if (!is.finite(scale[t]) || scale[t] <= 0) {
        return(list(failed = TRUE, log_likelihood = -Inf, message = sprintf("Zero probability at t=%d", t)))
      }
      alpha[t, ] <- alpha[t, ] / scale[t]
    }
  }

  beta[n_time, ] <- 1
  if (n_time >= 2L) {
    for (t in (n_time - 1L):1L) {
      trans_next <- transition[[paste0("t", t + 1L)]]
      beta[t, ] <- as.numeric(trans_next %*% (emission[t + 1L, ] * beta[t + 1L, ])) / scale[t + 1L]
    }
  }

  gamma <- alpha * beta
  gamma_den <- rowSums(gamma)
  if (any(!is.finite(gamma_den) | gamma_den <= 0)) {
    return(list(failed = TRUE, log_likelihood = -Inf, message = "Failed to normalize posterior gamma"))
  }
  gamma <- gamma / gamma_den

  xi <- array(0, dim = c(n_time - 1L, c, c))
  if (n_time >= 2L) {
    for (t in 1:(n_time - 1L)) {
      trans_next <- transition[[paste0("t", t + 1L)]]
      w <- emission[t + 1L, ] * beta[t + 1L, ]

      for (a in seq_len(c)) {
        xi[t, a, ] <- alpha[t, a] * trans_next[a, ] * w
      }

      xi_sum <- sum(xi[t, , ])
      if (!is.finite(xi_sum) || xi_sum <= 0) {
        return(list(failed = TRUE, log_likelihood = -Inf, message = sprintf("Failed to normalize posterior xi at t=%d", t)))
      }
      xi[t, , ] <- xi[t, , ] / xi_sum
    }
  }

  list(
    gamma = gamma,
    xi = xi,
    scale = scale,
    log_likelihood = sum(log(scale)),
    failed = FALSE
  )
}


#' @keywords internal
.initialize_expected_counts_cat_em <- function(order, c, n_time, n_blocks, homogeneous) {
  if (order == 0L) {
    if (homogeneous || n_blocks == 1L) {
      return(list(marginal = array(0, dim = c(n_time, c)), transition = list()))
    }
    return(list(marginal = array(0, dim = c(n_blocks, n_time, c)), transition = list()))
  }

  if (homogeneous || n_blocks == 1L) {
    return(list(
      marginal = numeric(c),
      transition = array(0, dim = c(n_time - 1L, c, c))
    ))
  }

  list(
    marginal = array(0, dim = c(n_blocks, c)),
    transition = array(0, dim = c(n_blocks, n_time - 1L, c, c))
  )
}


#' @keywords internal
.accumulate_counts_cat_em <- function(counts, fb_result, order, block_i, homogeneous, n_time) {
  if (order == 0L) {
    if (homogeneous) {
      counts$marginal <- counts$marginal + fb_result$gamma
    } else {
      counts$marginal[block_i, , ] <- counts$marginal[block_i, , ] + fb_result$gamma
    }
    return(counts)
  }

  if (homogeneous) {
    counts$marginal <- counts$marginal + fb_result$gamma[1, ]
    counts$transition <- counts$transition + fb_result$xi
  } else {
    counts$marginal[block_i, ] <- counts$marginal[block_i, ] + fb_result$gamma[1, ]
    counts$transition[block_i, , , ] <- counts$transition[block_i, , , ] + fb_result$xi
  }

  counts
}


#' @keywords internal
.e_step_cat_em <- function(y, order, marginal, transition, blocks_id, homogeneous, c) {
  n_subjects <- nrow(y)
  n_time <- ncol(y)
  n_blocks <- max(blocks_id)

  expected_counts <- .initialize_expected_counts_cat_em(order, c, n_time, n_blocks, homogeneous)
  total_log_likelihood <- 0

  for (i in seq_len(n_subjects)) {
    y_i <- y[i, ]
    block_i <- blocks_id[i]

    if (homogeneous) {
      marg_i <- marginal
      trans_i <- transition
    } else {
      marg_i <- marginal[[block_i]]
      trans_i <- transition[[block_i]]
    }

    if (order == 0L) {
      fb_result <- .forward_backward_cat_order0(y_i, marg_i, c)
    } else {
      fb_result <- .forward_backward_cat_order1(y_i, marg_i, trans_i, c)
    }

    if (isTRUE(fb_result$failed)) {
      return(list(
        failed = TRUE,
        log_likelihood = -Inf,
        failed_subject = i,
        message = sprintf("Subject %d: %s", i, fb_result$message)
      ))
    }

    total_log_likelihood <- total_log_likelihood + fb_result$log_likelihood
    expected_counts <- .accumulate_counts_cat_em(
      counts = expected_counts, fb_result = fb_result, order = order,
      block_i = block_i, homogeneous = homogeneous, n_time = n_time
    )
  }

  list(
    counts = expected_counts,
    log_likelihood = total_log_likelihood,
    failed = FALSE
  )
}


#' Convert EM expected counts to fit_cat-style cell_counts
#'
#' @param counts Expected counts from the E-step.
#' @param order Model order.
#' @param n_time Number of time points.
#' @param n_blocks Number of blocks.
#' @param homogeneous Logical; whether parameters are shared across blocks.
#'
#' @return A list shaped like \code{fit_cat(...)} \code{cell_counts}.
#'
#' @keywords internal
.expected_counts_to_cell_counts_cat_em <- function(counts, order, n_time, n_blocks, homogeneous) {
  if (order == 0L) {
    if (homogeneous || n_blocks == 1L) {
      out <- list()
      for (t in seq_len(n_time)) {
        out[[paste0("t", t)]] <- as.numeric(counts$marginal[t, ])
      }
      return(out)
    }

    out <- vector("list", n_blocks)
    for (b in seq_len(n_blocks)) {
      out_b <- list()
      for (t in seq_len(n_time)) {
        out_b[[paste0("t", t)]] <- as.numeric(counts$marginal[b, t, ])
      }
      out[[b]] <- out_b
    }
    names(out) <- paste0("block_", seq_len(n_blocks))
    return(out)
  }

  if (order == 1L) {
    if (homogeneous || n_blocks == 1L) {
      out <- list()
      out[["t1_to_t1"]] <- as.numeric(counts$marginal)
      for (t in 2:n_time) {
        out[[paste0("t", t - 1L, "_to_t", t)]] <- unname(counts$transition[t - 1L, , ])
      }
      return(out)
    }

    out <- vector("list", n_blocks)
    for (b in seq_len(n_blocks)) {
      out_b <- list()
      out_b[["t1_to_t1"]] <- as.numeric(counts$marginal[b, ])
      for (t in 2:n_time) {
        out_b[[paste0("t", t - 1L, "_to_t", t)]] <- unname(counts$transition[b, t - 1L, , ])
      }
      out[[b]] <- out_b
    }
    names(out) <- paste0("block_", seq_len(n_blocks))
    return(out)
  }

  stop("Only orders 0 and 1 are supported in em_cat")
}


#' Safeguard M-step update via step-halving
#'
#' @param y Data matrix.
#' @param order Model order.
#' @param old_marginal Previous marginal parameters.
#' @param old_transition Previous transition parameters.
#' @param new_marginal Proposed marginal parameters from M-step.
#' @param new_transition Proposed transition parameters from M-step.
#' @param blocks_id Normalized block ids.
#' @param homogeneous Whether parameters are shared across blocks.
#' @param c Number of categories.
#' @param logL_old Observed-data log-likelihood at old parameters.
#' @param n_time Number of time points.
#' @param n_blocks Number of blocks.
#'
#' @return List with safeguarded parameters, acceptance flag, and step size.
#'
#' @keywords internal
.safeguard_update_cat_em <- function(y, order, old_marginal, old_transition,
                                     new_marginal, new_transition,
                                     blocks_id, homogeneous, c, logL_old,
                                     n_time, n_blocks) {
  # Try full update first.
  e_new <- .e_step_cat_em(
    y = y, order = order, marginal = new_marginal, transition = new_transition,
    blocks_id = blocks_id, homogeneous = homogeneous, c = c
  )
  if (!isTRUE(e_new$failed) && (e_new$log_likelihood + 1e-10 >= logL_old)) {
    return(list(marginal = new_marginal, transition = new_transition, accepted = TRUE, step = 1))
  }

  # Step-halving fallback.
  step <- 0.5
  while (step >= (1 / 1024)) {
    blended <- .blend_cat_params_em(
      old_marginal = old_marginal,
      old_transition = old_transition,
      new_marginal = new_marginal,
      new_transition = new_transition,
      step = step,
      order = order,
      n_time = n_time,
      n_blocks = n_blocks,
      homogeneous = homogeneous
    )
    e_try <- .e_step_cat_em(
      y = y, order = order, marginal = blended$marginal, transition = blended$transition,
      blocks_id = blocks_id, homogeneous = homogeneous, c = c
    )
    if (!isTRUE(e_try$failed) && (e_try$log_likelihood + 1e-10 >= logL_old)) {
      return(list(marginal = blended$marginal, transition = blended$transition, accepted = TRUE, step = step))
    }
    step <- step / 2
  }

  list(marginal = old_marginal, transition = old_transition, accepted = FALSE, step = 0)
}


#' Blend old/new CAT parameters and renormalize probabilities
#'
#' @param old_marginal Previous marginal parameters.
#' @param old_transition Previous transition parameters.
#' @param new_marginal Proposed marginal parameters.
#' @param new_transition Proposed transition parameters.
#' @param step Step size in (0, 1].
#' @param order Model order.
#' @param n_time Number of time points.
#' @param n_blocks Number of blocks.
#' @param homogeneous Whether parameters are shared across blocks.
#'
#' @return List with blended \code{marginal} and \code{transition}.
#'
#' @keywords internal
.blend_cat_params_em <- function(old_marginal, old_transition, new_marginal, new_transition,
                                 step, order, n_time, n_blocks, homogeneous) {
  blend_vec <- function(v_old, v_new) {
    v <- (1 - step) * as.numeric(v_old) + step * as.numeric(v_new)
    v <- pmax(v, 1e-12)
    v / sum(v)
  }

  blend_mat <- function(m_old, m_new) {
    m <- (1 - step) * m_old + step * m_new
    m <- pmax(m, 1e-12)
    sweep(m, 1, rowSums(m), "/")
  }

  if (order == 0L) {
    if (homogeneous || n_blocks == 1L) {
      marginal <- list()
      for (t in seq_len(n_time)) {
        nm <- paste0("t", t)
        marginal[[nm]] <- blend_vec(old_marginal[[nm]], new_marginal[[nm]])
      }
      return(list(marginal = marginal, transition = list()))
    }

    marginal <- vector("list", n_blocks)
    transition <- vector("list", n_blocks)
    for (b in seq_len(n_blocks)) {
      marg_b <- list()
      for (t in seq_len(n_time)) {
        nm <- paste0("t", t)
        marg_b[[nm]] <- blend_vec(old_marginal[[b]][[nm]], new_marginal[[b]][[nm]])
      }
      marginal[[b]] <- marg_b
      transition[[b]] <- list()
    }
    names(marginal) <- paste0("block_", seq_len(n_blocks))
    names(transition) <- paste0("block_", seq_len(n_blocks))
    return(list(marginal = marginal, transition = transition))
  }

  if (homogeneous || n_blocks == 1L) {
    marginal <- list(t1 = blend_vec(old_marginal$t1, new_marginal$t1))
    transition <- list()
    for (t in 2:n_time) {
      nm <- paste0("t", t)
      transition[[nm]] <- blend_mat(old_transition[[nm]], new_transition[[nm]])
    }
    return(list(marginal = marginal, transition = transition))
  }

  marginal <- vector("list", n_blocks)
  transition <- vector("list", n_blocks)
  for (b in seq_len(n_blocks)) {
    marginal[[b]] <- list(t1 = blend_vec(old_marginal[[b]]$t1, new_marginal[[b]]$t1))
    trans_b <- list()
    for (t in 2:n_time) {
      nm <- paste0("t", t)
      trans_b[[nm]] <- blend_mat(old_transition[[b]][[nm]], new_transition[[b]][[nm]])
    }
    transition[[b]] <- trans_b
  }
  names(marginal) <- paste0("block_", seq_len(n_blocks))
  names(transition) <- paste0("block_", seq_len(n_blocks))

  list(marginal = marginal, transition = transition)
}


#' @keywords internal
.m_step_cat_em <- function(counts, order, c, n_time, n_blocks, homogeneous, epsilon) {
  if (order == 0L) {
    if (homogeneous || n_blocks == 1L) {
      marginal <- list()
      for (t in seq_len(n_time)) {
        probs_t <- counts$marginal[t, ] + epsilon
        probs_t <- probs_t / sum(probs_t)
        names(probs_t) <- paste0("cat_", seq_len(c))
        marginal[[paste0("t", t)]] <- probs_t
      }
      return(list(marginal = marginal, transition = list()))
    }

    marginal <- vector("list", n_blocks)
    transition <- vector("list", n_blocks)
    for (b in seq_len(n_blocks)) {
      marg_b <- list()
      for (t in seq_len(n_time)) {
        probs_t <- counts$marginal[b, t, ] + epsilon
        probs_t <- probs_t / sum(probs_t)
        names(probs_t) <- paste0("cat_", seq_len(c))
        marg_b[[paste0("t", t)]] <- probs_t
      }
      marginal[[b]] <- marg_b
      transition[[b]] <- list()
    }
    names(marginal) <- paste0("block_", seq_len(n_blocks))
    names(transition) <- paste0("block_", seq_len(n_blocks))
    return(list(marginal = marginal, transition = transition))
  }

  if (homogeneous || n_blocks == 1L) {
    pi1 <- counts$marginal + epsilon
    pi1 <- pi1 / sum(pi1)
    names(pi1) <- paste0("cat_", seq_len(c))
    marginal <- list(t1 = pi1)

    transition <- list()
    for (t in 2:n_time) {
      trans_t <- counts$transition[t - 1L, , ] + epsilon
      trans_t <- sweep(trans_t, 1, rowSums(trans_t), "/")
      transition[[paste0("t", t)]] <- trans_t
    }
    return(list(marginal = marginal, transition = transition))
  }

  marginal <- vector("list", n_blocks)
  transition <- vector("list", n_blocks)
  for (b in seq_len(n_blocks)) {
    pi1 <- counts$marginal[b, ] + epsilon
    pi1 <- pi1 / sum(pi1)
    names(pi1) <- paste0("cat_", seq_len(c))
    marginal[[b]] <- list(t1 = pi1)

    trans_b <- list()
    for (t in 2:n_time) {
      trans_t <- counts$transition[b, t - 1L, , ] + epsilon
      trans_t <- sweep(trans_t, 1, rowSums(trans_t), "/")
      trans_b[[paste0("t", t)]] <- trans_t
    }
    transition[[b]] <- trans_b
  }
  names(marginal) <- paste0("block_", seq_len(n_blocks))
  names(transition) <- paste0("block_", seq_len(n_blocks))

  list(marginal = marginal, transition = transition)
}

# ci_cat.R - Confidence intervals for categorical antedependence model parameters

#' Confidence intervals for fitted categorical AD models
#'
#' Computes Wald-based confidence intervals for the transition probability
#' parameters of a fitted categorical antedependence model.
#'
#' @param fit A fitted model object of class \code{"cat_fit"} from \code{fit_cat()}.
#' @param y Optional data matrix. If NULL, \code{fit$cell_counts} is used
#'   (observed counts for closed-form fits; expected counts for EM fits).
#' @param level Confidence level (default 0.95).
#' @param parameters Which parameters to compute CIs for: "all" (default),
#'   "marginal", or "transition".
#'
#' @return A list of class \code{"cat_ci"} containing:
#'   \item{marginal}{Data frame of CIs for marginal parameters (if requested)}
#'   \item{transition}{List of data frames of CIs for transition parameters (if requested)}
#'   \item{level}{Confidence level used}
#'   \item{settings}{Model settings from fit}
#'
#' @details
#' Confidence intervals are computed using the Wald method based on the
#' asymptotic normality of maximum likelihood estimators.
#'
#' For a probability estimate \eqn{\hat{\pi}} based on count N, the standard error is:
#' \deqn{SE(\hat{\pi}) = \sqrt{\frac{\hat{\pi}(1-\hat{\pi})}{N}}}
#'
#' For conditional probabilities \eqn{\hat{\pi}_{j|i}} based on conditioning count
#' \eqn{N_i}, the standard error is:
#' \deqn{SE(\hat{\pi}_{j|i}) = \sqrt{\frac{\hat{\pi}_{j|i}(1-\hat{\pi}_{j|i})}{N_i}}}
#'
#' The confidence interval is then:
#' \deqn{\hat{\pi} \pm z_{\alpha/2} \times SE(\hat{\pi})}
#'
#' Note: CIs are truncated to the interval from 0 to 1 when they exceed these bounds.
#'
#' Missing-data fits with \code{na_action = "marginalize"} are not currently
#' supported because observed cell counts are not stored for that path.
#'
#' @examples
#' \donttest{
#' # Fit a model
#' set.seed(123)
#' y <- simulate_cat(200, 5, order = 1, n_categories = 2)
#' fit <- fit_cat(y, order = 1)
#'
#' # Compute confidence intervals
#' ci <- ci_cat(fit)
#' print(ci)
#'
#' # Just marginal CIs
#' ci_marg <- ci_cat(fit, parameters = "marginal")
#' }
#'
#' @references
#' Agresti, A. (2013). Categorical Data Analysis (3rd ed.). Wiley.
#'
#' @seealso \code{\link{fit_cat}}
#'
#' @export
ci_cat <- function(fit, y = NULL, level = 0.95, parameters = "all") {
  
  if (!inherits(fit, "cat_fit")) {
    stop("fit must be a cat_fit object from fit_cat()")
  }

  fit_na_action <- NULL
  if (!is.null(fit$settings$na_action_effective)) {
    fit_na_action <- fit$settings$na_action_effective
  } else if (!is.null(fit$settings$na_action)) {
    fit_na_action <- fit$settings$na_action
  }
  if (identical(fit_na_action, "marginalize") || is.null(fit$cell_counts)) {
    stop(
      "ci_cat currently supports complete-data fits only. Missing-data CAT confidence intervals are not implemented yet; refit with na_action = 'complete'.",
      call. = FALSE
    )
  }

  if (!is.null(y) && anyNA(y)) {
    stop(
      "ci_cat currently supports complete y only. Missing-data CAT confidence intervals are not implemented yet.",
      call. = FALSE
    )
  }
  
  if (level <= 0 || level >= 1) {
    stop("level must be between 0 and 1")
  }
  
  parameters <- match.arg(parameters, c("all", "marginal", "transition"))
  
  # Get z critical value
  alpha <- 1 - level
  z <- stats::qnorm(1 - alpha / 2)
  
  # Extract settings
  p <- fit$settings$order
  c <- fit$settings$n_categories
  n_time <- fit$settings$n_time
  n_subjects <- fit$settings$n_subjects
  homogeneous <- fit$settings$homogeneous
  n_blocks <- fit$settings$n_blocks
  
  # Determine if we have heterogeneous groups
  if (!homogeneous && n_blocks > 1) {
    # Handle heterogeneous case
    marginal_ci <- vector("list", n_blocks)
    transition_ci <- vector("list", n_blocks)
    
    for (g in seq_len(n_blocks)) {
      marg_g <- fit$marginal[[g]]
      trans_g <- fit$transition[[g]]
      counts_g <- fit$cell_counts[[g]]
      
      # Get sample size for this block
      if (!is.null(fit$settings$blocks)) {
        n_g <- sum(fit$settings$blocks == g)
      } else {
        n_g <- n_subjects / n_blocks  # Approximate
      }
      
      if (parameters %in% c("all", "marginal")) {
        marginal_ci[[g]] <- .compute_marginal_ci(marg_g, counts_g, p, c, n_time, n_g, z, level)
      }
      
      if (parameters %in% c("all", "transition") && p >= 1) {
        transition_ci[[g]] <- .compute_transition_ci(trans_g, counts_g, p, c, n_time, z, level)
      }
    }
    names(marginal_ci) <- paste0("block_", seq_len(n_blocks))
    names(transition_ci) <- paste0("block_", seq_len(n_blocks))
    
  } else {
    # Homogeneous case
    marginal_ci <- NULL
    transition_ci <- NULL
    
    if (parameters %in% c("all", "marginal")) {
      marginal_ci <- .compute_marginal_ci(fit$marginal, fit$cell_counts, 
                                           p, c, n_time, n_subjects, z, level)
    }
    
    if (parameters %in% c("all", "transition") && p >= 1) {
      transition_ci <- .compute_transition_ci(fit$transition, fit$cell_counts,
                                               p, c, n_time, z, level)
    }
  }
  
  out <- list(
    marginal = marginal_ci,
    transition = transition_ci,
    level = level,
    settings = fit$settings
  )
  
  class(out) <- "cat_ci"
  out
}


#' Compute CIs for marginal parameters
#'
#' @param marginal Marginal parameter list from fit
#' @param cell_counts Cell counts from fit
#' @param p Order
#' @param c Number of categories
#' @param n_time Number of time points
#' @param n_subjects Number of subjects
#' @param z Z critical value
#' @param level Confidence level
#'
#' @return Data frame with CIs
#'
#' @keywords internal
.compute_marginal_ci <- function(marginal, cell_counts, p, c, n_time, n_subjects, z, level) {
  
  results <- list()
  
  if (p == 0) {
    # Independence model: marginal at each time point
    for (k in seq_len(n_time)) {
      probs <- marginal[[k]]
      
      for (cat in seq_len(c)) {
        prob <- probs[cat]
        se <- sqrt(prob * (1 - prob) / n_subjects)
        lower <- max(0, prob - z * se)
        upper <- min(1, prob + z * se)
        
        results[[length(results) + 1]] <- data.frame(
          parameter = paste0("pi_t", k, "_cat", cat),
          time = k,
          category = cat,
          estimate = prob,
          se = se,
          lower = lower,
          upper = upper,
          level = level,
          stringsAsFactors = FALSE
        )
      }
    }
  } else {
    # AD(p): marginal at t1
    probs_t1 <- marginal[["t1"]]
    
    for (cat in seq_len(c)) {
      prob <- probs_t1[cat]
      se <- sqrt(prob * (1 - prob) / n_subjects)
      lower <- max(0, prob - z * se)
      upper <- min(1, prob + z * se)
      
      results[[length(results) + 1]] <- data.frame(
        parameter = paste0("pi_t1_cat", cat),
        time = 1,
        category = cat,
        estimate = prob,
        se = se,
        lower = lower,
        upper = upper,
        level = level,
        stringsAsFactors = FALSE
      )
    }
    
    # For order >= 2, also have conditionals in marginal
    if (p >= 2 && n_time >= 2) {
      for (k in 2:min(p, n_time)) {
        trans_name <- paste0("t", k, "_given_1to", k - 1)
        if (!is.null(marginal[[trans_name]])) {
          # These are conditional probabilities - need conditioning counts
          # For now, skip detailed CIs for these (complex structure)
        }
      }
    }
  }
  
  if (length(results) > 0) {
    do.call(rbind, results)
  } else {
    NULL
  }
}


#' Compute CIs for transition parameters
#'
#' @param transition Transition parameter list from fit
#' @param cell_counts Cell counts from fit
#' @param p Order
#' @param c Number of categories
#' @param n_time Number of time points
#' @param z Z critical value
#' @param level Confidence level
#'
#' @return List of data frames with CIs for each time point
#'
#' @keywords internal
.compute_transition_ci <- function(transition, cell_counts, p, c, n_time, z, level) {
  
  results_list <- list()
  
  for (k in (p + 1):n_time) {
    trans_name <- paste0("t", k)
    trans_k <- transition[[trans_name]]
    
    # Get corresponding counts
    count_name <- paste0("t", k - p, "_to_t", k)
    counts_k <- cell_counts[[count_name]]
    
    if (is.null(trans_k) || is.null(counts_k)) next
    
    results <- list()
    
    if (p == 1) {
      # Order 1: trans_k is c x c matrix
      # trans_k[i, j] = P(Y_k = j | Y_{k-1} = i)
      # Conditioning count is sum over j of counts_k[i, j]
      
      for (i in seq_len(c)) {
        n_cond <- sum(counts_k[i, ])  # Conditioning count
        
        for (j in seq_len(c)) {
          prob <- trans_k[i, j]
          
          if (n_cond > 0) {
            se <- sqrt(prob * (1 - prob) / n_cond)
          } else {
            se <- NA
          }
          
          lower <- if (is.na(se)) NA else max(0, prob - z * se)
          upper <- if (is.na(se)) NA else min(1, prob + z * se)
          
          results[[length(results) + 1]] <- data.frame(
            parameter = paste0("pi_t", k, "_", j, "|", i),
            time = k,
            from = i,
            to = j,
            estimate = prob,
            se = se,
            lower = lower,
            upper = upper,
            n_cond = n_cond,
            level = level,
            stringsAsFactors = FALSE
          )
        }
      }
      
    } else if (p == 2) {
      # Order 2: trans_k is c x c x c array
      # trans_k[i, j, k] = P(Y_k = k | Y_{k-2} = i, Y_{k-1} = j)
      
      for (i in seq_len(c)) {
        for (j in seq_len(c)) {
          n_cond <- sum(counts_k[i, j, ])  # Conditioning count
          
          for (cat in seq_len(c)) {
            prob <- trans_k[i, j, cat]
            
            if (n_cond > 0) {
              se <- sqrt(prob * (1 - prob) / n_cond)
            } else {
              se <- NA
            }
            
            lower <- if (is.na(se)) NA else max(0, prob - z * se)
            upper <- if (is.na(se)) NA else min(1, prob + z * se)
            
            results[[length(results) + 1]] <- data.frame(
              parameter = paste0("pi_t", k, "_", cat, "|", i, ",", j),
              time = k,
              from1 = i,
              from2 = j,
              to = cat,
              estimate = prob,
              se = se,
              lower = lower,
              upper = upper,
              n_cond = n_cond,
              level = level,
              stringsAsFactors = FALSE
            )
          }
        }
      }
    }
    
    if (length(results) > 0) {
      results_list[[trans_name]] <- do.call(rbind, results)
    }
  }
  
  results_list
}


#' Print method for cat_ci objects
#'
#' @param x A cat_ci object
#' @param ... Additional arguments (ignored)
#' @return Invisibly returns \code{x}.
#' @export
print.cat_ci <- function(x, ...) {
  cat("Confidence Intervals for Categorical AD Model\n")
  cat("==============================================\n\n")
  
  cat("Confidence level:", x$level * 100, "%\n")
  cat("Order:", x$settings$order, "\n")
  cat("Categories:", x$settings$n_categories, "\n")
  cat("Time points:", x$settings$n_time, "\n\n")
  
  if (!is.null(x$marginal)) {
    cat("Marginal probabilities:\n")
    if (is.data.frame(x$marginal)) {
      print(x$marginal[, c("parameter", "estimate", "se", "lower", "upper")], 
            row.names = FALSE, digits = 4)
    } else {
      # Heterogeneous case
      for (g in names(x$marginal)) {
        cat("\n", g, ":\n", sep = "")
        if (!is.null(x$marginal[[g]])) {
          print(x$marginal[[g]][, c("parameter", "estimate", "se", "lower", "upper")],
                row.names = FALSE, digits = 4)
        }
      }
    }
    cat("\n")
  }
  
  if (!is.null(x$transition)) {
    cat("Transition probabilities:\n")
    if (is.list(x$transition) && !is.null(names(x$transition))) {
      # Check if it's per-time or per-block
      if (grepl("^t[0-9]+$", names(x$transition)[1])) {
        # Per-time
        for (t_name in names(x$transition)) {
          cat("\n", t_name, ":\n", sep = "")
          df <- x$transition[[t_name]]
          cols_to_show <- intersect(c("parameter", "estimate", "se", "lower", "upper", "n_cond"),
                                    names(df))
          print(df[, cols_to_show], row.names = FALSE, digits = 4)
        }
      } else {
        # Per-block (heterogeneous)
        for (g in names(x$transition)) {
          cat("\n", g, ":\n", sep = "")
          for (t_name in names(x$transition[[g]])) {
            cat("  ", t_name, ":\n", sep = "")
            df <- x$transition[[g]][[t_name]]
            cols_to_show <- intersect(c("parameter", "estimate", "se", "lower", "upper"),
                                      names(df))
            print(df[, cols_to_show], row.names = FALSE, digits = 4)
          }
        }
      }
    }
  }
  
  invisible(x)
}


#' Summary method for cat_ci objects
#'
#' @param object A cat_ci object
#' @param ... Additional arguments (ignored)
#'
#' @return A data frame summarizing all CIs
#'
#' @export
summary.cat_ci <- function(object, ...) {
  
  all_cis <- list()
  
  # Collect marginal CIs
  if (!is.null(object$marginal)) {
    if (is.data.frame(object$marginal)) {
      object$marginal$type <- "marginal"
      all_cis[[length(all_cis) + 1]] <- object$marginal
    } else {
      for (g in names(object$marginal)) {
        if (!is.null(object$marginal[[g]])) {
          df <- object$marginal[[g]]
          df$type <- "marginal"
          df$block <- g
          all_cis[[length(all_cis) + 1]] <- df
        }
      }
    }
  }
  
  # Collect transition CIs
  if (!is.null(object$transition)) {
    if (is.list(object$transition) && !is.null(names(object$transition))) {
      if (grepl("^t[0-9]+$", names(object$transition)[1])) {
        # Per-time
        for (t_name in names(object$transition)) {
          df <- object$transition[[t_name]]
          df$type <- "transition"
          all_cis[[length(all_cis) + 1]] <- df
        }
      } else {
        # Per-block
        for (g in names(object$transition)) {
          for (t_name in names(object$transition[[g]])) {
            df <- object$transition[[g]][[t_name]]
            df$type <- "transition"
            df$block <- g
            all_cis[[length(all_cis) + 1]] <- df
          }
        }
      }
    }
  }
  
  if (length(all_cis) > 0) {
    # Combine, keeping only common columns
    result <- do.call(rbind, lapply(all_cis, function(df) {
      df[, intersect(names(df), c("parameter", "type", "estimate", "se", "lower", "upper", "level"))]
    }))
    return(result)
  }
  
  NULL
}

# Internal helpers for CAT hypothesis-test statistics

.array_get <- function(x, idx) {
  if (length(idx) == 0L) {
    return(x)
  }
  do.call(`[`, c(list(x), as.list(idx), list(drop = TRUE)))
}

.cat_pk <- function(k, p) {
  min(k - 1L, p)
}

.cat_prob_under_order <- function(k, p, history, y_cat, marginal, transition) {
  if (k == 1L) {
    return(as.numeric(marginal[["t1"]][y_cat]))
  }

  if (p == 0L) {
    return(as.numeric(marginal[[paste0("t", k)]][y_cat]))
  }

  if (p == 1L) {
    mat_k <- transition[[paste0("t", k)]]
    return(as.numeric(mat_k[history[length(history)], y_cat]))
  }

  if (p == 2L) {
    if (k == 2L) {
      mat_2 <- marginal[["t2_given_1to1"]]
      return(as.numeric(mat_2[history[length(history)], y_cat]))
    }
    arr_k <- transition[[paste0("t", k)]]
    i1 <- history[length(history) - 1L]
    i2 <- history[length(history)]
    return(as.numeric(arr_k[i1, i2, y_cat]))
  }

  stop("Only orders 0, 1, and 2 are supported")
}

.cat_split_populations <- function(y, blocks, fit) {
  y <- as.matrix(y)
  homogeneous <- isTRUE(fit$settings$homogeneous)

  if (homogeneous) {
    return(list(list(
      y = y,
      marginal = fit$marginal,
      transition = fit$transition
    )))
  }

  blocks_eff <- blocks
  if (is.null(blocks_eff)) {
    blocks_eff <- fit$settings$blocks
  }
  if (is.null(blocks_eff)) {
    stop("blocks are required for heterogeneous CAT score statistics")
  }

  block_levels <- fit$settings$block_levels
  if (is.null(block_levels)) {
    block_levels <- as.character(unique(blocks_eff))
  }

  pops <- vector("list", length(block_levels))
  for (i in seq_along(block_levels)) {
    mask_i <- as.character(blocks_eff) == block_levels[[i]]
    pops[[i]] <- list(
      y = y[mask_i, , drop = FALSE],
      marginal = fit$marginal[[i]],
      transition = fit$transition[[i]]
    )
  }
  pops
}

.cat_score_order_single <- function(y, p, q, c, marginal, transition) {
  y <- as.matrix(y)
  n_time <- ncol(y)
  stat <- 0

  # First summand: j = p+1, ..., q
  if (q >= (p + 1L)) {
    for (j in seq.int(p + 1L, min(q, n_time))) {
      if (j == 1L) {
        counts_j <- .count_cells_table_cat(y, 1L, c, subject_mask = NULL)
        N <- nrow(y)
        idx <- which(counts_j > 0)
        for (cat_j in idx) {
          obs <- counts_j[cat_j]
          prob <- .cat_prob_under_order(
            k = 1L, p = p, history = integer(0), y_cat = cat_j,
            marginal = marginal, transition = transition
          )
          expct <- N * prob
          if (expct > 0) {
            stat <- stat + (obs - expct)^2 / expct
          }
        }
      } else {
        counts_j <- .count_cells_table_cat(y, seq_len(j), c, subject_mask = NULL)
        denom_j <- apply(counts_j, seq_len(j - 1L), sum)
        idx <- which(counts_j > 0, arr.ind = TRUE)
        for (r in seq_len(nrow(idx))) {
          id <- idx[r, ]
          history <- id[seq_len(j - 1L)]
          y_cat <- id[j]
          obs <- .array_get(counts_j, id)
          denom <- .array_get(denom_j, history)
          prob <- .cat_prob_under_order(
            k = j, p = p, history = history, y_cat = y_cat,
            marginal = marginal, transition = transition
          )
          expct <- denom * prob
          if (expct > 0) {
            stat <- stat + (obs - expct)^2 / expct
          }
        }
      }
    }
  }

  # Second summand: k = q+1, ..., n
  if (n_time >= (q + 1L)) {
    for (k in seq.int(q + 1L, n_time)) {
      counts_k <- .count_cells_table_cat(y, (k - q):k, c, subject_mask = NULL)
      denom_k <- apply(counts_k, seq_len(q), sum)
      idx <- which(counts_k > 0, arr.ind = TRUE)
      for (r in seq_len(nrow(idx))) {
        id <- idx[r, ]
        history <- id[seq_len(q)]
        y_cat <- id[q + 1L]
        obs <- .array_get(counts_k, id)
        denom <- .array_get(denom_k, history)
        prob <- .cat_prob_under_order(
          k = k, p = p, history = history, y_cat = y_cat,
          marginal = marginal, transition = transition
        )
        expct <- denom * prob
        if (expct > 0) {
          stat <- stat + (obs - expct)^2 / expct
        }
      }
    }
  }

  stat
}

.cat_score_homogeneity <- function(y, blocks, p, c, fit_null) {
  y <- as.matrix(y)
  blocks <- as.integer(blocks)
  n_time <- ncol(y)
  stat <- 0

  marginal_pool <- fit_null$marginal
  transition_pool <- fit_null$transition
  groups <- sort(unique(blocks))

  for (g in groups) {
    y_g <- y[blocks == g, , drop = FALSE]
    N_g <- nrow(y_g)

    for (k in seq_len(n_time)) {
      p_k <- .cat_pk(k, p)

      if (p_k == 0L) {
        counts_k <- .count_cells_table_cat(y_g, k, c, subject_mask = NULL)
        idx <- which(counts_k > 0)
        for (y_cat in idx) {
          obs <- counts_k[y_cat]
          prob <- .cat_prob_under_order(
            k = k, p = p, history = integer(0), y_cat = y_cat,
            marginal = marginal_pool, transition = transition_pool
          )
          expct <- N_g * prob
          if (expct > 0) {
            stat <- stat + (obs - expct)^2 / expct
          }
        }
      } else {
        counts_k <- .count_cells_table_cat(y_g, (k - p_k):k, c, subject_mask = NULL)
        denom_k <- apply(counts_k, seq_len(p_k), sum)
        idx <- which(counts_k > 0, arr.ind = TRUE)
        for (r in seq_len(nrow(idx))) {
          id <- idx[r, ]
          history <- id[seq_len(p_k)]
          y_cat <- id[p_k + 1L]
          obs <- .array_get(counts_k, id)
          denom <- .array_get(denom_k, history)
          prob <- .cat_prob_under_order(
            k = k, p = p, history = history, y_cat = y_cat,
            marginal = marginal_pool, transition = transition_pool
          )
          expct <- denom * prob
          if (expct > 0) {
            stat <- stat + (obs - expct)^2 / expct
          }
        }
      }
    }
  }

  stat
}

.cat_score_timeinvariance_single <- function(y, p, c, pooled_transition) {
  y <- as.matrix(y)
  n_time <- ncol(y)
  stat <- 0

  for (k in seq.int(p + 1L, n_time)) {
    counts_k <- .count_cells_table_cat(y, (k - p):k, c, subject_mask = NULL)
    denom_k <- apply(counts_k, seq_len(p), sum)
    idx <- which(counts_k > 0, arr.ind = TRUE)
    for (r in seq_len(nrow(idx))) {
      id <- idx[r, ]
      history <- id[seq_len(p)]
      y_cat <- id[p + 1L]
      obs <- .array_get(counts_k, id)
      denom <- .array_get(denom_k, history)
      prob <- .array_get(pooled_transition, c(history, y_cat))
      expct <- denom * prob
      if (expct > 0) {
        stat <- stat + (obs - expct)^2 / expct
      }
    }
  }

  stat
}

.cat_score_stationarity_single <- function(y, p, c, stationary_marginal, stationary_transition) {
  y <- as.matrix(y)
  n_time <- ncol(y)
  N <- nrow(y)
  stat <- 0

  if (p == 0L) {
    for (k in seq_len(n_time)) {
      counts_k <- .count_cells_table_cat(y, k, c, subject_mask = NULL)
      idx <- which(counts_k > 0)
      for (y_cat in idx) {
        obs <- counts_k[y_cat]
        prob <- stationary_marginal[y_cat]
        expct <- N * prob
        if (expct > 0) {
          stat <- stat + (obs - expct)^2 / expct
        }
      }
    }
    return(stat)
  }

  if (p %in% c(1L, 2L)) {
    # Under the current stationary null implementation, the first p time points
    # are evaluated using the stationary marginal, and k > p uses a pooled
    # pth-order transition.
    for (k in seq_len(min(p, n_time))) {
      counts_k <- .count_cells_table_cat(y, k, c, subject_mask = NULL)
      idx <- which(counts_k > 0)
      for (y_cat in idx) {
        obs <- counts_k[y_cat]
        prob <- stationary_marginal[y_cat]
        expct <- N * prob
        if (expct > 0) {
          stat <- stat + (obs - expct)^2 / expct
        }
      }
    }

    if (n_time >= (p + 1L)) {
      for (k in seq.int(p + 1L, n_time)) {
        counts_k <- .count_cells_table_cat(y, (k - p):k, c, subject_mask = NULL)
        denom_k <- apply(counts_k, seq_len(p), sum)
        idx <- which(counts_k > 0, arr.ind = TRUE)
        for (r in seq_len(nrow(idx))) {
          id <- idx[r, ]
          history <- id[seq_len(p)]
          y_cat <- id[p + 1L]
          obs <- .array_get(counts_k, id)
          denom <- .array_get(denom_k, history)
          prob <- .array_get(stationary_transition, c(history, y_cat))
          expct <- denom * prob
          if (expct > 0) {
            stat <- stat + (obs - expct)^2 / expct
          }
        }
      }
    }

    return(stat)
  }

  stop(
    "score stationarity test for categorical AD currently supports orders 0, 1, and 2 only",
    call. = FALSE
  )
}

.cat_f_lawley <- function(N, pi) {
  N * pi * log(N * pi) + (1 - pi) / 2 + (1 - pi^2) / (12 * N * pi)
}

.cat_precompute_order_probs <- function(marginal, transition, q, c, n_time) {
  probs <- list(
    marginal = vector("list", n_time),
    pair = vector("list", n_time),
    triple = vector("list", n_time)
  )

  if (q == 0L) {
    for (k in seq_len(n_time)) {
      probs$marginal[[k]] <- as.numeric(marginal[[paste0("t", k)]])
    }
    return(probs)
  }

  probs$marginal[[1L]] <- as.numeric(marginal[["t1"]])

  if (q == 1L) {
    for (k in 2:n_time) {
      t_k <- transition[[paste0("t", k)]]
      pair_k <- diag(probs$marginal[[k - 1L]]) %*% t_k
      probs$pair[[k]] <- pair_k
      probs$marginal[[k]] <- colSums(pair_k)
    }
    return(probs)
  }

  # q == 2
  t2 <- as.matrix(marginal[["t2_given_1to1"]])
  pair_2 <- diag(probs$marginal[[1L]]) %*% t2
  probs$pair[[2L]] <- pair_2
  probs$marginal[[2L]] <- colSums(pair_2)

  if (n_time >= 3L) {
    for (k in 3:n_time) {
      arr_k <- transition[[paste0("t", k)]]
      triple_k <- array(0, dim = c(c, c, c))
      for (a in seq_len(c)) {
        for (b in seq_len(c)) {
          triple_k[a, b, ] <- probs$pair[[k - 1L]][a, b] * arr_k[a, b, ]
        }
      }
      probs$triple[[k]] <- triple_k

      pair_k <- apply(triple_k, c(2, 3), sum)
      probs$pair[[k]] <- pair_k
      probs$marginal[[k]] <- colSums(pair_k)
    }
  }

  probs
}

.cat_hist_prob_from_precomputed <- function(pre, k, ord, history) {
  if (ord == 0L) {
    return(1)
  }
  if (ord == 1L) {
    return(pre$marginal[[k - 1L]][history[1L]])
  }
  if (ord == 2L) {
    return(pre$pair[[k - 1L]][history[1L], history[2L]])
  }
  stop("Only orders 0, 1, and 2 are supported")
}

.cat_event_prob_from_precomputed <- function(pre, k, ord, history, y_cat) {
  if (ord == 0L) {
    return(pre$marginal[[k]][y_cat])
  }
  if (ord == 1L) {
    return(pre$pair[[k]][history[1L], y_cat])
  }
  if (ord == 2L) {
    return(pre$triple[[k]][history[1L], history[2L], y_cat])
  }
  stop("Only orders 0, 1, and 2 are supported")
}

.cat_mlrt_order_e_single <- function(N, p, q, c, n_time, marginal_q, transition_q) {
  if (!isTRUE(q > p)) {
    stop("q must be greater than p for modified order test")
  }

  base <- list(order = p, marginal = vector("list", n_time), pair = vector("list", n_time))

  if (p == 0L) {
    for (k in seq_len(n_time)) {
      base$marginal[[k]] <- as.numeric(marginal_q[[paste0("t", k)]])
    }
  } else if (p == 1L) {
    base$marginal[[1L]] <- as.numeric(marginal_q[["t1"]])
    base$transition <- transition_q
    for (k in 2:n_time) {
      t_k <- as.matrix(transition_q[[paste0("t", k)]])
      pair_k <- diag(base$marginal[[k - 1L]]) %*% t_k
      base$pair[[k]] <- pair_k
      base$marginal[[k]] <- colSums(pair_k)
    }
  } else if (p == 2L) {
    pre2 <- .cat_precompute_order_probs(
      marginal = marginal_q,
      transition = transition_q,
      q = 2L,
      c = c,
      n_time = n_time
    )
    base$marginal <- pre2$marginal
    base$pair <- pre2$pair
    base$triple <- pre2$triple
  } else {
    stop("Only orders 0, 1, and 2 are supported")
  }

  hist_prob_base <- function(k, ord, history) {
    if (ord == 0L) return(1)

    if (ord == 1L) {
      if (p == 0L) {
        return(base$marginal[[k - 1L]][history[1L]])
      }
      return(base$marginal[[k - 1L]][history[1L]])
    }

    if (ord == 2L) {
      if (p == 0L) {
        return(base$marginal[[k - 2L]][history[1L]] * base$marginal[[k - 1L]][history[2L]])
      }
      return(base$pair[[k - 1L]][history[1L], history[2L]])
    }

    stop("Only orders 0, 1, and 2 are supported")
  }

  event_prob_base <- function(k, ord, history, y_cat) {
    if (ord == 0L) {
      return(base$marginal[[k]][y_cat])
    }

    if (ord == 1L) {
      if (p == 0L) {
        return(base$marginal[[k - 1L]][history[1L]] * base$marginal[[k]][y_cat])
      }
      return(base$pair[[k]][history[1L], y_cat])
    }

    if (ord == 2L) {
      if (p == 0L) {
        return(
          base$marginal[[k - 2L]][history[1L]] *
            base$marginal[[k - 1L]][history[2L]] *
            base$marginal[[k]][y_cat]
        )
      }
      if (p == 1L) {
        t_k <- as.matrix(base$transition[[paste0("t", k)]])
        return(base$pair[[k - 1L]][history[1L], history[2L]] * t_k[history[2L], y_cat])
      }
      return(base$triple[[k]][history[1L], history[2L], y_cat])
    }

    stop("Only orders 0, 1, and 2 are supported")
  }

  e_val <- 0
  for (k in 2:n_time) {
    qk <- .cat_pk(k, q)
    pk <- .cat_pk(k, p)
    if (qk <= pk) {
      next
    }

    # First summation in e(p,q): q-order terms, evaluated under AD(p) fit
    term_q <- 0
    if (qk == 1L) {
      for (h1 in seq_len(c)) {
        pi_h <- hist_prob_base(k, qk, c(h1))
        if (pi_h <= 0) next
        for (yc in seq_len(c)) {
          pi_e <- event_prob_base(k, qk, c(h1), yc)
          if (pi_e <= 0) next
          pi_cond <- pi_e / pi_h
          term_q <- term_q + (.cat_f_lawley(N, pi_e) - pi_cond * .cat_f_lawley(N, pi_h))
        }
      }
    } else if (qk == 2L) {
      for (h1 in seq_len(c)) {
        for (h2 in seq_len(c)) {
          pi_h <- hist_prob_base(k, qk, c(h1, h2))
          if (pi_h <= 0) next
          for (yc in seq_len(c)) {
            pi_e <- event_prob_base(k, qk, c(h1, h2), yc)
            if (pi_e <= 0) next
            pi_cond <- pi_e / pi_h
            term_q <- term_q + (.cat_f_lawley(N, pi_e) - pi_cond * .cat_f_lawley(N, pi_h))
          }
        }
      }
    }

    # Second summation in e(p,q): p-order terms
    term_p <- 0
    if (pk == 0L) {
      for (yc in seq_len(c)) {
        pi_e <- event_prob_base(k, 0L, integer(0), yc)
        if (pi_e <= 0) next
        term_p <- term_p + (.cat_f_lawley(N, pi_e) - pi_e * .cat_f_lawley(N, 1))
      }
    } else if (pk == 1L) {
      for (h1 in seq_len(c)) {
        pi_h <- hist_prob_base(k, 1L, c(h1))
        if (pi_h <= 0) next
        for (yc in seq_len(c)) {
          pi_e <- event_prob_base(k, 1L, c(h1), yc)
          if (pi_e <= 0) next
          pi_cond <- pi_e / pi_h
          term_p <- term_p + (.cat_f_lawley(N, pi_e) - pi_cond * .cat_f_lawley(N, pi_h))
        }
      }
    } else {
      for (h1 in seq_len(c)) {
        for (h2 in seq_len(c)) {
          pi_h <- hist_prob_base(k, 2L, c(h1, h2))
          if (pi_h <= 0) next
          for (yc in seq_len(c)) {
            pi_e <- event_prob_base(k, 2L, c(h1, h2), yc)
            if (pi_e <= 0) next
            pi_cond <- pi_e / pi_h
            term_p <- term_p + (.cat_f_lawley(N, pi_e) - pi_cond * .cat_f_lawley(N, pi_h))
          }
        }
      }
    }

    e_val <- e_val + 2 * (term_q - term_p)
  }

  e_val
}

.cat_mlrt_order_e <- function(fit_null, fit_alt) {
  p <- fit_null$settings$order
  q <- fit_alt$settings$order
  c <- fit_null$settings$n_categories
  n_time <- fit_null$settings$n_time

  if (!isTRUE(fit_null$settings$homogeneous)) {
    blocks <- fit_null$settings$blocks
    if (is.null(blocks)) {
      stop("Cannot compute CAT modified LRT for heterogeneous fit without block assignments")
    }
    levels <- fit_null$settings$block_levels
    if (is.null(levels)) {
      levels <- as.character(unique(blocks))
    }

    e_total <- 0
    for (i in seq_along(levels)) {
      N_i <- sum(as.character(blocks) == levels[[i]])
      e_total <- e_total + .cat_mlrt_order_e_single(
        N = N_i,
        p = p,
        q = q,
        c = c,
        n_time = n_time,
        marginal_q = fit_null$marginal[[i]],
        transition_q = fit_null$transition[[i]]
      )
    }
    return(e_total)
  }

  .cat_mlrt_order_e_single(
    N = fit_alt$settings$n_subjects,
    p = p,
    q = q,
    c = c,
    n_time = n_time,
    marginal_q = fit_null$marginal,
    transition_q = fit_null$transition
  )
}

.cat_prob_vector_under_order <- function(k, p, history, c, marginal, transition) {
  probs <- numeric(c)
  for (y_cat in seq_len(c)) {
    probs[y_cat] <- .cat_prob_under_order(
      k = k,
      p = p,
      history = history,
      y_cat = y_cat,
      marginal = marginal,
      transition = transition
    )
  }
  probs
}

.cat_wald_row_multinom <- function(pi_alt, pi_null, N_hist) {
  c <- length(pi_alt)
  if (c <= 1L || N_hist <= 0) {
    return(0)
  }

  p_alt_sub <- as.numeric(pi_alt[seq_len(c - 1L)])
  p_null_sub <- as.numeric(pi_null[seq_len(c - 1L)])
  d <- p_alt_sub - p_null_sub

  if (all(abs(d) <= .Machine$double.eps^0.5)) {
    return(0)
  }

  p_alt_sub <- as.numeric(p_alt_sub)
  V <- diag(p_alt_sub, nrow = length(p_alt_sub), ncol = length(p_alt_sub)) -
    tcrossprod(p_alt_sub)
  ridge <- 1e-10
  V <- V + diag(ridge, nrow = nrow(V))

  w <- tryCatch(
    as.vector(qr.solve(V, d, tol = 1e-10)),
    error = function(e) NULL
  )
  if (is.null(w) || any(!is.finite(w))) {
    return(0)
  }

  as.numeric(N_hist * crossprod(d, w))
}

.cat_wald_order_single <- function(y, p, q, c, marginal_null, transition_null,
                                   marginal_alt, transition_alt) {
  y <- as.matrix(y)
  n_time <- ncol(y)
  stat <- 0

  for (k in 2:n_time) {
    pk <- .cat_pk(k, p)
    qk <- .cat_pk(k, q)
    if (qk <= pk) {
      next
    }

    counts_k <- .count_cells_table_cat(y, (k - qk):k, c, subject_mask = NULL)
    denom_k <- apply(counts_k, seq_len(qk), sum)
    idx_hist <- which(denom_k > 0, arr.ind = TRUE)
    if (is.null(dim(idx_hist))) {
      idx_hist <- matrix(idx_hist, nrow = 1)
    }

    for (r in seq_len(nrow(idx_hist))) {
      history_q <- as.integer(idx_hist[r, seq_len(qk)])
      N_hist <- .array_get(denom_k, history_q)
      if (!is.finite(N_hist) || N_hist <= 0) {
        next
      }

      if (pk == 0L) {
        history_p <- integer(0)
      } else {
        history_p <- history_q[(qk - pk + 1L):qk]
      }

      pi_alt <- .cat_prob_vector_under_order(
        k = k,
        p = q,
        history = history_q,
        c = c,
        marginal = marginal_alt,
        transition = transition_alt
      )
      pi_null <- .cat_prob_vector_under_order(
        k = k,
        p = p,
        history = history_p,
        c = c,
        marginal = marginal_null,
        transition = transition_null
      )

      stat <- stat + .cat_wald_row_multinom(pi_alt, pi_null, N_hist)
    }
  }

  stat
}

.cat_resolve_blocks <- function(blocks, fit) {
  if (!is.null(blocks)) {
    return(as.integer(blocks))
  }
  fit_blocks <- fit$settings$blocks
  if (is.null(fit_blocks)) {
    return(rep.int(1L, fit$settings$n_subjects))
  }
  as.integer(fit_blocks)
}

.cat_default_mlrt_nsim <- function() {
  nsim <- getOption("antedep.cat_mlrt_nsim", 120L)
  nsim <- as.integer(nsim)[1L]
  if (!is.finite(nsim) || nsim < 10L) {
    nsim <- 120L
  }
  nsim
}

.cat_mlrt_expected_lrt_boot <- function(simulate_fn, lrt_raw_fn,
                                        nsim = .cat_default_mlrt_nsim()) {
  vals <- {
    out <- numeric(nsim)
    for (b in seq_len(nsim)) {
      y_b <- simulate_fn()
      out[b] <- lrt_raw_fn(y_b)
    }
    out
  }

  keep <- is.finite(vals) & vals >= 0
  if (!any(keep)) {
    return(NA_real_)
  }
  mean(vals[keep])
}

.cat_timeinvariant_sim_params <- function(fit_null) {
  p <- fit_null$settings$order
  n_time <- fit_null$settings$n_time

  make_one <- function(marginal_one, transition_one) {
    trans <- vector("list", max(0L, n_time - p))
    if (n_time > p) {
      names(trans) <- paste0("t", (p + 1L):n_time)
      for (k in (p + 1L):n_time) {
        trans[[paste0("t", k)]] <- transition_one[["pooled"]]
      }
    } else {
      trans <- NULL
    }
    list(marginal = marginal_one, transition = trans)
  }

  if (isTRUE(fit_null$settings$homogeneous)) {
    return(make_one(fit_null$marginal, fit_null$transition))
  }

  G <- fit_null$settings$n_blocks
  marginal <- vector("list", G)
  transition <- vector("list", G)
  for (g in seq_len(G)) {
    one <- make_one(fit_null$marginal[[g]], fit_null$transition[[g]])
    marginal[[g]] <- one$marginal
    transition[[g]] <- one$transition
  }
  list(marginal = marginal, transition = transition)
}

.cat_stationary_sim_params <- function(fit_null) {
  p <- fit_null$settings$order
  n_time <- fit_null$settings$n_time

  if (p > 1L) {
    stop("modified stationarity test currently supports CAT order 0 or 1 only")
  }

  make_one <- function(marginal_one, transition_one) {
    marg <- list()
    trans <- NULL

    if (p == 0L) {
      pi <- as.numeric(marginal_one[["stationary"]])
      for (k in seq_len(n_time)) {
        marg[[paste0("t", k)]] <- pi
      }
      return(list(marginal = marg, transition = NULL))
    }

    pi <- as.numeric(marginal_one[["stationary"]])
    Q <- transition_one[["stationary"]]
    marg[["t1"]] <- pi
    trans <- vector("list", max(0L, n_time - 1L))
    if (n_time >= 2L) {
      names(trans) <- paste0("t", 2:n_time)
      for (k in 2:n_time) {
        trans[[paste0("t", k)]] <- Q
      }
    } else {
      trans <- NULL
    }
    list(marginal = marg, transition = trans)
  }

  if (isTRUE(fit_null$settings$homogeneous)) {
    return(make_one(fit_null$marginal, fit_null$transition))
  }

  G <- fit_null$settings$n_blocks
  marginal <- vector("list", G)
  transition <- vector("list", G)
  for (g in seq_len(G)) {
    one <- make_one(fit_null$marginal[[g]], fit_null$transition[[g]])
    marginal[[g]] <- one$marginal
    transition[[g]] <- one$transition
  }
  list(marginal = marginal, transition = transition)
}

#' Cattle growth data (Treatments A and B)
#'
#' Longitudinal cattle growth measurements for two treatment groups from
#' Zimmerman and Nunez-Anton antedependence book companion data. This dataset
#' is continuous-response data suitable for Gaussian AD modeling.
#'
#' @format A list with five components:
#'   \describe{
#'     \item{y}{numeric matrix of dimension N by n_time containing all subjects}
#'     \item{y_A}{numeric matrix for Treatment A subjects}
#'     \item{y_B}{numeric matrix for Treatment B subjects}
#'     \item{blocks}{integer vector of length N (1 = Treatment A, 2 = Treatment B)}
#'     \item{time}{integer vector of measurement occasions}
#'   }
#'
#' @source \url{https://homepage.divms.uiowa.edu/~dzimmer/Data-for-AD/cattle_growth_data_Treatment%20A.txt}
#'   and
#'   \url{https://homepage.divms.uiowa.edu/~dzimmer/Data-for-AD/cattle_growth_data_Treatment%20B.txt}
"cattle_growth"

# test_stationarity_cat.R - Likelihood ratio test for stationarity in categorical AD

#' Likelihood ratio test for stationarity (categorical AD data)
#'
#' Tests whether a categorical antedependence process satisfies stationarity
#' constraints in the AD parameterization.
#'
#' @param y Integer matrix with n_subjects rows and n_time columns. Each entry
#'   should be a category code from 1 to c.
#' @param order Antedependence order p. Default is 1.
#' @param blocks Optional integer vector of length n_subjects specifying group
#'   membership.
#' @param homogeneous Logical. If TRUE (default), parameters are shared across
#'   all groups.
#' @param n_categories Number of categories. If NULL, inferred from data.
#' @param test Type of test statistic. One of \code{"lrt"} (default),
#'   \code{"score"}, or \code{"mlrt"}.
#'
#' @return A list of class \code{"cat_lrt"} containing:
#' \describe{
#'   \item{method}{Inference method used: one of \code{"lrt"}, \code{"score"},
#'     or \code{"mlrt"}.}
#'   \item{lrt_stat}{Likelihood ratio test statistic}
#'   \item{df}{Degrees of freedom}
#'   \item{p_value}{P-value from chi-square distribution}
#'   \item{fit_null}{Fitted stationary model (H0)}
#'   \item{fit_alt}{Fitted non-stationary model (H1)}
#'   \item{table}{Summary data frame}
#' }
#'
#' @details
#' The tested constraints are:
#' \enumerate{
#'   \item The marginal distribution P(Yk) is constant for all k
#'   \item The transition probabilities P(Yk | Y(k-p), ..., Y(k-1)) are
#'     constant for all k > p
#' }
#'
#' For AD order 1, these two constraints correspond to strict stationarity.
#' For AD order greater than 1, this function should be interpreted as testing
#' marginal-constancy plus time-invariant transitions; these constraints are
#' not, in general, sufficient for full strict stationarity.
#'
#' This is stronger than time-invariance alone, which only requires condition 2.
#'
#' This function currently supports complete data only.
#'
#' The null hypothesis is tested against the general (non-stationary) AD(p)
#' model. The degrees of freedom are computed from the fitted parameter counts:
#' \deqn{df = n_{params}(H_1) - n_{params}(H_0)}
#' where \eqn{H_1} is the unconstrained non-stationary model and \eqn{H_0} is
#' the stationary model.
#'
#' @examples
#' \donttest{
#' # Simulate stationary AD(1) data
#' set.seed(123)
#' y <- simulate_cat(200, 6, order = 1, n_categories = 2)
#'
#' # Test stationarity
#' test <- test_stationarity_cat(y, order = 1)
#' print(test)
#' }
#'
#' @references
#' Xie, Y. and Zimmerman, D. L. (2013). Antedependence models for nonstationary
#' categorical longitudinal data with ignorable missingness: likelihood-based
#' inference. \emph{Statistics in Medicine}, 32, 3274-3289.
#'
#' @seealso \code{\link{test_timeinvariance_cat}}, \code{\link{test_order_cat}}
#'
#' @export
test_stationarity_cat <- function(y, order = 1, blocks = NULL,
                                  homogeneous = TRUE, n_categories = NULL,
                                  test = c("lrt", "score", "mlrt")) {
  test <- match.arg(test)
  if (anyNA(y)) {
    .stop_cat_missing_inference("test_stationarity_cat")
  }
  
  # Validate data
 validated <- .validate_y_cat(y, n_categories)
  y <- validated$y
  c <- validated$n_categories
  n_time <- ncol(y)
  n_subjects <- nrow(y)
  p <- as.integer(order)
  
  if (p < 0) {
    stop("order must be non-negative")
  }
  if (p >= n_time) {
    stop("order must be less than n_time")
  }
  if (p >= 2L && test %in% c("lrt", "mlrt")) {
    stop(
      "test_stationarity_cat with test='lrt' or test='mlrt' is not supported for order >= 2 due to a known null-model specification issue; use test='score' for order 2.",
      call. = FALSE
    )
  }
  if (p >= 2L && identical(test, "score")) {
    warning(
      "For order >= 2, test_stationarity_cat evaluates marginal-constancy plus time-invariant transitions and should not be interpreted as a full strict-stationarity test.",
      call. = FALSE
    )
  }
  
  # Fit unconstrained (non-stationary) model - this is the alternative
  fit_alt <- fit_cat(y, order = p, blocks = blocks,
                     homogeneous = homogeneous, n_categories = c)
  
  # Fit constrained (stationary) model
  fit_null <- .fit_cat_stationary(y, p, c, blocks, homogeneous)
  
  # Compute LRT statistic
  lrt_stat_raw <- -2 * (fit_null$log_l - fit_alt$log_l)
  
  # Degrees of freedom
  df <- fit_alt$n_params - fit_null$n_params
  
  # Select test statistic
  stat_value <- lrt_stat_raw
  e_hat_mlrt <- NA_real_
  if (identical(test, "score")) {
    pops <- .cat_split_populations(y, blocks = blocks, fit = fit_null)
    stat_value <- 0
    for (pop in pops) {
      stat_marg <- pop$marginal[["stationary"]]
      stat_trans <- pop$transition[["stationary"]]
      stat_value <- stat_value + .cat_score_stationarity_single(
        y = pop$y,
        p = p,
        c = c,
        stationary_marginal = stat_marg,
        stationary_transition = stat_trans
      )
    }
  } else if (identical(test, "mlrt")) {
    blocks_eff <- .cat_resolve_blocks(blocks, fit_alt)
    sim_params <- tryCatch(
      .cat_stationary_sim_params(fit_null),
      error = function(e) e
    )

    if (inherits(sim_params, "error")) {
      warning(
        paste0(
          "Modified stationarity LRT unavailable for this model: ",
          sim_params$message,
          ". Returning unmodified LRT statistic."
        ),
        call. = FALSE
      )
    } else {
      simulate_fn <- function() {
        simulate_cat(
          n_subjects = n_subjects,
          n_time = n_time,
          order = p,
          n_categories = c,
          marginal = sim_params$marginal,
          transition = sim_params$transition,
          blocks = blocks_eff,
          homogeneous = homogeneous
        )
      }
      lrt_raw_fn <- function(y_b) {
        fit1_b <- fit_cat(
          y_b,
          order = p,
          blocks = blocks_eff,
          homogeneous = homogeneous,
          n_categories = c,
          na_action = "fail"
        )
        fit0_b <- .fit_cat_stationary(
          y_b,
          p = p,
          c = c,
          blocks = blocks_eff,
          homogeneous = homogeneous
        )
        -2 * (fit0_b$log_l - fit1_b$log_l)
      }

      e_hat_mlrt <- .cat_mlrt_expected_lrt_boot(simulate_fn, lrt_raw_fn)
      if (is.finite(e_hat_mlrt) && e_hat_mlrt > 0) {
        stat_value <- lrt_stat_raw * df / e_hat_mlrt
      } else {
        warning(
          "Modified LRT scaling factor could not be estimated; returning unmodified LRT statistic.",
          call. = FALSE
        )
      }
    }
  }

  # P-value from chi-square
  p_value <- stats::pchisq(stat_value, df = df, lower.tail = FALSE)
  
  # Build summary table
  table_df <- data.frame(
    model = c("Stationary (H0)", "Non-stationary (H1)"),
    log_l = c(fit_null$log_l, fit_alt$log_l),
    n_params = c(fit_null$n_params, fit_alt$n_params),
    aic = c(fit_null$aic, fit_alt$aic),
    bic = c(fit_null$bic, fit_alt$bic)
  )
  
  # Assemble output
  out <- list(
    method = test,
    lrt_stat = stat_value,
    statistic = stat_value,
    lrt_stat_raw = lrt_stat_raw,
    e_hat_mlrt = e_hat_mlrt,
    df = df,
    p_value = p_value,
    test = test,
    fit_null = fit_null,
    fit_alt = fit_alt,
    order = p,
    table = table_df
  )
  
  class(out) <- "cat_lrt"
  out
}


#' Fit stationary categorical AD model
#'
#' Internal function to fit a model where both marginal and transition
#' probabilities are constant over time.
#'
#' @param y Data matrix
#' @param p Order
#' @param c Number of categories
#' @param blocks Block vector (or NULL)
#' @param homogeneous Whether to pool across blocks
#'
#' @return A cat_fit-like object with stationary constraints
#'
#' @keywords internal
.fit_cat_stationary <- function(y, p, c, blocks = NULL, homogeneous = TRUE) {
  n_subjects <- nrow(y)
  n_time <- ncol(y)
  
  # Process blocks
  if (is.null(blocks)) {
    blocks <- rep(1L, n_subjects)
  }
  n_blocks <- max(blocks)
  
  if (homogeneous) {
    n_pops <- 1
  } else {
    n_pops <- n_blocks
  }
  
  # For each population, fit stationary model
  if (n_pops == 1) {
    result <- .fit_cat_stationary_single(y, p, c, n_time, subject_mask = NULL)
    marginal <- result$marginal
    transition <- result$transition
    log_l <- result$log_l
    n_params <- result$n_params
  } else {
    marginal <- vector("list", n_pops)
    transition <- vector("list", n_pops)
    log_l <- 0
    n_params <- 0
    
    for (g in seq_len(n_pops)) {
      mask <- (blocks == g)
      result_g <- .fit_cat_stationary_single(y, p, c, n_time, subject_mask = mask)
      marginal[[g]] <- result_g$marginal
      transition[[g]] <- result_g$transition
      log_l <- log_l + result_g$log_l
      n_params <- n_params + result_g$n_params
    }
    names(marginal) <- paste0("block_", seq_len(n_pops))
    names(transition) <- paste0("block_", seq_len(n_pops))
  }
  
  # Compute AIC and BIC
  aic <- -2 * log_l + 2 * n_params
  bic <- -2 * log_l + n_params * log(n_subjects)
  
  out <- list(
    marginal = marginal,
    transition = transition,
    log_l = log_l,
    aic = aic,
    bic = bic,
    n_params = n_params,
    convergence = 0L,
    settings = list(
      order = p,
      n_categories = c,
      n_time = n_time,
      n_subjects = n_subjects,
      blocks = if (n_blocks > 1) blocks else NULL,
      homogeneous = homogeneous,
      n_blocks = n_blocks,
      stationary = TRUE
    )
  )
  
  class(out) <- "cat_fit"
  out
}


#' Fit stationary model for single population
#'
#' @param y Data matrix
#' @param p Order
#' @param c Number of categories
#' @param n_time Number of time points
#' @param subject_mask Logical mask for subjects (NULL = all)
#'
#' @return List with marginal, transition, log_l, n_params
#'
#' @keywords internal
.fit_cat_stationary_single <- function(y, p, c, n_time, subject_mask = NULL) {
  
  # Apply mask
  if (!is.null(subject_mask)) {
    y_sub <- y[subject_mask, , drop = FALSE]
  } else {
    y_sub <- y
  }
  N <- nrow(y_sub)
  
  log_l <- 0
  
  if (p == 0) {
    # Independence + stationarity = same marginal at all time points
    # Pool all observations across all time points
    pooled_counts <- rep(0L, c)
    for (k in seq_len(n_time)) {
      for (cat in seq_len(c)) {
        pooled_counts[cat] <- pooled_counts[cat] + sum(y_sub[, k] == cat)
      }
    }
    
    # Estimate common marginal
    total <- sum(pooled_counts)
    pooled_probs <- pooled_counts / total
    
    marginal <- list(stationary = pooled_probs)
    names(marginal$stationary) <- paste0("cat_", 1:c)
    transition <- NULL
    
    # Log-likelihood: evaluate at each time point with pooled probs
    for (k in seq_len(n_time)) {
      counts_k <- .count_cells_table_cat(y_sub, k, c, subject_mask = NULL)
      log_l <- log_l + .loglik_contribution(counts_k, pooled_probs)
    }
    
    # Number of parameters: just (c-1) for the common marginal
    n_params <- c - 1
    
  } else {
    # AD(p) with p >= 1
    # Stationarity means:
    # 1. Marginal P(Y_k) is constant for all k
    # 2. Transition P(Y_k | Y_{k-p},...,Y_{k-1}) is constant for k > p
    
    # Step 1: Estimate stationary marginal by pooling across all time points
    pooled_marginal_counts <- rep(0L, c)
    for (k in seq_len(n_time)) {
      for (cat in seq_len(c)) {
        pooled_marginal_counts[cat] <- pooled_marginal_counts[cat] + sum(y_sub[, k] == cat)
      }
    }
    stationary_marginal <- pooled_marginal_counts / sum(pooled_marginal_counts)
    
    # Step 2: Estimate stationary transition by pooling across k = p+1 to n
    pooled_trans_counts <- array(0L, dim = rep(c, p + 1))
    for (k in (p + 1):n_time) {
      time_indices <- (k - p):k
      counts_k <- .count_cells_table_cat(y_sub, time_indices, c, subject_mask = NULL)
      pooled_trans_counts <- pooled_trans_counts + counts_k
    }
    stationary_trans <- .counts_to_transition_probs(pooled_trans_counts)
    
    # Store parameters
    marginal <- list(stationary = stationary_marginal)
    names(marginal$stationary) <- paste0("cat_", 1:c)
    transition <- list(stationary = stationary_trans)
    
    # Step 3: Compute log-likelihood
    # For k = 1: use stationary marginal
    counts_1 <- .count_cells_table_cat(y_sub, 1, c, subject_mask = NULL)
    log_l <- log_l + .loglik_contribution(counts_1, stationary_marginal)
    
    # For k = 2 to p: need joint probabilities under stationarity
    # Under stationarity, P(Y_1,...,Y_k) = P(Y_1) * prod_{j=2}^{k} P(Y_j|Y_{j-1},...,Y_{j-min(j-1,p)})
    # For simplicity, we use the stationary marginal for the marginal contribution
    # and compute the conditional part
    if (p >= 2 && n_time >= 2) {
      for (k in 2:min(p, n_time)) {
        # For these early time points, we need to handle carefully
        # Use stationary marginal for the marginal part
        counts_k <- .count_cells_table_cat(y_sub, k, c, subject_mask = NULL)
        log_l <- log_l + .loglik_contribution(counts_k, stationary_marginal)
      }
    }
    
    # For k = p+1 to n: use stationary transitions
    for (k in (p + 1):n_time) {
      time_indices <- (k - p):k
      counts_k <- .count_cells_table_cat(y_sub, time_indices, c, subject_mask = NULL)
      log_l <- log_l + .loglik_contribution(counts_k, stationary_trans)
    }
    
    # Number of parameters:
    # - Stationary marginal: (c-1)
    # - Stationary transition: (c-1) * c^p
    n_params <- (c - 1) + (c - 1) * c^p
  }
  
  list(
    marginal = marginal,
    transition = transition,
    log_l = log_l,
    n_params = n_params
  )
}


#' Run all stationarity-related tests for categorical AD
#'
#' Performs tests for time-invariance and stationarity constraints. For
#' \code{order = 1}, the stationarity test corresponds to strict stationarity;
#' for \code{order > 1}, it tests marginal-constancy plus time-invariant
#' transitions.
#' Currently supports complete data only.
#'
#' @param y Integer matrix with n_subjects rows and n_time columns. Each entry
#'   should be a category code from 1 to c.
#' @param order Antedependence order p. Default is 1.
#' @param blocks Optional integer vector of length n_subjects specifying group
#'   membership.
#' @param homogeneous Logical. If TRUE (default), parameters are shared across
#'   all groups.
#' @param n_categories Number of categories. If NULL, inferred from data.
#' @param test Type of test statistic. One of \code{"lrt"} (default),
#'   \code{"score"}, or \code{"mlrt"}.
#'
#' @return A list containing:
#' \describe{
#'   \item{time_invariance}{Result of test_timeinvariance_cat}
#'   \item{stationarity}{Result of test_stationarity_cat}
#'   \item{table}{Summary data frame}
#' }
#'
#' @examples
#' \donttest{
#' y <- simulate_cat(200, 6, order = 1, n_categories = 2)
#' result <- run_stationarity_tests_cat(y, order = 1)
#' print(result$table)
#' }
#'
#' @export
run_stationarity_tests_cat <- function(y, order = 1, blocks = NULL,
                                        homogeneous = TRUE, n_categories = NULL,
                                        test = c("lrt", "score", "mlrt")) {
  test <- match.arg(test)
  if (anyNA(y)) {
    .stop_cat_missing_inference("run_stationarity_tests_cat")
  }
  
  # Run time-invariance test
  test_ti <- test_timeinvariance_cat(y, order = order, blocks = blocks,
                                     homogeneous = homogeneous, 
                                     n_categories = n_categories,
                                     test = test)
  
  # Run stationarity test
  test_stat <- test_stationarity_cat(y, order = order, blocks = blocks,
                                     homogeneous = homogeneous,
                                     n_categories = n_categories,
                                     test = test)
  
  # Build summary table
  table_df <- data.frame(
    test = c("Time-invariance", "Stationarity"),
    method = c(test_ti$method, test_stat$method),
    lrt_stat = c(test_ti$lrt_stat, test_stat$lrt_stat),
    df = c(test_ti$df, test_stat$df),
    p_value = c(test_ti$p_value, test_stat$p_value),
    significant = c(test_ti$p_value < 0.05, test_stat$p_value < 0.05),
    stringsAsFactors = FALSE
  )
  
  list(
    method = test,
    time_invariance = test_ti,
    stationarity = test_stat,
    table = table_df
  )
}

# test-stationarity_ci_cat.R - Tests for stationarity testing and CIs

# ============================================================
# Test test_stationarity_cat
# ============================================================
test_that("test_stationarity_cat works for order 0", {
  skip_on_cran()
  set.seed(100)
  # Simulate with constant marginal (stationary under independence)
  y <- simulate_cat(80, 5, order = 0, n_categories = 2)
  
  test <- test_stationarity_cat(y, order = 0)
  
  expect_s3_class(test, "cat_lrt")
  expect_true(test$lrt_stat >= 0)
  expect_true(test$df > 0)
  expect_equal(test$order, 0)
  expect_true(test$p_value >= 0 && test$p_value <= 1)
})


test_that("test_stationarity_cat works for order 1", {
  skip_on_cran()
  set.seed(101)
  # Simulate with time-invariant transitions (should be approximately stationary)
  y <- simulate_cat(80, 5, order = 1, n_categories = 2)
  
  test <- test_stationarity_cat(y, order = 1)
  
  expect_s3_class(test, "cat_lrt")
  expect_true(test$lrt_stat >= 0)
  expect_true(test$df > 0)
  expect_equal(test$order, 1)
})


test_that("test_stationarity_cat detects non-stationarity", {
  skip_on_cran()
  set.seed(102)
  # Simulate with time-varying marginals AND transitions
  marg <- list(t1 = c(0.9, 0.1))  # Very skewed initial
  trans_varying <- list(
    t2 = matrix(c(0.5, 0.5, 0.5, 0.5), 2, byrow = TRUE),
    t3 = matrix(c(0.9, 0.1, 0.1, 0.9), 2, byrow = TRUE),
    t4 = matrix(c(0.2, 0.8, 0.8, 0.2), 2, byrow = TRUE),
    t5 = matrix(c(0.6, 0.4, 0.4, 0.6), 2, byrow = TRUE)
  )

  y <- simulate_cat(400, 5, order = 1, n_categories = 2,
                    marginal = marg, transition = trans_varying)
  
  test <- test_stationarity_cat(y, order = 1)
  
  expect_s3_class(test, "cat_lrt")
  # Should reject stationarity
  expect_true(test$p_value < 0.05)
})


test_that("test_stationarity_cat df is correct", {
  skip_on_cran()
  set.seed(103)
  y <- simulate_cat(50, 5, order = 1, n_categories = 2)
  
  test <- test_stationarity_cat(y, order = 1)
  
  # For stationary model:
  # - params: (c-1) for marginal + (c-1)*c^p for transition = 1 + 2 = 3
  # For non-stationary model:
  # - params from fit_cat
  # df = non-stationary params - stationary params
  expect_true(test$df > 0)
  expect_equal(test$fit_null$n_params, 3)  # (2-1) + (2-1)*2^1 = 1 + 2 = 3
})


test_that("run_stationarity_tests_cat works", {
  skip_on_cran()
  set.seed(104)
  y <- simulate_cat(60, 5, order = 1, n_categories = 2)
  
  result <- run_stationarity_tests_cat(y, order = 1)
  
  expect_true(is.list(result))
  expect_true("time_invariance" %in% names(result))
  expect_true("stationarity" %in% names(result))
  expect_true("table" %in% names(result))
  
  expect_equal(nrow(result$table), 2)
  expect_true(all(c("test", "lrt_stat", "df", "p_value") %in% names(result$table)))
})


# ============================================================
# Test ci_cat
# ============================================================
test_that("ci_cat works for order 0", {
  skip_on_cran()
  set.seed(200)
  y <- simulate_cat(80, 4, order = 0, n_categories = 2)
  fit <- fit_cat(y, order = 0)
  
  ci <- ci_cat(fit)
  
  expect_s3_class(ci, "cat_ci")
  expect_equal(ci$level, 0.95)
  expect_true(!is.null(ci$marginal))
  expect_true(is.null(ci$transition))  # No transitions for order 0
  
  # Check marginal CIs
  expect_true(is.data.frame(ci$marginal))
  expect_true(all(c("parameter", "estimate", "se", "lower", "upper") %in% names(ci$marginal)))
  
  # CIs should be within [0, 1]
  expect_true(all(ci$marginal$lower >= 0, na.rm = TRUE))
  expect_true(all(ci$marginal$upper <= 1, na.rm = TRUE))
  expect_true(all(ci$marginal$lower <= ci$marginal$upper, na.rm = TRUE))
})


test_that("ci_cat works for order 1", {
  skip_on_cran()
  set.seed(201)
  y <- simulate_cat(80, 4, order = 1, n_categories = 2)
  fit <- fit_cat(y, order = 1)
  
  ci <- ci_cat(fit)
  
  expect_s3_class(ci, "cat_ci")
  expect_true(!is.null(ci$marginal))
  expect_true(!is.null(ci$transition))
  
  # Check transition CIs
  expect_true(is.list(ci$transition))
  expect_true(length(ci$transition) > 0)
  
  # Check first transition time point
  trans_df <- ci$transition[[1]]
  expect_true(is.data.frame(trans_df))
  expect_true(all(c("parameter", "estimate", "se", "lower", "upper") %in% names(trans_df)))
})


test_that("ci_cat works for order 2", {
  skip_on_cran()
  set.seed(202)
  y <- simulate_cat(100, 5, order = 2, n_categories = 2)
  fit <- fit_cat(y, order = 2)
  
  ci <- ci_cat(fit)
  
  expect_s3_class(ci, "cat_ci")
  expect_true(!is.null(ci$transition))
  
  # For order 2, transition CIs should have from1, from2, to columns
  trans_df <- ci$transition[[1]]
  expect_true(all(c("from1", "from2", "to") %in% names(trans_df)))
})


test_that("ci_cat respects level parameter", {
  skip_on_cran()
  set.seed(203)
  y <- simulate_cat(80, 4, order = 1, n_categories = 2)
  fit <- fit_cat(y, order = 1)
  
  ci_95 <- ci_cat(fit, level = 0.95)
  ci_99 <- ci_cat(fit, level = 0.99)
  
  expect_equal(ci_95$level, 0.95)
  expect_equal(ci_99$level, 0.99)
  
  # 99% CIs should be wider than 95% CIs
  width_95 <- ci_95$marginal$upper - ci_95$marginal$lower
  width_99 <- ci_99$marginal$upper - ci_99$marginal$lower
  expect_true(all(width_99 >= width_95 - 1e-10))  # Allow tiny numerical tolerance
})


test_that("ci_cat parameters argument works", {
  skip_on_cran()
  set.seed(204)
  y <- simulate_cat(80, 4, order = 1, n_categories = 2)
  fit <- fit_cat(y, order = 1)
  
  ci_marg <- ci_cat(fit, parameters = "marginal")
  ci_trans <- ci_cat(fit, parameters = "transition")
  ci_all <- ci_cat(fit, parameters = "all")
  
  expect_true(!is.null(ci_marg$marginal))
  expect_true(is.null(ci_marg$transition))
  
  expect_true(is.null(ci_trans$marginal))
  expect_true(!is.null(ci_trans$transition))
  
  expect_true(!is.null(ci_all$marginal))
  expect_true(!is.null(ci_all$transition))
})


test_that("ci_cat works with heterogeneous model", {
  skip_on_cran()
  set.seed(205)
  y1 <- simulate_cat(50, 3, order = 1, n_categories = 2)
  y2 <- simulate_cat(50, 3, order = 1, n_categories = 2)
  y <- rbind(y1, y2)
  blocks <- c(rep(1, 50), rep(2, 50))
  
  fit <- fit_cat(y, order = 1, blocks = blocks, homogeneous = FALSE)
  ci <- ci_cat(fit)
  
  expect_s3_class(ci, "cat_ci")
  
  # Should have per-block CIs
  expect_true(is.list(ci$marginal))
  expect_true("block_1" %in% names(ci$marginal))
  expect_true("block_2" %in% names(ci$marginal))
})


test_that("ci_cat validates inputs", {
  skip_on_cran()
  set.seed(206)
  y <- simulate_cat(50, 4, order = 1, n_categories = 2)
  fit <- fit_cat(y, order = 1)
  
  # Invalid level
  expect_error(ci_cat(fit, level = 0))
  expect_error(ci_cat(fit, level = 1))
  expect_error(ci_cat(fit, level = 1.5))
  
  # Invalid fit
  expect_error(ci_cat("not a fit"))
})


test_that("ci_cat covers true values", {
  skip_on_cran()
  set.seed(207)
  # Simulate with known parameters
  true_marg <- c(0.6, 0.4)
  true_trans <- matrix(c(0.8, 0.2, 0.3, 0.7), 2, byrow = TRUE)

  marginal <- list(t1 = true_marg)
  transition <- list(
    t2 = true_trans,
    t3 = true_trans,
    t4 = true_trans
  )

  y <- simulate_cat(1000, 4, order = 1, n_categories = 2,
                    marginal = marginal, transition = transition)
  fit <- fit_cat(y, order = 1)
  ci <- ci_cat(fit, level = 0.95)
  
  # Check that true marginal values are covered by CIs
  marg_ci <- ci$marginal
  for (i in 1:2) {
    row_i <- marg_ci[marg_ci$category == i, ]
    expect_true(row_i$lower <= true_marg[i] && true_marg[i] <= row_i$upper,
                info = paste("Marginal category", i, "not covered"))
  }
  
  # Check that true transition values are covered (at least for t2)
  trans_ci <- ci$transition[["t2"]]
  for (i in 1:2) {
    for (j in 1:2) {
      row_ij <- trans_ci[trans_ci$from == i & trans_ci$to == j, ]
      expect_true(row_ij$lower <= true_trans[i, j] && true_trans[i, j] <= row_ij$upper,
                  info = paste("Transition", i, "->", j, "not covered"))
    }
  }
})


test_that("print.cat_ci works", {
  skip_on_cran()
  set.seed(208)
  y <- simulate_cat(50, 4, order = 1, n_categories = 2)
  fit <- fit_cat(y, order = 1)
  ci <- ci_cat(fit)
  
  # Just check it doesn't error
  expect_output(print(ci), "Confidence Intervals")
})


test_that("summary.cat_ci works", {
  skip_on_cran()
  set.seed(209)
  y <- simulate_cat(50, 4, order = 1, n_categories = 2)
  fit <- fit_cat(y, order = 1)
  ci <- ci_cat(fit)
  
  summ <- summary(ci)
  
  expect_true(is.data.frame(summ))
  expect_true(all(c("parameter", "type", "estimate", "lower", "upper") %in% names(summ)))
})

# diagnostics.R - Diagnostic and visualization functions for antedependence models
#
# Functions:
#   - partial_corr()      : Compute intervenor-adjusted partial correlation matrix
#   - plot_prism()        : PRISM (Partial Residual Intervenor Scatterplot Matrix)
#   - plot_profile()      : Profile/spaghetti plot with mean and SD bands
#
# Internal helpers:
#   - .prism_residuals()  : Compute partial residuals for PRISM
# Suppress R CMD check notes for ggplot2 aesthetics
if (getRversion() >= "2.15.1") {
    utils::globalVariables(c("time", "value", "subject", "mean_val", "sd_val"))
}

# ==============================================================================
# PARTIAL CORRELATION MATRIX
# ==============================================================================

#' Compute intervenor-adjusted partial correlation matrix
#'
#' Computes the partial correlation between `Y[i]` and `Y[j]` adjusting for the
#' "intervenor" variables `Y[i+1], ..., Y[j-1]`. Under an antedependence model
#' of order p, partial correlations for |i-j| > p should be approximately zero.
#'
#' @param y Numeric matrix with n_subjects rows and n_time columns.
#' @param test Logical; if TRUE, returns significance flags based on approximate
#'   threshold 2/sqrt(n_eff) where n_eff = n_subjects - (lag - 1). Default FALSE.
#' @param n_digits Integer; number of decimal places for rounding. Default 3.
#'
#' @importFrom stats var cor cov lm residuals sd
#' @importFrom graphics par mtext
#'
#' @return A list with components:
#'   \item{correlation}{Matrix with correlations (upper triangle) and variances (diagonal)}
#'   \item{partial_correlation}{Matrix with partial correlations (lower triangle) and variances (diagonal)}
#'   \item{significant}{(If test=TRUE) Matrix flagging significant partial correlations (1 = significant)}
#'   \item{n_subjects}{Number of subjects}
#'   \item{n_time}{Number of time points}
#'
#' @details
#' The intervenor-adjusted partial correlation between `Y[i]` and `Y[j]` (i < j) is
#' computed as the correlation between the residuals from regressing `Y[i]` and `Y[j]`
#' on the intervenor set `Y[i+1], ..., Y[j-1]`.
#'
#' For adjacent time points (|i-j| = 1), the partial correlation equals the
#' ordinary correlation since there are no intervenors.
#'
#' The diagonal of both returned matrices contains variances (not correlations).
#' This keeps scale information available alongside correlation structure.
#'
#' The significance test uses an approximate threshold of 2/sqrt(n_eff), which
#' corresponds roughly to a 95% confidence bound under normality. This is a
#' rough screening tool, not a formal hypothesis test.
#'
#' @examples
#' \donttest{
#' data("bolus_inad")
#' pc <- partial_corr(bolus_inad$y, test = TRUE)
#'
#' # View partial correlations (lower triangle)
#' pc$partial_correlation
#'
#' # Extract variances from the diagonal
#' variances <- diag(pc$partial_correlation)
#'
#' # Check which are "significant" (rough screen for AD order)
#' pc$significant
#' }
#'
#' @references
#' Zimmerman, D. L. and Nunez-Anton, V. (2009). Antedependence Models for
#' Longitudinal Data. CRC Press.
#'
#' @seealso \code{\link{plot_prism}} for visual diagnostics
#'
#' @export
partial_corr <- function(y, test = FALSE, n_digits = 3) {

    # Input validation
    if (!is.matrix(y)) y <- as.matrix(y)
    if (any(!is.finite(y))) stop("y must contain finite values only")

    n_subjects <- nrow(y)
    n_time <- ncol(y)

    if (n_time < 2) stop("y must have at least 2 time points")
    if (n_subjects < 3) stop("y must have at least 3 subjects")

    # Compute variance and correlation matrices
    var_y <- var(y)
    cor_y <- cor(y)

    # Upper triangle = correlations, diagonal = variances
    CV <- cor_y
    CV[!upper.tri(cor_y, diag = TRUE)] <- 0
    diag(CV) <- diag(var_y)

    # Lower triangle = partial correlations, diagonal = variances
    PC <- diag(diag(var_y))

    for (i in 2:n_time) {
        for (j in 1:(i - 1)) {
            if (i - j == 1) {
                # Adjacent: partial correlation = ordinary correlation
                PC[i, j] <- cor_y[i, j]
            } else {
                # Non-adjacent: condition on intervenors
                y_A <- y[, c(j, i), drop = FALSE]  # Variables of interest
                y_B <- y[, (j + 1):(i - 1), drop = FALSE]  # Intervenors

                s_AA <- var(y_A)
                s_BB <- var(y_B)
                s_AB <- cov(y_A, y_B)

                # Conditional variance: Var(A|B) = Var(A) - Cov(A,B) Var(B)^{-1} Cov(B,A)
                s_BB_inv <- tryCatch(
                    solve(s_BB),
                    error = function(e) {
                        warning("Singular intervenor covariance at lag ", i - j,
                                "; using generalized inverse")
                        MASS::ginv(s_BB)
                    }
                )
                s_AA_B <- s_AA - s_AB %*% s_BB_inv %*% t(s_AB)

                # Partial correlation from conditional covariance
                denom <- sqrt(s_AA_B[1, 1] * s_AA_B[2, 2])
                if (denom > 1e-10) {
                    PC[i, j] <- s_AA_B[1, 2] / denom
                } else {
                    PC[i, j] <- NA
                    warning("Near-zero conditional variance at (", i, ",", j, ")")
                }
            }
        }
    }

    # Round for display
    CV_rounded <- round(CV, n_digits)
    PC_rounded <- round(PC, n_digits)

    result <- list(
        correlation = CV_rounded,
        partial_correlation = PC_rounded,
        n_subjects = n_subjects,
        n_time = n_time
    )

    # Optional significance testing
    if (test) {
        sig_mat <- diag(n_time) * NA  # NA on diagonal
        for (i in 2:n_time) {
            for (j in 1:(i - 1)) {
                lag <- i - j
                n_eff <- n_subjects - (lag - 1)  # Effective sample size
                if (n_eff > 2) {
                    threshold <- 2 / sqrt(n_eff)
                    sig_mat[i, j] <- as.integer(abs(PC[i, j]) >= threshold)
                } else {
                    sig_mat[i, j] <- NA
                }
            }
        }
        result$significant <- sig_mat
    }

    class(result) <- "partial_corr"
    return(result)
}

#' @export
print.partial_corr <- function(x, ...) {
    cat("Partial Correlation Analysis\n")
    cat("----------------------------\n")
    cat("Subjects:", x$n_subjects, "| Time points:", x$n_time, "\n\n")

    cat("Matrix layout:\n")
    cat("  - Upper triangle: ordinary correlations\n")
    cat("  - Lower triangle: intervenor-adjusted partial correlations\n")
    cat("  - Diagonal: variances\n\n")

    # Combine into single display matrix
    display <- x$correlation
    display[lower.tri(display)] <- x$partial_correlation[lower.tri(x$partial_correlation)]
    print(display)

    if (!is.null(x$significant)) {
        cat("\nSignificance flags (lower triangle, 1 = |r| >= 2/sqrt(n_eff)):\n")
        print(x$significant)
    }

    invisible(x)
}


# ==============================================================================
# PRISM PLOTS
# ==============================================================================

#' Compute partial residuals for PRISM plot
#'
#' @param y Data matrix (n_subjects x n_time).
#' @param i First time index.
#' @param j Second time index (j > i + 1).
#'
#' @return List with resid_i and resid_j (partial residuals).
#'
#' @keywords internal
.prism_residuals <- function(y, i, j) {
    if (j <= i + 1) {
        stop("PRISM requires j > i + 1 (need at least one intervenor)")
    }

    y_i <- y[, i]
    y_j <- y[, j]
    intervenors <- y[, (i + 1):(j - 1), drop = FALSE]

    fit_i <- lm(y_i ~ intervenors)
    fit_j <- lm(y_j ~ intervenors)

    list(
        resid_i = residuals(fit_i),
        resid_j = residuals(fit_j)
    )
}


#' PRISM plot (Partial Residual Intervenor Scatterplot Matrix)
#'
#' Creates a matrix of scatterplots for diagnosing antedependence structure.
#' The upper triangle shows ordinary scatterplots of `Y[i]` vs `Y[j]`.
#' The lower triangle shows PRISM plots: residuals from regressing `Y[i]` and `Y[j]`
#' on the intervenor variables `Y[i+1], ..., Y[j-1]`.
#'
#' @param y Numeric matrix with n_subjects rows and n_time columns.
#' @param time_labels Optional character vector of time point labels.
#'   Default uses column names or "T1", "T2", etc.
#' @param pch Point character for scatterplots. Default 20 (filled circle).
#' @param cex Point size. Default 0.6.
#' @param col_upper Color for upper triangle plots. Default "steelblue".
#' @param col_lower Color for lower triangle (PRISM) plots. Default "firebrick".
#' @param main Overall title. Default "PRISM Diagnostic Plot".
#'
#' @return Invisibly returns NULL. Called for side effect (plotting).
#'
#' @details
#' Under an antedependence model of order p, the partial correlation between
#' `Y[i]` and `Y[j]` given the intervenors should be zero when |i-j| > p.
#' This means PRISM plots in the lower triangle should show no association
#' for lags greater than p.
#'
#' Interpretation:
#' \itemize{
#'   \item Upper triangle: Shows marginal associations between time points
#'   \item Lower triangle (PRISM): Shows conditional associations after
#'     removing effects of intervenor variables
#'   \item If AD(1) holds: Only the first sub-diagonal of lower triangle
#'     should show association
#'   \item If AD(2) holds: First two sub-diagonals should show association
#' }
#'
#' @examples
#' \donttest{
#' data("bolus_inad")
#' plot_prism(bolus_inad$y)
#'
#' # With custom labels
#' plot_prism(bolus_inad$y, time_labels = paste0("Hour ", seq(0, 44, by = 4)))
#' }
#'
#' @references
#' Zimmerman, D. L. and Nunez-Anton, V. (2009). Antedependence Models for
#' Longitudinal Data. CRC Press. Chapter 2.
#'
#' @seealso \code{\link{partial_corr}} for numerical partial correlations
#'
#' @export
plot_prism <- function(y,
                       time_labels = NULL,
                       pch = 20,
                       cex = 0.6,
                       col_upper = "steelblue",
                       col_lower = "firebrick",
                       main = "PRISM Diagnostic Plot") {

    # Input validation
    if (!is.matrix(y)) y <- as.matrix(y)
    if (any(!is.finite(y))) stop("y must contain finite values only")

    n_subjects <- nrow(y)
    n_time <- ncol(y)

    if (n_time < 3) stop("PRISM requires at least 3 time points")

    # Time labels
    if (is.null(time_labels)) {
        if (!is.null(colnames(y))) {
            time_labels <- colnames(y)
        } else {
            time_labels <- paste0("T", 1:n_time)
        }
    }
    if (length(time_labels) != n_time) {
        stop("time_labels must have length equal to number of columns in y")
    }

    # Set up plot grid
    n_panels <- n_time - 1
    old_par <- par(no.readonly = TRUE)
    on.exit(par(old_par))

    par(mfrow = c(n_panels, n_panels),
        mar = c(2, 2, 2, 1),
        oma = c(0, 0, 3, 0))

    for (row in 1:n_panels) {
        for (col in 1:n_panels) {

            if (row <= col) {
                # Upper triangle + diagonal: ordinary scatterplot
                # Plot Y[row] vs Y[col + 1]
                i <- row
                j <- col + 1

                plot(y[, i], y[, j],
                     pch = pch, cex = cex, col = col_upper,
                     xlab = "", ylab = "",
                     main = if (row == 1) time_labels[j] else "")

                if (col == 1) {
                    mtext(time_labels[i], side = 2, line = 2.5, cex = 0.7)
                }

            } else {
                # Lower triangle: PRISM plot
                # Partial residuals of Y[col] and Y[row + 1] given intervenors
                i <- col
                j <- row + 1

                resids <- .prism_residuals(y, i, j)

                plot(resids$resid_i, resids$resid_j,
                     pch = pch, cex = cex, col = col_lower,
                     xlab = "", ylab = "",
                     main = "")

                # Add correlation to plot
                r <- cor(resids$resid_i, resids$resid_j)
                mtext(sprintf("r=%.2f", r), side = 3, line = -1.5,
                      cex = 0.6, col = col_lower)
            }
        }
    }

    # Overall title
    mtext(main, outer = TRUE, cex = 1.2, font = 2)

    invisible(NULL)
}


# ==============================================================================
# PROFILE PLOTS
# ==============================================================================

#' Profile plot (spaghetti plot) for longitudinal data
#'
#' Creates a profile plot showing individual subject trajectories with
#' overlaid mean trajectory and standard deviation bands.
#'
#' @param y Numeric matrix with n_subjects rows and n_time columns,
#'   or a data frame with measurements.
#' @param time_points Optional numeric vector of time points for x-axis.
#'   Default uses 1:n_time or attempts to extract from column names.
#' @param blocks Optional integer vector of block memberships for stratified plotting.
#'   If provided, creates separate panels for each block.
#' @param block_labels Optional character vector of labels for blocks.
#' @param title Plot title. Default "Profile Plot".
#' @param xlab X-axis label. Default "Time".
#' @param ylab Y-axis label. Default "Measurement".
#' @param ylim Optional y-axis limits as c(min, max).
#' @param show_sd Logical; if TRUE (default), show +/- 1 SD error bars.
#' @param individual_alpha Alpha (transparency) for individual trajectories. Default 0.3.
#' @param individual_color Color for individual trajectories. Default "grey50".
#' @param mean_color Color for mean trajectory. Default "blue".
#' @param sd_color Color for SD error bars. Default "red".
#' @param mean_lwd Line width for mean trajectory. Default 2.
#'
#' @return A ggplot2 object (invisibly). Called primarily for side effect (plotting).
#'
#' @details
#' This function provides a quick visual summary of longitudinal data showing:
#' \itemize{
#'   \item Individual subject trajectories (light grey lines)
#'   \item Mean trajectory across subjects (bold colored line)
#'   \item +/- 1 standard deviation bands (error bars)
#' }
#'
#' When `blocks` is provided, the plot is faceted by block membership,
#' allowing comparison of trajectories across treatment groups or other strata.
#'
#' @examples
#' \donttest{
#' data("bolus_inad")
#'
#' # Basic profile plot
#' plot_profile(bolus_inad$y)
#'
#' # With block stratification
#' plot_profile(bolus_inad$y, blocks = bolus_inad$blocks,
#'              block_labels = c("2mg", "1mg"))
#'
#' # Customized
#' plot_profile(bolus_inad$y,
#'              time_points = seq(0, 44, by = 4),
#'              title = "Bolus Counts Over Time",
#'              xlab = "Hours", ylab = "Count")
#' }
#'
#' @importFrom ggplot2 ggplot aes geom_line geom_errorbar labs theme_minimal
#'   theme element_text facet_wrap coord_cartesian
#'
#' @export
plot_profile <- function(y,
                         time_points = NULL,
                         blocks = NULL,
                         block_labels = NULL,
                         title = "Profile Plot",
                         xlab = "Time",
                         ylab = "Measurement",
                         ylim = NULL,
                         show_sd = TRUE,
                         individual_alpha = 0.3,
                         individual_color = "grey50",
                         mean_color = "blue",
                         sd_color = "red",
                         mean_lwd = 2) {

    # Check ggplot2 availability
    if (!requireNamespace("ggplot2", quietly = TRUE)) {
        stop("Package 'ggplot2' is required for plot_profile()")
    }

    # Input validation
    if (!is.matrix(y)) y <- as.matrix(y)
    if (any(!is.finite(y[!is.na(y)]))) stop("y must contain finite values (NA allowed)")

    n_subjects <- nrow(y)
    n_time <- ncol(y)

    # Time points
    if (is.null(time_points)) {
        if (!is.null(colnames(y))) {
            # Try to extract numbers from column names
            time_points <- suppressWarnings(
                as.numeric(gsub("[^0-9.-]", "", colnames(y)))
            )
            if (any(is.na(time_points))) {
                time_points <- 1:n_time
            }
        } else {
            time_points <- 1:n_time
        }
    }
    if (length(time_points) != n_time) {
        stop("time_points must have length equal to number of columns in y")
    }

    # Blocks validation
    if (!is.null(blocks)) {
        if (length(blocks) != n_subjects) {
            stop("blocks must have length equal to number of rows in y")
        }
        blocks <- as.factor(blocks)
        if (!is.null(block_labels)) {
            if (length(block_labels) != nlevels(blocks)) {
                stop("block_labels must match number of unique blocks")
            }
            levels(blocks) <- block_labels
        }
    }

    # Convert to long format
    df <- data.frame(
        subject = rep(1:n_subjects, times = n_time),
        time = rep(time_points, each = n_subjects),
        value = as.vector(y)
    )

    if (!is.null(blocks)) {
        df$block <- rep(blocks, times = n_time)
    }

    # Compute summary statistics
    if (!is.null(blocks)) {
        stats <- do.call(rbind, lapply(split(df, list(df$time, df$block)), function(x) {
            data.frame(
                time = x$time[1],
                block = x$block[1],
                mean_val = mean(x$value, na.rm = TRUE),
                sd_val = sd(x$value, na.rm = TRUE)
            )
        }))
    } else {
        stats <- do.call(rbind, lapply(split(df, df$time), function(x) {
            data.frame(
                time = x$time[1],
                mean_val = mean(x$value, na.rm = TRUE),
                sd_val = sd(x$value, na.rm = TRUE)
            )
        }))
    }
    rownames(stats) <- NULL

    # Build plot
    p <- ggplot2::ggplot()

    # Individual trajectories
    if (!is.null(blocks)) {
        p <- p + ggplot2::geom_line(
            data = df,
            ggplot2::aes(x = time, y = value, group = subject),
            color = individual_color,
            alpha = individual_alpha
        )
    } else {
        p <- p + ggplot2::geom_line(
            data = df,
            ggplot2::aes(x = time, y = value, group = subject),
            color = individual_color,
            alpha = individual_alpha
        )
    }

    # Mean trajectory: use version-compatible line width argument.
    mean_line_args <- list(
        data = stats,
        mapping = ggplot2::aes(x = time, y = mean_val),
        color = mean_color
    )
    if (utils::packageVersion("ggplot2") >= "3.4.0") {
        mean_line_args$linewidth <- mean_lwd
    } else {
        mean_line_args$size <- mean_lwd
    }
    p <- p + do.call(ggplot2::geom_line, mean_line_args)

    # SD error bars
    if (show_sd) {
        eb_width <- diff(range(time_points)) * 0.03
        p <- p + ggplot2::geom_errorbar(
            data = stats,
            ggplot2::aes(x = time,
                         ymin = mean_val - sd_val,
                         ymax = mean_val + sd_val),
            width = eb_width,
            color = sd_color
        )
    }

    # Labels and theme
    p <- p +
        ggplot2::labs(title = title, x = xlab, y = ylab) +
        ggplot2::theme_minimal(base_size = 12) +
        ggplot2::theme(
            plot.title = ggplot2::element_text(hjust = 0.5, face = "bold")
        )

    # Y-axis limits
    if (!is.null(ylim)) {
        p <- p + ggplot2::coord_cartesian(ylim = ylim)
    }

    # Facet by block
    if (!is.null(blocks)) {
        p <- p + ggplot2::facet_wrap(~ block)
    }

    print(p)
    invisible(p)
}

# Tests for diagnostics.R

test_that("partial_corr returns expected structure", {
  data("bolus_inad")
  y <- bolus_inad$y
  
  result <- partial_corr(y)
  
  expect_type(result, "list")
  expect_true("partial_correlation" %in% names(result))
  expect_true(is.matrix(result$partial_correlation))
  expect_equal(nrow(result$partial_correlation), ncol(y))
  expect_equal(ncol(result$partial_correlation), ncol(y))
})

test_that("partial_corr with test = TRUE returns significance info", {
  data("bolus_inad")
  y <- bolus_inad$y
  
  result <- partial_corr(y, test = TRUE)
  
  expect_true("significant" %in% names(result))
  expect_true(is.matrix(result$significant))
})

test_that("partial_corr diagonal contains variances", {
  data("bolus_inad")
  y <- bolus_inad$y
  
  result <- partial_corr(y)
  
  diag_vals <- diag(result$partial_correlation)
  expect_true(all(diag_vals > 0))
})

test_that("partial_corr validates input", {
  # Test with single column matrix - should error
  expect_error(partial_corr(matrix(1:5, nrow = 5, ncol = 1)), "at least 2")
})

test_that("plot_prism runs without error", {
  skip_on_cran()
  data("bolus_inad")
  y <- bolus_inad$y[, 1:4]
  
  expect_silent(plot_prism(y))
})

test_that("plot_profile runs without error", {
  skip_on_cran()
  skip_if_not_installed("ggplot2")
  data("bolus_inad")
  y <- bolus_inad$y
  blocks <- bolus_inad$blocks
  
  p <- plot_profile(y, blocks = blocks)
  expect_s3_class(p, "ggplot")
})

test_that("plot_profile works without blocks", {
  skip_on_cran()
  skip_if_not_installed("ggplot2")
  data("bolus_inad")
  y <- bolus_inad$y
  
  p <- plot_profile(y)
  expect_s3_class(p, "ggplot")
})

# File: R/test_homogeneity_gau.R
# Likelihood ratio test for homogeneity of covariance across groups (Section 6.4)

#' Likelihood ratio test for homogeneity across groups (Gaussian AD data)
#'
#' Tests the null hypothesis that G groups have the same AD(p) covariance
#' structure against the alternative that they have different AD(p) structures.
#' This implements Theorem 6.6 of Zimmerman & Núñez-Antón (2009).
#'
#' @param y Numeric matrix with n_subjects rows and n_time columns.
#' @param blocks Integer vector of length n_subjects indicating group membership.
#' @param p Antedependence order. This is the same order parameter named
#'   \code{order} in \code{\link{fit_gau}}.
#' @param use_modified Logical. If TRUE (default), use modified test statistic
#'   for better small-sample approximation.
#'
#' @return A list with class \code{gau_homogeneity_test} containing:
#' \describe{
#'   \item{method}{Inference method used (\code{"lrt"}).}
#'   \item{statistic}{Test statistic value}
#'   \item{statistic_modified}{Modified test statistic (if use_modified = TRUE)}
#'   \item{df}{Degrees of freedom}
#'   \item{p_value}{P-value from chi-square distribution}
#'   \item{p_value_modified}{P-value from modified test}
#'   \item{G}{Number of groups}
#'   \item{group_sizes}{Sample sizes for each group}
#'   \item{order}{Antedependence order}
#' }
#'
#' @details
#' The test compares:
#' \itemize{
#'   \item H0: All G groups have the same AD(p) covariance matrix Sigma(theta)
#'   \item H1: Groups have different AD(p) covariance matrices Sigma(theta_g)
#' }
#'
#' The likelihood ratio test statistic (Theorem 6.6) involves comparing
#' pooled and within-group RSS values. The degrees of freedom are
#' (G-1)(2n - p)(p + 1)/2.
#'
#' This test is useful for determining whether a common covariance structure
#' can be assumed across treatment groups before performing mean comparisons.
#'
#' @references
#' Zimmerman, D.L. and Núñez-Antón, V. (2009). Antedependence Models for
#' Longitudinal Data. Chapman & Hall/CRC. Section 6.4.
#'
#' Kenward, M.G. (1987). A method for comparing profiles of repeated measurements.
#' Applied Statistics, 36, 296-308.
#'
#' @seealso \code{\link{test_order_gau}}, \code{\link{test_two_sample_gau}}
#'
#' @importFrom stats setNames
#'
#' @examples
#' \donttest{
#' # Simulate data from two groups with same covariance
#' n1 <- 30
#' n2 <- 35
#' y1 <- simulate_gau(n1, n_time = 6, order = 1, phi = 0.5, sigma = 1)
#' y2 <- simulate_gau(n2, n_time = 6, order = 1, phi = 0.5, sigma = 1)
#' y <- rbind(y1, y2)
#' blocks <- c(rep(1, n1), rep(2, n2))
#'
#' # Test homogeneity
#' test <- test_homogeneity_gau(y, blocks, p = 1)
#' print(test)
#' }
#'
#' @export
test_homogeneity_gau <- function(y, blocks, p = 1L, use_modified = TRUE) {

    if (!is.matrix(y)) y <- as.matrix(y)
    if (any(!is.finite(y[!is.na(y)]))) stop("y must contain finite values")

    n <- nrow(y)  # Total N
    n_time <- ncol(y)  # n in the book

    blocks <- as.integer(blocks)
    if (length(blocks) != n) stop("blocks must have length nrow(y)")

    unique_blocks <- sort(unique(blocks))
    G <- length(unique_blocks)

    if (G < 2) stop("Need at least 2 groups for homogeneity test")

    p <- as.integer(p)
    if (p < 0 || p >= n_time) stop("p must be in [0, n_time - 1)")

    missing_info <- .validate_missing(y)
    has_missing <- isTRUE(missing_info$has_missing)

    if (has_missing) {
        allowed_patterns <- c("complete", "dropout", "all_missing")
        bad_patterns <- setdiff(unique(missing_info$patterns), allowed_patterns)
        if (length(bad_patterns) > 0) {
            stop(
                paste0(
                    "test_homogeneity_gau with missing data currently supports monotone dropout only ",
                    "(missing values allowed only at the end of each subject profile). ",
                    "Found unsupported patterns: ",
                    paste(bad_patterns, collapse = ", "),
                    "."
                ),
                call. = FALSE
            )
        }
        if (sum(missing_info$patterns != "all_missing") == 0L) {
            stop("At least one subject must have observed data.", call. = FALSE)
        }
    }

    # Relabel blocks to 1, 2, ..., G
    block_map <- setNames(seq_along(unique_blocks), unique_blocks)
    blocks <- as.integer(block_map[as.character(blocks)])

    # Group indices and sizes
    group_indices <- lapply(1:G, function(g) which(blocks == g))
    group_sizes <- sapply(group_indices, length)
    names(group_sizes) <- paste0("g", 1:G)

    # Check minimum group size
    if (any(group_sizes < p + 2)) {
        warning("Some groups have very small sample sizes relative to order p")
    }

    # Compute pooled and within-group RSS
    # Under H0: common covariance, use pooled RSS
    # Under H1: separate covariances, sum within-group RSS

    # For the pooled case, we assume common mean structure (saturated within groups)
    # Actually for homogeneity of covariance, we typically assume means can differ

    # Group-specific means
    group_means <- lapply(1:G, function(g) {
        colMeans(y[group_indices[[g]], , drop = FALSE])
    })

    # Within-group innovation variance estimates
    delta2_g <- matrix(NA_real_, nrow = G, ncol = n_time)
    if (!has_missing) {
        for (g in 1:G) {
            y_g <- y[group_indices[[g]], , drop = FALSE]
            mu_g <- group_means[[g]]
            rss_g <- .rss_vector_gau(y_g, p, mu = mu_g)
            delta2_g[g, ] <- rss_g / group_sizes[g]
        }
    }

    # Pooled innovation variance estimates under H0 (common covariance structure).
    # For complete data: center by group means across all subjects and estimate common phi.
    # For monotone dropout: apply theorem adaptation using subjects complete through time i.
    delta2_pooled <- rep(NA_real_, n_time)
    Nig_mat <- matrix(NA_integer_, nrow = G, ncol = n_time)

    for (i in 1:n_time) {
        p_i <- min(p, i - 1L)

        if (has_missing) {
            keep_i <- rowSums(is.na(y[, 1:i, drop = FALSE])) == 0L
        } else {
            keep_i <- rep(TRUE, n)
        }

        if (!any(keep_i)) {
            next
        }

        y_i <- y[keep_i, 1:i, drop = FALSE]
        blocks_i <- blocks[keep_i]
        N_i <- nrow(y_i)
        # Group means at time i based on subjects complete through i.
        group_means_i <- vector("list", G)
        for (g in 1:G) {
            idx_g <- which(blocks_i == g)
            N_ig <- length(idx_g)
            Nig_mat[g, i] <- N_ig
            if (N_ig > 0) {
                group_means_i[[g]] <- colMeans(y_i[idx_g, , drop = FALSE])
            } else {
                group_means_i[[g]] <- rep(NA_real_, i)
            }
        }

        # Pooled RSS under H0: center by group means and fit one common regression.
        y_centered_i <- y_i
        for (g in 1:G) {
            idx_g <- which(blocks_i == g)
            if (length(idx_g) > 0) {
                y_centered_i[idx_g, ] <- sweep(
                    y_i[idx_g, , drop = FALSE],
                    2,
                    group_means_i[[g]]
                )
            }
        }

        rss_pooled_i <- .rss_gau(y_centered_i, i = i, p = p_i, mu = rep(0, i))
        delta2_pooled[i] <- rss_pooled_i / N_i

        # Within-group RSS under H1.
        for (g in 1:G) {
            idx_g <- which(blocks_i == g)
            N_ig <- length(idx_g)
            if (N_ig <= 0) {
                next
            }
            y_g <- y_i[idx_g, , drop = FALSE]
            mu_g <- group_means_i[[g]]
            rss_g <- .rss_gau(y_g, i = i, p = p_i, mu = mu_g)
            delta2_g[g, i] <- rss_g / N_ig
        }
    }

    # LRT statistic (Theorem 6.6 part a)
    # complete: sum_g N(g) * sum_i [log(RSS_i(p)/N) - log(RSS_ig(p)/N(g))]
    # dropout: replace N and N(g) by N_i and N_i(g) at each time i

    statistic <- 0
    for (i in 1:n_time) {
        if (!is.finite(delta2_pooled[i]) || delta2_pooled[i] <= 0) {
            next
        }
        for (g in 1:G) {
            if (!is.finite(delta2_g[g, i]) || delta2_g[g, i] <= 0) {
                next
            }
            weight <- if (has_missing) Nig_mat[g, i] else group_sizes[g]
            if (!is.finite(weight) || weight <= 0) {
                next
            }
            statistic <- statistic + weight * (log(delta2_pooled[i]) - log(delta2_g[g, i]))
        }
    }

    # Degrees of freedom: (G - 1)(2n - p)(p + 1)/2
    df <- (G - 1) * (2 * n_time - p) * (p + 1) / 2

    # P-value
    p_value <- stats::pchisq(statistic, df = df, lower.tail = FALSE)

    # Modified test statistic
    statistic_modified <- NULL
    p_value_modified <- NULL

    if (use_modified) {
        # Modified statistic — Eq. (6.13) of Zimmerman & Núñez-Antón (2009):
        #
        #   G̃² = df × G² / Σ_i Σ_g N_ig [ψ(N_i - N_ig - G + 1, N_ig - p_i - 1) - log(N_i / N_ig)]
        #
        # where df = (G-1)(2n-p)(p+1)/2, G² is the standard statistic,
        # N_i  = total subjects observed through time i (= N for complete data),
        # N_ig = subjects in group g observed through time i (= N_g for complete data).
        # With monotone dropout, N_i and N_ig are the time-varying effective counts
        # already stored in Nig_mat.

        denominator <- 0

        for (i in 1:n_time) {
            p_i    <- min(p, i - 1L)
            N_i    <- if (has_missing) sum(Nig_mat[, i], na.rm = TRUE) else n

            for (g in 1:G) {
                N_ig <- if (has_missing) Nig_mat[g, i] else group_sizes[g]
                if (!is.finite(N_ig) || N_ig <= 0L) next

                arg1 <- N_i - N_ig - G + 1L
                arg2 <- N_ig - p_i - 1L
                if (arg1 > 1L && arg2 > 1L) {
                    psi_val  <- .psi_kenward(arg1, arg2)
                    log_term <- log(N_i / N_ig)
                    denominator <- denominator + N_ig * (psi_val - log_term)
                }
            }
        }

        if (denominator > 0) {
            statistic_modified <- df * statistic / denominator
            p_value_modified <- stats::pchisq(statistic_modified, df = df, lower.tail = FALSE)
        }
    }

    out <- list(
        method = "lrt",
        statistic = statistic,
        statistic_modified = statistic_modified,
        df = df,
        p_value = p_value,
        p_value_modified = p_value_modified,
        G = G,
        group_sizes = group_sizes,
        order = p,
        n_time = n_time
    )

    class(out) <- "gau_homogeneity_test"
    out
}


#' Print method for AD homogeneity test
#'
#' @param x Object of class \code{gau_homogeneity_test}.
#' @param ... Unused.
#' @return Invisibly returns \code{x}.
#' @export
print.gau_homogeneity_test <- function(x, ...) {
    cat("Test for Homogeneity of AD(", x$order, ") Covariance Across Groups\n", sep = "")
    cat("================================================================\n\n")

    cat("H0: Sigma_1 = Sigma_2 = ... = Sigma_G (common AD covariance)\n")
    cat("H1: At least one Sigma_g differs\n\n")

    cat("Number of groups:", x$G, "\n")
    cat("Group sizes:", paste(names(x$group_sizes), "=", x$group_sizes, collapse = ", "), "\n")
    cat("Time points:", x$n_time, "\n\n")

    cat("Standard LRT:\n")
    cat("  Statistic:", round(x$statistic, 4), "\n")
    cat("  df:", round(x$df, 2), "\n")
    cat("  p-value:", format.pval(x$p_value, digits = 4), "\n")

    if (!is.null(x$statistic_modified)) {
        cat("\nModified LRT:\n")
        cat("  Statistic:", round(x$statistic_modified, 4), "\n")
        cat("  p-value:", format.pval(x$p_value_modified, digits = 4), "\n")
    }

    invisible(x)
}

testthat::test_that("test_homogeneity_gau runs on complete data", {
  skip_on_cran()
  set.seed(1201)
  y <- simulate_gau(
    n_subjects = 60,
    n_time = 6,
    order = 1,
    phi = 0.35,
    sigma = 1
  )
  blocks <- rep(1:2, each = 30)

  res <- test_homogeneity_gau(y, blocks = blocks, p = 1, use_modified = FALSE)

  testthat::expect_s3_class(res, "gau_homogeneity_test")
  testthat::expect_true(is.finite(res$statistic))
  testthat::expect_equal(res$df, (2 - 1) * (2 * ncol(y) - 1) * (1 + 1) / 2)
  testthat::expect_true(res$p_value >= 0 && res$p_value <= 1)
})

testthat::test_that("test_homogeneity_gau matches textbook cochlear example with dropout handling", {
  skip_on_cran()
  data("cochlear_implant", package = "antedep")

  res <- test_homogeneity_gau(
    cochlear_implant$y,
    blocks = cochlear_implant$blocks,
    p = 1,
    use_modified = FALSE
  )

  # Book example (speech recognition / cochlear): stat ~ 3.56, p ~ 0.83.
  testthat::expect_s3_class(res, "gau_homogeneity_test")
  testthat::expect_equal(unname(res$statistic), 3.560827, tolerance = 1e-4)
  testthat::expect_equal(unname(res$df), 7)
  testthat::expect_equal(unname(res$p_value), 0.8287414, tolerance = 1e-4)
})

testthat::test_that("test_homogeneity_gau rejects non-monotone missingness", {
  skip_on_cran()
  y <- matrix(
    c(1, 2, 3, 4,
      2, NA, 3, 4,
      1, 1, 2, NA,
      2, 2, 2, 2),
    nrow = 4,
    byrow = TRUE
  )
  blocks <- c(1, 1, 2, 2)

  testthat::expect_error(
    test_homogeneity_gau(y, blocks = blocks, p = 1),
    "monotone dropout"
  )
})

# File: R/gau_utils.R
# Shared utility functions for Gaussian antedependence models

#' Compute residual sum of squares from AD regression
#'
#' For time point i, regress Y_i on its p_i predecessors (Y_(i-1), ..., Y_(i-p_i))
#' and return the residual sum of squares.
#'
#' @param y Numeric matrix n_subjects by n_time.
#' @param i Time index (1-based).
#' @param p Number of predecessors to include in regression (can be 0).
#' @param mu Optional mean vector. If provided, data is centered by mu before regression.
#'   If NULL, regression includes an intercept.
#' @param include_intercept Logical. If TRUE and mu is NULL, include intercept in regression.
#'
#' @return Residual sum of squares (scalar).
#'
#' @details
#' When mu is provided, the regression is:
#'   \code{(Y_i - mu_i) ~ (Y_(i-1) - mu_(i-1)) + ... + (Y_(i-p) - mu_(i-p))}
#'   with no intercept.
#'
#' When mu is NULL and include_intercept is TRUE:
#'   \code{Y_i ~ 1 + Y_(i-1) + ... + Y_(i-p)}
#'
#' For i = 1 or p = 0, the RSS is simply the sum of squared deviations from the mean
#' (or from mu_1 if provided).
#'
#' @keywords internal
.rss_gau <- function(y, i, p, mu = NULL, include_intercept = TRUE) {
    n <- nrow(y)

    if (i < 1 || i > ncol(y)) stop("i out of range")

    # Effective order for this time point
    p_i <- min(p, i - 1L)

    if (!is.null(mu)) {
        # Center data by provided mean
        response <- y[, i] - mu[i]

        if (p_i == 0L) {
            # No predecessors - RSS is sum of squared centered values
            return(sum(response^2))
        }

        # Build design matrix (centered predecessors, no intercept)
        X <- matrix(NA_real_, nrow = n, ncol = p_i)
        for (k in 1:p_i) {
            X[, k] <- y[, i - k] - mu[i - k]
        }

        # Regression without intercept
        fit <- stats::lm.fit(X, response)
        rss <- sum(fit$residuals^2)

    } else {
        # No centering - use intercept
        response <- y[, i]

        if (p_i == 0L) {
            # No predecessors - RSS from mean
            if (include_intercept) {
                return(sum((response - mean(response))^2))
            } else {
                return(sum(response^2))
            }
        }

        # Build design matrix
        if (include_intercept) {
            X <- cbind(1, matrix(NA_real_, nrow = n, ncol = p_i))
            for (k in 1:p_i) {
                X[, k + 1] <- y[, i - k]
            }
        } else {
            X <- matrix(NA_real_, nrow = n, ncol = p_i)
            for (k in 1:p_i) {
                X[, k] <- y[, i - k]
            }
        }

        fit <- stats::lm.fit(X, response)
        rss <- sum(fit$residuals^2)
    }

    rss
}


#' Compute RSS vector for all time points under AD(p)
#'
#' @param y Numeric matrix n_subjects by n_time.
#' @param p Antedependence order.
#' @param mu Optional mean vector for centering.
#'
#' @return Numeric vector of length n_time with RSS for each time point.
#'
#' @keywords internal
.rss_vector_gau <- function(y, p, mu = NULL) {
    N <- ncol(y)
    rss <- numeric(N)

    for (i in 1:N) {
        rss[i] <- .rss_gau(y, i, p, mu = mu)
    }

    rss
}


#' Compute intervenor-adjusted sample partial correlation
#'
#' Computes the sample partial correlation between Y_i and Y_j, adjusted for
#' the intervenors Y_(j+1), ..., Y_(i-1), from the residuals of the fitted
#' mean structure.
#'
#' @param y Numeric matrix n_subjects by n_time.
#' @param i Row/column index (larger time index).
#' @param j Row/column index (smaller time index), j < i.
#' @param mu Optional mean vector for centering.
#'

#' @return Sample partial correlation (scalar).
#'
#' @details
#' The intervenor-adjusted partial correlation r_(i,j|(j+1:i-1)) is computed
#' from residuals of regressing Y_i and Y_j on the intervenors.
#'
#' Under AD(p), the partial correlation r_(i,i-k|(i-k+1:i-1)) should be
#' approximately zero for k > p.
#'
#' @keywords internal
.partial_corr_gau <- function(y, i, j, mu = NULL) {
    if (j >= i) stop("j must be less than i")

    n <- nrow(y)

    # Center data if mu provided
    if (!is.null(mu)) {
        y_i <- y[, i] - mu[i]
        y_j <- y[, j] - mu[j]
    } else {
        y_i <- y[, i] - mean(y[, i])
        y_j <- y[, j] - mean(y[, j])
    }

    # If adjacent (i = j + 1), no intervenors - return ordinary correlation
    if (i == j + 1L) {
        return(stats::cor(y_i, y_j))
    }

    # Build intervenor matrix: Y_{j+1}, ..., Y_{i-1}
    n_intervenors <- i - j - 1L
    Z <- matrix(NA_real_, nrow = n, ncol = n_intervenors)
    for (k in 1:n_intervenors) {
        idx <- j + k
        if (!is.null(mu)) {
            Z[, k] <- y[, idx] - mu[idx]
        } else {
            Z[, k] <- y[, idx] - mean(y[, idx])
        }
    }

    # Regress y_i and y_j on intervenors.
    # When mu is provided the data is already centered, so no intercept is needed.
    # When mu is NULL the data is centered by column means, so an intercept is included.
    if (!is.null(mu)) {
        fit_i <- stats::lm.fit(Z, y_i)
        fit_j <- stats::lm.fit(Z, y_j)
    } else {
        fit_i <- stats::lm.fit(cbind(1, Z), y_i)
        fit_j <- stats::lm.fit(cbind(1, Z), y_j)
    }

    # Partial correlation is correlation of residuals
    stats::cor(fit_i$residuals, fit_j$residuals)
}


#' Compute matrix of intervenor-adjusted partial correlations
#'
#' @param y Numeric matrix n_subjects by n_time.
#' @param mu Optional mean vector for centering.
#'
#' @return Matrix where entry (i,j) for i > j is r_(i,j|(j+1:i-1)).
#'
#' @keywords internal
.partial_corr_matrix_gau <- function(y, mu = NULL) {
    N <- ncol(y)
    R <- matrix(0, nrow = N, ncol = N)

    for (i in 2:N) {
        for (j in 1:(i-1)) {
            R[i, j] <- .partial_corr_gau(y, i, j, mu = mu)
            R[j, i] <- R[i, j]  # Symmetric
        }
    }

    R
}


#' Compute RSS for two-sample case (pooled and separate)
#'
#' @param y Numeric matrix n_subjects by n_time.
#' @param blocks Integer vector indicating group membership (1 or 2).
#' @param i Time index.
#' @param p Antedependence order.
#' @param pooled Logical. If TRUE, compute pooled RSS; if FALSE, within-group RSS.
#'
#' @return RSS value.
#'
#' @keywords internal
.rss_two_sample_gau <- function(y, blocks, i, p, pooled = TRUE) {
    g1 <- which(blocks == 1)
    g2 <- which(blocks == 2)

    if (pooled) {
        # Pooled: common intercept across groups
        .rss_gau(y, i, p, mu = NULL, include_intercept = TRUE)
    } else {
        # Separate: sum of within-group RSS
        rss1 <- .rss_gau(y[g1, , drop = FALSE], i, p, mu = NULL, include_intercept = TRUE)
        rss2 <- .rss_gau(y[g2, , drop = FALSE], i, p, mu = NULL, include_intercept = TRUE)
        rss1 + rss2
    }
}


#' Compute innovation variance estimates under AD(p)
#'
#' The MLE of the innovation variance at time i is RSS_i(p) / N.
#'
#' @param y Numeric matrix n_subjects by n_time.
#' @param p Antedependence order.
#' @param mu Optional mean vector for centering.
#'
#' @return Numeric vector of innovation variance estimates.
#'
#' @keywords internal
.innovation_var_gau <- function(y, p, mu = NULL) {
    n <- nrow(y)
    rss <- .rss_vector_gau(y, p, mu = mu)
    rss / n
}


#' Compute MLE of mean vector under AD(p)
#'
#' Under AD(p), the MLE of mu is the sample mean vector when there are no
#' covariates.
#'
#' @param y Numeric matrix n_subjects by n_time.
#'
#' @return Numeric vector of mean estimates.
#'
#' @keywords internal
.mle_mean_gau <- function(y) {
    colMeans(y)
}


#' Compute group-specific means for two-sample case
#'
#' @param y Numeric matrix n_subjects by n_time.
#' @param blocks Integer vector indicating group membership.
#'
#' @return List with mu1 and mu2 vectors.
#'
#' @keywords internal
.group_means_gau <- function(y, blocks) {
    g1 <- which(blocks == 1)
    g2 <- which(blocks == 2)

    list(
        mu1 = colMeans(y[g1, , drop = FALSE]),
        mu2 = colMeans(y[g2, , drop = FALSE])
    )
}


#' Modified psi function for test statistic correction
#'
#' Computes the psi function used in the modified likelihood ratio test
#' (Kenward, 1987) for better small-sample approximation.
#'
#' @param x First argument.
#' @param y Second argument.
#'
#' @return Value of psi(x, y).
#'
#' @details
#' psi(x, y) is defined as:
#' \itemize{
#'   \item 0 if x = 0
#'   \item (2y - 1) / (2y(y - 1)) if x = 1
#'   \item 2 * sum from l=1 to x/2 of (y + 2l - 2)^(-1) if x > 0 and even
#'   \item psi(1, y) + psi(x - 1, y + 1) if x > 1 and odd
#' }
#'
#' @keywords internal
.psi_kenward <- function(x, y) {
    if (x == 0) {
        return(0)
    } else if (x == 1) {
        return((2 * y - 1) / (2 * y * (y - 1)))
    } else if (x > 0 && x %% 2 == 0) {
        # Even
        return(2 * sum(1 / (y + 2 * (1:(x/2)) - 2)))
    } else {
        # Odd and > 1
        return(.psi_kenward(1, y) + .psi_kenward(x - 1, y + 1))
    }
}


#' Count parameters for AD model
#'
#' @param order Model order.
#' @param n_time Number of time points.
#' @param n_blocks Number of blocks (1 if no blocks).
#' @param estimate_mu Whether mu is estimated.
#'
#' @return Number of free parameters.
#'
#' @keywords internal
.count_params_gau <- function(order, n_time, n_blocks = 1L, estimate_mu = FALSE) {
    k <- 0L

    # mu parameters
    if (estimate_mu) {
        k <- k + n_time
    }

    # sigma (innovation variance) parameters: n_time
    k <- k + n_time

    # phi parameters
    # Order p: for i = 2, ..., n, we have min(p, i-1) phi parameters
    # Total = sum_{i=2}^{n} min(p, i-1) = (n-1) + (n-2) + ... for p >= n-1
    # For constant order p: (p)(n - (p+1)/2) approximately
    if (order >= 1L) {
        for (i in 2:n_time) {
            k <- k + min(order, i - 1L)
        }
    }

    # tau (block effect) parameters
    if (n_blocks > 1L) {
        k <- k + (n_blocks - 1L)
    }

    k
}

#' @rdname bic_gau
#' @export
bic_inad <- function(fit, n_subjects = NULL) {
    if (is.null(n_subjects)) {
        n_subjects <- fit$settings$n_subjects
        if (is.null(n_subjects) && !is.null(fit$settings$blocks)) {
            warning(
                "n_subjects inferred from legacy fit$settings$blocks; please refit or pass n_subjects explicitly.",
                call. = FALSE
            )
            n_subjects <- length(fit$settings$blocks)
        }
    }
    if (is.null(n_subjects) || length(n_subjects) != 1 || !is.finite(n_subjects) || n_subjects <= 0) {
        stop("n_subjects must be a positive finite scalar")
    }

    k <- .count_params_inad_fit(fit)
    -2 * fit$log_l + k * log(n_subjects)
}

#' @rdname bic_gau
#' @export
aic_inad <- function(fit) {
    k <- .count_params_inad_fit(fit)
    -2 * fit$log_l + 2 * k
}

#' @keywords internal
.count_params_inad_fit <- function(fit) {
    if (is.null(fit$log_l) || !is.finite(fit$log_l)) stop("fit$log_l must be finite")
    if (is.null(fit$settings$order)) stop("fit$settings$order is missing")
    if (is.null(fit$settings$innovation)) stop("fit$settings$innovation is missing")

    ord <- fit$settings$order
    innovation <- fit$settings$innovation

    N <- length(fit$theta)
    if (N < 1) stop("fit$theta must be nonempty")

    k <- 0
    k <- k + N

    if (ord == 1) k <- k + (N - 1)
    if (ord == 2) k <- k + (2 * N - 3)

    if (!is.null(fit$tau)) {
        B <- length(fit$tau)
        if (B >= 1) k <- k + (B - 1)
    }

    if (!is.null(fit$nb_inno_size) && innovation == "nbinom") {
        if (length(fit$nb_inno_size) == 1L) k <- k + 1 else k <- k + N
    }

    k
}

test_that("bic_inad matches manual formula on bolus_inad", {
    skip_on_cran()
    skip_if_not_installed("nloptr")

    data(bolus_inad, package = "antedep")

    y <- bolus_inad$y
    blocks <- bolus_inad$bolus
    n_subjects <- nrow(y)

    fit <- fit_inad(
        y = y,
        order = 1,
        thinning = "nbinom",
        innovation = "bell",
        blocks = blocks,
        max_iter = 20,
        tol = 1e-6,
        verbose = FALSE
    )

    k_manual <- function(fit) {
        ord <- fit$settings$order
        innovation <- fit$settings$innovation
        N <- length(fit$theta)

        k <- N

        if (ord == 1) k <- k + (N - 1)
        if (ord == 2) k <- k + (2 * N - 3)

        if (!is.null(fit$tau)) {
            B <- length(fit$tau)
            k <- k + (B - 1)
        }

        if (!is.null(fit$nb_inno_size) && innovation == "nbinom") {
            if (length(fit$nb_inno_size) == 1L) k <- k + 1 else k <- k + N
        }

        k
    }

    bic_manual <- -2 * fit$log_l + k_manual(fit) * log(n_subjects)
    bic_pkg <- bic_inad(fit = fit, n_subjects = n_subjects)
    bic_pkg_infer <- bic_inad(fit = fit)

    expect_true(is.numeric(bic_pkg))
    expect_equal(length(bic_pkg), 1)
    expect_true(is.finite(bic_pkg))
    expect_equal(bic_pkg, bic_manual, tolerance = 0)
    expect_equal(bic_pkg_infer, bic_pkg, tolerance = 0)
})

test_that("bic_inad warns on legacy n_subjects fallback via settings$blocks", {
    skip_on_cran()
    fit_legacy <- list(
        log_l = -100,
        theta = c(1, 1, 1),
        tau = NULL,
        nb_inno_size = NULL,
        settings = list(
            order = 1,
            innovation = "pois",
            blocks = rep(1, 25)
        )
    )

    expect_warning(
        bic_legacy <- bic_inad(fit_legacy),
        "legacy fit\\$settings\\$blocks"
    )

    k_manual <- length(fit_legacy$theta) + (length(fit_legacy$theta) - 1)
    bic_manual <- -2 * fit_legacy$log_l + k_manual * log(length(fit_legacy$settings$blocks))
    expect_equal(as.numeric(bic_legacy), as.numeric(bic_manual), tolerance = 1e-12)
})

test_that("aic_inad matches manual formula for INAD fit", {
  set.seed(411)
  y <- simulate_inad(
    n_subjects = 80,
    n_time = 6,
    order = 1,
    thinning = "binom",
    innovation = "pois",
    alpha = 0.3,
    theta = 2
  )

  fit <- fit_inad(
    y = y,
    order = 1,
    thinning = "binom",
    innovation = "pois",
    blocks = NULL
  )

  N <- ncol(y)
  k <- N + (N - 1)
  aic_manual <- -2 * fit$log_l + 2 * k

  expect_equal(aic_inad(fit), aic_manual)
})

#' Compute observed-data log-likelihood for AD with missing values
#'
#' Uses multivariate normal marginalization to compute the likelihood
#' of observed values, marginalizing over missing values.
#'
#' @param y Data matrix (n_subjects x n_time), may contain NA
#' @param order Antedependence order (0, 1, or 2)
#' @param mu Mean vector (length n_time)
#' @param phi Dependence parameter(s)
#' @param sigma Innovation standard deviations (length n_time)
#' @param blocks Block membership vector (optional)
#' @param tau Block effects, first element constrained to zero
#'
#' @return Observed-data log-likelihood (scalar)
#'
#' @keywords internal
.logL_gau_missing <- function(y, order, mu, phi, sigma, blocks = NULL, tau = 0) {
  n_subjects <- nrow(y)
  n_time <- ncol(y)
  
  # Build full covariance matrix from AD parameters
  Sigma <- .build_gau_covariance(order, phi, sigma, n_time)
  
  # Adjust mean for blocks if present
  if (!is.null(blocks)) {
    mu_matrix <- matrix(mu, nrow = n_subjects, ncol = n_time, byrow = TRUE)
    tau_matrix <- matrix(tau[blocks], nrow = n_subjects, ncol = n_time)
    mu_matrix <- mu_matrix + tau_matrix
  } else {
    mu_matrix <- matrix(mu, nrow = n_subjects, ncol = n_time, byrow = TRUE)
  }
  
  # Compute log-likelihood by marginalizing over missing values
  log_lik <- 0
  
  for (s in 1:n_subjects) {
    y_s <- y[s, ]
    mu_s <- mu_matrix[s, ]
    
    obs_idx <- which(!is.na(y_s))
    
    if (length(obs_idx) == 0) {
      # All missing - skip
      next
    }
    
    # Extract observed values and parameters
    y_obs <- y_s[obs_idx]
    mu_obs <- mu_s[obs_idx]
    Sigma_obs <- Sigma[obs_idx, obs_idx, drop = FALSE]
    
    # Compute MVN density on observed values
    log_lik_s <- tryCatch({
      # Use Cholesky decomposition for numerical stability
      L <- chol(Sigma_obs)
      z <- backsolve(L, y_obs - mu_obs, transpose = TRUE)
      log_det <- 2 * sum(log(diag(L)))
      -0.5 * (length(obs_idx) * log(2 * pi) + log_det + sum(z^2))
    }, error = function(e) {
      NA_real_
    })

    if (!is.finite(log_lik_s)) return(-Inf)
    log_lik <- log_lik + log_lik_s
  }
  
  return(log_lik)
}

#' Build AD covariance matrix from parameters
#'
#' @param order Antedependence order
#' @param phi Dependence parameters
#' @param sigma Innovation standard deviations
#' @param n_time Number of time points
#' @return Covariance matrix (n_time x n_time)
#' @keywords internal
.build_gau_covariance <- function(order, phi, sigma, n_time) {
  Sigma <- matrix(0, nrow = n_time, ncol = n_time)

  if (length(sigma) != n_time || any(!is.finite(sigma)) || any(sigma <= 0)) {
    stop("sigma must be positive and have length n_time")
  }

  if (order == 0) {
    diag(Sigma) <- sigma^2
    return(Sigma)
  }

  if (order == 1) {
    if (length(phi) == n_time) {
      phi1 <- as.numeric(phi)
    } else if (length(phi) == n_time - 1) {
      phi1 <- c(0, as.numeric(phi))
    } else {
      stop("phi must have length n_time or n_time - 1 for order 1")
    }

    V <- numeric(n_time)
    V[1] <- sigma[1]^2
    for (t in 2:n_time) {
      V[t] <- phi1[t]^2 * V[t - 1] + sigma[t]^2
    }

    for (i in 1:n_time) {
      Sigma[i, i] <- V[i]
      if (i < n_time) {
        for (j in (i + 1):n_time) {
          prod_phi <- prod(phi1[(i + 1):j])
          Sigma[i, j] <- prod_phi * V[i]
          Sigma[j, i] <- Sigma[i, j]
        }
      }
    }
    return(Sigma)
  }

  if (order == 2) {
    if (is.matrix(phi)) {
      if (nrow(phi) == 2 && ncol(phi) == n_time) {
        phi2 <- phi
      } else if (nrow(phi) == n_time - 2 && ncol(phi) == 2) {
        phi2 <- matrix(0, nrow = 2, ncol = n_time)
        if (n_time >= 3) {
          phi2[1, 3:n_time] <- phi[, 1]
          phi2[2, 3:n_time] <- phi[, 2]
        }
      } else {
        stop("phi must be 2 x n_time or (n_time - 2) x 2 matrix for order 2")
      }
    } else {
      phi_vec <- as.numeric(phi)
      if (length(phi_vec) == 2 * n_time) {
        phi2 <- matrix(phi_vec, nrow = 2, byrow = TRUE)
      } else if (length(phi_vec) == 2 * (n_time - 2)) {
        phi2 <- matrix(0, nrow = 2, ncol = n_time)
        if (n_time >= 3) {
          phi_compact <- matrix(phi_vec, ncol = 2, byrow = TRUE)
          phi2[1, 3:n_time] <- phi_compact[, 1]
          phi2[2, 3:n_time] <- phi_compact[, 2]
        }
      } else {
        stop("phi must be compatible with order 2")
      }
    }

    Sigma[1, 1] <- sigma[1]^2

    if (n_time >= 2) {
      Sigma[2, 1] <- phi2[1, 2] * Sigma[1, 1]
      Sigma[1, 2] <- Sigma[2, 1]
      Sigma[2, 2] <- phi2[1, 2]^2 * Sigma[1, 1] + sigma[2]^2
    }

    if (n_time >= 3) {
      for (t in 3:n_time) {
        for (k in 1:(t - 1)) {
          Sigma[t, k] <- phi2[1, t] * Sigma[t - 1, k] + phi2[2, t] * Sigma[t - 2, k]
          Sigma[k, t] <- Sigma[t, k]
        }
        Sigma[t, t] <- phi2[1, t] * Sigma[t - 1, t] +
          phi2[2, t] * Sigma[t - 2, t] + sigma[t]^2
      }
    }

    return(Sigma)
  }

  stop("order must be 0, 1, or 2")
}

#' Missing data utilities
#'
#' Internal helper functions used by antedependence model fitting when the input
#' contains missing values (NA). These functions are not exported.
#'
#' @importFrom stats complete.cases
#' @keywords internal
#' @noRd
NULL

.validate_missing <- function(y) {
    if (!is.matrix(y)) y <- as.matrix(y)

    n_subjects <- nrow(y)
    n_time <- ncol(y)

    if (!any(is.na(y))) {
        return(list(
            has_missing = FALSE,
            n_complete = n_subjects,
            n_intermittent = 0,
            patterns = rep("complete", n_subjects),
            pct_missing = 0
        ))
    }

    patterns <- apply(y, 1, function(row) {
        obs <- which(!is.na(row))

        if (length(obs) == 0) return("all_missing")
        if (length(obs) == n_time) return("complete")

        if (length(obs) >= 2 && all(diff(obs) == 1)) {
            if (obs[1] == 1) return("dropout")
            if (obs[length(obs)] == n_time) return("dropin")
            return("monotone_middle")
        }

        "intermittent"
    })

    list(
        has_missing = TRUE,
        n_complete = sum(patterns == "complete"),
        n_intermittent = sum(patterns == "intermittent"),
        patterns = patterns,
        pct_missing = mean(is.na(y)) * 100
    )
}

.get_truncation_bound <- function(y, subject_idx, time_idx,
                                  buffer = 1.2, min_bound = 10) {
    if (!is.matrix(y)) y <- as.matrix(y)

    subject_obs <- y[subject_idx, ]
    subj_vals <- subject_obs[!is.na(subject_obs)]
    subject_max <- if (length(subj_vals) > 0) max(subj_vals) else 0

    time_obs <- y[-subject_idx, time_idx]
    time_vals <- time_obs[!is.na(time_obs)]
    time_max <- if (length(time_vals) > 0) max(time_vals) else 0

    raw_bound <- buffer * max(subject_max, time_max)

    if (!is.finite(raw_bound) || raw_bound < min_bound) {
        raw_bound <- min_bound
    }

    floor(raw_bound)
}

.safe_log <- function(x) {
    out <- rep(NA_real_, length(x))

    is_zero <- (x == 0)
    is_neg <- (x < 0)
    is_pos <- (x > 0)

    out[is_zero] <- 0
    out[is_neg] <- -Inf
    out[is_pos] <- log(x[is_pos])

    out
}

.warn_intermittent_missing <- function(y, missing_info, max_intermittent = 3) {
    if (!is.matrix(y)) y <- as.matrix(y)

    if (is.null(missing_info$n_intermittent) || missing_info$n_intermittent <= 0) {
        return(invisible(NULL))
    }

    miss_per_subject <- rowSums(is.na(y))
    max_per_subject <- if (length(miss_per_subject) > 0) max(miss_per_subject) else 0

    if (is.finite(max_per_subject) && max_per_subject > max_intermittent) {
        warning(
            paste0(
                "Subject(s) with more than ", max_intermittent,
                " missing values. Computation may be slow. ",
                "Consider na_action = 'complete'."
            ),
            call. = FALSE
        )
    }

    invisible(NULL)
}

.extract_complete_cases <- function(y, blocks = NULL, warn = TRUE) {
    if (!is.matrix(y)) y <- as.matrix(y)

    complete_rows <- stats::complete.cases(y)
    n_complete <- sum(complete_rows)
    n_total <- nrow(y)

    if (n_complete == 0) {
        stop("No complete cases available for initialization.", call. = FALSE)
    }

    if (isTRUE(warn) && n_complete < n_total) {
        pct_complete <- round(100 * n_complete / n_total)
        warning(
            paste0(
                "Only ", n_complete, "/", n_total,
                " subjects have complete data (", pct_complete, "%).\n",
                "Consider using na_action = 'em' for less information loss."
            ),
            call. = FALSE
        )
    }

    y_complete <- y[complete_rows, , drop = FALSE]
    blocks_complete <- if (!is.null(blocks)) blocks[complete_rows] else NULL

    list(
        y = y_complete,
        blocks = blocks_complete,
        n_complete = n_complete,
        n_removed = n_total - n_complete
    )
}

.normalize_blocks <- function(blocks, n_subjects) {
    if (is.null(blocks)) {
        return(list(
            blocks_id = rep(1L, n_subjects),
            block_levels = "1",
            n_blocks = 1L
        ))
    }

    if (length(blocks) != n_subjects) {
        stop("blocks must have length equal to number of subjects (", n_subjects, ")", call. = FALSE)
    }
    if (any(is.na(blocks))) {
        stop("blocks must not contain missing values", call. = FALSE)
    }

    block_fac <- factor(blocks)
    list(
        blocks_id = as.integer(block_fac),
        block_levels = levels(block_fac),
        n_blocks = nlevels(block_fac)
    )
}

#' Test suite for missing data support
#'
#' Tests for missing_utils.R and AD missing data functionality

library(testthat)

# ==== Test missing_utils.R ====

test_that(".validate_missing works for complete data", {
  skip_on_cran()
  y <- matrix(rnorm(50), nrow = 10, ncol = 5)
  
  result <- .validate_missing(y)
  
  expect_false(result$has_missing)
  expect_equal(result$n_complete, 10)
  expect_equal(result$n_intermittent, 0)
  expect_equal(result$pct_missing, 0)
  expect_true(all(result$patterns == "complete"))
})

test_that(".validate_missing identifies monotone dropout", {
  skip_on_cran()
  y <- matrix(c(
    1, 2, 3, 4, 5,
    1, 2, 3, NA, NA,
    1, 2, NA, NA, NA
  ), nrow = 3, byrow = TRUE)
  
  result <- .validate_missing(y)
  
  expect_true(result$has_missing)
  expect_equal(result$n_complete, 1)
  expect_equal(result$patterns[1], "complete")
  expect_equal(result$patterns[2], "dropout")
  expect_equal(result$patterns[3], "dropout")
})

test_that(".validate_missing identifies monotone drop-in", {
  skip_on_cran()
  y <- matrix(c(
    1, 2, 3, 4, 5,
    NA, NA, 3, 4, 5,
    NA, 2, 3, 4, 5
  ), nrow = 3, byrow = TRUE)
  
  result <- .validate_missing(y)
  
  expect_equal(result$patterns[2], "dropin")
  expect_equal(result$patterns[3], "dropin")
})

test_that(".validate_missing identifies intermittent missing", {
  skip_on_cran()
  y <- matrix(c(
    1, NA, 3, NA, 5,
    1, 2, NA, 4, 5
  ), nrow = 2, byrow = TRUE)
  
  result <- .validate_missing(y)
  
  expect_equal(result$n_intermittent, 2)
  expect_true(all(result$patterns == "intermittent"))
})

test_that(".get_truncation_bound works correctly", {
  skip_on_cran()
  y <- matrix(c(
    5, 10, 15, 20,
    3, 8, NA, 18,
    4, 9, 14, 19
  ), nrow = 3, byrow = TRUE)
  
  # Subject 2, time 3 is missing
  # Subject 2 max = 18, other subjects at time 3: 14
  # Expected: 1.2 * max(18, 14) = 1.2 * 18 = 21.6 -> 21
  bound <- .get_truncation_bound(y, subject_idx = 2, time_idx = 3, buffer = 1.2)
  
  expect_equal(bound, 21)
})

test_that(".get_truncation_bound handles edge cases", {
  skip_on_cran()
  # All missing in column
  y <- matrix(c(
    1, NA, 3,
    2, NA, 4,
    3, NA, 5
  ), nrow = 3, byrow = TRUE)
  
  bound <- .get_truncation_bound(y, subject_idx = 1, time_idx = 2, min_bound = 10)
  expect_gte(bound, 10)  # Should return at least min_bound
})

test_that(".safe_log handles zero correctly", {
  skip_on_cran()
  x <- c(-1, 0, 1, 2, exp(1))
  
  result <- .safe_log(x)
  
  expect_equal(result[1], -Inf)
  expect_equal(result[2], 0)  # 0 * log(0) = 0 convention
  expect_equal(result[3], log(1))
  expect_equal(result[4], log(2))
  expect_equal(result[5], 1)
})

test_that(".extract_complete_cases works", {
  skip_on_cran()
  y <- matrix(c(
    1, 2, 3,
    1, NA, 3,
    1, 2, 3,
    NA, NA, NA
  ), nrow = 4, byrow = TRUE)
  
  blocks <- c(1, 1, 2, 2)
  
  result <- .extract_complete_cases(y, blocks, warn = FALSE)
  
  expect_equal(nrow(result$y), 2)
  expect_equal(result$blocks, c(1, 2))
  expect_equal(result$n_removed, 2)
})

test_that(".extract_complete_cases warns when many removed", {
  skip_on_cran()
  y <- matrix(rnorm(100), nrow = 20, ncol = 5)
  y[1:15, 1] <- NA  # Make 75% incomplete
  
  expect_warning(
    result <- .extract_complete_cases(y, NULL),
    "Only 5/20"
  )
})

test_that(".extract_complete_cases errors when no complete cases", {
  skip_on_cran()
  y <- matrix(NA, nrow = 5, ncol = 3)
  
  expect_error(
    .extract_complete_cases(y, NULL),
    "No complete cases"
  )
})

# ==== Test logL_gau with missing data ====

test_that("logL_gau with na_action='fail' errors on missing data", {
  skip_on_cran()
  y <- matrix(c(1, 2, NA, 4, 5, 6), nrow = 2, byrow = TRUE)
  
  expect_error(
    logL_gau(y, order = 1, mu = rep(0, 3), phi = c(0.5, 0.5), 
            sigma = rep(1, 3), na_action = "fail"),
    "contains NA"
  )
})

test_that("logL_gau with na_action='complete' uses only complete cases", {
  skip_on_cran()
  set.seed(123)
  y_complete <- matrix(rnorm(30), nrow = 10, ncol = 3)
  y_missing <- y_complete
  y_missing[1:5, 1] <- NA  # Make first 5 subjects incomplete
  
  mu <- c(0, 0, 0)
  phi <- c(0.5, 0.5)
  sigma <- c(1, 1, 1)
  
  # Should only use last 5 subjects
  ll_complete <- logL_gau(y_missing, order = 1, mu = mu, phi = phi, sigma = sigma,
                         na_action = "complete")
  ll_last5 <- logL_gau(y_complete[6:10, ], order = 1, mu = mu, phi = phi, sigma = sigma,
                      na_action = "fail")
  
  expect_equal(ll_complete, ll_last5, tolerance = 1e-10)
})

test_that("logL_gau with na_action='marginalize' works for monotone dropout", {
  skip_on_cran()
  set.seed(123)
  y <- matrix(rnorm(15), nrow = 5, ncol = 3)
  y[1, 3] <- NA  # One missing at end
  
  mu <- c(0, 0, 0)
  phi <- c(0.5, 0.5)
  sigma <- c(1, 1, 1)
  
  # Should not error
  ll <- logL_gau(y, order = 1, mu = mu, phi = phi, sigma = sigma,
                na_action = "marginalize")
  
  expect_true(is.finite(ll))
})

test_that(".build_gau_covariance works for order 0", {
  skip_on_cran()
  sigma <- c(1, 2, 3)
  Sigma <- .build_gau_covariance(order = 0, phi = numeric(0), sigma = sigma, n_time = 3)
  
  expect_equal(diag(Sigma), sigma^2)
  expect_equal(Sigma[1, 2], 0)
  expect_equal(Sigma[1, 3], 0)
})

test_that(".build_gau_covariance works for order 1", {
  skip_on_cran()
  sigma <- c(1, 1, 1)
  phi <- c(0.5, 0.5)
  
  Sigma <- .build_gau_covariance(order = 1, phi = phi, sigma = sigma, n_time = 3)
  
  # Check variances increase (unless phi=0)
  expect_gte(Sigma[2, 2], Sigma[1, 1])
  expect_gte(Sigma[3, 3], Sigma[2, 2])
  
  # Check symmetry
  expect_equal(Sigma[1, 2], Sigma[2, 1])
})

test_that(".build_gau_covariance works for order 2", {
  skip_on_cran()
  set.seed(321)

  n_time <- 6
  sigma <- c(1.0, 1.1, 1.2, 1.0, 0.9, 1.1)
  phi <- matrix(0, nrow = 2, ncol = n_time)
  phi[1, 2:n_time] <- 0.45
  phi[2, 3:n_time] <- -0.20

  Sigma <- .build_gau_covariance(order = 2, phi = phi, sigma = sigma, n_time = n_time)

  expect_equal(dim(Sigma), c(n_time, n_time))
  expect_true(isSymmetric(Sigma, tol = 1e-10))
  expect_true(all(diag(Sigma) > 0))

  y <- simulate_gau(
    n_subjects = 5000,
    n_time = n_time,
    order = 2,
    mu = 0,
    phi = phi,
    sigma = sigma
  )
  Sigma_emp <- stats::cov(y)

  expect_equal(diag(Sigma_emp), diag(Sigma), tolerance = 0.12)
  expect_equal(Sigma_emp[3, 1], Sigma[3, 1], tolerance = 0.12)
  expect_equal(Sigma_emp[6, 4], Sigma[6, 4], tolerance = 0.12)
})

# ==== Test fit_gau with missing data ====

test_that("fit_gau with na_action='fail' errors on missing data", {
  skip_on_cran()
  y <- matrix(c(1, 2, NA, 4, 5, 6), nrow = 2, byrow = TRUE)
  
  expect_error(
    fit_gau(y, order = 1, na_action = "fail"),
    "contains NA"
  )
})

test_that("fit_gau with na_action='complete' removes incomplete subjects", {
  skip_on_cran()
  set.seed(123)
  y <- matrix(rnorm(50), nrow = 10, ncol = 5)
  y[1:3, 1] <- NA
  
  expect_warning(
    fit <- fit_gau(y, order = 1, na_action = "complete"),
    "Only 7/10"
  )
  
  expect_equal(fit$settings$n_subjects, 7)
  expect_equal(fit$n_obs, 7 * 5)
  expect_equal(fit$n_missing, 0)
})

test_that("fit_gau with complete data gives same result regardless of na_action", {
  skip_on_cran()
  set.seed(123)
  y <- matrix(rnorm(50), nrow = 10, ncol = 5)
  
  fit1 <- fit_gau(y, order = 1, na_action = "fail")
  fit2 <- fit_gau(y, order = 1, na_action = "complete")
  
  expect_equal(fit1$log_l, fit2$log_l, tolerance = 1e-6)
  expect_equal(fit1$mu, fit2$mu, tolerance = 1e-6)
})

# ==== Test EM algorithm ====

test_that(".initialize_gau_em works with enough complete cases", {
  skip_on_cran()
  set.seed(123)
  y <- matrix(rnorm(60), nrow = 20, ncol = 3)
  y[1:5, 1] <- NA  # 75% complete
  
  init <- .initialize_gau_em(y, order = 1, blocks = NULL, estimate_mu = TRUE)
  
  expect_length(init$mu, 3)
  expect_length(init$sigma, 3)
  expect_true(length(init$phi) %in% c(2, 3))
  if (length(init$phi) == 3) {
    expect_equal(init$phi[1], 0, tolerance = 1e-10)
  }
  expect_true(all(is.finite(init$mu)))
  expect_true(all(init$sigma > 0))
})

test_that(".initialize_gau_em falls back to marginal when few complete cases", {
  skip_on_cran()
  set.seed(123)
  y <- matrix(rnorm(30), nrow = 10, ncol = 3)
  y[1:9, 1] <- NA  # Only 1 complete case
  
  init <- .initialize_gau_em(y, order = 1, blocks = NULL, estimate_mu = TRUE)
  
  # Should still initialize
  expect_length(init$mu, 3)
  expect_true(all(is.finite(init$mu)))
})

test_that(".em_e_step_gau computes sufficient statistics", {
  skip_on_cran()
  set.seed(123)
  y <- matrix(rnorm(15), nrow = 5, ncol = 3)
  y[1, 3] <- NA
  
  mu <- c(0, 0, 0)
  phi <- c(0.5, 0.5)
  sigma <- c(1, 1, 1)
  
  suff_stats <- .em_e_step_gau(y, order = 1, mu, phi, sigma, blocks = NULL, tau = 0)
  
  expect_length(suff_stats$S1, 3)
  expect_equal(dim(suff_stats$S2), c(3, 3))
  expect_equal(suff_stats$n, 5)
  expect_true(all(is.finite(suff_stats$S1)))
})

test_that(".em_m_step_gau updates parameters", {
  skip_on_cran()
  # Simple sufficient statistics
  S1 <- c(5, 10, 15)
  S2 <- matrix(c(
    5, 7, 9,
    7, 20, 25,
    9, 25, 50
  ), nrow = 3, byrow = TRUE)
  suff_stats <- list(S1 = S1, S2 = S2, n = 10)
  
  params <- .em_m_step_gau(suff_stats, order = 1, blocks = NULL,
                          estimate_mu = TRUE, n_subjects = 10, n_time = 3)
  
  expect_length(params$mu, 3)
  expect_length(params$phi, 2)
  expect_length(params$sigma, 3)
  expect_true(all(params$sigma > 0))
})

test_that("fit_gau with EM converges on monotone missing data", {
  skip_on_cran()
  
  set.seed(123)
  y <- matrix(rnorm(30), nrow = 10, ncol = 3)
  y[1:3, 3] <- NA  # Monotone dropout
  
  fit <- fit_gau(y, order = 1, na_action = "em", em_max_iter = 50,
                em_verbose = FALSE)
  
  expect_true(fit$em_converged)
  expect_lte(fit$em_iterations, 50)
  expect_true(is.finite(fit$log_l))
  expect_gt(fit$n_missing, 0)
  expect_gt(fit$pct_missing, 0)
})

test_that("fit_gau EM handles blocks", {
  skip_on_cran()
  
  set.seed(123)
  y <- matrix(rnorm(40), nrow = 20, ncol = 2)
  y[1:5, 2] <- NA
  blocks <- c(rep(1, 10), rep(2, 10))
  
  fit <- fit_gau(y, order = 1, blocks = blocks, na_action = "em",
                em_max_iter = 30, em_verbose = FALSE)
  
  expect_true(fit$em_converged)
  expect_length(fit$tau, 2)
  expect_equal(fit$tau[1], 0)
})

test_that("EM log-likelihood is monotone increasing", {
  skip_on_cran()
  
  set.seed(123)
  y <- matrix(rnorm(25), nrow = 5, ncol = 5)
  y[sample(length(y), 5)] <- NA  # Random missing
  
  fit <- fit_gau(y, order = 1, na_action = "em", em_max_iter = 20,
                em_verbose = FALSE)
  
  ll_trace <- fit$em_ll_trace
  
  # Check monotonicity (allowing tiny numerical errors)
  diffs <- diff(ll_trace)
  expect_true(all(diffs >= -1e-6))
})

test_that("fit_gau EM order 2 converges and recovers dependence parameters", {
  skip_on_cran()

  set.seed(2026)

  n_subjects <- 400
  n_time <- 6
  mu_true <- seq(0.2, 0.7, length.out = n_time)
  sigma_true <- c(1.0, 1.1, 1.0, 0.9, 1.1, 1.0)
  phi_true <- matrix(0, nrow = 2, ncol = n_time)
  phi_true[1, 2:n_time] <- 0.50
  phi_true[2, 3:n_time] <- -0.25

  y_complete <- simulate_gau(
    n_subjects = n_subjects,
    n_time = n_time,
    order = 2,
    mu = mu_true,
    phi = phi_true,
    sigma = sigma_true
  )

  y_missing <- y_complete
  missing_idx <- sample(length(y_missing), size = round(0.12 * length(y_missing)))
  y_missing[missing_idx] <- NA

  fit_em <- NULL
  expect_warning(
    fit_em <- fit_gau(
      y_missing,
      order = 2,
      na_action = "em",
      em_max_iter = 120,
      em_tol = 1e-6,
      em_verbose = FALSE
    ),
    "provisional implementation"
  )
  fit_complete <- fit_gau(y_complete, order = 2, na_action = "fail")

  expect_true(fit_em$em_converged)
  expect_true(is.matrix(fit_em$phi))
  expect_equal(dim(fit_em$phi), c(2, n_time))
  expect_equal(fit_em$phi[1, 1], 0, tolerance = 1e-10)
  expect_equal(fit_em$phi[2, 1], 0, tolerance = 1e-10)
  expect_equal(fit_em$phi[2, 2], 0, tolerance = 1e-10)
  expect_true(all(diff(fit_em$em_ll_trace) >= -1e-6))

  # Agreement with truth and complete-data fit.
  expect_equal(fit_em$phi[1, 3:n_time], phi_true[1, 3:n_time], tolerance = 0.25)
  expect_equal(fit_em$phi[2, 3:n_time], phi_true[2, 3:n_time], tolerance = 0.25)
  expect_equal(fit_em$phi[1, 3:n_time], fit_complete$phi[1, 3:n_time], tolerance = 0.20)
  expect_equal(fit_em$phi[2, 3:n_time], fit_complete$phi[2, 3:n_time], tolerance = 0.20)
  expect_equal(fit_em$sigma[3:n_time], fit_complete$sigma[3:n_time], tolerance = 0.20)
})

# ==== Integration test ====

test_that("Complete workflow: simulate -> introduce missing -> fit with EM", {
  skip_on_cran()
  
  set.seed(42)
  
  # Simulate complete data (would use simulate_gau in practice)
  n <- 30
  p <- 4
  y_complete <- matrix(rnorm(n * p), nrow = n, ncol = p)
  
  # Introduce 10% MCAR missingness
  n_missing <- round(0.1 * n * p)
  missing_idx <- sample(n * p, n_missing)
  y_missing <- y_complete
  y_missing[missing_idx] <- NA
  
  # Fit with EM
  fit_em <- fit_gau(y_missing, order = 1, na_action = "em", 
                   em_max_iter = 50, em_verbose = FALSE)
  
  # Fit complete-case for comparison
  fit_cc <- suppressWarnings(
    fit_gau(y_missing, order = 1, na_action = "complete")
  )
  
  # EM should converge
  expect_true(fit_em$em_converged)
  
  # EM should have better (or equal) log-likelihood
  # (comparing on different data, so just check both are finite)
  expect_true(is.finite(fit_em$log_l))
  expect_true(is.finite(fit_cc$log_l))
  
  # EM uses more data
  expect_gt(fit_em$n_obs, fit_cc$n_obs)
})

#' Fit INAD antedependence model by maximum likelihood
#'
#' Fits INAD models by maximum likelihood.
#'
#' No fixed effect: time separable optimization using logL_inad_i with theta eliminated
#' by moment equations for order 1 and 2.
#'
#' Fixed effect: block coordinate descent using nloptr BOBYQA, updating tau, alpha,
#' theta, and nb_inno_size if needed.
#'
#' @param y Integer matrix n_sub by n_time.
#' @param order Integer in \{0, 1, 2\}.
#' @param thinning One of "binom", "pois", "nbinom".
#' @param innovation One of "pois", "bell", "nbinom".
#' @param blocks Optional integer vector length n_sub. Default NULL.
#' @param max_iter Max iterations for FE coordinate descent.
#' @param tol Tolerance for FE log likelihood stopping.
#' @param verbose Logical.
#' @param init_alpha Optional initial alpha. For order 1 numeric length 1 or n_time.
#'   For order 2 matrix n_time by 2 or list(alpha1, alpha2).
#' @param init_theta Optional initial theta numeric length 1 or n_time.
#' @param init_tau Optional initial tau. Scalar expands to c(0, x, ..., x). Vector forces first to 0.
#' @param init_nb_inno_size Optional initial size for innovation nbinom, length 1 or n_time.
#' @param nb_inno_size_ub Upper bound for innovation negative binomial size
#'   parameter when \code{innovation = "nbinom"}. Default is 50.
#' @param na_action How to handle missing values:
#'   \itemize{
#'     \item \code{"fail"}: stop if any NA is present.
#'     \item \code{"complete"}: fit using complete-case subjects only.
#'     \item \code{"marginalize"}: maximize observed-data likelihood under MAR.
#'   }
#'
#' @return A list of class \code{"inad_fit"} containing:
#' \describe{
#'   \item{alpha}{Estimated antedependence parameter(s)}
#'   \item{theta}{Estimated innovation parameter(s)}
#'   \item{tau}{Estimated block effects (if applicable)}
#'   \item{nb_inno_size}{Estimated innovation NB size parameter(s), when
#'     \code{innovation = "nbinom"}}
#'   \item{log_l}{Maximized log-likelihood}
#'   \item{aic}{Akaike information criterion}
#'   \item{bic}{Bayesian information criterion}
#'   \item{n_params}{Number of free parameters}
#'   \item{convergence}{Convergence code}
#'   \item{settings}{Model and fitting settings}
#' }
#'
#' @examples
#' set.seed(1)
#' y <- simulate_inad(
#'   n_subjects = 60,
#'   n_time = 5,
#'   order = 1,
#'   thinning = "binom",
#'   innovation = "pois",
#'   alpha = 0.3,
#'   theta = 2
#' )
#' fit <- fit_inad(y, order = 1, thinning = "binom", innovation = "pois", max_iter = 20)
#' fit$log_l
#' @export
fit_inad <- function(
        y,
        order = 1,
        thinning = c("binom", "pois", "nbinom"),
        innovation = c("pois", "bell", "nbinom"),
        blocks = NULL,
        max_iter = 50,
        tol = 1e-6,
        verbose = FALSE,
        init_alpha = NULL,
        init_theta = NULL,
        init_tau = 0.4,
        init_nb_inno_size = 1,
        nb_inno_size_ub = 50,
        na_action = c("fail", "complete", "marginalize")
) {
    thinning <- match.arg(thinning)
    innovation <- match.arg(innovation)
    na_action <- match.arg(na_action)

    if (!is.matrix(y)) y <- as.matrix(y)
    y_obs <- y[!is.na(y)]
    if (any(!is.finite(y_obs)) || any(y_obs < 0) || any(y_obs != floor(y_obs))) {
        stop("y must be a nonnegative integer matrix")
    }
    if (!(order %in% c(0, 1, 2))) stop("order must be 0, 1, or 2")
    if (!is.numeric(nb_inno_size_ub) || length(nb_inno_size_ub) != 1L ||
        !is.finite(nb_inno_size_ub) || nb_inno_size_ub <= 0) {
        stop("nb_inno_size_ub must be a positive finite scalar")
    }
    has_missing <- any(is.na(y))

    if (has_missing) {
        if (na_action == "fail") {
            stop("y contains NA values. Use na_action = 'complete' or 'marginalize'.", call. = FALSE)
        }

        if (na_action == "complete") {
            cc <- .extract_complete_cases(y, blocks, warn = TRUE)
            y <- cc$y
            blocks <- cc$blocks
            has_missing <- FALSE
        }

        if (na_action == "marginalize" && has_missing) {
            block_info <- .normalize_blocks(blocks, nrow(y))
            blocks_id <- if (is.null(blocks)) NULL else block_info$blocks_id
            fit <- .fit_inad_missing(
                y = y,
                order = order,
                thinning = thinning,
                innovation = innovation,
                blocks = blocks_id,
                max_iter = max_iter,
                tol = tol,
                verbose = verbose,
                init_alpha = init_alpha,
                init_theta = init_theta,
                init_tau = init_tau,
                init_nb_inno_size = init_nb_inno_size,
                nb_inno_size_ub = nb_inno_size_ub
            )
            return(.finalize_inad_fit(
                fit, y = y, na_action = na_action,
                blocks_id = blocks_id,
                block_levels = if (is.null(blocks)) NULL else block_info$block_levels
            ))
        }
    }

    if (is.null(blocks)) {
        fit <- fit_inad_no_fe(
            y = y,
            order = order,
            thinning = thinning,
            innovation = innovation,
            init_alpha = init_alpha,
            init_theta = init_theta,
            init_nb_inno_size = init_nb_inno_size,
            nb_inno_size_ub = nb_inno_size_ub
        )
        return(.finalize_inad_fit(fit, y = y, na_action = na_action, blocks_id = NULL, block_levels = NULL))
    }

    block_info <- .normalize_blocks(blocks, nrow(y))
    blocks_id <- block_info$blocks_id
    fit <- fit_inad_fe(
        y = y,
        order = order,
        thinning = thinning,
        innovation = innovation,
        blocks = blocks_id,
        max_iter = max_iter,
        tol = tol,
        verbose = verbose,
        init_alpha = init_alpha,
        init_theta = init_theta,
        init_tau = init_tau,
        init_nb_inno_size = init_nb_inno_size,
        nb_inno_size_ub = nb_inno_size_ub
    )
    .finalize_inad_fit(
        fit, y = y, na_action = na_action,
        blocks_id = blocks_id,
        block_levels = block_info$block_levels
    )
}

#' @keywords internal
.finalize_inad_fit <- function(fit, y, na_action = NULL, blocks_id = NULL, block_levels = NULL) {
    if (is.null(fit$settings)) fit$settings <- list()
    if (is.null(fit$settings$n_subjects)) fit$settings$n_subjects <- nrow(y)
    if (is.null(fit$settings$n_time)) fit$settings$n_time <- ncol(y)
    if (!is.null(na_action)) fit$settings$na_action <- na_action
    if (!is.null(blocks_id)) {
        n_blocks <- length(unique(blocks_id))
        fit$settings$blocks <- if (n_blocks > 1L) blocks_id else NULL
        fit$settings$n_blocks <- n_blocks
        fit$settings$block_levels <- block_levels
    }

    fit$n_obs <- sum(!is.na(y))
    fit$n_missing <- sum(is.na(y))
    fit$pct_missing <- mean(is.na(y)) * 100

    # Keep detailed convergence diagnostics while exposing a comparable code.
    if (is.list(fit$convergence)) {
        fit$convergence_info <- fit$convergence
        if (!is.null(fit$convergence$code)) {
            fit$convergence <- as.integer(fit$convergence$code)
        } else if (!is.null(fit$convergence$per_time)) {
            per_time <- as.integer(fit$convergence$per_time)
            if (length(per_time) == 0L || all(is.na(per_time))) {
                fit$convergence <- NA_integer_
            } else {
                fit$convergence <- as.integer(max(per_time, na.rm = TRUE))
            }
        } else {
            fit$convergence <- NA_integer_
        }
    } else if (is.null(fit$convergence)) {
        fit$convergence <- NA_integer_
    } else {
        fit$convergence <- as.integer(fit$convergence)
    }

    fit$n_params <- tryCatch(
        as.integer(.count_params_inad_fit(fit)),
        error = function(e) NA_integer_
    )

    if (is.finite(fit$log_l) && is.finite(fit$n_params)) {
        fit$aic <- -2 * fit$log_l + 2 * fit$n_params
        fit$bic <- -2 * fit$log_l + fit$n_params * log(fit$settings$n_subjects)
    } else {
        fit$aic <- NA_real_
        fit$bic <- NA_real_
    }

    class(fit) <- "inad_fit"
    fit
}

.fit_inad_missing <- function(
        y,
        order,
        thinning,
        innovation,
        blocks,
        max_iter,
        tol,
        verbose,
        init_alpha,
        init_theta,
        init_tau,
        init_nb_inno_size,
        nb_inno_size_ub
) {
    n <- nrow(y)
    N <- ncol(y)
    eps_pos <- 1e-8
    eps_alpha <- 1e-8
    alpha_ub <- 1 - eps_alpha

    # Normalize blocks and tau setup.
    if (!is.null(blocks)) {
        if (length(blocks) != n) stop("length(blocks) must equal nrow(y)")
        blocks <- as.integer(factor(blocks))
        B <- max(blocks)
    } else {
        B <- 1L
    }

    # Build a fallback complete-data initializer from complete cases when possible.
    fit_init <- NULL
    cc <- tryCatch(.extract_complete_cases(y, blocks, warn = FALSE), error = function(e) NULL)
    if (!is.null(cc) && nrow(cc$y) >= max(5L, order + 2L)) {
        fit_init <- tryCatch(
            fit_inad(
                y = cc$y,
                order = order,
                thinning = thinning,
                innovation = innovation,
                blocks = cc$blocks,
                max_iter = max_iter,
                tol = tol,
                verbose = FALSE,
                init_alpha = init_alpha,
                init_theta = init_theta,
                init_tau = init_tau,
                init_nb_inno_size = init_nb_inno_size,
                nb_inno_size_ub = nb_inno_size_ub,
                na_action = "fail"
            ),
            error = function(e) NULL
        )
    }

    theta_init <- if (!is.null(fit_init)) {
        as.numeric(fit_init$theta)
    } else if (!is.null(init_theta)) {
        th <- as.numeric(init_theta)
        if (length(th) == 1L) th <- rep(th, N)
        if (length(th) != N) stop("init_theta must be length 1 or ncol(y)")
        th
    } else {
        col_mean <- colMeans(y, na.rm = TRUE)
        col_mean[!is.finite(col_mean)] <- 1
        pmax(col_mean, 0.5)
    }
    theta_init <- pmax(theta_init, eps_pos)

    nb_init <- NULL
    if (innovation == "nbinom") {
        nb_init <- if (!is.null(fit_init) && !is.null(fit_init$nb_inno_size)) {
            as.numeric(fit_init$nb_inno_size)
        } else {
            nb <- as.numeric(init_nb_inno_size)
            if (length(nb) == 1L) nb <- rep(nb, N)
            if (length(nb) != N) stop("init_nb_inno_size must be length 1 or ncol(y)")
            nb
        }
        nb_init <- pmin(pmax(nb_init, eps_pos), nb_inno_size_ub)
    }

    alpha_init <- NULL
    if (order == 1) {
        if (!is.null(fit_init) && !is.null(fit_init$alpha)) {
            alpha_init <- as.numeric(fit_init$alpha)
        } else if (!is.null(init_alpha)) {
            a <- as.numeric(init_alpha)
            if (length(a) == 1L) a <- rep(a, N)
            if (length(a) != N) stop("init_alpha must be length 1 or ncol(y) for order=1")
            alpha_init <- a
        } else {
            alpha_init <- rep(0, N)
            if (N >= 2) alpha_init[2:N] <- 0.3
        }
        alpha_init[1] <- 0
        alpha_init <- pmin(pmax(alpha_init, 0), alpha_ub)
    }
    if (order == 2) {
        if (!is.null(fit_init) && !is.null(fit_init$alpha)) {
            alpha_init <- as.matrix(fit_init$alpha)
        } else if (!is.null(init_alpha)) {
            if (is.matrix(init_alpha) && ncol(init_alpha) >= 2) {
                alpha_init <- init_alpha
                if (nrow(alpha_init) == 1L) alpha_init <- alpha_init[rep.int(1L, N), , drop = FALSE]
                if (nrow(alpha_init) != N) stop("for order=2, init_alpha matrix must have nrow 1 or ncol(y)")
                alpha_init <- alpha_init[, 1:2, drop = FALSE]
            } else if (is.list(init_alpha) && length(init_alpha) >= 2) {
                a1 <- as.numeric(init_alpha[[1]])
                a2 <- as.numeric(init_alpha[[2]])
                if (length(a1) == 1L) a1 <- rep(a1, N)
                if (length(a2) == 1L) a2 <- rep(a2, N)
                if (length(a1) != N || length(a2) != N) {
                    stop("init_alpha list entries must be length 1 or ncol(y)")
                }
                alpha_init <- cbind(a1, a2)
            } else {
                stop("for order=2, init_alpha must be matrix with 2 columns or list(alpha1, alpha2)")
            }
        } else {
            alpha_init <- matrix(0, nrow = N, ncol = 2)
            if (N >= 2) alpha_init[2:N, 1] <- 0.3
            if (N >= 3) alpha_init[3:N, 2] <- 0.1
        }
        alpha_init[1, 1] <- 0
        alpha_init[1, 2] <- 0
        if (N >= 2) alpha_init[2, 2] <- 0
        alpha_init <- pmin(pmax(alpha_init, 0), alpha_ub)
    }

    tau_init <- NULL
    if (B > 1L) {
        if (!is.null(fit_init) && !is.null(fit_init$tau)) {
            tau_init <- as.numeric(fit_init$tau)
        } else {
            tau_init <- as.numeric(init_tau)
        }
        if (length(tau_init) == 1L) {
            tau_init <- c(0, rep(tau_init, B - 1L))
        } else {
            if (length(tau_init) < B) stop("init_tau must be length 1 or at least max(blocks)")
            tau_init <- tau_init[seq_len(B)]
            tau_init[1] <- 0
        }
    }

    pack_init <- function() {
        out <- c(log(theta_init))
        if (innovation == "nbinom") out <- c(out, log(nb_init))
        if (order == 1 && N >= 2) {
            out <- c(out, stats::qlogis(pmin(pmax(alpha_init[2:N] / alpha_ub, eps_alpha), 1 - eps_alpha)))
        }
        if (order == 2 && N >= 2) {
            out <- c(out, stats::qlogis(pmin(pmax(alpha_init[2:N, 1] / alpha_ub, eps_alpha), 1 - eps_alpha)))
            if (N >= 3) {
                out <- c(out, stats::qlogis(pmin(pmax(alpha_init[3:N, 2] / alpha_ub, eps_alpha), 1 - eps_alpha)))
            }
        }
        if (B > 1L) out <- c(out, tau_init[2:B])
        out
    }

    unpack_par <- function(par) {
        idx <- 1L
        theta <- exp(par[idx:(idx + N - 1L)])
        idx <- idx + N

        nb <- NULL
        if (innovation == "nbinom") {
            nb <- exp(par[idx:(idx + N - 1L)])
            idx <- idx + N
        }

        alpha <- NULL
        if (order == 1) {
            alpha <- rep(0, N)
            if (N >= 2) {
                a_free <- par[idx:(idx + N - 2L)]
                alpha[2:N] <- alpha_ub * stats::plogis(a_free)
                idx <- idx + N - 1L
            }
        }
        if (order == 2) {
            alpha <- matrix(0, nrow = N, ncol = 2)
            if (N >= 2) {
                a1_free <- par[idx:(idx + N - 2L)]
                alpha[2:N, 1] <- alpha_ub * stats::plogis(a1_free)
                idx <- idx + N - 1L
            }
            if (N >= 3) {
                a2_free <- par[idx:(idx + N - 3L)]
                alpha[3:N, 2] <- alpha_ub * stats::plogis(a2_free)
                idx <- idx + N - 2L
            }
        }

        tau <- 0
        if (B > 1L) {
            tau <- c(0, par[idx:(idx + B - 2L)])
        }

        list(
            theta = pmax(theta, eps_pos),
            nb_inno_size = if (is.null(nb)) NULL else pmin(pmax(nb, eps_pos), nb_inno_size_ub),
            alpha = alpha,
            tau = tau
        )
    }

    obj <- function(par) {
        cur <- unpack_par(par)
        ll <- tryCatch(
            logL_inad(
                y = y,
                order = order,
                thinning = thinning,
                innovation = innovation,
                alpha = cur$alpha,
                theta = cur$theta,
                nb_inno_size = cur$nb_inno_size,
                blocks = blocks,
                tau = cur$tau,
                na_action = "marginalize"
            ),
            error = function(e) -Inf
        )
        if (!is.finite(ll)) return(1e10)
        -ll
    }

    par0 <- pack_init()
    control <- list(maxit = max(200L, 20L * max_iter), reltol = tol)
    opt <- optim(par = par0, fn = obj, method = "BFGS", control = control)
    if (!is.finite(opt$value)) {
        opt <- optim(par = par0, fn = obj, method = "Nelder-Mead", control = control)
    }

    cur <- unpack_par(opt$par)
    log_l <- logL_inad(
        y = y,
        order = order,
        thinning = thinning,
        innovation = innovation,
        alpha = cur$alpha,
        theta = cur$theta,
        nb_inno_size = cur$nb_inno_size,
        blocks = blocks,
        tau = cur$tau,
        na_action = "marginalize"
    )

    if (verbose) {
        message("Missing-data fit completed: logL=", round(log_l, 4), " convergence=", opt$convergence)
    }

    list(
        alpha = cur$alpha,
        theta = cur$theta,
        nb_inno_size = cur$nb_inno_size,
        tau = if (is.null(blocks)) NULL else cur$tau,
        log_l = log_l,
        loglik_i = NULL,
        n_obs = sum(!is.na(y)),
        n_missing = sum(is.na(y)),
        pct_missing = mean(is.na(y)) * 100,
        convergence = list(code = opt$convergence, message = opt$message, method = opt$method),
        settings = list(
            order = order,
            thinning = thinning,
            innovation = innovation,
            blocks = blocks,
            n_subjects = n,
            n_time = N,
            na_action = "marginalize"
        )
    )
}

fit_inad_no_fe <- function(
        y,
        order,
        thinning,
        innovation,
        init_alpha,
        init_theta,
        init_nb_inno_size,
        nb_inno_size_ub
) {
    N <- ncol(y)
    eps_alpha <- 1e-10
    eps_pos <- 1e-10
    alpha_ub <- 1 - eps_alpha

    mean_y <- colMeans(y)

    theta_from_inno_mean <- function(mu, innovation, eps_pos) {
        if (!is.finite(mu) || mu <= eps_pos) return(NA_real_)
        if (innovation == "bell") return(solve_theta_bell(mu))
        mu
    }

    default_theta0 <- function(innovation) {
        if (innovation == "bell") return(1)
        10
    }

    theta <- if (is.null(init_theta)) {
        rep(default_theta0(innovation), N)
    } else {
        th <- as.numeric(init_theta)
        if (length(th) == 1L) th <- rep(th, N)
        if (length(th) != N) stop("init_theta must be length 1 or ncol(y)")
        th
    }

    nb_inno_size <- NULL
    if (innovation == "nbinom") {
        nb_inno_size <- as.numeric(init_nb_inno_size)
        if (length(nb_inno_size) == 1L) nb_inno_size <- rep(nb_inno_size, N)
        if (length(nb_inno_size) != N) stop("init_nb_inno_size must be length 1 or ncol(y)")
        nb_inno_size <- pmin(pmax(nb_inno_size, eps_pos), nb_inno_size_ub)
    }

    init_alpha_order1 <- function(init_alpha, N) {
        if (is.null(init_alpha)) {
            a <- rep(0, N)
            if (N >= 2) a[2:N] <- 0.4
            return(a)
        }
        a <- as.numeric(init_alpha)
        if (length(a) == 1L) a <- rep(a, N)
        if (length(a) != N) stop("init_alpha must be length 1 or ncol(y) for order=1")
        a
    }

    init_alpha_order2 <- function(init_alpha, N) {
        if (is.null(init_alpha)) {
            a1 <- rep(0, N)
            a2 <- rep(0, N)
            if (N >= 2) a1[2:N] <- 0.4
            if (N >= 3) a2[3:N] <- 0.4
            return(list(a1 = a1, a2 = a2))
        }

        if (is.matrix(init_alpha) && ncol(init_alpha) >= 2) {
            a <- init_alpha
            if (nrow(a) == 1L) a <- a[rep.int(1L, N), , drop = FALSE]
            if (nrow(a) != N) stop("for order=2, init_alpha matrix must have nrow 1 or ncol(y)")
            return(list(a1 = as.numeric(a[, 1]), a2 = as.numeric(a[, 2])))
        }

        if (is.list(init_alpha) && length(init_alpha) >= 2) {
            a1 <- as.numeric(init_alpha[[1]])
            a2 <- as.numeric(init_alpha[[2]])
            if (length(a1) == 1L) a1 <- rep(a1, N)
            if (length(a2) == 1L) a2 <- rep(a2, N)
            if (length(a1) != N || length(a2) != N) stop("init_alpha list entries must be length 1 or ncol(y)")
            return(list(a1 = a1, a2 = a2))
        }

        stop("for order=2, init_alpha must be a matrix with 2 columns or a list(alpha1, alpha2)")
    }

    if (order == 0) {
        alpha_out <- NULL
    } else if (order == 1) {
        alpha1 <- init_alpha_order1(init_alpha, N)
        alpha1[1] <- 0
        alpha1 <- pmin(pmax(alpha1, 0), alpha_ub)
        alpha_out <- alpha1
    } else {
        ainit <- init_alpha_order2(init_alpha, N)
        alpha1 <- ainit$a1
        alpha2 <- ainit$a2
        alpha1[1] <- 0
        alpha2[1] <- 0
        if (N >= 2) alpha2[2] <- 0
        alpha1 <- pmin(pmax(alpha1, 0), alpha_ub)
        alpha2 <- pmin(pmax(alpha2, 0), alpha_ub)
        alpha_out <- cbind(alpha1, alpha2)
    }

    loglik_i <- rep(NA_real_, N)
    conv <- rep(NA_integer_, N)

    for (i in seq_len(N)) {
        if (order == 0) {
            if (innovation == "nbinom") {
                obj0_nb <- function(par) {
                    th <- pmax(par[1], eps_pos)
                    sz <- pmax(par[2], eps_pos)
                    ll <- tryCatch(
                        logL_inad_i(
                            y = y,
                            i = i,
                            order = 0,
                            thinning = thinning,
                            innovation = innovation,
                            alpha = 0,
                            theta = replace(theta, i, th),
                            nb_inno_size = replace(nb_inno_size, i, sz)
                        ),
                        error = function(e) NA_real_
                    )
                    if (!is.finite(ll)) return(1e10)
                    -ll
                }

                par0 <- c(pmax(theta[i], eps_pos), pmax(nb_inno_size[i], eps_pos))
                fit0 <- optim(
                    par = par0,
                    fn = obj0_nb,
                    method = "L-BFGS-B",
                    lower = c(eps_pos, eps_pos),
                    upper = c(Inf, nb_inno_size_ub)
                )

                theta[i] <- pmax(fit0$par[1], eps_pos)
                nb_inno_size[i] <- pmin(pmax(fit0$par[2], eps_pos), nb_inno_size_ub)
                loglik_i[i] <- -fit0$value
                conv[i] <- fit0$convergence
            } else {
                obj0_th <- function(th) {
                    th <- pmax(th, eps_pos)
                    ll <- tryCatch(
                        logL_inad_i(
                            y = y,
                            i = i,
                            order = 0,
                            thinning = thinning,
                            innovation = innovation,
                            alpha = 0,
                            theta = replace(theta, i, th),
                            nb_inno_size = NULL
                        ),
                        error = function(e) NA_real_
                    )
                    if (!is.finite(ll)) return(1e10)
                    -ll
                }

                fit0 <- optim(
                    par = pmax(theta[i], eps_pos),
                    fn = obj0_th,
                    method = "L-BFGS-B",
                    lower = eps_pos,
                    upper = Inf
                )

                theta[i] <- pmax(fit0$par, eps_pos)
                loglik_i[i] <- -fit0$value
                conv[i] <- fit0$convergence
            }
            next
        }

        if (i == 1) {
            if (innovation == "nbinom") {
                obj1_nb <- function(par) {
                    th <- pmax(par[1], eps_pos)
                    sz <- pmax(par[2], eps_pos)

                    th_try <- theta
                    sz_try <- nb_inno_size
                    th_try[1] <- th
                    sz_try[1] <- sz

                    ll <- tryCatch(
                        logL_inad_i(
                            y = y,
                            i = 1,
                            order = 0,
                            thinning = thinning,
                            innovation = innovation,
                            alpha = 0,
                            theta = th_try,
                            nb_inno_size = sz_try
                        ),
                        error = function(e) NA_real_
                    )
                    if (!is.finite(ll)) return(1e10)
                    -ll
                }

                fit1 <- optim(
                    par = c(pmax(theta[1], eps_pos), pmax(nb_inno_size[1], eps_pos)),
                    fn = obj1_nb,
                    method = "L-BFGS-B",
                    lower = c(eps_pos, eps_pos),
                    upper = c(Inf, nb_inno_size_ub)
                )

                theta[1] <- pmax(fit1$par[1], eps_pos)
                nb_inno_size[1] <- pmin(pmax(fit1$par[2], eps_pos), nb_inno_size_ub)
                loglik_i[1] <- -fit1$value
                conv[1] <- fit1$convergence
            } else {
                obj1_th <- function(th) {
                    th <- pmax(th, eps_pos)
                    th_try <- theta
                    th_try[1] <- th

                    ll <- tryCatch(
                        logL_inad_i(
                            y = y,
                            i = 1,
                            order = 0,
                            thinning = thinning,
                            innovation = innovation,
                            alpha = 0,
                            theta = th_try,
                            nb_inno_size = NULL
                        ),
                        error = function(e) NA_real_
                    )
                    if (!is.finite(ll)) return(1e10)
                    -ll
                }

                fit1 <- optim(
                    par = pmax(theta[1], eps_pos),
                    fn = obj1_th,
                    method = "L-BFGS-B",
                    lower = eps_pos,
                    upper = Inf
                )

                theta[1] <- pmax(fit1$par, eps_pos)
                loglik_i[1] <- -fit1$value
                conv[1] <- fit1$convergence
            }
            next
        }

        if (order == 1 || i == 2) {
            m1 <- mean_y[i - 1]

            obj_a1 <- function(par) {
                a <- pmin(pmax(par[1], 0), alpha_ub)
                mu_inno <- mean_y[i] - a * m1
                th <- theta_from_inno_mean(mu_inno, innovation, eps_pos)
                if (!is.finite(th)) return(1e10)

                alpha_try <- alpha_out
                theta_try <- theta

                if (order == 1) {
                    alpha_try[i] <- a
                } else {
                    alpha_try[i, 1] <- a
                }
                theta_try[i] <- th

                if (innovation == "nbinom") {
                    sz <- pmax(par[2], eps_pos)
                    nb_try <- nb_inno_size
                    nb_try[i] <- sz
                    ll <- tryCatch(
                        logL_inad_i(
                            y = y,
                            i = i,
                            order = order,
                            thinning = thinning,
                            innovation = innovation,
                            alpha = alpha_try,
                            theta = theta_try,
                            nb_inno_size = nb_try
                        ),
                        error = function(e) NA_real_
                    )
                } else {
                    ll <- tryCatch(
                        logL_inad_i(
                            y = y,
                            i = i,
                            order = order,
                            thinning = thinning,
                            innovation = innovation,
                            alpha = alpha_try,
                            theta = theta_try,
                            nb_inno_size = NULL
                        ),
                        error = function(e) NA_real_
                    )
                }

                if (!is.finite(ll)) return(1e10)
                -ll
            }

            if (order == 1) {
                alpha_start <- alpha_out[i]
                if (innovation == "nbinom") {
                    fit_i <- optim(
                        par = c(alpha_start, nb_inno_size[i]),
                        fn = obj_a1,
                        method = "L-BFGS-B",
                        lower = c(0, eps_pos),
                        upper = c(alpha_ub, nb_inno_size_ub)
                    )
                    alpha_out[i] <- pmin(pmax(fit_i$par[1], 0), alpha_ub)
                    nb_inno_size[i] <- pmin(pmax(fit_i$par[2], eps_pos), nb_inno_size_ub)
                } else {
                    fit_i <- optim(
                        par = alpha_start,
                        fn = function(a) obj_a1(c(a)),
                        method = "L-BFGS-B",
                        lower = 0,
                        upper = alpha_ub
                    )
                    alpha_out[i] <- pmin(pmax(fit_i$par, 0), alpha_ub)
                }
            } else {
                alpha_start <- alpha_out[i, 1]
                if (innovation == "nbinom") {
                    fit_i <- optim(
                        par = c(alpha_start, nb_inno_size[i]),
                        fn = obj_a1,
                        method = "L-BFGS-B",
                        lower = c(0, eps_pos),
                        upper = c(alpha_ub, nb_inno_size_ub)
                    )
                    alpha_out[i, 1] <- pmin(pmax(fit_i$par[1], 0), alpha_ub)
                    nb_inno_size[i] <- pmin(pmax(fit_i$par[2], eps_pos), nb_inno_size_ub)
                } else {
                    fit_i <- optim(
                        par = alpha_start,
                        fn = function(a) obj_a1(c(a)),
                        method = "L-BFGS-B",
                        lower = 0,
                        upper = alpha_ub
                    )
                    alpha_out[i, 1] <- pmin(pmax(fit_i$par, 0), alpha_ub)
                }
            }

            if (order == 1) {
                mu_inno <- mean_y[i] - alpha_out[i] * m1
                th <- theta_from_inno_mean(mu_inno, innovation, eps_pos)
                if (!is.finite(th)) th <- eps_pos
                theta[i] <- th
            } else {
                theta[i] <- pmax(mean_y[i] - alpha_out[i, 1] * m1, eps_pos)
                alpha_out[i, 2] <- 0
            }

            loglik_i[i] <- -fit_i$value
            conv[i] <- fit_i$convergence
            next
        }

        m1 <- mean_y[i - 1]
        m2 <- mean_y[i - 2]

        obj_a12 <- function(par) {
            a1 <- pmin(pmax(par[1], 0), alpha_ub)
            a2 <- pmin(pmax(par[2], 0), alpha_ub)
            mu_inno <- mean_y[i] - a1 * m1 - a2 * m2
            th <- theta_from_inno_mean(mu_inno, innovation, eps_pos)
            if (!is.finite(th)) return(1e10)

            alpha_try <- alpha_out
            alpha_try[i, 1] <- a1
            alpha_try[i, 2] <- a2

            theta_try <- theta
            theta_try[i] <- th

            if (innovation == "nbinom") {
                sz <- pmax(par[3], eps_pos)
                nb_try <- nb_inno_size
                nb_try[i] <- sz
                ll <- tryCatch(
                    logL_inad_i(
                        y = y,
                        i = i,
                        order = order,
                        thinning = thinning,
                        innovation = innovation,
                        alpha = alpha_try,
                        theta = theta_try,
                        nb_inno_size = nb_try
                    ),
                    error = function(e) NA_real_
                )
            } else {
                ll <- tryCatch(
                    logL_inad_i(
                        y = y,
                        i = i,
                        order = order,
                        thinning = thinning,
                        innovation = innovation,
                        alpha = alpha_try,
                        theta = theta_try,
                        nb_inno_size = NULL
                    ),
                    error = function(e) NA_real_
                )
            }

            if (!is.finite(ll)) return(1e10)
            -ll
        }

        if (innovation == "nbinom") {
            fit_i <- optim(
                par = c(alpha_out[i, 1], alpha_out[i, 2], nb_inno_size[i]),
                fn = obj_a12,
                method = "L-BFGS-B",
                lower = c(0, 0, eps_pos),
                upper = c(alpha_ub, alpha_ub, nb_inno_size_ub)
            )
            alpha_out[i, 1] <- pmin(pmax(fit_i$par[1], 0), alpha_ub)
            alpha_out[i, 2] <- pmin(pmax(fit_i$par[2], 0), alpha_ub)
            nb_inno_size[i] <- pmin(pmax(fit_i$par[3], eps_pos), nb_inno_size_ub)
        } else {
            fit_i <- optim(
                par = c(alpha_out[i, 1], alpha_out[i, 2]),
                fn = function(p) obj_a12(p),
                method = "L-BFGS-B",
                lower = c(0, 0),
                upper = c(alpha_ub, alpha_ub)
            )
            alpha_out[i, 1] <- pmin(pmax(fit_i$par[1], 0), alpha_ub)
            alpha_out[i, 2] <- pmin(pmax(fit_i$par[2], 0), alpha_ub)
        }

        mu_inno <- mean_y[i] - alpha_out[i, 1] * m1 - alpha_out[i, 2] * m2
        th <- theta_from_inno_mean(mu_inno, innovation, eps_pos)
        if (!is.finite(th)) th <- eps_pos
        theta[i] <- th

        loglik_i[i] <- -fit_i$value
        conv[i] <- fit_i$convergence
    }

    if (!is.null(alpha_out)) alpha_out <- unname(alpha_out)
    theta <- unname(theta)
    if (!is.null(nb_inno_size)) nb_inno_size <- unname(nb_inno_size)

    list(
        alpha = alpha_out,
        theta = theta,
        nb_inno_size = nb_inno_size,
        tau = NULL,
        log_l = sum(loglik_i),
        loglik_i = loglik_i,
        convergence = list(per_time = conv),
        settings = list(
            order = order,
            thinning = thinning,
            innovation = innovation,
            blocks = NULL,
            n_subjects = nrow(y),
            n_time = ncol(y)
        )
    )
}

fit_inad_fe <- function(
        y,
        order,
        thinning,
        innovation,
        blocks,
        max_iter,
        tol,
        verbose,
        init_alpha,
        init_theta,
        init_tau,
        init_nb_inno_size,
        nb_inno_size_ub
) {
    if (!requireNamespace("nloptr", quietly = TRUE)) {
        stop("nloptr is required for fixed effect fitting")
    }

    n <- nrow(y)
    N <- ncol(y)

    blocks <- as.integer(blocks)
    if (length(blocks) != n) stop("length(blocks) must equal nrow(y)")
    if (any(blocks < 1)) stop("blocks must be positive integers starting at 1")
    B <- max(blocks)

    eps_alpha <- 1e-10
    eps_pos <- 1e-10
    alpha_ub <- 1 - eps_alpha

    default_theta0 <- function(innovation) {
        if (innovation == "bell") return(1)
        10
    }

    theta <- if (is.null(init_theta)) rep(default_theta0(innovation), N) else {
        th <- as.numeric(init_theta)
        if (length(th) == 1L) th <- rep(th, N)
        if (length(th) != N) stop("init_theta must be length 1 or ncol(y)")
        pmax(th, eps_pos)
    }

    nb_inno_size <- NULL
    if (innovation == "nbinom") {
        nb_inno_size <- as.numeric(init_nb_inno_size)
        if (length(nb_inno_size) == 1L) nb_inno_size <- rep(nb_inno_size, N)
        if (length(nb_inno_size) != N) stop("init_nb_inno_size must be length 1 or ncol(y)")
        nb_inno_size <- pmin(pmax(nb_inno_size, eps_pos), nb_inno_size_ub)
    }

    init_alpha_order1 <- function(init_alpha, N) {
        if (is.null(init_alpha)) {
            a <- rep(0, N)
            if (N >= 2) a[2:N] <- 0.4
            return(a)
        }
        a <- as.numeric(init_alpha)
        if (length(a) == 1L) a <- rep(a, N)
        if (length(a) != N) stop("init_alpha must be length 1 or ncol(y) for order=1")
        a
    }

    init_alpha_order2 <- function(init_alpha, N) {
        if (is.null(init_alpha)) {
            a1 <- rep(0, N)
            a2 <- rep(0, N)
            if (N >= 2) a1[2:N] <- 0.4
            if (N >= 3) a2[3:N] <- 0.4
            return(list(a1 = a1, a2 = a2))
        }

        if (is.matrix(init_alpha) && ncol(init_alpha) >= 2) {
            a <- init_alpha
            if (nrow(a) == 1L) a <- a[rep.int(1L, N), , drop = FALSE]
            if (nrow(a) != N) stop("for order=2, init_alpha matrix must have nrow 1 or ncol(y)")
            return(list(a1 = as.numeric(a[, 1]), a2 = as.numeric(a[, 2])))
        }

        if (is.list(init_alpha) && length(init_alpha) >= 2) {
            a1 <- as.numeric(init_alpha[[1]])
            a2 <- as.numeric(init_alpha[[2]])
            if (length(a1) == 1L) a1 <- rep(a1, N)
            if (length(a2) == 1L) a2 <- rep(a2, N)
            if (length(a1) != N || length(a2) != N) stop("init_alpha list entries must be length 1 or ncol(y)")
            return(list(a1 = a1, a2 = a2))
        }

        stop("for order=2, init_alpha must be a matrix with 2 columns or a list(alpha1, alpha2)")
    }

    alpha_full <- NULL
    if (order == 1) {
        alpha1 <- init_alpha_order1(init_alpha, N)
        alpha1[1] <- 0
        alpha1 <- pmin(pmax(alpha1, 0), alpha_ub)
        alpha_full <- alpha1
    }
    if (order == 2) {
        ainit <- init_alpha_order2(init_alpha, N)
        alpha1 <- ainit$a1
        alpha2 <- ainit$a2
        alpha1[1] <- 0
        alpha2[1] <- 0
        if (N >= 2) alpha2[2] <- 0
        alpha1 <- pmin(pmax(alpha1, 0), alpha_ub)
        alpha2 <- pmin(pmax(alpha2, 0), alpha_ub)
        alpha_full <- cbind(alpha1, alpha2)
    }

    tau <- as.numeric(init_tau)
    if (B <= 1L) {
        tau <- 0
    } else if (length(tau) == 1L) {
        tau <- c(0, rep(tau, B - 1L))
    } else {
        if (length(tau) < B) stop("init_tau must be length 1 or at least max(blocks)")
        tau <- tau[seq_len(B)]
        tau[1] <- 0
    }

    lambda_ok <- function(theta_vec, tau_vec) {
        mu <- if (innovation == "bell") {
            outer(rep(1, n), .bell_mean_from_theta(theta_vec)) + matrix(tau_vec[blocks], n, N)
        } else {
            outer(rep(1, n), theta_vec) + matrix(tau_vec[blocks], n, N)
        }
        all(is.finite(mu)) && all(mu > eps_pos)
    }

    log_old <- logL_inad(
        y = y,
        order = order,
        thinning = thinning,
        innovation = innovation,
        alpha = alpha_full,
        theta = theta,
        nb_inno_size = nb_inno_size,
        blocks = blocks,
        tau = tau
    )

    converged <- FALSE
    for (iter in seq_len(max_iter)) {
        if (verbose) message("iter ", iter, " logL=", log_old)

        if (B > 1L) {
            obj_tau <- function(tau_free) {
                tau_try <- c(0, tau_free)
                if (!lambda_ok(theta, tau_try)) return(1e8)
                -logL_inad(
                    y = y,
                    order = order,
                    thinning = thinning,
                    innovation = innovation,
                    alpha = alpha_full,
                    theta = theta,
                    nb_inno_size = nb_inno_size,
                    blocks = blocks,
                    tau = tau_try
                )
            }

            opt_tau <- nloptr::nloptr(
                x0 = tau[2:B],
                eval_f = obj_tau,
                opts = list(
                    algorithm = "NLOPT_LN_BOBYQA",
                    xtol_rel = 1e-6,
                    maxeval = 1000
                )
            )
            tau <- c(0, opt_tau$solution)
        } else {
            tau <- 0
        }

        if (order == 1) {
            if (N >= 2) {
                obj_a <- function(a_free) {
                    a_try <- alpha_full
                    a_try[1] <- 0
                    a_try[2:N] <- pmin(pmax(a_free, 0), alpha_ub)
                    -logL_inad(
                        y = y,
                        order = order,
                        thinning = thinning,
                        innovation = innovation,
                        alpha = a_try,
                        theta = theta,
                        nb_inno_size = nb_inno_size,
                        blocks = blocks,
                        tau = tau
                    )
                }

                opt_a <- nloptr::nloptr(
                    x0 = alpha_full[2:N],
                    eval_f = obj_a,
                    lb = rep(0, N - 1),
                    ub = rep(alpha_ub, N - 1),
                    opts = list(
                        algorithm = "NLOPT_LN_BOBYQA",
                        xtol_rel = 1e-6,
                        maxeval = 1500
                    )
                )

                alpha_full[2:N] <- pmin(pmax(opt_a$solution, 0), alpha_ub)
                alpha_full[1] <- 0
            }
        }

        if (order == 2) {
            if (N >= 3) {
                obj_a <- function(a_free) {
                    a1 <- alpha_full[, 1]
                    a2 <- alpha_full[, 2]

                    a1[1] <- 0
                    a2[1] <- 0
                    a2[2] <- 0

                    a1[2:N] <- pmin(pmax(a_free[seq_len(N - 1)], 0), alpha_ub)
                    a2[3:N] <- pmin(pmax(a_free[(N):(N + (N - 3))], 0), alpha_ub)

                    a_try <- cbind(a1, a2)

                    -logL_inad(
                        y = y,
                        order = order,
                        thinning = thinning,
                        innovation = innovation,
                        alpha = a_try,
                        theta = theta,
                        nb_inno_size = nb_inno_size,
                        blocks = blocks,
                        tau = tau
                    )
                }

                x0 <- c(alpha_full[2:N, 1], alpha_full[3:N, 2])
                lb <- rep(0, length(x0))
                ub <- rep(alpha_ub, length(x0))

                opt_a <- nloptr::nloptr(
                    x0 = x0,
                    eval_f = obj_a,
                    lb = lb,
                    ub = ub,
                    opts = list(
                        algorithm = "NLOPT_LN_BOBYQA",
                        xtol_rel = 1e-6,
                        maxeval = 2000
                    )
                )

                sol <- opt_a$solution
                alpha_full[1, 1] <- 0
                alpha_full[1, 2] <- 0
                alpha_full[2, 2] <- 0
                alpha_full[2:N, 1] <- pmin(pmax(sol[seq_len(N - 1)], 0), alpha_ub)
                alpha_full[3:N, 2] <- pmin(pmax(sol[(N):(N + (N - 3))], 0), alpha_ub)
            }
        }

        obj_t <- function(t_try) {
            t_try <- pmax(t_try, eps_pos)
            if (!lambda_ok(t_try, tau)) return(1e8)
            -logL_inad(
                y = y,
                order = order,
                thinning = thinning,
                innovation = innovation,
                alpha = alpha_full,
                theta = t_try,
                nb_inno_size = nb_inno_size,
                blocks = blocks,
                tau = tau
            )
        }

        opt_t <- nloptr::nloptr(
            x0 = theta,
            eval_f = obj_t,
            lb = rep(eps_pos, N),
            opts = list(
                algorithm = "NLOPT_LN_BOBYQA",
                xtol_rel = 1e-6,
                maxeval = 2000
            )
        )
        theta <- pmax(opt_t$solution, eps_pos)

        if (innovation == "nbinom") {
            obj_sz <- function(sz_try) {
                sz_try <- pmax(sz_try, eps_pos)
                -logL_inad(
                    y = y,
                    order = order,
                    thinning = thinning,
                    innovation = innovation,
                    alpha = alpha_full,
                    theta = theta,
                    nb_inno_size = sz_try,
                    blocks = blocks,
                    tau = tau
                )
            }

            opt_sz <- nloptr::nloptr(
                x0 = nb_inno_size,
                eval_f = obj_sz,
                lb = rep(eps_pos, N),
                ub = rep(nb_inno_size_ub, N),
                opts = list(
                    algorithm = "NLOPT_LN_BOBYQA",
                    xtol_rel = 1e-6,
                    maxeval = 2000
                )
            )
            nb_inno_size <- pmin(pmax(opt_sz$solution, eps_pos), nb_inno_size_ub)
        }

        log_new <- logL_inad(
            y = y,
            order = order,
            thinning = thinning,
            innovation = innovation,
            alpha = alpha_full,
            theta = theta,
            nb_inno_size = nb_inno_size,
            blocks = blocks,
            tau = tau
        )

        if (!is.finite(log_new)) break
        if (abs(log_new - log_old) < tol) {
            converged <- TRUE
            log_old <- log_new
            break
        }
        log_old <- log_new
    }

    list(
        alpha = alpha_full,
        theta = theta,
        tau = tau,
        nb_inno_size = nb_inno_size,
        log_l = log_old,
        convergence = if (converged) 0L else 1L,
        convergence_info = list(converged = converged, max_iter = max_iter),
        settings = list(
            order = order,
            thinning = thinning,
            innovation = innovation,
            blocks = blocks,
            n_subjects = n,
            n_time = N
        )
    )
}

solve_theta_bell <- function(mu, lower = 0, upper = 10) {
    if (!is.finite(mu) || mu <= 0) return(NA_real_)

    f <- function(theta) theta * exp(theta) - mu

    up <- upper
    fu <- f(up)
    while (is.finite(fu) && fu < 0 && up < 100) {
        up <- up * 2
        fu <- f(up)
    }
    if (!is.finite(fu) || fu < 0) return(NA_real_)

    uniroot(f, c(lower, up))$root
}

test_that("fit_inad works for bolus_inad without fixed effect", {
    skip_on_cran()
    skip_if_not(exists("bolus_inad", where = asNamespace("antedep"), inherits = FALSE) ||
                    exists("bolus_inad", where = parent.env(environment()), inherits = TRUE))

    data("bolus_inad", package = "antedep", envir = environment())
    y <- bolus_inad$y

    fit <- fit_inad(y, order = 1, thinning = "binom", innovation = "bell")

    expect_type(fit, "list")
    expect_true(is.null(fit$tau))
    expect_true(is.matrix(y))
    expect_equal(length(fit$alpha), ncol(y))
    expect_equal(length(fit$theta), ncol(y))

    expect_true(is.finite(fit$log_l))
    expect_true(is.finite(fit$aic))
    expect_true(is.finite(fit$bic))
    expect_true(is.integer(fit$n_params) || is.numeric(fit$n_params))
    expect_equal(fit$n_obs, length(y))
    expect_equal(fit$n_missing, 0)
    expect_true(is.integer(fit$convergence))

    expect_equal(fit$alpha[1], 0, tolerance = 1e-12)
    expect_true(all(is.finite(fit$alpha)))
    expect_true(all(fit$alpha >= -1e-12))
    expect_true(all(fit$alpha < 1))

    expect_true(all(is.finite(fit$theta)))
    expect_true(all(fit$theta > 0))
})

test_that("fit_inad returns log_l equal to sum(loglik_i) in no FE case", {
    skip_on_cran()
    data("bolus_inad", package = "antedep", envir = environment())
    y <- bolus_inad$y

    fit <- fit_inad(y, order = 1, thinning = "binom", innovation = "bell")

    expect_true(!is.null(fit$loglik_i))
    expect_equal(fit$log_l, sum(fit$loglik_i), tolerance = 1e-8)
})

test_that("fit_inad works for order 0 without fixed effect", {
    skip_on_cran()
    data("bolus_inad", package = "antedep", envir = environment())
    y <- bolus_inad$y

    fit <- fit_inad(y, order = 0, thinning = "binom", innovation = "bell")

    expect_true(is.null(fit$alpha) || length(fit$alpha) == 0)
    expect_equal(length(fit$theta), ncol(y))
    expect_true(is.finite(fit$log_l))
    expect_true(all(is.finite(fit$theta)))
    expect_true(all(fit$theta > 0))
})

test_that("fit_inad handles order 2 structural zeros without fixed effect", {
    skip_on_cran()
    data("bolus_inad", package = "antedep", envir = environment())
    y <- bolus_inad$y

    fit <- fit_inad(y, order = 2, thinning = "binom", innovation = "bell")

    expect_true(is.matrix(fit$alpha))
    expect_equal(dim(fit$alpha), c(ncol(y), 2))

    expect_equal(fit$alpha[1, 1], 0, tolerance = 1e-12)
    expect_equal(fit$alpha[1, 2], 0, tolerance = 1e-12)
    expect_equal(fit$alpha[2, 2], 0, tolerance = 1e-12)

    expect_true(all(is.finite(fit$alpha)))
    expect_true(all(fit$alpha >= -1e-12))
    expect_true(all(fit$alpha < 1))

    expect_true(is.finite(fit$log_l))
})

test_that("fit_inad works with fixed effect and tau normalization", {
    skip_on_cran()
    skip_if_not_installed("nloptr")

    data("bolus_inad", package = "antedep", envir = environment())
    y <- bolus_inad$y
    blocks <- bolus_inad$blocks

    fit1 <- fit_inad(
        y,
        order = 1,
        thinning = "binom",
        innovation = "bell",
        blocks = blocks,
        init_tau = 0.4,
        max_iter = 5
    )

    expect_true(is.finite(fit1$log_l))
    expect_true(!is.null(fit1$tau))
    expect_equal(fit1$tau[1], 0, tolerance = 1e-12)
    expect_equal(sort(fit1$settings$block_levels), c("1", "2"))

    B <- max(as.integer(blocks))
    expect_equal(length(fit1$tau), B)

    fit2 <- fit_inad(
        y,
        order = 1,
        thinning = "binom",
        innovation = "bell",
        blocks = blocks,
        init_tau = c(999, rep(0.2, B - 1)),
        max_iter = 5
    )

    expect_equal(fit2$tau[1], 0, tolerance = 1e-12)
})

test_that("fit_inad preserves original block labels in settings", {
    skip_on_cran()
    skip_if_not_installed("nloptr")

    data("bolus_inad", package = "antedep", envir = environment())
    y <- bolus_inad$y
    blocks_labeled <- ifelse(bolus_inad$blocks == 1, 5, 2)

    fit <- fit_inad(
        y,
        order = 1,
        thinning = "binom",
        innovation = "bell",
        blocks = blocks_labeled,
        max_iter = 5
    )

    expect_equal(sort(fit$settings$block_levels), c("2", "5"))
    expect_equal(length(fit$tau), 2)
})

test_that("fit_inad works for innovation nbinom and returns nb_inno_size", {
    skip_on_cran()
    skip_if_not_installed("nloptr")

    data("bolus_inad", package = "antedep", envir = environment())
    y <- bolus_inad$y

    fit <- fit_inad(
        y,
        order = 1,
        thinning = "binom",
        innovation = "nbinom",
        init_nb_inno_size = 1
    )

    expect_true(!is.null(fit$nb_inno_size))
    expect_equal(length(fit$nb_inno_size), ncol(y))
    expect_true(all(is.finite(fit$nb_inno_size)))
    expect_true(all(fit$nb_inno_size > 0))
    expect_true(is.finite(fit$log_l))
})

test_that("fit_inad respects nb_inno_size_ub upper bound", {
    skip_on_cran()
    set.seed(42)
    y <- matrix(rpois(60, lambda = 3), nrow = 20, ncol = 3)

    ub <- 2
    fit <- fit_inad(
        y,
        order = 1,
        thinning = "binom",
        innovation = "nbinom",
        nb_inno_size_ub = ub,
        init_nb_inno_size = 1,
        max_iter = 10
    )

    expect_true(all(fit$nb_inno_size <= ub + 1e-8))
    expect_true(is.finite(fit$log_l))
})

test_that("fit_inad validates nb_inno_size_ub", {
    skip_on_cran()
    y <- matrix(rpois(40, 2), nrow = 10, ncol = 4)

    expect_error(
        fit_inad(y, order = 1, thinning = "binom", innovation = "nbinom",
                 nb_inno_size_ub = -1),
        "nb_inno_size_ub must be a positive finite scalar"
    )
    expect_error(
        fit_inad(y, order = 1, thinning = "binom", innovation = "nbinom",
                 nb_inno_size_ub = Inf),
        "nb_inno_size_ub must be a positive finite scalar"
    )
})

test_that("fit_inad with na_action='fail' errors on missing data", {
    skip_on_cran()
    y <- matrix(c(1L, 2L, NA, 3L, 2L, 1L), nrow = 2, byrow = TRUE)

    expect_error(
        fit_inad(
            y,
            order = 1,
            thinning = "binom",
            innovation = "pois",
            na_action = "fail"
        ),
        "contains NA"
    )
})

test_that("fit_inad with na_action='complete' runs on complete-case subset", {
    skip_on_cran()
    set.seed(101)
    y <- matrix(rpois(120, lambda = 2), nrow = 30, ncol = 4)
    y[1:8, 4] <- NA

    fit <- suppressWarnings(
        fit_inad(
            y,
            order = 1,
            thinning = "binom",
            innovation = "pois",
            na_action = "complete"
        )
    )

    expect_true(is.finite(fit$log_l))
    expect_true(all(is.finite(fit$alpha)))
    expect_true(all(is.finite(fit$theta)))
})

test_that("fit_inad marginalize handles monotone MAR missingness", {
    skip_on_cran()

    set.seed(202)
    y_complete <- simulate_inad(
        n_subjects = 50,
        n_time = 5,
        order = 1,
        thinning = "binom",
        innovation = "pois",
        alpha = 0.35,
        theta = 2.0
    )

    # MAR monotone dropout probability depends on observed baseline count.
    y_mis <- y_complete
    p_drop <- stats::plogis(-1 + 0.2 * y_complete[, 1])
    drop <- runif(nrow(y_complete)) < p_drop
    y_mis[drop, 4:5] <- NA

    fit_marg <- fit_inad(
        y_mis,
        order = 1,
        thinning = "binom",
        innovation = "pois",
        na_action = "marginalize",
        max_iter = 12,
        tol = 1e-5
    )
    fit_full <- fit_inad(
        y_complete,
        order = 1,
        thinning = "binom",
        innovation = "pois"
    )

    expect_true(is.finite(fit_marg$log_l))
    expect_gt(fit_marg$n_missing, 0)
    expect_true(is.finite(fit_marg$aic))
    expect_true(is.finite(fit_marg$bic))
    expect_true(is.integer(fit_marg$convergence))
    expect_true(all(is.finite(fit_marg$alpha)))
    expect_true(all(is.finite(fit_marg$theta)))
    expect_equal(fit_marg$theta, fit_full$theta, tolerance = 1.2)
})

test_that("fit_inad marginalize handles intermittent MAR missingness", {
    skip_on_cran()

    set.seed(303)
    y_complete <- simulate_inad(
        n_subjects = 40,
        n_time = 6,
        order = 1,
        thinning = "binom",
        innovation = "pois",
        alpha = 0.30,
        theta = 1.8
    )

    # Intermittent MAR: missingness depends on observed baseline and time.
    y_mis <- y_complete
    for (t in 2:ncol(y_mis)) {
        p_mis_t <- stats::plogis(-2 + 0.18 * y_complete[, 1] + 0.15 * t)
        miss_t <- runif(nrow(y_mis)) < p_mis_t
        y_mis[miss_t, t] <- NA
    }

    fit_marg <- fit_inad(
        y_mis,
        order = 1,
        thinning = "binom",
        innovation = "pois",
        na_action = "marginalize",
        max_iter = 12,
        tol = 1e-5
    )

    expect_true(is.finite(fit_marg$log_l))
    expect_gt(fit_marg$n_missing, 0)
    expect_true(is.finite(fit_marg$aic))
    expect_true(is.finite(fit_marg$bic))
    expect_true(is.integer(fit_marg$convergence))
    expect_true(all(is.finite(fit_marg$alpha)))
    expect_true(all(is.finite(fit_marg$theta)))
    expect_true(!is.null(fit_marg$settings$na_action))
    expect_equal(fit_marg$settings$na_action, "marginalize")
})

# bic_cat.R - BIC computation for categorical antedependence models

#' Bayesian information criterion for fitted categorical AD models
#'
#' Computes BIC using the fitted log likelihood and a parameter count for
#' categorical antedependence parameters.
#'
#' @param fit A fitted model object of class \code{"cat_fit"} returned by
#'   \code{\link{fit_cat}}.
#' @param n_subjects Number of subjects. If NULL, extracted from fit.
#'
#' @return A numeric scalar BIC value.
#'
#' @details
#' The BIC is computed as:
#' \deqn{BIC = -2 \times \ell + k \times \log(N)}
#' where \eqn{\ell} is the log-likelihood, \eqn{k} is the number of free parameters,
#' and \eqn{N} is the number of subjects.
#'
#' @examples
#' set.seed(1)
#' y <- simulate_cat(40, 5, order = 1, n_categories = 2)
#'
#' # Fit models of different orders
#' fit0 <- fit_cat(y, order = 0)
#' fit1 <- fit_cat(y, order = 1)
#' fit2 <- fit_cat(y, order = 2)
#'
#' # Compare BIC
#' c(BIC_0 = bic_cat(fit0), BIC_1 = bic_cat(fit1), BIC_2 = bic_cat(fit2))
#'
#' @export
bic_cat <- function(fit, n_subjects = NULL) {
  if (!inherits(fit, "cat_fit")) {
    stop("fit must be a cat_fit object from fit_cat()")
  }
  
  if (is.null(n_subjects)) {
    n_subjects <- fit$settings$n_subjects
  }
  
  if (is.null(n_subjects) || n_subjects < 1) {
    stop("n_subjects must be a positive integer")
  }
  
  # BIC = -2 * log_l + k * log(N)
  -2 * fit$log_l + fit$n_params * log(n_subjects)
}


#' Akaike information criterion for fitted categorical AD models
#'
#' Computes AIC using the fitted log likelihood and a parameter count for
#' categorical antedependence parameters.
#'
#' @param fit A fitted model object of class \code{"cat_fit"} returned by
#'   \code{\link{fit_cat}}.
#'
#' @return A numeric scalar AIC value.
#'
#' @details
#' The AIC is computed as:
#' \deqn{AIC = -2 \times \ell + 2k}
#' where \eqn{\ell} is the log-likelihood and \eqn{k} is the number of free
#' parameters.
#'
#' @examples
#' set.seed(1)
#' y <- simulate_cat(40, 5, order = 1, n_categories = 2)
#' fit <- fit_cat(y, order = 1)
#' aic_cat(fit)
#'
#' @export
aic_cat <- function(fit) {
  if (!inherits(fit, "cat_fit")) {
    stop("fit must be a cat_fit object from fit_cat()")
  }
  
  # AIC = -2 * log_l + 2 * k
  -2 * fit$log_l + 2 * fit$n_params
}


#' BIC-based order selection for categorical AD models
#'
#' Fits AD models of increasing orders and selects the best by BIC.
#'
#' @param y Integer matrix of categorical data (n_subjects x n_time).
#' @param max_order Maximum order to consider. Default is 2.
#' @param blocks Optional block membership vector.
#' @param homogeneous Whether to use homogeneous parameters across blocks.
#' @param n_categories Number of categories (inferred if NULL).
#' @param criterion Which criterion to use: "bic" (default) or "aic".
#'
#' @return A list containing:
#'   \item{table}{Data frame with order, log_l, n_params, aic, bic}
#'   \item{bic}{Named numeric vector of BIC values by order}
#'   \item{best_order}{Order with lowest criterion value}
#'   \item{criterion}{Criterion used for order selection (\code{"bic"} or \code{"aic"})}
#'   \item{fits}{List of fitted models}
#'
#' @examples
#' \donttest{
#' y <- simulate_cat(100, 5, order = 1, n_categories = 2)
#' result <- bic_order_cat(y, max_order = 2)
#' print(result$table)
#' print(result$best_order)
#' }
#'
#' @export
bic_order_cat <- function(y, max_order = 2, blocks = NULL, homogeneous = TRUE,
                          n_categories = NULL, criterion = "bic") {
  
  # Validate criterion
  criterion <- match.arg(criterion, c("bic", "aic"))
  
  # Validate data
  validated <- .validate_y_cat(y, n_categories)
  y <- validated$y
  n_time <- ncol(y)
  
  # Determine maximum allowable order
  max_possible <- n_time - 1
  if (max_order > max_possible) {
    warning("max_order reduced to ", max_possible, " (n_time - 1)")
    max_order <- max_possible
  }
  if (max_order > 2) {
    warning("max_order > 2 not supported; setting to 2")
    max_order <- 2
  }
  
  # Fit models for each order
  orders <- 0:max_order
  fits <- vector("list", length(orders))
  names(fits) <- paste0("order_", orders)
  
  for (i in seq_along(orders)) {
    p <- orders[i]
    fits[[i]] <- fit_cat(y, order = p, blocks = blocks, homogeneous = homogeneous,
                         n_categories = n_categories)
  }
  
  # Build comparison table
  table_df <- data.frame(
    order = orders,
    log_l = sapply(fits, function(f) f$log_l),
    n_params = sapply(fits, function(f) f$n_params),
    aic = sapply(fits, function(f) f$aic),
    bic = sapply(fits, function(f) f$bic)
  )
  
  # Find best order
  if (criterion == "bic") {
    best_idx <- which.min(table_df$bic)
  } else {
    best_idx <- which.min(table_df$aic)
  }
  best_order <- orders[best_idx]
  bic_vals <- stats::setNames(table_df$bic, paste0("order_", orders))
  
  list(
    table = table_df,
    bic = bic_vals,
    best_order = best_order,
    criterion = criterion,
    fits = fits
  )
}

#' Fit Gaussian antedependence model by maximum likelihood
#'
#' Fits an AD(0), AD(1), or AD(2) model for Gaussian longitudinal data
#' by maximum likelihood. Missing values can be handled by complete-case
#' deletion or by EM (see \code{\link{em_gau}} for an explicit EM wrapper).
#'
#' @details
#' For missing data with \code{na_action = "em"}, AD orders 0 and 1 are the
#' primary production path. AD order 2 is available, but the current EM
#' implementation uses simplified second-order updates and should be treated as
#' provisional for high-stakes inference.
#'
#' For observed-data likelihood evaluation under MAR without fitting, use
#' \code{\link{logL_gau}} with \code{na_action = "marginalize"}. In contrast,
#' \code{fit_gau} handles missingness via complete-case fitting
#' (\code{na_action = "complete"}) or EM (\code{na_action = "em"}).
#'
#' @param y Numeric matrix (n_subjects x n_time). May contain NA.
#' @param order Integer 0, 1, or 2.
#' @param blocks Optional vector of block membership (length n_subjects).
#' @param na_action One of "fail", "complete", or "em".
#' @param estimate_mu Logical, whether to estimate mu (default TRUE).
#' @param em_max_iter Maximum EM iterations (only used when na_action = "em").
#' @param em_tol EM convergence tolerance (only used when na_action = "em").
#' @param em_verbose Logical, print EM progress (only used when na_action = "em").
#' @param ... Passed through to the EM fitter.
#'
#' @return A list with components including mu, phi, sigma, tau, log_l, n_obs, n_missing.
#'
#' @examples
#' set.seed(1)
#' y <- simulate_gau(n_subjects = 30, n_time = 5, order = 1, phi = 0.3)
#' fit <- fit_gau(y, order = 1)
#' fit$log_l
#'
#' y_miss <- y
#' y_miss[1, 2] <- NA
#' fit_em <- fit_gau(y_miss, order = 1, na_action = "em", em_max_iter = 20)
#' fit_em$settings$na_action
#'
#' @seealso \code{\link{em_gau}},
#'   \code{\link{fit_cat}}, \code{\link{fit_inad}}
#' @export
fit_gau <- function(y, order = 1, blocks = NULL,
                   na_action = c("fail", "complete", "em"),
                   estimate_mu = TRUE,
                   em_max_iter = 100, em_tol = 1e-6, em_verbose = FALSE, ...) {
    na_action <- match.arg(na_action)
    blocks_input <- blocks

    if (!is.matrix(y)) y <- as.matrix(y)
    storage.mode(y) <- "double"

    if (!order %in% c(0, 1, 2)) {
        stop("order must be 0, 1, or 2.", call. = FALSE)
    }

    if (!is.null(blocks) && length(blocks) != nrow(y)) {
        stop("blocks must have length n_subjects.", call. = FALSE)
    }

    has_missing <- any(is.na(y))

    if (has_missing) {
        if (na_action == "fail") {
            stop("y contains NA values. Use na_action = 'complete' or 'em'.", call. = FALSE)
        }

        if (na_action == "complete") {
            cc <- .extract_complete_cases(y, blocks)
            y <- cc$y
            blocks <- cc$blocks
            has_missing <- FALSE
        }

        if (na_action == "em") {
            block_info <- .normalize_blocks(blocks, nrow(y))
            blocks_id <- if (is.null(blocks_input)) NULL else block_info$blocks_id
            if (order == 2L) {
                warning(
                    "na_action = 'em' with order = 2 uses a provisional implementation; validate results with care.",
                    call. = FALSE
                )
            }
            fit <- .fit_gau_em(
                y = y,
                order = order,
                blocks = blocks_id,
                estimate_mu = estimate_mu,
                max_iter = em_max_iter,
                tol = em_tol,
                verbose = em_verbose,
                ...
            )
            if (!is.null(blocks_input)) {
                fit$settings$block_levels <- block_info$block_levels
            }
            return(fit)
        }
    }

    if (!is.matrix(y)) y <- as.matrix(y)
    n_subjects <- nrow(y)
    n_time <- ncol(y)
    block_levels <- NULL

    if (n_subjects < 2) {
        stop("Need at least 2 subjects to fit the model.", call. = FALSE)
    }

    if (!is.null(blocks)) {
        block_info <- .normalize_blocks(blocks, n_subjects)
        blocks <- block_info$blocks_id
        block_levels <- block_info$block_levels

        k <- length(unique(blocks))
        tau <- numeric(k)
        tau[1] <- 0

        idx_ref <- which(blocks == 1)
        if (length(idx_ref) == 0) {
            stop("Reference block (level 1) has no subjects.", call. = FALSE)
        }

        mu <- if (isTRUE(estimate_mu)) {
            colMeans(y[idx_ref, , drop = FALSE])
        } else {
            rep(0, n_time)
        }

        if (k > 1) {
            for (b in 2:k) {
                idx_b <- which(blocks == b)
                if (length(idx_b) == 0) next
                diffs <- y[idx_b, , drop = FALSE] - matrix(mu, nrow = length(idx_b), ncol = n_time, byrow = TRUE)
                tau[b] <- mean(diffs)
            }
        }

        mu_mat <- matrix(mu, nrow = n_subjects, ncol = n_time, byrow = TRUE)
        tau_mat <- matrix(tau[blocks], nrow = n_subjects, ncol = n_time)
        y_center <- y - mu_mat - tau_mat
    } else {
        tau <- 0
        mu <- if (isTRUE(estimate_mu)) colMeans(y) else rep(0, n_time)
        y_center <- y - matrix(mu, nrow = n_subjects, ncol = n_time, byrow = TRUE)
    }

    if (order == 0) {
        phi <- numeric(0)
        sigma <- apply(y_center, 2, function(z) sqrt(max(mean(z^2), 1e-8)))
    }

    if (order == 1) {
        phi <- numeric(n_time)
        sigma <- numeric(n_time)
        phi[1] <- 0

        sigma[1] <- sqrt(max(mean(y_center[, 1]^2), 1e-8))

        for (t in 2:n_time) {
            x <- y_center[, t - 1]
            yy <- y_center[, t]
            denom <- sum(x^2)

            if (!is.finite(denom) || denom <= 1e-12) {
                phi[t] <- 0
                resid <- yy
            } else {
                phi[t] <- sum(x * yy) / denom
                resid <- yy - phi[t] * x
            }

            sigma[t] <- sqrt(max(mean(resid^2), 1e-8))
        }
    }

    if (order == 2) {
        phi <- matrix(0, nrow = 2, ncol = n_time)
        sigma <- numeric(n_time)

        sigma[1] <- sqrt(max(mean(y_center[, 1]^2), 1e-8))

        if (n_time >= 2) {
            x <- y_center[, 1]
            yy <- y_center[, 2]
            denom <- sum(x^2)

            if (!is.finite(denom) || denom <= 1e-12) {
                phi[1, 2] <- 0
                resid <- yy
            } else {
                phi[1, 2] <- sum(x * yy) / denom
                resid <- yy - phi[1, 2] * x
            }
            sigma[2] <- sqrt(max(mean(resid^2), 1e-8))
        }

        if (n_time >= 3) {
            for (t in 3:n_time) {
                X <- cbind(y_center[, t - 1], y_center[, t - 2])
                yy <- y_center[, t]

                fit <- tryCatch(
                    stats::lm.fit(x = X, y = yy),
                    error = function(e) NULL
                )

                if (is.null(fit) || any(!is.finite(fit$coefficients))) {
                    phi[, t] <- c(0, 0)
                    resid <- yy
                } else {
                    phi[, t] <- fit$coefficients
                    resid <- fit$residuals
                }

                sigma[t] <- sqrt(max(mean(resid^2), 1e-8))
            }
        }
    }

    log_l <- logL_gau(
        y = y,
        order = order,
        mu = mu,
        phi = phi,
        sigma = sigma,
        blocks = blocks,
        tau = tau,
        na_action = "fail"
    )

    n_blocks <- if (is.null(blocks)) 1 else length(unique(blocks))
    n_params <- .count_params_gau(order, n_time, n_blocks)

    result <- list(
        mu = mu,
        phi = phi,
        sigma = sigma,
        tau = tau,
        log_l = log_l,
        aic = -2 * log_l + 2 * n_params,
        bic = -2 * log_l + n_params * log(n_subjects),
        n_params = as.integer(n_params),
        convergence = 0L,
        n_obs = sum(!is.na(y)),
        n_missing = sum(is.na(y)),
        pct_missing = mean(is.na(y)) * 100,
        missing_pattern = "complete",
        settings = list(
            order = order,
            n_time = n_time,
            n_subjects = n_subjects,
            blocks = blocks,
            block_levels = block_levels,
            estimate_mu = estimate_mu,
            na_action = na_action
        )
    )

    class(result) <- c("gau_fit", "list")
    result
}

#' Print method for gau_fit objects
#'
#' @param x A \code{gau_fit} object.
#' @param ... Additional arguments (ignored).
#'
#' @export
print.gau_fit <- function(x, ...) {
    cat("Gaussian Antedependence Model Fit\n")
    cat("=================================\n\n")

    cat("Order:", x$settings$order, "\n")
    cat("Time points:", x$settings$n_time, "\n")
    cat("Subjects:", x$settings$n_subjects, "\n")
    cat("Missing values:", x$n_missing, "(", round(x$pct_missing, 2), "%)\n")

    cat("\nLog-likelihood:", round(x$log_l, 4), "\n")
    if (!is.null(x$aic)) cat("AIC:", round(x$aic, 4), "\n")
    if (!is.null(x$bic)) cat("BIC:", round(x$bic, 4), "\n")
    if (!is.null(x$n_params)) cat("Parameters:", x$n_params, "\n")
    if (!is.null(x$convergence)) cat("Convergence code:", x$convergence, "\n")

    invisible(x)
}

test_that("aic_gau matches manual formula for Gaussian AD fit", {
  set.seed(410)
  y <- simulate_gau(
    n_subjects = 30,
    n_time = 5,
    order = 1,
    phi = c(0, rep(0.35, 4))
  )

  fit <- fit_gau(y, order = 1, estimate_mu = TRUE)
  aic_pkg <- aic_gau(fit)

  N <- ncol(y)
  k <- N + N + (N - 1)
  aic_manual <- -2 * fit$log_l + 2 * k

  expect_equal(aic_pkg, aic_manual)
})

# File: tests/testthat/test-bic_gau.R

test_that("bic_gau matches manual formula for order 1 without blocks", {
    skip_on_cran()
    skip_if_not(exists("simulate_gau"))
    skip_if_not(exists("fit_gau"))

    set.seed(1)
    y <- simulate_gau(
        n_subjects = 30,
        n_time = 5,
        order = 1,
        mu = seq(0, 0.4, length.out = 5),
        sigma = rep(1, 5),
        phi = c(0, rep(0.4, 4))
    )

    fit <- fit_gau(y, order = 1, estimate_mu = TRUE)
    b1 <- bic_gau(fit, n_subjects = nrow(y))
    b2 <- bic_gau(fit)

    N <- ncol(y)
    k <- N + N + (N - 1)

    expect_equal(b1, -2 * fit$log_l + k * log(nrow(y)))
    expect_equal(b2, b1)
})

test_that("bic_gau counts tau and order 2 antedependence parameters", {
    skip_on_cran()
    N <- 6

    fit <- list(
        log_l = -100,
        settings = list(order = 2, estimate_mu = FALSE),
        sigma = rep(1, N),
        mu = rep(0, N),
        phi = matrix(0, nrow = 2, ncol = N),
        tau = c(0, 0.1, 0.2)
    )

    b <- bic_gau(fit, n_subjects = 50)

    k <- 0
    k <- k + N
    k <- k + (2 * N - 3)
    k <- k + (3 - 1)

    expect_equal(b, -2 * fit$log_l + k * log(50))
})

testthat::test_that("fit_gau returns correctly shaped outputs", {
    skip_on_cran()
    testthat::skip_if_not(exists("fit_gau"))
    testthat::skip_if_not(exists("simulate_gau"))
    testthat::skip_if_not(exists("logL_gau"))

    set.seed(1)
    y <- simulate_gau(
        n_subjects = 30,
        n_time = 6,
        order = 1,
        mu = seq(0, 0.5, length.out = 6),
        phi = 0.5,
        sigma = 1
    )

    fit <- fit_gau(y, order = 1, estimate_mu = FALSE)

    testthat::expect_true(is.list(fit))
    testthat::expect_true(is.numeric(fit$mu))
    testthat::expect_equal(length(fit$mu), ncol(y))

    testthat::expect_true(is.numeric(fit$sigma))
    testthat::expect_equal(length(fit$sigma), ncol(y))
    testthat::expect_true(all(is.finite(fit$sigma)))
    testthat::expect_true(all(fit$sigma > 0))

    testthat::expect_true(is.numeric(fit$phi))
    testthat::expect_equal(length(fit$phi), ncol(y))
    testthat::expect_equal(fit$phi[1], 0)

    testthat::expect_true(is.finite(fit$log_l))
    testthat::expect_true(is.finite(fit$aic))
    testthat::expect_true(is.finite(fit$bic))
    testthat::expect_true(is.integer(fit$n_params))
    testthat::expect_true(is.integer(fit$convergence))
    testthat::expect_equal(fit$convergence, 0L)
    testthat::expect_output(print(fit), "Gaussian Antedependence Model Fit")
})

testthat::test_that("fit_gau log_l equals logL_gau at returned parameters", {
    skip_on_cran()
    testthat::skip_if_not(exists("fit_gau"))
    testthat::skip_if_not(exists("simulate_gau"))
    testthat::skip_if_not(exists("logL_gau"))

    set.seed(2)
    y <- simulate_gau(
        n_subjects = 40,
        n_time = 5,
        order = 0,
        mu = 0.2,
        sigma = 1.3
    )

    fit <- fit_gau(y, order = 0, estimate_mu = TRUE)

    ll <- logL_gau(
        y = y,
        order = 0,
        mu = fit$mu,
        sigma = fit$sigma
    )

    testthat::expect_equal(fit$log_l, ll, tolerance = 1e-6)
})

testthat::test_that("fit_gau respects structural zeros for order 2", {
    skip_on_cran()
    testthat::skip_if_not(exists("fit_gau"))
    testthat::skip_if_not(exists("simulate_gau"))

    set.seed(3)
    phi_true <- matrix(0, nrow = 2, ncol = 6)
    phi_true[1, 2:6] <- 0.4
    phi_true[2, 3:6] <- -0.2

    y <- simulate_gau(
        n_subjects = 50,
        n_time = 6,
        order = 2,
        mu = 0,
        phi = phi_true,
        sigma = 1
    )

    fit <- fit_gau(y, order = 2, estimate_mu = FALSE)

    testthat::expect_true(is.matrix(fit$phi))
    testthat::expect_equal(dim(fit$phi), c(2, 6))

    testthat::expect_equal(fit$phi[1, 1], 0)
    testthat::expect_equal(fit$phi[2, 1], 0)
    testthat::expect_equal(fit$phi[2, 2], 0)
})

testthat::test_that("fit_gau handles blocks and returns tau with tau[1] = 0", {
    skip_on_cran()
    testthat::skip_if_not(exists("fit_gau"))
    testthat::skip_if_not(exists("simulate_gau"))

    set.seed(4)
    blocks <- rep(1:2, each = 30)

    y <- simulate_gau(
        n_subjects = 60,
        n_time = 5,
        order = 1,
        mu = 0,
        phi = 0.3,
        sigma = 1,
        blocks = blocks,
        tau = 0.7
    )

    fit <- fit_gau(
        y,
        order = 1,
        blocks = blocks,
        estimate_mu = TRUE,
        init_tau = 0.2
    )

    testthat::expect_true(is.numeric(fit$tau))
    testthat::expect_equal(length(fit$tau), 2)
    testthat::expect_equal(fit$tau[1], 0)
    testthat::expect_equal(sort(fit$settings$block_levels), c("1", "2"))
})

testthat::test_that("fit_gau preserves original block labels in settings", {
    skip_on_cran()
    testthat::skip_if_not(exists("fit_gau"))

    set.seed(41)
    y <- matrix(rnorm(40 * 5), nrow = 40, ncol = 5)
    blocks <- rep(c("treat_B", "treat_A"), each = 20)

    fit <- fit_gau(y, order = 0, blocks = blocks, estimate_mu = TRUE)

    testthat::expect_equal(sort(fit$settings$block_levels), c("treat_A", "treat_B"))
    testthat::expect_equal(length(fit$tau), 2)
})

testthat::test_that("fit_gau estimates are closer to truth than a very wrong phi", {
    skip_on_cran()
    testthat::skip_if_not(exists("fit_gau"))
    testthat::skip_if_not(exists("simulate_gau"))
    testthat::skip_if_not(exists("logL_gau"))

    set.seed(5)
    y <- simulate_gau(
        n_subjects = 120,
        n_time = 6,
        order = 1,
        mu = 0,
        phi = 0.6,
        sigma = 1
    )

    fit <- fit_gau(y, order = 1, estimate_mu = FALSE)

    ll_fit <- fit$log_l
    ll_bad <- logL_gau(
        y = y,
        order = 1,
        mu = colMeans(y),
        phi = c(0, rep(-2, 5)),
        sigma = rep(1, 6)
    )

    testthat::expect_true(ll_fit > ll_bad)
})

testthat::test_that("fit_gau warns for order-2 EM path", {
    skip_on_cran()
    testthat::skip_if_not(exists("fit_gau"))
    testthat::skip_if_not(exists("simulate_gau"))

    set.seed(6)
    phi_true <- matrix(0, nrow = 2, ncol = 5)
    phi_true[1, 2:5] <- 0.2
    phi_true[2, 3:5] <- 0.1
    y <- simulate_gau(n_subjects = 40, n_time = 5, order = 2, phi = phi_true, sigma = 1)
    y[sample(length(y), 8)] <- NA

    testthat::expect_warning(
        fit_gau(y, order = 2, na_action = "em", em_max_iter = 5, em_tol = 1e-4),
        "provisional implementation"
    )
})

test_that("em_gau matches fit_gau with na_action = 'em'", {
  skip_on_cran()
  set.seed(321)
  y <- simulate_gau(
    n_subjects = 60,
    n_time = 5,
    order = 1,
    phi = c(0, rep(0.4, 4))
  )
  y[sample(length(y), 15)] <- NA

  fit_direct <- fit_gau(
    y = y,
    order = 1,
    na_action = "em",
    em_max_iter = 20,
    em_tol = 1e-6,
    em_verbose = FALSE
  )
  fit_wrapper <- em_gau(
    y = y,
    order = 1,
    max_iter = 20,
    tol = 1e-6,
    verbose = FALSE
  )

  expect_equal(fit_wrapper$log_l, fit_direct$log_l)
  expect_equal(fit_wrapper$em_iterations, fit_direct$em_iterations)
  expect_equal(fit_wrapper$em_converged, fit_direct$em_converged)
  expect_equal(fit_wrapper$settings$order, fit_direct$settings$order)
})

# simulate_cat.R - Data simulation for categorical antedependence models

#' Simulate categorical antedependence series
#'
#' Generate simulated longitudinal categorical data from an AD(p) model with
#' specified transition probabilities.
#'
#' @param n_subjects Number of subjects to simulate.
#' @param n_time Number of time points.
#' @param order Antedependence order p. Must be 0, 1, or 2. Default is 1.
#' @param n_categories Number of categories c. Default is 2 (binary).
#' @param marginal List of marginal/joint probabilities for initial time points.
#'   If NULL, uniform probabilities are used. See Details for structure.
#' @param transition List of transition probability arrays for time points
#'   k = p+1 to n. If NULL, uniform transitions are used. See Details.
#' @param blocks Optional integer vector of length n_subjects specifying group
#'   membership. Used with homogeneous = FALSE.
#' @param homogeneous Logical. If TRUE (default), same parameters for all subjects.
#'   If FALSE, marginal and transition should be lists indexed by block.
#' @param seed Optional random seed for reproducibility.
#'
#' @return Integer matrix with n_subjects rows and n_time columns, where each
#'   entry is a category code from 1 to c.
#'
#' @details
#' Data are simulated sequentially:
#' \enumerate{
#'   \item For k = 1: Draw Y(1) from marginal distribution
#'   \item For k = 2 to p: Draw Y(k) conditional on Y(1), ..., Y(k-1)
#'   \item For k = p+1 to n: Draw Y(k) conditional on Y(k-p), ..., Y(k-1)
#' }
#'
#' Parameter structure for marginal:
#' \itemize{
#'   \item Order 0: List with elements t1, t2, ..., tn, each a vector of length c
#'     summing to 1
#'   \item Order 1: List with element t1 (vector of length c)
#'   \item Order 2: List with t1 (vector), t2_given_1to1 (c x c matrix where rows
#'     represent conditioning values and columns represent outcomes)
#' }
#'
#' Parameter structure for transition:
#' \itemize{
#'   \item Order 0: Not used (NULL)
#'   \item Order 1: List with elements t2, t3, ..., tn, each c x c matrix where
#'     rows are previous values and columns are current values (rows sum to 1)
#'   \item Order 2: List with elements t3, t4, ..., tn, each c x c x c array where
#'     first two indices are conditioning values and third is outcome
#' }
#'
#' @references
#' Xie, Y. and Zimmerman, D. L. (2013). Antedependence models for nonstationary
#' categorical longitudinal data with ignorable missingness: likelihood-based
#' inference. \emph{Statistics in Medicine}, 32, 3274-3289.
#'
#' @examples
#' y <- simulate_cat(n_subjects = 30, n_time = 5, order = 1, n_categories = 3, seed = 1)
#' dim(y)
#'
#' @export
simulate_cat <- function(n_subjects, n_time, order = 1, n_categories = 2,
                         marginal = NULL, transition = NULL,
                         blocks = NULL, homogeneous = TRUE, seed = NULL) {
  
  # Set seed if provided
  if (!is.null(seed)) set.seed(seed)
  
  # Validate inputs
  n_subjects <- as.integer(n_subjects)
  n_time <- as.integer(n_time)
  p <- as.integer(order)
  c <- as.integer(n_categories)
  
  if (n_subjects < 1) stop("n_subjects must be at least 1")
  if (n_time < 1) stop("n_time must be at least 1")
  if (p < 0) stop("order must be non-negative")
  if (p > 2) stop("order > 2 not currently supported")
  if (p >= n_time) stop("order must be less than n_time")
  if (c < 2) stop("n_categories must be at least 2")
  
  # Process blocks
  if (is.null(blocks)) {
    blocks <- rep(1L, n_subjects)
  } else {
    blocks <- .validate_blocks_cat(blocks, n_subjects)
  }
  n_blocks <- max(blocks)
  
  # Set up default parameters if not provided
  if (is.null(marginal)) {
    marginal <- .default_marginal_cat(p, c, n_time, n_blocks, homogeneous)
  }
  if (is.null(transition) && p >= 1) {
    transition <- .default_transition_cat(p, c, n_time, n_blocks, homogeneous)
  }
  
  # Allocate output matrix
  y <- matrix(0L, nrow = n_subjects, ncol = n_time)
  
  # Simulate each subject
  for (s in seq_len(n_subjects)) {
    # Get parameters for this subject's group
    if (homogeneous || n_blocks == 1) {
      marg_s <- marginal
      trans_s <- transition
    } else {
      g <- blocks[s]
      marg_s <- marginal[[g]]
      trans_s <- transition[[g]]
    }
    
    y[s, ] <- .simulate_subject_cat(n_time, p, c, marg_s, trans_s)
  }
  
  y
}


#' Simulate one subject's trajectory
#'
#' @param n_time Number of time points
#' @param p Order
#' @param c Number of categories
#' @param marginal Marginal parameters
#' @param transition Transition parameters
#'
#' @return Integer vector of length n_time
#'
#' @keywords internal
.simulate_subject_cat <- function(n_time, p, c, marginal, transition) {
  y <- integer(n_time)
  
  if (p == 0) {
    # Independence: draw from marginal at each time
    for (k in seq_len(n_time)) {
      probs <- marginal[[k]]
      y[k] <- sample.int(c, size = 1, prob = probs)
    }
    return(y)
  }
  
  # AD(p) with p >= 1
  
  # Draw Y_1 from marginal
  y[1] <- sample.int(c, size = 1, prob = marginal[["t1"]])
  
  # Draw Y_2 to Y_p (if applicable)
  if (p >= 2 && n_time >= 2) {
    for (k in 2:min(p, n_time)) {
      trans_name <- paste0("t", k, "_given_1to", k - 1)
      trans_k <- marginal[[trans_name]]
      
      # Get the row of transition matrix corresponding to history
      # history is y[1:(k-1)]
      idx <- y[1:(k-1)]
      
      if (k == 2) {
        probs <- trans_k[idx[1], ]
      } else {
        # Need to index into multidimensional array
        probs <- trans_k[matrix(c(idx, NA), nrow = 1)[, -k]]
        # Actually, extract the conditional distribution
        probs <- .extract_conditional_probs(trans_k, idx)
      }
      
      y[k] <- sample.int(c, size = 1, prob = probs)
    }
  }
  
  # Draw Y_(p+1) to Y_n
  if (n_time > p) {
    for (k in (p + 1):n_time) {
      trans_name <- paste0("t", k)
      trans_k <- transition[[trans_name]]
      
      # History is y[(k-p):(k-1)]
      history <- y[(k - p):(k - 1)]
      
      # Extract conditional probabilities
      probs <- .extract_conditional_probs(trans_k, history)
      
      y[k] <- sample.int(c, size = 1, prob = probs)
    }
  }
  
  y
}


#' Extract conditional probabilities from transition array
#'
#' @param trans_array Transition array with last dimension being outcome
#' @param history Vector of conditioning values
#'
#' @return Vector of probabilities for current time point
#'
#' @keywords internal
.extract_conditional_probs <- function(trans_array, history) {
  dims <- dim(trans_array)
  n_dims <- length(dims)
  
  if (n_dims == 1) {
    # Marginal distribution (no conditioning)
    return(trans_array)
  }
  
  if (length(history) == 1 && n_dims == 2) {
    # Order 1: matrix
    return(trans_array[history[1], ])
  }
  
  if (length(history) == 2 && n_dims == 3) {
    # Order 2: 3D array
    return(trans_array[history[1], history[2], ])
  }
  
  # General case: build index
  idx <- as.list(history)
  idx[[length(idx) + 1]] <- TRUE  # Select all values of outcome
  
  probs <- do.call(`[`, c(list(trans_array), idx))
  as.numeric(probs)
}


#' Create default marginal parameters (uniform)
#'
#' @param p Order
#' @param c Number of categories
#' @param n_time Number of time points
#' @param n_blocks Number of groups
#' @param homogeneous Whether to share parameters
#'
#' @return List of marginal parameters
#'
#' @keywords internal
.default_marginal_cat <- function(p, c, n_time, n_blocks, homogeneous) {
  # Create uniform marginal for one population
  create_one <- function() {
    marg <- list()
    
    if (p == 0) {
      # Independence: uniform marginal at each time point
      for (k in seq_len(n_time)) {
        marg[[paste0("t", k)]] <- rep(1/c, c)
      }
    } else {
      # AD(p): need marginal for t1 and conditionals for t2 to t_p
      marg[["t1"]] <- rep(1/c, c)
      
      if (p >= 2 && n_time >= 2) {
        for (k in 2:min(p, n_time)) {
          # Uniform conditional: each row sums to 1
          dims <- rep(c, k)
          trans_k <- array(1/c, dim = dims)
          marg[[paste0("t", k, "_given_1to", k - 1)]] <- trans_k
        }
      }
    }
    marg
  }
  
  if (homogeneous || n_blocks == 1) {
    return(create_one())
  } else {
    result <- vector("list", n_blocks)
    for (g in seq_len(n_blocks)) {
      result[[g]] <- create_one()
    }
    return(result)
  }
}


#' Create default transition parameters (uniform)
#'
#' @param p Order
#' @param c Number of categories
#' @param n_time Number of time points
#' @param n_blocks Number of groups
#' @param homogeneous Whether to share parameters
#'
#' @return List of transition parameters
#'
#' @keywords internal
.default_transition_cat <- function(p, c, n_time, n_blocks, homogeneous) {
  if (p == 0) return(NULL)
  
  # Create uniform transitions for one population
  create_one <- function() {
    trans <- list()
    
    for (k in (p + 1):n_time) {
      # Uniform transition: array of dimension c^(p+1), each "row" sums to 1
      dims <- rep(c, p + 1)
      trans_k <- array(1/c, dim = dims)
      trans[[paste0("t", k)]] <- trans_k
    }
    trans
  }
  
  if (homogeneous || n_blocks == 1) {
    return(create_one())
  } else {
    result <- vector("list", n_blocks)
    for (g in seq_len(n_blocks)) {
      result[[g]] <- create_one()
    }
    return(result)
  }
}


#' Validate transition probability parameters
#'
#' Checks that transition probabilities are valid (non-negative, sum to 1).
#'
#' @param marginal Marginal parameters
#' @param transition Transition parameters
#' @param p Order
#' @param c Number of categories
#' @param n_time Number of time points
#' @param tol Tolerance for checking sums
#'
#' @return TRUE if valid, otherwise throws error
#'
#' @keywords internal
.validate_params_cat <- function(marginal, transition, p, c, n_time, tol = 1e-8) {
  # Check marginal
  if (p == 0) {
    for (k in seq_len(n_time)) {
      probs <- marginal[[k]]
      if (any(probs < 0)) stop("Negative probabilities in marginal[[", k, "]]")
      if (abs(sum(probs) - 1) > tol) stop("Probabilities don't sum to 1 in marginal[[", k, "]]")
    }
  } else {
    # Check t1
    if (any(marginal[["t1"]] < 0)) stop("Negative probabilities in marginal$t1")
    if (abs(sum(marginal[["t1"]]) - 1) > tol) stop("Probabilities don't sum to 1 in marginal$t1")
    
    # Check conditionals for t2 to tp
    if (p >= 2) {
      for (k in 2:min(p, n_time)) {
        trans_name <- paste0("t", k, "_given_1to", k - 1)
        trans_k <- marginal[[trans_name]]
        if (any(trans_k < 0)) stop("Negative probabilities in marginal$", trans_name)
      }
    }
  }
  
  # Check transitions
  if (p >= 1 && n_time > p) {
    for (k in (p + 1):n_time) {
      trans_name <- paste0("t", k)
      trans_k <- transition[[trans_name]]
      if (any(trans_k < 0)) stop("Negative probabilities in transition$", trans_name)
    }
  }
  
  TRUE
}

#' Morphine bolus analgesia counts
#'
#' Morphine bolus self administration counts for two treatment groups
#' recorded at 12 four hour time points. The data are stored in matrix
#' form to facilitate use with antedependence models.
#'
#' @format A list with four components:
#'   \describe{
#'     \item{y}{integer matrix of dimension N by n_time containing all
#'       subjects and time points}
#'     \item{y_2mg}{integer matrix with rows corresponding to the 2 mg
#'       treatment group}
#'     \item{y_1mg}{integer matrix with rows corresponding to the 1 mg
#'       treatment group}
#'     \item{blocks}{integer vector of length N giving the block or
#'       treatment group indicator, 1 for 2 mg and 2 for 1 mg}
#'   }
#'
#' @source Dataset `bolus` from the \pkg{cold} package, converted to
#'   matrix form and grouped by treatment.
"bolus_inad"

#' 100km race split-time data
#'
#' Split times (in minutes) from a 100km race example with 10 consecutive sections.
#' This is continuous-response longitudinal data suitable for Gaussian AD modeling.
#'
#' @format A list with five components:
#'   \describe{
#'     \item{y}{numeric matrix of dimension N by 10 containing section split times}
#'     \item{age}{numeric vector of subject ages (may include missing values)}
#'     \item{subject}{integer subject identifiers}
#'     \item{time}{integer vector of section indices (1:10)}
#'     \item{section_labels}{character vector of split-time column names}
#'   }
#'
#' @source Combined table extracted from textbook race example data, stored in
#'   \code{data-raw/external/100km_race_combined_extracted.csv}.
"race_100km"

#' EM algorithm for AD with missing data
#'
#' Implements the EM algorithm for fitting AD(p) models with missing data
#' under MAR (Missing At Random) assumption.
#'
#' @param y Data matrix (n_subjects x n_time), may contain NA
#' @param order Antedependence order (0, 1, or 2)
#' @param blocks Block membership vector (optional)
#' @param estimate_mu Logical, whether to estimate mu (default TRUE)
#' @param max_iter Maximum number of EM iterations (default 100)
#' @param tol Convergence tolerance on log-likelihood (default 1e-6)
#' @param verbose Logical, print iteration info (default FALSE)
#'
#' @return List with fitted model (same structure as fit_gau, plus EM info)
#'
#' @keywords internal
.fit_gau_em <- function(y, order, blocks = NULL, estimate_mu = TRUE,
                       max_iter = 100, tol = 1e-6, verbose = FALSE, ...) {
  
  n_subjects <- nrow(y)
  n_time <- ncol(y)
  
  # Validate missing data
  missing_info <- .validate_missing(y)
  
  if (verbose) {
    cat("\n=== EM Algorithm for AD with Missing Data ===\n")
    cat("Missing data: ", missing_info$pct_missing, "% (", 
        sum(is.na(y)), " values)\n", sep = "")
    cat("Complete subjects: ", missing_info$n_complete, "/", n_subjects, "\n")
    cat("Intermittent missing: ", missing_info$n_intermittent, " subjects\n\n")
  }
  
  # Initialize parameters from complete cases or simple estimates
  init <- .initialize_gau_em(y, order, blocks, estimate_mu)
  mu <- init$mu
  phi <- init$phi
  sigma <- init$sigma
  tau <- init$tau
  
  # Track log-likelihood
  ll_trace <- numeric(max_iter)
  converged <- FALSE
  
  for (iter in 1:max_iter) {
    # E-step: Compute sufficient statistics
    suff_stats <- .em_e_step_gau(y, order, mu, phi, sigma, blocks, tau)
    
    # M-step: Update parameters
    params <- .em_m_step_gau(suff_stats, order, blocks, estimate_mu, n_subjects, n_time)
    mu <- params$mu
    phi <- params$phi
    sigma <- params$sigma
    tau <- params$tau
    
    # Compute observed-data log-likelihood
    ll <- .logL_gau_missing(y, order, mu, phi, sigma, blocks, tau)
    ll_trace[iter] <- ll
    
    # Check for invalid log-likelihood
    if (!is.finite(ll)) {
      warning("Non-finite log-likelihood at iteration ", iter, 
              ". EM terminated.", call. = FALSE)
      converged <- FALSE
      break
    }
    
    if (verbose && (iter %% 10 == 0 || iter == 1)) {
      cat("Iteration ", iter, ": log-lik = ", round(ll, 4), "\n", sep = "")
    }
    
    # Check convergence
    if (iter > 1) {
      if (abs(ll_trace[iter] - ll_trace[iter - 1]) < tol) {
        converged <- TRUE
        if (verbose) {
          cat("\nConverged at iteration ", iter, "\n")
        }
        break
      }
      
      # Check for non-monotonic increase (should not happen with EM)
      if (ll_trace[iter] < ll_trace[iter - 1] - 1e-6) {
        warning("EM log-likelihood decreased at iteration ", iter,
                ". This should not happen.", call. = FALSE)
      }
    }
  }
  
  if (!converged && verbose) {
    cat("\nWarning: EM did not converge in ", max_iter, " iterations\n")
  }
  
  # Compute final quantities
  n_blocks <- if (is.null(blocks)) 1 else length(unique(blocks))
  n_params <- .count_params_gau(order, n_time, n_blocks)
  
  # Return fitted model
  result <- list(
    mu = mu,
    phi = phi,
    sigma = sigma,
    tau = if (n_blocks > 1) tau else NULL,
    log_l = ll,
    aic = -2 * ll + 2 * n_params,
    bic = -2 * ll + n_params * log(n_subjects),
    n_params = as.integer(n_params),
    convergence = if (converged) 0L else 1L,
    message = if (converged) "EM converged" else "EM did not converge",
    
    # Missing data info
    n_obs = sum(!is.na(y)),
    n_missing = sum(is.na(y)),
    pct_missing = missing_info$pct_missing,
    missing_pattern = if (missing_info$n_intermittent > 0) "intermittent" 
                      else "monotone",
    
    # EM convergence info
    em_converged = converged,
    em_iterations = iter,
    em_ll_trace = ll_trace[1:iter],
    
    settings = list(
      order = order,
      n_time = n_time,
      n_subjects = n_subjects,
      blocks = blocks,
      estimate_mu = estimate_mu
    )
  )
  
  class(result) <- c("gau_fit", "list")
  result
}

#' Initialize parameters for AD EM algorithm
#'
#' @keywords internal
.initialize_gau_em <- function(y, order, blocks, estimate_mu) {
  # Try complete-case initialization
  cc <- .extract_complete_cases(y, blocks, warn = FALSE)
  
  if (nrow(cc$y) >= 10) {
    # Enough complete cases - initialize from the complete-case MLE.
    tryCatch({
      fit_cc <- fit_gau(
        y = cc$y,
        order = order,
        blocks = cc$blocks,
        na_action = "fail",
        estimate_mu = estimate_mu
      )

      tau <- fit_cc$tau
      if (is.null(tau)) tau <- 0

      list(
        mu = fit_cc$mu,
        phi = fit_cc$phi,
        sigma = fit_cc$sigma,
        tau = tau
      )
    }, error = function(e) {
      # Fallback to marginal estimates
      .initialize_gau_marginal(y, order, blocks)
    })
  } else {
    # Too few complete cases - use marginal estimates
    .initialize_gau_marginal(y, order, blocks)
  }
}

#' Initialize from marginal statistics (ignoring dependence)
#'
#' @keywords internal
.initialize_gau_marginal <- function(y, order, blocks) {
  n_time <- ncol(y)
  
  mu <- colMeans(y, na.rm = TRUE)
  sigma <- apply(y, 2, sd, na.rm = TRUE)
  sigma[sigma == 0 | is.na(sigma)] <- 1  # Avoid zero sigma
  
  # Default phi: moderate positive dependence
  if (order == 1) {
    phi <- rep(0.3, n_time - 1)
  } else if (order == 2) {
    phi <- matrix(0, nrow = 2, ncol = n_time)
    if (n_time >= 2) phi[1, 2:n_time] <- 0.25
    if (n_time >= 3) phi[2, 3:n_time] <- 0.10
  } else {
    phi <- numeric(0)
  }
  
  if (!is.null(blocks)) {
    n_blocks <- length(unique(blocks))
    tau <- numeric(n_blocks)
    tau[1] <- 0
    
    # Simple block means
    if (n_blocks > 1) {
      for (b in 2:n_blocks) {
        block_data <- y[blocks == b, , drop = FALSE]
        tau[b] <- mean(block_data, na.rm = TRUE) - mean(y, na.rm = TRUE)
      }
    }
  } else {
    tau <- 0
  }
  
  list(mu = mu, phi = phi, sigma = sigma, tau = tau)
}

#' E-step: Compute expected sufficient statistics
#'
#' Computes conditional expectations for missing values and second moments.
#'
#' @keywords internal
.em_e_step_gau <- function(y, order, mu, phi, sigma, blocks, tau) {
  n_subjects <- nrow(y)
  n_time <- ncol(y)
  
  # Build full covariance matrix
  Sigma <- .build_gau_covariance(order, phi, sigma, n_time)
  
  # Adjust mean for blocks if present
  if (!is.null(blocks)) {
    mu_matrix <- matrix(mu, nrow = n_subjects, ncol = n_time, byrow = TRUE)
    tau_matrix <- matrix(tau[blocks], nrow = n_subjects, ncol = n_time)
    mu_matrix <- mu_matrix + tau_matrix
  } else {
    mu_matrix <- matrix(mu, nrow = n_subjects, ncol = n_time, byrow = TRUE)
  }
  
  # Initialize sufficient statistics
  # S1[t] = sum_s E[Y_t^(s)]
  # S2[t,t'] = sum_s E[Y_t^(s) * Y_t'^(s)]
  S1 <- numeric(n_time)
  S2 <- matrix(0, n_time, n_time)
  
  for (s in 1:n_subjects) {
    y_s <- y[s, ]
    mu_s <- mu_matrix[s, ]
    
    obs_idx <- which(!is.na(y_s))
    mis_idx <- which(is.na(y_s))
    
    if (length(mis_idx) == 0) {
      # Complete case
      y_full <- y_s
      S2_s <- tcrossprod(y_s)  # y * y'
      
    } else {
      # Has missing values
      y_obs <- y_s[obs_idx]
      mu_obs <- mu_s[obs_idx]
      mu_mis <- mu_s[mis_idx]
      
      Sigma_obs <- Sigma[obs_idx, obs_idx, drop = FALSE]
      Sigma_mis <- Sigma[mis_idx, mis_idx, drop = FALSE]
      Sigma_cross <- Sigma[mis_idx, obs_idx, drop = FALSE]
      
      # Conditional mean: E[Y_mis | Y_obs]
      Sigma_obs_inv <- tryCatch(
        solve(Sigma_obs),
        error = function(e) {
          solve(Sigma_obs + diag(1e-8, nrow(Sigma_obs)))
        }
      )
      E_mis <- mu_mis + Sigma_cross %*% Sigma_obs_inv %*% (y_obs - mu_obs)
      
      # Conditional covariance: Var[Y_mis | Y_obs]
      Var_mis <- Sigma_mis - Sigma_cross %*% Sigma_obs_inv %*% t(Sigma_cross)
      Var_mis <- 0.5 * (Var_mis + t(Var_mis))
      
      # Impute missing values
      y_full <- y_s
      y_full[mis_idx] <- E_mis
      
      # Second moment matrix
      S2_s <- tcrossprod(y_full)
      
      # Add conditional variance for missing-missing pairs
      S2_s[mis_idx, mis_idx] <- S2_s[mis_idx, mis_idx] + Var_mis
    }
    
    # Accumulate sufficient statistics
    S1 <- S1 + y_full
    S2 <- S2 + S2_s
  }
  
  list(S1 = S1, S2 = S2, n = n_subjects)
}

#' M-step: Update parameters from sufficient statistics
#'
#' @keywords internal
.em_m_step_gau <- function(suff_stats, order, blocks, estimate_mu, 
                          n_subjects, n_time) {
  S1 <- suff_stats$S1
  S2 <- suff_stats$S2
  n <- suff_stats$n
  
  # Update mu (marginal means)
  if (estimate_mu) {
    mu <- S1 / n
  } else {
    mu <- rep(0, n_time)  # Assume centered if not estimating
  }
  
  # Update block effects if present
  # (Simplified - full implementation would need block-specific sufficient stats)
  if (!is.null(blocks)) {
    n_blocks <- length(unique(blocks))
    tau <- numeric(n_blocks)
    tau[1] <- 0
    # TODO: Implement proper block effect estimation
  } else {
    tau <- 0
  }
  
  # Compute centered second moments
  S2_centered <- S2 / n - tcrossprod(mu)
  S2_centered <- 0.5 * (S2_centered + t(S2_centered))
  eps <- 1e-8
  
  # Update phi and sigma based on order
  if (order == 0) {
    phi <- numeric(0)
    sigma <- sqrt(pmax(diag(S2_centered), eps))
    
  } else if (order == 1) {
    phi <- numeric(n_time - 1)
    sigma <- numeric(n_time)
    sigma[1] <- sqrt(max(S2_centered[1, 1], eps))

    if (n_time >= 2) for (t in 2:n_time) {
      # phi[t] = Cov(Y_t, Y_{t-1}) / Var(Y_{t-1})
      denom <- max(S2_centered[t - 1, t - 1], eps)
      phi[t - 1] <- S2_centered[t, t - 1] / denom
      
      # sigma[t]^2 = Var(Y_t) - phi[t]^2 * Var(Y_{t-1})
      sigma2_t <- S2_centered[t, t] - phi[t - 1] * S2_centered[t, t - 1]
      sigma[t] <- sqrt(max(sigma2_t, eps))
    }
    
  } else if (order == 2) {
    # Exact order-2 conditional least-squares updates from expected moments.
    phi <- matrix(0, nrow = 2, ncol = n_time)
    sigma2 <- pmax(diag(S2_centered), eps)

    # Time 2 depends on time 1 only.
    if (n_time >= 2) {
      var1 <- max(S2_centered[1, 1], eps)
      cov21 <- S2_centered[2, 1]
      phi[1, 2] <- cov21 / var1
      sigma2[2] <- max(S2_centered[2, 2] - phi[1, 2] * cov21, eps)
    }

    # Times 3..n depend on the two previous time points.
    if (n_time >= 3) for (t in 3:n_time) {
      G <- matrix(
        c(
          S2_centered[t - 1, t - 1], S2_centered[t - 1, t - 2],
          S2_centered[t - 2, t - 1], S2_centered[t - 2, t - 2]
        ),
        nrow = 2,
        byrow = TRUE
      )
      rhs <- c(S2_centered[t, t - 1], S2_centered[t, t - 2])

      beta <- tryCatch(
        solve(G, rhs),
        error = function(e) solve(G + diag(eps, 2), rhs)
      )

      phi[, t] <- as.numeric(beta)
      sigma2[t] <- max(S2_centered[t, t] - sum(phi[, t] * rhs), eps)
    }

    sigma <- sqrt(sigma2)
  }
  
  list(mu = mu, phi = phi, sigma = sigma, tau = tau)
}

# File: R/lrt_mean_gau.R
# Likelihood ratio tests for the mean structure (Chapter 7)

#' One-sample test for mean structure under antedependence
#'
#' Tests the null hypothesis that the mean vector equals a specified value
#' mu = mu_0 against the alternative mu != mu_0, under an AD(p) covariance
#' structure. This implements Theorem 7.1 of Zimmerman & Núñez-Antón (2009).
#'
#' @param y Numeric matrix with n_subjects rows and n_time columns.
#' @param mu0 Hypothesized mean vector under the null (length n_time).
#' @param p Antedependence order of the covariance structure. This is the same
#'   order parameter named \code{order} in \code{\link{fit_gau}}.
#' @param order Optional alias for \code{p}. Supply only one of \code{p} or
#'   \code{order}.
#' @param use_modified Logical. If TRUE (default), use the modified test statistic
#'   (formula 7.7) for better small-sample approximation.
#'
#' @return A list with class \code{gau_mean_test} containing:
#' \describe{
#'   \item{method}{Inference method used (\code{"lrt"}).}
#'   \item{test_type}{"one-sample"}
#'   \item{mu0}{Hypothesized mean under null}
#'   \item{mu_hat}{MLE of mean (sample mean)}
#'   \item{statistic}{Test statistic value}
#'   \item{statistic_modified}{Modified test statistic (if use_modified = TRUE)}
#'   \item{df}{Degrees of freedom (n_time)}
#'   \item{p_value}{P-value from chi-square distribution}
#'   \item{p_value_modified}{P-value from modified test}
#'   \item{order}{Antedependence order used}
#' }
#'
#' @details
#' The test exploits the AD structure to gain power over tests that don't
#' assume any covariance structure. The likelihood ratio test statistic
#' (Theorem 7.1) is:
#' \deqn{N \sum_{i=1}^{n} [\log RSS_i(\mu_0) - \log RSS_i(\hat{\mu})]}
#'
#' where RSS_i(mu) is the residual sum of squares from the regression of
#' Y_i - mu_i on its p predecessors Y_(i-1) - mu_(i-1), ..., Y_(i-p) - mu_(i-p).
#'
#' The test has n degrees of freedom (one for each component of mu).
#'
#' @references
#' Zimmerman, D.L. and Núñez-Antón, V. (2009). Antedependence Models for
#' Longitudinal Data. Chapman & Hall/CRC. Chapter 7.
#'
#' @seealso \code{\link{test_two_sample_gau}}, \code{\link{test_order_gau}}
#'
#' @examples
#' \donttest{
#' # Simulate data with known mean
#' mu_true <- c(10, 11, 12, 13, 14, 15)
#' y <- simulate_gau(n_subjects = 50, n_time = 6, order = 1, mu = mu_true)
#'
#' # Test if mean is zero
#' test <- test_one_sample_gau(y, mu0 = rep(0, 6), p = 1)
#' print(test)
#'
#' # Test if mean equals true value (should not reject)
#' test2 <- test_one_sample_gau(y, mu0 = mu_true, p = 1)
#' print(test2)
#' }
#'
#' @export
test_one_sample_gau <- function(y, mu0, p = 1L, order = NULL, use_modified = TRUE) {

    if (!is.matrix(y)) y <- as.matrix(y)
    if (anyNA(y)) {
        stop(
            "test_one_sample_gau currently supports complete data only. Missing-data AD likelihood-ratio tests are not implemented yet.",
            call. = FALSE
        )
    }
    if (any(!is.finite(y))) stop("y must contain finite values")

    n <- nrow(y)  # N in the book
    n_time <- ncol(y)  # n in the book

    if (!is.null(order)) {
        if (!missing(p)) {
            stop("Specify only one of 'p' or 'order'.", call. = FALSE)
        }
        p <- order
    }
    p <- as.integer(p)
    if (p < 0 || p >= n_time) stop("p must be in [0, n_time - 1)")

    mu0 <- as.numeric(mu0)
    if (length(mu0) != n_time) stop("mu0 must have length n_time")

    # MLE of mu is the sample mean
    mu_hat <- colMeans(y)

    # Compute RSS under null and alternative
    rss_null <- .rss_vector_gau(y, p, mu = mu0)
    rss_alt <- .rss_vector_gau(y, p, mu = mu_hat)

    # Standard LRT statistic (Theorem 7.1)
    # N * sum [log RSS_i(mu0) - log RSS_i(mu_hat)]
    log_ratios <- log(rss_null) - log(rss_alt)
    statistic <- n * sum(log_ratios)

    # Degrees of freedom = n_time
    df <- n_time

    # P-value
    p_value <- stats::pchisq(statistic, df = df, lower.tail = FALSE)

    # Modified test statistic (formula 7.7)
    statistic_modified <- NULL
    p_value_modified <- NULL

    if (use_modified) {
        # Modified criterion uses psi weights
        numerator <- 0
        denominator <- 0

        for (i in 1:n_time) {
            p_i <- min(p, i - 1)
            if (rss_null[i] > 0 && rss_alt[i] > 0) {
                log_ratio <- log(rss_null[i]) - log(rss_alt[i])
                numerator <- numerator + log_ratio

                psi_val <- .psi_kenward(1, n - 1 - p_i)
                denominator <- denominator + psi_val
            }
        }

        if (denominator > 0) {
            # Eq. 7.7: leading factor is n_time (number of time points), not N_subjects.
            # Analogous to Eq. 6.9 where Σ diff_i = n_time for the mean test (each diff = 1).
            statistic_modified <- n_time * numerator / denominator
            p_value_modified <- stats::pchisq(statistic_modified, df = df, lower.tail = FALSE)
        }
    }

    out <- list(
        method = "lrt",
        test_type = "one-sample",
        mu0 = mu0,
        mu_hat = mu_hat,
        statistic = statistic,
        statistic_modified = statistic_modified,
        df = df,
        p_value = p_value,
        p_value_modified = p_value_modified,
        order = p,
        n_subjects = n,
        n_time = n_time
    )

    class(out) <- "gau_mean_test"
    out
}


#' Two-sample test for equality of mean profiles under antedependence
#'
#' Tests the null hypothesis that two groups have equal mean profiles
#' mu_1 = mu_2 against the alternative mu_1 != mu_2, assuming a common
#' AD(p) covariance structure. This implements Theorem 7.3 of
#' Zimmerman & Núñez-Antón (2009).
#'
#' @param y Numeric matrix with n_subjects rows and n_time columns.
#' @param blocks Integer vector of length n_subjects indicating group membership
#'   (must contain exactly two unique values, typically 1 and 2).
#' @param p Antedependence order of the common covariance structure. This is the
#'   same order parameter named \code{order} in \code{\link{fit_gau}}.
#' @param order Optional alias for \code{p}. Supply only one of \code{p} or
#'   \code{order}.
#' @param use_modified Logical. If TRUE (default), use modified test statistic.
#'
#' @return A list with class \code{gau_mean_test} containing:
#' \describe{
#'   \item{method}{Inference method used (\code{"lrt"}).}
#'   \item{test_type}{"two-sample"}
#'   \item{mu1_hat}{Estimated mean for group 1}
#'   \item{mu2_hat}{Estimated mean for group 2}
#'   \item{mu_pooled}{Pooled mean estimate under H0}
#'   \item{statistic}{Test statistic value}
#'   \item{statistic_modified}{Modified test statistic}
#'   \item{df}{Degrees of freedom (n_time)}
#'   \item{p_value}{P-value from chi-square distribution}
#'   \item{p_value_modified}{P-value from modified test}
#'   \item{order}{Antedependence order used}
#' }
#'
#' @details
#' This test is also known as a "profile comparison" test. The likelihood
#' ratio test statistic (Theorem 7.3) compares the pooled RSS (under H0:
#' common mean) to the sum of within-group RSS (under H1: separate means):
#'
#' \deqn{N \sum_{i=1}^{n} [\log RSS_i(\mu) - \log RSS_i(\mu_1, \mu_2)]}
#'
#' where RSS_i(mu) uses a common mean and RSS_i(mu_1, mu_2) uses group-specific
#' means.
#'
#' @references
#' Zimmerman, D.L. and Núñez-Antón, V. (2009). Antedependence Models for
#' Longitudinal Data. Chapman & Hall/CRC. Chapter 7.
#'
#' @examples
#' \donttest{
#' # Simulate data from two groups with different means
#' n1 <- 30
#' n2 <- 35
#' y1 <- simulate_gau(n1, n_time = 6, order = 1, mu = rep(10, 6))
#' y2 <- simulate_gau(n2, n_time = 6, order = 1, mu = rep(12, 6))
#' y <- rbind(y1, y2)
#' blocks <- c(rep(1, n1), rep(2, n2))
#'
#' # Test equality of profiles
#' test <- test_two_sample_gau(y, blocks, p = 1)
#' print(test)
#' }
#'
#' @export
test_two_sample_gau <- function(y, blocks, p = 1L, order = NULL, use_modified = TRUE) {

    if (!is.matrix(y)) y <- as.matrix(y)
    if (anyNA(y)) {
        stop(
            "test_two_sample_gau currently supports complete data only. Missing-data AD likelihood-ratio tests are not implemented yet.",
            call. = FALSE
        )
    }
    if (any(!is.finite(y))) stop("y must contain finite values")

    n <- nrow(y)
    n_time <- ncol(y)

    blocks <- as.integer(blocks)
    if (length(blocks) != n) stop("blocks must have length nrow(y)")

    unique_blocks <- sort(unique(blocks))
    if (length(unique_blocks) != 2) {
        stop("blocks must contain exactly two unique values for two-sample test")
    }

    # Relabel to 1 and 2
    blocks <- ifelse(blocks == unique_blocks[1], 1L, 2L)

    g1 <- which(blocks == 1)
    g2 <- which(blocks == 2)
    n1 <- length(g1)
    n2 <- length(g2)

    if (!is.null(order)) {
        if (!missing(p)) {
            stop("Specify only one of 'p' or 'order'.", call. = FALSE)
        }
        p <- order
    }
    p <- as.integer(p)
    if (p < 0 || p >= n_time) stop("p must be in [0, n_time - 1)")

    # Group means
    mu1_hat <- colMeans(y[g1, , drop = FALSE])
    mu2_hat <- colMeans(y[g2, , drop = FALSE])

    # Pooled mean (under H0)
    mu_pooled <- colMeans(y)

    # RSS under null (common mean)
    rss_null <- numeric(n_time)
    for (i in 1:n_time) {
        rss_null[i] <- .rss_gau(y, i, p, mu = mu_pooled)
    }

    # RSS under alternative (group-specific means)
    # Sum of within-group RSS
    rss_alt <- numeric(n_time)
    for (i in 1:n_time) {
        rss1 <- .rss_gau(y[g1, , drop = FALSE], i, p, mu = mu1_hat)
        rss2 <- .rss_gau(y[g2, , drop = FALSE], i, p, mu = mu2_hat)
        rss_alt[i] <- rss1 + rss2
    }

    # Standard LRT statistic (Theorem 7.3)
    log_ratios <- log(rss_null) - log(rss_alt)
    statistic <- n * sum(log_ratios)

    # Degrees of freedom = n_time
    df <- n_time

    # P-value
    p_value <- stats::pchisq(statistic, df = df, lower.tail = FALSE)

    # Modified test statistic (formula 7.11)
    statistic_modified <- NULL
    p_value_modified <- NULL

    if (use_modified) {
        numerator <- 0
        denominator <- 0

        for (i in 1:n_time) {
            p_i <- min(p, i - 1)
            if (rss_null[i] > 0 && rss_alt[i] > 0) {
                log_ratio <- log(rss_null[i]) - log(rss_alt[i])
                numerator <- numerator + log_ratio

                psi_val <- .psi_kenward(1, n - 2 - p_i)
                denominator <- denominator + psi_val
            }
        }

        if (denominator > 0) {
            # Eq. 7.11: leading factor is n_time (number of time points), not N_subjects.
            # Analogous to Eq. 6.9 where Σ diff_i = n_time for the mean test (each diff = 1).
            statistic_modified <- n_time * numerator / denominator
            p_value_modified <- stats::pchisq(statistic_modified, df = df, lower.tail = FALSE)
        }
    }

    out <- list(
        method = "lrt",
        test_type = "two-sample",
        mu1_hat = mu1_hat,
        mu2_hat = mu2_hat,
        mu_pooled = mu_pooled,
        statistic = statistic,
        statistic_modified = statistic_modified,
        df = df,
        p_value = p_value,
        p_value_modified = p_value_modified,
        order = p,
        n_subjects = n,
        n1 = n1,
        n2 = n2,
        n_time = n_time
    )

    class(out) <- "gau_mean_test"
    out
}


#' Test linear hypotheses on the mean under antedependence
#'
#' Tests the null hypothesis C * mu = c for a specified contrast matrix C
#' and vector c, under an AD(p) covariance structure. This implements
#' Theorem 7.2 of Zimmerman & Núñez-Antón (2009).
#'
#' @param y Numeric matrix with n_subjects rows and n_time columns.
#' @param C Contrast matrix with c rows and n_time columns, where c is the
#'   number of contrasts being tested. Rows must be linearly independent.
#' @param c Right-hand side vector of length equal to nrow(C). Default is
#'   a vector of zeros.
#' @param p Antedependence order of the covariance structure. This is the same
#'   order parameter named \code{order} in \code{\link{fit_gau}}.
#'
#' @return A list with class \code{gau_contrast_test} containing:
#' \describe{
#'   \item{method}{Inference method used (\code{"wald"}).}
#'   \item{C}{Contrast matrix}
#'   \item{c}{Right-hand side vector}
#'   \item{mu_hat}{Estimated mean vector}
#'   \item{contrast_est}{Estimated value of C * mu}
#'   \item{statistic}{Wald test statistic}
#'   \item{df}{Degrees of freedom (number of contrasts)}
#'   \item{p_value}{P-value from chi-square distribution}
#' }
#'
#' @details
#' The Wald test statistic (Theorem 7.2) is:
#' \deqn{(C\bar{Y} - c)^T (C \hat{\Sigma} C^T)^{-1} (C\bar{Y} - c)}
#'
#' where \eqn{\hat{\Sigma}} is the REML estimator of the covariance matrix
#' under the AD(p) model.
#'
#' Common examples include:
#' \itemize{
#'   \item Testing if mean is constant: C is the first-difference matrix
#'   \item Testing for linear trend: C tests deviations from linearity
#' }
#'
#' @references
#' Zimmerman, D.L. and Núñez-Antón, V. (2009). Antedependence Models for
#' Longitudinal Data. Chapman & Hall/CRC. Chapter 7.
#'
#' @examples
#' \donttest{
#' y <- simulate_gau(n_subjects = 50, n_time = 5, order = 1)
#'
#' # Test if mean is constant (all differences = 0)
#' # C is 4x5 matrix of first differences
#' C <- matrix(0, nrow = 4, ncol = 5)
#' for (i in 1:4) {
#'   C[i, i] <- 1
#'   C[i, i+1] <- -1
#' }
#' test <- test_contrast_gau(y, C = C, p = 1)
#' print(test)
#' }
#'
#' @export
test_contrast_gau <- function(y, C, c = NULL, p = 1L) {

    if (!is.matrix(y)) y <- as.matrix(y)
    if (!is.matrix(C)) C <- as.matrix(C)
    if (anyNA(y)) {
        stop(
            "test_contrast_gau currently supports complete data only. Missing-data AD likelihood-ratio tests are not implemented yet.",
            call. = FALSE
        )
    }
    if (any(!is.finite(y))) stop("y must contain finite values")

    n <- nrow(y)
    n_time <- ncol(y)

    if (ncol(C) != n_time) {
        stop("C must have n_time columns")
    }

    n_contrasts <- nrow(C)

    if (is.null(c)) {
        c <- rep(0, n_contrasts)
    }
    c <- as.numeric(c)
    if (length(c) != n_contrasts) {
        stop("c must have length equal to nrow(C)")
    }

    p <- as.integer(p)

    # MLE of mean
    mu_hat <- colMeans(y)

    # Estimate covariance matrix under AD(p)
    # For this we need the phi and sigma estimates
    fit <- fit_gau(y, order = p, estimate_mu = FALSE)

    # Construct the AD covariance matrix from fit
    Sigma_hat <- .construct_gau_covariance(fit, n_time)

    # Wald test
    contrast_est <- as.numeric(C %*% mu_hat)
    diff <- contrast_est - c

    # (C Sigma C')^{-1}
    CSigmaC <- C %*% Sigma_hat %*% t(C)
    CSigmaC_inv <- tryCatch(
        solve(CSigmaC),
        error = function(e) {
            warning("Singular contrast covariance matrix")
            NULL
        }
    )

    if (is.null(CSigmaC_inv)) {
        statistic <- NA
        p_value <- NA
    } else {
        # Wald statistic: N * diff' (C Sigma C')^{-1} diff
        statistic <- n * as.numeric(t(diff) %*% CSigmaC_inv %*% diff)
        p_value <- stats::pchisq(statistic, df = n_contrasts, lower.tail = FALSE)
    }

    out <- list(
        method = "wald",
        C = C,
        c = c,
        mu_hat = mu_hat,
        contrast_est = contrast_est,
        statistic = statistic,
        df = n_contrasts,
        p_value = p_value,
        order = p
    )

    class(out) <- "gau_contrast_test"
    out
}


#' Construct AD covariance matrix from fitted model
#'
#' @param fit Fitted AD model from fit_gau().
#' @param n_time Number of time points.
#'
#' @return Covariance matrix.
#'
#' @keywords internal
.construct_gau_covariance <- function(fit, n_time) {
    ord <- fit$settings$order
    sigma <- fit$sigma
    phi <- fit$phi

    # Innovation variances (delta_i^2 in the book)
    delta2 <- sigma^2

    # Build the modified Cholesky factor T and D
    # Sigma^{-1} = T' D^{-1} T
    # where T is unit lower triangular and D is diagonal

    T_mat <- diag(n_time)
    D_vec <- delta2

    if (ord >= 1 && !is.null(phi)) {
        if (is.matrix(phi)) {
            # Order 2
            for (i in 2:n_time) {
                T_mat[i, i-1] <- -phi[1, i]
                if (i >= 3 && ord >= 2) {
                    T_mat[i, i-2] <- -phi[2, i]
                }
            }
        } else {
            # Order 1
            for (i in 2:n_time) {
                T_mat[i, i-1] <- -phi[i]
            }
        }
    }

    # Sigma = T^{-1} D T^{-T}
    T_inv <- solve(T_mat)
    Sigma <- T_inv %*% diag(D_vec) %*% t(T_inv)

    Sigma
}


#' Print method for AD mean test
#'
#' @param x Object of class \code{gau_mean_test}.
#' @param ... Unused.
#' @return Invisibly returns \code{x}.
#' @export
print.gau_mean_test <- function(x, ...) {
    if (x$test_type == "one-sample") {
        cat("One-Sample Test for Mean Under AD(", x$order, ")\n", sep = "")
        cat("================================================\n\n")
        cat("H0: mu = mu_0\n")
        cat("H1: mu != mu_0\n\n")
    } else {
        cat("Two-Sample Test for Mean Profiles Under AD(", x$order, ")\n", sep = "")
        cat("======================================================\n\n")
        cat("H0: mu_1 = mu_2\n")
        cat("H1: mu_1 != mu_2\n\n")
        cat("Group sizes: n1 =", x$n1, ", n2 =", x$n2, "\n")
    }

    cat("Sample size:", x$n_subjects, "subjects,", x$n_time, "time points\n\n")

    cat("Standard LRT:\n")
    cat("  Statistic:", round(x$statistic, 4), "\n")
    cat("  df:", x$df, "\n")
    cat("  p-value:", format.pval(x$p_value, digits = 4), "\n")

    if (!is.null(x$statistic_modified)) {
        cat("\nModified LRT:\n")
        cat("  Statistic:", round(x$statistic_modified, 4), "\n")
        cat("  p-value:", format.pval(x$p_value_modified, digits = 4), "\n")
    }

    invisible(x)
}


#' Print method for AD contrast test
#'
#' @param x Object of class \code{gau_contrast_test}.
#' @param ... Unused.
#' @return Invisibly returns \code{x}.
#' @export
print.gau_contrast_test <- function(x, ...) {
    cat("Wald Test for Linear Contrasts Under AD(", x$order, ")\n", sep = "")
    cat("===================================================\n\n")

    cat("H0: C * mu = c\n")
    cat("Number of contrasts:", x$df, "\n\n")

    cat("Wald statistic:", round(x$statistic, 4), "\n")
    cat("df:", x$df, "\n")
    cat("p-value:", format.pval(x$p_value, digits = 4), "\n")

    invisible(x)
}

test_that("test_one_sample_gau accepts order alias for p", {
  skip_on_cran()
  set.seed(1001)
  y <- simulate_gau(n_subjects = 40, n_time = 5, order = 1, phi = 0.3)
  mu0 <- rep(0, ncol(y))

  out_p <- test_one_sample_gau(y, mu0 = mu0, p = 1, use_modified = FALSE)
  out_order <- test_one_sample_gau(y, mu0 = mu0, order = 1, use_modified = FALSE)

  expect_equal(out_order$order, out_p$order)
  expect_equal(out_order$statistic, out_p$statistic)
  expect_equal(out_order$p_value, out_p$p_value)
})

test_that("test_two_sample_gau accepts order alias for p", {
  skip_on_cran()
  set.seed(1002)
  y <- simulate_gau(n_subjects = 50, n_time = 5, order = 1, phi = 0.25)
  blocks <- c(rep(1, 25), rep(2, 25))

  out_p <- test_two_sample_gau(y, blocks = blocks, p = 1, use_modified = FALSE)
  out_order <- test_two_sample_gau(y, blocks = blocks, order = 1, use_modified = FALSE)

  expect_equal(out_order$order, out_p$order)
  expect_equal(out_order$statistic, out_p$statistic)
  expect_equal(out_order$p_value, out_p$p_value)
})

test_that("test_order_gau accepts order_null and order_alt aliases", {
  skip_on_cran()
  set.seed(1004)
  y <- simulate_gau(n_subjects = 40, n_time = 5, order = 1, phi = 0.3)

  out_pq       <- test_order_gau(y, p = 0, q = 1, use_modified = FALSE)
  out_aliases  <- test_order_gau(y, order_null = 0, order_alt = 1,
                                 use_modified = FALSE)

  expect_equal(out_aliases$statistic, out_pq$statistic)
  expect_equal(out_aliases$p_value,   out_pq$p_value)
  expect_equal(out_aliases$order_null, 0L)
  expect_equal(out_aliases$order_alt,  1L)
})

test_that("test_*_gau p and order cannot both be supplied", {
  skip_on_cran()
  set.seed(1003)
  y <- simulate_gau(n_subjects = 30, n_time = 4, order = 1, phi = 0.2)
  blocks <- c(rep(1, 15), rep(2, 15))

  expect_error(
    test_one_sample_gau(y, mu0 = rep(0, ncol(y)), p = 1, order = 1),
    "Specify only one of 'p' or 'order'"
  )
  expect_error(
    test_two_sample_gau(y, blocks = blocks, p = 1, order = 1),
    "Specify only one of 'p' or 'order'"
  )
})

test_that("CAT missing-data inference support boundaries are explicit", {
  skip_on_cran()
  y_miss <- matrix(
    c(1, 2, 1,
      2, 1, NA,
      1, 2, 2,
      2, 1, 1),
    nrow = 4,
    byrow = TRUE
  )

  fit_missing_ci <- structure(
    list(
      settings = list(
        order = 1,
        na_action = "marginalize",
        na_action_effective = "marginalize"
      ),
      cell_counts = NULL
    ),
    class = "cat_fit"
  )
  expect_error(ci_cat(fit_missing_ci), "complete-data fits only")

  expect_s3_class(test_order_cat(y_miss, order_null = 0, order_alt = 1), "cat_lrt")
  expect_s3_class(
    test_homogeneity_cat(y_miss, blocks = c(1, 1, 2, 2), order = 1),
    "cat_lrt"
  )
  expect_true(is.list(run_order_tests_cat(y_miss)))

  expect_error(test_timeinvariance_cat(y_miss, order = 1), "complete data only")
  expect_error(test_stationarity_cat(y_miss, order = 1), "complete data only")
  expect_error(run_stationarity_tests_cat(y_miss, order = 1), "complete data only")

  fit_null <- structure(
    list(
      settings = list(order = 0, homogeneous = TRUE, n_blocks = 1,
                      na_action_effective = "marginalize"),
      cell_counts = NULL,
      log_l = -10,
      n_params = 2,
      aic = 24,
      bic = 25
    ),
    class = "cat_fit"
  )
  fit_alt <- structure(
    list(
      settings = list(order = 1, homogeneous = TRUE, n_blocks = 1,
                      na_action_effective = "marginalize"),
      cell_counts = NULL,
      log_l = -9,
      n_params = 4,
      aic = 26,
      bic = 27
    ),
    class = "cat_fit"
  )
  expect_s3_class(
    test_order_cat(fit_null = fit_null, fit_alt = fit_alt),
    "cat_lrt"
  )
})

test_that("INAD missing-data LRT support is enabled while CI remains complete-data only", {
  skip_on_cran()
  set.seed(2026)
  y_miss <- simulate_inad(
    n_subjects = 10,
    n_time = 3,
    order = 1,
    thinning = "binom",
    innovation = "pois",
    alpha = 0.35,
    theta = 2.5
  )
  y_miss[sample(length(y_miss), 4)] <- NA
  blocks <- rep(1:2, each = 5)

  fit_stub <- list(
    settings = list(order = 1, thinning = "binom", innovation = "pois"),
    theta = rep(1, ncol(y_miss)),
    log_l = -10
  )

  expect_error(ci_inad(y_miss, fit_stub, blocks = blocks), "complete data only")
  ord_test <- suppressWarnings(
    test_order_inad(y_miss, order_null = 0, order_alt = 1, blocks = blocks)
  )
  expect_s3_class(ord_test, "test_order_inad")
  expect_s3_class(
    test_stationarity_inad(y_miss, order = 1, blocks = blocks),
    "test_stationarity_inad"
  )
  expect_s3_class(
    test_homogeneity_inad(y_miss, blocks = blocks, order = 1),
    "test_homogeneity_inad"
  )
  stat_tests <- suppressWarnings(
    run_stationarity_tests_inad(y_miss, order = 1, blocks = blocks, verbose = FALSE)
  )
  expect_s3_class(stat_tests, "stationarity_tests_inad")
  expect_s3_class(
    run_homogeneity_tests_inad(y_miss, blocks = blocks, order = 1),
    "homogeneity_tests_inad"
  )
})

test_that("AD likelihood-ratio utilities reject missing-data inference clearly", {
  skip_on_cran()
  y_miss <- matrix(
    c(1.0, 2.0, 3.0,
      2.0, 1.0, 0.0,
      0.5, NA, 1.5,
      3.0, 2.0, 1.0),
    nrow = 4,
    byrow = TRUE
  )
  blocks <- c(1, 1, 2, 2)

  expect_error(test_order_gau(y_miss, p = 0, q = 1), "complete data only")
  expect_error(test_homogeneity_gau(y_miss, blocks = blocks, p = 1), "monotone dropout")
  expect_error(
    test_one_sample_gau(y_miss, mu0 = c(0, 0, 0), p = 1),
    "complete data only"
  )
  expect_error(test_two_sample_gau(y_miss, blocks = blocks, p = 1), "complete data only")
  expect_error(
    test_contrast_gau(y_miss, C = matrix(c(1, -1, 0), nrow = 1), p = 1),
    "complete data only"
  )
  expect_error(test_stationarity_gau(y_miss, order = 1), "complete data only")
  expect_error(run_stationarity_tests_gau(y_miss, order = 1), "complete data only")
})

# File: R/test_homogeneity_inad.R
# Likelihood ratio tests for homogeneity across groups in INAD models
# Based on Section 3.7 of Li & Zimmerman (2026) Biostatistics

#' Likelihood ratio test for homogeneity across groups (INAD data)
#'
#' Tests hypotheses about parameter equality across treatment or grouping
#' factors in integer-valued antedependence models. Implements the homogeneity
#' testing framework from Section 3.7 of Li & Zimmerman (2026).
#'
#' @param y Integer matrix with n_subjects rows and n_time columns.
#' @param blocks Integer vector of length n_subjects specifying group membership.
#' @param order Antedependence order (0, 1, or 2).
#' @param thinning Thinning operator: "binom", "pois", or "nbinom".
#' @param innovation Innovation distribution: "pois", "bell", or "nbinom".
#' @param test Type of homogeneity test:
#'   \itemize{
#'     \item \code{"all"}: Tests M1 (pooled) vs M3 (fully heterogeneous)
#'     \item \code{"mean"}: Tests M1 (pooled) vs M2 (shared dependence, different means)
#'     \item \code{"dependence"}: Tests M2 (INADFE) vs M3 (fully heterogeneous)
#'   }
#' @param fit_pooled Optional pre-computed pooled fit (M1).
#' @param fit_inadfe Optional pre-computed INADFE fit (M2).
#' @param fit_hetero Optional pre-computed heterogeneous fit (M3).
#' @param ... Additional arguments passed to \code{fit_inad}.
#'
#' @return A list with class \code{"test_homogeneity_inad"} containing:
#' \describe{
#'   \item{method}{Inference method used (\code{"lrt"}).}
#'   \item{statistic}{Test statistic value}
#'   \item{lrt_stat}{Likelihood ratio test statistic}
#'   \item{df}{Degrees of freedom}
#'   \item{p_value}{P-value from chi-square distribution}
#'   \item{test}{Type of test performed}
#'   \item{fit_null}{Fitted model under H0}
#'   \item{fit_alt}{Fitted model under H1}
#'   \item{bic_null}{BIC under H0}
#'   \item{bic_alt}{BIC under H1}
#'   \item{bic_selected}{Which model BIC prefers}
#'   \item{table}{Summary data frame}
#' }
#'
#' @details
#' The function supports three nested model comparisons as described in the paper:
#'
#' \strong{M1 (Pooled)}: All parameters are common across groups. This corresponds
#' to fitting \code{fit_inad(y, blocks = NULL)}.
#'
#' \strong{M2 (INADFE)}: The thinning parameters \eqn{\alpha} are shared across
#' groups, but innovation means differ via block effects \eqn{\tau}. This is the
#' standard INADFE model fitted via \code{fit_inad(y, blocks = blocks)}.
#'
#' \strong{M3 (Fully Heterogeneous)}: Both \eqn{\alpha} and \eqn{\theta}
#' parameters can differ across groups. This is fitted by running separate
#' \code{fit_inad} calls for each group.
#'
#' The three test types correspond to:
#' \itemize{
#'   \item \code{"all"}: H0: M1 vs H1: M3 (complete homogeneity vs complete heterogeneity)
#'   \item \code{"mean"}: H0: M1 vs H1: M2 (test for group differences in means only)
#'   \item \code{"dependence"}: H0: M2 vs H1: M3 (test for group differences in dependence)
#' }
#'
#' Degrees of freedom are computed as the difference in free parameters between
#' the null and alternative models.
#'
#' @references
#' Li, C. and Zimmerman, D.L. (2026). Integer-valued antedependence models for
#' longitudinal count data. \emph{Biostatistics}. Section 3.7.
#'
#' @seealso \code{\link{fit_inad}}, \code{\link{test_order_inad}},
#'   \code{\link{test_stationarity_inad}}
#'
#' @examples
#' \donttest{
#' data("bolus_inad")
#' y <- bolus_inad$y
#' blocks <- bolus_inad$blocks
#'
#' # Test for any group differences (M1 vs M3)
#' test_all <- test_homogeneity_inad(y, blocks, order = 1,
#'                                   thinning = "nbinom", innovation = "bell",
#'                                   test = "all")
#' print(test_all)
#'
#' # Test only for mean differences (M1 vs M2)
#' test_mean <- test_homogeneity_inad(y, blocks, order = 1,
#'                                    thinning = "nbinom", innovation = "bell",
#'                                    test = "mean")
#' print(test_mean)
#'
#' # Test for dependence differences given different means (M2 vs M3)
#' test_dep <- test_homogeneity_inad(y, blocks, order = 1,
#'                                   thinning = "nbinom", innovation = "bell",
#'                                   test = "dependence")
#' print(test_dep)
#' }
#'
#' @importFrom stats pchisq
#' @export
test_homogeneity_inad <- function(y, blocks, order = 1,
                                  thinning = "binom", innovation = "pois",
                                  test = c("all", "mean", "dependence"),
                                  fit_pooled = NULL, fit_inadfe = NULL,
                                  fit_hetero = NULL, ...) {
  
  # Validate inputs
  if (!is.matrix(y)) y <- as.matrix(y)
  y_obs <- y[!is.na(y)]
  if (any(y_obs < 0) || any(y_obs != floor(y_obs))) {
    stop("y must contain non-negative integers")
  }
  
  n_subjects <- nrow(y)
  n_time <- ncol(y)
  
  # Validate blocks
  if (missing(blocks) || is.null(blocks)) {
    stop("blocks must be provided for homogeneity testing")
  }
  blocks <- as.integer(blocks)
  if (length(blocks) != n_subjects) {
    stop("blocks must have length equal to nrow(y)")
  }
  
  unique_blocks <- sort(unique(blocks))
  n_blocks <- length(unique_blocks)
  
  if (n_blocks < 2) {
    stop("Need at least 2 groups for homogeneity testing")
  }
  
  # Normalize block labels to 1, 2, ..., G
  block_map <- setNames(seq_along(unique_blocks), unique_blocks)
  blocks_norm <- as.integer(block_map[as.character(blocks)])
  
  # Validate other arguments
  test <- match.arg(test)
  thinning <- match.arg(thinning, c("binom", "pois", "nbinom"))
  innovation <- match.arg(innovation, c("pois", "bell", "nbinom"))
  order <- as.integer(order)
  if (!order %in% c(0L, 1L, 2L)) stop("order must be 0, 1, or 2")
  fit_args <- list(...)
  if (anyNA(y) && is.null(fit_args$na_action)) {
    fit_args$na_action <- "marginalize"
  }
  
  # Group sizes
  group_sizes <- table(blocks_norm)
  group_indices <- lapply(1:n_blocks, function(g) which(blocks_norm == g))
  
  # Fit required models
  # M1: Pooled model (no block effects)
  if (is.null(fit_pooled) && test %in% c("all", "mean")) {
    fit_pooled <- do.call(
      fit_inad,
      c(
        list(y = y, order = order, thinning = thinning, innovation = innovation, blocks = NULL),
        fit_args
      )
    )
  }
  
  # M2: INADFE model (shared dependence, different means via tau)
  if (is.null(fit_inadfe) && test %in% c("mean", "dependence")) {
    fit_inadfe <- do.call(
      fit_inad,
      c(
        list(y = y, order = order, thinning = thinning, innovation = innovation, blocks = blocks_norm),
        fit_args
      )
    )
  }
  
  # M3: Fully heterogeneous (separate fits per group)
  if (is.null(fit_hetero) && test %in% c("all", "dependence")) {
    fit_hetero <- do.call(
      .fit_inad_heterogeneous,
      c(
        list(
          y = y,
          blocks = blocks_norm,
          n_blocks = n_blocks,
          group_indices = group_indices,
          order = order,
          thinning = thinning,
          innovation = innovation
        ),
        fit_args
      )
    )
  }
  
  # Determine null and alternative models based on test type
  if (test == "all") {
    fit_null <- fit_pooled
    fit_alt <- fit_hetero
    null_name <- "M1 (Pooled)"
    alt_name <- "M3 (Heterogeneous)"
  } else if (test == "mean") {
    fit_null <- fit_pooled
    fit_alt <- fit_inadfe
    null_name <- "M1 (Pooled)"
    alt_name <- "M2 (INADFE)"
  } else {  # test == "dependence"
    fit_null <- fit_inadfe
    fit_alt <- fit_hetero
    null_name <- "M2 (INADFE)"
    alt_name <- "M3 (Heterogeneous)"
  }
  
  # Compute LRT statistic
  logL_null <- fit_null$log_l
  logL_alt <- fit_alt$log_l
  lrt_stat <- 2 * (logL_alt - logL_null)
  
  if (lrt_stat < 0) {
    warning("LRT statistic is negative; setting to 0")
    lrt_stat <- 0
  }
  
  # Count parameters
  n_params_null <- .count_params_homogeneity(fit_null, test, n_blocks, 
                                              order, n_time, thinning, innovation,
                                              is_null = TRUE)
  n_params_alt <- .count_params_homogeneity(fit_alt, test, n_blocks, 
                                             order, n_time, thinning, innovation,
                                             is_null = FALSE)
  
  df <- n_params_alt - n_params_null
  
  if (df <= 0) {
    warning("Degrees of freedom <= 0; p-value may not be meaningful")
    df <- max(1, df)
  }
  
  # P-value from chi-square distribution
  p_value <- pchisq(lrt_stat, df = df, lower.tail = FALSE)
  
  # Compute BIC
  bic_null <- -2 * logL_null + n_params_null * log(n_subjects)
  bic_alt <- -2 * logL_alt + n_params_alt * log(n_subjects)
  bic_selected <- if (bic_null <= bic_alt) "null" else "alt"
  
  # Summary table
  table_df <- data.frame(
    model = c(null_name, alt_name),
    logLik = c(logL_null, logL_alt),
    n_params = c(n_params_null, n_params_alt),
    BIC = c(bic_null, bic_alt),
    stringsAsFactors = FALSE
  )
  
  # Assemble output
  result <- list(
    method = "lrt",
    statistic = lrt_stat,
    lrt_stat = lrt_stat,
    df = df,
    p_value = p_value,
    test = test,
    fit_null = fit_null,
    fit_alt = fit_alt,
    bic_null = bic_null,
    bic_alt = bic_alt,
    bic_selected = bic_selected,
    table = table_df,
    settings = list(
      order = order,
      thinning = thinning,
      innovation = innovation,
      n_subjects = n_subjects,
      n_time = n_time,
      n_blocks = n_blocks,
      group_sizes = as.integer(group_sizes)
    )
  )
  
  class(result) <- "test_homogeneity_inad"
  result
}


#' Fit fully heterogeneous INAD model
#'
#' Internal function that fits separate INAD models for each group and
#' combines the log-likelihoods.
#'
#' @param y Data matrix
#' @param blocks Normalized block vector
#' @param n_blocks Number of groups
#' @param group_indices List of subject indices per group
#' @param order AD order
#' @param thinning Thinning type
#' @param innovation Innovation type
#' @param ... Additional arguments passed to fit_inad
#'
#' @return A list mimicking fit_inad output with combined log-likelihood
#'
#' @keywords internal
.fit_inad_heterogeneous <- function(y, blocks, n_blocks, group_indices,
                                     order, thinning, innovation, ...) {
  
  fits <- vector("list", n_blocks)
  total_log_l <- 0
  
  for (g in seq_len(n_blocks)) {
    y_g <- y[group_indices[[g]], , drop = FALSE]
    fits[[g]] <- fit_inad(y_g, order = order, thinning = thinning,
                          innovation = innovation, blocks = NULL, ...)
    total_log_l <- total_log_l + fits[[g]]$log_l
  }
  
  names(fits) <- paste0("group_", seq_len(n_blocks))
  
  # Return a list with combined info
  list(
    log_l = total_log_l,
    fits_by_group = fits,
    settings = list(
      order = order,
      thinning = thinning,
      innovation = innovation,
      n_blocks = n_blocks,
      heterogeneous = TRUE
    )
  )
}


#' Count parameters for homogeneity test
#'
#' @param fit Fitted model
#' @param test Test type
#' @param n_blocks Number of groups
#' @param order AD order
#' @param n_time Number of time points
#' @param thinning Thinning type
#' @param innovation Innovation type
#' @param is_null Whether this is the null model
#'
#' @return Number of free parameters
#'
#' @keywords internal
.count_params_homogeneity <- function(fit, test, n_blocks, order, n_time,
                                       thinning, innovation, is_null) {

  # Base parameter counts (per group, excluding NB innovation size)
  n_alpha <- switch(as.character(order),
                    "0" = 0,
                    "1" = n_time - 1,
                    "2" = 2 * (n_time - 2))
  n_theta <- n_time

  # NB innovation size params: count directly from the fit so this matches
  # what fit_inad() actually produced (length 1 or n_time per group). For
  # heterogeneous (M3) fits, sum across per-group fits.
  count_nb <- function(f) {
    if (innovation != "nbinom") return(0L)
    if (!is.null(f$nb_inno_size)) return(length(f$nb_inno_size))
    if (!is.null(f$fits_by_group)) {
      return(sum(vapply(f$fits_by_group,
                        function(g) length(g$nb_inno_size), integer(1))))
    }
    n_time  # conservative fallback
  }

  n_nb_inno <- count_nb(fit)
  k_single  <- n_alpha + n_theta + n_nb_inno

  if (test == "all") {
    if (is_null) {
      # M1: Pooled - single set of parameters (one shared NB size vector)
      return(k_single)
    } else {
      # M3: Heterogeneous - G sets of alpha+theta, NB sizes already counted
      # across groups via count_nb(fit)
      return(n_blocks * (n_alpha + n_theta) + n_nb_inno)
    }
  } else if (test == "mean") {
    if (is_null) {
      # M1: Pooled
      return(k_single)
    } else {
      # M2: INADFE - shared alpha, theta, and NB size; group-specific tau
      return(n_alpha + n_theta + (n_blocks - 1) + n_nb_inno)
    }
  } else {  # test == "dependence"
    if (is_null) {
      # M2: INADFE
      return(n_alpha + n_theta + (n_blocks - 1) + n_nb_inno)
    } else {
      # M3: Heterogeneous
      return(n_blocks * (n_alpha + n_theta) + n_nb_inno)
    }
  }
}


#' Run all homogeneity tests for INAD
#'
#' Convenience function to run all three homogeneity tests at once and
#' return a summary.
#'
#' @inheritParams test_homogeneity_inad
#'
#' @return A list with class \code{"homogeneity_tests_inad"} containing results
#'   for all three tests and a summary table.
#'
#' @examples
#' \donttest{
#' data("bolus_inad")
#' tests <- run_homogeneity_tests_inad(bolus_inad$y, bolus_inad$blocks,
#'                                      order = 1, thinning = "nbinom",
#'                                      innovation = "bell")
#' print(tests)
#' }
#'
#' @export
run_homogeneity_tests_inad <- function(y, blocks, order = 1,
                                        thinning = "binom", innovation = "pois",
                                        ...) {
  
  # Fit all three models once
  if (!is.matrix(y)) y <- as.matrix(y)
  n_subjects <- nrow(y)
  n_time <- ncol(y)
  fit_args <- list(...)
  if (anyNA(y) && is.null(fit_args$na_action)) {
    fit_args$na_action <- "marginalize"
  }
  
  blocks <- as.integer(blocks)
  unique_blocks <- sort(unique(blocks))
  n_blocks <- length(unique_blocks)
  block_map <- setNames(seq_along(unique_blocks), unique_blocks)
  blocks_norm <- as.integer(block_map[as.character(blocks)])
  group_indices <- lapply(1:n_blocks, function(g) which(blocks_norm == g))
  
  # Fit M1 (Pooled)
  fit_pooled <- do.call(
    fit_inad,
    c(
      list(y = y, order = order, thinning = thinning, innovation = innovation, blocks = NULL),
      fit_args
    )
  )
  
  # Fit M2 (INADFE)
  fit_inadfe <- do.call(
    fit_inad,
    c(
      list(y = y, order = order, thinning = thinning, innovation = innovation, blocks = blocks_norm),
      fit_args
    )
  )
  
  # Fit M3 (Heterogeneous)
  fit_hetero <- do.call(
    .fit_inad_heterogeneous,
    c(
      list(
        y = y,
        blocks = blocks_norm,
        n_blocks = n_blocks,
        group_indices = group_indices,
        order = order,
        thinning = thinning,
        innovation = innovation
      ),
      fit_args
    )
  )
  
  # Run all three tests
  test_all <- test_homogeneity_inad(y, blocks, order = order,
                                    thinning = thinning, innovation = innovation,
                                    test = "all",
                                    fit_pooled = fit_pooled,
                                    fit_hetero = fit_hetero,
                                    na_action = fit_args$na_action)
  
  test_mean <- test_homogeneity_inad(y, blocks, order = order,
                                     thinning = thinning, innovation = innovation,
                                     test = "mean",
                                     fit_pooled = fit_pooled,
                                     fit_inadfe = fit_inadfe,
                                     na_action = fit_args$na_action)
  
  test_dep <- test_homogeneity_inad(y, blocks, order = order,
                                    thinning = thinning, innovation = innovation,
                                    test = "dependence",
                                    fit_inadfe = fit_inadfe,
                                    fit_hetero = fit_hetero,
                                    na_action = fit_args$na_action)
  
  # Create summary table
  summary_table <- data.frame(
    test = c("M1 vs M3 (all)", "M1 vs M2 (mean)", "M2 vs M3 (dependence)"),
    method = c(test_all$method, test_mean$method, test_dep$method),
    statistic = c(test_all$statistic, test_mean$statistic, test_dep$statistic),
    lrt_stat = c(test_all$lrt_stat, test_mean$lrt_stat, test_dep$lrt_stat),
    df = c(test_all$df, test_mean$df, test_dep$df),
    p_value = c(test_all$p_value, test_mean$p_value, test_dep$p_value),
    bic_selected = c(test_all$bic_selected, test_mean$bic_selected, test_dep$bic_selected),
    stringsAsFactors = FALSE
  )
  
  # Model comparison table
  model_table <- data.frame(
    model = c("M1 (Pooled)", "M2 (INADFE)", "M3 (Heterogeneous)"),
    logLik = c(fit_pooled$log_l, fit_inadfe$log_l, fit_hetero$log_l),
    n_params = c(
      .count_params_homogeneity(fit_pooled, "mean", n_blocks, order, n_time, 
                                 thinning, innovation, is_null = TRUE),
      .count_params_homogeneity(fit_inadfe, "dependence", n_blocks, order, n_time, 
                                 thinning, innovation, is_null = TRUE),
      .count_params_homogeneity(fit_hetero, "all", n_blocks, order, n_time, 
                                 thinning, innovation, is_null = FALSE)
    ),
    stringsAsFactors = FALSE
  )
  model_table$BIC <- -2 * model_table$logLik + model_table$n_params * log(n_subjects)
  
  result <- list(
    method = "lrt",
    test_all = test_all,
    test_mean = test_mean,
    test_dependence = test_dep,
    summary_table = summary_table,
    model_table = model_table,
    fits = list(
      pooled = fit_pooled,
      inadfe = fit_inadfe,
      heterogeneous = fit_hetero
    ),
    best_model_bic = model_table$model[which.min(model_table$BIC)],
    settings = list(
      order = order,
      thinning = thinning,
      innovation = innovation,
      n_subjects = n_subjects,
      n_time = n_time,
      n_blocks = n_blocks
    )
  )
  
  class(result) <- "homogeneity_tests_inad"
  result
}


#' Print method for test_homogeneity_inad
#'
#' @param x Object of class \code{test_homogeneity_inad}
#' @param digits Number of digits for printing
#' @param ... Unused
#' @return Invisibly returns \code{x}.
#' @export
print.test_homogeneity_inad <- function(x, digits = 4, ...) {
  stat <- if (!is.null(x$statistic)) x$statistic else x$lrt_stat
  cat("\nLikelihood Ratio Test for Homogeneity (INAD)\n")
  cat("=============================================\n\n")
  
  test_desc <- switch(x$test,
    "all" = "H0: M1 (Pooled) vs H1: M3 (Heterogeneous)",
    "mean" = "H0: M1 (Pooled) vs H1: M2 (INADFE - different means)",
    "dependence" = "H0: M2 (INADFE) vs H1: M3 (Heterogeneous - different dependence)"
  )
  cat(test_desc, "\n\n")
  
  cat("Number of groups:", x$settings$n_blocks, "\n")
  cat("Group sizes:", paste(x$settings$group_sizes, collapse = ", "), "\n")
  cat("Order:", x$settings$order, "\n")
  cat("Thinning:", x$settings$thinning, " Innovation:", x$settings$innovation, "\n\n")
  
  print(x$table, row.names = FALSE, digits = digits)
  
  cat("\nLRT statistic:", round(stat, digits), "\n")
  cat("Degrees of freedom:", x$df, "\n")
  cat("P-value:", format.pval(x$p_value, digits = digits), "\n")
  cat("BIC selects:", ifelse(x$bic_selected == "null", 
                              x$table$model[1], x$table$model[2]), "\n")
  
  invisible(x)
}


#' Print method for homogeneity_tests_inad
#'
#' @param x Object of class \code{homogeneity_tests_inad}
#' @param digits Number of digits for printing
#' @param ... Unused
#' @return Invisibly returns \code{x}.
#' @export
print.homogeneity_tests_inad <- function(x, digits = 4, ...) {
  cat("\nHomogeneity Tests for INAD Models\n")
  cat("==================================\n\n")
  
  cat("Settings: Order", x$settings$order, ",", 
      x$settings$thinning, "thinning,", x$settings$innovation, "innovations\n")
  cat("Groups:", x$settings$n_blocks, "  N:", x$settings$n_subjects, 
      "  T:", x$settings$n_time, "\n\n")
  
  cat("Model Comparison:\n")
  print(x$model_table, row.names = FALSE, digits = digits)
  cat("\n")
  
  cat("Test Results:\n")
  print(x$summary_table, row.names = FALSE, digits = digits)
  cat("\n")
  
  cat("Best model by BIC:", x$best_model_bic, "\n")
  
  invisible(x)
}

# test-test_homogeneity_inad.R

testthat::test_that("test_homogeneity_inad validates inputs correctly", {
  skip_on_cran()
  set.seed(123)
  y <- matrix(rpois(100, 5), nrow = 20, ncol = 5)
  blocks <- c(rep(1, 10), rep(2, 10))
  
  # blocks required
  testthat::expect_error(
    test_homogeneity_inad(y, order = 1),
    "blocks must be provided"
  )
  
  # blocks length must match
  testthat::expect_error(
    test_homogeneity_inad(y, blocks = c(1, 2, 3), order = 1),
    "length equal to nrow"
  )
  
  # Need at least 2 groups
  testthat::expect_error(
    test_homogeneity_inad(y, blocks = rep(1, 20), order = 1),
    "at least 2 groups"
  )
  
  # y must be non-negative integers
  y_bad <- y
  y_bad[1, 1] <- -1
  testthat::expect_error(
    test_homogeneity_inad(y_bad, blocks, order = 1),
    "non-negative integers"
  )
})


testthat::test_that("test_homogeneity_inad runs all test types", {
  skip_on_cran()
  set.seed(456)
  
  # Small simulated data
  n1 <- 25
  n2 <- 25
  n_time <- 5
  
  # Simulate from same model (homogeneous)
  y1 <- simulate_inad(n1, n_time, order = 1, thinning = "binom", 
                      innovation = "pois", alpha = 0.3, theta = 3)
  y2 <- simulate_inad(n2, n_time, order = 1, thinning = "binom", 
                      innovation = "pois", alpha = 0.3, theta = 3)
  y <- rbind(y1, y2)
  blocks <- c(rep(1, n1), rep(2, n2))
  
  # Test all three test types
  test_all <- test_homogeneity_inad(y, blocks, order = 1, test = "all")
  testthat::expect_s3_class(test_all, "test_homogeneity_inad")
  testthat::expect_true(test_all$lrt_stat >= 0)
  testthat::expect_true(test_all$df > 0)
  testthat::expect_true(test_all$p_value >= 0 && test_all$p_value <= 1)
  testthat::expect_equal(test_all$test, "all")
  
  test_mean <- test_homogeneity_inad(y, blocks, order = 1, test = "mean")
  testthat::expect_s3_class(test_mean, "test_homogeneity_inad")
  testthat::expect_equal(test_mean$test, "mean")
  
  test_dep <- test_homogeneity_inad(y, blocks, order = 1, test = "dependence")
  testthat::expect_s3_class(test_dep, "test_homogeneity_inad")
  testthat::expect_equal(test_dep$test, "dependence")
})


testthat::test_that("test_homogeneity_inad detects heterogeneous groups", {
  skip_on_cran()
  set.seed(789)

  n1 <- 20
  n2 <- 20
  n_time <- 5

  # Simulate from DIFFERENT models
  y1 <- simulate_inad(n1, n_time, order = 1, thinning = "binom",
                      innovation = "pois", alpha = 0.2, theta = 2)
  y2 <- simulate_inad(n2, n_time, order = 1, thinning = "binom",
                      innovation = "pois", alpha = 0.6, theta = 6)
  y <- rbind(y1, y2)
  blocks <- c(rep(1, n1), rep(2, n2))
  
  # Test for overall heterogeneity - should detect differences
  test_all <- test_homogeneity_inad(y, blocks, order = 1, test = "all")
  
  # With very different parameters, we expect to reject H0
  # (p_value should be small)
  testthat::expect_true(test_all$p_value < 0.10)  # Relaxed threshold for randomness
})


testthat::test_that("test_homogeneity_inad detects mean differences", {
  skip_on_cran()
  set.seed(101)

  n1 <- 25
  n2 <- 25
  n_time <- 5

  # Same dependence, different means
  y1 <- simulate_inad(n1, n_time, order = 1, thinning = "binom",
                      innovation = "pois", alpha = 0.4, theta = 2)
  y2 <- simulate_inad(n2, n_time, order = 1, thinning = "binom",
                      innovation = "pois", alpha = 0.4, theta = 6)
  y <- rbind(y1, y2)
  blocks <- c(rep(1, n1), rep(2, n2))
  
  # Test for mean differences - should detect
  test_mean <- test_homogeneity_inad(y, blocks, order = 1, test = "mean")
  testthat::expect_true(test_mean$p_value < 0.10)
})


testthat::test_that("run_homogeneity_tests_inad works correctly", {
  skip_on_cran()
  set.seed(202)
  
  n1 <- 30
  n2 <- 30
  n_time <- 5
  
  y1 <- simulate_inad(n1, n_time, order = 1, thinning = "binom", 
                      innovation = "pois", alpha = 0.3, theta = 4)
  y2 <- simulate_inad(n2, n_time, order = 1, thinning = "binom", 
                      innovation = "pois", alpha = 0.3, theta = 4)
  y <- rbind(y1, y2)
  blocks <- c(rep(1, n1), rep(2, n2))
  
  # Run all tests at once
  tests <- run_homogeneity_tests_inad(y, blocks, order = 1)
  
  testthat::expect_s3_class(tests, "homogeneity_tests_inad")
  testthat::expect_true("test_all" %in% names(tests))
  testthat::expect_true("test_mean" %in% names(tests))
  testthat::expect_true("test_dependence" %in% names(tests))
  testthat::expect_true("summary_table" %in% names(tests))
  testthat::expect_true("model_table" %in% names(tests))
  testthat::expect_true("best_model_bic" %in% names(tests))
  
  # Check summary table structure
  testthat::expect_equal(nrow(tests$summary_table), 3)
  testthat::expect_true(all(c("test", "lrt_stat", "df", "p_value") %in% 
                             names(tests$summary_table)))
  
  # Check model table structure
  testthat::expect_equal(nrow(tests$model_table), 3)
  testthat::expect_true(all(c("model", "logLik", "n_params", "BIC") %in% 
                             names(tests$model_table)))
})


testthat::test_that("test_homogeneity_inad works with order 0", {
  skip_on_cran()
  set.seed(303)
  
  n1 <- 25
  n2 <- 25
  n_time <- 5
  
  # Order 0 means independence
  y1 <- matrix(rpois(n1 * n_time, 3), nrow = n1, ncol = n_time)
  y2 <- matrix(rpois(n2 * n_time, 5), nrow = n2, ncol = n_time)
  y <- rbind(y1, y2)
  blocks <- c(rep(1, n1), rep(2, n2))
  
  test <- test_homogeneity_inad(y, blocks, order = 0, test = "all")
  
  testthat::expect_s3_class(test, "test_homogeneity_inad")
  testthat::expect_true(test$lrt_stat >= 0)
})


testthat::test_that("test_homogeneity_inad works with more than 2 groups", {
  skip_on_cran()
  set.seed(404)
  
  n1 <- 20
  n2 <- 25
  n3 <- 20
  n_time <- 5
  
  y1 <- simulate_inad(n1, n_time, order = 1, thinning = "binom", 
                      innovation = "pois", alpha = 0.3, theta = 3)
  y2 <- simulate_inad(n2, n_time, order = 1, thinning = "binom", 
                      innovation = "pois", alpha = 0.3, theta = 3)
  y3 <- simulate_inad(n3, n_time, order = 1, thinning = "binom", 
                      innovation = "pois", alpha = 0.3, theta = 3)
  y <- rbind(y1, y2, y3)
  blocks <- c(rep(1, n1), rep(2, n2), rep(3, n3))
  
  test <- test_homogeneity_inad(y, blocks, order = 1, test = "all")
  
  testthat::expect_s3_class(test, "test_homogeneity_inad")
  testthat::expect_equal(test$settings$n_blocks, 3)
})


testthat::test_that("test_homogeneity_inad BIC selects simpler model when homogeneous", {
  skip_on_cran()
  set.seed(505)

  n1 <- 25
  n2 <- 25
  n_time <- 5

  # Same parameters (homogeneous case) - use scalar values
  y1 <- simulate_inad(n1, n_time, order = 1, thinning = "binom",
                      innovation = "pois", alpha = 0.4, theta = 4)
  y2 <- simulate_inad(n2, n_time, order = 1, thinning = "binom",
                      innovation = "pois", alpha = 0.4, theta = 4)
  y <- rbind(y1, y2)
  blocks <- c(rep(1, n1), rep(2, n2))
  
  tests <- run_homogeneity_tests_inad(y, blocks, order = 1)
  
  # BIC should prefer simpler model (pooled or INADFE) over heterogeneous
  # when data is truly homogeneous
  testthat::expect_true(tests$best_model_bic %in% c("M1 (Pooled)", "M2 (INADFE)"))
})


testthat::test_that("test_homogeneity_inad df is correct for innovation='nbinom'", {
  skip_on_cran()
  set.seed(808)

  n_time <- 4
  n1 <- 20; n2 <- 20

  y1 <- simulate_inad(n1, n_time, order = 1, thinning = "binom",
                      innovation = "nbinom", alpha = 0.3, theta = 3,
                      nb_inno_size = 2)
  y2 <- simulate_inad(n2, n_time, order = 1, thinning = "binom",
                      innovation = "nbinom", alpha = 0.3, theta = 3,
                      nb_inno_size = 2)
  y <- rbind(y1, y2)
  blocks <- c(rep(1, n1), rep(2, n2))

  # order=1: n_alpha = n_time-1 = 3, n_theta = 4, n_nb = 4 per group
  # M1 k = 3+4+4 = 11;  M3 k = 2*(3+4)+2*4 = 22  ->  df = 11
  test_all <- test_homogeneity_inad(y, blocks, order = 1, test = "all",
                                    innovation = "nbinom",
                                    thinning = "binom", max_iter = 10)
  expected_df <- (2 - 1) * (3 + n_time + n_time)   # (n_blocks-1)*(n_alpha+n_theta+n_nb)
  testthat::expect_equal(test_all$df, expected_df)
})


testthat::test_that("print methods work without error", {
  skip_on_cran()
  set.seed(606)
  
  n1 <- 20
  n2 <- 20
  n_time <- 4
  
  y1 <- simulate_inad(n1, n_time, order = 1, thinning = "binom", 
                      innovation = "pois", alpha = 0.3, theta = 3)
  y2 <- simulate_inad(n2, n_time, order = 1, thinning = "binom", 
                      innovation = "pois", alpha = 0.3, theta = 3)
  y <- rbind(y1, y2)
  blocks <- c(rep(1, n1), rep(2, n2))
  
  # Single test print
  test <- test_homogeneity_inad(y, blocks, order = 1, test = "mean")
  testthat::expect_output(print(test), "Homogeneity")
  
  # All tests print
  tests <- run_homogeneity_tests_inad(y, blocks, order = 1)
  testthat::expect_output(print(tests), "Homogeneity Tests")
})


testthat::test_that("test_homogeneity_inad uses pre-fitted models", {
  skip_on_cran()
  set.seed(707)

  n1 <- 15
  n2 <- 15
  n_time <- 4
  
  y1 <- simulate_inad(n1, n_time, order = 1, thinning = "binom", 
                      innovation = "pois", alpha = 0.3, theta = 3)
  y2 <- simulate_inad(n2, n_time, order = 1, thinning = "binom", 
                      innovation = "pois", alpha = 0.3, theta = 3)
  y <- rbind(y1, y2)
  blocks <- c(rep(1, n1), rep(2, n2))
  
  # Pre-fit models
  fit_pooled <- fit_inad(y, order = 1, thinning = "binom", 
                         innovation = "pois", blocks = NULL)
  fit_inadfe <- fit_inad(y, order = 1, thinning = "binom", 
                         innovation = "pois", blocks = blocks)
  
  # Use pre-fitted pooled and inadfe
  test_mean <- test_homogeneity_inad(y, blocks, order = 1, test = "mean",
                                     fit_pooled = fit_pooled, 
                                     fit_inadfe = fit_inadfe)
  
  testthat::expect_s3_class(test_mean, "test_homogeneity_inad")
  testthat::expect_equal(test_mean$fit_null$log_l, fit_pooled$log_l)
  testthat::expect_equal(test_mean$fit_alt$log_l, fit_inadfe$log_l)
})

#' Log-likelihood for Gaussian AD models (with missing data support)
#'
#' Computes the log-likelihood for Gaussian antedependence models of order 0, 1, or 2.
#' Supports missing data under MAR assumption via na_action parameter.
#'
#' @param y Numeric matrix with n_subjects rows and n_time columns. May contain NA.
#' @param order Antedependence order, one of 0, 1, or 2.
#' @param mu Mean vector (length n_time).
#' @param phi Dependence coefficient(s). For order 1: vector of length n_time-1.
#'   For order 2: matrix with 2 columns or vector of length 2*(n_time-2).
#' @param sigma Innovation standard deviations (length n_time).
#' @param blocks Integer vector of block membership (length n_subjects), or NULL.
#' @param tau Block effects, first element constrained to zero
#' @param na_action How to handle missing values:
#'   \itemize{
#'     \item \code{fail}: Error if any NA is present (default)
#'     \item \code{complete}: Use only complete cases
#'     \item \code{marginalize}: Compute observed-data likelihood
#'   }
#'
#' @return Scalar log-likelihood value.
#'
#' @details
#' For complete data (no NA), all three na_action options give the same result.
#'
#' For missing data:
#' \itemize{
#'   \item marginalize: Uses MVN marginalization to compute P(Y_obs). This is
#'     the correct observed-data likelihood for MAR missing data.
#'   \item complete: Removes subjects with any missing values. May lose information.
#'   \item fail: Stops with error. Useful to ensure no missing data present.
#' }
#'
#' @examples
#' set.seed(1)
#' y <- simulate_gau(n_subjects = 30, n_time = 5, order = 1, phi = 0.3)
#' fit <- fit_gau(y, order = 1)
#' logL_gau(y, order = 1, mu = fit$mu, phi = fit$phi, sigma = fit$sigma)
#'
#' @export
#' @importFrom stats dnorm
logL_gau <- function(y, order, mu = NULL, phi = NULL, sigma = NULL, blocks = NULL, tau = 0,
                    na_action = c("fail", "complete", "marginalize")) {
  na_action <- match.arg(na_action)

  if (!is.matrix(y)) y <- as.matrix(y)

  n_subjects <- nrow(y)
  n_time <- ncol(y)

  if (!order %in% c(0, 1, 2)) {
    stop("order must be 0, 1, or 2", call. = FALSE)
  }

  .expand_vec <- function(x, default, name) {
    if (is.null(x)) {
      out <- rep(default, n_time)
    } else if (length(x) == 1) {
      out <- rep(as.numeric(x), n_time)
    } else {
      out <- as.numeric(x)
    }
    if (length(out) != n_time) {
      stop(paste0(name, " must have length n_time"), call. = FALSE)
    }
    out
  }

  mu <- .expand_vec(mu, 0, "mu")
  sigma <- .expand_vec(sigma, 1, "sigma")

  if (any(!is.finite(mu))) return(-Inf)
  if (any(!is.finite(sigma)) || any(sigma <= 0)) return(-Inf)

  if (order == 0) {
    phi_norm <- numeric(0)
  } else if (order == 1) {
    if (is.null(phi)) {
      phi_norm <- rep(0, n_time)
    } else if (length(phi) == 1) {
      phi_norm <- rep(as.numeric(phi), n_time)
    } else {
      phi_vec <- as.numeric(phi)
      if (length(phi_vec) == n_time) {
        phi_norm <- phi_vec
      } else if (length(phi_vec) == n_time - 1) {
        phi_norm <- c(0, phi_vec)
      } else {
        stop("phi must have length n_time or n_time - 1 for order 1", call. = FALSE)
      }
    }
  } else {
    if (is.null(phi)) {
      phi_norm <- matrix(0, nrow = 2, ncol = n_time)
    } else if (is.matrix(phi)) {
      if (nrow(phi) == 2 && ncol(phi) == n_time) {
        phi_norm <- phi
      } else {
        stop("phi must be a 2 x n_time matrix for order 2", call. = FALSE)
      }
    } else {
      phi_vec <- as.numeric(phi)
      if (length(phi_vec) == 2 * n_time) {
        phi_norm <- matrix(phi_vec, nrow = 2, byrow = TRUE)
      } else if (length(phi_vec) == 2 * (n_time - 2)) {
        phi_norm <- matrix(0, nrow = 2, ncol = n_time)
        if (n_time >= 3) {
          phi_compact <- matrix(phi_vec, ncol = 2, byrow = TRUE)
          phi_norm[1, 3:n_time] <- phi_compact[, 1]
          phi_norm[2, 3:n_time] <- phi_compact[, 2]
        }
      } else {
        stop("phi must be compatible with order 2", call. = FALSE)
      }
    }
  }

  has_missing <- any(is.na(y))

  if (has_missing) {
    if (na_action == "fail") {
      stop("y contains NA values. Use na_action = 'marginalize' or 'complete'",
           call. = FALSE)
    }

    if (na_action == "complete") {
      cc <- .extract_complete_cases(y, blocks, warn = FALSE)
      y <- cc$y
      blocks <- cc$blocks
      n_subjects <- nrow(y)
      has_missing <- FALSE
    }

    if (na_action == "marginalize" && has_missing) {
      return(.logL_gau_missing(y, order, mu, phi_norm, sigma, blocks, tau))
    }
  }

  if (!is.null(blocks)) {
    if (length(blocks) != n_subjects) stop("blocks must have length n_subjects", call. = FALSE)
    n_blocks <- length(unique(blocks))
    if (length(tau) == 1 && n_blocks > 1) {
      tau <- c(0, rep(as.numeric(tau), n_blocks - 1))
    }
    if (length(tau) != n_blocks) stop("tau must have length equal to number of blocks", call. = FALSE)
    if (abs(tau[1]) > 1e-10) {
      warning("tau[1] should be 0 for identifiability. Setting tau[1] = 0.")
      tau[1] <- 0
    }
  } else {
    tau <- 0
  }

  log_lik <- 0

  for (s in 1:n_subjects) {
    y_s <- y[s, ]

    if (!is.null(blocks)) {
      mu_s <- mu + tau[blocks[s]]
    } else {
      mu_s <- mu
    }

    log_lik <- log_lik + dnorm(y_s[1], mean = mu_s[1], sd = sigma[1], log = TRUE)

    if (order == 0) {
      for (t in 2:n_time) {
        log_lik <- log_lik + dnorm(y_s[t], mean = mu_s[t], sd = sigma[t], log = TRUE)
      }

    } else if (order == 1) {
      for (t in 2:n_time) {
        cond_mean <- mu_s[t] + phi_norm[t] * (y_s[t - 1] - mu_s[t - 1])
        log_lik <- log_lik + dnorm(y_s[t], mean = cond_mean, sd = sigma[t], log = TRUE)
      }

    } else if (order == 2) {
      if (n_time >= 2) {
        cond_mean_2 <- mu_s[2] + phi_norm[1, 2] * (y_s[1] - mu_s[1])
        log_lik <- log_lik + dnorm(y_s[2], mean = cond_mean_2, sd = sigma[2], log = TRUE)
      }

      for (t in 3:n_time) {
        cond_mean <- mu_s[t] +
          phi_norm[1, t] * (y_s[t - 1] - mu_s[t - 1]) +
          phi_norm[2, t] * (y_s[t - 2] - mu_s[t - 2])
        log_lik <- log_lik + dnorm(y_s[t], mean = cond_mean, sd = sigma[t], log = TRUE)
      }
    }
  }

  log_lik
}

testthat::test_that("logL_gau matches manual computation for order 0", {
    set.seed(123)
    n <- 5
    N <- 3
    y <- matrix(rnorm(n * N), nrow = n, ncol = N)
    mu <- c(1, 2, 3)
    sigma <- c(1, 1.5, 2)
    blocks <- c(1, 1, 2, 2, 3)
    tau <- c(0, 0.5, -0.5)

    ll_fun <- logL_gau(
        y = y, order = 0, mu = mu, sigma = sigma,
        blocks = blocks, tau = tau
    )

    ll_manual <- 0
    for (s in seq_len(n)) {
        for (t in seq_len(N)) {
            m <- mu[t] + tau[blocks[s]]
            ll_manual <- ll_manual + dnorm(y[s, t], mean = m, sd = sigma[t], log = TRUE)
        }
    }

    testthat::expect_equal(ll_fun, ll_manual)
})

testthat::test_that("logL_gau matches manual computation for order 1", {
    set.seed(234)
    n <- 5
    N <- 4
    y <- matrix(rnorm(n * N), nrow = n, ncol = N)
    mu <- c(0, 1, 2, 3)
    phi <- c(0, 0.5, 0.6, 0.7)
    sigma <- rep(1, N)
    blocks <- c(1, 1, 2, 2, 2)
    tau <- c(0, 1)

    ll_fun <- logL_gau(
        y = y, order = 1, mu = mu, phi = phi, sigma = sigma,
        blocks = blocks, tau = tau
    )

    ll_manual <- 0
    for (s in seq_len(n)) {
        m1 <- mu[1] + tau[blocks[s]]
        ll_manual <- ll_manual + dnorm(y[s, 1], mean = m1, sd = sigma[1], log = TRUE)
        for (t in 2:N) {
            m_t <- mu[t] + tau[blocks[s]]
            m_tm1 <- mu[t - 1] + tau[blocks[s]]
            cond_mean <- m_t + phi[t] * (y[s, t - 1] - m_tm1)
            ll_manual <- ll_manual + dnorm(y[s, t], mean = cond_mean, sd = sigma[t], log = TRUE)
        }
    }

    testthat::expect_equal(ll_fun, ll_manual)
})

testthat::test_that("logL_gau matches manual computation for order 2", {
    set.seed(345)
    n <- 6
    N <- 5
    y <- matrix(rnorm(n * N), nrow = n, ncol = N)
    mu <- 1:N
    phi <- matrix(0, nrow = 2, ncol = N)
    phi[1, 2:N] <- c(0.3, 0.4, 0.5, 0.6)
    phi[2, 3:N] <- c(0.1, 0.15, 0.2)
    sigma <- rep(1, N)

    ll_fun <- logL_gau(
        y = y, order = 2, mu = mu, phi = phi, sigma = sigma
    )

    ll_manual <- 0
    for (s in seq_len(n)) {
        ll_manual <- ll_manual + dnorm(y[s, 1], mean = mu[1], sd = sigma[1], log = TRUE)
        ll_manual <- ll_manual + dnorm(y[s, 2],
            mean = mu[2] + phi[1, 2] * (y[s, 1] - mu[1]),
            sd = sigma[2], log = TRUE
        )
        for (t in 3:N) {
            cond_mean <- mu[t] +
                phi[1, t] * (y[s, t - 1] - mu[t - 1]) +
                phi[2, t] * (y[s, t - 2] - mu[t - 2])
            ll_manual <- ll_manual + dnorm(y[s, t], mean = cond_mean, sd = sigma[t], log = TRUE)
        }
    }

    testthat::expect_equal(ll_fun, ll_manual)
})

testthat::test_that("logL_gau input validation behavior", {
    y <- matrix(rnorm(20), nrow = 5, ncol = 4)

    testthat::expect_error(logL_gau(y, order = 3))
    testthat::expect_error(logL_gau(y, order = 1, mu = 1:2))
    testthat::expect_error(logL_gau(y, order = 2, phi = matrix(0, nrow = 2, ncol = 2)))
    testthat::expect_error(logL_gau(y, order = 1, blocks = 1))

    testthat::expect_equal(logL_gau(y, order = 0, sigma = -1), -Inf)

    y_bad <- y
    y_bad[1, 1] <- NA
    testthat::expect_error(
        logL_gau(y_bad, order = 0),
        "na_action = 'marginalize' or 'complete'"
    )
    ll_marginalize <- logL_gau(y_bad, order = 0, na_action = "marginalize")
    testthat::expect_true(is.finite(ll_marginalize))
})

testthat::test_that("logL_gau is higher near the generating parameters", {
    set.seed(456)
    n <- 50
    N <- 6
    true_phi <- 0.6
    y <- matrix(0, nrow = n, ncol = N)
    y[, 1] <- rnorm(n)
    for (t in 2:N) {
        y[, t] <- true_phi * y[, t - 1] + rnorm(n)
    }

    ll_true <- logL_gau(y, order = 1, mu = 0, phi = 0.6, sigma = 1)
    ll_wrong <- logL_gau(y, order = 1, mu = 0, phi = 0.1, sigma = 1)

    testthat::expect_true(ll_true > ll_wrong)
})

#' Simulate INAD antedependence series
#'
#' Generate longitudinal count data from an INAD model using a thinning
#' operator and an innovation distribution.
#'
#' Time 1 observations are generated from the innovation distribution alone.
#' For times 2 to `n_time`, counts are generated as thinning of previous
#' counts plus independent innovations. When `order = 0`, all time points
#' are generated from the innovation distribution and the thinning operator
#' and `alpha` are ignored.
#'
#' If `blocks` is provided, innovations include a block effect. For Poisson and
#' negative binomial innovations, the innovation mean is `theta[t] + tau[blocks[i]]`.
#' For Bell innovations, the innovation mean is `theta[t] * exp(theta[t]) + tau[blocks[i]]`.
#'
#' @param n_subjects number of subjects
#' @param n_time number of time points
#' @param order antedependence order, 0, 1 or 2
#' @param thinning thinning operator, one of `"binom"`, `"pois"`, `"nbinom"`
#' @param innovation innovation distribution, one of `"pois"`, `"bell"`,
#'   `"nbinom"`
#' @param alpha thinning parameter or vector or matrix; if `NULL`, defaults
#'   are used depending on the order
#' @param theta innovation parameter or vector; if `NULL`, defaults are used
#'   depending on the innovation type. For Poisson and negative binomial
#'   innovations, `theta` is the innovation mean parameter. For Bell
#'   innovations, `theta` is the Bell rate parameter, with innovation mean
#'   `theta * exp(theta)`.
#' @param nb_inno_size size (dispersion) parameter for negative binomial innovations
#'   when `innovation = "nbinom"`; must be positive. If `NULL`, defaults to 1.
#'   Larger values correspond to less overdispersion (approaching Poisson as
#'   size -> Inf).
#' @param blocks integer vector of length `n_subjects` indicating block
#'   membership for each subject; if `NULL`, no block effect is applied
#' @param tau group effect vector indexed by block; `tau[1]` is forced to 0.
#'   If scalar x, it is expanded to c(0, x, ..., x) with length equal to the
#'   number of blocks
#' @param seed optional random seed for reproducibility
#'
#' @return integer matrix of counts with dimension `n_subjects` by `n_time`
#'
#' @examples
#' y <- simulate_inad(
#'   n_subjects = 20,
#'   n_time = 6,
#'   order = 1,
#'   thinning = "binom",
#'   innovation = "pois",
#'   alpha = 0.3,
#'   theta = 2,
#'   seed = 42
#' )
#' dim(y)
#' @export
simulate_inad <- function(n_subjects,
                          n_time,
                          order = 1L,
                          thinning = c("binom", "pois", "nbinom"),
                          innovation = c("pois", "bell", "nbinom"),
                          alpha = NULL,
                          theta = NULL,
                          nb_inno_size = NULL,
                          blocks = NULL,
                          tau = 0,
                          seed = NULL) {

    if (!is.null(seed)) set.seed(seed)

    thinning <- match.arg(thinning)
    innovation <- match.arg(innovation)

    if (!order %in% c(0L, 1L, 2L)) {
        stop("order must be 0, 1 or 2")
    }

    if (length(n_subjects) != 1L || n_subjects <= 0) {
        stop("n_subjects must be a positive integer")
    }
    if (length(n_time) != 1L || n_time <= 0) {
        stop("n_time must be a positive integer")
    }

    n_subjects <- as.integer(n_subjects)
    n_time <- as.integer(n_time)
    order <- as.integer(order)

    if (!is.null(blocks)) {
        if (length(blocks) != n_subjects) {
            stop("blocks must have length n_subjects")
        }
        if (any(is.na(blocks))) {
            stop("blocks must have no missing values")
        }
        blocks <- as.integer(factor(blocks))
        B <- max(blocks)
        if (length(tau) == 1L) {
            tau <- c(0, rep(as.numeric(tau), max(B - 1L, 0L)))
        } else {
            tau <- as.numeric(tau)
            if (length(tau) != B) {
                stop("when blocks is provided, tau must be scalar or have length equal to max(blocks)")
            }
            tau[1L] <- 0
        }
    } else {
        B <- 1L
        blocks <- rep(1L, n_subjects)
        tau <- 0
    }

    if (is.null(theta)) {
        if (innovation == "pois") {
            theta <- rep(10, n_time)
        } else if (innovation == "bell") {
            theta <- rep(2, n_time)
        } else if (innovation == "nbinom") {
            theta <- rep(10, n_time)
        }
    } else if (length(theta) == 1L) {
        theta <- rep(theta, n_time)
    }

    if (length(theta) != n_time) {
        stop("theta must be NULL, scalar, or length n_time")
    }

    # For negative binomial innovations:
    # nb_inno_size is the size (dispersion) parameter r > 0
    # theta is the mean parameter mu > 0
    # Variance = mu + mu^2/size = mu * (1 + mu/size)
    # This matches the parameterization in loglik_inad.R: dnbinom(size=sz, mu=lam)
    if (innovation == "nbinom") {
        if (is.null(nb_inno_size)) {
            nb_inno_size <- rep(1, n_time)
        } else if (length(nb_inno_size) == 1L) {
            nb_inno_size <- rep(nb_inno_size, n_time)
        }
        if (length(nb_inno_size) != n_time) {
            stop("nb_inno_size must be NULL, scalar, or length n_time")
        }
        if (any(nb_inno_size <= 0)) {
            stop("nb_inno_size must be positive")
        }
    }

    if (is.null(alpha)) {
        if (order == 0L) {
            alpha <- 0
        } else if (order == 1L) {
            alpha <- rep(0.5, n_time)
            alpha[1L] <- 0
        } else {
            alpha <- matrix(0, nrow = 2L, ncol = n_time)
            alpha[1L, 2:n_time] <- 0.5
            if (n_time >= 3L) {
                alpha[2L, 3:n_time] <- 0.25
            }
        }
    } else {
        if (order == 0L) {
            if (length(alpha) != 1L || alpha != 0) {
                warning("order = 0, alpha is ignored and treated as 0")
            }
        } else if (order == 1L) {
            if (length(alpha) == 1L) {
                alpha <- rep(alpha, n_time)
            }
            if (length(alpha) != n_time) {
                stop("for order 1, alpha must be scalar or length n_time")
            }
        } else {
            if (!is.matrix(alpha) || nrow(alpha) != 2L || ncol(alpha) != n_time) {
                stop("for order 2, alpha must be a 2 by n_time matrix")
            }
        }
    }

    if (order > 0L) {
        if (thinning %in% c("binom", "nbinom")) {
            if (order == 1L) {
                if (any(alpha < 0 | alpha >= 1)) {
                    stop("alpha must lie in [0, 1) for binom and nbinom thinning")
                }
            } else {
                if (any(alpha < 0 | alpha >= 1)) {
                    stop("alpha must lie in [0, 1) for binom and nbinom thinning")
                }
            }
        } else if (thinning == "pois") {
            if (order == 1L) {
                if (any(alpha < 0)) {
                    stop("alpha must be nonnegative for pois thinning")
                }
            } else {
                if (any(alpha < 0)) {
                    stop("alpha must be nonnegative for pois thinning")
                }
            }
        }
    }

    y <- matrix(0L, nrow = n_subjects, ncol = n_time)

    draw_innovation <- function(t_index) {
        eff_mean <- .inad_effective_innovation_mean(theta[t_index], tau[blocks], innovation)
        if (any(!is.finite(eff_mean)) || any(eff_mean <= 0)) {
            if (innovation == "bell") {
                stop("theta[t] * exp(theta[t]) + tau[block] must be positive for all subjects")
            }
            stop("theta[t] + tau[block] must be positive for all subjects")
        }
        if (innovation == "pois") {
            stats::rpois(n_subjects, lambda = eff_mean)
        } else if (innovation == "bell") {
            eff_theta <- .bell_theta_from_mean(eff_mean)
            as.integer(vapply(eff_theta, function(th) rbell(1L, theta = th), integer(1L)))
        } else {
            # For nbinom: size = nb_inno_size[t], mu = eff (mean)
            # This matches dnbinom(size=sz, mu=lam) in loglik_inad.R
            stats::rnbinom(
                n_subjects,
                size = nb_inno_size[t_index],
                mu = eff_mean
            )
        }
    }

    draw_nbinom_thinning <- function(y_prev, alpha_t) {
        prob_nb <- 1 / (1 + alpha_t)
        thinned <- integer(n_subjects)
        idx_pos <- y_prev > 0L
        if (any(idx_pos)) {
            thinned[idx_pos] <- stats::rnbinom(
                n = sum(idx_pos),
                size = y_prev[idx_pos],
                prob = prob_nb
            )
        }
        thinned
    }

    if (order == 0L) {
        for (t in seq_len(n_time)) {
            y[, t] <- as.integer(draw_innovation(t))
        }
        return(y)
    }

    y[, 1L] <- as.integer(draw_innovation(1L))

    if (n_time >= 2L) {
        for (t in 2L:n_time) {
            thinned_total <- integer(n_subjects)

            if (order >= 1L) {
                alpha_t1 <- if (order == 1L) alpha[t] else alpha[1L, t]
                if (alpha_t1 > 0) {
                    y_prev1 <- y[, t - 1L]
                    if (thinning == "binom") {
                        thinned1 <- stats::rbinom(
                            n = n_subjects,
                            size = y_prev1,
                            prob = alpha_t1
                        )
                    } else if (thinning == "pois") {
                        thinned1 <- stats::rpois(
                            n = n_subjects,
                            lambda = alpha_t1 * y_prev1
                        )
                    } else {
                        # nbinom thinning: alpha * Y | Y ~ NegBin(Y, 1/(1+alpha))
                        thinned1 <- draw_nbinom_thinning(y_prev1, alpha_t1)
                    }
                    thinned_total <- thinned_total + thinned1
                }
            }

            if (order >= 2L && t > 2L) {
                alpha_t2 <- alpha[2L, t]
                if (alpha_t2 > 0) {
                    y_prev2 <- y[, t - 2L]
                    if (thinning == "binom") {
                        thinned2 <- stats::rbinom(
                            n = n_subjects,
                            size = y_prev2,
                            prob = alpha_t2
                        )
                    } else if (thinning == "pois") {
                        thinned2 <- stats::rpois(
                            n = n_subjects,
                            lambda = alpha_t2 * y_prev2
                        )
                    } else {
                        thinned2 <- draw_nbinom_thinning(y_prev2, alpha_t2)
                    }
                    thinned_total <- thinned_total + thinned2
                }
            }

            eps_t <- draw_innovation(t)
            y[, t] <- as.integer(thinned_total + eps_t)
        }
    }

    y
}

test_that("simulate_inad basic structure and defaults", {
    skip_on_cran()
    set.seed(1)
    y <- simulate_inad(
        n_subjects = 5,
        n_time = 7
    )

    expect_true(is.matrix(y))
    expect_equal(dim(y), c(5L, 7L))
    expect_true(is.integer(y))
    expect_true(all(y >= 0))

    y_seed1 <- simulate_inad(n_subjects = 5, n_time = 7, seed = 777)
    y_seed2 <- simulate_inad(n_subjects = 5, n_time = 7, seed = 777)
    expect_equal(y_seed1, y_seed2)
})

test_that("order 0 ignores thinning choice", {
    skip_on_cran()
    set.seed(1)
    y1 <- simulate_inad(
        n_subjects = 10,
        n_time = 6,
        order = 0,
        thinning = "binom",
        innovation = "pois"
    )

    set.seed(1)
    y2 <- simulate_inad(
        n_subjects = 10,
        n_time = 6,
        order = 0,
        thinning = "nbinom",
        innovation = "pois"
    )

    expect_equal(y1, y2)
})

test_that("order 0 matches order 1 with zero alpha", {
    skip_on_cran()
    n_sub <- 8
    n_time <- 6

    set.seed(2)
    y0 <- simulate_inad(
        n_subjects = n_sub,
        n_time = n_time,
        order = 0,
        thinning = "binom",
        innovation = "pois",
        theta = 5
    )

    zero_alpha <- rep(0, n_time)

    set.seed(2)
    y1 <- simulate_inad(
        n_subjects = n_sub,
        n_time = n_time,
        order = 1,
        thinning = "binom",
        innovation = "pois",
        alpha = zero_alpha,
        theta = 5
    )

    expect_equal(y0, y1)
})

test_that("order 2 runs and produces nonnegative integers", {
    skip_on_cran()
    set.seed(3)
    y <- simulate_inad(
        n_subjects = 4,
        n_time = 8,
        order = 2,
        thinning = "binom",
        innovation = "bell"
    )

    expect_true(is.matrix(y))
    expect_equal(dim(y), c(4L, 8L))
    expect_true(is.integer(y))
    expect_true(all(y >= 0))
})

test_that("negative binomial innovations use size and mu parameterization", {
    skip_on_cran()
    # nb_inno_size is the size (dispersion) parameter (> 0)
    # theta is the mean parameter
    # Mean of NB(size, mu) = mu = theta
    # Variance = mu + mu^2/size = theta * (1 + theta/size)
    
    # With same theta but different size, variance changes
    # Smaller size -> more overdispersion -> higher variance
    set.seed(4)
    y1 <- simulate_inad(
        n_subjects = 100,
        n_time = 5,
        order = 0,  # order 0 so we only test innovation
        innovation = "nbinom",
        theta = 10,
        nb_inno_size = 1  # high overdispersion
    )

    set.seed(5)
    y2 <- simulate_inad(
        n_subjects = 100,
        n_time = 5,
        order = 0,
        innovation = "nbinom",
        theta = 10,
        nb_inno_size = 100  # low overdispersion (closer to Poisson)
    )

    # Both should have mean close to theta = 10
    expect_true(abs(mean(y1) - 10) < 2)
    expect_true(abs(mean(y2) - 10) < 2)
    
    # y1 should have higher variance (smaller size = more overdispersion)
    expect_true(var(as.vector(y1)) > var(as.vector(y2)))
})

test_that("negative binomial innovation mean matches theta", {
    skip_on_cran()
    # Simulation-estimation consistency check
    set.seed(123)
    true_theta <- 8
    true_size <- 2
    
    y <- simulate_inad(
        n_subjects = 500,
        n_time = 1,
        order = 0,
        innovation = "nbinom",
        theta = true_theta,
        nb_inno_size = true_size
    )
    
    # Sample mean should be close to true_theta
    expect_equal(mean(y), true_theta, tolerance = 0.5)
    
    # Theoretical variance = theta * (1 + theta/size) = 8 * (1 + 8/2) = 40
    theo_var <- true_theta * (1 + true_theta / true_size)
    expect_equal(var(as.vector(y)), theo_var, tolerance = 10)
})

test_that("blocks and tau affect innovations as intended", {
    skip_on_cran()
    n_sub <- 200
    n_time <- 3
    blocks <- c(rep(1L, n_sub / 2), rep(2L, n_sub / 2))

    set.seed(10)
    y0 <- simulate_inad(
        n_subjects = n_sub,
        n_time = n_time,
        order = 0,
        innovation = "pois",
        theta = 10,
        blocks = blocks,
        tau = 3
    )

    m1 <- mean(y0[blocks == 1L, ])
    m2 <- mean(y0[blocks == 2L, ])
    expect_true(m2 > m1)

    set.seed(11)
    y1 <- simulate_inad(
        n_subjects = n_sub,
        n_time = n_time,
        order = 0,
        innovation = "pois",
        theta = 10,
        blocks = blocks,
        tau = c(100, 3)
    )

    m1b <- mean(y1[blocks == 1L, ])
    m2b <- mean(y1[blocks == 2L, ])
    expect_true(abs(m1b - 10) < 1.5)
    expect_true(m2b > m1b)

    blocks_nonseq <- c(rep(2L, n_sub / 2), rep(5L, n_sub / 2))
    set.seed(12)
    y_nonseq <- simulate_inad(
        n_subjects = n_sub,
        n_time = n_time,
        order = 0,
        innovation = "pois",
        theta = 10,
        blocks = blocks_nonseq,
        tau = c(0, 3)
    )
    mn1 <- mean(y_nonseq[blocks_nonseq == 2L, ])
    mn2 <- mean(y_nonseq[blocks_nonseq == 5L, ])
    expect_true(mn2 > mn1)
})

test_that("nbinom thinning handles zero previous counts without introducing NA", {
    skip_on_cran()
    set.seed(99)
    y1 <- simulate_inad(
        n_subjects = 200,
        n_time = 6,
        order = 1,
        thinning = "nbinom",
        innovation = "pois",
        alpha = rep(0.6, 6),
        theta = 1e-12
    )
    expect_true(any(y1[, 1L] == 0L))
    expect_false(anyNA(y1))

    set.seed(100)
    y2 <- simulate_inad(
        n_subjects = 200,
        n_time = 7,
        order = 2,
        thinning = "nbinom",
        innovation = "pois",
        alpha = rbind(c(0, 0.6, rep(0.6, 5)), c(0, 0, rep(0.4, 5))),
        theta = 1e-12
    )
    expect_true(any(y2[, 1L] == 0L))
    expect_false(anyNA(y2))
})

test_that("bell block effects are applied on innovation mean scale", {
    skip_on_cran()
    n_sub <- 4000
    blocks <- c(rep(1L, n_sub / 2), rep(2L, n_sub / 2))

    set.seed(321)
    y <- simulate_inad(
        n_subjects = n_sub,
        n_time = 1,
        order = 0,
        innovation = "bell",
        theta = 0.5,
        blocks = blocks,
        tau = -0.5
    )

    # Base Bell mean at theta=0.5 is 0.5*exp(0.5) ~= 0.824.
    # Group 2 mean should be lower by about 0.5 on the innovation-mean scale.
    m1 <- mean(y[blocks == 1L, 1L])
    m2 <- mean(y[blocks == 2L, 1L])
    expect_gt(m1, m2)
    expect_equal(m1 - m2, 0.5, tolerance = 0.12)
})

#' Log-likelihood for INAD models (with missing data support)
#'
#' If blocks is NULL, this computes the log likelihood as the sum of per time
#' contributions from logL_inad_i for computational convenience.
#'
#' @param y Integer matrix n_sub by n_time.
#' @param order Integer in \{0, 1, 2\}.
#' @param thinning One of "binom", "pois", "nbinom".
#' @param innovation One of "pois", "bell", "nbinom".
#' @param alpha Thinning parameters. For order 1, numeric length 1 or n_time.
#'   For order 2, either a matrix n_time by 2 or a list(alpha1, alpha2).
#' @param theta Innovation parameter(s). Numeric length 1 or n_time. For
#'   Poisson and negative binomial innovations, this is the innovation mean.
#'   For Bell innovations, this is the Bell rate parameter
#'   (mean \eqn{\theta e^\theta}).
#' @param nb_inno_size Size parameter for innovation "nbinom". Numeric length 1 or n_time.
#' @param blocks Optional integer vector of length n_sub. If NULL, no fixed effect.
#' @param tau Optional numeric vector. Only used if blocks is not NULL.
#' @param na_action How to handle missing values:
#'   \itemize{
#'     \item \code{"fail"}: error if any NA is present.
#'     \item \code{"complete"}: use only complete-case subjects.
#'     \item \code{"marginalize"}: observed-data likelihood under MAR via truncated-state recursion.
#'   }
#'
#' @return A scalar log likelihood.
#'
#' @examples
#' set.seed(1)
#' y <- simulate_inad(
#'   n_subjects = 60,
#'   n_time = 5,
#'   order = 1,
#'   thinning = "binom",
#'   innovation = "pois",
#'   alpha = 0.3,
#'   theta = 2
#' )
#' fit <- fit_inad(y, order = 1, thinning = "binom", innovation = "pois", max_iter = 20)
#' logL_inad(
#'   y,
#'   order = 1,
#'   thinning = "binom",
#'   innovation = "pois",
#'   alpha = fit$alpha,
#'   theta = fit$theta
#' )
#' @export
logL_inad <- function(
        y,
        order = 1,
        thinning = c("binom", "pois", "nbinom"),
        innovation = c("pois", "bell", "nbinom"),
        alpha,
        theta,
        nb_inno_size = NULL,
        blocks = NULL,
        tau = 0,
        na_action = c("fail", "complete", "marginalize")
) {
    thinning <- match.arg(thinning)
    innovation <- match.arg(innovation)
    na_action <- match.arg(na_action)

    if (!is.matrix(y)) y <- as.matrix(y)
    y_obs <- y[!is.na(y)]
    if (any(!is.finite(y_obs)) || any(y_obs < 0) || any(y_obs != floor(y_obs))) return(-Inf)

    n <- nrow(y)
    N <- ncol(y)
    if (!(order %in% c(0, 1, 2))) stop("order must be 0, 1, or 2")
    has_missing <- any(is.na(y))

    if (has_missing) {
        if (na_action == "fail") {
            stop("y contains NA values. Use na_action = 'complete' or 'marginalize'.", call. = FALSE)
        }

        if (na_action == "complete") {
            cc <- .extract_complete_cases(y, blocks, warn = FALSE)
            y <- cc$y
            blocks <- cc$blocks
            n <- nrow(y)
            has_missing <- FALSE
        }

        if (na_action == "marginalize" && has_missing) {
            return(.logL_inad_missing(
                y = y,
                order = order,
                thinning = thinning,
                innovation = innovation,
                alpha = alpha,
                theta = theta,
                nb_inno_size = nb_inno_size,
                blocks = blocks,
                tau = tau
            ))
        }
    }

    if (is.null(blocks)) {
        return(sum(vapply(
            seq_len(N),
            function(i) logL_inad_i(
                y = y,
                i = i,
                order = order,
                thinning = thinning,
                innovation = innovation,
                alpha = alpha,
                theta = theta,
                nb_inno_size = nb_inno_size
            ),
            numeric(1)
        )))
    }

    if (length(blocks) != n) stop("length(blocks) must equal nrow(y)")
    blocks <- as.integer(blocks)

    B <- max(blocks)
    tau <- as.numeric(tau)

    if (B <= 1L) {
        tau <- 0
    } else if (length(tau) == 1L) {
        tau <- c(0, rep(tau, B - 1L))
    } else {
        if (length(tau) < B) stop("tau must be length 1 or at least max(blocks)")
        tau <- tau[seq_len(B)]
        tau[1] <- 0
    }

    theta <- as.numeric(theta)
    if (length(theta) == 1L) theta <- rep(theta, N)
    if (length(theta) != N) stop("theta must be length 1 or ncol(y)")

    if (innovation == "nbinom") {
        if (is.null(nb_inno_size)) stop("nb_inno_size is required for innovation='nbinom'")
        nb_inno_size <- as.numeric(nb_inno_size)
        if (length(nb_inno_size) == 1L) nb_inno_size <- rep(nb_inno_size, N)
        if (length(nb_inno_size) != N) stop("nb_inno_size must be length 1 or ncol(y)")
        if (any(nb_inno_size <= 0)) return(-Inf)
    }

    ap <- .unpack_alpha(order, alpha, N)
    alpha1 <- ap$a1
    alpha2 <- ap$a2

    loglik <- 0

    lam_eff <- matrix(NA_real_, nrow = B, ncol = N)
    for (b in seq_len(B)) {
        lam_b <- .inad_effective_innovation_param(theta, tau[b], innovation)
        if (any(!is.finite(lam_b)) || any(lam_b <= 0)) return(-Inf)
        lam_eff[b, ] <- lam_b
    }

    for (s in 1:n) {
        b <- blocks[s]
        for (i in 1:N) {
            lam <- lam_eff[b, i]
            if (!is.finite(lam) || lam <= 0) return(-Inf)

            y_si <- y[s, i]

            if (i == 1 || order == 0) {
                p0 <- .innov_vec(y_si, lam, innovation, if (innovation == "nbinom") nb_inno_size[i] else NA_real_)
                if (!is.finite(p0) || p0 <= 0) return(-Inf)
                loglik <- loglik + log(p0)
                next
            }

            if (order == 1 || i == 2) {
                y_prev <- y[s, i - 1]
                k_vals <- 0:y_si

                thin_vals <- .thin_vec(k_vals, y_prev, alpha1[i], thinning)
                innov_vals <- .innov_vec(y_si - k_vals, lam, innovation, if (innovation == "nbinom") nb_inno_size[i] else NA_real_)

                conv <- sum(thin_vals * innov_vals)
                if (!is.finite(conv) || conv <= 0) return(-Inf)

                loglik <- loglik + log(conv)
                next
            }

            # Order 2, i >= 3
            y1 <- y[s, i - 1]
            y2 <- y[s, i - 2]

            k1_vals <- 0:y_si
            log_terms <- rep(-Inf, length(k1_vals))

            for (idx in seq_along(k1_vals)) {
                k1 <- k1_vals[idx]
                thin1 <- .thin_vec(k1, y1, alpha1[i], thinning)
                if (!is.finite(thin1) || thin1 <= 0) next

                remain <- y_si - k1
                k2_vals <- 0:remain

                thin2_vals <- .thin_vec(k2_vals, y2, alpha2[i], thinning)
                innov_vals <- .innov_vec(remain - k2_vals, lam, innovation, if (innovation == "nbinom") nb_inno_size[i] else NA_real_)

                inner_sum <- sum(thin2_vals * innov_vals)
                if (!is.finite(inner_sum) || inner_sum <= 0) next

                log_terms[idx] <- log(thin1) + log(inner_sum)
            }

            log_conv <- .log_sum_exp(log_terms)
            if (!is.finite(log_conv)) return(-Inf)
            loglik <- loglik + log_conv
        }
    }

    loglik
}

#' INAD log likelihood contribution at time i (no fixed effect)
#'
#' Returns the time i contribution, summed over subjects, for the no fixed effect model.
#'
#' @param y Integer matrix n_sub by n_time.
#' @param i Time index in 1..ncol(y).
#' @param order Integer in \{0, 1, 2\}.
#' @param thinning One of "binom", "pois", "nbinom".
#' @param innovation One of "pois", "bell", "nbinom".
#' @param alpha Thinning parameters. For order 1, numeric length 1 or n_time.
#'   For order 2, either a matrix n_time by 2 or a list(alpha1, alpha2).
#' @param theta Innovation parameter at time i, or a vector length 1 or n_time.
#'   For Poisson and negative binomial innovations, this is the innovation
#'   mean. For Bell innovations, this is the Bell rate parameter
#'   (mean \eqn{\theta e^\theta}).
#' @param nb_inno_size Size parameter for innovation "nbinom". Numeric length 1 or n_time.
#'
#' @return A scalar log likelihood contribution for time i.
#'
#' @examples
#' set.seed(1)
#' y <- simulate_inad(
#'   n_subjects = 50,
#'   n_time = 5,
#'   order = 1,
#'   thinning = "binom",
#'   innovation = "pois",
#'   alpha = 0.3,
#'   theta = 2
#' )
#' fit <- fit_inad(y, order = 1, thinning = "binom", innovation = "pois", max_iter = 20)
#' logL_inad_i(
#'   y = y,
#'   i = 3,
#'   order = 1,
#'   thinning = "binom",
#'   innovation = "pois",
#'   alpha = fit$alpha,
#'   theta = fit$theta
#' )
#' @export
logL_inad_i <- function(
        y,
        i,
        order = 1,
        thinning = c("binom", "pois", "nbinom"),
        innovation = c("pois", "bell", "nbinom"),
        alpha,
        theta,
        nb_inno_size = NULL
) {
    thinning <- match.arg(thinning)
    innovation <- match.arg(innovation)

    if (!is.matrix(y)) y <- as.matrix(y)
    if (any(!is.finite(y)) || any(y < 0) || any(y != floor(y))) return(-Inf)

    n <- nrow(y)
    N <- ncol(y)

    if (!(order %in% c(0, 1, 2))) stop("order must be 0, 1, or 2")
    if (length(i) != 1L || i < 1L || i > N) stop("i must be in 1..ncol(y)")

    theta <- as.numeric(theta)
    if (length(theta) == 1L) theta <- rep(theta, N)
    if (length(theta) != N) stop("theta must be length 1 or ncol(y)")

    if (innovation == "nbinom") {
        if (is.null(nb_inno_size)) stop("nb_inno_size is required for innovation='nbinom'")
        nb_inno_size <- as.numeric(nb_inno_size)
        if (length(nb_inno_size) == 1L) nb_inno_size <- rep(nb_inno_size, N)
        if (length(nb_inno_size) != N) stop("nb_inno_size must be length 1 or ncol(y)")
        if (any(nb_inno_size <= 0)) return(-Inf)
    }

    ap <- .unpack_alpha(order, alpha, N)
    alpha1 <- ap$a1
    alpha2 <- ap$a2

    loglik_i <- 0
    th_i <- theta[i]
    if (!is.finite(th_i) || th_i <= 0) return(-Inf)

    for (s in 1:n) {
        y_si <- y[s, i]

        if (i == 1 || order == 0) {
            p0 <- .innov_vec(y_si, th_i, innovation, if (innovation == "nbinom") nb_inno_size[i] else NA_real_)
            if (!is.finite(p0) || p0 <= 0) return(-Inf)
            loglik_i <- loglik_i + log(p0)
            next
        }

        if (order == 1 || i == 2) {
            y_prev <- y[s, i - 1]
            k_vals <- 0:y_si

            thin_vals <- .thin_vec(k_vals, y_prev, alpha1[i], thinning)
            innov_vals <- .innov_vec(y_si - k_vals, th_i, innovation, if (innovation == "nbinom") nb_inno_size[i] else NA_real_)

            conv <- sum(thin_vals * innov_vals)
            if (!is.finite(conv) || conv <= 0) return(-Inf)

            loglik_i <- loglik_i + log(conv)
            next
        }

        # Order 2, i >= 3
        y1 <- y[s, i - 1]
        y2 <- y[s, i - 2]

        k1_vals <- 0:y_si
        log_terms <- rep(-Inf, length(k1_vals))

        for (idx in seq_along(k1_vals)) {
            k1 <- k1_vals[idx]
            thin1 <- .thin_vec(k1, y1, alpha1[i], thinning)
            if (!is.finite(thin1) || thin1 <= 0) next

            remain <- y_si - k1
            k2_vals <- 0:remain

            thin2_vals <- .thin_vec(k2_vals, y2, alpha2[i], thinning)
            innov_vals <- .innov_vec(remain - k2_vals, th_i, innovation, if (innovation == "nbinom") nb_inno_size[i] else NA_real_)

            inner_sum <- sum(thin2_vals * innov_vals)
            if (!is.finite(inner_sum) || inner_sum <= 0) next

            log_terms[idx] <- log(thin1) + log(inner_sum)
        }

        log_conv <- .log_sum_exp(log_terms)
        if (!is.finite(log_conv)) return(-Inf)
        loglik_i <- loglik_i + log_conv
    }

    loglik_i
}

#' Observed-data INAD likelihood with missing values under MAR
#'
#' Uses forward recursion over a truncated count state space.
#'
#' @keywords internal
.logL_inad_missing <- function(
        y,
        order,
        thinning,
        innovation,
        alpha,
        theta,
        nb_inno_size = NULL,
        blocks = NULL,
        tau = 0
) {
    n <- nrow(y)
    N <- ncol(y)

    theta <- as.numeric(theta)
    if (length(theta) == 1L) theta <- rep(theta, N)
    if (length(theta) != N) stop("theta must be length 1 or ncol(y)")

    if (innovation == "nbinom") {
        if (is.null(nb_inno_size)) stop("nb_inno_size is required for innovation='nbinom'")
        nb_inno_size <- as.numeric(nb_inno_size)
        if (length(nb_inno_size) == 1L) nb_inno_size <- rep(nb_inno_size, N)
        if (length(nb_inno_size) != N) stop("nb_inno_size must be length 1 or ncol(y)")
        if (any(nb_inno_size <= 0) || any(!is.finite(nb_inno_size))) return(-Inf)
    }

    ap <- .unpack_alpha(order, alpha, N)
    alpha1 <- ap$a1
    alpha2 <- ap$a2

    if (is.null(blocks)) {
        blocks <- rep(1L, n)
        tau_use <- 0
        n_blocks <- 1L
    } else {
        if (length(blocks) != n) stop("length(blocks) must equal nrow(y)")
        blocks <- as.integer(blocks)
        if (any(!is.finite(blocks)) || any(blocks < 1L)) stop("blocks must be positive integers")

        n_blocks <- max(blocks)
        tau_use <- as.numeric(tau)
        if (n_blocks <= 1L) {
            tau_use <- 0
        } else if (length(tau_use) == 1L) {
            tau_use <- c(0, rep(tau_use, n_blocks - 1L))
        } else {
            if (length(tau_use) < n_blocks) stop("tau must be length 1 or at least max(blocks)")
            tau_use <- tau_use[seq_len(n_blocks)]
            tau_use[1] <- 0
        }
    }

    K <- .inad_state_max(
        y = y,
        order = order,
        alpha1 = alpha1,
        alpha2 = alpha2,
        theta = theta,
        tau = tau_use,
        innovation = innovation
    )

    trans_by_block <- vector("list", n_blocks)
    for (b in seq_len(n_blocks)) {
        tau_b <- if (length(tau_use) == 1L) tau_use else tau_use[b]
        lam <- .inad_effective_innovation_param(theta, tau_b, innovation)
        if (any(!is.finite(lam)) || any(lam <= 0)) return(-Inf)
        trans_by_block[[b]] <- .inad_build_transitions(
            order = order,
            N = N,
            K = K,
            alpha1 = alpha1,
            alpha2 = alpha2,
            thinning = thinning,
            innovation = innovation,
            lambda = lam,
            nb_inno_size = nb_inno_size
        )
    }

    ll <- 0
    for (s in seq_len(n)) {
        y_s <- y[s, ]
        if (all(is.na(y_s))) next

        tr <- trans_by_block[[blocks[s]]]
        ll_s <- if (order == 0) {
            .inad_subject_ll_order0(y_s, tr$innov_list, K)
        } else if (order == 1) {
            .inad_subject_ll_order1(y_s, tr$innov_list[[1]], tr$trans1, K)
        } else {
            .inad_subject_ll_order2(y_s, tr$innov_list[[1]], tr$trans1[[2]], tr$order2_info, K)
        }

        if (!is.finite(ll_s)) return(-Inf)
        ll <- ll + ll_s
    }

    ll
}

#' Build transition objects for truncated-state missing-data likelihood
#' @keywords internal
.inad_build_transitions <- function(
        order, N, K, alpha1, alpha2, thinning, innovation, lambda, nb_inno_size
) {
    innov_list <- vector("list", N)
    for (t in seq_len(N)) {
        sz_t <- if (innovation == "nbinom") nb_inno_size[t] else NA_real_
        innov_list[[t]] <- .innov_vec(0:K, lambda[t], innovation, sz_t)
    }

    if (order == 0) {
        return(list(innov_list = innov_list, trans1 = NULL, order2_info = NULL))
    }

    trans1 <- vector("list", N)
    order2_info <- vector("list", N)

    for (t in 2:N) {
        thin1 <- lapply(0:K, function(prev) .inad_make_thin_pmf(prev, alpha1[t], thinning, K))

        if (order == 1 || t == 2) {
            M <- matrix(0, nrow = K + 1L, ncol = K + 1L)
            for (prev in 0:K) {
                M[prev + 1L, ] <- .inad_conv_trunc(thin1[[prev + 1L]], innov_list[[t]], K)
            }
            trans1[[t]] <- M
        } else {
            thin2 <- lapply(0:K, function(prev) .inad_make_thin_pmf(prev, alpha2[t], thinning, K))
            order2_info[[t]] <- list(
                thin1 = thin1,
                thin2 = thin2,
                innov = innov_list[[t]],
                cache = new.env(parent = emptyenv())
            )
        }
    }

    list(innov_list = innov_list, trans1 = trans1, order2_info = order2_info)
}

#' Subject-level observed-data likelihood for INAD(0)
#' @keywords internal
.inad_subject_ll_order0 <- function(y_s, innov_list, K) {
    obs_idx <- which(!is.na(y_s))
    if (length(obs_idx) == 0) return(0)

    ll <- 0
    for (t in obs_idx) {
        y_t <- y_s[t]
        if (y_t > K) return(-Inf)
        p <- innov_list[[t]][y_t + 1L]
        if (!is.finite(p) || p <= 0) return(-Inf)
        ll <- ll + log(p)
    }
    ll
}

#' Subject-level observed-data likelihood for INAD(1)
#' @keywords internal
.inad_subject_ll_order1 <- function(y_s, init_pmf, trans1, K) {
    N <- length(y_s)
    if (N == 0) return(0)

    y1 <- y_s[1]
    if (is.na(y1)) {
        p_prev <- init_pmf
    } else {
        if (y1 > K) return(-Inf)
        p_prev <- rep(0, K + 1L)
        p_prev[y1 + 1L] <- init_pmf[y1 + 1L]
    }

    z0 <- sum(p_prev)
    if (!is.finite(z0) || z0 <= 0) return(-Inf)
    ll <- log(z0)
    p_prev <- p_prev / z0

    for (t in 2:N) {
        M <- trans1[[t]]
        if (is.null(M)) return(-Inf)

        y_t <- y_s[t]
        p_next <- rep(0, K + 1L)

        if (is.na(y_t)) {
            p_next <- as.numeric(p_prev %*% M)
        } else {
            if (y_t > K) return(-Inf)
            v <- sum(p_prev * M[, y_t + 1L])
            p_next[y_t + 1L] <- v
        }

        z <- sum(p_next)
        if (!is.finite(z) || z <= 0) return(-Inf)
        ll <- ll + log(z)
        p_prev <- p_next / z
    }

    ll
}

#' Subject-level observed-data likelihood for INAD(2)
#' @keywords internal
.inad_subject_ll_order2 <- function(y_s, init_pmf, trans_t2, order2_info, K) {
    N <- length(y_s)
    if (N == 0) return(0)
    if (N == 1) {
        y1 <- y_s[1]
        if (is.na(y1)) return(log(sum(init_pmf)))
        if (y1 > K) return(-Inf)
        p <- init_pmf[y1 + 1L]
        if (!is.finite(p) || p <= 0) return(-Inf)
        return(log(p))
    }

    y1 <- y_s[1]
    if (is.na(y1)) {
        p_y1 <- init_pmf
    } else {
        if (y1 > K) return(-Inf)
        p_y1 <- rep(0, K + 1L)
        p_y1[y1 + 1L] <- init_pmf[y1 + 1L]
    }
    z1 <- sum(p_y1)
    if (!is.finite(z1) || z1 <= 0) return(-Inf)
    ll <- log(z1)
    p_y1 <- p_y1 / z1

    y2 <- y_s[2]
    pair <- matrix(0, nrow = K + 1L, ncol = K + 1L)
    for (prev2 in 0:K) {
        w <- p_y1[prev2 + 1L]
        if (w <= 0) next
        if (is.na(y2)) {
            pair[prev2 + 1L, ] <- pair[prev2 + 1L, ] + w * trans_t2[prev2 + 1L, ]
        } else {
            if (y2 > K) return(-Inf)
            pair[prev2 + 1L, y2 + 1L] <- pair[prev2 + 1L, y2 + 1L] + w * trans_t2[prev2 + 1L, y2 + 1L]
        }
    }

    z <- sum(pair)
    if (!is.finite(z) || z <= 0) return(-Inf)
    ll <- ll + log(z)
    pair <- pair / z

    if (N == 2) return(ll)

    for (t in 3:N) {
        info <- order2_info[[t]]
        if (is.null(info)) return(-Inf)
        y_t <- y_s[t]

        next_pair <- matrix(0, nrow = K + 1L, ncol = K + 1L)
        nz <- which(pair > 0, arr.ind = TRUE)

        if (is.na(y_t)) {
            for (k in seq_len(nrow(nz))) {
                prev2 <- nz[k, 1] - 1L
                prev1 <- nz[k, 2] - 1L
                w <- pair[nz[k, 1], nz[k, 2]]
                dist <- .inad_order2_dist(prev2, prev1, info, K)
                next_pair[prev1 + 1L, ] <- next_pair[prev1 + 1L, ] + w * dist
            }
        } else {
            if (y_t > K) return(-Inf)
            col_idx <- y_t + 1L
            for (k in seq_len(nrow(nz))) {
                prev2 <- nz[k, 1] - 1L
                prev1 <- nz[k, 2] - 1L
                w <- pair[nz[k, 1], nz[k, 2]]
                dist <- .inad_order2_dist(prev2, prev1, info, K)
                next_pair[prev1 + 1L, col_idx] <- next_pair[prev1 + 1L, col_idx] + w * dist[col_idx]
            }
        }

        z <- sum(next_pair)
        if (!is.finite(z) || z <= 0) return(-Inf)
        ll <- ll + log(z)
        pair <- next_pair / z
    }

    ll
}

#' Retrieve/calculate order-2 transition distribution for one previous state pair
#' @keywords internal
.inad_order2_dist <- function(prev2, prev1, info, K) {
    key <- paste0(prev2, "_", prev1)
    if (exists(key, envir = info$cache, inherits = FALSE)) {
        return(get(key, envir = info$cache, inherits = FALSE))
    }

    p12 <- .inad_conv_trunc(info$thin1[[prev1 + 1L]], info$thin2[[prev2 + 1L]], K)
    dist <- .inad_conv_trunc(p12, info$innov, K)
    assign(key, dist, envir = info$cache)
    dist
}

#' Truncated discrete convolution
#' @keywords internal
.inad_conv_trunc <- function(p, q, K) {
    out <- numeric(K + 1L)
    idx_p <- which(p > 0) - 1L
    for (i in idx_p) {
        pi <- p[i + 1L]
        max_j <- min(K - i, K)
        if (max_j < 0L) next
        out[(i + 0:max_j) + 1L] <- out[(i + 0:max_j) + 1L] + pi * q[1:(max_j + 1L)]
    }
    out
}

#' Build truncated thinning pmf for one previous count
#' @keywords internal
.inad_make_thin_pmf <- function(prev, alpha, thinning, K) {
    out <- numeric(K + 1L)
    k_max <- min(prev, K)
    if (k_max >= 0L) {
        vals <- .thin_vec(0:k_max, prev, alpha, thinning)
        out[1:(k_max + 1L)] <- vals
    }
    out
}

#' Heuristic state-space bound for INAD missing-data recursion
#' @keywords internal
.inad_state_max <- function(y, order, alpha1, alpha2, theta, tau, innovation = "pois") {
    y_obs <- y[!is.na(y)]
    obs_max <- if (length(y_obs) == 0L) 0 else max(y_obs)

    if (length(tau) == 1L) {
        mu_eff <- .inad_effective_innovation_mean(theta, tau, innovation = innovation)
    } else {
        mu_eff <- as.numeric(outer(
            tau,
            theta,
            FUN = function(tb, th) .inad_effective_innovation_mean(th, tb, innovation = innovation)
        ))
    }
    lam_max <- max(mu_eff, na.rm = TRUE)

    a_sum <- 0
    if (order == 1) {
        if (!is.null(alpha1) && length(alpha1) >= 2L) a_sum <- max(alpha1[2:length(alpha1)], na.rm = TRUE)
    }
    if (order == 2) {
        a2 <- if (is.null(alpha2)) rep(0, length(alpha1)) else alpha2
        a_sum <- max(alpha1 + a2, na.rm = TRUE)
    }

    if (!is.finite(a_sum)) a_sum <- 0
    a_sum <- min(max(a_sum, 0), 0.98)
    gain <- 1 / (1 - a_sum)

    mu_upper <- max(obs_max, lam_max * gain)
    k <- ceiling(mu_upper + 8 * sqrt(mu_upper + 1) + 10)
    k <- max(k, obs_max + 10, 20)
    k <- min(k, 300L)
    if (obs_max > k) k <- obs_max
    as.integer(k)
}

test_that("no fixed effect: logL_inad equals sum of logL_inad_i", {
    skip_on_cran()
    set.seed(1)
    y <- matrix(rpois(60, 2), nrow = 6)
    N <- ncol(y)

    alpha <- rep(0.3, N)
    theta <- rep(2.0, N)

    ll_full <- logL_inad(
        y = y,
        order = 1,
        thinning = "binom",
        innovation = "pois",
        alpha = alpha,
        theta = theta,
        blocks = NULL
    )

    ll_sum <- sum(vapply(
        seq_len(N),
        function(i) logL_inad_i(
            y = y,
            i = i,
            order = 1,
            thinning = "binom",
            innovation = "pois",
            alpha = alpha,
            theta = theta
        ),
        numeric(1)
    ))

    expect_equal(ll_full, ll_sum, tolerance = 1e-12)
})

test_that("order 0 reduces to innovation only and matches time sum", {
    skip_on_cran()
    set.seed(2)
    y <- matrix(rpois(30, 3), nrow = 5)
    theta <- rep(1.7, ncol(y))

    ll0 <- logL_inad(
        y = y,
        order = 0,
        thinning = "binom",
        innovation = "pois",
        alpha = 0.2,
        theta = theta,
        blocks = NULL
    )

    ll0_sum <- sum(vapply(
        seq_len(ncol(y)),
        function(i) logL_inad_i(
            y = y,
            i = i,
            order = 0,
            thinning = "binom",
            innovation = "pois",
            alpha = 0.2,
            theta = theta
        ),
        numeric(1)
    ))

    expect_equal(ll0, ll0_sum, tolerance = 1e-12)
})

test_that("fixed effect: tau length 1 expands and tau[1] is forced to 0", {
    skip_on_cran()
    y <- matrix(0L, nrow = 3, ncol = 4)
    blocks <- c(1L, 2L, 2L)
    theta <- rep(1.0, ncol(y))

    ll_a <- logL_inad(
        y = y,
        order = 0,
        thinning = "binom",
        innovation = "pois",
        alpha = 0.2,
        theta = theta,
        blocks = blocks,
        tau = 0.4
    )

    ll_b <- logL_inad(
        y = y,
        order = 0,
        thinning = "binom",
        innovation = "pois",
        alpha = 0.2,
        theta = theta,
        blocks = blocks,
        tau = c(999, 0.4)
    )

    expect_equal(ll_a, ll_b, tolerance = 1e-12)
})

test_that("fixed effect: invalid lambda yields -Inf", {
    skip_on_cran()
    y <- matrix(0L, nrow = 2, ncol = 3)
    blocks <- c(1L, 2L)

    ll <- logL_inad(
        y = y,
        order = 0,
        thinning = "binom",
        innovation = "pois",
        alpha = 0.1,
        theta = rep(0.5, ncol(y)),
        blocks = blocks,
        tau = -1
    )

    expect_equal(ll, -Inf)
})

test_that("bell FE tau is constrained by innovation mean, not raw Bell theta", {
    skip_on_cran()
    y <- matrix(0L, nrow = 2, ncol = 3)
    blocks <- c(1L, 2L)
    theta <- rep(0.5, ncol(y))

    # Valid under mean-shifted Bell FE: 0.5*exp(0.5) - 0.5 > 0
    ll_ok <- logL_inad(
        y = y,
        order = 0,
        thinning = "binom",
        innovation = "bell",
        alpha = 0.1,
        theta = theta,
        blocks = blocks,
        tau = -0.5
    )
    expect_true(is.finite(ll_ok))

    # Invalid because Bell innovation mean becomes nonpositive.
    ll_bad <- logL_inad(
        y = y,
        order = 0,
        thinning = "binom",
        innovation = "bell",
        alpha = 0.1,
        theta = theta,
        blocks = blocks,
        tau = -1.0
    )
    expect_equal(ll_bad, -Inf)
})

test_that("innovation nbinom requires nb_inno_size", {
    skip_on_cran()
    set.seed(3)
    y <- matrix(rpois(20, 2), nrow = 4)

    expect_error(
        logL_inad(
            y = y,
            order = 0,
            thinning = "binom",
            innovation = "nbinom",
            alpha = 0.2,
            theta = rep(1.0, ncol(y)),
            blocks = NULL
        )
    )
})

test_that("logL_inad with na_action='fail' errors on missing data", {
    skip_on_cran()
    y <- matrix(c(1L, 2L, NA, 2L, 1L, 0L), nrow = 2, byrow = TRUE)
    alpha <- rep(0.2, ncol(y))
    theta <- rep(1.5, ncol(y))

    expect_error(
        logL_inad(
            y = y,
            order = 1,
            thinning = "binom",
            innovation = "pois",
            alpha = alpha,
            theta = theta,
            na_action = "fail"
        ),
        "contains NA"
    )
})

test_that("logL_inad with na_action='complete' matches complete-case subset", {
    skip_on_cran()
    set.seed(11)
    y_full <- matrix(rpois(80, lambda = 2.0), nrow = 20, ncol = 4)
    y_mis <- y_full
    y_mis[1:6, 4] <- NA

    alpha <- rep(0.3, ncol(y_mis))
    theta <- rep(1.8, ncol(y_mis))

    ll_complete <- logL_inad(
        y = y_mis,
        order = 1,
        thinning = "binom",
        innovation = "pois",
        alpha = alpha,
        theta = theta,
        na_action = "complete"
    )

    cc_idx <- complete.cases(y_mis)
    ll_cc <- logL_inad(
        y = y_mis[cc_idx, , drop = FALSE],
        order = 1,
        thinning = "binom",
        innovation = "pois",
        alpha = alpha,
        theta = theta,
        na_action = "fail"
    )

    expect_equal(ll_complete, ll_cc, tolerance = 1e-8)
})

test_that("logL_inad with na_action='marginalize' handles monotone and intermittent missingness", {
    skip_on_cran()
    set.seed(12)
    y <- simulate_inad(
        n_subjects = 40,
        n_time = 5,
        order = 1,
        thinning = "binom",
        innovation = "pois",
        alpha = 0.35,
        theta = 2.2
    )

    y_mon <- y
    y_mon[1:8, 4:5] <- NA

    y_int <- y
    y_int[sample(length(y_int), size = 25)] <- NA

    alpha <- rep(0.35, ncol(y))
    theta <- rep(2.2, ncol(y))

    ll_mon <- logL_inad(
        y = y_mon,
        order = 1,
        thinning = "binom",
        innovation = "pois",
        alpha = alpha,
        theta = theta,
        na_action = "marginalize"
    )
    ll_int <- logL_inad(
        y = y_int,
        order = 1,
        thinning = "binom",
        innovation = "pois",
        alpha = alpha,
        theta = theta,
        na_action = "marginalize"
    )

    expect_true(is.finite(ll_mon))
    expect_true(is.finite(ll_int))
})

test_that(".thin_vec nbinom thinning is degenerate at zero lag count", {
    skip_on_cran()
    k_vals <- 0:6
    probs <- .thin_vec(k_vals, yprev = 0, a = 0.4, thinning = "nbinom")
    expect_equal(probs, c(1, rep(0, length(k_vals) - 1)))
})

# cat_utils.R - Internal utilities for categorical antedependence models

#' Validate categorical data matrix
#'
#' @param y Data matrix
#' @param n_categories Number of categories (NULL to infer)
#' @param allow_na Logical; if TRUE, allow missing values.
#'
#' @return List with validated y and n_categories
#'
#' @keywords internal
.validate_y_cat <- function(y, n_categories = NULL, allow_na = FALSE) {
  # Convert to matrix if needed

  if (!is.matrix(y)) y <- as.matrix(y)
  
  # Check for numeric/integer
  if (!is.numeric(y)) stop("y must be numeric")
  
  # Check for integer values
  if (any(y != floor(y), na.rm = TRUE)) {
    stop("y must be integer-valued (category codes)")
  }
  
  # Check for positive values
  if (any(y < 1, na.rm = TRUE)) {
    stop("y must have category codes >= 1")
  }
  
  # Check for NA
  if (!allow_na && any(is.na(y))) {
    stop("Missing data not supported in this call; y must be complete")
  }
  
  # Infer number of categories
  c_inferred <- max(y, na.rm = TRUE)
  if (!is.finite(c_inferred)) {
    stop("Unable to infer categories: all values are NA")
  }
  
  if (!is.null(n_categories)) {
    if (n_categories < c_inferred) {
      stop("n_categories (", n_categories, ") is smaller than max observed category (", 
           c_inferred, ")")
    }
    c_inferred <- n_categories
  }
  
  # Convert to integer storage
  storage.mode(y) <- "integer"
  
  list(y = y, n_categories = c_inferred)
}


#' Validate blocks parameter
#'
#' @param blocks Block membership vector
#' @param n_subjects Number of subjects
#'
#' @return Validated blocks vector (integer, 1-indexed)
#'
#' @keywords internal
.validate_blocks_cat <- function(blocks, n_subjects) {
  if (is.null(blocks)) {
    return(rep(1L, n_subjects))
  }
  
  if (length(blocks) != n_subjects) {
    stop("blocks must have length equal to number of subjects (", n_subjects, ")")
  }
  
  blocks <- as.integer(blocks)
  
  if (any(blocks < 1)) {
    stop("blocks must have values >= 1")
  }
  
  # Ensure contiguous from 1
  unique_blocks <- sort(unique(blocks))
  if (!identical(unique_blocks, seq_along(unique_blocks))) {
    # Remap to contiguous
    blocks <- match(blocks, unique_blocks)
  }
  
  blocks
}


#' Count free parameters for AD(p) categorical model
#'
#' @param order AD order p
#' @param n_categories Number of categories c
#' @param n_time Number of time points n
#' @param n_blocks Number of blocks/groups
#' @param homogeneous Whether parameters are shared across blocks
#'
#' @return Number of free parameters
#'
#' @keywords internal
.count_params_cat <- function(order, n_categories, n_time, n_blocks = 1, 
                               homogeneous = TRUE) {
  c <- n_categories
  n <- n_time
  p <- order
  
  if (p >= n) {
    stop("order must be less than n_time")
  }
  
  # Number of free parameters per population
  # For AD(p): (c-1) * sum_{k=1}^{n} c^{min(k-1, p)}
  # = (c-1) * [c^0 + c^1 + ... + c^{p-1} + (n-p) * c^p]  for p >= 1
  # = (c-1) * n  for p = 0
  
  if (p == 0) {
    n_params_per_pop <- (c - 1) * n
  } else {
    # Sum of c^0 + c^1 + ... + c^{p-1} = (c^p - 1) / (c - 1)
    sum_geometric <- (c^p - 1) / (c - 1)
    n_params_per_pop <- (c - 1) * (sum_geometric + (n - p) * c^p)
  }
  
  # Multiply by number of populations if heterogeneous
  if (homogeneous || n_blocks == 1) {
    return(as.integer(n_params_per_pop))
  } else {
    return(as.integer(n_params_per_pop * n_blocks))
  }
}


#' Get all category combinations of given length
#'
#' @param n_categories Number of categories c
#' @param length Number of positions
#'
#' @return Matrix where each row is a combination (c^length rows, length columns)
#'
#' @keywords internal
.get_combinations_cat <- function(n_categories, length) {
  if (length == 0) {
    return(matrix(nrow = 1, ncol = 0))
  }
  
  if (length == 1) {
    return(matrix(1:n_categories, ncol = 1))
  }
  
  # Use expand.grid and convert to matrix
  args <- rep(list(1:n_categories), length)
  combos <- expand.grid(args)
  as.matrix(combos)
}


#' Count cells for contingency table
#'
#' Counts occurrences of each combination of categories at specified time indices.
#'
#' @param y Data matrix (n_subjects x n_time)
#' @param time_indices Integer vector of time indices to count
#' @param n_categories Number of categories
#' @param subject_mask Logical vector indicating which subjects to include (NULL = all)
#'
#' @return Array of counts with dimensions c x c x ... (length = length(time_indices))
#'
#' @keywords internal
.count_cells_cat <- function(y, time_indices, n_categories, subject_mask = NULL) {
  n_subjects <- nrow(y)
  n_positions <- length(time_indices)
  c <- n_categories
  
  # Apply subject mask
  if (!is.null(subject_mask)) {
    y_sub <- y[subject_mask, , drop = FALSE]
  } else {
    y_sub <- y
  }
  
  n_sub <- nrow(y_sub)
  
  if (n_positions == 0) {
    return(array(n_sub, dim = 1))
  }
  
  # Extract relevant columns
  y_cols <- y_sub[, time_indices, drop = FALSE]
  
  # Create count array
  dims <- rep(c, n_positions)
  counts <- array(0L, dim = dims)
  
  # Count each combination
  for (i in seq_len(n_sub)) {
    # Get the index into the array
    idx <- as.list(y_cols[i, ])
    counts[matrix(unlist(idx), nrow = 1)] <- counts[matrix(unlist(idx), nrow = 1)] + 1L
  }
  
  counts
}


#' Count cells efficiently using table()
#'
#' Alternative implementation using R's table function.
#'
#' @param y Data matrix (n_subjects x n_time)
#' @param time_indices Integer vector of time indices
#' @param n_categories Number of categories
#' @param subject_mask Logical vector for subject selection
#'
#' @return Array of counts
#'
#' @keywords internal
.count_cells_table_cat <- function(y, time_indices, n_categories, subject_mask = NULL) {
  c <- n_categories
  n_positions <- length(time_indices)
  
  if (!is.null(subject_mask)) {
    y_sub <- y[subject_mask, , drop = FALSE]
  } else {
    y_sub <- y
  }
  
  if (n_positions == 0) {
    return(array(nrow(y_sub), dim = 1))
  }
  
  # Extract columns and convert to factors
  y_cols <- y_sub[, time_indices, drop = FALSE]
  
  # Create list of factors with consistent levels
  factors <- lapply(seq_len(n_positions), function(j) {
    factor(y_cols[, j], levels = 1:c)
  })
  
  # Use table to count
  counts <- table(factors)
  
  # Convert to array (table already returns array-like object)
  array(as.integer(counts), dim = rep(c, n_positions))
}


#' Convert counts to probabilities (with safe division)
#'
#' @param counts Array of counts
#' @param margin Which margins to condition on (sum over). NULL = no conditioning (joint).
#'
#' @return Array of probabilities
#'
#' @keywords internal
.counts_to_probs_cat <- function(counts, margin = NULL) {
  if (is.null(margin)) {
    # Joint probability: divide by total
    total <- sum(counts)
    if (total == 0) {
      return(counts * 0)  # All zeros
    }
    return(counts / total)
  }
  
  # Conditional probability: divide by marginal over specified dimensions
  # margin specifies which dimensions to keep (condition on)
  marginal <- apply(counts, margin, sum)
  
  # Divide each slice by its marginal
  # This is tricky with arrays - use sweep or manual
  probs <- counts
  dims <- dim(counts)
  n_dims <- length(dims)
  other_dims <- setdiff(seq_len(n_dims), margin)
  
  if (length(other_dims) == 1) {
    # Simple case: condition on all but one dimension
    # The last dimension (other_dims) is what we're computing P(Y_last | rest)
    for (idx in seq_len(prod(dims[margin]))) {
      # Convert linear index to subscripts for margin dimensions
      subs <- arrayInd(idx, dims[margin])
      
      # Build index for full array
      full_idx <- vector("list", n_dims)
      full_idx[margin] <- as.list(subs)
      full_idx[other_dims] <- list(TRUE)
      
      # Get denominator
      denom <- do.call(`[`, c(list(marginal), as.list(subs)))
      
      if (denom == 0) {
        # Set to 0 (or could use uniform 1/c)
        do.call(`[<-`, c(list(probs), full_idx, list(0)))
      } else {
        slice <- do.call(`[`, c(list(counts), full_idx))
        do.call(`[<-`, c(list(probs), full_idx, list(slice / denom)))
      }
    }
  } else {
    # More complex case - use sweep
    # Actually, let's just loop over all conditioning combinations
    cond_combos <- .get_combinations_cat(dims[margin[1]], length(margin))
    # This gets complicated - let's use a simpler approach
    
    # For transition probabilities, we typically have:
    # counts[y_{k-p}, ..., y_{k-1}, y_k] and want P(y_k | y_{k-p}, ..., y_{k-1})
    # So margin = 1:(n_dims-1) and we divide by sum over last dimension
    
    probs <- counts / (marginal + (marginal == 0))  # Avoid division by zero
    probs[marginal == 0] <- 0
  }
  
  probs
}


#' Compute transition probabilities from counts (simpler version)
#'
#' Given counts array with last dimension being Y_k and earlier dimensions
#' being conditioning variables, compute P(Y_k | conditioning).
#'
#' @param counts Array of counts, last dimension is response
#'
#' @return Array of transition probabilities (same dimensions)
#'
#' @keywords internal
.counts_to_transition_probs <- function(counts) {
  dims <- dim(counts)
  n_dims <- length(dims)
  
  if (n_dims == 1) {
    # Just marginal probabilities
    total <- sum(counts)
    if (total == 0) return(rep(0, dims[1]))
    return(counts / total)
  }
  
  # Sum over last dimension to get denominator
  margin_dims <- seq_len(n_dims - 1)
  denom <- apply(counts, margin_dims, sum)
  
  # Divide - need to replicate denom to match counts dimensions
  # Use sweep for this
  probs <- sweep(counts, margin_dims, denom, FUN = function(x, d) {
    ifelse(d == 0, 0, x / d)
  })
  
  probs
}


#' Safe log function (0 * log(0) = 0)
#'
#' @param x Numeric vector or array
#'
#' @return Log of x, with 0 for x = 0
#'
#' @keywords internal
.safe_log <- function(x) {
  result <- log(x)
  result[x == 0] <- 0
  result[x < 0] <- -Inf
  result
}


#' Compute log-likelihood contribution from counts and probabilities
#'
#' Computes sum of N * log(pi) over all cells.
#'
#' @param counts Array of counts
#' @param probs Array of probabilities (same dimensions as counts)
#'
#' @return Scalar log-likelihood contribution
#'
#' @keywords internal
.loglik_contribution <- function(counts, probs) {
  # N * log(pi), with 0 * log(0) = 0
  sum(counts * .safe_log(probs))
}


#' Extract history for a subject at time k
#'
#' @param y_row Single row of data (one subject)
#' @param k Current time index
#' @param order AD order p
#'
#' @return Integer vector of length min(k-1, order) with category history
#'
#' @keywords internal
.get_history <- function(y_row, k, order) {
  if (k == 1) return(integer(0))
  
  history_length <- min(k - 1, order)
  start_idx <- k - history_length
  
  y_row[start_idx:(k - 1)]
}


#' Convert history to array index
#'
#' @param history Integer vector of category values
#' @param n_categories Number of categories
#'
#' @return Integer index (1-based) into flattened array
#'
#' @keywords internal
.history_to_index <- function(history, n_categories) {
  if (length(history) == 0) return(1L)
  
  # history[1] is earliest, history[end] is most recent
  # Treat as base-c number
  idx <- 1L
  multiplier <- 1L
  for (i in seq_along(history)) {
    idx <- idx + (history[i] - 1L) * multiplier
    multiplier <- multiplier * n_categories
  }
  idx
}


#' Print method for cat_fit objects
#'
#' @param x A cat_fit object
#' @param ... Additional arguments (ignored)
#' @return Invisibly returns \code{x}.
#' @export
print.cat_fit <- function(x, ...) {
  cat("Categorical Antedependence Model Fit\n")
  cat("====================================\n\n")
  
  cat("Order:", x$settings$order, "\n")
  cat("Categories:", x$settings$n_categories, "\n")
  cat("Time points:", x$settings$n_time, "\n")
  cat("Subjects:", x$settings$n_subjects, "\n")
  
  if (!is.null(x$settings$blocks) && !x$settings$homogeneous) {
    cat("Groups:", length(unique(x$settings$blocks)), "(heterogeneous)\n")
  } else if (!is.null(x$settings$blocks)) {
    cat("Groups:", length(unique(x$settings$blocks)), "(homogeneous)\n")
  }
  
  cat("\nLog-likelihood:", round(x$log_l, 4), "\n")
  cat("AIC:", round(x$aic, 4), "\n")
  cat("BIC:", round(x$bic, 4), "\n")
  cat("Parameters:", x$n_params, "\n")
  if (!is.null(x$n_missing)) {
    cat("Missing values:", x$n_missing, "\n")
  }
  if (!is.null(x$convergence)) {
    cat("Convergence code:", x$convergence, "\n")
  }
  
  invisible(x)
}

# test-cat.R - Tests for categorical antedependence module

# ============================================================
# Test 1: Basic validation
# ============================================================
test_that("input validation works correctly", {
  skip_on_cran()
  # Valid data
  y_valid <- matrix(c(1, 2, 1, 2,
                      2, 1, 1, 1,
                      1, 1, 2, 2), nrow = 3, byrow = TRUE)
  
  validated <- antedep:::.validate_y_cat(y_valid)
  expect_equal(validated$n_categories, 2)
  expect_equal(dim(validated$y), c(3, 4))
  
  # Override n_categories
  validated2 <- antedep:::.validate_y_cat(y_valid, n_categories = 3)
  expect_equal(validated2$n_categories, 3)
  
  # Invalid: non-integer
  expect_error(antedep:::.validate_y_cat(matrix(c(1.5, 2), nrow = 1)))
  
  # Invalid: values < 1
  expect_error(antedep:::.validate_y_cat(matrix(c(0, 1, 2), nrow = 1)))
})


test_that("parameter counting is correct", {
  skip_on_cran()
  # Order 0: (c-1) * n = 1 * 4 = 4
  n_params_0 <- antedep:::.count_params_cat(order = 0, n_categories = 2, n_time = 4)
  expect_equal(n_params_0, 4)
  
  # Order 1: (c-1) * [1 + (n-1)*c] = 1 * [1 + 3*2] = 7
  n_params_1 <- antedep:::.count_params_cat(order = 1, n_categories = 2, n_time = 4)
  expect_equal(n_params_1, 7)
  
  # Order 2: (c-1) * [1 + c + (n-2)*c^2] = 1 * [1 + 2 + 2*4] = 11
  n_params_2 <- antedep:::.count_params_cat(order = 2, n_categories = 2, n_time = 4)
  expect_equal(n_params_2, 11)
  
  # With 3 categories, order 1, 5 time points
  # (c-1) * [1 + (n-1)*c] = 2 * [1 + 4*3] = 2 * 13 = 26
  n_params_3cat <- antedep:::.count_params_cat(order = 1, n_categories = 3, n_time = 5)
  expect_equal(n_params_3cat, 26)
})


# ============================================================
# Test 2: Cell counting
# ============================================================
test_that("cell counting works correctly", {
  skip_on_cran()
  y_test <- matrix(c(1, 1, 2,
                     1, 2, 1,
                     2, 1, 1,
                     1, 1, 1), nrow = 4, byrow = TRUE)
  
  # Count at time 1
  counts_t1 <- antedep:::.count_cells_table_cat(y_test, 1, n_categories = 2)
  expect_equal(counts_t1[1], 3)  # Three 1's
  expect_equal(counts_t1[2], 1)  # One 2
  
  # Count pairs (t1, t2)
  counts_t12 <- antedep:::.count_cells_table_cat(y_test, c(1, 2), n_categories = 2)
  # (1,1): subjects 1, 4 -> 2
  # (1,2): subject 2 -> 1
  # (2,1): subject 3 -> 1
  # (2,2): none -> 0
  expect_equal(counts_t12[1, 1], 2)
  expect_equal(counts_t12[1, 2], 1)
  expect_equal(counts_t12[2, 1], 1)
  expect_equal(counts_t12[2, 2], 0)
})


# ============================================================
# Test 3: Transition probability computation
# ============================================================
test_that("transition probability computation is correct", {
  skip_on_cran()
  y_test <- matrix(c(1, 1, 2,
                     1, 2, 1,
                     2, 1, 1,
                     1, 1, 1), nrow = 4, byrow = TRUE)
  
  counts_t12 <- antedep:::.count_cells_table_cat(y_test, c(1, 2), n_categories = 2)
  trans_probs <- antedep:::.counts_to_transition_probs(counts_t12)
  
  # P(Y2=1|Y1=1) = 2/3, P(Y2=2|Y1=1) = 1/3
  # P(Y2=1|Y1=2) = 1/1 = 1, P(Y2=2|Y1=2) = 0/1 = 0
  expect_equal(trans_probs[1, 1], 2/3, tolerance = 1e-10)
  expect_equal(trans_probs[1, 2], 1/3, tolerance = 1e-10)
  expect_equal(trans_probs[2, 1], 1, tolerance = 1e-10)
  expect_equal(trans_probs[2, 2], 0, tolerance = 1e-10)
})


# ============================================================
# Test 4: fit_cat basic functionality
# ============================================================
test_that("fit_cat works for order 0", {
  skip_on_cran()
  set.seed(42)
  y_sim <- matrix(sample(1:2, 100 * 5, replace = TRUE), nrow = 100, ncol = 5)
  
  fit0 <- fit_cat(y_sim, order = 0)
  expect_s3_class(fit0, "cat_fit")
  expect_true(is.finite(fit0$log_l))
  expect_true(fit0$log_l < 0)
  expect_equal(fit0$n_params, 5)  # (c-1)*n = 1*5 = 5
  expect_equal(fit0$n_obs, length(y_sim))
  expect_equal(fit0$n_missing, 0)
})


test_that("fit_cat works for order 1", {
  skip_on_cran()
  set.seed(42)
  y_sim <- matrix(sample(1:2, 100 * 5, replace = TRUE), nrow = 100, ncol = 5)
  
  fit0 <- fit_cat(y_sim, order = 0)
  fit1 <- fit_cat(y_sim, order = 1)
  
  expect_true(is.finite(fit1$log_l))
  expect_true(fit1$log_l >= fit0$log_l)  # Higher order fits at least as well
  expect_equal(fit1$n_params, 9)  # 1 + 4*2 = 9
})


test_that("fit_cat works for order 2", {
  skip_on_cran()
  set.seed(42)
  y_sim <- matrix(sample(1:2, 100 * 5, replace = TRUE), nrow = 100, ncol = 5)
  
  fit1 <- fit_cat(y_sim, order = 1)
  fit2 <- fit_cat(y_sim, order = 2)
  
  expect_true(is.finite(fit2$log_l))
  expect_true(fit2$log_l >= fit1$log_l)
})


test_that("fit_cat missing-data modes behave correctly", {
  skip_on_cran()
  set.seed(43)
  y <- simulate_cat(50, 4, order = 1, n_categories = 2)
  y_miss <- y
  y_miss[1, 2] <- NA
  y_miss[4, 4] <- NA

  expect_error(fit_cat(y_miss, order = 1, na_action = "fail"), "Missing data")

  keep <- stats::complete.cases(y_miss)
  fit_complete_mode <- fit_cat(y_miss, order = 1, na_action = "complete")
  fit_complete_ref <- fit_cat(y_miss[keep, , drop = FALSE], order = 1)
  expect_equal(as.numeric(fit_complete_mode$log_l), as.numeric(fit_complete_ref$log_l), tolerance = 1e-10)
  expect_equal(fit_complete_mode$n_missing, 0)

  # With no missing values, marginalize should reduce to complete-data fit.
  fit_full <- fit_cat(y, order = 1)
  fit_marg_no_missing <- fit_cat(y, order = 1, na_action = "marginalize")
  expect_equal(as.numeric(fit_marg_no_missing$log_l), as.numeric(fit_full$log_l), tolerance = 1e-10)
  expect_equal(fit_marg_no_missing$settings$na_action_effective, "complete")
  expect_equal(fit_marg_no_missing$n_missing, 0)

  # EM entry point via fit_cat should be available for order 1.
  fit_em_mode <- fit_cat(y_miss, order = 1, na_action = "em", em_max_iter = 10)
  fit_em_direct <- em_cat(y_miss, order = 1, max_iter = 10)
  expect_equal(as.numeric(fit_em_mode$log_l), as.numeric(fit_em_direct$log_l), tolerance = 1e-8)
  expect_equal(fit_em_mode$settings$na_action, "em")
  expect_equal(fit_em_mode$settings$method, fit_em_direct$settings$method)
})


test_that("fit_cat marginalize handles missingness for order 2", {
  skip_on_cran()
  set.seed(44)
  y <- simulate_cat(60, 5, order = 2, n_categories = 2)
  
  # Create mixed monotone/intermittent missingness.
  y_miss <- y
  dropout_time <- sample(3:5, nrow(y_miss), replace = TRUE)
  dropout_flag <- runif(nrow(y_miss)) < 0.25
  for (i in seq_len(nrow(y_miss))) {
    if (dropout_flag[i]) {
      y_miss[i, dropout_time[i]:ncol(y_miss)] <- NA
    }
  }
  mask <- matrix(runif(length(y_miss)) < 0.08, nrow = nrow(y_miss))
  y_miss[mask] <- NA
  
  fit_miss <- fit_cat(y_miss, order = 2, na_action = "marginalize")
  ll_miss <- logL_cat(
    y_miss, order = 2, marginal = fit_miss$marginal, transition = fit_miss$transition,
    na_action = "marginalize"
  )
  
  expect_true(is.finite(fit_miss$log_l))
  expect_true(is.finite(ll_miss))

  expect_error(
    fit_cat(y_miss, order = 2, na_action = "em"),
    "currently supports order 0 and 1"
  )
})


# ============================================================
# Test 5: simulate_cat functionality
# ============================================================
test_that("simulate_cat produces valid output with defaults", {
  skip_on_cran()
  set.seed(123)
  y_uniform <- simulate_cat(n_subjects = 50, n_time = 4, order = 1, n_categories = 3)
  
  expect_equal(nrow(y_uniform), 50)
  expect_equal(ncol(y_uniform), 4)
  expect_true(all(y_uniform >= 1 & y_uniform <= 3))
})


test_that("simulate_cat works with custom parameters", {
  skip_on_cran()
  marginal_custom <- list(t1 = c(0.7, 0.3))
  transition_custom <- list(
    t2 = matrix(c(0.9, 0.1, 0.2, 0.8), nrow = 2, byrow = TRUE),
    t3 = matrix(c(0.9, 0.1, 0.2, 0.8), nrow = 2, byrow = TRUE),
    t4 = matrix(c(0.9, 0.1, 0.2, 0.8), nrow = 2, byrow = TRUE)
  )

  set.seed(456)
  y_custom <- simulate_cat(400, 4, order = 1, n_categories = 2,
                           marginal = marginal_custom,
                           transition = transition_custom)

  # Check that marginal of Y1 is approximately (0.7, 0.3)
  prop_y1 <- table(y_custom[, 1]) / nrow(y_custom)
  expect_equal(unname(prop_y1[1]), 0.7, tolerance = 0.08)
})


# ============================================================
# Test 6: logL_cat consistency with fit_cat
# ============================================================
test_that("logL_cat is consistent with fit_cat", {
  skip_on_cran()
  set.seed(42)
  y_sim <- matrix(sample(1:2, 100 * 5, replace = TRUE), nrow = 100, ncol = 5)
  
  fit_test <- fit_cat(y_sim, order = 1)
  ll_check <- logL_cat(y_sim, order = 1, 
                       marginal = fit_test$marginal, 
                       transition = fit_test$transition)
  
  # Use unname to ignore name attributes
  expect_equal(unname(ll_check), unname(fit_test$log_l), tolerance = 1e-8)
})


# ============================================================
# Test 7: BIC comparison
# ============================================================
test_that("bic_order_cat compares models correctly", {
  skip_on_cran()
  set.seed(42)
  y_sim <- matrix(sample(1:2, 100 * 5, replace = TRUE), nrow = 100, ncol = 5)
  
  result <- bic_order_cat(y_sim, max_order = 2)
  
  expect_equal(nrow(result$table), 3)  # Orders 0, 1, 2
  expect_true(all(c("order", "log_l", "n_params", "aic", "bic") %in% names(result$table)))
  expect_true(result$best_order %in% 0:2)
})


# ============================================================
# Test 8: Simulation-estimation consistency
# ============================================================
test_that("simulation-estimation is consistent", {
  skip_on_cran()
  set.seed(789)
  true_marginal <- list(t1 = c(0.6, 0.4))
  true_transition <- list(
    t2 = matrix(c(0.8, 0.2, 0.3, 0.7), nrow = 2, byrow = TRUE),
    t3 = matrix(c(0.8, 0.2, 0.3, 0.7), nrow = 2, byrow = TRUE),
    t4 = matrix(c(0.8, 0.2, 0.3, 0.7), nrow = 2, byrow = TRUE),
    t5 = matrix(c(0.8, 0.2, 0.3, 0.7), nrow = 2, byrow = TRUE)
  )

  y_large <- simulate_cat(5000, 5, order = 1, n_categories = 2,
                          marginal = true_marginal,
                          transition = true_transition)

  fit_large <- fit_cat(y_large, order = 1)

  # Check marginal recovery - use unname to strip names
  est_marg <- unname(fit_large$marginal$t1)
  expect_equal(est_marg, true_marginal$t1, tolerance = 0.03)

  # Check transition recovery - use unname to strip dimnames
  est_trans <- unname(fit_large$transition$t2)
  expect_equal(est_trans, true_transition$t2, tolerance = 0.03)
})


# ============================================================
# Test 9: Blocks (heterogeneous)
# ============================================================
test_that("block handling works for homogeneous model", {
  skip_on_cran()
  set.seed(321)
  marginal_g1 <- list(t1 = c(0.8, 0.2))
  marginal_g2 <- list(t1 = c(0.3, 0.7))
  trans_g1 <- list(
    t2 = matrix(c(0.9, 0.1, 0.1, 0.9), nrow = 2, byrow = TRUE),
    t3 = matrix(c(0.9, 0.1, 0.1, 0.9), nrow = 2, byrow = TRUE)
  )
  trans_g2 <- list(
    t2 = matrix(c(0.5, 0.5, 0.5, 0.5), nrow = 2, byrow = TRUE),
    t3 = matrix(c(0.5, 0.5, 0.5, 0.5), nrow = 2, byrow = TRUE)
  )

  y_g1 <- simulate_cat(60, 3, order = 1, n_categories = 2,
                       marginal = marginal_g1, transition = trans_g1)
  y_g2 <- simulate_cat(60, 3, order = 1, n_categories = 2,
                       marginal = marginal_g2, transition = trans_g2)

  y_combined <- rbind(y_g1, y_g2)
  blocks_vec <- c(rep(1, 60), rep(2, 60))

  # Fit homogeneous model
  fit_homo <- fit_cat(y_combined, order = 1, blocks = blocks_vec, homogeneous = TRUE)
  expect_true(is.finite(fit_homo$log_l))
})


test_that("block handling works for heterogeneous model", {
  skip_on_cran()
  set.seed(321)
  marginal_g1 <- list(t1 = c(0.8, 0.2))
  marginal_g2 <- list(t1 = c(0.3, 0.7))
  trans_g1 <- list(
    t2 = matrix(c(0.9, 0.1, 0.1, 0.9), nrow = 2, byrow = TRUE),
    t3 = matrix(c(0.9, 0.1, 0.1, 0.9), nrow = 2, byrow = TRUE)
  )
  trans_g2 <- list(
    t2 = matrix(c(0.5, 0.5, 0.5, 0.5), nrow = 2, byrow = TRUE),
    t3 = matrix(c(0.5, 0.5, 0.5, 0.5), nrow = 2, byrow = TRUE)
  )

  y_g1 <- simulate_cat(60, 3, order = 1, n_categories = 2,
                       marginal = marginal_g1, transition = trans_g1)
  y_g2 <- simulate_cat(60, 3, order = 1, n_categories = 2,
                       marginal = marginal_g2, transition = trans_g2)

  y_combined <- rbind(y_g1, y_g2)
  blocks_vec <- c(rep(1, 60), rep(2, 60))

  # Fit heterogeneous model
  fit_homo <- fit_cat(y_combined, order = 1, blocks = blocks_vec, homogeneous = TRUE)
  fit_hetero <- fit_cat(y_combined, order = 1, blocks = blocks_vec, homogeneous = FALSE)

  # Heterogeneous should fit better
  expect_true(fit_hetero$log_l > fit_homo$log_l)
  expect_equal(sort(fit_hetero$settings$block_levels), c("1", "2"))

  # Check parameter separation - block 1 has higher category-1 probability
  # than block 2 (true values 0.8 vs 0.3), direction should be correct
  est_g1_t1 <- unname(fit_hetero$marginal$block_1$t1)
  est_g2_t1 <- unname(fit_hetero$marginal$block_2$t1)
  expect_true(est_g1_t1[1] > est_g2_t1[1])
})

test_that("fit_cat preserves original block labels in settings", {
  skip_on_cran()
  set.seed(654)
  y <- simulate_cat(120, 4, order = 1, n_categories = 2)
  blocks <- rep(c(5, 2), each = 60)

  fit <- fit_cat(y, order = 1, blocks = blocks, homogeneous = FALSE)

  expect_equal(sort(fit$settings$block_levels), c("2", "5"))
  expect_equal(length(fit$marginal), 2)
})


# ============================================================
# Test 10: Order 2 model
# ============================================================
test_that("order 2 model works correctly", {
  skip_on_cran()
  set.seed(999)
  marginal_o2 <- list(
    t1 = c(0.5, 0.5),
    t2_given_1to1 = matrix(c(0.6, 0.4, 0.4, 0.6), nrow = 2, byrow = TRUE)
  )

  # For order 2, transition depends on (t-2, t-1)
  # Array dimensions: [y_{t-2}, y_{t-1}, y_t]
  trans_o2_single <- array(c(
    0.8, 0.2,  # y_{t-2}=1, y_{t-1}=1
    0.3, 0.7,  # y_{t-2}=1, y_{t-1}=2
    0.6, 0.4,  # y_{t-2}=2, y_{t-1}=1
    0.4, 0.6   # y_{t-2}=2, y_{t-1}=2
  ), dim = c(2, 2, 2))

  trans_o2 <- list(t3 = trans_o2_single, t4 = trans_o2_single, t5 = trans_o2_single)

  y_o2 <- simulate_cat(150, 5, order = 2, n_categories = 2,
                       marginal = marginal_o2, transition = trans_o2)

  fit_o2 <- fit_cat(y_o2, order = 2)
  expect_true(is.finite(fit_o2$log_l))

  # Compare with order 1 (order 2 should fit better on true AD(2) data)
  fit_o1_on_o2 <- fit_cat(y_o2, order = 1)
  expect_true(fit_o2$log_l > fit_o1_on_o2$log_l)
})

#' EM algorithm for Gaussian AD model estimation
#'
#' Convenience wrapper around \code{\link{fit_gau}} with
#' \code{na_action = "em"} to provide a parallel entry point to
#' \code{\link{em_inad}}.
#'
#' @param y Numeric matrix (n_subjects x n_time), may contain NA.
#' @param order Integer 0, 1, or 2.
#' @param blocks Optional vector of block membership (length n_subjects).
#' @param estimate_mu Logical, whether to estimate mu (default TRUE).
#' @param max_iter Maximum EM iterations.
#' @param tol EM convergence tolerance.
#' @param verbose Logical, print EM progress.
#' @param ... Additional arguments passed to \code{\link{fit_gau}}.
#'
#' @return An \code{gau_fit} object as returned by \code{\link{fit_gau}}.
#'
#' @details
#' This is an alias-style helper for users who prefer explicit \code{em_*}
#' entry points across model families.
#'
#' @examples
#' set.seed(1)
#' y <- simulate_gau(n_subjects = 35, n_time = 5, order = 1, phi = 0.3)
#' y[sample(length(y), 8)] <- NA
#' fit <- em_gau(y, order = 1, max_iter = 20, tol = 1e-5)
#' fit$settings$na_action
#'
#' @seealso \code{\link{fit_gau}}, \code{\link{em_inad}}, \code{\link{em_cat}},
#'   \code{\link{fit_cat}}
#' @export
em_gau <- function(y, order = 1, blocks = NULL, estimate_mu = TRUE,
                  max_iter = 100, tol = 1e-6, verbose = FALSE, ...) {
  fit_gau(
    y = y,
    order = order,
    blocks = blocks,
    na_action = "em",
    estimate_mu = estimate_mu,
    em_max_iter = max_iter,
    em_tol = tol,
    em_verbose = verbose,
    ...
  )
}

#' Simulate Gaussian antedependence series
#'
#' Generate longitudinal continuous data from a Gaussian antedependence (AD)
#' model of order 0, 1, or 2 using a conditional regression on predecessors.
#'
#' For `order = 0`, each time point is generated independently as
#' `Y[, t] = mu[t] + tau[block] + eps`, with `eps ~ N(0, sigma[t]^2)`.
#'
#' For `order = 1`, for `t >= 2`:
#' `Y[, t] = m_t + phi[t] * (Y[, t - 1] - m_{t-1}) + eps_t`,
#' where `m_t = mu[t] + tau[block]` and `eps_t ~ N(0, sigma[t]^2)`.
#'
#' For `order = 2`, for `t >= 3`:
#' `Y[, t] = m_t + phi[1, t] * (Y[, t - 1] - m_{t-1}) +
#'                 phi[2, t] * (Y[, t - 2] - m_{t-2}) + eps_t`.
#'
#' If `blocks` is provided, each subject s belongs to a block and receives a
#' mean shift `tau[blocks[s]]`. `tau[1]` is forced to 0.
#'
#' @param n_subjects number of subjects
#' @param n_time number of time points
#' @param order antedependence order, 0, 1 or 2
#' @param mu mean parameter; `NULL`, scalar, or length `n_time`
#' @param phi dependence parameter; ignored when `order = 0`.
#'   For `order = 1`, `NULL`, scalar, or length `n_time`.
#'   For `order = 2`, `NULL` or a 2 by `n_time` matrix.
#' @param sigma innovation standard deviation; `NULL`, scalar, or length `n_time`
#' @param blocks integer vector of length `n_subjects` indicating block membership
#'   for each subject; if `NULL`, no block effect is applied
#' @param tau group effect vector indexed by block; `tau[1]` is forced to 0.
#'   If scalar x, it is expanded to `c(0, x, ..., x)` with length equal to the
#'   number of blocks
#' @param seed optional random seed for reproducibility
#'
#' @return numeric matrix with dimension `n_subjects` by `n_time`
#'
#' @examples
#' y <- simulate_gau(
#'   n_subjects = 20,
#'   n_time = 6,
#'   order = 1,
#'   phi = 0.4,
#'   seed = 42
#' )
#' dim(y)
#' @export
simulate_gau <- function(n_subjects,
                        n_time,
                        order = 1L,
                        mu = NULL,
                        phi = NULL,
                        sigma = NULL,
                        blocks = NULL,
                        tau = 0,
                        seed = NULL) {

    if (!is.null(seed)) set.seed(seed)

    if (!order %in% c(0L, 1L, 2L)) {
        stop("order must be 0, 1 or 2")
    }

    if (length(n_subjects) != 1L || n_subjects <= 0) {
        stop("n_subjects must be a positive integer")
    }
    if (length(n_time) != 1L || n_time <= 0) {
        stop("n_time must be a positive integer")
    }

    n_subjects <- as.integer(n_subjects)
    n_time <- as.integer(n_time)
    order <- as.integer(order)

    if (!is.null(blocks)) {
        if (length(blocks) != n_subjects) {
            stop("blocks must have length n_subjects")
        }
        if (any(is.na(blocks))) {
            stop("blocks must have no missing values")
        }
        blocks <- as.integer(factor(blocks))
        B <- max(blocks)
        if (length(tau) == 1L) {
            tau <- c(0, rep(as.numeric(tau), max(B - 1L, 0L)))
        } else {
            tau <- as.numeric(tau)
            if (length(tau) != B) {
                stop("when blocks is provided, tau must be scalar or have length equal to max(blocks)")
            }
            tau[1L] <- 0
        }
    } else {
        B <- 1L
        blocks <- rep(1L, n_subjects)
        tau <- 0
    }

    if (is.null(mu)) {
        mu <- rep(0, n_time)
    } else if (length(mu) == 1L) {
        mu <- rep(as.numeric(mu), n_time)
    } else {
        mu <- as.numeric(mu)
    }
    if (length(mu) != n_time) {
        stop("mu must be NULL, scalar, or length n_time")
    }

    if (is.null(sigma)) {
        sigma <- rep(1, n_time)
    } else if (length(sigma) == 1L) {
        sigma <- rep(as.numeric(sigma), n_time)
    } else {
        sigma <- as.numeric(sigma)
    }
    if (length(sigma) != n_time) {
        stop("sigma must be NULL, scalar, or length n_time")
    }
    if (any(!is.finite(sigma)) || any(sigma <= 0)) {
        stop("sigma must be positive and finite")
    }

    if (order == 0L) {
        phi_use <- 0
    } else if (order == 1L) {
        if (is.null(phi)) {
            phi_use <- rep(0.5, n_time)
            phi_use[1L] <- 0
        } else if (length(phi) == 1L) {
            phi_use <- rep(as.numeric(phi), n_time)
        } else {
            phi_use <- as.numeric(phi)
        }
        if (length(phi_use) != n_time) {
            stop("for order 1, phi must be NULL, scalar, or length n_time")
        }
        if (any(!is.finite(phi_use))) {
            stop("phi must be finite")
        }
    } else {
        if (is.null(phi)) {
            phi_use <- matrix(0, nrow = 2L, ncol = n_time)
            if (n_time >= 2L) {
                phi_use[1L, 2:n_time] <- 0.5
            }
            if (n_time >= 3L) {
                phi_use[2L, 3:n_time] <- 0.25
            }
        } else {
            if (!is.matrix(phi) || nrow(phi) != 2L || ncol(phi) != n_time) {
                stop("for order 2, phi must be NULL or a 2 by n_time matrix")
            }
            phi_use <- phi
        }
        if (any(!is.finite(phi_use))) {
            stop("phi must be finite")
        }
    }

    y <- matrix(NA_real_, nrow = n_subjects, ncol = n_time)

    m_at <- function(t_index) {
        mu[t_index] + tau[blocks]
    }

    if (n_time >= 1L) {
        y[, 1L] <- m_at(1L) + stats::rnorm(n_subjects, mean = 0, sd = sigma[1L])
    }

    if (order == 0L) {
        if (n_time >= 2L) {
            for (t in 2L:n_time) {
                y[, t] <- m_at(t) + stats::rnorm(n_subjects, mean = 0, sd = sigma[t])
            }
        }
        return(y)
    }

    if (order == 1L) {
        if (n_time >= 2L) {
            for (t in 2L:n_time) {
                mt <- m_at(t)
                mtm1 <- m_at(t - 1L)
                y[, t] <- mt +
                    phi_use[t] * (y[, t - 1L] - mtm1) +
                    stats::rnorm(n_subjects, mean = 0, sd = sigma[t])
            }
        }
        return(y)
    }

    if (n_time >= 2L) {
        t <- 2L
        mt <- m_at(t)
        mtm1 <- m_at(t - 1L)
        y[, t] <- mt +
            phi_use[1L, t] * (y[, t - 1L] - mtm1) +
            stats::rnorm(n_subjects, mean = 0, sd = sigma[t])
    }

    if (n_time >= 3L) {
        for (t in 3L:n_time) {
            mt <- m_at(t)
            mtm1 <- m_at(t - 1L)
            mtm2 <- m_at(t - 2L)
            y[, t] <- mt +
                phi_use[1L, t] * (y[, t - 1L] - mtm1) +
                phi_use[2L, t] * (y[, t - 2L] - mtm2) +
                stats::rnorm(n_subjects, mean = 0, sd = sigma[t])
        }
    }

    y
}

testthat::test_that("simulate_gau returns a numeric matrix with correct dimensions", {
    skip_on_cran()
    testthat::skip_if_not(exists("simulate_gau"))

    set.seed(1)
    y <- simulate_gau(n_subjects = 10, n_time = 5, order = 1)

    testthat::expect_true(is.matrix(y))
    testthat::expect_type(y, "double")
    testthat::expect_equal(dim(y), c(10, 5))
    testthat::expect_true(all(is.finite(y)))
})

testthat::test_that("simulate_gau is reproducible given the same seed", {
    skip_on_cran()
    testthat::skip_if_not(exists("simulate_gau"))

    set.seed(123)
    y1 <- simulate_gau(20, 6, order = 2)

    set.seed(123)
    y2 <- simulate_gau(20, 6, order = 2)

    testthat::expect_equal(y1, y2)

    y3 <- simulate_gau(20, 6, order = 2, seed = 999)
    y4 <- simulate_gau(20, 6, order = 2, seed = 999)
    testthat::expect_equal(y3, y4)
})

testthat::test_that("simulate_gau works for order 0, 1, 2", {
    skip_on_cran()
    testthat::skip_if_not(exists("simulate_gau"))

    set.seed(2)
    y0 <- simulate_gau(8, 4, order = 0)
    y1 <- simulate_gau(8, 4, order = 1)
    y2 <- simulate_gau(8, 4, order = 2)

    testthat::expect_equal(dim(y0), c(8, 4))
    testthat::expect_equal(dim(y1), c(8, 4))
    testthat::expect_equal(dim(y2), c(8, 4))
})

testthat::test_that("simulate_gau validates key inputs", {
    skip_on_cran()
    testthat::skip_if_not(exists("simulate_gau"))

    testthat::expect_error(simulate_gau(10, 5, order = 3))
    testthat::expect_error(simulate_gau(0, 5))
    testthat::expect_error(simulate_gau(10, 0))

    testthat::expect_error(simulate_gau(10, 5, mu = 1:3))
    testthat::expect_error(simulate_gau(10, 5, sigma = 0))
    testthat::expect_error(simulate_gau(10, 5, sigma = c(1, 1, -1, 1, 1)))

    testthat::expect_error(simulate_gau(10, 5, blocks = 1:9))
    y_nonseq_ok <- simulate_gau(10, 5, blocks = c(rep(1, 9), 0))
    testthat::expect_equal(dim(y_nonseq_ok), c(10, 5))
})

testthat::test_that("simulate_gau enforces phi shape rules for order 2", {
    skip_on_cran()
    testthat::skip_if_not(exists("simulate_gau"))

    testthat::expect_error(simulate_gau(10, 6, order = 2, phi = rep(0.2, 6)))
    testthat::expect_error(simulate_gau(10, 6, order = 2, phi = matrix(0, nrow = 2, ncol = 5)))

    phi_ok <- matrix(0, nrow = 2, ncol = 6)
    phi_ok[1, 2:6] <- 0.4
    phi_ok[2, 3:6] <- 0.1

    set.seed(7)
    y <- simulate_gau(10, 6, order = 2, phi = phi_ok)
    testthat::expect_equal(dim(y), c(10, 6))
})

testthat::test_that("simulate_gau handles blocks and tau as intended for order 0", {
    skip_on_cran()
    testthat::skip_if_not(exists("simulate_gau"))

    blocks <- rep(1:2, each = 5)

    set.seed(10)
    y_scalar_tau <- simulate_gau(
        n_subjects = 10,
        n_time = 3,
        order = 0,
        mu = 0,
        sigma = 1e-10,
        blocks = blocks,
        tau = 2
    )

    d1 <- mean(y_scalar_tau[blocks == 2, 1]) - mean(y_scalar_tau[blocks == 1, 1])
    testthat::expect_equal(d1, 2, tolerance = 1e-6)

    set.seed(11)
    y_vec_tau <- simulate_gau(
        n_subjects = 10,
        n_time = 3,
        order = 0,
        mu = 0,
        sigma = 1e-10,
        blocks = blocks,
        tau = c(999, 2)
    )

    m_block1 <- mean(y_vec_tau[blocks == 1, 1])
    m_block2 <- mean(y_vec_tau[blocks == 2, 1])

    testthat::expect_equal(m_block1, 0, tolerance = 1e-6)
    testthat::expect_equal(m_block2, 2, tolerance = 1e-6)

    testthat::expect_error(simulate_gau(
        n_subjects = 10,
        n_time = 3,
        order = 0,
        blocks = blocks,
        tau = c(0, 1, 2)
    ))

    blocks_nonseq <- c(rep(2, 5), rep(5, 5))
    set.seed(12)
    y_nonseq <- simulate_gau(
        n_subjects = 10,
        n_time = 3,
        order = 0,
        mu = 0,
        sigma = 1e-10,
        blocks = blocks_nonseq,
        tau = c(0, 2)
    )
    d_nonseq <- mean(y_nonseq[blocks_nonseq == 5, 1]) - mean(y_nonseq[blocks_nonseq == 2, 1])
    testthat::expect_equal(d_nonseq, 2, tolerance = 1e-6)
})

# File: R/test_stationarity_gau.R
# Likelihood ratio tests for Gaussian AD stationarity constraints

#' Likelihood ratio test for stationarity (Gaussian AD data)
#'
#' Tests whether time-varying Gaussian AD covariance parameters can be
#' constrained to be constant over time.
#'
#' @param y Numeric matrix with n_subjects rows and n_time columns.
#' @param order Antedependence order (0, 1, or 2).
#' @param blocks Optional vector of block memberships (length n_subjects).
#' @param constrain Constraint to test:
#'   for order 0: "sigma" (or "all");
#'   for order 1: "phi", "sigma", or "both"/"all";
#'   for order 2: "phi1", "phi2", "phi", "sigma", or "all"/"both".
#' @param fit_unconstrained Optional pre-computed unconstrained fit from
#'   \code{\link{fit_gau}}.
#' @param verbose Logical; if TRUE, prints fitting progress.
#' @param max_iter Maximum number of optimization iterations for constrained fit.
#' @param rel_tol Relative tolerance for constrained optimization.
#' @param ... Additional arguments passed to \code{\link{fit_gau}} when
#'   \code{fit_unconstrained} is not provided.
#'
#' @return A list with class \code{"test_stationarity_gau"} containing:
#' \describe{
#'   \item{method}{Inference method used (\code{"lrt"}).}
#'   \item{fit_unconstrained}{Unconstrained Gaussian AD fit}
#'   \item{fit_constrained}{Constrained Gaussian AD fit}
#'   \item{constraint}{Human-readable null constraint description}
#'   \item{lrt_stat}{Likelihood-ratio statistic}
#'   \item{df}{Degrees of freedom}
#'   \item{p_value}{Chi-square p-value}
#'   \item{bic_unconstrained}{BIC of unconstrained model}
#'   \item{bic_constrained}{BIC of constrained model}
#'   \item{bic_selected}{Model selected by BIC}
#'   \item{table}{Two-row model summary table}
#' }
#'
#' @details
#' The mean structure is kept unrestricted in both models (time-specific means
#' plus optional block shifts), and the test constrains covariance parameters:
#' innovation standard deviations and/or antedependence coefficients.
#'
#' The likelihood-ratio statistic is:
#' \deqn{\lambda = 2(\ell_{alt} - \ell_{null})}
#' where \eqn{\ell_{null}} and \eqn{\ell_{alt}} are maximized log-likelihoods
#' under the constrained and unconstrained models, respectively.
#'
#' Degrees of freedom are computed from the number of constraints implied by
#' \code{constrain}.
#'
#' @references
#' Zimmerman, D.L. and Nunez-Anton, V. (2009). Antedependence Models for
#' Longitudinal Data. Chapman & Hall/CRC. Chapter 6.
#'
#' @examples
#' set.seed(1)
#' y <- simulate_gau(n_subjects = 80, n_time = 6, order = 1, phi = 0.4, sigma = 1)
#'
#' # Test jointly constant phi and sigma (order 1)
#' out <- test_stationarity_gau(y, order = 1, constrain = "both")
#' out$p_value
#'
#' @seealso \code{\link{run_stationarity_tests_gau}}, \code{\link{test_order_gau}},
#'   \code{\link{fit_gau}}
#'
#' @export
test_stationarity_gau <- function(y, order = 1L, blocks = NULL, constrain = "both",
                                 fit_unconstrained = NULL, verbose = FALSE,
                                 max_iter = 2000L, rel_tol = 1e-8, ...) {
    if (!is.matrix(y)) y <- as.matrix(y)
    if (anyNA(y)) {
        stop(
            "test_stationarity_gau currently supports complete data only. Missing-data AD likelihood-ratio tests are not implemented yet.",
            call. = FALSE
        )
    }
    if (any(!is.finite(y))) stop("y must contain finite values", call. = FALSE)

    n_subjects <- nrow(y)
    n_time <- ncol(y)

    order <- as.integer(order)
    if (!order %in% c(0L, 1L, 2L)) stop("order must be 0, 1, or 2", call. = FALSE)
    if (order >= n_time) stop("order must be less than n_time", call. = FALSE)

    block_info <- .normalize_blocks(blocks, n_subjects)
    blocks_id <- block_info$blocks_id
    n_blocks <- block_info$n_blocks
    block_levels <- block_info$block_levels
    blocks_fit <- if (is.null(blocks)) NULL else blocks_id

    cinfo <- .parse_stationarity_constraint_gau(constrain, order, n_time)

    fit_args <- list(...)
    if (is.null(fit_unconstrained)) {
        if (verbose) cat("Fitting unconstrained Gaussian AD model...\n")
        fit_unconstrained <- do.call(
            fit_gau,
            c(
                list(
                    y = y,
                    order = order,
                    blocks = blocks_fit,
                    na_action = "fail",
                    estimate_mu = TRUE
                ),
                fit_args
            )
        )
    } else {
        if (!is.list(fit_unconstrained) || is.null(fit_unconstrained$log_l)) {
            stop("fit_unconstrained must be a fit_gau-like object with log_l.", call. = FALSE)
        }
        fit_order <- fit_unconstrained$settings$order
        if (!is.null(fit_order) && as.integer(fit_order) != order) {
            stop("fit_unconstrained$settings$order does not match requested order.", call. = FALSE)
        }
    }

    if (verbose) {
        cat("Fitting constrained Gaussian AD model under H0:", cinfo$description, "\n")
    }
    fit_constrained <- .fit_constrained_stationarity_gau(
        y = y,
        order = order,
        blocks = blocks_fit,
        cinfo = cinfo,
        fit_init = fit_unconstrained,
        block_levels = if (is.null(blocks)) NULL else block_levels,
        max_iter = as.integer(max_iter),
        rel_tol = as.numeric(rel_tol),
        verbose = isTRUE(verbose)
    )

    logL_uncon <- fit_unconstrained$log_l
    logL_con <- fit_constrained$log_l

    lrt_stat <- 2 * (logL_uncon - logL_con)
    if (!is.finite(lrt_stat)) {
        lrt_stat <- NA_real_
    } else if (lrt_stat < 0) {
        if (lrt_stat > -1e-6) {
            lrt_stat <- 0
        } else {
            warning(
                "LRT statistic was negative. Returning zero (numerical tolerance).",
                call. = FALSE
            )
            lrt_stat <- 0
        }
    }

    df <- as.integer(cinfo$df)
    p_value <- if (is.finite(lrt_stat)) {
        stats::pchisq(lrt_stat, df = df, lower.tail = FALSE)
    } else {
        NA_real_
    }

    n_mean_params <- n_time + max(n_blocks - 1L, 0L)
    n_cov_uncon <- .count_params_gau(order, n_time, n_blocks = 1L, estimate_mu = FALSE)
    n_params_uncon <- n_mean_params + n_cov_uncon
    n_params_con <- n_params_uncon - df

    bic_uncon <- -2 * logL_uncon + n_params_uncon * log(n_subjects)
    bic_con <- -2 * logL_con + n_params_con * log(n_subjects)
    bic_selected <- if (bic_uncon <= bic_con) "unconstrained" else "constrained"

    table_df <- data.frame(
        model = c(
            "Unconstrained",
            paste0("Constrained: ", cinfo$description)
        ),
        logLik = c(logL_uncon, logL_con),
        n_params = c(n_params_uncon, n_params_con),
        BIC = c(bic_uncon, bic_con),
        stringsAsFactors = FALSE
    )

    out <- list(
        method = "lrt",
        fit_unconstrained = fit_unconstrained,
        fit_constrained = fit_constrained,
        constraint = cinfo$description,
        lrt_stat = lrt_stat,
        statistic = lrt_stat,
        df = df,
        p_value = p_value,
        bic_unconstrained = bic_uncon,
        bic_constrained = bic_con,
        bic_selected = bic_selected,
        table = table_df,
        settings = list(
            order = order,
            n_subjects = n_subjects,
            n_time = n_time,
            n_blocks = n_blocks,
            constrain = constrain,
            blocks = blocks_fit,
            block_levels = if (is.null(blocks)) NULL else block_levels,
            max_iter = as.integer(max_iter),
            rel_tol = as.numeric(rel_tol)
        )
    )

    class(out) <- "test_stationarity_gau"
    out
}


#' Run all stationarity-related tests for Gaussian AD
#'
#' Runs a standard set of stationarity constraints for Gaussian AD models.
#'
#' @param y Numeric matrix with n_subjects rows and n_time columns.
#' @param order Antedependence order (0, 1, or 2).
#' @param blocks Optional vector of block memberships (length n_subjects).
#' @param verbose Logical; if TRUE, prints progress.
#' @param max_iter Maximum number of optimization iterations for constrained fits.
#' @param rel_tol Relative tolerance for constrained optimization.
#' @param ... Additional arguments passed to \code{\link{fit_gau}} for the
#'   unconstrained fit.
#'
#' @return A list with class \code{"stationarity_tests_gau"} containing:
#' \describe{
#'   \item{fit_unconstrained}{Unconstrained Gaussian AD fit}
#'   \item{tests}{Named list of \code{\link{test_stationarity_gau}} results}
#'   \item{summary}{Summary table of all constraints}
#' }
#'
#' @examples
#' set.seed(1)
#' y <- simulate_gau(n_subjects = 80, n_time = 6, order = 1, phi = 0.4, sigma = 1)
#' out <- run_stationarity_tests_gau(y, order = 1, verbose = FALSE)
#' out$summary
#'
#' @seealso \code{\link{test_stationarity_gau}}
#'
#' @export
run_stationarity_tests_gau <- function(y, order = 1L, blocks = NULL, verbose = FALSE,
                                       max_iter = 2000L, rel_tol = 1e-8, ...) {
    if (!is.matrix(y)) y <- as.matrix(y)
    if (anyNA(y)) {
        stop(
            "run_stationarity_tests_gau currently supports complete data only. Missing-data AD likelihood-ratio tests are not implemented yet.",
            call. = FALSE
        )
    }

    n_subjects <- nrow(y)
    n_time <- ncol(y)

    order <- as.integer(order)
    if (!order %in% c(0L, 1L, 2L)) stop("order must be 0, 1, or 2", call. = FALSE)
    if (order >= n_time) stop("order must be less than n_time", call. = FALSE)

    block_info <- .normalize_blocks(blocks, n_subjects)
    blocks_fit <- if (is.null(blocks)) NULL else block_info$blocks_id

    fit_args <- list(...)
    if (verbose) cat("Fitting unconstrained Gaussian AD model...\n")
    fit_uncon <- do.call(
        fit_gau,
        c(
            list(
                y = y,
                order = order,
                blocks = blocks_fit,
                na_action = "fail",
                estimate_mu = TRUE
            ),
            fit_args
        )
    )

    tests_to_run <- if (order == 0L) {
        c("sigma")
    } else if (order == 1L) {
        c("phi", "sigma", "both")
    } else {
        c("phi1", "phi2", "phi", "sigma", "all")
    }

    results <- vector("list", length(tests_to_run))
    names(results) <- tests_to_run

    for (name in tests_to_run) {
        if (verbose) cat("Testing constraint:", name, "\n")
        results[[name]] <- test_stationarity_gau(
            y = y,
            order = order,
            blocks = blocks_fit,
            constrain = name,
            fit_unconstrained = fit_uncon,
            verbose = FALSE,
            max_iter = max_iter,
            rel_tol = rel_tol
        )
    }

    summary_df <- data.frame(
        constraint = tests_to_run,
        method = vapply(results, function(x) x$method, character(1)),
        df = vapply(results, function(x) x$df, numeric(1)),
        LRT = vapply(results, function(x) x$lrt_stat, numeric(1)),
        p_value = vapply(results, function(x) x$p_value, numeric(1)),
        BIC_selected = vapply(results, function(x) x$bic_selected, character(1)),
        stringsAsFactors = FALSE
    )

    out <- list(
        method = "lrt",
        fit_unconstrained = fit_uncon,
        tests = results,
        summary = summary_df,
        settings = list(
            order = order,
            n_subjects = n_subjects,
            n_time = n_time,
            n_blocks = block_info$n_blocks,
            blocks = blocks_fit
        )
    )
    class(out) <- "stationarity_tests_gau"
    out
}


#' @export
print.test_stationarity_gau <- function(x, digits = 4, ...) {
    cat("\nLikelihood Ratio Test for Gaussian AD Stationarity\n")
    cat("==================================================\n\n")
    cat("Constraint: H0:", x$constraint, "\n\n")
    print(x$table, row.names = FALSE, digits = digits)
    cat(
        "\nLRT:", round(x$lrt_stat, digits),
        " df:", x$df,
        " p-value:", format.pval(x$p_value, digits = digits)
    )
    cat("\nBIC selects:", x$bic_selected, "\n")
    invisible(x)
}


#' @export
print.stationarity_tests_gau <- function(x, digits = 4, ...) {
    cat("\nStationarity Tests for Gaussian AD Model\n")
    cat("========================================\n\n")
    print(x$summary, row.names = FALSE, digits = digits)
    invisible(x)
}


#' @keywords internal
.parse_stationarity_constraint_gau <- function(constrain, order, n_time) {
    constrain <- tolower(as.character(constrain))
    if (length(constrain) == 0L || all(is.na(constrain))) {
        stop("constrain must be provided.", call. = FALSE)
    }

    if (order == 0L) {
        sigma_const <- any(constrain %in% c("sigma", "all", "both"))
        if (!sigma_const) {
            stop("For order 0, constrain must be 'sigma' (or 'all').", call. = FALSE)
        }
        df <- n_time - 1L
        return(list(
            order = 0L,
            sigma_const = TRUE,
            df = df,
            description = "sigma constant over time"
        ))
    }

    if (order == 1L) {
        if (length(constrain) == 1L) {
            if (constrain %in% c("both", "all")) {
                phi_const <- TRUE
                sigma_const <- TRUE
            } else if (constrain == "phi") {
                phi_const <- TRUE
                sigma_const <- FALSE
            } else if (constrain == "sigma") {
                phi_const <- FALSE
                sigma_const <- TRUE
            } else {
                stop("Invalid constraint for order 1. Use 'phi', 'sigma', or 'both'.", call. = FALSE)
            }
        } else {
            phi_const <- "phi" %in% constrain
            sigma_const <- "sigma" %in% constrain
            if ("both" %in% constrain || "all" %in% constrain) {
                phi_const <- TRUE
                sigma_const <- TRUE
            }
        }

        df <- 0L
        if (phi_const) df <- df + (n_time - 2L)
        if (sigma_const) df <- df + (n_time - 1L)
        if (df <= 0L) {
            stop("Selected constraint does not impose restrictions (df = 0).", call. = FALSE)
        }

        desc <- paste(
            c(
                if (phi_const) "phi constant over time",
                if (sigma_const) "sigma constant over time"
            ),
            collapse = ", "
        )
        return(list(
            order = 1L,
            phi_const = phi_const,
            sigma_const = sigma_const,
            df = df,
            description = desc
        ))
    }

    if (length(constrain) == 1L) {
        if (constrain %in% c("all", "both")) {
            phi1_const <- TRUE
            phi2_const <- TRUE
            sigma_const <- TRUE
        } else if (constrain == "phi") {
            phi1_const <- TRUE
            phi2_const <- TRUE
            sigma_const <- FALSE
        } else if (constrain == "phi1") {
            phi1_const <- TRUE
            phi2_const <- FALSE
            sigma_const <- FALSE
        } else if (constrain == "phi2") {
            phi1_const <- FALSE
            phi2_const <- TRUE
            sigma_const <- FALSE
        } else if (constrain == "sigma") {
            phi1_const <- FALSE
            phi2_const <- FALSE
            sigma_const <- TRUE
        } else {
            stop(
                "Invalid constraint for order 2. Use 'phi1', 'phi2', 'phi', 'sigma', or 'all'.",
                call. = FALSE
            )
        }
    } else {
        phi1_const <- "phi1" %in% constrain
        phi2_const <- "phi2" %in% constrain
        sigma_const <- "sigma" %in% constrain
        if ("phi" %in% constrain) {
            phi1_const <- TRUE
            phi2_const <- TRUE
        }
        if ("all" %in% constrain || "both" %in% constrain) {
            phi1_const <- TRUE
            phi2_const <- TRUE
            sigma_const <- TRUE
        }
    }

    df <- 0L
    if (phi1_const) df <- df + (n_time - 2L)
    if (phi2_const) df <- df + max(n_time - 3L, 0L)
    if (sigma_const) df <- df + (n_time - 1L)
    if (df <= 0L) {
        stop("Selected constraint does not impose restrictions (df = 0).", call. = FALSE)
    }

    desc <- paste(
        c(
            if (phi1_const) "phi1 constant over time",
            if (phi2_const) "phi2 constant over time",
            if (sigma_const) "sigma constant over time"
        ),
        collapse = ", "
    )

    list(
        order = 2L,
        phi1_const = phi1_const,
        phi2_const = phi2_const,
        sigma_const = sigma_const,
        df = df,
        description = desc
    )
}


#' @keywords internal
.fit_constrained_stationarity_gau <- function(y, order, blocks, cinfo, fit_init,
                                              block_levels = NULL, max_iter = 2000L,
                                              rel_tol = 1e-8, verbose = FALSE) {
    n_subjects <- nrow(y)
    n_time <- ncol(y)
    block_info <- .normalize_blocks(blocks, n_subjects)
    blocks_id <- block_info$blocks_id
    n_blocks <- block_info$n_blocks
    blocks_fit <- if (is.null(blocks)) NULL else blocks_id

    mu_init <- if (!is.null(fit_init$mu) && length(fit_init$mu) == n_time) {
        as.numeric(fit_init$mu)
    } else {
        colMeans(y)
    }

    tau_init <- if (n_blocks > 1L) {
        if (!is.null(fit_init$tau) && length(fit_init$tau) == n_blocks) {
            as.numeric(fit_init$tau)
        } else {
            rep(0, n_blocks)
        }
    } else {
        0
    }
    if (n_blocks > 1L) tau_init[1] <- 0

    sigma_full_init <- if (!is.null(fit_init$sigma) && length(fit_init$sigma) == n_time) {
        pmax(as.numeric(fit_init$sigma), 1e-8)
    } else {
        rep(max(stats::sd(as.numeric(y)), 1e-2), n_time)
    }

    phi1_init <- NULL
    phi2_init <- NULL
    if (order == 1L) {
        phi_full <- rep(0, n_time)
        if (!is.null(fit_init$phi)) {
            phi_raw <- as.numeric(fit_init$phi)
            if (length(phi_raw) == n_time) {
                phi_full <- phi_raw
            } else if (length(phi_raw) == n_time - 1L) {
                phi_full <- c(0, phi_raw)
            }
        }
        phi1_init <- if (n_time >= 2L) phi_full[2:n_time] else numeric(0)
    } else if (order == 2L) {
        phi_mat <- matrix(0, nrow = 2L, ncol = n_time)
        if (is.matrix(fit_init$phi) &&
            nrow(fit_init$phi) == 2L &&
            ncol(fit_init$phi) == n_time) {
            phi_mat <- fit_init$phi
        }
        if (n_time >= 2L) phi1_init <- phi_mat[1, 2:n_time]
        if (n_time >= 3L) phi2_init <- phi_mat[2, 3:n_time]
    }

    par0 <- numeric(0)
    par0 <- c(par0, mu_init)

    if (n_blocks > 1L) {
        par0 <- c(par0, tau_init[-1])
    }

    if (isTRUE(cinfo$sigma_const)) {
        par0 <- c(par0, log(exp(mean(log(sigma_full_init)))))
    } else {
        par0 <- c(par0, log(sigma_full_init))
    }

    if (order == 1L) {
        if (isTRUE(cinfo$phi_const)) {
            par0 <- c(par0, mean(phi1_init))
        } else {
            par0 <- c(par0, phi1_init)
        }
    } else if (order == 2L) {
        if (isTRUE(cinfo$phi1_const)) {
            par0 <- c(par0, mean(phi1_init))
        } else {
            par0 <- c(par0, phi1_init)
        }

        if (isTRUE(cinfo$phi2_const)) {
            par0 <- c(par0, mean(phi2_init))
        } else if (n_time >= 3L) {
            par0 <- c(par0, phi2_init)
        }
    }

    unpack <- function(par) {
        idx <- 1L

        mu <- par[idx:(idx + n_time - 1L)]
        idx <- idx + n_time

        if (n_blocks > 1L) {
            tau <- c(0, par[idx:(idx + n_blocks - 2L)])
            idx <- idx + n_blocks - 1L
        } else {
            tau <- 0
        }

        if (isTRUE(cinfo$sigma_const)) {
            sigma_scalar <- exp(par[idx])
            sigma <- rep(sigma_scalar, n_time)
            idx <- idx + 1L
        } else {
            sigma <- exp(par[idx:(idx + n_time - 1L)])
            idx <- idx + n_time
        }

        phi <- NULL

        if (order == 1L) {
            if (n_time >= 2L) {
                if (isTRUE(cinfo$phi_const)) {
                    phi_tail <- rep(par[idx], n_time - 1L)
                    idx <- idx + 1L
                } else {
                    phi_tail <- par[idx:(idx + n_time - 2L)]
                    idx <- idx + n_time - 1L
                }
                phi <- c(0, phi_tail)
            } else {
                phi <- numeric(0)
            }
        } else if (order == 2L) {
            phi_mat <- matrix(0, nrow = 2L, ncol = n_time)

            if (n_time >= 2L) {
                if (isTRUE(cinfo$phi1_const)) {
                    phi1 <- rep(par[idx], n_time - 1L)
                    idx <- idx + 1L
                } else {
                    phi1 <- par[idx:(idx + n_time - 2L)]
                    idx <- idx + n_time - 1L
                }
                phi_mat[1, 2:n_time] <- phi1
            }

            if (n_time >= 3L) {
                if (isTRUE(cinfo$phi2_const)) {
                    phi2 <- rep(par[idx], n_time - 2L)
                    idx <- idx + 1L
                } else {
                    phi2 <- par[idx:(idx + n_time - 3L)]
                    idx <- idx + n_time - 2L
                }
                phi_mat[2, 3:n_time] <- phi2
            }

            phi <- phi_mat
        }

        list(mu = mu, tau = tau, sigma = sigma, phi = phi)
    }

    objective <- function(par) {
        params <- unpack(par)
        log_l <- logL_gau(
            y = y,
            order = order,
            mu = params$mu,
            phi = params$phi,
            sigma = params$sigma,
            blocks = blocks_fit,
            tau = params$tau,
            na_action = "fail"
        )
        if (!is.finite(log_l)) return(1e20)
        -log_l
    }

    control <- list(maxit = as.integer(max_iter), reltol = as.numeric(rel_tol))
    if (isTRUE(verbose)) control$trace <- 1

    opt <- stats::optim(par = par0, fn = objective, method = "BFGS", control = control)
    params_hat <- unpack(opt$par)
    log_l <- -opt$value

    n_mean_params <- n_time + max(n_blocks - 1L, 0L)
    n_cov_uncon <- .count_params_gau(order, n_time, n_blocks = 1L, estimate_mu = FALSE)
    n_params <- n_mean_params + (n_cov_uncon - cinfo$df)

    out <- list(
        mu = params_hat$mu,
        phi = params_hat$phi,
        sigma = params_hat$sigma,
        tau = params_hat$tau,
        log_l = log_l,
        aic = -2 * log_l + 2 * n_params,
        bic = -2 * log_l + n_params * log(n_subjects),
        n_params = as.integer(n_params),
        convergence = opt$convergence,
        n_obs = n_subjects * n_time,
        n_missing = 0L,
        pct_missing = 0,
        missing_pattern = "complete",
        settings = list(
            order = order,
            n_time = n_time,
            n_subjects = n_subjects,
            blocks = blocks_fit,
            block_levels = block_levels,
            estimate_mu = TRUE,
            na_action = "fail",
            stationary = TRUE,
            constraints = cinfo
        )
    )

    class(out) <- c("gau_fit", "list")
    out
}

test_that("test_stationarity_gau returns expected df for order-1 constraints", {
  skip_on_cran()
  set.seed(1201)
  y <- simulate_gau(
    n_subjects = 40,
    n_time = 5,
    order = 1,
    mu = 0,
    phi = 0.45,
    sigma = 1
  )

  fit_uncon <- fit_gau(y, order = 1, na_action = "fail")

  res_phi <- test_stationarity_gau(
    y,
    order = 1,
    constrain = "phi",
    fit_unconstrained = fit_uncon,
    max_iter = 700
  )
  res_sigma <- test_stationarity_gau(
    y,
    order = 1,
    constrain = "sigma",
    fit_unconstrained = fit_uncon,
    max_iter = 700
  )
  res_both <- test_stationarity_gau(
    y,
    order = 1,
    constrain = "both",
    fit_unconstrained = fit_uncon,
    max_iter = 700
  )

  expect_s3_class(res_phi, "test_stationarity_gau")
  expect_s3_class(res_sigma, "test_stationarity_gau")
  expect_s3_class(res_both, "test_stationarity_gau")

  expect_equal(res_phi$df, ncol(y) - 2)
  expect_equal(res_sigma$df, ncol(y) - 1)
  expect_equal(res_both$df, (ncol(y) - 2) + (ncol(y) - 1))

  expect_true(is.finite(res_phi$lrt_stat))
  expect_true(is.finite(res_sigma$lrt_stat))
  expect_true(is.finite(res_both$lrt_stat))
  expect_true(is.finite(res_phi$p_value))
  expect_true(is.finite(res_sigma$p_value))
  expect_true(is.finite(res_both$p_value))
})


test_that("test_stationarity_gau detects strong order-1 non-stationarity", {
  skip_on_cran()
  set.seed(1202)
  y <- simulate_gau(
    n_subjects = 280,
    n_time = 6,
    order = 1,
    mu = 0,
    phi = c(0, 0.15, 0.85, -0.2, 0.8, -0.15),
    sigma = c(1.0, 1.8, 0.6, 1.7, 0.55, 1.6)
  )

  res <- test_stationarity_gau(
    y,
    order = 1,
    constrain = "both",
    max_iter = 1000
  )

  expect_s3_class(res, "test_stationarity_gau")
  expect_lt(res$p_value, 0.05)
})


test_that("run_stationarity_tests_gau returns expected order-2 battery", {
  skip_on_cran()
  set.seed(1203)
  phi <- matrix(0, nrow = 2, ncol = 6)
  phi[1, 2:6] <- 0.4
  phi[2, 3:6] <- 0.2

  y <- simulate_gau(
    n_subjects = 30,
    n_time = 6,
    order = 2,
    mu = 0,
    phi = phi,
    sigma = 1
  )

  out <- run_stationarity_tests_gau(y, order = 2, verbose = FALSE, max_iter = 700)

  expect_s3_class(out, "stationarity_tests_gau")
  expect_setequal(out$summary$constraint, c("phi1", "phi2", "phi", "sigma", "all"))
  expect_true(all(out$summary$df > 0))
  expect_equal(length(out$tests), 5)
})

# File: R/bic_gau.R

#' Bayesian information criterion for fitted Gaussian AD models
#'
#' Computes BIC using the fitted log likelihood and a parameter count that
#' respects identifiability constraints for the Gaussian antedependence
#' parameters.
#'
#' @param fit A fitted model object returned by \code{\link{fit_gau}}.
#' @param n_subjects Number of subjects, typically \code{nrow(y)}. If
#'   \code{NULL}, inferred from \code{fit$settings$n_subjects}.
#'
#' @return A numeric scalar BIC value.
#'
#' @details
#' The BIC is computed as:
#' \deqn{BIC = -2 \times \ell + k \times \log(N)}
#' where \eqn{\ell} is the log-likelihood, \eqn{k} is the number of free parameters,
#' and \eqn{N} is the number of subjects.
#'
#' This function applies to Gaussian AD fits from \code{\link{fit_gau}}.
#' For categorical and INAD models, use \code{\link{bic_cat}} and
#' \code{\link{bic_inad}}.
#'
#' @examples
#' set.seed(1)
#' y <- simulate_gau(n_subjects = 30, n_time = 5, order = 1, phi = 0.3)
#' fit <- fit_gau(y, order = 1)
#' bic_gau(fit, n_subjects = nrow(y))
#' @export
bic_gau <- function(fit, n_subjects = NULL) {
    if (is.null(n_subjects)) n_subjects <- fit$settings$n_subjects
    if (is.null(n_subjects) || length(n_subjects) != 1 || !is.finite(n_subjects) || n_subjects <= 0) {
        stop("n_subjects must be a positive finite scalar")
    }

    k <- .count_params_gau_fit(fit)
    -2 * fit$log_l + k * log(n_subjects)
}

#' Akaike information criterion for fitted Gaussian AD models
#'
#' Computes AIC using the fitted log likelihood and a parameter count that
#' respects identifiability constraints for the Gaussian antedependence
#' parameters.
#'
#' @param fit A fitted model object returned by \code{\link{fit_gau}}.
#'
#' @return A numeric scalar AIC value.
#'
#' @details
#' The AIC is computed as:
#' \deqn{AIC = -2 \times \ell + 2k}
#' where \eqn{\ell} is the log-likelihood and \eqn{k} is the number of free
#' parameters.
#'
#' This function applies to Gaussian AD fits from \code{\link{fit_gau}}.
#'
#' @examples
#' set.seed(1)
#' y <- simulate_gau(n_subjects = 30, n_time = 5, order = 1, phi = 0.3)
#' fit <- fit_gau(y, order = 1)
#' aic_gau(fit)
#' @export
aic_gau <- function(fit) {
    k <- .count_params_gau_fit(fit)
    -2 * fit$log_l + 2 * k
}

#' @keywords internal
.count_params_gau_fit <- function(fit) {
    if (is.null(fit$log_l) || !is.finite(fit$log_l)) stop("fit$log_l must be finite")
    if (is.null(fit$settings$order)) stop("fit$settings$order is missing")

    ord <- fit$settings$order
    if (!(ord %in% c(0, 1, 2))) stop("fit$settings$order must be 0, 1, or 2")

    N <- NULL
    if (!is.null(fit$sigma)) N <- length(as.numeric(fit$sigma))
    if (is.null(N) && !is.null(fit$mu)) N <- length(as.numeric(fit$mu))
    if (is.null(N) || N < 1) stop("Cannot infer n_time from fit; need fit$sigma or fit$mu")

    estimate_mu <- fit$settings$estimate_mu
    if (is.null(estimate_mu)) estimate_mu <- !is.null(fit$mu)
    estimate_mu <- isTRUE(estimate_mu)

    k <- 0

    if (estimate_mu) k <- k + N

    k <- k + N

    if (ord == 1) k <- k + (N - 1)
    if (ord == 2) k <- k + (2 * N - 3)

    if (!is.null(fit$tau)) {
        B <- length(as.numeric(fit$tau))
        if (B >= 1) k <- k + (B - 1)
    }

    k
}

#' Cochlear implant speech recognition data
#'
#' Longitudinal speech recognition outcomes for two groups (A/B), including
#' incomplete records, from Zimmerman and Nunez-Anton antedependence book
#' companion data. This dataset is continuous-response data suitable for
#' Gaussian AD modeling.
#'
#' @format A list with six components:
#'   \describe{
#'     \item{y}{numeric matrix of dimension N by n_time containing all subjects}
#'     \item{y_A}{numeric matrix for Group A subjects}
#'     \item{y_B}{numeric matrix for Group B subjects}
#'     \item{blocks}{integer vector of length N (1 = Group A, 2 = Group B)}
#'     \item{group}{character vector of group labels ("A"/"B")}
#'     \item{time}{integer vector of measurement occasions}
#'   }
#'
#' @source \url{https://homepage.divms.uiowa.edu/~dzimmer/Data-for-AD/speech_recognition_data.txt}
"cochlear_implant"

# Internal utility functions for INAD models
# These functions are shared across loglik_inad.R and other INAD functions

#' Log-sum-exp for numerical stability
#' @keywords internal
.log_sum_exp <- function(x) {
    m <- max(x)
    if (!is.finite(m)) return(-Inf)
    m + log(sum(exp(x - m)))
}

#' Convert Bell parameter to Bell mean
#' @keywords internal
.bell_mean_from_theta <- function(theta) {
    theta * exp(theta)
}

#' Convert Bell mean to Bell parameter
#' @keywords internal
.bell_theta_from_mean <- function(mu) {
    mu <- as.numeric(mu)
    out <- rep(NA_real_, length(mu))
    ok <- is.finite(mu) & mu > 0
    if (!any(ok)) return(out)

    mu_ok <- mu[ok]
    uniq <- unique(mu_ok)
    th_uniq <- vapply(uniq, solve_theta_bell, numeric(1))
    out[ok] <- th_uniq[match(mu_ok, uniq)]
    out
}

#' Effective innovation distribution parameter under block effects
#'
#' For Poisson and negative binomial innovations, block effects are additive on
#' the mean. For Bell innovations, block effects are additive on the innovation
#' mean `theta * exp(theta)`, then mapped back to the Bell parameter.
#'
#' @keywords internal
.inad_effective_innovation_param <- function(theta, tau, innovation) {
    if (innovation == "bell") {
        mu <- .bell_mean_from_theta(theta) + tau
        if (any(!is.finite(mu) | mu <= 0)) return(rep(NA_real_, length(mu)))
        return(.bell_theta_from_mean(mu))
    }
    theta + tau
}

#' Effective innovation mean under block effects
#' @keywords internal
.inad_effective_innovation_mean <- function(theta, tau, innovation) {
    if (innovation == "bell") {
        return(.bell_mean_from_theta(theta) + tau)
    }
    theta + tau
}

#' Thinning probability vector
#'
#' Computes the probability mass for thinned counts.
#'
#' @param k_vals Integer vector of possible thinned values.
#' @param yprev Previous count (size parameter for thinning).
#' @param a Thinning parameter alpha.
#' @param thinning One of "binom", "pois", "nbinom".
#'
#' @return Numeric vector of probabilities.
#' @keywords internal
.thin_vec <- function(k_vals, yprev, a, thinning) {
    if (!is.finite(a) || a < 0) return(rep(0, length(k_vals)))
    if (!is.finite(yprev) || yprev < 0) return(rep(0, length(k_vals)))

    if (thinning == "binom") {
        if (a > 1) return(rep(0, length(k_vals)))
        return(stats::dbinom(k_vals, size = yprev, prob = a))
    }

    if (thinning == "pois") {
        return(stats::dpois(k_vals, lambda = a * yprev))
    }

    # nbinom thinning: alpha * Y | Y ~ NegBin(Y, 1/(1+alpha))
    # See Supplement A.1 of the paper
    if (yprev == 0) {
        return(as.numeric(k_vals == 0))
    }
    p_nb <- 1 / (1 + a)
    stats::dnbinom(k_vals, size = yprev, prob = p_nb)
}

#' Innovation probability vector
#'
#' Computes the probability mass for innovation counts.
#'
#' @param u_vals Integer vector of possible innovation values.
#' @param lam Innovation parameter used by the PMF. This is the mean for
#'   `"pois"` and `"nbinom"`, and the Bell parameter for `"bell"`.
#' @param innovation One of "pois", "bell", "nbinom".
#' @param sz Size parameter for negative binomial innovation (ignored otherwise).
#'
#' @return Numeric vector of probabilities.
#' @keywords internal
.innov_vec <- function(u_vals, lam, innovation, sz) {
    if (!is.finite(lam) || lam <= 0) return(rep(0, length(u_vals)))

    if (innovation == "pois") return(stats::dpois(u_vals, lambda = lam))
    if (innovation == "bell") return(dbell(u_vals, theta = lam))
    # nbinom: size = sz (dispersion), mu = lam (mean)
    # See Supplement B.3 of the paper
    stats::dnbinom(u_vals, size = sz, mu = lam)
}

#' Unpack alpha parameters
#'
#' Converts alpha from various input formats to a standardized list.
#'
#' @param order Model order (0, 1, or 2).
#' @param alpha Alpha parameter in various formats.
#' @param N Number of time points.
#'
#' @return List with elements a1 and a2.
#' @keywords internal
.unpack_alpha <- function(order, alpha, N) {
    if (order == 0) return(list(a1 = NULL, a2 = NULL))

    if (order == 1) {
        a <- as.numeric(alpha)
        if (length(a) == 1L) a <- rep(a, N)
        if (length(a) != N) stop("alpha must be length 1 or ncol(y) for order=1")
        return(list(a1 = a, a2 = NULL))
    }

    if (is.matrix(alpha) && ncol(alpha) >= 2) {
        if (nrow(alpha) == 1L) alpha <- alpha[rep.int(1L, N), , drop = FALSE]
        if (nrow(alpha) != N) stop("for order=2, alpha matrix must have nrow 1 or ncol(y)")
        return(list(a1 = as.numeric(alpha[, 1]), a2 = as.numeric(alpha[, 2])))
    }

    if (is.list(alpha) && length(alpha) >= 2) {
        a1 <- as.numeric(alpha[[1]])
        a2 <- as.numeric(alpha[[2]])
        if (length(a1) == 1L) a1 <- rep(a1, N)
        if (length(a2) == 1L) a2 <- rep(a2, N)
        if (length(a1) != N || length(a2) != N) stop("alpha[[1]] and alpha[[2]] must be length 1 or ncol(y)")
        return(list(a1 = a1, a2 = a2))
    }

    stop("for order=2, alpha must be a matrix with 2 columns or a list(alpha1, alpha2)")
}

# loglik_cat.R - Log-likelihood computation for categorical antedependence models

#' Log-likelihood for categorical AD models (with missing data support)
#'
#' Evaluates the log-likelihood of an AD(p) model for categorical longitudinal
#' data at given parameter values.
#'
#' @param y Integer matrix with n_subjects rows and n_time columns. Each entry
#'   should be a category code from 1 to c.
#' @param order Antedependence order p. Must be 0, 1, or 2.
#' @param marginal List of marginal/joint probabilities for initial time points.
#'   Structure depends on order (see Details).
#' @param transition List of transition probability arrays for time points
#'   k = p+1 to n. Each element should be an array of dimension c^p x c where
#'   the last dimension corresponds to the current time point.
#' @param blocks Optional integer vector of length n_subjects specifying group
#'   membership. Required if homogeneous = FALSE.
#' @param homogeneous Logical. If TRUE (default), same parameters used for all
#'   subjects. If FALSE, marginal and transition should be lists indexed by block.
#' @param n_categories Number of categories. If NULL, inferred from data.
#' @param na_action Handling of missing values in \code{y}. One of
#'   \code{"fail"} (default, error if any missing), \code{"complete"}
#'   (drop subjects with any missing values), or \code{"marginalize"}
#'   (integrate over missing categorical outcomes under the AD model).
#'
#' @return Scalar log-likelihood value.
#'
#' @details
#' The log-likelihood for AD(p) decomposes into contributions from initial
#' time points and transition time points.
#'
#' For order 0 (independence), the log-likelihood is the sum of log marginal
#' probabilities at each time point.
#'
#' Parameter structure for marginal:
#' \itemize{
#'   \item Order 0: List with elements t1, t2, ..., tn, each a vector of length c
#'   \item Order 1: List with element t1 (vector of length c)
#'   \item Order 2: List with t1 (vector), t2_given_1to1 (c x c matrix)
#' }
#'
#' Parameter structure for transition:
#' \itemize{
#'   \item Order 0: Not used (NULL or empty list)
#'   \item Order 1: List with elements t2, t3, ..., tn, each c x c matrix
#'   \item Order 2: List with elements t3, t4, ..., tn, each c x c x c array
#' }
#'
#' @examples
#' set.seed(1)
#' y <- simulate_cat(n_subjects = 40, n_time = 5, order = 1, n_categories = 3)
#' fit <- fit_cat(y, order = 1, n_categories = 3)
#' logL_cat(
#'   y = y,
#'   order = 1,
#'   marginal = fit$marginal,
#'   transition = fit$transition,
#'   n_categories = 3
#' )
#'
#' @references
#' Xie, Y. and Zimmerman, D. L. (2013). Antedependence models for nonstationary
#' categorical longitudinal data with ignorable missingness: likelihood-based
#' inference. \emph{Statistics in Medicine}, 32, 3274-3289.
#'
#' @export
logL_cat <- function(y, order, marginal, transition = NULL, blocks = NULL,
                     homogeneous = TRUE, n_categories = NULL,
                     na_action = c("fail", "complete", "marginalize")) {
  na_action <- match.arg(na_action)
  
  # Validate data
  validated <- .validate_y_cat(y, n_categories, allow_na = (na_action != "fail"))
  y <- validated$y
  c <- validated$n_categories
  
  n_subjects <- nrow(y)
  n_time <- ncol(y)
  p <- as.integer(order)
  
  # Validate order
  if (p < 0) stop("order must be non-negative")
  if (p >= n_time) stop("order must be less than number of time points")
  if (na_action == "marginalize" && p > 2) {
    stop("na_action = 'marginalize' currently supports order 0, 1, and 2")
  }
  
  # Validate blocks
  blocks <- .validate_blocks_cat(blocks, n_subjects)
  
  # Handle missing values according to na_action
  if (na_action == "complete") {
    keep <- stats::complete.cases(y)
    y <- y[keep, , drop = FALSE]
    blocks <- blocks[keep]
    if (nrow(y) == 0) {
      stop("No complete subjects remain after na_action = 'complete'")
    }
    blocks <- .validate_blocks_cat(blocks, nrow(y))
  }
  
  n_subjects <- nrow(y)
  n_blocks <- max(blocks)
  
  # Determine number of populations
  if (homogeneous) {
    n_pops <- 1
  } else {
    n_pops <- n_blocks
    if (n_pops > 1 && !is.list(marginal[[1]])) {
      stop("For heterogeneous model, marginal and transition must be lists indexed by block")
    }
  }
  
  # Compute log-likelihood
  log_l <- 0
  
  for (s in seq_len(n_subjects)) {
    # Determine which population parameters to use
    if (homogeneous) {
      marg_s <- marginal
      trans_s <- transition
    } else {
      g <- blocks[s]
      marg_s <- marginal[[g]]
      trans_s <- transition[[g]]
    }
    
    # Add contribution from subject s
    log_l <- log_l + .logL_subject_cat_observed(y[s, ], p, c, marg_s, trans_s,
                                                na_action = na_action)
  }
  
  log_l
}


#' Compute log-likelihood contribution from one subject
#'
#' @param y_s Integer vector of length n_time (one subject's data)
#' @param p Order
#' @param c Number of categories
#' @param marginal Marginal parameters for this population
#' @param transition Transition parameters for this population
#'
#' @return Scalar log-likelihood contribution
#'
#' @keywords internal
.logL_subject_cat <- function(y_s, p, c, marginal, transition) {
  n_time <- length(y_s)
  ll <- 0
  
  if (p == 0) {
    # Independence model
    for (k in seq_len(n_time)) {
      prob <- marginal[[k]][y_s[k]]
      ll <- ll + .safe_log(prob)
    }
    return(ll)
  }
  
  # AD(p) model with p >= 1
  
  # Initial time points (k = 1 to p)
  # k = 1: P(Y_1)
  prob_1 <- marginal[["t1"]][y_s[1]]
  ll <- ll + .safe_log(prob_1)
  
  # k = 2 to min(p, n_time): need conditional on full history
  if (p >= 2 && n_time >= 2) {
    for (k in 2:min(p, n_time)) {
      # Get P(Y_k | Y_1, ..., Y_{k-1})
      trans_name <- paste0("t", k, "_given_1to", k - 1)
      trans_k <- marginal[[trans_name]]
      
      # Index into the array: [y_1, y_2, ..., y_{k-1}, y_k]
      idx <- c(y_s[1:(k-1)], y_s[k])
      prob_k <- trans_k[matrix(idx, nrow = 1)]
      ll <- ll + .safe_log(prob_k)
    }
  }
  
  # Time points k = p+1 to n_time: condition only on last p values
  if (n_time > p) {
    for (k in (p + 1):n_time) {
      trans_name <- paste0("t", k)
      trans_k <- transition[[trans_name]]
      
      # Index: [y_{k-p}, ..., y_{k-1}, y_k]
      idx <- y_s[(k - p):k]
      prob_k <- trans_k[matrix(idx, nrow = 1)]
      ll <- ll + .safe_log(prob_k)
    }
  }
  
  ll
}


#' Compute observed-data log-likelihood contribution from one subject
#'
#' @param y_s Integer vector of length n_time (one subject's data; may contain NA)
#' @param p Order
#' @param c Number of categories
#' @param marginal Marginal parameters for this population
#' @param transition Transition parameters for this population
#' @param na_action Missing-data handling mode
#'
#' @return Scalar log-likelihood contribution
#'
#' @keywords internal
.logL_subject_cat_observed <- function(y_s, p, c, marginal, transition, na_action) {
  has_na <- any(is.na(y_s))
  
  if (!has_na) {
    return(.logL_subject_cat(y_s, p, c, marginal, transition))
  }
  
  if (na_action == "fail") {
    stop("Missing data not supported in this call; use na_action = 'complete' or 'marginalize'")
  }
  
  if (na_action == "complete") {
    stop("Internal error: complete-case subject path should not contain missing values")
  }
  
  # na_action == "marginalize"
  if (p == 0) {
    ll <- 0
    n_time <- length(y_s)
    for (k in seq_len(n_time)) {
      if (!is.na(y_s[k])) {
        ll <- ll + log(marginal[[k]][y_s[k]])
      }
    }
    return(ll)
  }
  
  if (p == 1) {
    return(.logL_subject_cat_marginalize_p1(y_s, c, marginal, transition))
  }
  
  if (p == 2) {
    return(.logL_subject_cat_marginalize_p2(y_s, c, marginal, transition))
  }
  
  stop("na_action = 'marginalize' currently supports order 0, 1, and 2")
}


#' Stable log-sum-exp
#'
#' @param x Numeric vector
#'
#' @return Scalar log(sum(exp(x)))
#'
#' @keywords internal
.logsumexp_cat <- function(x) {
  m <- max(x)
  if (!is.finite(m)) return(-Inf)
  m + log(sum(exp(x - m)))
}


#' Observed-data log-likelihood for order-1 CAT model
#'
#' @param y_s Subject row (may contain NA)
#' @param c Number of categories
#' @param marginal Marginal list
#' @param transition Transition list
#'
#' @return Scalar log-likelihood contribution
#'
#' @keywords internal
.logL_subject_cat_marginalize_p1 <- function(y_s, c, marginal, transition) {
  n_time <- length(y_s)
  log_alpha <- rep(-Inf, c)
  
  # k = 1
  if (is.na(y_s[1])) {
    log_alpha <- log(marginal[["t1"]])
  } else {
    log_alpha[y_s[1]] <- log(marginal[["t1"]][y_s[1]])
  }
  
  if (n_time == 1) {
    return(.logsumexp_cat(log_alpha))
  }
  
  # k = 2,...,n
  for (k in 2:n_time) {
    trans_k <- transition[[paste0("t", k)]]
    log_trans_k <- log(trans_k)
    log_trans_k[trans_k <= 0] <- -Inf
    
    log_next <- rep(-Inf, c)
    if (is.na(y_s[k])) {
      for (j in seq_len(c)) {
        log_next[j] <- .logsumexp_cat(log_alpha + log_trans_k[, j])
      }
    } else {
      j_obs <- y_s[k]
      log_next[j_obs] <- .logsumexp_cat(log_alpha + log_trans_k[, j_obs])
    }
    log_alpha <- log_next
  }
  
  .logsumexp_cat(log_alpha)
}


#' Observed-data log-likelihood for order-2 CAT model
#'
#' @param y_s Subject row (may contain NA)
#' @param c Number of categories
#' @param marginal Marginal list
#' @param transition Transition list
#'
#' @return Scalar log-likelihood contribution
#'
#' @keywords internal
.logL_subject_cat_marginalize_p2 <- function(y_s, c, marginal, transition) {
  n_time <- length(y_s)
  
  # k = 1
  log_alpha1 <- rep(-Inf, c)
  if (is.na(y_s[1])) {
    log_alpha1 <- log(marginal[["t1"]])
  } else {
    log_alpha1[y_s[1]] <- log(marginal[["t1"]][y_s[1]])
  }
  
  if (n_time == 1) {
    return(.logsumexp_cat(log_alpha1))
  }
  
  # k = 2: state is (Y1, Y2)
  trans_2 <- marginal[["t2_given_1to1"]]
  log_trans_2 <- log(trans_2)
  log_trans_2[trans_2 <= 0] <- -Inf
  log_state <- matrix(-Inf, nrow = c, ncol = c)
  
  for (a in seq_len(c)) {
    if (!is.finite(log_alpha1[a])) next
    if (!is.na(y_s[1]) && y_s[1] != a) next
    for (b in seq_len(c)) {
      if (!is.na(y_s[2]) && y_s[2] != b) next
      log_state[a, b] <- log_alpha1[a] + log_trans_2[a, b]
    }
  }
  
  if (n_time == 2) {
    return(.logsumexp_cat(as.vector(log_state)))
  }
  
  # k = 3,...,n: state is (Y_{k-1}, Y_k)
  for (k in 3:n_time) {
    trans_k <- transition[[paste0("t", k)]]
    log_trans_k <- log(trans_k)
    log_trans_k[trans_k <= 0] <- -Inf
    log_next <- matrix(-Inf, nrow = c, ncol = c)
    
    for (b in seq_len(c)) {
      for (cat_k in seq_len(c)) {
        if (!is.na(y_s[k]) && y_s[k] != cat_k) next
        
        candidates <- rep(-Inf, c)
        for (a in seq_len(c)) {
          if (!is.finite(log_state[a, b])) next
          candidates[a] <- log_state[a, b] + log_trans_k[a, b, cat_k]
        }
        log_next[b, cat_k] <- .logsumexp_cat(candidates)
      }
    }
    
    log_state <- log_next
  }
  
  .logsumexp_cat(as.vector(log_state))
}


#' Compute log-likelihood from cell counts (faster for large datasets)
#'
#' Instead of looping over subjects, this version uses aggregated cell counts.
#' Equivalent to logL_cat but more efficient.
#'
#' @param y Data matrix
#' @param order AD order p
#' @param marginal Marginal parameters
#' @param transition Transition parameters
#' @param n_categories Number of categories
#'
#' @return Scalar log-likelihood
#'
#' @keywords internal
.logL_from_counts_cat <- function(y, order, marginal, transition, n_categories) {
  c <- n_categories
  n_time <- ncol(y)
  p <- order
  
  log_l <- 0
  
  if (p == 0) {
    # Independence: sum over each time point
    for (k in seq_len(n_time)) {
      counts_k <- .count_cells_table_cat(y, k, c)
      probs_k <- marginal[[k]]
      log_l <- log_l + sum(counts_k * .safe_log(probs_k))
    }
    return(log_l)
  }
  
  # AD(p) with p >= 1
  
  # Initial time points
  for (k in seq_len(min(p, n_time))) {
    time_indices <- seq_len(k)
    counts_k <- .count_cells_table_cat(y, time_indices, c)
    
    if (k == 1) {
      probs_k <- marginal[["t1"]]
      log_l <- log_l + sum(counts_k * .safe_log(probs_k))
    } else {
      trans_name <- paste0("t", k, "_given_1to", k - 1)
      probs_k <- marginal[[trans_name]]
      log_l <- log_l + sum(counts_k * .safe_log(probs_k))
    }
  }
  
  # Transition time points
  for (k in (p + 1):n_time) {
    time_indices <- (k - p):k
    counts_k <- .count_cells_table_cat(y, time_indices, c)
    trans_name <- paste0("t", k)
    probs_k <- transition[[trans_name]]
    log_l <- log_l + sum(counts_k * .safe_log(probs_k))
  }
  
  log_l
}

#' Labor force longitudinal categorical data (Table 1)
#'
#' Five-year employment-status sequences reconstructed from Table 1 in the
#' labor-force example used in Xie and Zimmerman score/Wald antedependence
#' testing work. Category coding is 1 = employed, 2 = unemployed.
#'
#' @format A list with five components:
#'   \describe{
#'     \item{y}{integer matrix of dimension N by 5 containing expanded subject-level sequences}
#'     \item{counts}{data frame with columns Y1, Y2, Y3, Y4, Y5, Count}
#'     \item{n_categories}{number of categories (2)}
#'     \item{time}{integer vector of calendar years (1967:1971)}
#'     \item{status_labels}{character vector c("employed", "unemployed")}
#'   }
#'
#' @source Table 1 (labor-force example) from:
#'   Xie, Y. and Zimmerman, D. L. (2013).
#'   Score and Wald tests for antedependence in categorical longitudinal data.
"labor_force_cat"

# File: R/test_order_gau.R
# Likelihood ratio tests for the order of antedependence (Chapter 6.2)

#' Likelihood ratio test for antedependence order (Gaussian AD data)
#'
#' Tests the null hypothesis that the data follow an AD(p) model against the
#' alternative that they follow an AD(p+q) model, using the likelihood ratio
#' test described in Theorem 6.4 and 6.5 of Zimmerman & Núñez-Antón (2009).
#'
#' @param y Numeric matrix with n_subjects rows and n_time columns.
#' @param p Order under the null hypothesis (default 0). This is the same
#'   antedependence order parameter named \code{order} in \code{\link{fit_gau}}.
#' @param q Order increment under the alternative (default 1, so alternative is AD(p+q)).
#' @param order_null Optional alias for \code{p} (null order).
#' @param order_alt Optional absolute order under the alternative. When supplied,
#'   \code{q} is computed as \code{order_alt - p}.
#' @param mu Optional mean vector. If NULL, the saturated mean (sample means) is used.
#' @param use_modified Logical. If TRUE (default), use the modified test statistic
#'   (formula 6.9) which has better small-sample properties.
#'
#' @return A list with class \code{gau_order_test} containing:
#' \describe{
#'   \item{method}{Inference method used (\code{"lrt"}).}
#'   \item{p}{Order under null hypothesis}
#'   \item{q}{Order increment}
#'   \item{statistic}{Test statistic value}
#'   \item{statistic_modified}{Modified test statistic (if use_modified = TRUE)}
#'   \item{df}{Degrees of freedom}
#'   \item{p_value}{P-value from chi-square distribution}
#'   \item{p_value_modified}{P-value from modified test (if use_modified = TRUE)}
#'   \item{n_subjects}{Number of subjects}
#'   \item{n_time}{Number of time points}
#' }
#'
#' @details
#' The test is based on the intervenor-adjusted sample partial correlations.
#' Under the null hypothesis AD(p), the partial correlations r_(i,i-k|(i-k+1:i-1))
#' should be zero for k > p.
#'
#' The likelihood ratio test statistic (Theorem 6.4) is:
#' \deqn{-N \sum_{j=1}^{q} \sum_{i=p+j+1}^{n} \log(1 - r^2_{i,i-p-j\cdot(i-p-j+1:i-1)})}
#'
#' which is asymptotically chi-square with (2n - 2p - q - 1)(q/2) degrees of freedom.
#'
#' The modified test (formula 6.9) adjusts for small-sample bias using Kenward's
#' (1987) correction.
#'
#' @references
#' Zimmerman, D.L. and Núñez-Antón, V. (2009). Antedependence Models for
#' Longitudinal Data. Chapman & Hall/CRC. Chapter 6.
#'
#' Kenward, M.G. (1987). A method for comparing profiles of repeated measurements.
#' Applied Statistics, 36, 296-308.
#'
#' @seealso \code{\link{test_one_sample_gau}}, \code{\link{test_homogeneity_gau}}
#'
#' @examples
#' \donttest{
#' # Simulate AD(1) data
#' y <- simulate_gau(n_subjects = 50, n_time = 6, order = 1, phi = 0.5)
#'
#' # Test AD(0) vs AD(1)
#' test01 <- test_order_gau(y, p = 0, q = 1)
#' print(test01)
#'
#' # Test AD(1) vs AD(2)
#' test12 <- test_order_gau(y, p = 1, q = 1)
#' print(test12)
#' }
#'
#' @export
test_order_gau <- function(y, p = 0L, q = 1L, mu = NULL, use_modified = TRUE,
                           order_null = NULL, order_alt = NULL) {

    if (!is.matrix(y)) y <- as.matrix(y)
    if (anyNA(y)) {
        stop(
            "test_order_gau currently supports complete data only. Missing-data AD likelihood-ratio tests are not implemented yet.",
            call. = FALSE
        )
    }
    if (any(!is.finite(y))) stop("y must contain finite values")

    n <- nrow(y)  # Number of subjects (N in the book)
    n_time <- ncol(y)  # Number of time points (n in the book)

    if (!is.null(order_null)) p <- order_null
    if (!is.null(order_alt)) q <- as.integer(order_alt) - as.integer(p)

    p <- as.integer(p)
    q <- as.integer(q)

    if (p < 0) stop("p must be non-negative")
    if (q < 1) stop("q must be at least 1")
    if (p + q >= n_time) stop("p + q must be less than n_time")

    # Use saturated mean if not provided
    if (is.null(mu)) {
        mu <- colMeans(y)
    }
    if (length(mu) != n_time) stop("mu must have length n_time")

    # Compute intervenor-adjusted partial correlations and test statistic
    # Formula (6.5): -N * sum_{j=1}^{q} sum_{i=p+j+1}^{n} log(1 - r^2)

    log_terms <- numeric(0)
    psi_terms <- numeric(0)

    for (j in 1:q) {
        for (i in (p + j + 1):n_time) {
            # Partial correlation r_{i, i-p-j · (i-p-j+1 : i-1)}
            # This is the correlation between Y_i and Y_{i-p-j} adjusting for intervenors
            lag <- p + j
            r <- .partial_corr_gau(y, i, i - lag, mu = mu)

            log_terms <- c(log_terms, log(1 - r^2))

            # For modified test, compute psi weights
            if (use_modified) {
                # psi(p+q)_i from the book for time i
                p_i <- min(p + q, i - 1)
                psi_val <- .psi_kenward(p_i, n - 2 - p_i)
                psi_terms <- c(psi_terms, psi_val)
            }
        }
    }

    # Standard LRT statistic
    statistic <- -n * sum(log_terms)

    # Degrees of freedom: (2n - 2p - q - 1)(q/2)
    # But this is for the book's "n" which is n_time
    df <- (2 * n_time - 2 * p - q - 1) * q / 2

    # P-value
    p_value <- stats::pchisq(statistic, df = df, lower.tail = FALSE)

    # Modified test statistic (formula 6.9)
    statistic_modified <- NULL
    p_value_modified <- NULL

    if (use_modified) {
        # The modified statistic uses a weighted sum
        # For simplicity, we use the RSS-based formulation (Theorem 6.5)
        rss_p <- .rss_vector_gau(y, p, mu = mu)
        rss_pq <- .rss_vector_gau(y, p + q, mu = mu)

        # Modified statistic (6.9):
        # G~^2 = [sum_i diff_i] * [sum_i log(RSS_i(p)/RSS_i(p+q))] / sum_i psi_i
        # where diff_i = (p+q)_i - p_i  (additional parameters at time i)
        # This is a product of two separate sums (not an inner product).

        sum_diff <- 0
        sum_lr   <- 0
        denominator <- 0

        for (i in (p + 2):n_time) {
            p_i_null <- min(p, i - 1)
            p_i_alt <- min(p + q, i - 1)
            diff_order <- p_i_alt - p_i_null

            if (diff_order > 0 && rss_p[i] > 0 && rss_pq[i] > 0) {
                log_ratio <- log(rss_p[i]) - log(rss_pq[i])
                sum_diff <- sum_diff + diff_order
                sum_lr   <- sum_lr   + log_ratio

                # m = number of mean parameters (1 for saturated within each regression)
                m <- 1
                psi_val <- .psi_kenward(diff_order, n - m - p_i_alt)
                denominator <- denominator + psi_val
            }
        }

        if (denominator > 0) {
            # Eq. 6.9: [Σ diff_i] × [Σ log-ratio_i] / Σ psi_i
            statistic_modified <- sum_diff * sum_lr / denominator
            p_value_modified <- stats::pchisq(statistic_modified, df = df, lower.tail = FALSE)
        }
    }

    out <- list(
        method = "lrt",
        p = p,
        q = q,
        order_null = p,
        order_alt = p + q,
        statistic = statistic,
        statistic_modified = statistic_modified,
        df = df,
        p_value = p_value,
        p_value_modified = p_value_modified,
        n_subjects = n,
        n_time = n_time
    )

    class(out) <- "gau_order_test"
    out
}


#' BIC-based order selection for Gaussian AD models
#'
#' Fits AD models of increasing orders and selects the best by BIC.
#'
#' @param y Numeric matrix with n_subjects rows and n_time columns.
#' @param max_order Maximum order to consider.
#' @param ... Additional arguments passed to \code{\link{fit_gau}}.
#'
#' @return A list with class \code{gau_bic_order} containing:
#' \describe{
#'   \item{fits}{List of fitted models}
#'   \item{bic}{BIC values for each order}
#'   \item{best_order}{Order with lowest BIC}
#'   \item{table}{Summary table}
#' }
#'
#' @seealso \code{\link{bic_order_cat}}, \code{\link{bic_order_inad}},
#'   \code{\link{fit_gau}}
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' y <- simulate_gau(n_subjects = 80, n_time = 6, order = 1, phi = 0.4)
#' ord <- bic_order_gau(y, max_order = 2)
#' ord$best_order
#' ord$table
#' }
#'
#' @export
bic_order_gau <- function(y, max_order = 2L, ...) {

    if (!is.matrix(y)) y <- as.matrix(y)

    n <- nrow(y)
    n_time <- ncol(y)

    max_order <- min(as.integer(max_order), n_time - 1L)

    fits <- list()
    bic_vals <- numeric(max_order + 1)

    for (ord in 0:max_order) {
        fits[[ord + 1]] <- fit_gau(y, order = ord, ...)
        bic_vals[ord + 1] <- bic_gau(fits[[ord + 1]], n)
    }

    names(fits) <- paste0("order", 0:max_order)
    names(bic_vals) <- paste0("order", 0:max_order)

    best_order <- which.min(bic_vals) - 1L

    tbl <- data.frame(
        order = 0:max_order,
        log_l = sapply(fits, function(f) f$log_l),
        bic = bic_vals,
        selected = (0:max_order) == best_order
    )

    out <- list(
        fits = fits,
        bic = bic_vals,
        best_order = best_order,
        table = tbl
    )

    class(out) <- "gau_bic_order"
    out
}


#' Print method for AD order test
#'
#' @param x Object of class \code{gau_order_test}.
#' @param ... Unused.
#' @return Invisibly returns \code{x}.
#' @export
print.gau_order_test <- function(x, ...) {
    cat("Order Test for Antedependence\n")
    cat("==================================================\n\n")

    cat("H0: AD(", x$p, ")\n", sep = "")
    cat("H1: AD(", x$p + x$q, ")\n\n", sep = "")

    cat("Sample size: n =", x$n_subjects, "subjects,", x$n_time, "time points\n\n")

    cat("Standard statistic (LRT):\n")
    cat("  Statistic:", round(x$statistic, 4), "\n")
    cat("  df:", x$df, "\n")
    cat("  p-value:", format.pval(x$p_value, digits = 4), "\n")

    if (!is.null(x$statistic_modified)) {
        cat("\nModified LRT (Kenward correction):\n")
        cat("  Statistic:", round(x$statistic_modified, 4), "\n")
        cat("  p-value:", format.pval(x$p_value_modified, digits = 4), "\n")
    }

    invisible(x)
}


#' Print method for BIC order selection
#'
#' @param x Object of class \code{gau_bic_order}.
#' @param ... Unused.
#' @return Invisibly returns \code{x}.
#' @export
print.gau_bic_order <- function(x, ...) {
    cat("BIC-based Order Selection for Gaussian AD\n")
    cat("==========================================\n\n")

    print(x$table, row.names = FALSE)

    cat("\nBest order by BIC:", x$best_order, "\n")

    invisible(x)
}

#' Likelihood ratio test for antedependence order (INAD data)
#'
#' Performs a likelihood ratio test comparing INAD models of different orders.
#'
#' @param y Integer matrix with n_subjects rows and n_time columns.
#' @param order_null Order under null hypothesis (0 or 1).
#' @param order_alt Order under alternative hypothesis (1 or 2). Must be order_null + 1.
#' @param thinning Thinning operator: "binom", "pois", or "nbinom".
#' @param innovation Innovation distribution: "pois", "bell", or "nbinom".
#' @param blocks Optional integer vector for block effects.
#' @param use_chibar Logical; if TRUE, use chi-bar-square for boundary test.
#' @param weights Optional weights for chi-bar-square mixture.
#' @param fit_null Optional pre-computed null fit.
#' @param fit_alt Optional pre-computed alternative fit.
#' @param ... Additional arguments passed to fit_inad.
#'
#' @return A list with class \code{"test_order_inad"} containing:
#' \describe{
#'   \item{method}{Inference method used (\code{"lrt"}).}
#'   \item{fit_null}{Fitted model under H0}
#'   \item{fit_alt}{Fitted model under H1}
#'   \item{statistic}{Test statistic value}
#'   \item{lrt_stat}{Likelihood ratio test statistic}
#'   \item{df}{Degrees of freedom}
#'   \item{p_value}{Chi-square p-value}
#'   \item{p_value_chibar}{Chi-bar-square p-value (if \code{use_chibar = TRUE})}
#'   \item{bic_null}{BIC under H0}
#'   \item{bic_alt}{BIC under H1}
#'   \item{bic_selected}{Which model BIC prefers}
#'   \item{table}{Two-row model comparison table}
#'   \item{settings}{Input and derived settings for the test}
#' }
#'
#' @details
#' The test compares nested INAD models of orders \code{order_null} and
#' \code{order_alt = order_null + 1} using:
#' \deqn{\lambda = 2(\ell_{alt} - \ell_{null})}
#' where \eqn{\ell_{null}} and \eqn{\ell_{alt}} are maximized log-likelihoods
#' under the null and alternative models.
#'
#' The default p-value uses the chi-square approximation with degrees of freedom
#' matching the number of additional dependence parameters introduced under the
#' higher-order model. When \code{use_chibar = TRUE}, a chi-bar-square mixture
#' p-value is also reported for boundary-aware inference.
#'
#' Missing-data inputs are supported through the same \code{na_action} options
#' available in \code{\link{fit_inad}}. If \code{y} has missing values and
#' \code{na_action} is not supplied via \code{...}, this function defaults to
#' \code{na_action = "marginalize"}.
#'
#' @references
#' Li, C. and Zimmerman, D.L. (2026). Integer-valued antedependence models for
#' longitudinal count data. \emph{Biostatistics}.
#'
#' @seealso \code{\link{fit_inad}}, \code{\link{bic_order_inad}},
#'   \code{\link{test_stationarity_inad}}
#'
#' @importFrom stats pchisq
#'
#' @examples
#' set.seed(1)
#' y <- simulate_inad(
#'   n_subjects = 40,
#'   n_time = 5,
#'   order = 1,
#'   thinning = "binom",
#'   innovation = "pois",
#'   alpha = 0.3,
#'   theta = 2
#' )
#' out <- test_order_inad(
#'   y,
#'   order_null = 0,
#'   order_alt = 1,
#'   thinning = "binom",
#'   innovation = "pois",
#'   max_iter = 20
#' )
#' out$statistic
#' @export
test_order_inad <- function(y, order_null = 0, order_alt = 1,
                           thinning = "binom", innovation = "pois",
                           blocks = NULL, use_chibar = TRUE, weights = NULL,
                           fit_null = NULL, fit_alt = NULL, ...) {
  if (!is.matrix(y)) y <- as.matrix(y)
  y_obs <- y[!is.na(y)]
  if (any(y_obs < 0) || any(y_obs != floor(y_obs))) stop("y must contain non-negative integers")
  n_subjects <- nrow(y); n_time <- ncol(y)
  if (!order_null %in% c(0, 1)) stop("order_null must be 0 or 1")
  if (!order_alt %in% c(1, 2)) stop("order_alt must be 1 or 2")
  if (order_alt != order_null + 1) stop("order_alt must be order_null + 1")
  if (order_alt >= n_time) stop("order_alt must be less than n_time")
  thinning <- match.arg(thinning, c("binom", "pois", "nbinom"))
  innovation <- match.arg(innovation, c("pois", "bell", "nbinom"))
  if (is.null(blocks)) blocks <- rep(1L, n_subjects)
  n_blocks <- length(unique(blocks))
  fit_args <- list(...)
  if (anyNA(y) && is.null(fit_args$na_action)) {
    fit_args$na_action <- "marginalize"
  }
  
  if (is.null(fit_null)) {
    fit_null <- do.call(
      fit_inad,
      c(
        list(y = y, order = order_null, thinning = thinning, innovation = innovation, blocks = blocks),
        fit_args
      )
    )
  }
  if (is.null(fit_alt)) {
    fit_alt <- do.call(
      fit_inad,
      c(
        list(y = y, order = order_alt, thinning = thinning, innovation = innovation, blocks = blocks),
        fit_args
      )
    )
  }
  
  logL_null <- fit_null$log_l; logL_alt <- fit_alt$log_l
  lrt_stat <- 2 * (logL_alt - logL_null)
  if (lrt_stat < 0) { warning("LRT negative, setting to 0"); lrt_stat <- 0 }
  
  df <- if (order_null == 0) n_time - 1 else n_time - 2
  p_value <- 1 - pchisq(lrt_stat, df = df)
  p_value_chibar <- if (use_chibar && df > 0) .chi_bar_pvalue(lrt_stat, df, weights) else NA_real_
  
  bic_null <- bic_inad(fit_null, n_subjects); bic_alt <- bic_inad(fit_alt, n_subjects)
  bic_selected <- if (bic_null <= bic_alt) "null" else "alt"
  
  table <- data.frame(model = c(paste0("Order ", order_null), paste0("Order ", order_alt)),
                      order = c(order_null, order_alt), logLik = c(logL_null, logL_alt),
                      n_params = c(.count_params_inad(order_null, n_time, n_blocks, thinning, innovation),
                                   .count_params_inad(order_alt, n_time, n_blocks, thinning, innovation)),
                      BIC = c(bic_null, bic_alt), stringsAsFactors = FALSE)
  
  result <- list(method = "lrt", fit_null = fit_null, fit_alt = fit_alt,
                 statistic = lrt_stat, lrt_stat = lrt_stat, df = df,
                 p_value = p_value, p_value_chibar = p_value_chibar,
                 bic_null = bic_null, bic_alt = bic_alt, bic_selected = bic_selected, table = table,
                 settings = list(order_null = order_null, order_alt = order_alt, thinning = thinning,
                                 innovation = innovation, n_subjects = n_subjects, n_time = n_time,
                                 n_blocks = n_blocks, use_chibar = use_chibar))
  class(result) <- "test_order_inad"
  result
}

#' @export
print.test_order_inad <- function(x, digits = 4, ...) {
  stat <- if (!is.null(x$statistic)) x$statistic else x$lrt_stat
  cat("\nLikelihood Ratio Test for INAD Model Order\n")
  cat("===========================================\n\n")
  cat("H0: Order", x$settings$order_null, " vs H1: Order", x$settings$order_alt, "\n\n")
  print(x$table, row.names = FALSE, digits = digits)
  cat("\nLRT:", round(stat, digits), " df:", x$df, " p-value:", format.pval(x$p_value, digits = digits))
  if (!is.na(x$p_value_chibar)) cat(" p-value(chi-bar):", format.pval(x$p_value_chibar, digits = digits))
  cat("\nBIC selects: Order", ifelse(x$bic_selected == "null", x$settings$order_null, x$settings$order_alt), "\n")
  invisible(x)
}

#' @keywords internal
.chi_bar_pvalue <- function(stat, r, weights = NULL) {
  if (r == 0) return(as.numeric(stat > 0))
  if (is.null(weights)) weights <- choose(r, 0:r) * 2^(-r)
  tails <- vapply(0:r, function(j) if (j == 0) as.numeric(stat > 0) else 1 - pchisq(stat, df = j), numeric(1))
  sum(weights * tails)
}

#' @keywords internal
.count_params_inad <- function(order, n_time, n_blocks, thinning, innovation) {
  n_alpha <- switch(as.character(order), "0" = 0, "1" = n_time - 1, "2" = 2 * (n_time - 2))
  n_theta <- n_time
  n_tau <- max(0, n_blocks - 1)
  n_nb_inno <- if (innovation == "nbinom") 1 else 0
  n_alpha + n_theta + n_tau + n_nb_inno
}

#' BIC-based order selection for INAD models
#'
#' Fits INAD models across candidate orders and reports BIC-based selection.
#' @param y Integer matrix.
#' @param max_order Maximum order (1 or 2).
#' @param thinning Thinning operator.
#' @param innovation Innovation distribution.
#' @param blocks Optional block assignments.
#' @param ... Additional arguments.
#' @return A list with class \code{"bic_order_inad"} containing:
#' \describe{
#'   \item{fits}{List of fitted INAD models by candidate order}
#'   \item{bic}{Named numeric vector of BIC values by order}
#'   \item{best_order}{Order with minimum BIC}
#'   \item{table}{Data frame with order, logLik, n_params, and BIC}
#'   \item{settings}{Input and derived settings used for selection}
#' }
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' y <- simulate_inad(
#'   n_subjects = 30,
#'   n_time = 5,
#'   order = 1,
#'   thinning = "binom",
#'   innovation = "pois",
#'   alpha = 0.3,
#'   theta = 2
#' )
#' ord <- bic_order_inad(y, max_order = 2, thinning = "binom", innovation = "pois", max_iter = 10)
#' ord$best_order
#' }
#' @export
bic_order_inad <- function(y, max_order = 2, thinning = "binom", innovation = "pois",
                           blocks = NULL, ...) {
  if (!is.matrix(y)) y <- as.matrix(y)
  n_subjects <- nrow(y); n_time <- ncol(y)
  if (is.null(blocks)) blocks <- rep(1L, n_subjects)
  n_blocks <- length(unique(blocks))
  max_order <- min(max_order, n_time - 1); orders <- 0:max_order
  fits <- lapply(orders, function(ord) fit_inad(y, order = ord, thinning = thinning, 
                                                 innovation = innovation, blocks = blocks, ...))
  names(fits) <- paste0("order_", orders)
  table <- data.frame(order = orders, logLik = sapply(fits, function(f) f$log_l),
                      n_params = sapply(orders, function(ord) .count_params_inad(ord, n_time, n_blocks, thinning, innovation)),
                      stringsAsFactors = FALSE)
  table$BIC <- -2 * table$logLik + table$n_params * log(n_subjects)
  bic_vals <- stats::setNames(table$BIC, paste0("order_", orders))
  result <- list(fits = fits, bic = bic_vals, table = table, best_order = orders[which.min(table$BIC)],
                 settings = list(thinning = thinning, innovation = innovation, n_subjects = n_subjects, 
                                 n_time = n_time, n_blocks = n_blocks))
  class(result) <- "bic_order_inad"
  result
}

#' @export
print.bic_order_inad <- function(x, digits = 4, ...) {
  cat("\nBIC Model Order Comparison for INAD\n====================================\n\n")
  print(x$table, row.names = FALSE, digits = digits)
  cat("\nBest order by BIC:", x$best_order, "\n")
  invisible(x)
}

# R/plot_methods.R
# plot() S3 methods for antedependence model fit objects and diagnostics.
#
# Provides:
#   plot.gau_fit       - parameter panel: mean / phi / sigma over time
#   plot.cat_fit       - marginal probability bar chart over time
#   plot.inad_fit      - alpha and theta parameter traces over time
#   plot.partial_corr  - heatmap of the partial correlation matrix

# Suppress R CMD check note for ggplot2 aesthetics
if (getRversion() >= "2.15.1") {
    utils::globalVariables(c("x", "y", "fill", "label"))
}

# ===========================================================================
# plot.gau_fit
# ===========================================================================

#' Plot estimated parameters of a Gaussian AD fit
#'
#' Produces a multi-panel base-graphics plot showing the estimated mean
#' (\eqn{\mu}), innovation standard deviation (\eqn{\sigma}), and (for
#' AD(1) or AD(2)) the antedependence coefficients (\eqn{\phi}) over time.
#'
#' @param x A \code{gau_fit} object from \code{\link{fit_gau}}.
#' @param ... Unused; present for S3 consistency.
#'
#' @return \code{x}, invisibly.
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' y <- simulate_gau(n_subjects = 40, n_time = 6, order = 1, phi = 0.4)
#' fit <- fit_gau(y, order = 1)
#' plot(fit)
#' }
#'
#' @export
plot.gau_fit <- function(x, ...) {
    ord  <- x$settings$order
    T    <- x$settings$n_time
    tpts <- seq_len(T)

    has_phi <- !is.null(x$phi) && ord >= 1

    n_panels <- if (has_phi) 3L else 2L
    old_par  <- graphics::par(no.readonly = TRUE)
    on.exit(graphics::par(old_par))
    graphics::par(mfrow = c(1L, n_panels), mar = c(4, 4, 3, 1))

    # Panel 1: mean
    mu_vals <- as.numeric(x$mu)
    graphics::plot(tpts, mu_vals, type = "b", pch = 16,
                   xlab = "Time", ylab = expression(mu[t]),
                   main = "Estimated Mean", xaxt = "n")
    graphics::axis(1, at = tpts)

    # Panel 2: sigma
    sig_vals <- as.numeric(x$sigma)
    graphics::plot(tpts, sig_vals, type = "b", pch = 16, col = "steelblue",
                   xlab = "Time", ylab = expression(sigma[t]),
                   main = "Innovation Std Dev", xaxt = "n",
                   ylim = c(0, max(sig_vals) * 1.1))
    graphics::axis(1, at = tpts)
    graphics::abline(h = 0, lty = 2, col = "grey70")

    # Panel 3: phi (if any)
    if (has_phi) {
        if (ord == 1) {
            phi_vals <- as.numeric(x$phi)
            # phi[1] is always 0 (reference), plot from t=2
            graphics::plot(2:T, phi_vals[2:T], type = "b", pch = 16,
                           col = "firebrick",
                           xlab = "Time", ylab = expression(phi[t]),
                           main = expression(paste("Antedep. Coeff. (", phi, ")")),
                           xaxt = "n",
                           ylim = c(-1, 1))
            graphics::axis(1, at = 2:T)
            graphics::abline(h = 0, lty = 2, col = "grey70")
        } else {
            phi_mat <- as.matrix(x$phi)   # 2 x T
            graphics::plot(2:T, phi_mat[1, 2:T], type = "b", pch = 16,
                           col = "firebrick",
                           xlab = "Time",
                           ylab = expression(phi[t]),
                           main = expression(paste("Antedep. Coeff. (", phi, ")")),
                           xaxt = "n",
                           ylim = c(-1, 1))
            if (T >= 3)
                graphics::lines(3:T, phi_mat[2, 3:T], type = "b", pch = 17,
                                col = "darkorange")
            graphics::axis(1, at = 2:T)
            graphics::abline(h = 0, lty = 2, col = "grey70")
            graphics::legend("topright",
                             legend = c(expression(phi[1][t]), expression(phi[2][t])),
                             col = c("firebrick", "darkorange"), pch = c(16, 17),
                             lty = 1, cex = 0.8)
        }
    }

    invisible(x)
}


# ===========================================================================
# plot.cat_fit
# ===========================================================================

#' Plot marginal probabilities of a categorical AD fit
#'
#' Displays a stacked bar chart of the estimated marginal category
#' probabilities at each time point.
#'
#' @param x A \code{cat_fit} object from \code{\link{fit_cat}}.
#' @param ... Unused.
#'
#' @return \code{x}, invisibly.
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' y <- simulate_cat(n_subjects = 80, n_time = 5, order = 1, n_categories = 2)
#' fit <- fit_cat(y, order = 1, n_categories = 2)
#' plot(fit)
#' }
#'
#' @export
plot.cat_fit <- function(x, ...) {
    ord    <- x$settings$order
    T      <- x$settings$n_time
    n_cat  <- x$settings$n_categories

    # Extract marginal probabilities at each time point
    marg <- x$marginal

    # Build a matrix: rows = categories, cols = time points
    prob_mat <- matrix(NA_real_, nrow = n_cat, ncol = T)

    get_t1_probs <- function(marg) {
        if (!is.null(marg[["t1"]])) return(as.numeric(marg[["t1"]]))
        if (!is.null(marg[[1]]))    return(as.numeric(marg[[1]]))
        NULL
    }

    t1_probs <- get_t1_probs(marg)
    if (!is.null(t1_probs) && length(t1_probs) == n_cat)
        prob_mat[, 1] <- t1_probs

    for (t in 2:T) {
        key <- paste0("t", t)
        if (!is.null(marg[[key]])) {
            prob_mat[, t] <- as.numeric(marg[[key]])
        } else {
            # Marginalise from transition probs using t1 marginal
            if (t == 2 && !is.null(x$marginal[["t2_given_1to1"]]) &&
                    !is.null(t1_probs)) {
                trans <- x$marginal[["t2_given_1to1"]]
                prob_mat[, 2] <- as.numeric(crossprod(trans, t1_probs))
            } else {
                trans_key <- paste0("t", t)
                if (!is.null(x$transition[[trans_key]]) && !is.null(prob_mat[, t - 1])) {
                    trans <- x$transition[[trans_key]]
                    if (is.matrix(trans))
                        prob_mat[, t] <- as.numeric(crossprod(trans, prob_mat[, t - 1]))
                }
            }
        }
    }

    # If any column is all NA fall back to uniform
    for (t in seq_len(T)) {
        if (any(is.na(prob_mat[, t])))
            prob_mat[, t] <- rep(1 / n_cat, n_cat)
    }

    cat_labels <- paste0("Cat ", seq_len(n_cat))
    col_pal    <- grDevices::hcl.colors(n_cat, palette = "Set2")

    old_par <- graphics::par(no.readonly = TRUE)
    on.exit(graphics::par(old_par))
    graphics::par(mar = c(4, 4, 3, 7))

    graphics::barplot(prob_mat,
                      names.arg = paste0("T", seq_len(T)),
                      col       = col_pal,
                      xlab      = "Time",
                      ylab      = "Probability",
                      main      = "Estimated Marginal Probabilities",
                      legend.text = cat_labels,
                      args.legend = list(x = "topright", bty = "n",
                                         inset = c(-0.22, 0), xpd = TRUE))

    invisible(x)
}


# ===========================================================================
# plot.inad_fit
# ===========================================================================

#' Plot estimated parameters of an INAD fit
#'
#' Produces a two-panel base-graphics plot of the estimated thinning
#' parameters (\eqn{\alpha}) and innovation means (\eqn{\theta}) over time.
#'
#' @param x An \code{inad_fit} object from \code{\link{fit_inad}}.
#' @param ... Unused.
#'
#' @return \code{x}, invisibly.
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' y <- simulate_inad(n_subjects = 60, n_time = 5, order = 1,
#'                    thinning = "binom", innovation = "pois",
#'                    alpha = 0.3, theta = 2)
#' fit <- fit_inad(y, order = 1, thinning = "binom",
#'                 innovation = "pois", max_iter = 20)
#' plot(fit)
#' }
#'
#' @export
plot.inad_fit <- function(x, ...) {
    T    <- x$settings$n_time
    tpts <- seq_len(T)
    ord  <- x$settings$order

    has_alpha <- !is.null(x$alpha) && ord >= 1

    n_panels <- if (has_alpha) 2L else 1L
    old_par  <- graphics::par(no.readonly = TRUE)
    on.exit(graphics::par(old_par))
    graphics::par(mfrow = c(1L, n_panels), mar = c(4, 4, 3, 1))

    # Panel 1: theta
    theta_vals <- as.numeric(x$theta)
    graphics::plot(tpts, theta_vals, type = "b", pch = 16, col = "steelblue",
                   xlab = "Time", ylab = expression(theta[t]),
                   main = "Innovation Mean (theta)", xaxt = "n",
                   ylim = c(0, max(theta_vals) * 1.15))
    graphics::axis(1, at = tpts)
    graphics::abline(h = 0, lty = 2, col = "grey70")

    # Panel 2: alpha
    if (has_alpha) {
        if (ord == 1) {
            alpha_vals <- as.numeric(x$alpha)
            graphics::plot(2:T, alpha_vals[2:T], type = "b", pch = 16,
                           col = "firebrick",
                           xlab = "Time", ylab = expression(alpha[t]),
                           main = "Thinning Parameter (alpha)", xaxt = "n",
                           ylim = c(0, 1))
            graphics::axis(1, at = 2:T)
            graphics::abline(h = c(0, 1), lty = 2, col = "grey70")
        } else {
            alpha_mat <- as.matrix(x$alpha)   # T x 2
            graphics::plot(2:T, alpha_mat[2:T, 1], type = "b", pch = 16,
                           col = "firebrick",
                           xlab = "Time", ylab = expression(alpha[t]),
                           main = "Thinning Parameters (alpha)", xaxt = "n",
                           ylim = c(0, 1))
            if (T >= 3)
                graphics::lines(3:T, alpha_mat[3:T, 2], type = "b", pch = 17,
                                col = "darkorange")
            graphics::axis(1, at = 2:T)
            graphics::abline(h = c(0, 1), lty = 2, col = "grey70")
            graphics::legend("topright",
                             legend = c(expression(alpha[1][t]), expression(alpha[2][t])),
                             col = c("firebrick", "darkorange"), pch = c(16, 17),
                             lty = 1, cex = 0.8)
        }
    }

    invisible(x)
}


# ===========================================================================
# plot.partial_corr
# ===========================================================================

#' Heatmap plot for a partial_corr object
#'
#' Displays the partial correlation matrix as a colour heatmap using base
#' graphics.  Upper triangle shows ordinary correlations; lower triangle
#' shows intervenor-adjusted partial correlations; the diagonal shows
#' standardised variances.
#'
#' @param x A \code{partial_corr} object from \code{\link{partial_corr}}.
#' @param ... Unused.
#'
#' @return \code{x}, invisibly.
#'
#' @examples
#' \donttest{
#' data("bolus_inad")
#' pc <- partial_corr(bolus_inad$y)
#' plot(pc)
#' }
#'
#' @export
plot.partial_corr <- function(x, ...) {
    T <- x$n_time

    # Build combined display matrix (upper = corr, lower = partial corr)
    mat <- x$correlation
    mat[lower.tri(mat)] <- x$partial_correlation[lower.tri(x$partial_correlation)]

    # Standardise diagonal to [-1, 1] range for colour mapping
    diag(mat) <- 0

    # Colour palette centred at 0
    pal <- grDevices::colorRampPalette(c("royalblue", "white", "firebrick"))(101)

    old_par <- graphics::par(no.readonly = TRUE)
    on.exit(graphics::par(old_par))
    graphics::par(mar = c(4, 4, 4, 5))

    # image() expects matrix[col, row] with col increasing left-to-right
    # and row increasing bottom-to-top, so we flip/transpose
    graphics::image(seq_len(T), seq_len(T), t(mat[T:1, ]),
                    zlim = c(-1, 1),
                    col  = pal,
                    xlab = "Time index j",
                    ylab = "Time index i",
                    main = "Correlation / Partial Correlation Matrix",
                    axes = FALSE)
    graphics::axis(1, at = seq_len(T), labels = seq_len(T))
    graphics::axis(2, at = seq_len(T), labels = T:1)

    # Cell labels
    for (i in seq_len(T)) {
        for (j in seq_len(T)) {
            val <- mat[i, j]
            if (is.finite(val))
                graphics::text(j, T + 1 - i,
                               labels = sprintf("%.2f", val),
                               cex = 0.65,
                               col = if (abs(val) > 0.5) "white" else "black")
        }
    }

    # Colour bar legend (approximate)
    legend_vals <- seq(-1, 1, length.out = 101)
    legend_cols <- pal
    graphics::image(x = c(T + 0.6, T + 0.9),
                    y = seq(1, T, length.out = 101),
                    z = matrix(legend_vals, nrow = 1),
                    col = legend_cols, add = TRUE)
    graphics::mtext("Corr.", side = 4, line = 1, cex = 0.8)

    graphics::box()
    invisible(x)
}

#' Likelihood ratio test for stationarity (INAD data)
#'
#' Tests whether time-varying INAD parameters can be constrained to be constant
#' over time.
#'
#' @param y Integer matrix with n_subjects rows and n_time columns.
#' @param order Model order (1 or 2).
#' @param thinning Thinning operator: "binom", "pois", or "nbinom".
#' @param innovation Innovation distribution: "pois", "bell", or "nbinom".
#' @param blocks Optional integer vector for block effects.
#' @param constrain Which parameters to constrain: "alpha", "theta", "both" for order 1;
#'   "alpha1", "alpha2", "alpha", "theta", "all" for order 2.
#' @param fit_unconstrained Optional pre-computed unconstrained fit.
#' @param verbose Logical; if TRUE, print progress.
#' @param ... Additional arguments.
#'
#' @return A list with class \code{"test_stationarity_inad"} containing:
#' \describe{
#'   \item{method}{Inference method used (\code{"lrt"}).}
#'   \item{fit_unconstrained}{Unconstrained INAD fit}
#'   \item{fit_constrained}{Constrained INAD fit}
#'   \item{constraint}{Human-readable null constraint description}
#'   \item{statistic}{Test statistic value}
#'   \item{lrt_stat}{Likelihood ratio test statistic}
#'   \item{df}{Degrees of freedom}
#'   \item{p_value}{Chi-square p-value}
#'   \item{bic_unconstrained}{BIC of unconstrained model}
#'   \item{bic_constrained}{BIC of constrained model}
#'   \item{bic_selected}{Model selected by BIC}
#'   \item{table}{Two-row model summary table}
#' }
#'
#' @details
#' For order 1, the test can constrain \code{alpha}, \code{theta}, or both.
#' For order 2, it can constrain \code{alpha1}, \code{alpha2},
#' \code{alpha} (both), \code{theta}, or all supported time-varying
#' parameters.
#'
#' Degrees of freedom are computed from the number of equality constraints
#' imposed under the null model relative to the unconstrained model.
#'
#' Missing-data inputs are supported through the same \code{na_action} options
#' available in \code{\link{fit_inad}}. If \code{y} has missing values and
#' \code{na_action} is not supplied via \code{...}, this function defaults to
#' \code{na_action = "marginalize"}.
#'
#' @references
#' Li, C. and Zimmerman, D.L. (2026). Integer-valued antedependence models for
#' longitudinal count data. \emph{Biostatistics}.
#'
#' @seealso \code{\link{run_stationarity_tests_inad}},
#'   \code{\link{test_order_inad}}, \code{\link{fit_inad}}
#'
#' @importFrom stats pchisq optim
#'
#' @examples
#' set.seed(1)
#' y <- simulate_inad(
#'   n_subjects = 30,
#'   n_time = 5,
#'   order = 1,
#'   thinning = "binom",
#'   innovation = "pois",
#'   alpha = 0.3,
#'   theta = 2
#' )
#' out <- test_stationarity_inad(
#'   y,
#'   order = 1,
#'   thinning = "binom",
#'   innovation = "pois",
#'   constrain = "both",
#'   max_iter = 20
#' )
#' out$p_value
#' @export
test_stationarity_inad <- function(y, order = 1, thinning = "binom", innovation = "pois",
                                  blocks = NULL, constrain = "both",
                                  fit_unconstrained = NULL, verbose = FALSE, ...) {
  if (!is.matrix(y)) y <- as.matrix(y)
  y_obs <- y[!is.na(y)]
  if (any(y_obs < 0) || any(y_obs != floor(y_obs))) stop("y must contain non-negative integers")
  n_subjects <- nrow(y); n_time <- ncol(y)
  if (!order %in% c(1, 2)) stop("order must be 1 or 2 for stationarity testing")
  if (order >= n_time) stop("order must be less than n_time")
  thinning <- match.arg(thinning, c("binom", "pois", "nbinom"))
  innovation <- match.arg(innovation, c("pois", "bell", "nbinom"))
  if (is.null(blocks)) blocks <- rep(1L, n_subjects)
  n_blocks <- length(unique(blocks))
  fit_args <- list(...)
  if (anyNA(y) && is.null(fit_args$na_action)) {
    fit_args$na_action <- "marginalize"
  }
  na_action_inference <- if (!is.null(fit_args$na_action)) fit_args$na_action else "fail"
  
  constraint_info <- .parse_constraint(constrain, order, n_time)
  
  if (is.null(fit_unconstrained)) {
    if (verbose) cat("Fitting unconstrained model...\n")
    fit_unconstrained <- do.call(
      fit_inad,
      c(
        list(y = y, order = order, thinning = thinning, innovation = innovation, blocks = blocks),
        fit_args
      )
    )
  }
  if (verbose) cat("Fitting constrained model...\n")
  fit_constrained <- .fit_constrained(
    y, order, thinning, innovation, blocks, constraint_info, fit_unconstrained,
    na_action = na_action_inference, ...
  )
  
  logL_uncon <- fit_unconstrained$log_l; logL_con <- fit_constrained$log_l
  lrt_stat <- 2 * (logL_uncon - logL_con)
  if (lrt_stat < 0) { warning("LRT negative, setting to 0"); lrt_stat <- 0 }
  df <- constraint_info$df
  p_value <- 1 - pchisq(lrt_stat, df = df)
  
  bic_uncon <- bic_inad(fit_unconstrained, n_subjects)
  n_params_con <- .count_params_inad(order, n_time, n_blocks, thinning, innovation) - df
  bic_con <- -2 * logL_con + n_params_con * log(n_subjects)
  bic_selected <- if (bic_uncon <= bic_con) "unconstrained" else "constrained"
  
  table <- data.frame(model = c("Unconstrained", paste("Constrained:", constraint_info$description)),
                      logLik = c(logL_uncon, logL_con),
                      n_params = c(.count_params_inad(order, n_time, n_blocks, thinning, innovation), n_params_con),
                      BIC = c(bic_uncon, bic_con), stringsAsFactors = FALSE)
  
  result <- list(method = "lrt", fit_unconstrained = fit_unconstrained, fit_constrained = fit_constrained,
                 constraint = constraint_info$description, statistic = lrt_stat, lrt_stat = lrt_stat, df = df, p_value = p_value,
                 bic_unconstrained = bic_uncon, bic_constrained = bic_con, bic_selected = bic_selected,
                 estimates = list(unconstrained = list(alpha = fit_unconstrained$alpha, theta = fit_unconstrained$theta, tau = fit_unconstrained$tau),
                                  constrained = list(alpha = fit_constrained$alpha, theta = fit_constrained$theta, tau = fit_constrained$tau)),
                 table = table, settings = list(order = order, thinning = thinning, innovation = innovation,
                                                n_subjects = n_subjects, n_time = n_time, n_blocks = n_blocks, constrain = constrain,
                                                na_action = na_action_inference))
  class(result) <- "test_stationarity_inad"
  result
}

#' @export
print.test_stationarity_inad <- function(x, digits = 4, ...) {
  stat <- if (!is.null(x$statistic)) x$statistic else x$lrt_stat
  cat("\nLikelihood Ratio Test for INAD Stationarity\n============================================\n\n")
  cat("Constraint: H0:", x$constraint, "\n\n")
  print(x$table, row.names = FALSE, digits = digits)
  cat("\nLRT:", round(stat, digits), " df:", x$df, " p-value:", format.pval(x$p_value, digits = digits))
  cat("\nBIC selects:", x$bic_selected, "\n")
  invisible(x)
}

#' @keywords internal
.parse_constraint <- function(constrain, order, n_time) {
  constrain <- tolower(constrain)
  if (order == 1) {
    if (length(constrain) == 1) {
      if (constrain %in% c("both", "all")) { alpha_const <- TRUE; theta_const <- TRUE }
      else if (constrain == "alpha") { alpha_const <- TRUE; theta_const <- FALSE }
      else if (constrain == "theta") { alpha_const <- FALSE; theta_const <- TRUE }
      else stop("Invalid constraint for order 1. Use: 'alpha', 'theta', or 'both'")
    } else { alpha_const <- "alpha" %in% constrain; theta_const <- "theta" %in% constrain }
    df <- (if (alpha_const) n_time - 2 else 0) + (if (theta_const) n_time - 1 else 0)
    desc <- paste(c(if(alpha_const) "alpha constant", if(theta_const) "theta constant"), collapse = ", ")
    list(order = 1, alpha_const = alpha_const, theta_const = theta_const, df = df, description = desc)
  } else {
    alpha1_const <- alpha2_const <- theta_const <- FALSE
    if (length(constrain) == 1) {
      if (constrain %in% c("both", "all")) { alpha1_const <- alpha2_const <- theta_const <- TRUE }
      else if (constrain == "alpha") { alpha1_const <- alpha2_const <- TRUE }
      else if (constrain == "alpha1") alpha1_const <- TRUE
      else if (constrain == "alpha2") alpha2_const <- TRUE
      else if (constrain == "theta") theta_const <- TRUE
      else stop("Invalid constraint for order 2")
    } else {
      alpha1_const <- "alpha1" %in% constrain; alpha2_const <- "alpha2" %in% constrain
      if ("alpha" %in% constrain) { alpha1_const <- alpha2_const <- TRUE }
      theta_const <- "theta" %in% constrain
    }
    df <- (if (alpha1_const) n_time - 3 else 0) + (if (alpha2_const) n_time - 3 else 0) + (if (theta_const) n_time - 1 else 0)
    desc <- paste(c(if(alpha1_const) "alpha1 constant", if(alpha2_const) "alpha2 constant", if(theta_const) "theta constant"), collapse = ", ")
    list(order = 2, alpha1_const = alpha1_const, alpha2_const = alpha2_const, theta_const = theta_const, df = df, description = desc)
  }
}

#' @keywords internal
.fit_constrained <- function(y, order, thinning, innovation, blocks, cinfo, init_from,
                             na_action = "fail", ...) {
  n <- nrow(y); N <- ncol(y); B <- length(unique(blocks))
  theta_default <- colMeans(y, na.rm = TRUE)
  theta_default[!is.finite(theta_default)] <- 1
  nb_inno_size_fixed <- NULL
  if (innovation == "nbinom") {
    nb_inno_size_fixed <- init_from$nb_inno_size
    if (is.null(nb_inno_size_fixed)) nb_inno_size_fixed <- rep(1, N)
    nb_inno_size_fixed <- as.numeric(nb_inno_size_fixed)
    if (length(nb_inno_size_fixed) == 1L) nb_inno_size_fixed <- rep(nb_inno_size_fixed, N)
    nb_inno_size_fixed <- pmax(nb_inno_size_fixed, 1e-8)
  }

  lambda_ok <- function(theta, tau) {
    mu <- if (innovation == "bell") {
      outer(rep(1, n), .bell_mean_from_theta(theta)) + matrix(tau[blocks], n, N)
    } else {
      outer(rep(1, n), theta) + matrix(tau[blocks], n, N)
    }
    all(is.finite(mu)) && all(mu > 0)
  }
  
  if (order == 1) {
    obj_fn <- function(par) {
      idx <- 1
      if (cinfo$alpha_const) { alpha <- c(0, rep(par[idx], N - 1)); idx <- idx + 1 }
      else { alpha <- c(0, par[idx:(idx + N - 2)]); idx <- idx + N - 1 }
      if (cinfo$theta_const) { theta <- rep(par[idx], N); idx <- idx + 1 }
      else { theta <- par[idx:(idx + N - 1)]; idx <- idx + N }
      tau <- if (B > 1) c(0, par[idx:(idx + B - 2)]) else 0
      alpha <- pmax(alpha, 0); if (thinning == "binom") alpha <- pmin(alpha, 0.999)
      theta <- pmax(theta, 1e-8)
      if (!lambda_ok(theta, tau)) return(1e20)
      -logL_inad(y = y, order = 1, thinning = thinning, innovation = innovation,
                 alpha = alpha, theta = theta, nb_inno_size = nb_inno_size_fixed,
                 blocks = blocks, tau = tau, na_action = na_action)
    }
    alpha_init <- if (!is.null(init_from$alpha)) init_from$alpha else c(0, rep(0.3, N - 1))
    theta_init <- if (!is.null(init_from$theta)) init_from$theta else theta_default
    tau_init <- if (!is.null(init_from$tau)) init_from$tau else rep(0, B)
    par0 <- c(); lower <- c(); upper <- c()
    if (cinfo$alpha_const) { par0 <- c(par0, mean(alpha_init[-1])); lower <- c(lower, 0); upper <- c(upper, if(thinning == "binom") 0.999 else 10) }
    else { par0 <- c(par0, alpha_init[-1]); lower <- c(lower, rep(0, N-1)); upper <- c(upper, rep(if(thinning == "binom") 0.999 else 10, N-1)) }
    if (cinfo$theta_const) { par0 <- c(par0, mean(theta_init)); lower <- c(lower, 1e-8); upper <- c(upper, 1e6) }
    else { par0 <- c(par0, theta_init); lower <- c(lower, rep(1e-8, N)); upper <- c(upper, rep(1e6, N)) }
    if (B > 1) { par0 <- c(par0, tau_init[-1]); lower <- c(lower, rep(-1e3, B-1)); upper <- c(upper, rep(1e3, B-1)) }
  } else {
    obj_fn <- function(par) {
      idx <- 1
      if (cinfo$alpha1_const) { alpha1 <- c(0, 0, rep(par[idx], N - 2)); idx <- idx + 1 }
      else { alpha1 <- c(0, 0, par[idx:(idx + N - 3)]); idx <- idx + N - 2 }
      if (cinfo$alpha2_const) { alpha2 <- c(0, 0, rep(par[idx], N - 2)); idx <- idx + 1 }
      else { alpha2 <- c(0, 0, par[idx:(idx + N - 3)]); idx <- idx + N - 2 }
      if (cinfo$theta_const) { theta <- rep(par[idx], N); idx <- idx + 1 }
      else { theta <- par[idx:(idx + N - 1)]; idx <- idx + N }
      tau <- if (B > 1) c(0, par[idx:(idx + B - 2)]) else 0
      alpha1 <- pmax(alpha1, 0); alpha2 <- pmax(alpha2, 0)
      if (thinning == "binom") { alpha1 <- pmin(alpha1, 0.999); alpha2 <- pmin(alpha2, 0.999) }
      theta <- pmax(theta, 1e-8)
      if (!lambda_ok(theta, tau)) return(1e20)
      -logL_inad(y = y, order = 2, thinning = thinning, innovation = innovation,
                 alpha = cbind(alpha1, alpha2), theta = theta, nb_inno_size = nb_inno_size_fixed,
                 blocks = blocks, tau = tau, na_action = na_action)
    }
    if (!is.null(init_from$alpha)) { au <- .unpack_alpha(2, init_from$alpha, N); alpha1_init <- au$a1; alpha2_init <- au$a2 }
    else { alpha1_init <- c(0, 0, rep(0.3, N-2)); alpha2_init <- c(0, 0, rep(0.1, N-2)) }
    theta_init <- if (!is.null(init_from$theta)) init_from$theta else theta_default
    tau_init <- if (!is.null(init_from$tau)) init_from$tau else rep(0, B)
    par0 <- c(); lower <- c(); upper <- c()
    if (cinfo$alpha1_const) { par0 <- c(par0, mean(alpha1_init[-(1:2)])); lower <- c(lower, 0); upper <- c(upper, if(thinning == "binom") 0.999 else 10) }
    else { par0 <- c(par0, alpha1_init[-(1:2)]); lower <- c(lower, rep(0, N-2)); upper <- c(upper, rep(if(thinning == "binom") 0.999 else 10, N-2)) }
    if (cinfo$alpha2_const) { par0 <- c(par0, mean(alpha2_init[-(1:2)])); lower <- c(lower, 0); upper <- c(upper, if(thinning == "binom") 0.999 else 10) }
    else { par0 <- c(par0, alpha2_init[-(1:2)]); lower <- c(lower, rep(0, N-2)); upper <- c(upper, rep(if(thinning == "binom") 0.999 else 10, N-2)) }
    if (cinfo$theta_const) { par0 <- c(par0, mean(theta_init)); lower <- c(lower, 1e-8); upper <- c(upper, 1e6) }
    else { par0 <- c(par0, theta_init); lower <- c(lower, rep(1e-8, N)); upper <- c(upper, rep(1e6, N)) }
    if (B > 1) { par0 <- c(par0, tau_init[-1]); lower <- c(lower, rep(-1e3, B-1)); upper <- c(upper, rep(1e3, B-1)) }
  }
  
  opt <- tryCatch(nloptr::nloptr(x0 = par0, eval_f = obj_fn, lb = lower, ub = upper,
                                  opts = list(algorithm = "NLOPT_LN_BOBYQA", xtol_rel = 1e-7, maxeval = 4000)),
                  error = function(e) optim(par0, obj_fn, method = "L-BFGS-B", lower = lower, upper = upper))
  sol <- if (inherits(opt, "nloptr")) opt$solution else opt$par
  log_l <- if (inherits(opt, "nloptr")) -opt$objective else -opt$value
  
  if (order == 1) {
    idx <- 1
    if (cinfo$alpha_const) { alpha <- c(0, rep(sol[idx], N - 1)); idx <- idx + 1 }
    else { alpha <- c(0, sol[idx:(idx + N - 2)]); idx <- idx + N - 1 }
    if (cinfo$theta_const) { theta <- rep(sol[idx], N); idx <- idx + 1 }
    else { theta <- sol[idx:(idx + N - 1)]; idx <- idx + N }
    tau <- if (B > 1) c(0, sol[idx:(idx + B - 2)]) else 0
    list(alpha = pmax(alpha, 0), theta = pmax(theta, 1e-8), tau = tau,
         nb_inno_size = nb_inno_size_fixed, log_l = log_l)
  } else {
    idx <- 1
    if (cinfo$alpha1_const) { alpha1 <- c(0, 0, rep(sol[idx], N - 2)); idx <- idx + 1 }
    else { alpha1 <- c(0, 0, sol[idx:(idx + N - 3)]); idx <- idx + N - 2 }
    if (cinfo$alpha2_const) { alpha2 <- c(0, 0, rep(sol[idx], N - 2)); idx <- idx + 1 }
    else { alpha2 <- c(0, 0, sol[idx:(idx + N - 3)]); idx <- idx + N - 2 }
    if (cinfo$theta_const) { theta <- rep(sol[idx], N); idx <- idx + 1 }
    else { theta <- sol[idx:(idx + N - 1)]; idx <- idx + N }
    tau <- if (B > 1) c(0, sol[idx:(idx + B - 2)]) else 0
    list(alpha = cbind(pmax(alpha1, 0), pmax(alpha2, 0)), theta = pmax(theta, 1e-8), tau = tau,
         nb_inno_size = nb_inno_size_fixed, log_l = log_l)
  }
}

#' Run all stationarity-related tests for INAD
#' @param y Integer matrix.
#' @param order Model order (1 or 2).
#' @param thinning Thinning operator.
#' @param innovation Innovation distribution.
#' @param blocks Optional block assignments.
#' @param verbose Logical.
#' @param ... Additional arguments.
#' @return A list with class "stationarity_tests_inad".
#'
#' @examples
#' set.seed(1)
#' y <- simulate_inad(
#'   n_subjects = 25,
#'   n_time = 5,
#'   order = 1,
#'   thinning = "binom",
#'   innovation = "pois",
#'   alpha = 0.3,
#'   theta = 2
#' )
#' out <- run_stationarity_tests_inad(
#'   y,
#'   order = 1,
#'   thinning = "binom",
#'   innovation = "pois",
#'   verbose = FALSE,
#'   max_iter = 15
#' )
#' out$summary
#' @export
run_stationarity_tests_inad <- function(y, order = 1, thinning = "binom", innovation = "pois",
                                        blocks = NULL, verbose = FALSE, ...) {
  if (!is.matrix(y)) y <- as.matrix(y)
  n_subjects <- nrow(y)
  fit_args <- list(...)
  if (anyNA(y) && is.null(fit_args$na_action)) {
    fit_args$na_action <- "marginalize"
  }
  if (is.null(blocks)) blocks <- rep(1L, n_subjects)
  if (verbose) cat("Fitting unconstrained model...\n")
  fit_uncon <- do.call(
    fit_inad,
    c(
      list(y = y, order = order, thinning = thinning, innovation = innovation, blocks = blocks),
      fit_args
    )
  )
  tests <- if (order == 1) c("alpha", "theta", "both") else c("alpha1", "alpha2", "alpha", "theta", "all")
  results <- list()
  for (test in tests) {
    if (verbose) cat("Testing:", test, "constant...\n")
    results[[test]] <- test_stationarity_inad(y, order, thinning, innovation, blocks, test, fit_uncon, FALSE, ...)
  }
  summary_table <- data.frame(constraint = tests, method = sapply(results, function(r) r$method),
                              statistic = sapply(results, function(r) r$statistic),
                              df = sapply(results, function(r) r$df),
                              LRT = sapply(results, function(r) r$lrt_stat),
                              p_value = sapply(results, function(r) r$p_value),
                              BIC_selected = sapply(results, function(r) r$bic_selected), stringsAsFactors = FALSE)
  output <- list(method = "lrt", fit_unconstrained = fit_uncon, tests = results, summary = summary_table,
                 settings = list(order = order, thinning = thinning, innovation = innovation,
                                 n_subjects = n_subjects, n_time = ncol(y)))
  class(output) <- "stationarity_tests_inad"
  output
}

#' @export
print.stationarity_tests_inad <- function(x, digits = 4, ...) {
  cat("\nStationarity Tests for INAD Model\n==================================\n\n")
  print(x$summary, row.names = FALSE, digits = digits)
  invisible(x)
}

#' Bell numbers
#'
#' Compute Bell numbers up to order z using a truncated series representation.
#'
#' This function is for internal use. The results are stored in \code{.Bell_num}
#' and used by the Bell distribution functions.
#'
#' @param z nonnegative integer giving the maximum order.
#'
#' @return numeric vector of length \code{z + 1} containing B_0, ..., B_z.
#' @keywords internal
Bz <- function(z) {
    v <- seq(0, z, by = 1L)
    k <- seq(0, 20, by = 1L)
    sapply(
        v,
        function(n) {
            sum(k ^ n / factorial(k)) / exp(1)
        }
    )
}

.BELL_MAX_Z <- 236L
.Bell_num <- Bz(.BELL_MAX_Z)

# Precompute log of Bell numbers for numerical stability
.log_Bell_num <- log(.Bell_num)

#' The Bell distribution
#'
#' Density, distribution function, quantile function and random generation
#' for the Bell distribution with parameter \code{theta}.
#'
#' Let \eqn{B_x} denote the xth Bell number. The Bell distribution has
#' probability mass function
#' \deqn{
#' P(X = x) = \theta^x \exp(-\exp(\theta) + 1) \frac{B_x}{x!},
#' }
#' for nonnegative integers \eqn{x} and \eqn{\theta \ge 0}.
#'
#' For \eqn{\theta > 0}, the Bell mean is \eqn{E[X] = \theta e^\theta}.
#' At \eqn{\theta = 0}, the distribution is degenerate at 0.
#'
#' The functions follow the standard naming used in base R:
#' \code{dbell} for the density, \code{pbell} for the distribution function,
#' \code{qbell} for the quantile function and \code{rbell} for random
#' generation.
#'
#' @name Bell
#' @importFrom stats rmultinom
#' @param x vector of nonnegative integers (for \code{dbell} and \code{pbell}).
#' @param p numeric vector of probabilities between 0 and 1 inclusive (for \code{qbell}).
#' @param n number of observations to generate (for \code{rbell}).
#' @param theta scalar nonnegative Bell parameter.
#' @param log logical; if TRUE, probabilities p are given as log(p).
#' @param max_z maximum support value used for approximation in
#'   \code{rbell} and \code{qbell}.
#'
#' @return
#' For \code{dbell}, a numeric vector of probabilities.
#' For \code{pbell}, a numeric vector of cumulative probabilities.
#' For \code{qbell}, an integer vector of quantiles.
#' For \code{rbell}, an integer vector of random values.
#'
#' @examples
#' dbell(0:5, theta = 1)
#' pbell(0:5, theta = 1)
#' qbell(c(0.25, 0.5, 0.9), theta = 1)
#' set.seed(1)
#' rbell(10, theta = 1)
#'
#' @export
dbell <- function(x, theta, log = FALSE) {
    x_int <- as.integer(x)
    if (any(x_int < 0L)) {
        stop("x must be nonnegative")
    }
    if (any(x_int > .BELL_MAX_Z)) {
        stop("x exceeds precomputed Bell numbers; increase .BELL_MAX_Z if needed")
    }
    theta <- .check_bell_theta(theta)

    if (theta == 0) {
        log_p0 <- ifelse(x_int == 0L, 0, -Inf)
        if (log) return(log_p0)
        return(exp(log_p0))
    }

    # Use log-scale computation to avoid overflow for large x
    # log P(X=x) = x*log(theta) - exp(theta) + 1 + log(B_x) - log(x!)
    log_p <- x_int * log(theta) - exp(theta) + 1 + .log_Bell_num[x_int + 1L] - lfactorial(x_int)

    if (log) {
        return(log_p)
    }
    exp(log_p)
}

#' @rdname Bell
#' @export
pbell <- function(x, theta) {
    theta <- .check_bell_theta(theta)
    x_int <- as.integer(x)
    if (any(x_int < 0L)) {
        stop("x must be nonnegative")
    }
    sapply(
        x_int,
        function(z) {
            sum(dbell(0:z, theta))
        }
    )
}

#' @rdname Bell
#' @export
rbell <- function(n, theta, max_z = 100L) {
    theta <- .check_bell_theta(theta)
    if (length(n) != 1L || n < 0L) {
        stop("n must be a nonnegative integer")
    }
    if (max_z > .BELL_MAX_Z) {
        stop("max_z exceeds precomputed Bell numbers; increase .BELL_MAX_Z if needed")
    }
    z_vals <- 0:max_z
    p <- dbell(z_vals, theta)
    p <- p / sum(p)
    if (n == 0L) {
        return(integer(0L))
    }
    sampling <- rmultinom(n, size = 1L, prob = p)
    out <- integer(n)
    for (i in seq_len(n)) {
        out[i] <- which(sampling[, i] == 1L) - 1L
    }
    out
}

#' @rdname Bell
#' @export
qbell <- function(p, theta, max_z = 100L) {
    theta <- .check_bell_theta(theta)
    if (any(p < 0 | p > 1)) {
        stop("p must be between 0 and 1 inclusive")
    }
    if (max_z > .BELL_MAX_Z) {
        stop("max_z exceeds precomputed Bell numbers; increase .BELL_MAX_Z if needed")
    }
    z_vals <- 0:max_z
    pmf <- dbell(z_vals, theta)
    pmf <- pmf / sum(pmf)
    cdf <- cumsum(pmf)
    sapply(
        p,
        function(prob) {
            which(cdf >= prob)[1L] - 1L
        }
    )
}

#' @keywords internal
.check_bell_theta <- function(theta) {
    if (length(theta) != 1L || !is.finite(theta) || theta < 0) {
        stop("theta must be a finite scalar >= 0")
    }
    as.numeric(theta)
}

test_that("dbell probabilities approximately sum to one", {
    theta <- 1
    z_vals <- 0:.BELL_MAX_Z
    p <- dbell(z_vals, theta)
    s <- sum(p)
    expect_true(s > 0.99)
    expect_true(s < 1.01)
})

test_that("pbell is nondecreasing and bounded between zero and one", {
    theta <- 1
    x_vals <- 0:20
    F_vals <- pbell(x_vals, theta)
    expect_true(all(diff(F_vals) >= -1e-12))
    expect_true(all(F_vals >= 0))
    expect_true(all(F_vals <= 1))
})

test_that("rbell returns nonnegative integer values", {
    set.seed(1)
    n <- 50
    theta <- 1
    x <- rbell(n, theta = theta)
    expect_equal(length(x), n)
    expect_true(is.integer(x))
    expect_true(all(x >= 0))
})

test_that("rbell mean is close to theoretical mean for large n", {
    set.seed(2)
    n <- 5000
    theta <- 1
    x <- rbell(n, theta = theta)
    sample_mean <- mean(x)
    theo_mean <- exp(theta)
    expect_equal(sample_mean, theo_mean, tolerance = 0.2)
})

test_that("qbell inverts pbell approximately", {
    theta <- 1
    probs <- c(0.1, 0.5, 0.9)
    q <- qbell(probs, theta = theta, max_z = 80L)
    F_q <- pbell(q, theta = theta)
    expect_true(all(F_q >= probs))
    expect_true(all(F_q - probs < 0.1))
})

# R/generics.R
# S3 generic methods for antedependence model fit objects:
#   logLik, coef, nobs, fitted, residuals
#
# Once logLik methods are registered, stats::AIC() and stats::BIC() work
# automatically for all three fit families without any extra code.

# ===========================================================================
# logLik
# ===========================================================================

#' Log-likelihood for antedependence model fits
#'
#' Extracts the maximised log-likelihood from a fitted antedependence model
#' and returns a \code{"logLik"} object compatible with \code{\link[stats]{AIC}}
#' and \code{\link[stats]{BIC}}.
#'
#' @param object A fitted model object of class \code{gau_fit},
#'   \code{cat_fit}, or \code{inad_fit}.
#' @param ... Unused; present for S3 consistency.
#'
#' @return A scalar of class \code{"logLik"} with attributes
#'   \code{df} (number of free parameters) and \code{nobs}
#'   (number of subjects).
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' y <- simulate_gau(n_subjects = 40, n_time = 5, order = 1, phi = 0.4)
#' fit <- fit_gau(y, order = 1)
#' logLik(fit)
#' AIC(fit)
#' BIC(fit)
#' }
#'
#' @name logLik.antedep
NULL

#' @rdname logLik.antedep
#' @export
logLik.gau_fit <- function(object, ...) {
    ll <- object$log_l
    attr(ll, "df")   <- object$n_params
    attr(ll, "nobs") <- object$settings$n_subjects
    class(ll) <- "logLik"
    ll
}

#' @rdname logLik.antedep
#' @export
logLik.cat_fit <- function(object, ...) {
    ll <- object$log_l
    attr(ll, "df")   <- object$n_params
    attr(ll, "nobs") <- object$settings$n_subjects
    class(ll) <- "logLik"
    ll
}

#' @rdname logLik.antedep
#' @export
logLik.inad_fit <- function(object, ...) {
    ll <- object$log_l
    attr(ll, "df")   <- object$n_params
    attr(ll, "nobs") <- object$settings$n_subjects
    class(ll) <- "logLik"
    ll
}

# ===========================================================================
# nobs
# ===========================================================================

#' Number of observations for antedependence model fits
#'
#' @param object A fitted model object of class \code{gau_fit},
#'   \code{cat_fit}, or \code{inad_fit}.
#' @param ... Unused.
#'
#' @return Integer scalar: number of subjects.
#'
#' @name nobs.antedep
NULL

#' @rdname nobs.antedep
#' @importFrom stats nobs
#' @export
nobs.gau_fit <- function(object, ...) object$settings$n_subjects

#' @rdname nobs.antedep
#' @export
nobs.cat_fit <- function(object, ...) object$settings$n_subjects

#' @rdname nobs.antedep
#' @export
nobs.inad_fit <- function(object, ...) object$settings$n_subjects

# ===========================================================================
# coef
# ===========================================================================

#' Extract model coefficients from antedependence fits
#'
#' Returns the estimated parameters of a fitted antedependence model as a
#' named list.
#'
#' @param object A fitted model object of class \code{gau_fit},
#'   \code{cat_fit}, or \code{inad_fit}.
#' @param ... Unused.
#'
#' @return For \code{gau_fit}: a named list with elements \code{mu},
#'   \code{phi}, \code{sigma}, and (if a block effect was estimated)
#'   \code{tau}.
#'   For \code{cat_fit}: a list with \code{marginal} and \code{transition}
#'   probability arrays.
#'   For \code{inad_fit}: a named list with elements \code{alpha},
#'   \code{theta}, and (if applicable) \code{tau} and \code{nb_inno_size}.
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' y <- simulate_gau(n_subjects = 40, n_time = 5, order = 1, phi = 0.4)
#' fit <- fit_gau(y, order = 1)
#' coef(fit)
#' }
#'
#' @name coef.antedep
NULL

#' @rdname coef.antedep
#' @export
coef.gau_fit <- function(object, ...) {
    out <- list()
    if (!is.null(object$mu))    out$mu    <- object$mu
    if (!is.null(object$phi))   out$phi   <- object$phi
    if (!is.null(object$sigma)) out$sigma <- object$sigma
    if (!is.null(object$tau) && any(as.numeric(object$tau) != 0))
        out$tau <- object$tau
    out
}

#' @rdname coef.antedep
#' @export
coef.cat_fit <- function(object, ...) {
    list(marginal = object$marginal, transition = object$transition)
}

#' @rdname coef.antedep
#' @export
coef.inad_fit <- function(object, ...) {
    out <- list()
    if (!is.null(object$alpha))         out$alpha         <- object$alpha
    if (!is.null(object$theta))         out$theta         <- object$theta
    if (!is.null(object$tau) && any(as.numeric(object$tau) != 0))
        out$tau <- object$tau
    if (!is.null(object$nb_inno_size))  out$nb_inno_size  <- object$nb_inno_size
    out
}

# ===========================================================================
# fitted / residuals  (Gaussian only -- meaningful for complete-data fits)
# ===========================================================================

#' Fitted values and residuals for Gaussian AD fits
#'
#' Computes fitted (conditional mean) values and residuals for a
#' complete-data Gaussian antedependence fit.  The fitted value for subject
#' \eqn{s} at time \eqn{t} is the conditional mean
#' \eqn{E[Y_{st} | Y_{s,t-p}, \ldots, Y_{s,t-1}]}.
#'
#' @param object A \code{gau_fit} object (complete-data fit only).
#' @param y The original data matrix used to produce \code{object}
#'   (n_subjects \eqn{\times} n_time).  Required because the fit object
#'   does not store the raw data.
#' @param ... Unused.
#'
#' @return A numeric matrix of the same dimensions as \code{y}.
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' y <- simulate_gau(n_subjects = 40, n_time = 5, order = 1, phi = 0.4)
#' fit <- fit_gau(y, order = 1)
#' yhat <- fitted(fit, y)
#' resid <- residuals(fit, y)
#' }
#'
#' @name fitted.gau_fit
NULL

#' @rdname fitted.gau_fit
#' @export
fitted.gau_fit <- function(object, y, ...) {
    if (missing(y)) stop("'y' must be supplied; it is not stored in the fit object.")
    if (!is.matrix(y)) y <- as.matrix(y)
    n <- nrow(y)
    T <- ncol(y)
    ord <- object$settings$order
    mu  <- as.numeric(object$mu)
    yhat <- matrix(NA_real_, nrow = n, ncol = T)

    # block mean shifts
    tau_vec <- rep(0, n)
    if (!is.null(object$settings$blocks) && !is.null(object$tau)) {
        tau_vec <- as.numeric(object$tau)[object$settings$blocks]
    }
    m <- function(t) mu[t] + tau_vec   # length-n mean vector at time t

    yhat[, 1] <- m(1)

    if (ord == 0) {
        for (t in seq_len(T)) yhat[, t] <- m(t)
        return(yhat)
    }

    if (ord == 1) {
        phi <- as.numeric(object$phi)
        for (t in 2:T)
            yhat[, t] <- m(t) + phi[t] * (y[, t - 1] - m(t - 1))
        return(yhat)
    }

    if (ord == 2) {
        phi <- as.matrix(object$phi)   # 2 x T
        yhat[, 2] <- m(2) + phi[1, 2] * (y[, 1] - m(1))
        for (t in 3:T)
            yhat[, t] <- m(t) +
                phi[1, t] * (y[, t - 1] - m(t - 1)) +
                phi[2, t] * (y[, t - 2] - m(t - 2))
        return(yhat)
    }

    yhat
}

#' @rdname fitted.gau_fit
#' @importFrom stats fitted
#' @export
residuals.gau_fit <- function(object, y, ...) {
    if (missing(y)) stop("'y' must be supplied; it is not stored in the fit object.")
    if (!is.matrix(y)) y <- as.matrix(y)
    y - fitted(object, y)
}

# ===========================================================================
# as.ts  (simulate_* output -> ts / mts)
# ===========================================================================

#' Convert antedependence simulation output to a time-series object
#'
#' Converts the panel matrix returned by \code{\link{simulate_gau}},
#' \code{\link{simulate_cat}}, or \code{\link{simulate_inad}} into an
#' \code{mts} / \code{\link[stats]{ts}} object suitable for time-series
#' functions.  Rows of the original matrix (subjects) become the columns of
#' the returned object; columns (time points) become the rows.
#'
#' @param x A matrix of class \code{gau_sim}, \code{cat_sim}, or
#'   \code{inad_sim} as returned by the corresponding \code{simulate_*}
#'   function.
#' @param ... Additional arguments passed to \code{\link[stats]{ts}}
#'   (e.g., \code{start}, \code{frequency}).
#'
#' @return A \code{ts} (\code{mts}) object with one column per subject.
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' y <- simulate_gau(n_subjects = 5, n_time = 10, order = 1, phi = 0.4)
#' y_ts <- as.ts(y)
#' class(y_ts)    # "mts" "ts" "matrix" "array"
#' dim(y_ts)      # 10 x 5
#' }
#'
#' @name as.ts.antedep_sim
NULL

#' @rdname as.ts.antedep_sim
#' @importFrom stats as.ts
#' @export
as.ts.gau_sim <- function(x, ...) stats::ts(t(unclass(x)), ...)

#' @rdname as.ts.antedep_sim
#' @export
as.ts.cat_sim <- function(x, ...) stats::ts(t(unclass(x)), ...)

#' @rdname as.ts.antedep_sim
#' @export
as.ts.inad_sim <- function(x, ...) stats::ts(t(unclass(x)), ...)

# ===========================================================================
# deviance
# ===========================================================================

#' Deviance for antedependence model fits
#'
#' Returns the deviance \eqn{-2 \ell} for a fitted antedependence model.
#'
#' @param object A fitted model object of class \code{gau_fit},
#'   \code{cat_fit}, or \code{inad_fit}.
#' @param ... Unused.
#'
#' @return A scalar deviance value.
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' y <- simulate_gau(n_subjects = 40, n_time = 5, order = 1, phi = 0.4)
#' fit <- fit_gau(y, order = 1)
#' deviance(fit)
#' }
#'
#' @name deviance.antedep
NULL

#' @rdname deviance.antedep
#' @importFrom stats deviance
#' @export
deviance.gau_fit <- function(object, ...) -2 * as.numeric(logLik(object))

#' @rdname deviance.antedep
#' @export
deviance.cat_fit <- function(object, ...) -2 * as.numeric(logLik(object))

#' @rdname deviance.antedep
#' @export
deviance.inad_fit <- function(object, ...) -2 * as.numeric(logLik(object))

# ===========================================================================
# confint
# ===========================================================================

#' Confidence intervals for antedependence model fits
#'
#' Computes Wald confidence intervals for the parameters of a fitted
#' antedependence model by delegating to the corresponding
#' \code{ci_gau}, \code{ci_cat}, or \code{ci_inad} function.
#'
#' @param object A fitted model object of class \code{gau_fit},
#'   \code{cat_fit}, or \code{inad_fit}.
#' @param parm Unused (all parameters are returned).
#' @param level Confidence level; default \code{0.95}.
#' @param y (\code{inad_fit} only) The original data matrix used for
#'   fitting, required by \code{\link{ci_inad}}.
#' @param ... Passed to the underlying \code{ci_*} function.
#'
#' @return For \code{gau_fit} and \code{cat_fit}: a matrix with columns
#'   \code{lower} and \code{upper} and one row per parameter.
#'   For \code{inad_fit}: the \code{inad_ci} list returned by
#'   \code{\link{ci_inad}}.
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' y <- simulate_gau(n_subjects = 40, n_time = 5, order = 1, phi = 0.4)
#' fit <- fit_gau(y, order = 1)
#' confint(fit)
#' }
#'
#' @name confint.antedep
NULL

#' @rdname confint.antedep
#' @importFrom stats confint
#' @export
confint.gau_fit <- function(object, parm, level = 0.95, ...) {
    ci <- ci_gau(fit = object, level = level)
    rows <- do.call(rbind, Filter(Negate(is.null), list(ci$mu, ci$phi, ci$sigma)))
    if (is.null(rows)) return(matrix(NA_real_, nrow = 0, ncol = 2,
                                     dimnames = list(NULL, c("lower", "upper"))))
    mat <- matrix(c(rows$lower, rows$upper), ncol = 2,
                  dimnames = list(rows$param, c("lower", "upper")))
    mat
}

#' @rdname confint.antedep
#' @export
confint.cat_fit <- function(object, parm, level = 0.95, y = NULL, ...) {
    ci_cat(fit = object, y = y, level = level)
}

#' @rdname confint.antedep
#' @export
confint.inad_fit <- function(object, parm, level = 0.95, y, ...) {
    if (missing(y)) {
        stop(
            "confint() for inad_fit requires the original data matrix 'y'; ",
            "call ci_inad(y, fit, ...) directly.",
            call. = FALSE
        )
    }
    ci_inad(y = y, fit = object, level = level, ...)
}

# ===========================================================================
# vcov
# ===========================================================================

#' Variance-covariance matrix for Gaussian AD fits
#'
#' Returns an approximate variance-covariance matrix for the estimated
#' parameters of a \code{gau_fit} object.  The matrix is block-diagonal
#' (asymptotically exact for the Gaussian AD model given the factored
#' likelihood) with entries derived from the closed-form standard errors
#' used by \code{\link{ci_gau}}.
#'
#' @param object A \code{gau_fit} object.
#' @param ... Unused.
#'
#' @return A named square matrix of parameter variances (diagonal entries)
#'   and covariances (off-diagonal entries, all zero in the current
#'   implementation).
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' y <- simulate_gau(n_subjects = 40, n_time = 5, order = 1, phi = 0.4)
#' fit <- fit_gau(y, order = 1)
#' vcov(fit)
#' }
#'
#' @importFrom stats vcov
#' @export
vcov.gau_fit <- function(object, ...) {
    ci <- ci_gau(fit = object, level = 0.95)
    rows <- do.call(rbind, Filter(Negate(is.null), list(ci$mu, ci$phi, ci$sigma)))
    if (is.null(rows)) return(matrix(NA_real_, 0, 0))
    vars <- rows$se^2
    names(vars) <- rows$param
    diag(vars)
}

test_that("new packaged datasets have expected structure", {
  data("cattle_growth", package = "antedep")
  data("cochlear_implant", package = "antedep")
  data("labor_force_cat", package = "antedep")
  data("race_100km", package = "antedep")

  expect_true(is.list(cattle_growth))
  expect_true(is.matrix(cattle_growth$y))
  expect_equal(ncol(cattle_growth$y), 11)
  expect_equal(length(cattle_growth$blocks), nrow(cattle_growth$y))

  expect_true(is.list(cochlear_implant))
  expect_true(is.matrix(cochlear_implant$y))
  expect_equal(ncol(cochlear_implant$y), 4)
  expect_equal(length(cochlear_implant$blocks), nrow(cochlear_implant$y))
  expect_true(anyNA(cochlear_implant$y))

  expect_true(is.list(labor_force_cat))
  expect_true(is.matrix(labor_force_cat$y))
  expect_type(labor_force_cat$y, "integer")
  expect_equal(ncol(labor_force_cat$y), 5)
  expect_equal(labor_force_cat$n_categories, 2L)
  expect_false(anyNA(labor_force_cat$y))
  expect_equal(nrow(labor_force_cat$y), sum(labor_force_cat$counts$Count))
  expect_equal(labor_force_cat$time, 1967:1971)

  expect_true(is.list(race_100km))
  expect_true(is.matrix(race_100km$y))
  expect_equal(ncol(race_100km$y), 10)
  expect_equal(nrow(race_100km$y), length(race_100km$subject))
  expect_equal(length(race_100km$age), nrow(race_100km$y))
  expect_equal(race_100km$time, 1:10)
})

---
title: "Introduction to antedep"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to antedep}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(antedep)
set.seed(123)
```

## Overview

`antedep` supports three model families:

- AD (Gaussian) for continuous longitudinal outcomes
- INAD for count longitudinal outcomes
- CAT (categorical AD) for discrete state longitudinal outcomes

Packaged datasets include `bolus_inad`, `cattle_growth`, `cochlear_implant`, `labor_force_cat`, and `race_100km`.

```{r eval=FALSE}
data("race_100km")
dim(race_100km$y)
colMeans(race_100km$y)
```

## Production-Readiness Matrix

| Model | Data type | Complete-data fit/logLik | Missing-data fit/logLik | Notes |
|---|---|---|---|---|
| AD | Continuous | Ready | Ready (`fit_gau`, `logL_gau`) | Missing-data fit supports EM / observed-data likelihood modes |
| INAD | Counts | Ready | Ready (`fit_inad`, `logL_inad`) | Missing-data fit supports `na_action = "marginalize"` |
| CAT | Categorical states | Ready | Ready (`fit_cat`, `logL_cat`) | Missing-data fit supports orders 0, 1, 2 |

## Visualizing Longitudinal Data

```{r, warning=FALSE, message=FALSE, fig.width=10, fig.height=4.5, out.width='100%'}
data("bolus_inad")
plot_profile(
  bolus_inad$y,
  blocks = bolus_inad$blocks,
  block_labels = c("Group 1", "Group 2"),
  title = "Bolus Longitudinal Profiles"
)
```

```{r eval=FALSE}
# More detailed dependence diagnostic
plot_prism(bolus_inad$y)
```

## AD Workflow (Continuous Outcomes)

### Complete-data fit and order comparison

```{r, warning=FALSE, message=FALSE}
y_gau <- simulate_gau(n_subjects = 20, n_time = 4, order = 1, phi = 0.5)
fit_gau1 <- fit_gau(y_gau, order = 1)
fit_gau2 <- fit_gau(y_gau, order = 2)

c(order1_logLik = fit_gau1$log_l, order2_logLik = fit_gau2$log_l)
bic_order_gau(y_gau, max_order = 2)$table
```

```{r}
# Example AD order test
test_order_gau(y_gau, p = 0, q = 1)$p_value
```

### Missing-data AD fit

```{r}
y_gau_miss <- y_gau
y_gau_miss[sample(length(y_gau_miss), 8)] <- NA

fit_gau_em <- fit_gau(y_gau_miss, order = 1, na_action = "em", em_max_iter = 10)
fit_gau_cc <- fit_gau(y_gau_miss, order = 1, na_action = "complete")

c(em_logLik = fit_gau_em$log_l, complete_case_logLik = fit_gau_cc$log_l)
fit_gau_em$pct_missing
```

### AD mean and covariance structure tests (complete data)

These AD inference tests currently require complete data.

```{r}
# One-sample and two-sample mean-profile tests
mu0 <- colMeans(y_gau)
blocks_gau <- rep(1:2, each = nrow(y_gau) / 2)

test_one_sample_gau(y_gau, mu0 = mu0, p = 1)$p_value
test_two_sample_gau(y_gau, blocks = blocks_gau, p = 1)$p_value
```

```{r}
# Wald contrast test: H0 that adjacent means are equal
C <- diff(diag(ncol(y_gau)))
test_contrast_gau(y_gau, C = C, p = 1)$p_value
```

```{r}
# Covariance homogeneity across groups
test_homogeneity_gau(y_gau, blocks = blocks_gau, p = 1)$p_value
```

## INAD Workflow (Count Outcomes)

### Complete-data INAD fit

```{r}
y_inad <- simulate_inad(
  n_subjects = 20,
  n_time = 4,
  order = 1,
  thinning = "binom",
  innovation = "pois",
  alpha = 0.4,
  theta = 3
)

fit_inad1 <- fit_inad(y_inad, order = 1, thinning = "binom", innovation = "pois")
fit_inad1$log_l
bic_order_inad(y_inad, max_order = 2, thinning = "binom", innovation = "pois")$table
```

### Missing-data INAD fit

```{r}
y_inad_miss <- y_inad
y_inad_miss[sample(length(y_inad_miss), 10)] <- NA

fit_inad_miss <- fit_inad(
  y_inad_miss,
  order = 1,
  thinning = "binom",
  innovation = "pois",
  na_action = "marginalize",
  max_iter = 10
)

fit_inad_miss$log_l
fit_inad_miss$pct_missing
```

```{r}
# Missing-data LRTs (observed-data likelihood)
blocks_inad <- rep(1:2, each = nrow(y_inad_miss) / 2)
test_order_inad(y_inad_miss, order_null = 0, order_alt = 1, blocks = blocks_inad)$p_value
test_homogeneity_inad(y_inad_miss, blocks = blocks_inad, order = 1, test = "mean")$p_value
```

## CAT Workflow (Categorical Outcomes)

### Complete-data CAT fit and inference

```{r}
y_cat <- simulate_cat(n_subjects = 20, n_time = 4, order = 1, n_categories = 3)
fit_cat1 <- fit_cat(y_cat, order = 1)
fit_cat1$log_l

# Wald CIs (complete data)
ci_cat1 <- ci_cat(fit_cat1, parameters = "marginal")
head(ci_cat1$marginal, 3)
```

```{r}
# Order tests for CAT
run_order_tests_cat(y_cat, max_order = 2)$table
```

### Missing-data CAT fit

```{r}
y_cat_miss <- y_cat
y_cat_miss[sample(length(y_cat_miss), 12)] <- NA

fit_cat_miss <- fit_cat(y_cat_miss, order = 1, na_action = "marginalize")
fit_cat_miss$log_l
fit_cat_miss$settings$na_action_effective
```

```{r}
# EM alternative for CAT missing data (orders 0/1)
fit_cat_miss_em <- fit_cat(y_cat_miss, order = 1, na_action = "em", em_max_iter = 10)
fit_cat_miss_em$log_l
```

```{r}
# Missing-data order/homogeneity tests are supported
blocks_cat <- rep(1:2, each = nrow(y_cat_miss) / 2)
test_order_cat(y_cat_miss, order_null = 0, order_alt = 1)$p_value
test_homogeneity_cat(y_cat_miss, blocks = blocks_cat, order = 1)$p_value
```

## Additional Inference Helpers (Complete Data)

```{r eval=FALSE}
# INAD homogeneity/stationarity tools
blocks <- rep(1:2, each = nrow(y_inad) / 2)
run_homogeneity_tests_inad(y_inad, blocks = blocks, order = 1)
run_stationarity_tests_inad(y_inad, order = 1, blocks = blocks)

# CAT stationarity/time-invariance tools
test_timeinvariance_cat(y_cat, order = 1)
test_stationarity_cat(y_cat, order = 1)
```

## Known Limitations

- EM entry points: `em_gau` (Gaussian), `em_inad` (INAD), and `em_cat` (CAT, orders 0/1) are available; for CAT order 2 with missing data, use `fit_cat(na_action = "marginalize")`.
- For CAT missingness, `fit_cat(na_action = "marginalize")` supports orders 0/1/2, while `fit_cat(na_action = "em")` (or `em_cat`) supports orders 0/1 with optional safeguard step-halving.
- Missing-data confidence intervals are not implemented yet (`ci_gau`, `ci_inad`, `ci_cat` require complete-data fits).
- AD missing-data LRT/mean/covariance tests are not implemented yet.
- CAT missing-data time-invariance/stationarity tests (`test_timeinvariance_cat`, `test_stationarity_cat`, `run_stationarity_tests_cat`) are not implemented yet.
- Current order support is up to order 2 for AD, INAD, and CAT.
- CAT missing-data marginalization currently supports orders 0, 1, and 2.

## Recommended Check Command

Use the local one-command workflow before pushing:

```{r eval=FALSE}
Rscript scripts/check-package.R --as-cran --no-manual
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ci_inad.R
\name{.poster_general}
\alias{.poster_general}
\title{Posterior computations for all thinning-innovation combinations}
\usage{
.poster_general(m, n, alpha, lambda, thinning, innovation, nb_inno_size = NULL)
}
\description{
Computes conditional expectations and variances needed for Louis' identity.
See Appendix C of the paper for the derivation.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_mean_gau.R
\name{test_two_sample_gau}
\alias{test_two_sample_gau}
\title{Two-sample test for equality of mean profiles under antedependence}
\usage{
test_two_sample_gau(y, blocks, p = 1L, order = NULL, use_modified = TRUE)
}
\arguments{
\item{y}{Numeric matrix with n_subjects rows and n_time columns.}

\item{blocks}{Integer vector of length n_subjects indicating group membership
(must contain exactly two unique values, typically 1 and 2).}

\item{p}{Antedependence order of the common covariance structure. This is the
same order parameter named \code{order} in \code{\link{fit_gau}}.}

\item{order}{Optional alias for \code{p}. Supply only one of \code{p} or
\code{order}.}

\item{use_modified}{Logical. If TRUE (default), use modified test statistic.}
}
\value{
A list with class \code{gau_mean_test} containing:
\describe{
\item{method}{Inference method used (\code{"lrt"}).}
\item{test_type}{"two-sample"}
\item{mu1_hat}{Estimated mean for group 1}
\item{mu2_hat}{Estimated mean for group 2}
\item{mu_pooled}{Pooled mean estimate under H0}
\item{statistic}{Test statistic value}
\item{statistic_modified}{Modified test statistic}
\item{df}{Degrees of freedom (n_time)}
\item{p_value}{P-value from chi-square distribution}
\item{p_value_modified}{P-value from modified test}
\item{order}{Antedependence order used}
}
}
\description{
Tests the null hypothesis that two groups have equal mean profiles
mu_1 = mu_2 against the alternative mu_1 != mu_2, assuming a common
AD(p) covariance structure. This implements Theorem 7.3 of
Zimmerman & Núñez-Antón (2009).
}
\details{
This test is also known as a "profile comparison" test. The likelihood
ratio test statistic (Theorem 7.3) compares the pooled RSS (under H0:
common mean) to the sum of within-group RSS (under H1: separate means):

\deqn{N \sum_{i=1}^{n} [\log RSS_i(\mu) - \log RSS_i(\mu_1, \mu_2)]}

where RSS_i(mu) uses a common mean and RSS_i(mu_1, mu_2) uses group-specific
means.
}
\examples{
\donttest{
# Simulate data from two groups with different means
n1 <- 30
n2 <- 35
y1 <- simulate_gau(n1, n_time = 6, order = 1, mu = rep(10, 6))
y2 <- simulate_gau(n2, n_time = 6, order = 1, mu = rep(12, 6))
y <- rbind(y1, y2)
blocks <- c(rep(1, n1), rep(2, n2))

# Test equality of profiles
test <- test_two_sample_gau(y, blocks, p = 1)
print(test)
}

}
\references{
Zimmerman, D.L. and Núñez-Antón, V. (2009). Antedependence Models for
Longitudinal Data. Chapman & Hall/CRC. Chapter 7.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_gau_em.R
\name{.initialize_gau_em}
\alias{.initialize_gau_em}
\title{Initialize parameters for AD EM algorithm}
\usage{
.initialize_gau_em(y, order, blocks, estimate_mu)
}
\description{
Initialize parameters for AD EM algorithm
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{deviance.antedep}
\alias{deviance.antedep}
\alias{deviance.gau_fit}
\alias{deviance.cat_fit}
\alias{deviance.inad_fit}
\title{Deviance for Antedependence Model Fits}
\usage{
\method{deviance}{gau_fit}(object, ...)

\method{deviance}{cat_fit}(object, ...)

\method{deviance}{inad_fit}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{gau_fit},
\code{cat_fit}, or \code{inad_fit}.}

\item{...}{Unused.}
}
\value{
A scalar deviance value.
}
\description{
Returns the deviance \eqn{-2 \ell} for a fitted antedependence model.
}
\examples{
\donttest{
set.seed(1)
y <- simulate_gau(n_subjects = 40, n_time = 5, order = 1, phi = 0.4)
fit <- fit_gau(y, order = 1)
deviance(fit)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cat_utils.R
\name{.loglik_contribution}
\alias{.loglik_contribution}
\title{Compute log-likelihood contribution from counts and probabilities}
\usage{
.loglik_contribution(counts, probs)
}
\arguments{
\item{counts}{Array of counts}

\item{probs}{Array of probabilities (same dimensions as counts)}
}
\value{
Scalar log-likelihood contribution
}
\description{
Computes sum of N * log(pi) over all cells.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{plot_profile}
\alias{plot_profile}
\title{Profile plot (spaghetti plot) for longitudinal data}
\usage{
plot_profile(
  y,
  time_points = NULL,
  blocks = NULL,
  block_labels = NULL,
  title = "Profile Plot",
  xlab = "Time",
  ylab = "Measurement",
  ylim = NULL,
  show_sd = TRUE,
  individual_alpha = 0.3,
  individual_color = "grey50",
  mean_color = "blue",
  sd_color = "red",
  mean_lwd = 2
)
}
\arguments{
\item{y}{Numeric matrix with n_subjects rows and n_time columns,
or a data frame with measurements.}

\item{time_points}{Optional numeric vector of time points for x-axis.
Default uses 1:n_time or attempts to extract from column names.}

\item{blocks}{Optional integer vector of block memberships for stratified plotting.
If provided, creates separate panels for each block.}

\item{block_labels}{Optional character vector of labels for blocks.}

\item{title}{Plot title. Default "Profile Plot".}

\item{xlab}{X-axis label. Default "Time".}

\item{ylab}{Y-axis label. Default "Measurement".}

\item{ylim}{Optional y-axis limits as c(min, max).}

\item{show_sd}{Logical; if TRUE (default), show +/- 1 SD error bars.}

\item{individual_alpha}{Alpha (transparency) for individual trajectories. Default 0.3.}

\item{individual_color}{Color for individual trajectories. Default "grey50".}

\item{mean_color}{Color for mean trajectory. Default "blue".}

\item{sd_color}{Color for SD error bars. Default "red".}

\item{mean_lwd}{Line width for mean trajectory. Default 2.}
}
\value{
A ggplot2 object (invisibly). Called primarily for side effect (plotting).
}
\description{
Creates a profile plot showing individual subject trajectories with
overlaid mean trajectory and standard deviation bands.
}
\details{
This function provides a quick visual summary of longitudinal data showing:
\itemize{
\item Individual subject trajectories (light grey lines)
\item Mean trajectory across subjects (bold colored line)
\item +/- 1 standard deviation bands (error bars)
}

When \code{blocks} is provided, the plot is faceted by block membership,
allowing comparison of trajectories across treatment groups or other strata.
}
\examples{
\donttest{
data("bolus_inad")

# Basic profile plot
plot_profile(bolus_inad$y)

# With block stratification
plot_profile(bolus_inad$y, blocks = bolus_inad$blocks,
             block_labels = c("2mg", "1mg"))

# Customized
plot_profile(bolus_inad$y,
             time_points = seq(0, 44, by = 4),
             title = "Bolus Counts Over Time",
             xlab = "Hours", ylab = "Count")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logL_gau_missing.R
\name{.logL_gau_missing}
\alias{.logL_gau_missing}
\title{Compute observed-data log-likelihood for AD with missing values}
\usage{
.logL_gau_missing(y, order, mu, phi, sigma, blocks = NULL, tau = 0)
}
\arguments{
\item{y}{Data matrix (n_subjects x n_time), may contain NA}

\item{order}{Antedependence order (0, 1, or 2)}

\item{mu}{Mean vector (length n_time)}

\item{phi}{Dependence parameter(s)}

\item{sigma}{Innovation standard deviations (length n_time)}

\item{blocks}{Block membership vector (optional)}

\item{tau}{Block effects, first element constrained to zero}
}
\value{
Observed-data log-likelihood (scalar)
}
\description{
Uses multivariate normal marginalization to compute the likelihood
of observed values, marginalizing over missing values.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inad_utils.R
\name{.bell_mean_from_theta}
\alias{.bell_mean_from_theta}
\title{Convert Bell parameter to Bell mean}
\usage{
.bell_mean_from_theta(theta)
}
\description{
Convert Bell parameter to Bell mean
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_stationarity_inad.R
\name{test_stationarity_inad}
\alias{test_stationarity_inad}
\title{Likelihood ratio test for stationarity (INAD data)}
\usage{
test_stationarity_inad(
  y,
  order = 1,
  thinning = "binom",
  innovation = "pois",
  blocks = NULL,
  constrain = "both",
  fit_unconstrained = NULL,
  verbose = FALSE,
  ...
)
}
\arguments{
\item{y}{Integer matrix with n_subjects rows and n_time columns.}

\item{order}{Model order (1 or 2).}

\item{thinning}{Thinning operator: "binom", "pois", or "nbinom".}

\item{innovation}{Innovation distribution: "pois", "bell", or "nbinom".}

\item{blocks}{Optional integer vector for block effects.}

\item{constrain}{Which parameters to constrain: "alpha", "theta", "both" for order 1;
"alpha1", "alpha2", "alpha", "theta", "all" for order 2.}

\item{fit_unconstrained}{Optional pre-computed unconstrained fit.}

\item{verbose}{Logical; if TRUE, print progress.}

\item{...}{Additional arguments.}
}
\value{
A list with class \code{"test_stationarity_inad"} containing:
\describe{
\item{method}{Inference method used (\code{"lrt"}).}
\item{fit_unconstrained}{Unconstrained INAD fit}
\item{fit_constrained}{Constrained INAD fit}
\item{constraint}{Human-readable null constraint description}
\item{statistic}{Test statistic value}
\item{lrt_stat}{Likelihood ratio test statistic}
\item{df}{Degrees of freedom}
\item{p_value}{Chi-square p-value}
\item{bic_unconstrained}{BIC of unconstrained model}
\item{bic_constrained}{BIC of constrained model}
\item{bic_selected}{Model selected by BIC}
\item{table}{Two-row model summary table}
}
}
\description{
Tests whether time-varying INAD parameters can be constrained to be constant
over time.
}
\details{
For order 1, the test can constrain \code{alpha}, \code{theta}, or both.
For order 2, it can constrain \code{alpha1}, \code{alpha2},
\code{alpha} (both), \code{theta}, or all supported time-varying
parameters.

Degrees of freedom are computed from the number of equality constraints
imposed under the null model relative to the unconstrained model.

Missing-data inputs are supported through the same \code{na_action} options
available in \code{\link{fit_inad}}. If \code{y} has missing values and
\code{na_action} is not supplied via \code{...}, this function defaults to
\code{na_action = "marginalize"}.
}
\examples{
set.seed(1)
y <- simulate_inad(
  n_subjects = 30,
  n_time = 5,
  order = 1,
  thinning = "binom",
  innovation = "pois",
  alpha = 0.3,
  theta = 2
)
out <- test_stationarity_inad(
  y,
  order = 1,
  thinning = "binom",
  innovation = "pois",
  constrain = "both",
  max_iter = 20
)
out$p_value
}
\references{
Li, C. and Zimmerman, D.L. (2026). Integer-valued antedependence models for
longitudinal count data. \emph{Biostatistics}.
}
\seealso{
\code{\link{run_stationarity_tests_inad}},
\code{\link{test_order_inad}}, \code{\link{fit_inad}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loglik_inad.R
\name{.logL_inad_missing}
\alias{.logL_inad_missing}
\title{Observed-data INAD likelihood with missing values under MAR}
\usage{
.logL_inad_missing(
  y,
  order,
  thinning,
  innovation,
  alpha,
  theta,
  nb_inno_size = NULL,
  blocks = NULL,
  tau = 0
)
}
\description{
Uses forward recursion over a truncated count state space.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_stationarity_gau.R
\name{run_stationarity_tests_gau}
\alias{run_stationarity_tests_gau}
\title{Run all stationarity-related tests for Gaussian AD}
\usage{
run_stationarity_tests_gau(
  y,
  order = 1L,
  blocks = NULL,
  verbose = FALSE,
  max_iter = 2000L,
  rel_tol = 1e-08,
  ...
)
}
\arguments{
\item{y}{Numeric matrix with n_subjects rows and n_time columns.}

\item{order}{Antedependence order (0, 1, or 2).}

\item{blocks}{Optional vector of block memberships (length n_subjects).}

\item{verbose}{Logical; if TRUE, prints progress.}

\item{max_iter}{Maximum number of optimization iterations for constrained fits.}

\item{rel_tol}{Relative tolerance for constrained optimization.}

\item{...}{Additional arguments passed to \code{\link{fit_gau}} for the
unconstrained fit.}
}
\value{
A list with class \code{"stationarity_tests_gau"} containing:
\describe{
\item{fit_unconstrained}{Unconstrained Gaussian AD fit}
\item{tests}{Named list of \code{\link{test_stationarity_gau}} results}
\item{summary}{Summary table of all constraints}
}
}
\description{
Runs a standard set of stationarity constraints for Gaussian AD models.
}
\examples{
set.seed(1)
y <- simulate_gau(n_subjects = 80, n_time = 6, order = 1, phi = 0.4, sigma = 1)
out <- run_stationarity_tests_gau(y, order = 1, verbose = FALSE)
out$summary

}
\seealso{
\code{\link{test_stationarity_gau}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cat_utils.R
\name{summary.cat_fit}
\alias{summary.cat_fit}
\title{Summary method for cat_fit objects}
\usage{
\method{summary}{cat_fit}(object, ...)
}
\arguments{
\item{object}{A \code{cat_fit} object.}

\item{...}{Unused.}
}
\value{
\code{object}, invisibly.
}
\description{
Summary method for cat_fit objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ci_cat.R
\name{ci_cat}
\alias{ci_cat}
\title{Confidence intervals for fitted categorical AD models}
\usage{
ci_cat(fit, y = NULL, level = 0.95, parameters = "all")
}
\arguments{
\item{fit}{A fitted model object of class \code{"cat_fit"} from \code{fit_cat()}.}

\item{y}{Optional data matrix. If NULL, \code{fit$cell_counts} is used
(observed counts for closed-form fits; expected counts for EM fits).}

\item{level}{Confidence level (default 0.95).}

\item{parameters}{Which parameters to compute CIs for: "all" (default),
"marginal", or "transition".}
}
\value{
A list of class \code{"cat_ci"} containing:
\item{marginal}{Data frame of CIs for marginal parameters (if requested)}
\item{transition}{List of data frames of CIs for transition parameters (if requested)}
\item{level}{Confidence level used}
\item{settings}{Model settings from fit}
}
\description{
Computes Wald-based confidence intervals for the transition probability
parameters of a fitted categorical antedependence model.
}
\details{
Confidence intervals are computed using the Wald method based on the
asymptotic normality of maximum likelihood estimators.

For a probability estimate \eqn{\hat{\pi}} based on count N, the standard error is:
\deqn{SE(\hat{\pi}) = \sqrt{\frac{\hat{\pi}(1-\hat{\pi})}{N}}}

For conditional probabilities \eqn{\hat{\pi}_{j|i}} based on conditioning count
\eqn{N_i}, the standard error is:
\deqn{SE(\hat{\pi}_{j|i}) = \sqrt{\frac{\hat{\pi}_{j|i}(1-\hat{\pi}_{j|i})}{N_i}}}

The confidence interval is then:
\deqn{\hat{\pi} \pm z_{\alpha/2} \times SE(\hat{\pi})}

Note: CIs are truncated to the interval from 0 to 1 when they exceed these bounds.

Missing-data fits with \code{na_action = "marginalize"} are not currently
supported because observed cell counts are not stored for that path.
}
\examples{
\donttest{
# Fit a model
set.seed(123)
y <- simulate_cat(200, 5, order = 1, n_categories = 2)
fit <- fit_cat(y, order = 1)

# Compute confidence intervals
ci <- ci_cat(fit)
print(ci)

# Just marginal CIs
ci_marg <- ci_cat(fit, parameters = "marginal")
}

}
\references{
Agresti, A. (2013). Categorical Data Analysis (3rd ed.). Wiley.
}
\seealso{
\code{\link{fit_cat}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_methods.R
\name{plot.inad_fit}
\alias{plot.inad_fit}
\title{Plot estimated parameters of an INAD fit}
\usage{
\method{plot}{inad_fit}(x, ...)
}
\arguments{
\item{x}{An \code{inad_fit} object from \code{\link{fit_inad}}.}

\item{...}{Unused.}
}
\value{
\code{x}, invisibly.
}
\description{
Produces a two-panel base-graphics plot of the estimated thinning
parameters (\eqn{\alpha}) and innovation means (\eqn{\theta}) over time.
}
\examples{
\donttest{
set.seed(1)
y <- simulate_inad(n_subjects = 60, n_time = 5, order = 1,
                   thinning = "binom", innovation = "pois",
                   alpha = 0.3, theta = 2)
fit <- fit_inad(y, order = 1, thinning = "binom",
                innovation = "pois", max_iter = 20)
plot(fit)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gau_utils.R
\name{.count_params_gau}
\alias{.count_params_gau}
\title{Count parameters for AD model}
\usage{
.count_params_gau(order, n_time, n_blocks = 1L, estimate_mu = FALSE)
}
\arguments{
\item{order}{Model order.}

\item{n_time}{Number of time points.}

\item{n_blocks}{Number of blocks (1 if no blocks).}

\item{estimate_mu}{Whether mu is estimated.}
}
\value{
Number of free parameters.
}
\description{
Count parameters for AD model
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_cat.R
\name{.unpack_cat_params}
\alias{.unpack_cat_params}
\title{Unpack unconstrained CAT parameter vector}
\usage{
.unpack_cat_params(theta, p, c, n_time)
}
\arguments{
\item{theta}{Numeric parameter vector}

\item{p}{Order}

\item{c}{Number of categories}

\item{n_time}{Number of time points}
}
\value{
List with marginal and transition parameter lists
}
\description{
Unpack unconstrained CAT parameter vector
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{coef.antedep}
\alias{coef.antedep}
\alias{coef.gau_fit}
\alias{coef.cat_fit}
\alias{coef.inad_fit}
\title{Extract model coefficients from antedependence fits}
\usage{
\method{coef}{gau_fit}(object, ...)

\method{coef}{cat_fit}(object, ...)

\method{coef}{inad_fit}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{gau_fit},
\code{cat_fit}, or \code{inad_fit}.}

\item{...}{Unused.}
}
\value{
For \code{gau_fit}: a named list with elements \code{mu},
\code{phi}, \code{sigma}, and (if a block effect was estimated)
\code{tau}.
For \code{cat_fit}: a list with \code{marginal} and \code{transition}
probability arrays.
For \code{inad_fit}: a named list with elements \code{alpha},
\code{theta}, and (if applicable) \code{tau} and \code{nb_inno_size}.
}
\description{
Returns the estimated parameters of a fitted antedependence model as a
named list.
}
\examples{
\donttest{
set.seed(1)
y <- simulate_gau(n_subjects = 40, n_time = 5, order = 1, phi = 0.4)
fit <- fit_gau(y, order = 1)
coef(fit)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_stationarity_gau.R
\name{test_stationarity_gau}
\alias{test_stationarity_gau}
\title{Likelihood ratio test for stationarity (Gaussian AD data)}
\usage{
test_stationarity_gau(
  y,
  order = 1L,
  blocks = NULL,
  constrain = "both",
  fit_unconstrained = NULL,
  verbose = FALSE,
  max_iter = 2000L,
  rel_tol = 1e-08,
  ...
)
}
\arguments{
\item{y}{Numeric matrix with n_subjects rows and n_time columns.}

\item{order}{Antedependence order (0, 1, or 2).}

\item{blocks}{Optional vector of block memberships (length n_subjects).}

\item{constrain}{Constraint to test:
for order 0: "sigma" (or "all");
for order 1: "phi", "sigma", or "both"/"all";
for order 2: "phi1", "phi2", "phi", "sigma", or "all"/"both".}

\item{fit_unconstrained}{Optional pre-computed unconstrained fit from
\code{\link{fit_gau}}.}

\item{verbose}{Logical; if TRUE, prints fitting progress.}

\item{max_iter}{Maximum number of optimization iterations for constrained fit.}

\item{rel_tol}{Relative tolerance for constrained optimization.}

\item{...}{Additional arguments passed to \code{\link{fit_gau}} when
\code{fit_unconstrained} is not provided.}
}
\value{
A list with class \code{"test_stationarity_gau"} containing:
\describe{
\item{method}{Inference method used (\code{"lrt"}).}
\item{fit_unconstrained}{Unconstrained Gaussian AD fit}
\item{fit_constrained}{Constrained Gaussian AD fit}
\item{constraint}{Human-readable null constraint description}
\item{lrt_stat}{Likelihood-ratio statistic}
\item{df}{Degrees of freedom}
\item{p_value}{Chi-square p-value}
\item{bic_unconstrained}{BIC of unconstrained model}
\item{bic_constrained}{BIC of constrained model}
\item{bic_selected}{Model selected by BIC}
\item{table}{Two-row model summary table}
}
}
\description{
Tests whether time-varying Gaussian AD covariance parameters can be
constrained to be constant over time.
}
\details{
The mean structure is kept unrestricted in both models (time-specific means
plus optional block shifts), and the test constrains covariance parameters:
innovation standard deviations and/or antedependence coefficients.

The likelihood-ratio statistic is:
\deqn{\lambda = 2(\ell_{alt} - \ell_{null})}
where \eqn{\ell_{null}} and \eqn{\ell_{alt}} are maximized log-likelihoods
under the constrained and unconstrained models, respectively.

Degrees of freedom are computed from the number of constraints implied by
\code{constrain}.
}
\examples{
set.seed(1)
y <- simulate_gau(n_subjects = 80, n_time = 6, order = 1, phi = 0.4, sigma = 1)

# Test jointly constant phi and sigma (order 1)
out <- test_stationarity_gau(y, order = 1, constrain = "both")
out$p_value

}
\references{
Zimmerman, D.L. and Nunez-Anton, V. (2009). Antedependence Models for
Longitudinal Data. Chapman & Hall/CRC. Chapter 6.
}
\seealso{
\code{\link{run_stationarity_tests_gau}}, \code{\link{test_order_gau}},
\code{\link{fit_gau}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_homogeneity_cat.R
\name{test_homogeneity_cat}
\alias{test_homogeneity_cat}
\title{Likelihood ratio test for homogeneity across groups (categorical AD data)}
\usage{
test_homogeneity_cat(
  y = NULL,
  blocks = NULL,
  order = 1,
  n_categories = NULL,
  fit_null = NULL,
  fit_alt = NULL,
  test = c("lrt", "score", "mlrt")
)
}
\arguments{
\item{y}{Integer matrix with n_subjects rows and n_time columns. Each entry
should be a category code from 1 to c. Can be NULL if both fit_null and
fit_alt are provided.}

\item{blocks}{Integer vector of length n_subjects specifying group membership.
Required unless pre-fitted models are provided.}

\item{order}{Antedependence order p. Default is 1.}

\item{n_categories}{Number of categories. If NULL, inferred from data.}

\item{fit_null}{Optional pre-fitted homogeneous model (class "cat_fit" with
homogeneous = TRUE). If provided, y is not required for fitting under H0.}

\item{fit_alt}{Optional pre-fitted heterogeneous model (class "cat_fit" with
homogeneous = FALSE). If provided, y is not required for fitting under H1.}

\item{test}{Type of test statistic. One of \code{"lrt"} (default),
\code{"score"}, or \code{"mlrt"}.}
}
\value{
A list of class \code{"cat_lrt"} containing:
\describe{
\item{method}{Inference method used: one of \code{"lrt"}, \code{"score"},
\code{"mlrt"}, or \code{"wald"}.}
\item{lrt_stat}{Likelihood ratio test statistic}
\item{df}{Degrees of freedom}
\item{p_value}{P-value from chi-square distribution}
\item{fit_null}{Fitted homogeneous model (H0)}
\item{fit_alt}{Fitted heterogeneous model (H1)}
\item{n_groups}{Number of groups}
\item{table}{Summary data frame}
}
}
\description{
Tests whether multiple groups share the same transition probability parameters
in a categorical antedependence model.
}
\details{
The null hypothesis is that all G groups share the same transition probability
parameters:
\deqn{H_0: \pi^{(1)} = \pi^{(2)} = \ldots = \pi^{(G)}}

The alternative hypothesis allows each group to have its own parameters.

The degrees of freedom are:
\deqn{df = (G-1) \times k}
where G is the number of groups and k is the number of free parameters per
population.
}
\examples{
\donttest{
# Simulate data with different transition probabilities for two groups
set.seed(123)
marg1 <- list(t1 = c(0.7, 0.3))
marg2 <- list(t1 = c(0.4, 0.6))
trans1 <- list(t2 = matrix(c(0.9, 0.1, 0.2, 0.8), 2, byrow = TRUE),
               t3 = matrix(c(0.9, 0.1, 0.2, 0.8), 2, byrow = TRUE))
trans2 <- list(t2 = matrix(c(0.5, 0.5, 0.5, 0.5), 2, byrow = TRUE),
               t3 = matrix(c(0.5, 0.5, 0.5, 0.5), 2, byrow = TRUE))

y1 <- simulate_cat(100, 3, order = 1, n_categories = 2,
                   marginal = marg1, transition = trans1)
y2 <- simulate_cat(100, 3, order = 1, n_categories = 2,
                   marginal = marg2, transition = trans2)
y <- rbind(y1, y2)
blocks <- c(rep(1, 100), rep(2, 100))

# Test homogeneity
test <- test_homogeneity_cat(y, blocks, order = 1)
print(test)
}

}
\references{
Xie, Y. and Zimmerman, D. L. (2013). Antedependence models for nonstationary
categorical longitudinal data with ignorable missingness: likelihood-based
inference. \emph{Statistics in Medicine}, 32, 3274-3289.
}
\seealso{
\code{\link{fit_cat}}, \code{\link{test_order_cat}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inad_utils.R
\name{.log_sum_exp}
\alias{.log_sum_exp}
\title{Log-sum-exp for numerical stability}
\usage{
.log_sum_exp(x)
}
\description{
Log-sum-exp for numerical stability
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_inad.R
\name{simulate_inad}
\alias{simulate_inad}
\title{Simulate INAD antedependence series}
\usage{
simulate_inad(
  n_subjects,
  n_time,
  order = 1L,
  thinning = c("binom", "pois", "nbinom"),
  innovation = c("pois", "bell", "nbinom"),
  alpha = NULL,
  theta = NULL,
  nb_inno_size = NULL,
  blocks = NULL,
  tau = 0,
  seed = NULL
)
}
\arguments{
\item{n_subjects}{number of subjects}

\item{n_time}{number of time points}

\item{order}{antedependence order, 0, 1 or 2}

\item{thinning}{thinning operator, one of \code{"binom"}, \code{"pois"}, \code{"nbinom"}}

\item{innovation}{innovation distribution, one of \code{"pois"}, \code{"bell"},
\code{"nbinom"}}

\item{alpha}{thinning parameter or vector or matrix; if \code{NULL}, defaults
are used depending on the order}

\item{theta}{innovation parameter or vector; if \code{NULL}, defaults are used
depending on the innovation type. For Poisson and negative binomial
innovations, \code{theta} is the innovation mean parameter. For Bell
innovations, \code{theta} is the Bell rate parameter, with innovation mean
\code{theta * exp(theta)}.}

\item{nb_inno_size}{size (dispersion) parameter for negative binomial innovations
when \code{innovation = "nbinom"}; must be positive. If \code{NULL}, defaults to 1.
Larger values correspond to less overdispersion (approaching Poisson as
size -> Inf).}

\item{blocks}{integer vector of length \code{n_subjects} indicating block
membership for each subject; if \code{NULL}, no block effect is applied}

\item{tau}{group effect vector indexed by block; \code{tau[1]} is forced to 0.
If scalar x, it is expanded to c(0, x, ..., x) with length equal to the
number of blocks}

\item{seed}{optional random seed for reproducibility}
}
\value{
integer matrix of counts with dimension \code{n_subjects} by \code{n_time}
}
\description{
Generate longitudinal count data from an INAD model using a thinning
operator and an innovation distribution.
}
\details{
Time 1 observations are generated from the innovation distribution alone.
For times 2 to \code{n_time}, counts are generated as thinning of previous
counts plus independent innovations. When \code{order = 0}, all time points
are generated from the innovation distribution and the thinning operator
and \code{alpha} are ignored.

If \code{blocks} is provided, innovations include a block effect. For Poisson and
negative binomial innovations, the innovation mean is \code{theta[t] + tau[blocks[i]]}.
For Bell innovations, the innovation mean is \code{theta[t] * exp(theta[t]) + tau[blocks[i]]}.
}
\examples{
y <- simulate_inad(
  n_subjects = 20,
  n_time = 6,
  order = 1,
  thinning = "binom",
  innovation = "pois",
  alpha = 0.3,
  theta = 2,
  seed = 42
)
dim(y)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logL_gau_missing.R
\name{.build_gau_covariance}
\alias{.build_gau_covariance}
\title{Build AD covariance matrix from parameters}
\usage{
.build_gau_covariance(order, phi, sigma, n_time)
}
\arguments{
\item{order}{Antedependence order}

\item{phi}{Dependence parameters}

\item{sigma}{Innovation standard deviations}

\item{n_time}{Number of time points}
}
\value{
Covariance matrix (n_time x n_time)
}
\description{
Build AD covariance matrix from parameters
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{as.ts.antedep_sim}
\alias{as.ts.antedep_sim}
\alias{as.ts.gau_sim}
\alias{as.ts.cat_sim}
\alias{as.ts.inad_sim}
\title{Convert antedependence simulation output to a time-series object}
\usage{
\method{as.ts}{gau_sim}(x, ...)

\method{as.ts}{cat_sim}(x, ...)

\method{as.ts}{inad_sim}(x, ...)
}
\arguments{
\item{x}{A matrix of class \code{gau_sim}, \code{cat_sim}, or
\code{inad_sim} as returned by the corresponding \code{simulate_*}
function.}

\item{...}{Additional arguments passed to \code{\link[stats]{ts}}
(e.g., \code{start}, \code{frequency}).}
}
\value{
A \code{ts} (\code{mts}) object with one column per subject.
}
\description{
Converts the panel matrix returned by \code{\link{simulate_gau}},
\code{\link{simulate_cat}}, or \code{\link{simulate_inad}} into an
\code{mts} / \code{\link[stats]{ts}} object suitable for time-series
functions.  Rows of the original matrix (subjects) become the columns of
the returned object; columns (time points) become the rows.
}
\examples{
\donttest{
set.seed(1)
y <- simulate_gau(n_subjects = 5, n_time = 10, order = 1, phi = 0.4)
y_ts <- as.ts(y)
class(y_ts)    # "mts" "ts" "matrix" "array"
dim(y_ts)      # 10 x 5
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_order_inad.R
\name{bic_order_inad}
\alias{bic_order_inad}
\title{BIC-based order selection for INAD models}
\usage{
bic_order_inad(
  y,
  max_order = 2,
  thinning = "binom",
  innovation = "pois",
  blocks = NULL,
  ...
)
}
\arguments{
\item{y}{Integer matrix.}

\item{max_order}{Maximum order (1 or 2).}

\item{thinning}{Thinning operator.}

\item{innovation}{Innovation distribution.}

\item{blocks}{Optional block assignments.}

\item{...}{Additional arguments.}
}
\value{
A list with class \code{"bic_order_inad"} containing:
\describe{
\item{fits}{List of fitted INAD models by candidate order}
\item{bic}{Named numeric vector of BIC values by order}
\item{best_order}{Order with minimum BIC}
\item{table}{Data frame with order, logLik, n_params, and BIC}
\item{settings}{Input and derived settings used for selection}
}
}
\description{
Fits INAD models across candidate orders and reports BIC-based selection.
}
\examples{
\donttest{
set.seed(1)
y <- simulate_inad(
  n_subjects = 30,
  n_time = 5,
  order = 1,
  thinning = "binom",
  innovation = "pois",
  alpha = 0.3,
  theta = 2
)
ord <- bic_order_inad(y, max_order = 2, thinning = "binom", innovation = "pois", max_iter = 10)
ord$best_order
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_cat.R
\name{fit_cat}
\alias{fit_cat}
\title{Fit categorical antedependence model by maximum likelihood}
\usage{
fit_cat(
  y,
  order = 1,
  blocks = NULL,
  homogeneous = TRUE,
  n_categories = NULL,
  na_action = c("fail", "complete", "marginalize", "em"),
  em_max_iter = 100,
  em_tol = 1e-06,
  em_epsilon = 1e-08,
  em_safeguard = TRUE,
  em_verbose = FALSE
)
}
\arguments{
\item{y}{Integer matrix with n_subjects rows and n_time columns. Each entry
should be a category code from 1 to c, where c is the number of categories.}

\item{order}{Antedependence order p. Must be 0, 1, or 2. Default is 1.}

\item{blocks}{Optional integer vector of length n_subjects specifying group
membership. If NULL, all subjects are treated as one group.}

\item{homogeneous}{Logical. If TRUE (default), parameters are shared across
all groups (blocks are ignored for estimation). If FALSE, separate
transition probabilities are estimated for each group.}

\item{n_categories}{Number of categories. If NULL (default), inferred from
the maximum value in y.}

\item{na_action}{Handling of missing values in \code{y}. One of
\code{"fail"} (default, error if any missing), \code{"complete"}
(drop subjects with any missing values), or \code{"marginalize"}
(maximize observed-data likelihood by integrating over missing outcomes),
or \code{"em"} (use \code{\link{em_cat}} for orders 0 and 1).}

\item{em_max_iter}{Maximum EM iterations used when \code{na_action = "em"}.}

\item{em_tol}{EM convergence tolerance used when \code{na_action = "em"}.}

\item{em_epsilon}{Numerical smoothing constant used when \code{na_action = "em"}.}

\item{em_safeguard}{Logical; if \code{TRUE}, use step-halving safeguard in
\code{\link{em_cat}} when \code{na_action = "em"}.}

\item{em_verbose}{Logical; print EM progress when \code{na_action = "em"}.}
}
\value{
A list of class \code{"cat_fit"} containing:
\item{marginal}{List of marginal/joint probabilities for initial time points}
\item{transition}{List of transition probability arrays for k = p+1 to n}
\item{log_l}{Log-likelihood at MLE}
\item{aic}{Akaike Information Criterion}
\item{bic}{Bayesian Information Criterion}
\item{n_params}{Number of free parameters}
\item{cell_counts}{List of cell counts: observed counts for closed-form fits
(\code{na_action = "fail"/"complete"}), expected counts from the final E-step
for EM fits (\code{na_action = "em"}), and \code{NULL} for
\code{na_action = "marginalize"}}
\item{convergence}{Optimizer convergence code (0 for closed-form solutions)}
\item{settings}{List of model settings}
}
\description{
Computes maximum likelihood estimates for the parameters of an AD(p) model
for categorical longitudinal data. The model is parameterized by transition
probabilities, and MLEs are obtained in closed form.
}
\details{
For AD(p), the model decomposes as:
\deqn{P(Y_1, \ldots, Y_n) = P(Y_1, \ldots, Y_p) \times \prod_{k=p+1}^{n} P(Y_k | Y_{k-p}, \ldots, Y_{k-1})}

MLEs are computed as empirical proportions:
\itemize{
\item Marginal/joint probabilities: count / N
\item Transition probabilities: conditional count / marginal count
}

Empty cells receive probability 0 (if denominator is also 0).

When \code{na_action = "em"}, \code{fit_cat()} dispatches to
\code{\link{em_cat}}. In that case, \code{em_safeguard} controls step-halving
protection against likelihood-decreasing updates, and returned
\code{log_l}/AIC/BIC/\code{cell_counts} are synchronized via a final E-step
under the returned parameters.
For \code{order = 2}, \code{na_action = "em"} is not available and errors
explicitly; use \code{na_action = "marginalize"}.
}
\examples{
\donttest{
# Simulate binary AD(1) data
set.seed(123)
y <- simulate_cat(n_subjects = 100, n_time = 5, order = 1, n_categories = 2)

# Fit model
fit <- fit_cat(y, order = 1)
print(fit)

# Compare orders
fit0 <- fit_cat(y, order = 0)
fit1 <- fit_cat(y, order = 1)
fit2 <- fit_cat(y, order = 2)
c(AIC_0 = fit0$aic, AIC_1 = fit1$aic, AIC_2 = fit2$aic)

# EM fit with missing data
y_miss <- y
y_miss[sample(length(y_miss), size = round(0.15 * length(y_miss)))] <- NA
fit_em <- fit_cat(
  y_miss,
  order = 1,
  na_action = "em",
  em_max_iter = 80,
  em_tol = 1e-6
)
fit_em$settings$n_iter
fit_em$settings$cell_counts_type
}

}
\references{
Xie, Y. and Zimmerman, D. L. (2013). Antedependence models for nonstationary
categorical longitudinal data with ignorable missingness: likelihood-based
inference. \emph{Statistics in Medicine}, 32, 3274-3289.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{vcov.gau_fit}
\alias{vcov.gau_fit}
\title{Variance-Covariance Matrix for Gaussian AD Fits}
\usage{
\method{vcov}{gau_fit}(object, ...)
}
\arguments{
\item{object}{A \code{gau_fit} object.}

\item{...}{Unused.}
}
\value{
A named square matrix of parameter variances (diagonal entries)
and covariances (off-diagonal entries, all zero in the current
implementation).
}
\description{
Returns an approximate variance-covariance matrix for the estimated
parameters of a \code{gau_fit} object.  The matrix is block-diagonal
(asymptotically exact for the Gaussian AD model given the factored
likelihood) with entries derived from the closed-form standard errors
used by \code{\link{ci_gau}}.
}
\examples{
\donttest{
set.seed(1)
y <- simulate_gau(n_subjects = 40, n_time = 5, order = 1, phi = 0.4)
fit <- fit_gau(y, order = 1)
vcov(fit)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_cat.R
\name{.simulate_subject_cat}
\alias{.simulate_subject_cat}
\title{Simulate one subject's trajectory}
\usage{
.simulate_subject_cat(n_time, p, c, marginal, transition)
}
\arguments{
\item{n_time}{Number of time points}

\item{p}{Order}

\item{c}{Number of categories}

\item{marginal}{Marginal parameters}

\item{transition}{Transition parameters}
}
\value{
Integer vector of length n_time
}
\description{
Simulate one subject's trajectory
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_homogeneity_inad.R
\name{test_homogeneity_inad}
\alias{test_homogeneity_inad}
\title{Likelihood ratio test for homogeneity across groups (INAD data)}
\usage{
test_homogeneity_inad(
  y,
  blocks,
  order = 1,
  thinning = "binom",
  innovation = "pois",
  test = c("all", "mean", "dependence"),
  fit_pooled = NULL,
  fit_inadfe = NULL,
  fit_hetero = NULL,
  ...
)
}
\arguments{
\item{y}{Integer matrix with n_subjects rows and n_time columns.}

\item{blocks}{Integer vector of length n_subjects specifying group membership.}

\item{order}{Antedependence order (0, 1, or 2).}

\item{thinning}{Thinning operator: "binom", "pois", or "nbinom".}

\item{innovation}{Innovation distribution: "pois", "bell", or "nbinom".}

\item{test}{Type of homogeneity test:
\itemize{
\item \code{"all"}: Tests M1 (pooled) vs M3 (fully heterogeneous)
\item \code{"mean"}: Tests M1 (pooled) vs M2 (shared dependence, different means)
\item \code{"dependence"}: Tests M2 (INADFE) vs M3 (fully heterogeneous)
}}

\item{fit_pooled}{Optional pre-computed pooled fit (M1).}

\item{fit_inadfe}{Optional pre-computed INADFE fit (M2).}

\item{fit_hetero}{Optional pre-computed heterogeneous fit (M3).}

\item{...}{Additional arguments passed to \code{fit_inad}.}
}
\value{
A list with class \code{"test_homogeneity_inad"} containing:
\describe{
\item{method}{Inference method used (\code{"lrt"}).}
\item{statistic}{Test statistic value}
\item{lrt_stat}{Likelihood ratio test statistic}
\item{df}{Degrees of freedom}
\item{p_value}{P-value from chi-square distribution}
\item{test}{Type of test performed}
\item{fit_null}{Fitted model under H0}
\item{fit_alt}{Fitted model under H1}
\item{bic_null}{BIC under H0}
\item{bic_alt}{BIC under H1}
\item{bic_selected}{Which model BIC prefers}
\item{table}{Summary data frame}
}
}
\description{
Tests hypotheses about parameter equality across treatment or grouping
factors in integer-valued antedependence models. Implements the homogeneity
testing framework from Section 3.7 of Li & Zimmerman (2026).
}
\details{
The function supports three nested model comparisons as described in the paper:

\strong{M1 (Pooled)}: All parameters are common across groups. This corresponds
to fitting \code{fit_inad(y, blocks = NULL)}.

\strong{M2 (INADFE)}: The thinning parameters \eqn{\alpha} are shared across
groups, but innovation means differ via block effects \eqn{\tau}. This is the
standard INADFE model fitted via \code{fit_inad(y, blocks = blocks)}.

\strong{M3 (Fully Heterogeneous)}: Both \eqn{\alpha} and \eqn{\theta}
parameters can differ across groups. This is fitted by running separate
\code{fit_inad} calls for each group.

The three test types correspond to:
\itemize{
\item \code{"all"}: H0: M1 vs H1: M3 (complete homogeneity vs complete heterogeneity)
\item \code{"mean"}: H0: M1 vs H1: M2 (test for group differences in means only)
\item \code{"dependence"}: H0: M2 vs H1: M3 (test for group differences in dependence)
}

Degrees of freedom are computed as the difference in free parameters between
the null and alternative models.
}
\examples{
\donttest{
data("bolus_inad")
y <- bolus_inad$y
blocks <- bolus_inad$blocks

# Test for any group differences (M1 vs M3)
test_all <- test_homogeneity_inad(y, blocks, order = 1,
                                  thinning = "nbinom", innovation = "bell",
                                  test = "all")
print(test_all)

# Test only for mean differences (M1 vs M2)
test_mean <- test_homogeneity_inad(y, blocks, order = 1,
                                   thinning = "nbinom", innovation = "bell",
                                   test = "mean")
print(test_mean)

# Test for dependence differences given different means (M2 vs M3)
test_dep <- test_homogeneity_inad(y, blocks, order = 1,
                                  thinning = "nbinom", innovation = "bell",
                                  test = "dependence")
print(test_dep)
}

}
\references{
Li, C. and Zimmerman, D.L. (2026). Integer-valued antedependence models for
longitudinal count data. \emph{Biostatistics}. Section 3.7.
}
\seealso{
\code{\link{fit_inad}}, \code{\link{test_order_inad}},
\code{\link{test_stationarity_inad}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/em_cat.R
\name{.safeguard_update_cat_em}
\alias{.safeguard_update_cat_em}
\title{Safeguard M-step update via step-halving}
\usage{
.safeguard_update_cat_em(
  y,
  order,
  old_marginal,
  old_transition,
  new_marginal,
  new_transition,
  blocks_id,
  homogeneous,
  c,
  logL_old,
  n_time,
  n_blocks
)
}
\arguments{
\item{y}{Data matrix.}

\item{order}{Model order.}

\item{old_marginal}{Previous marginal parameters.}

\item{old_transition}{Previous transition parameters.}

\item{new_marginal}{Proposed marginal parameters from M-step.}

\item{new_transition}{Proposed transition parameters from M-step.}

\item{blocks_id}{Normalized block ids.}

\item{homogeneous}{Whether parameters are shared across blocks.}

\item{c}{Number of categories.}

\item{logL_old}{Observed-data log-likelihood at old parameters.}

\item{n_time}{Number of time points.}

\item{n_blocks}{Number of blocks.}
}
\value{
List with safeguarded parameters, acceptance flag, and step size.
}
\description{
Safeguard M-step update via step-halving
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_order_gau.R
\name{test_order_gau}
\alias{test_order_gau}
\title{Likelihood ratio test for antedependence order (Gaussian AD data)}
\usage{
test_order_gau(
  y,
  p = 0L,
  q = 1L,
  mu = NULL,
  use_modified = TRUE,
  order_null = NULL,
  order_alt = NULL
)
}
\arguments{
\item{y}{Numeric matrix with n_subjects rows and n_time columns.}

\item{p}{Order under the null hypothesis (default 0). This is the same
antedependence order parameter named \code{order} in \code{\link{fit_gau}}.}

\item{q}{Order increment under the alternative (default 1, so alternative is AD(p+q)).}

\item{mu}{Optional mean vector. If NULL, the saturated mean (sample means) is used.}

\item{use_modified}{Logical. If TRUE (default), use the modified test statistic
(formula 6.9) which has better small-sample properties.}

\item{order_null}{Optional alias for \code{p} (null order).}

\item{order_alt}{Optional absolute order under the alternative. When supplied,
\code{q} is computed as \code{order_alt - p}.}
}
\value{
A list with class \code{gau_order_test} containing:
\describe{
\item{method}{Inference method used (\code{"lrt"}).}
\item{p}{Order under null hypothesis}
\item{q}{Order increment}
\item{statistic}{Test statistic value}
\item{statistic_modified}{Modified test statistic (if use_modified = TRUE)}
\item{df}{Degrees of freedom}
\item{p_value}{P-value from chi-square distribution}
\item{p_value_modified}{P-value from modified test (if use_modified = TRUE)}
\item{n_subjects}{Number of subjects}
\item{n_time}{Number of time points}
}
}
\description{
Tests the null hypothesis that the data follow an AD(p) model against the
alternative that they follow an AD(p+q) model, using the likelihood ratio
test described in Theorem 6.4 and 6.5 of Zimmerman & Núñez-Antón (2009).
}
\details{
The test is based on the intervenor-adjusted sample partial correlations.
Under the null hypothesis AD(p), the partial correlations r_(i,i-k|(i-k+1:i-1))
should be zero for k > p.

The likelihood ratio test statistic (Theorem 6.4) is:
\deqn{-N \sum_{j=1}^{q} \sum_{i=p+j+1}^{n} \log(1 - r^2_{i,i-p-j\cdot(i-p-j+1:i-1)})}

which is asymptotically chi-square with (2n - 2p - q - 1)(q/2) degrees of freedom.

The modified test (formula 6.9) adjusts for small-sample bias using Kenward's
(1987) correction.
}
\examples{
\donttest{
# Simulate AD(1) data
y <- simulate_gau(n_subjects = 50, n_time = 6, order = 1, phi = 0.5)

# Test AD(0) vs AD(1)
test01 <- test_order_gau(y, p = 0, q = 1)
print(test01)

# Test AD(1) vs AD(2)
test12 <- test_order_gau(y, p = 1, q = 1)
print(test12)
}

}
\references{
Zimmerman, D.L. and Núñez-Antón, V. (2009). Antedependence Models for
Longitudinal Data. Chapman & Hall/CRC. Chapter 6.

Kenward, M.G. (1987). A method for comparing profiles of repeated measurements.
Applied Statistics, 36, 296-308.
}
\seealso{
\code{\link{test_one_sample_gau}}, \code{\link{test_homogeneity_gau}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_gau_em.R
\name{.em_e_step_gau}
\alias{.em_e_step_gau}
\title{E-step: Compute expected sufficient statistics}
\usage{
.em_e_step_gau(y, order, mu, phi, sigma, blocks, tau)
}
\description{
Computes conditional expectations for missing values and second moments.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_order_gau.R
\name{bic_order_gau}
\alias{bic_order_gau}
\title{BIC-based order selection for Gaussian AD models}
\usage{
bic_order_gau(y, max_order = 2L, ...)
}
\arguments{
\item{y}{Numeric matrix with n_subjects rows and n_time columns.}

\item{max_order}{Maximum order to consider.}

\item{...}{Additional arguments passed to \code{\link{fit_gau}}.}
}
\value{
A list with class \code{gau_bic_order} containing:
\describe{
\item{fits}{List of fitted models}
\item{bic}{BIC values for each order}
\item{best_order}{Order with lowest BIC}
\item{table}{Summary table}
}
}
\description{
Fits AD models of increasing orders and selects the best by BIC.
}
\examples{
\donttest{
set.seed(1)
y <- simulate_gau(n_subjects = 80, n_time = 6, order = 1, phi = 0.4)
ord <- bic_order_gau(y, max_order = 2)
ord$best_order
ord$table
}

}
\seealso{
\code{\link{bic_order_cat}}, \code{\link{bic_order_inad}},
\code{\link{fit_gau}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ci_cat.R
\name{.compute_marginal_ci}
\alias{.compute_marginal_ci}
\title{Compute CIs for marginal parameters}
\usage{
.compute_marginal_ci(marginal, cell_counts, p, c, n_time, n_subjects, z, level)
}
\arguments{
\item{marginal}{Marginal parameter list from fit}

\item{cell_counts}{Cell counts from fit}

\item{p}{Order}

\item{c}{Number of categories}

\item{n_time}{Number of time points}

\item{n_subjects}{Number of subjects}

\item{z}{Z critical value}

\item{level}{Confidence level}
}
\value{
Data frame with CIs
}
\description{
Compute CIs for marginal parameters
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_stationarity_cat.R
\name{test_stationarity_cat}
\alias{test_stationarity_cat}
\title{Likelihood ratio test for stationarity (categorical AD data)}
\usage{
test_stationarity_cat(
  y,
  order = 1,
  blocks = NULL,
  homogeneous = TRUE,
  n_categories = NULL,
  test = c("lrt", "score", "mlrt")
)
}
\arguments{
\item{y}{Integer matrix with n_subjects rows and n_time columns. Each entry
should be a category code from 1 to c.}

\item{order}{Antedependence order p. Default is 1.}

\item{blocks}{Optional integer vector of length n_subjects specifying group
membership.}

\item{homogeneous}{Logical. If TRUE (default), parameters are shared across
all groups.}

\item{n_categories}{Number of categories. If NULL, inferred from data.}

\item{test}{Type of test statistic. One of \code{"lrt"} (default),
\code{"score"}, or \code{"mlrt"}.}
}
\value{
A list of class \code{"cat_lrt"} containing:
\describe{
\item{method}{Inference method used: one of \code{"lrt"}, \code{"score"},
or \code{"mlrt"}.}
\item{lrt_stat}{Likelihood ratio test statistic}
\item{df}{Degrees of freedom}
\item{p_value}{P-value from chi-square distribution}
\item{fit_null}{Fitted stationary model (H0)}
\item{fit_alt}{Fitted non-stationary model (H1)}
\item{table}{Summary data frame}
}
}
\description{
Tests whether a categorical antedependence process satisfies stationarity
constraints in the AD parameterization.
}
\details{
The tested constraints are:
\enumerate{
\item The marginal distribution P(Yk) is constant for all k
\item The transition probabilities P(Yk | Y(k-p), ..., Y(k-1)) are
constant for all k > p
}

For AD order 1, these two constraints correspond to strict stationarity.
For AD order greater than 1, this function should be interpreted as testing
marginal-constancy plus time-invariant transitions; these constraints are
not, in general, sufficient for full strict stationarity.

This is stronger than time-invariance alone, which only requires condition 2.

This function currently supports complete data only.

The null hypothesis is tested against the general (non-stationary) AD(p)
model. The degrees of freedom are computed from the fitted parameter counts:
\deqn{df = n_{params}(H_1) - n_{params}(H_0)}
where \eqn{H_1} is the unconstrained non-stationary model and \eqn{H_0} is
the stationary model.
}
\examples{
\donttest{
# Simulate stationary AD(1) data
set.seed(123)
y <- simulate_cat(200, 6, order = 1, n_categories = 2)

# Test stationarity
test <- test_stationarity_cat(y, order = 1)
print(test)
}

}
\references{
Xie, Y. and Zimmerman, D. L. (2013). Antedependence models for nonstationary
categorical longitudinal data with ignorable missingness: likelihood-based
inference. \emph{Statistics in Medicine}, 32, 3274-3289.
}
\seealso{
\code{\link{test_timeinvariance_cat}}, \code{\link{test_order_cat}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_gau.R
\name{simulate_gau}
\alias{simulate_gau}
\title{Simulate Gaussian antedependence series}
\usage{
simulate_gau(
  n_subjects,
  n_time,
  order = 1L,
  mu = NULL,
  phi = NULL,
  sigma = NULL,
  blocks = NULL,
  tau = 0,
  seed = NULL
)
}
\arguments{
\item{n_subjects}{number of subjects}

\item{n_time}{number of time points}

\item{order}{antedependence order, 0, 1 or 2}

\item{mu}{mean parameter; \code{NULL}, scalar, or length \code{n_time}}

\item{phi}{dependence parameter; ignored when \code{order = 0}.
For \code{order = 1}, \code{NULL}, scalar, or length \code{n_time}.
For \code{order = 2}, \code{NULL} or a 2 by \code{n_time} matrix.}

\item{sigma}{innovation standard deviation; \code{NULL}, scalar, or length \code{n_time}}

\item{blocks}{integer vector of length \code{n_subjects} indicating block membership
for each subject; if \code{NULL}, no block effect is applied}

\item{tau}{group effect vector indexed by block; \code{tau[1]} is forced to 0.
If scalar x, it is expanded to \code{c(0, x, ..., x)} with length equal to the
number of blocks}

\item{seed}{optional random seed for reproducibility}
}
\value{
numeric matrix with dimension \code{n_subjects} by \code{n_time}
}
\description{
Generate longitudinal continuous data from a Gaussian antedependence (AD)
model of order 0, 1, or 2 using a conditional regression on predecessors.
}
\details{
For \code{order = 0}, each time point is generated independently as
\code{Y[, t] = mu[t] + tau[block] + eps}, with \code{eps ~ N(0, sigma[t]^2)}.

For \code{order = 1}, for \code{t >= 2}:
\verb{Y[, t] = m_t + phi[t] * (Y[, t - 1] - m_\{t-1\}) + eps_t},
where \code{m_t = mu[t] + tau[block]} and \code{eps_t ~ N(0, sigma[t]^2)}.

For \code{order = 2}, for \code{t >= 3}:
\verb{Y[, t] = m_t + phi[1, t] * (Y[, t - 1] - m_\{t-1\}) + phi[2, t] * (Y[, t - 2] - m_\{t-2\}) + eps_t}.

If \code{blocks} is provided, each subject s belongs to a block and receives a
mean shift \code{tau[blocks[s]]}. \code{tau[1]} is forced to 0.
}
\examples{
y <- simulate_gau(
  n_subjects = 20,
  n_time = 6,
  order = 1,
  phi = 0.4,
  seed = 42
)
dim(y)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_order_cat.R
\name{run_order_tests_cat}
\alias{run_order_tests_cat}
\title{Run all pairwise order tests}
\usage{
run_order_tests_cat(
  y,
  max_order = 2,
  blocks = NULL,
  homogeneous = TRUE,
  n_categories = NULL,
  test = c("lrt", "score", "mlrt", "wald")
)
}
\arguments{
\item{y}{Integer matrix of categorical data (n_subjects x n_time).}

\item{max_order}{Maximum order to test. Default is 2.}

\item{blocks}{Optional block membership vector.}

\item{homogeneous}{Whether to use homogeneous parameters across blocks.}

\item{n_categories}{Number of categories (inferred if NULL).}

\item{test}{Type of test statistic for each pairwise comparison. One of
\code{"lrt"} (default), \code{"score"}, \code{"mlrt"}, or \code{"wald"}.
Passed to \code{\link{test_order_cat}}.}
}
\value{
A list containing:
\item{tests}{List of test_order_cat results for each comparison}
\item{table}{Summary data frame with all comparisons}
\item{fits}{List of all fitted models}
\item{selected_order}{Recommended order based on sequential testing at alpha=0.05}
}
\description{
Performs sequential likelihood ratio tests for AD orders 0 vs 1, 1 vs 2, etc.
}
\details{
This function performs forward selection: starting from order 0, it tests
whether increasing the order provides significant improvement. The selected
order is the highest order where the test was significant (at alpha = 0.05).
}
\examples{
\donttest{
y <- simulate_cat(200, 6, order = 1, n_categories = 2)
result <- run_order_tests_cat(y, max_order = 2)
print(result$table)
cat("Selected order:", result$selected_order, "\n")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_methods.R
\name{plot.cat_fit}
\alias{plot.cat_fit}
\title{Plot marginal probabilities of a categorical AD fit}
\usage{
\method{plot}{cat_fit}(x, ...)
}
\arguments{
\item{x}{A \code{cat_fit} object from \code{\link{fit_cat}}.}

\item{...}{Unused.}
}
\value{
\code{x}, invisibly.
}
\description{
Displays a stacked bar chart of the estimated marginal category
probabilities at each time point.
}
\examples{
\donttest{
set.seed(1)
y <- simulate_cat(n_subjects = 80, n_time = 5, order = 1, n_categories = 2)
fit <- fit_cat(y, order = 1, n_categories = 2)
plot(fit)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loglik_cat.R
\name{.logsumexp_cat}
\alias{.logsumexp_cat}
\title{Stable log-sum-exp}
\usage{
.logsumexp_cat(x)
}
\arguments{
\item{x}{Numeric vector}
}
\value{
Scalar log(sum(exp(x)))
}
\description{
Stable log-sum-exp
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{partial_corr}
\alias{partial_corr}
\title{Compute intervenor-adjusted partial correlation matrix}
\usage{
partial_corr(y, test = FALSE, n_digits = 3)
}
\arguments{
\item{y}{Numeric matrix with n_subjects rows and n_time columns.}

\item{test}{Logical; if TRUE, returns significance flags based on approximate
threshold 2/sqrt(n_eff) where n_eff = n_subjects - (lag - 1). Default FALSE.}

\item{n_digits}{Integer; number of decimal places for rounding. Default 3.}
}
\value{
A list with components:
\item{correlation}{Matrix with correlations (upper triangle) and variances (diagonal)}
\item{partial_correlation}{Matrix with partial correlations (lower triangle) and variances (diagonal)}
\item{significant}{(If test=TRUE) Matrix flagging significant partial correlations (1 = significant)}
\item{n_subjects}{Number of subjects}
\item{n_time}{Number of time points}
}
\description{
Computes the partial correlation between \code{Y[i]} and \code{Y[j]} adjusting for the
"intervenor" variables \verb{Y[i+1], ..., Y[j-1]}. Under an antedependence model
of order p, partial correlations for |i-j| > p should be approximately zero.
}
\details{
The intervenor-adjusted partial correlation between \code{Y[i]} and \code{Y[j]} (i < j) is
computed as the correlation between the residuals from regressing \code{Y[i]} and \code{Y[j]}
on the intervenor set \verb{Y[i+1], ..., Y[j-1]}.

For adjacent time points (|i-j| = 1), the partial correlation equals the
ordinary correlation since there are no intervenors.

The diagonal of both returned matrices contains variances (not correlations).
This keeps scale information available alongside correlation structure.

The significance test uses an approximate threshold of 2/sqrt(n_eff), which
corresponds roughly to a 95\% confidence bound under normality. This is a
rough screening tool, not a formal hypothesis test.
}
\examples{
\donttest{
data("bolus_inad")
pc <- partial_corr(bolus_inad$y, test = TRUE)

# View partial correlations (lower triangle)
pc$partial_correlation

# Extract variances from the diagonal
variances <- diag(pc$partial_correlation)

# Check which are "significant" (rough screen for AD order)
pc$significant
}

}
\references{
Zimmerman, D. L. and Nunez-Anton, V. (2009). Antedependence Models for
Longitudinal Data. CRC Press.
}
\seealso{
\code{\link{plot_prism}} for visual diagnostics
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_cat.R
\name{.validate_params_cat}
\alias{.validate_params_cat}
\title{Validate transition probability parameters}
\usage{
.validate_params_cat(marginal, transition, p, c, n_time, tol = 1e-08)
}
\arguments{
\item{marginal}{Marginal parameters}

\item{transition}{Transition parameters}

\item{p}{Order}

\item{c}{Number of categories}

\item{n_time}{Number of time points}

\item{tol}{Tolerance for checking sums}
}
\value{
TRUE if valid, otherwise throws error
}
\description{
Checks that transition probabilities are valid (non-negative, sum to 1).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_inad.R
\name{fit_inad}
\alias{fit_inad}
\title{Fit INAD antedependence model by maximum likelihood}
\usage{
fit_inad(
  y,
  order = 1,
  thinning = c("binom", "pois", "nbinom"),
  innovation = c("pois", "bell", "nbinom"),
  blocks = NULL,
  max_iter = 50,
  tol = 1e-06,
  verbose = FALSE,
  init_alpha = NULL,
  init_theta = NULL,
  init_tau = 0.4,
  init_nb_inno_size = 1,
  nb_inno_size_ub = 50,
  na_action = c("fail", "complete", "marginalize")
)
}
\arguments{
\item{y}{Integer matrix n_sub by n_time.}

\item{order}{Integer in \{0, 1, 2\}.}

\item{thinning}{One of "binom", "pois", "nbinom".}

\item{innovation}{One of "pois", "bell", "nbinom".}

\item{blocks}{Optional integer vector length n_sub. Default NULL.}

\item{max_iter}{Max iterations for FE coordinate descent.}

\item{tol}{Tolerance for FE log likelihood stopping.}

\item{verbose}{Logical.}

\item{init_alpha}{Optional initial alpha. For order 1 numeric length 1 or n_time.
For order 2 matrix n_time by 2 or list(alpha1, alpha2).}

\item{init_theta}{Optional initial theta numeric length 1 or n_time.}

\item{init_tau}{Optional initial tau. Scalar expands to c(0, x, ..., x). Vector forces first to 0.}

\item{init_nb_inno_size}{Optional initial size for innovation nbinom, length 1 or n_time.}

\item{nb_inno_size_ub}{Upper bound for innovation negative binomial size
parameter when \code{innovation = "nbinom"}. Default is 50.}

\item{na_action}{How to handle missing values:
\itemize{
\item \code{"fail"}: stop if any NA is present.
\item \code{"complete"}: fit using complete-case subjects only.
\item \code{"marginalize"}: maximize observed-data likelihood under MAR.
}}
}
\value{
A list of class \code{"inad_fit"} containing:
\describe{
\item{alpha}{Estimated antedependence parameter(s)}
\item{theta}{Estimated innovation parameter(s)}
\item{tau}{Estimated block effects (if applicable)}
\item{nb_inno_size}{Estimated innovation NB size parameter(s), when
\code{innovation = "nbinom"}}
\item{log_l}{Maximized log-likelihood}
\item{aic}{Akaike information criterion}
\item{bic}{Bayesian information criterion}
\item{n_params}{Number of free parameters}
\item{convergence}{Convergence code}
\item{settings}{Model and fitting settings}
}
}
\description{
Fits INAD models by maximum likelihood.
}
\details{
No fixed effect: time separable optimization using logL_inad_i with theta eliminated
by moment equations for order 1 and 2.

Fixed effect: block coordinate descent using nloptr BOBYQA, updating tau, alpha,
theta, and nb_inno_size if needed.
}
\examples{
set.seed(1)
y <- simulate_inad(
  n_subjects = 60,
  n_time = 5,
  order = 1,
  thinning = "binom",
  innovation = "pois",
  alpha = 0.3,
  theta = 2
)
fit <- fit_inad(y, order = 1, thinning = "binom", innovation = "pois", max_iter = 20)
fit$log_l
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/em_inad.R
\name{em_inad}
\alias{em_inad}
\title{EM algorithm for INAD model estimation}
\usage{
em_inad(
  y,
  order = 1,
  thinning = "binom",
  innovation = "pois",
  blocks = NULL,
  max_iter = 200,
  tol = 1e-07,
  alpha_init = NULL,
  theta_init = NULL,
  tau_init = NULL,
  nb_inno_size = NULL,
  safeguard = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{y}{Integer matrix with n_subjects rows and n_time columns.}

\item{order}{Model order (1 or 2). Order 0 does not require EM.}

\item{thinning}{Thinning operator: "binom", "pois", or "nbinom".}

\item{innovation}{Innovation distribution: "pois", "bell", or "nbinom".}

\item{blocks}{Optional integer vector of length n_subjects for block effects.}

\item{max_iter}{Maximum number of EM iterations.}

\item{tol}{Convergence tolerance for log-likelihood change.}

\item{alpha_init}{Optional initial values for alpha parameters.}

\item{theta_init}{Optional initial values for theta parameters.}

\item{tau_init}{Optional initial values for tau parameters.}

\item{nb_inno_size}{Size parameter for negative binomial innovation (if used).}

\item{safeguard}{Logical; if TRUE, use step-halving when likelihood decreases.}

\item{verbose}{Logical; if TRUE, print iteration progress.}
}
\value{
A list with class "inad_fit" containing estimated parameters.
}
\description{
Fits INAD models using the Expectation-Maximization algorithm.
This is an alternative to direct likelihood optimization.
}
\details{
For Gaussian and CAT EM entry points, see \code{\link{em_gau}} and
\code{\link{em_cat}}. For CAT specifically, \code{fit_cat()} supports
\code{na_action = "em"} for orders 0/1 and \code{na_action = "marginalize"}
for order 2 missing-data fits.
}
\examples{
set.seed(1)
y <- simulate_inad(
  n_subjects = 50,
  n_time = 5,
  order = 1,
  thinning = "binom",
  innovation = "pois",
  alpha = 0.25,
  theta = 2
)
fit <- em_inad(y, order = 1, thinning = "binom", innovation = "pois", max_iter = 20, tol = 1e-6)
fit$log_l
}
\seealso{
\code{\link{em_gau}}, \code{\link{em_cat}}, \code{\link{fit_inad}},
\code{\link{fit_cat}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_cat.R
\name{.pack_cat_params}
\alias{.pack_cat_params}
\title{Pack CAT parameters into unconstrained vector}
\usage{
.pack_cat_params(marginal, transition, p, c, n_time)
}
\arguments{
\item{marginal}{Marginal parameter list}

\item{transition}{Transition parameter list}

\item{p}{Order}

\item{c}{Number of categories}

\item{n_time}{Number of time points}
}
\value{
Numeric parameter vector
}
\description{
Pack CAT parameters into unconstrained vector
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cat_utils.R
\name{.counts_to_transition_probs}
\alias{.counts_to_transition_probs}
\title{Compute transition probabilities from counts (simpler version)}
\usage{
.counts_to_transition_probs(counts)
}
\arguments{
\item{counts}{Array of counts, last dimension is response}
}
\value{
Array of transition probabilities (same dimensions)
}
\description{
Given counts array with last dimension being Y_k and earlier dimensions
being conditioning variables, compute P(Y_k | conditioning).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gau_utils.R
\name{.partial_corr_gau}
\alias{.partial_corr_gau}
\title{Compute intervenor-adjusted sample partial correlation}
\usage{
.partial_corr_gau(y, i, j, mu = NULL)
}
\arguments{
\item{y}{Numeric matrix n_subjects by n_time.}

\item{i}{Row/column index (larger time index).}

\item{j}{Row/column index (smaller time index), j < i.}

\item{mu}{Optional mean vector for centering.}
}
\value{
Sample partial correlation (scalar).
}
\description{
Computes the sample partial correlation between Y_i and Y_j, adjusted for
the intervenors Y_(j+1), ..., Y_(i-1), from the residuals of the fitted
mean structure.
}
\details{
The intervenor-adjusted partial correlation r_(i,j|(j+1:i-1)) is computed
from residuals of regressing Y_i and Y_j on the intervenors.

Under AD(p), the partial correlation r_(i,i-k|(i-k+1:i-1)) should be
approximately zero for k > p.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loglik_inad.R
\name{.inad_conv_trunc}
\alias{.inad_conv_trunc}
\title{Truncated discrete convolution}
\usage{
.inad_conv_trunc(p, q, K)
}
\description{
Truncated discrete convolution
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_order_gau.R
\name{print.gau_bic_order}
\alias{print.gau_bic_order}
\title{Print method for BIC order selection}
\usage{
\method{print}{gau_bic_order}(x, ...)
}
\arguments{
\item{x}{Object of class \code{gau_bic_order}.}

\item{...}{Unused.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Print method for BIC order selection
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_homogeneity_inad.R
\name{.fit_inad_heterogeneous}
\alias{.fit_inad_heterogeneous}
\title{Fit fully heterogeneous INAD model}
\usage{
.fit_inad_heterogeneous(
  y,
  blocks,
  n_blocks,
  group_indices,
  order,
  thinning,
  innovation,
  ...
)
}
\arguments{
\item{y}{Data matrix}

\item{blocks}{Normalized block vector}

\item{n_blocks}{Number of groups}

\item{group_indices}{List of subject indices per group}

\item{order}{AD order}

\item{thinning}{Thinning type}

\item{innovation}{Innovation type}

\item{...}{Additional arguments passed to fit_inad}
}
\value{
A list mimicking fit_inad output with combined log-likelihood
}
\description{
Internal function that fits separate INAD models for each group and
combines the log-likelihoods.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-cochlear_implant.R
\docType{data}
\name{cochlear_implant}
\alias{cochlear_implant}
\title{Cochlear implant speech recognition data}
\format{
A list with six components:
\describe{
\item{y}{numeric matrix of dimension N by n_time containing all subjects}
\item{y_A}{numeric matrix for Group A subjects}
\item{y_B}{numeric matrix for Group B subjects}
\item{blocks}{integer vector of length N (1 = Group A, 2 = Group B)}
\item{group}{character vector of group labels ("A"/"B")}
\item{time}{integer vector of measurement occasions}
}
}
\source{
\url{https://homepage.divms.uiowa.edu/~dzimmer/Data-for-AD/speech_recognition_data.txt}
}
\usage{
cochlear_implant
}
\description{
Longitudinal speech recognition outcomes for two groups (A/B), including
incomplete records, from Zimmerman and Nunez-Anton antedependence book
companion data. This dataset is continuous-response data suitable for
Gaussian AD modeling.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{confint.antedep}
\alias{confint.antedep}
\alias{confint.gau_fit}
\alias{confint.cat_fit}
\alias{confint.inad_fit}
\title{Confidence Intervals for Antedependence Model Fits}
\usage{
\method{confint}{gau_fit}(object, parm, level = 0.95, ...)

\method{confint}{cat_fit}(object, parm, level = 0.95, y = NULL, ...)

\method{confint}{inad_fit}(object, parm, level = 0.95, y, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{gau_fit},
\code{cat_fit}, or \code{inad_fit}.}

\item{parm}{Unused (all parameters are returned).}

\item{level}{Confidence level; default \code{0.95}.}

\item{y}{(\code{inad_fit} only) The original data matrix used for
fitting, required by \code{\link{ci_inad}}.}

\item{...}{Passed to the underlying \code{ci_*} function.}
}
\value{
For \code{gau_fit}: a matrix with columns \code{lower} and
\code{upper} and one row per parameter.
For \code{cat_fit}: the \code{gau_ci} list returned by \code{\link{ci_cat}}.
For \code{inad_fit}: the \code{inad_ci} list returned by
\code{\link{ci_inad}}.
}
\description{
Computes Wald confidence intervals for the parameters of a fitted
antedependence model by delegating to the corresponding
\code{ci_gau}, \code{ci_cat}, or \code{ci_inad} function.
}
\examples{
\donttest{
set.seed(1)
y <- simulate_gau(n_subjects = 40, n_time = 5, order = 1, phi = 0.4)
fit <- fit_gau(y, order = 1)
confint(fit)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_order_cat.R
\name{test_order_cat}
\alias{test_order_cat}
\title{Likelihood ratio test for antedependence order (categorical AD data)}
\usage{
test_order_cat(
  y = NULL,
  order_null = 0,
  order_alt = 1,
  blocks = NULL,
  homogeneous = TRUE,
  n_categories = NULL,
  fit_null = NULL,
  fit_alt = NULL,
  test = c("lrt", "score", "mlrt", "wald")
)
}
\arguments{
\item{y}{Integer matrix with n_subjects rows and n_time columns. Each entry
should be a category code from 1 to c. Can be NULL if both fit_null and
fit_alt are provided.}

\item{order_null}{Order under the null hypothesis (default 0).}

\item{order_alt}{Order under the alternative hypothesis (default 1).
Must be greater than order_null.}

\item{blocks}{Optional integer vector of length n_subjects specifying group
membership.}

\item{homogeneous}{Logical. If TRUE (default), parameters are shared across
all groups.}

\item{n_categories}{Number of categories. If NULL, inferred from data.}

\item{fit_null}{Optional pre-fitted model under null hypothesis (class "cat_fit").
If provided, y is not required for fitting under H0.}

\item{fit_alt}{Optional pre-fitted model under alternative hypothesis.
If provided, y is not required for fitting under H1.}

\item{test}{Type of test statistic. One of \code{"lrt"} (default),
\code{"score"}, \code{"mlrt"}, or \code{"wald"}.}
}
\value{
A list of class \code{"cat_lrt"} containing:
\describe{
\item{method}{Inference method used: one of \code{"lrt"}, \code{"score"},
\code{"mlrt"}, or \code{"wald"}.}
\item{lrt_stat}{Likelihood ratio test statistic}
\item{df}{Degrees of freedom}
\item{p_value}{P-value from chi-square distribution}
\item{fit_null}{Fitted model under H0}
\item{fit_alt}{Fitted model under H1}
\item{order_null}{Order under null}
\item{order_alt}{Order under alternative}
\item{table}{Summary data frame}
}
}
\description{
Tests whether a higher-order AD model provides significantly better fit
than a lower-order model for categorical longitudinal data.
}
\details{
The likelihood ratio test statistic is:
\deqn{\lambda = -2[\ell_0 - \ell_1]}
where \eqn{\ell_0} and \eqn{\ell_1} are the maximized log-likelihoods under
the null and alternative hypotheses.

Under H0, \eqn{\lambda} follows a chi-square distribution with degrees of
freedom equal to the difference in the number of free parameters.

For testing AD(p) vs AD(p+1), the degrees of freedom are:
\deqn{df = (c-1)^2 \times c^p \times (n - p - 1)}
where c is the number of categories and n is the number of time points.

If \code{y} contains missing values and models are fit internally, this
function defaults to \code{na_action = "marginalize"} for fitting.
Score- and Wald-based variants currently require complete data.
}
\examples{
\donttest{
# Simulate AD(1) data
set.seed(123)
y <- simulate_cat(200, 6, order = 1, n_categories = 2)

# Test AD(0) vs AD(1)
test_01 <- test_order_cat(y, order_null = 0, order_alt = 1)
print(test_01$table)

# Test AD(1) vs AD(2)
test_12 <- test_order_cat(y, order_null = 1, order_alt = 2)
print(test_12$table)

# Using pre-fitted models
fit0 <- fit_cat(y, order = 0)
fit1 <- fit_cat(y, order = 1)
test_prefitted <- test_order_cat(fit_null = fit0, fit_alt = fit1)
}

}
\references{
Xie, Y. and Zimmerman, D. L. (2013). Antedependence models for nonstationary
categorical longitudinal data with ignorable missingness: likelihood-based
inference. \emph{Statistics in Medicine}, 32, 3274-3289.
}
\seealso{
\code{\link{fit_cat}}, \code{\link{bic_order_cat}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inad_utils.R
\name{.inad_effective_innovation_param}
\alias{.inad_effective_innovation_param}
\title{Effective innovation distribution parameter under block effects}
\usage{
.inad_effective_innovation_param(theta, tau, innovation)
}
\description{
For Poisson and negative binomial innovations, block effects are additive on
the mean. For Bell innovations, block effects are additive on the innovation
mean \code{theta * exp(theta)}, then mapped back to the Bell parameter.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_gau.R
\name{print.gau_fit}
\alias{print.gau_fit}
\title{Print Method for gau_fit Objects}
\usage{
\method{print}{gau_fit}(x, ...)
}
\arguments{
\item{x}{A \code{gau_fit} object.}

\item{...}{Additional arguments (ignored).}
}
\value{
The input object, invisibly.
}
\description{
Print Method for gau_fit Objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/em_cat.R
\name{.blend_cat_params_em}
\alias{.blend_cat_params_em}
\title{Blend old/new CAT parameters and renormalize probabilities}
\usage{
.blend_cat_params_em(
  old_marginal,
  old_transition,
  new_marginal,
  new_transition,
  step,
  order,
  n_time,
  n_blocks,
  homogeneous
)
}
\arguments{
\item{old_marginal}{Previous marginal parameters.}

\item{old_transition}{Previous transition parameters.}

\item{new_marginal}{Proposed marginal parameters.}

\item{new_transition}{Proposed transition parameters.}

\item{step}{Step size in (0, 1].}

\item{order}{Model order.}

\item{n_time}{Number of time points.}

\item{n_blocks}{Number of blocks.}

\item{homogeneous}{Whether parameters are shared across blocks.}
}
\value{
List with blended \code{marginal} and \code{transition}.
}
\description{
Blend old/new CAT parameters and renormalize probabilities
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gau_utils.R
\name{.group_means_gau}
\alias{.group_means_gau}
\title{Compute group-specific means for two-sample case}
\usage{
.group_means_gau(y, blocks)
}
\arguments{
\item{y}{Numeric matrix n_subjects by n_time.}

\item{blocks}{Integer vector indicating group membership.}
}
\value{
List with mu1 and mu2 vectors.
}
\description{
Compute group-specific means for two-sample case
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inad_utils.R
\name{.innov_vec}
\alias{.innov_vec}
\title{Innovation probability vector}
\usage{
.innov_vec(u_vals, lam, innovation, sz)
}
\arguments{
\item{u_vals}{Integer vector of possible innovation values.}

\item{lam}{Innovation parameter used by the PMF. This is the mean for
\code{"pois"} and \code{"nbinom"}, and the Bell parameter for \code{"bell"}.}

\item{innovation}{One of "pois", "bell", "nbinom".}

\item{sz}{Size parameter for negative binomial innovation (ignored otherwise).}
}
\value{
Numeric vector of probabilities.
}
\description{
Computes the probability mass for innovation counts.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-race_100km.R
\docType{data}
\name{race_100km}
\alias{race_100km}
\title{100km race split-time data}
\format{
A list with five components:
\describe{
\item{y}{numeric matrix of dimension N by 10 containing section split times}
\item{age}{numeric vector of subject ages (may include missing values)}
\item{subject}{integer subject identifiers}
\item{time}{integer vector of section indices (1:10)}
\item{section_labels}{character vector of split-time column names}
}
}
\source{
Combined table extracted from textbook race example data, stored in
\code{data-raw/external/100km_race_combined_extracted.csv}.
}
\usage{
race_100km
}
\description{
Split times (in minutes) from a 100km race example with 10 consecutive sections.
This is continuous-response longitudinal data suitable for Gaussian AD modeling.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_stationarity_inad.R
\name{run_stationarity_tests_inad}
\alias{run_stationarity_tests_inad}
\title{Run all stationarity-related tests for INAD}
\usage{
run_stationarity_tests_inad(
  y,
  order = 1,
  thinning = "binom",
  innovation = "pois",
  blocks = NULL,
  verbose = FALSE,
  ...
)
}
\arguments{
\item{y}{Integer matrix.}

\item{order}{Model order (1 or 2).}

\item{thinning}{Thinning operator.}

\item{innovation}{Innovation distribution.}

\item{blocks}{Optional block assignments.}

\item{verbose}{Logical.}

\item{...}{Additional arguments.}
}
\value{
A list with class "stationarity_tests_inad".
}
\description{
Run all stationarity-related tests for INAD
}
\examples{
set.seed(1)
y <- simulate_inad(
  n_subjects = 25,
  n_time = 5,
  order = 1,
  thinning = "binom",
  innovation = "pois",
  alpha = 0.3,
  theta = 2
)
out <- run_stationarity_tests_inad(
  y,
  order = 1,
  thinning = "binom",
  innovation = "pois",
  verbose = FALSE,
  max_iter = 15
)
out$summary
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bic_cat.R
\name{bic_cat}
\alias{bic_cat}
\title{Bayesian information criterion for fitted categorical AD models}
\usage{
bic_cat(fit, n_subjects = NULL)
}
\arguments{
\item{fit}{A fitted model object of class \code{"cat_fit"} returned by
\code{\link{fit_cat}}.}

\item{n_subjects}{Number of subjects. If NULL, extracted from fit.}
}
\value{
A numeric scalar BIC value.
}
\description{
Computes BIC using the fitted log likelihood and a parameter count for
categorical antedependence parameters.
}
\details{
The BIC is computed as:
\deqn{BIC = -2 \times \ell + k \times \log(N)}
where \eqn{\ell} is the log-likelihood, \eqn{k} is the number of free parameters,
and \eqn{N} is the number of subjects.
}
\examples{
set.seed(1)
y <- simulate_cat(40, 5, order = 1, n_categories = 2)

# Fit models of different orders
fit0 <- fit_cat(y, order = 0)
fit1 <- fit_cat(y, order = 1)
fit2 <- fit_cat(y, order = 2)

# Compare BIC
c(BIC_0 = bic_cat(fit0), BIC_1 = bic_cat(fit1), BIC_2 = bic_cat(fit2))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bic_gau.R
\name{aic_gau}
\alias{aic_gau}
\title{Akaike information criterion for fitted Gaussian AD models}
\usage{
aic_gau(fit)
}
\arguments{
\item{fit}{A fitted model object returned by \code{\link{fit_gau}}.}
}
\value{
A numeric scalar AIC value.
}
\description{
Computes AIC using the fitted log likelihood and a parameter count that
respects identifiability constraints for the Gaussian antedependence
parameters.
}
\details{
The AIC is computed as:
\deqn{AIC = -2 \times \ell + 2k}
where \eqn{\ell} is the log-likelihood and \eqn{k} is the number of free
parameters.

This function applies to Gaussian AD fits from \code{\link{fit_gau}}.
}
\examples{
set.seed(1)
y <- simulate_gau(n_subjects = 30, n_time = 5, order = 1, phi = 0.3)
fit <- fit_gau(y, order = 1)
aic_gau(fit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bic_inad.R
\name{aic_inad}
\alias{aic_inad}
\title{Akaike information criterion for fitted INAD models}
\usage{
aic_inad(fit)
}
\arguments{
\item{fit}{A fitted model object returned by \code{\link{fit_inad}}.}
}
\value{
A numeric scalar AIC value.
}
\description{
Computes AIC using the fitted log likelihood and a parameter count that
respects structural zeros and identifiability constraints.
}
\details{
The AIC is computed as:
\deqn{AIC = -2 \times \ell + 2k}
where \eqn{\ell} is the log-likelihood and \eqn{k} is the number of free
parameters.
}
\examples{
set.seed(1)
y <- simulate_inad(
  n_subjects = 40,
  n_time = 5,
  order = 1,
  thinning = "binom",
  innovation = "pois",
  alpha = 0.3,
  theta = 2
)
fit <- fit_inad(y, order = 1, thinning = "binom", innovation = "pois", max_iter = 20)
aic_inad(fit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_homogeneity_inad.R
\name{.count_params_homogeneity}
\alias{.count_params_homogeneity}
\title{Count parameters for homogeneity test}
\usage{
.count_params_homogeneity(
  fit,
  test,
  n_blocks,
  order,
  n_time,
  thinning,
  innovation,
  is_null
)
}
\arguments{
\item{fit}{Fitted model}

\item{test}{Test type}

\item{n_blocks}{Number of groups}

\item{order}{AD order}

\item{n_time}{Number of time points}

\item{thinning}{Thinning type}

\item{innovation}{Innovation type}

\item{is_null}{Whether this is the null model}
}
\value{
Number of free parameters
}
\description{
Count parameters for homogeneity test
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_cat.R
\name{.fit_cat_single_pop}
\alias{.fit_cat_single_pop}
\title{Fit CAT model for a single population}
\usage{
.fit_cat_single_pop(y, p, c, n_time, subject_mask = NULL)
}
\arguments{
\item{y}{Data matrix}

\item{p}{Order}

\item{c}{Number of categories}

\item{n_time}{Number of time points}

\item{subject_mask}{Logical vector for subject selection (NULL = all)}
}
\value{
List with marginal, transition, cell_counts, log_l
}
\description{
Fit CAT model for a single population
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{nobs.antedep}
\alias{nobs.antedep}
\alias{nobs.gau_fit}
\alias{nobs.cat_fit}
\alias{nobs.inad_fit}
\title{Number of observations for antedependence model fits}
\usage{
\method{nobs}{gau_fit}(object, ...)

\method{nobs}{cat_fit}(object, ...)

\method{nobs}{inad_fit}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{gau_fit},
\code{cat_fit}, or \code{inad_fit}.}

\item{...}{Unused.}
}
\value{
Integer scalar: number of subjects.
}
\description{
Number of observations for antedependence model fits
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loglik_inad.R
\name{.inad_order2_dist}
\alias{.inad_order2_dist}
\title{Retrieve/calculate order-2 transition distribution for one previous state pair}
\usage{
.inad_order2_dist(prev2, prev1, info, K)
}
\description{
Retrieve/calculate order-2 transition distribution for one previous state pair
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{.prism_residuals}
\alias{.prism_residuals}
\title{Compute partial residuals for PRISM plot}
\usage{
.prism_residuals(y, i, j)
}
\arguments{
\item{y}{Data matrix (n_subjects x n_time).}

\item{i}{First time index.}

\item{j}{Second time index (j > i + 1).}
}
\value{
List with resid_i and resid_j (partial residuals).
}
\description{
Compute partial residuals for PRISM plot
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{plot_prism}
\alias{plot_prism}
\title{PRISM plot (Partial Residual Intervenor Scatterplot Matrix)}
\usage{
plot_prism(
  y,
  time_labels = NULL,
  pch = 20,
  cex = 0.6,
  col_upper = "steelblue",
  col_lower = "firebrick",
  main = "PRISM Diagnostic Plot"
)
}
\arguments{
\item{y}{Numeric matrix with n_subjects rows and n_time columns.}

\item{time_labels}{Optional character vector of time point labels.
Default uses column names or "T1", "T2", etc.}

\item{pch}{Point character for scatterplots. Default 20 (filled circle).}

\item{cex}{Point size. Default 0.6.}

\item{col_upper}{Color for upper triangle plots. Default "steelblue".}

\item{col_lower}{Color for lower triangle (PRISM) plots. Default "firebrick".}

\item{main}{Overall title. Default "PRISM Diagnostic Plot".}
}
\value{
Invisibly returns NULL. Called for side effect (plotting).
}
\description{
Creates a matrix of scatterplots for diagnosing antedependence structure.
The upper triangle shows ordinary scatterplots of \code{Y[i]} vs \code{Y[j]}.
The lower triangle shows PRISM plots: residuals from regressing \code{Y[i]} and \code{Y[j]}
on the intervenor variables \verb{Y[i+1], ..., Y[j-1]}.
}
\details{
Under an antedependence model of order p, the partial correlation between
\code{Y[i]} and \code{Y[j]} given the intervenors should be zero when |i-j| > p.
This means PRISM plots in the lower triangle should show no association
for lags greater than p.

Interpretation:
\itemize{
\item Upper triangle: Shows marginal associations between time points
\item Lower triangle (PRISM): Shows conditional associations after
removing effects of intervenor variables
\item If AD(1) holds: Only the first sub-diagonal of lower triangle
should show association
\item If AD(2) holds: First two sub-diagonals should show association
}
}
\examples{
\donttest{
data("bolus_inad")
plot_prism(bolus_inad$y)

# With custom labels
plot_prism(bolus_inad$y, time_labels = paste0("Hour ", seq(0, 44, by = 4)))
}

}
\references{
Zimmerman, D. L. and Nunez-Anton, V. (2009). Antedependence Models for
Longitudinal Data. CRC Press. Chapter 2.
}
\seealso{
\code{\link{partial_corr}} for numerical partial correlations
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bic_cat.R
\name{aic_cat}
\alias{aic_cat}
\title{Akaike information criterion for fitted categorical AD models}
\usage{
aic_cat(fit)
}
\arguments{
\item{fit}{A fitted model object of class \code{"cat_fit"} returned by
\code{\link{fit_cat}}.}
}
\value{
A numeric scalar AIC value.
}
\description{
Computes AIC using the fitted log likelihood and a parameter count for
categorical antedependence parameters.
}
\details{
The AIC is computed as:
\deqn{AIC = -2 \times \ell + 2k}
where \eqn{\ell} is the log-likelihood and \eqn{k} is the number of free
parameters.
}
\examples{
set.seed(1)
y <- simulate_cat(40, 5, order = 1, n_categories = 2)
fit <- fit_cat(y, order = 1)
aic_cat(fit)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cat_utils.R
\name{.history_to_index}
\alias{.history_to_index}
\title{Convert history to array index}
\usage{
.history_to_index(history, n_categories)
}
\arguments{
\item{history}{Integer vector of category values}

\item{n_categories}{Number of categories}
}
\value{
Integer index (1-based) into flattened array
}
\description{
Convert history to array index
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-bolus_inad.R
\docType{data}
\name{bolus_inad}
\alias{bolus_inad}
\title{Morphine bolus analgesia counts}
\format{
A list with four components:
\describe{
\item{y}{integer matrix of dimension N by n_time containing all
subjects and time points}
\item{y_2mg}{integer matrix with rows corresponding to the 2 mg
treatment group}
\item{y_1mg}{integer matrix with rows corresponding to the 1 mg
treatment group}
\item{blocks}{integer vector of length N giving the block or
treatment group indicator, 1 for 2 mg and 2 for 1 mg}
}
}
\source{
Dataset \code{bolus} from the \pkg{cold} package, converted to
matrix form and grouped by treatment.
}
\usage{
bolus_inad
}
\description{
Morphine bolus self administration counts for two treatment groups
recorded at 12 four hour time points. The data are stored in matrix
form to facilitate use with antedependence models.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gau_utils.R
\name{.rss_gau}
\alias{.rss_gau}
\title{Compute residual sum of squares from AD regression}
\usage{
.rss_gau(y, i, p, mu = NULL, include_intercept = TRUE)
}
\arguments{
\item{y}{Numeric matrix n_subjects by n_time.}

\item{i}{Time index (1-based).}

\item{p}{Number of predecessors to include in regression (can be 0).}

\item{mu}{Optional mean vector. If provided, data is centered by mu before regression.
If NULL, regression includes an intercept.}

\item{include_intercept}{Logical. If TRUE and mu is NULL, include intercept in regression.}
}
\value{
Residual sum of squares (scalar).
}
\description{
For time point i, regress Y_i on its p_i predecessors (Y_(i-1), ..., Y_(i-p_i))
and return the residual sum of squares.
}
\details{
When mu is provided, the regression is:
\code{(Y_i - mu_i) ~ (Y_(i-1) - mu_(i-1)) + ... + (Y_(i-p) - mu_(i-p))}
with no intercept.

When mu is NULL and include_intercept is TRUE:
\code{Y_i ~ 1 + Y_(i-1) + ... + Y_(i-p)}

For i = 1 or p = 0, the RSS is simply the sum of squared deviations from the mean
(or from mu_1 if provided).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ci_cat.R
\name{summary.cat_ci}
\alias{summary.cat_ci}
\title{Summary method for cat_ci objects}
\usage{
\method{summary}{cat_ci}(object, ...)
}
\arguments{
\item{object}{A cat_ci object}

\item{...}{Additional arguments (ignored)}
}
\value{
A data frame summarizing all CIs
}
\description{
Summary method for cat_ci objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loglik_cat.R
\name{.logL_from_counts_cat}
\alias{.logL_from_counts_cat}
\title{Compute log-likelihood from cell counts (faster for large datasets)}
\usage{
.logL_from_counts_cat(y, order, marginal, transition, n_categories)
}
\arguments{
\item{y}{Data matrix}

\item{order}{AD order p}

\item{marginal}{Marginal parameters}

\item{transition}{Transition parameters}

\item{n_categories}{Number of categories}
}
\value{
Scalar log-likelihood
}
\description{
Instead of looping over subjects, this version uses aggregated cell counts.
Equivalent to logL_cat but more efficient.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loglik_inad.R
\name{.inad_make_thin_pmf}
\alias{.inad_make_thin_pmf}
\title{Build truncated thinning pmf for one previous count}
\usage{
.inad_make_thin_pmf(prev, alpha, thinning, K)
}
\description{
Build truncated thinning pmf for one previous count
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_cat.R
\name{.uniform_cat_params}
\alias{.uniform_cat_params}
\title{Build uniform CAT parameter values}
\usage{
.uniform_cat_params(p, c, n_time)
}
\arguments{
\item{p}{Order}

\item{c}{Number of categories}

\item{n_time}{Number of time points}
}
\value{
List with marginal and transition
}
\description{
Build uniform CAT parameter values
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/em_cat.R
\name{em_cat}
\alias{em_cat}
\title{EM algorithm for categorical AD model estimation}
\usage{
em_cat(
  y,
  order = 1,
  blocks = NULL,
  homogeneous = TRUE,
  n_categories = NULL,
  max_iter = 100,
  tol = 1e-06,
  epsilon = 1e-08,
  safeguard = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{y}{Integer matrix with n_subjects rows and n_time columns. Values are
category codes in \code{1, ..., n_categories}; \code{NA} is allowed.}

\item{order}{Antedependence order. Supported values are \code{0} and
\code{1}. Order \code{2} is not yet implemented in \code{em_cat()}.}

\item{blocks}{Optional block/group vector of length \code{n_subjects}. Any
coding is accepted (e.g., non-sequential integers or factor levels).}

\item{homogeneous}{Logical. If \code{TRUE}, a single parameter set is fitted
across blocks. If \code{FALSE}, separate parameters are fitted by block.}

\item{n_categories}{Number of categories. If \code{NULL}, inferred from
observed data.}

\item{max_iter}{Maximum number of EM iterations.}

\item{tol}{Convergence tolerance on absolute log-likelihood change.}

\item{epsilon}{Small positive constant used for smoothing and numerical
stability.}

\item{safeguard}{Logical; if \code{TRUE}, apply step-halving when an M-step
update decreases observed-data log-likelihood.}

\item{verbose}{Logical; if \code{TRUE}, print EM progress.}
}
\value{
A \code{cat_fit} object with fields matching \code{\link{fit_cat}}.
In EM mode, \code{cell_counts} stores expected counts from the final
E-step, with \code{settings$cell_counts_type = "expected"}.
}
\description{
Fits categorical antedependence models with missing outcomes using the
Expectation-Maximization (EM) algorithm for orders 0 and 1.
}
\details{
For complete data (no missing values), this function defers to
\code{\link{fit_cat}} with closed-form MLEs.

For missing data and orders 0/1, each EM iteration computes expected
sufficient statistics with a forward-backward E-step, then updates
probabilities by normalized expected counts in the M-step.
If \code{safeguard = TRUE}, a step-halving line search is applied to the
M-step update whenever the observed-data likelihood decreases.

A final E-step is run before returning so that \code{log_l}/AIC/BIC and
expected cell counts correspond exactly to the returned parameter values.
}
\examples{
set.seed(1)
y <- simulate_cat(n_subjects = 40, n_time = 5, order = 1, n_categories = 3)
y[sample(length(y), 10)] <- NA
fit <- em_cat(y, order = 1, n_categories = 3, max_iter = 20, tol = 1e-5)
fit$settings$na_action

}
\seealso{
\code{\link{fit_cat}}, \code{\link{logL_cat}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inad_utils.R
\name{.bell_theta_from_mean}
\alias{.bell_theta_from_mean}
\title{Convert Bell mean to Bell parameter}
\usage{
.bell_theta_from_mean(mu)
}
\description{
Convert Bell mean to Bell parameter
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_gau_em.R
\name{.fit_gau_em}
\alias{.fit_gau_em}
\title{EM algorithm for AD with missing data}
\usage{
.fit_gau_em(
  y,
  order,
  blocks = NULL,
  estimate_mu = TRUE,
  max_iter = 100,
  tol = 1e-06,
  verbose = FALSE,
  ...
)
}
\arguments{
\item{y}{Data matrix (n_subjects x n_time), may contain NA}

\item{order}{Antedependence order (0, 1, or 2)}

\item{blocks}{Block membership vector (optional)}

\item{estimate_mu}{Logical, whether to estimate mu (default TRUE)}

\item{max_iter}{Maximum number of EM iterations (default 100)}

\item{tol}{Convergence tolerance on log-likelihood (default 1e-6)}

\item{verbose}{Logical, print iteration info (default FALSE)}
}
\value{
List with fitted model (same structure as fit_gau, plus EM info)
}
\description{
Implements the EM algorithm for fitting AD(p) models with missing data
under MAR (Missing At Random) assumption.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cat_utils.R
\name{print.cat_fit}
\alias{print.cat_fit}
\title{Print method for cat_fit objects}
\usage{
\method{print}{cat_fit}(x, ...)
}
\arguments{
\item{x}{A cat_fit object}

\item{...}{Additional arguments (ignored)}
}
\value{
Invisibly returns \code{x}.
}
\description{
Print method for cat_fit objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/em_cat.R
\name{.expected_counts_to_cell_counts_cat_em}
\alias{.expected_counts_to_cell_counts_cat_em}
\title{Convert EM expected counts to fit_cat-style cell_counts}
\usage{
.expected_counts_to_cell_counts_cat_em(
  counts,
  order,
  n_time,
  n_blocks,
  homogeneous
)
}
\arguments{
\item{counts}{Expected counts from the E-step.}

\item{order}{Model order.}

\item{n_time}{Number of time points.}

\item{n_blocks}{Number of blocks.}

\item{homogeneous}{Logical; whether parameters are shared across blocks.}
}
\value{
A list shaped like \code{fit_cat(...)} \code{cell_counts}.
}
\description{
Convert EM expected counts to fit_cat-style cell_counts
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cat_utils.R
\name{.validate_blocks_cat}
\alias{.validate_blocks_cat}
\title{Validate blocks parameter}
\usage{
.validate_blocks_cat(blocks, n_subjects)
}
\arguments{
\item{blocks}{Block membership vector}

\item{n_subjects}{Number of subjects}
}
\value{
Validated blocks vector (integer, 1-indexed)
}
\description{
Validate blocks parameter
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{logLik.antedep}
\alias{logLik.antedep}
\alias{logLik.gau_fit}
\alias{logLik.cat_fit}
\alias{logLik.inad_fit}
\title{Log-likelihood for antedependence model fits}
\usage{
\method{logLik}{gau_fit}(object, ...)

\method{logLik}{cat_fit}(object, ...)

\method{logLik}{inad_fit}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{gau_fit},
\code{cat_fit}, or \code{inad_fit}.}

\item{...}{Unused; present for S3 consistency.}
}
\value{
A scalar of class \code{"logLik"} with attributes
\code{df} (number of free parameters) and \code{nobs}
(number of subjects).
}
\description{
Extracts the maximised log-likelihood from a fitted antedependence model
and returns a \code{"logLik"} object compatible with \code{\link[stats]{AIC}}
and \code{\link[stats]{BIC}}.
}
\examples{
\donttest{
set.seed(1)
y <- simulate_gau(n_subjects = 40, n_time = 5, order = 1, phi = 0.4)
fit <- fit_gau(y, order = 1)
logLik(fit)
AIC(fit)
BIC(fit)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_homogeneity_gau.R
\name{test_homogeneity_gau}
\alias{test_homogeneity_gau}
\title{Likelihood ratio test for homogeneity across groups (Gaussian AD data)}
\usage{
test_homogeneity_gau(y, blocks, p = 1L, use_modified = TRUE)
}
\arguments{
\item{y}{Numeric matrix with n_subjects rows and n_time columns.}

\item{blocks}{Integer vector of length n_subjects indicating group membership.}

\item{p}{Antedependence order. This is the same order parameter named
\code{order} in \code{\link{fit_gau}}.}

\item{use_modified}{Logical. If TRUE (default), use modified test statistic
for better small-sample approximation.}
}
\value{
A list with class \code{gau_homogeneity_test} containing:
\describe{
\item{method}{Inference method used (\code{"lrt"}).}
\item{statistic}{Test statistic value}
\item{statistic_modified}{Modified test statistic (if use_modified = TRUE)}
\item{df}{Degrees of freedom}
\item{p_value}{P-value from chi-square distribution}
\item{p_value_modified}{P-value from modified test}
\item{G}{Number of groups}
\item{group_sizes}{Sample sizes for each group}
\item{order}{Antedependence order}
}
}
\description{
Tests the null hypothesis that G groups have the same AD(p) covariance
structure against the alternative that they have different AD(p) structures.
This implements Theorem 6.6 of Zimmerman & Núñez-Antón (2009).
}
\details{
The test compares:
\itemize{
\item H0: All G groups have the same AD(p) covariance matrix Sigma(theta)
\item H1: Groups have different AD(p) covariance matrices Sigma(theta_g)
}

The likelihood ratio test statistic (Theorem 6.6) involves comparing
pooled and within-group RSS values. The degrees of freedom are
(G-1)(2n - p)(p + 1)/2.

This test is useful for determining whether a common covariance structure
can be assumed across treatment groups before performing mean comparisons.
}
\examples{
\donttest{
# Simulate data from two groups with same covariance
n1 <- 30
n2 <- 35
y1 <- simulate_gau(n1, n_time = 6, order = 1, phi = 0.5, sigma = 1)
y2 <- simulate_gau(n2, n_time = 6, order = 1, phi = 0.5, sigma = 1)
y <- rbind(y1, y2)
blocks <- c(rep(1, n1), rep(2, n2))

# Test homogeneity
test <- test_homogeneity_gau(y, blocks, p = 1)
print(test)
}

}
\references{
Zimmerman, D.L. and Núñez-Antón, V. (2009). Antedependence Models for
Longitudinal Data. Chapman & Hall/CRC. Section 6.4.

Kenward, M.G. (1987). A method for comparing profiles of repeated measurements.
Applied Statistics, 36, 296-308.
}
\seealso{
\code{\link{test_order_gau}}, \code{\link{test_two_sample_gau}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gau_utils.R
\name{.psi_kenward}
\alias{.psi_kenward}
\title{Modified psi function for test statistic correction}
\usage{
.psi_kenward(x, y)
}
\arguments{
\item{x}{First argument.}

\item{y}{Second argument.}
}
\value{
Value of psi(x, y).
}
\description{
Computes the psi function used in the modified likelihood ratio test
(Kenward, 1987) for better small-sample approximation.
}
\details{
psi(x, y) is defined as:
\itemize{
\item 0 if x = 0
\item (2y - 1) / (2y(y - 1)) if x = 1
\item 2 * sum from l=1 to x/2 of (y + 2l - 2)^(-1) if x > 0 and even
\item psi(1, y) + psi(x - 1, y + 1) if x > 1 and odd
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_homogeneity_inad.R
\name{print.homogeneity_tests_inad}
\alias{print.homogeneity_tests_inad}
\title{Print method for homogeneity_tests_inad}
\usage{
\method{print}{homogeneity_tests_inad}(x, digits = 4, ...)
}
\arguments{
\item{x}{Object of class \code{homogeneity_tests_inad}}

\item{digits}{Number of digits for printing}

\item{...}{Unused}
}
\value{
Invisibly returns \code{x}.
}
\description{
Print method for homogeneity_tests_inad
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-cattle_growth.R
\docType{data}
\name{cattle_growth}
\alias{cattle_growth}
\title{Cattle growth data (Treatments A and B)}
\format{
A list with five components:
\describe{
\item{y}{numeric matrix of dimension N by n_time containing all subjects}
\item{y_A}{numeric matrix for Treatment A subjects}
\item{y_B}{numeric matrix for Treatment B subjects}
\item{blocks}{integer vector of length N (1 = Treatment A, 2 = Treatment B)}
\item{time}{integer vector of measurement occasions}
}
}
\source{
\url{https://homepage.divms.uiowa.edu/~dzimmer/Data-for-AD/cattle_growth_data_Treatment\%20A.txt}
and
\url{https://homepage.divms.uiowa.edu/~dzimmer/Data-for-AD/cattle_growth_data_Treatment\%20B.txt}
}
\usage{
cattle_growth
}
\description{
Longitudinal cattle growth measurements for two treatment groups from
Zimmerman and Nunez-Anton antedependence book companion data. This dataset
is continuous-response data suitable for Gaussian AD modeling.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loglik_inad.R
\name{logL_inad}
\alias{logL_inad}
\title{Log-likelihood for INAD models (with missing data support)}
\usage{
logL_inad(
  y,
  order = 1,
  thinning = c("binom", "pois", "nbinom"),
  innovation = c("pois", "bell", "nbinom"),
  alpha,
  theta,
  nb_inno_size = NULL,
  blocks = NULL,
  tau = 0,
  na_action = c("fail", "complete", "marginalize")
)
}
\arguments{
\item{y}{Integer matrix n_sub by n_time.}

\item{order}{Integer in \{0, 1, 2\}.}

\item{thinning}{One of "binom", "pois", "nbinom".}

\item{innovation}{One of "pois", "bell", "nbinom".}

\item{alpha}{Thinning parameters. For order 1, numeric length 1 or n_time.
For order 2, either a matrix n_time by 2 or a list(alpha1, alpha2).}

\item{theta}{Innovation parameter(s). Numeric length 1 or n_time. For
Poisson and negative binomial innovations, this is the innovation mean.
For Bell innovations, this is the Bell rate parameter
(mean \eqn{\theta e^\theta}).}

\item{nb_inno_size}{Size parameter for innovation "nbinom". Numeric length 1 or n_time.}

\item{blocks}{Optional integer vector of length n_sub. If NULL, no fixed effect.}

\item{tau}{Optional numeric vector. Only used if blocks is not NULL.}

\item{na_action}{How to handle missing values:
\itemize{
\item \code{"fail"}: error if any NA is present.
\item \code{"complete"}: use only complete-case subjects.
\item \code{"marginalize"}: observed-data likelihood under MAR via truncated-state recursion.
}}
}
\value{
A scalar log likelihood.
}
\description{
If blocks is NULL, this computes the log likelihood as the sum of per time
contributions from logL_inad_i for computational convenience.
}
\examples{
set.seed(1)
y <- simulate_inad(
  n_subjects = 60,
  n_time = 5,
  order = 1,
  thinning = "binom",
  innovation = "pois",
  alpha = 0.3,
  theta = 2
)
fit <- fit_inad(y, order = 1, thinning = "binom", innovation = "pois", max_iter = 20)
logL_inad(
  y,
  order = 1,
  thinning = "binom",
  innovation = "pois",
  alpha = fit$alpha,
  theta = fit$theta
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_order_cat.R
\name{print.cat_lrt}
\alias{print.cat_lrt}
\title{Print method for cat_lrt objects}
\usage{
\method{print}{cat_lrt}(x, ...)
}
\arguments{
\item{x}{A cat_lrt object}

\item{...}{Additional arguments (ignored)}
}
\value{
Invisibly returns \code{x}.
}
\description{
Print method for cat_lrt objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_cat.R
\name{.default_marginal_cat}
\alias{.default_marginal_cat}
\title{Create default marginal parameters (uniform)}
\usage{
.default_marginal_cat(p, c, n_time, n_blocks, homogeneous)
}
\arguments{
\item{p}{Order}

\item{c}{Number of categories}

\item{n_time}{Number of time points}

\item{n_blocks}{Number of groups}

\item{homogeneous}{Whether to share parameters}
}
\value{
List of marginal parameters
}
\description{
Create default marginal parameters (uniform)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ci_inad.R
\name{print.inad_ci}
\alias{print.inad_ci}
\title{Print method for INAD confidence intervals}
\usage{
\method{print}{inad_ci}(x, ...)
}
\arguments{
\item{x}{An object of class \code{inad_ci}.}

\item{...}{Unused.}
}
\value{
The input object, invisibly.
}
\description{
Print method for INAD confidence intervals
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ci_cat.R
\name{print.cat_ci}
\alias{print.cat_ci}
\title{Print method for cat_ci objects}
\usage{
\method{print}{cat_ci}(x, ...)
}
\arguments{
\item{x}{A cat_ci object}

\item{...}{Additional arguments (ignored)}
}
\value{
Invisibly returns \code{x}.
}
\description{
Print method for cat_ci objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inad_utils.R
\name{.inad_effective_innovation_mean}
\alias{.inad_effective_innovation_mean}
\title{Effective innovation mean under block effects}
\usage{
.inad_effective_innovation_mean(theta, tau, innovation)
}
\description{
Effective innovation mean under block effects
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_stationarity_cat.R
\name{.fit_cat_stationary}
\alias{.fit_cat_stationary}
\title{Fit stationary categorical AD model}
\usage{
.fit_cat_stationary(y, p, c, blocks = NULL, homogeneous = TRUE)
}
\arguments{
\item{y}{Data matrix}

\item{p}{Order}

\item{c}{Number of categories}

\item{blocks}{Block vector (or NULL)}

\item{homogeneous}{Whether to pool across blocks}
}
\value{
A cat_fit-like object with stationary constraints
}
\description{
Internal function to fit a model where both marginal and transition
probabilities are constant over time.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cat_utils.R
\name{.count_params_cat}
\alias{.count_params_cat}
\title{Count free parameters for AD(p) categorical model}
\usage{
.count_params_cat(
  order,
  n_categories,
  n_time,
  n_blocks = 1,
  homogeneous = TRUE
)
}
\arguments{
\item{order}{AD order p}

\item{n_categories}{Number of categories c}

\item{n_time}{Number of time points n}

\item{n_blocks}{Number of blocks/groups}

\item{homogeneous}{Whether parameters are shared across blocks}
}
\value{
Number of free parameters
}
\description{
Count free parameters for AD(p) categorical model
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inad_utils.R
\name{.unpack_alpha}
\alias{.unpack_alpha}
\title{Unpack alpha parameters}
\usage{
.unpack_alpha(order, alpha, N)
}
\arguments{
\item{order}{Model order (0, 1, or 2).}

\item{alpha}{Alpha parameter in various formats.}

\item{N}{Number of time points.}
}
\value{
List with elements a1 and a2.
}
\description{
Converts alpha from various input formats to a standardized list.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loglik_inad.R
\name{.inad_build_transitions}
\alias{.inad_build_transitions}
\title{Build transition objects for truncated-state missing-data likelihood}
\usage{
.inad_build_transitions(
  order,
  N,
  K,
  alpha1,
  alpha2,
  thinning,
  innovation,
  lambda,
  nb_inno_size
)
}
\description{
Build transition objects for truncated-state missing-data likelihood
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gau_utils.R
\name{.rss_vector_gau}
\alias{.rss_vector_gau}
\title{Compute RSS vector for all time points under AD(p)}
\usage{
.rss_vector_gau(y, p, mu = NULL)
}
\arguments{
\item{y}{Numeric matrix n_subjects by n_time.}

\item{p}{Antedependence order.}

\item{mu}{Optional mean vector for centering.}
}
\value{
Numeric vector of length n_time with RSS for each time point.
}
\description{
Compute RSS vector for all time points under AD(p)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cat_utils.R
\name{.get_history}
\alias{.get_history}
\title{Extract history for a subject at time k}
\usage{
.get_history(y_row, k, order)
}
\arguments{
\item{y_row}{Single row of data (one subject)}

\item{k}{Current time index}

\item{order}{AD order p}
}
\value{
Integer vector of length min(k-1, order) with category history
}
\description{
Extract history for a subject at time k
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cat_utils.R
\name{.count_cells_table_cat}
\alias{.count_cells_table_cat}
\title{Count cells efficiently using table()}
\usage{
.count_cells_table_cat(y, time_indices, n_categories, subject_mask = NULL)
}
\arguments{
\item{y}{Data matrix (n_subjects x n_time)}

\item{time_indices}{Integer vector of time indices}

\item{n_categories}{Number of categories}

\item{subject_mask}{Logical vector for subject selection}
}
\value{
Array of counts
}
\description{
Alternative implementation using R's table function.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_cat.R
\name{.cat_prob_to_theta}
\alias{.cat_prob_to_theta}
\title{Convert probability row to unconstrained logits}
\usage{
.cat_prob_to_theta(prob, c, eps = 1e-08)
}
\arguments{
\item{prob}{Probability vector of length c}

\item{c}{Number of categories}

\item{eps}{Lower bound for numerical stability}
}
\value{
Numeric vector of length c-1
}
\description{
Convert probability row to unconstrained logits
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_gau_em.R
\name{.initialize_gau_marginal}
\alias{.initialize_gau_marginal}
\title{Initialize from marginal statistics (ignoring dependence)}
\usage{
.initialize_gau_marginal(y, order, blocks)
}
\description{
Initialize from marginal statistics (ignoring dependence)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_mean_gau.R
\name{test_one_sample_gau}
\alias{test_one_sample_gau}
\title{One-sample test for mean structure under antedependence}
\usage{
test_one_sample_gau(y, mu0, p = 1L, order = NULL, use_modified = TRUE)
}
\arguments{
\item{y}{Numeric matrix with n_subjects rows and n_time columns.}

\item{mu0}{Hypothesized mean vector under the null (length n_time).}

\item{p}{Antedependence order of the covariance structure. This is the same
order parameter named \code{order} in \code{\link{fit_gau}}.}

\item{order}{Optional alias for \code{p}. Supply only one of \code{p} or
\code{order}.}

\item{use_modified}{Logical. If TRUE (default), use the modified test statistic
(formula 7.7) for better small-sample approximation.}
}
\value{
A list with class \code{gau_mean_test} containing:
\describe{
\item{method}{Inference method used (\code{"lrt"}).}
\item{test_type}{"one-sample"}
\item{mu0}{Hypothesized mean under null}
\item{mu_hat}{MLE of mean (sample mean)}
\item{statistic}{Test statistic value}
\item{statistic_modified}{Modified test statistic (if use_modified = TRUE)}
\item{df}{Degrees of freedom (n_time)}
\item{p_value}{P-value from chi-square distribution}
\item{p_value_modified}{P-value from modified test}
\item{order}{Antedependence order used}
}
}
\description{
Tests the null hypothesis that the mean vector equals a specified value
mu = mu_0 against the alternative mu != mu_0, under an AD(p) covariance
structure. This implements Theorem 7.1 of Zimmerman & Núñez-Antón (2009).
}
\details{
The test exploits the AD structure to gain power over tests that don't
assume any covariance structure. The likelihood ratio test statistic
(Theorem 7.1) is:
\deqn{N \sum_{i=1}^{n} [\log RSS_i(\mu_0) - \log RSS_i(\hat{\mu})]}

where RSS_i(mu) is the residual sum of squares from the regression of
Y_i - mu_i on its p predecessors Y_(i-1) - mu_(i-1), ..., Y_(i-p) - mu_(i-p).

The test has n degrees of freedom (one for each component of mu).
}
\examples{
\donttest{
# Simulate data with known mean
mu_true <- c(10, 11, 12, 13, 14, 15)
y <- simulate_gau(n_subjects = 50, n_time = 6, order = 1, mu = mu_true)

# Test if mean is zero
test <- test_one_sample_gau(y, mu0 = rep(0, 6), p = 1)
print(test)

# Test if mean equals true value (should not reject)
test2 <- test_one_sample_gau(y, mu0 = mu_true, p = 1)
print(test2)
}

}
\references{
Zimmerman, D.L. and Núñez-Antón, V. (2009). Antedependence Models for
Longitudinal Data. Chapman & Hall/CRC. Chapter 7.
}
\seealso{
\code{\link{test_two_sample_gau}}, \code{\link{test_order_gau}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loglik_inad.R
\name{.inad_subject_ll_order0}
\alias{.inad_subject_ll_order0}
\title{Subject-level observed-data likelihood for INAD(0)}
\usage{
.inad_subject_ll_order0(y_s, innov_list, K)
}
\description{
Subject-level observed-data likelihood for INAD(0)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loglik_cat.R
\name{.logL_subject_cat_observed}
\alias{.logL_subject_cat_observed}
\title{Compute observed-data log-likelihood contribution from one subject}
\usage{
.logL_subject_cat_observed(y_s, p, c, marginal, transition, na_action)
}
\arguments{
\item{y_s}{Integer vector of length n_time (one subject's data; may contain NA)}

\item{p}{Order}

\item{c}{Number of categories}

\item{marginal}{Marginal parameters for this population}

\item{transition}{Transition parameters for this population}

\item{na_action}{Missing-data handling mode}
}
\value{
Scalar log-likelihood contribution
}
\description{
Compute observed-data log-likelihood contribution from one subject
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cat_utils.R
\name{.safe_log}
\alias{.safe_log}
\title{Safe log function (0 * log(0) = 0)}
\usage{
.safe_log(x)
}
\arguments{
\item{x}{Numeric vector or array}
}
\value{
Log of x, with 0 for x = 0
}
\description{
Safe log function (0 * log(0) = 0)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_cat.R
\name{simulate_cat}
\alias{simulate_cat}
\title{Simulate categorical antedependence series}
\usage{
simulate_cat(
  n_subjects,
  n_time,
  order = 1,
  n_categories = 2,
  marginal = NULL,
  transition = NULL,
  blocks = NULL,
  homogeneous = TRUE,
  seed = NULL
)
}
\arguments{
\item{n_subjects}{Number of subjects to simulate.}

\item{n_time}{Number of time points.}

\item{order}{Antedependence order p. Must be 0, 1, or 2. Default is 1.}

\item{n_categories}{Number of categories c. Default is 2 (binary).}

\item{marginal}{List of marginal/joint probabilities for initial time points.
If NULL, uniform probabilities are used. See Details for structure.}

\item{transition}{List of transition probability arrays for time points
k = p+1 to n. If NULL, uniform transitions are used. See Details.}

\item{blocks}{Optional integer vector of length n_subjects specifying group
membership. Used with homogeneous = FALSE.}

\item{homogeneous}{Logical. If TRUE (default), same parameters for all subjects.
If FALSE, marginal and transition should be lists indexed by block.}

\item{seed}{Optional random seed for reproducibility.}
}
\value{
Integer matrix with n_subjects rows and n_time columns, where each
entry is a category code from 1 to c.
}
\description{
Generate simulated longitudinal categorical data from an AD(p) model with
specified transition probabilities.
}
\details{
Data are simulated sequentially:
\enumerate{
\item For k = 1: Draw Y(1) from marginal distribution
\item For k = 2 to p: Draw Y(k) conditional on Y(1), ..., Y(k-1)
\item For k = p+1 to n: Draw Y(k) conditional on Y(k-p), ..., Y(k-1)
}

Parameter structure for marginal:
\itemize{
\item Order 0: List with elements t1, t2, ..., tn, each a vector of length c
summing to 1
\item Order 1: List with element t1 (vector of length c)
\item Order 2: List with t1 (vector), t2_given_1to1 (c x c matrix where rows
represent conditioning values and columns represent outcomes)
}

Parameter structure for transition:
\itemize{
\item Order 0: Not used (NULL)
\item Order 1: List with elements t2, t3, ..., tn, each c x c matrix where
rows are previous values and columns are current values (rows sum to 1)
\item Order 2: List with elements t3, t4, ..., tn, each c x c x c array where
first two indices are conditioning values and third is outcome
}
}
\examples{
y <- simulate_cat(n_subjects = 30, n_time = 5, order = 1, n_categories = 3, seed = 1)
dim(y)

}
\references{
Xie, Y. and Zimmerman, D. L. (2013). Antedependence models for nonstationary
categorical longitudinal data with ignorable missingness: likelihood-based
inference. \emph{Statistics in Medicine}, 32, 3274-3289.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cat_utils.R
\name{.counts_to_probs_cat}
\alias{.counts_to_probs_cat}
\title{Convert counts to probabilities (with safe division)}
\usage{
.counts_to_probs_cat(counts, margin = NULL)
}
\arguments{
\item{counts}{Array of counts}

\item{margin}{Which margins to condition on (sum over). NULL = no conditioning (joint).}
}
\value{
Array of probabilities
}
\description{
Convert counts to probabilities (with safe division)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ci_inad.R
\name{summary.inad_ci}
\alias{summary.inad_ci}
\title{Summary method for inad_ci objects}
\usage{
\method{summary}{inad_ci}(object, ...)
}
\arguments{
\item{object}{An \code{inad_ci} object.}

\item{...}{Unused.}
}
\value{
A data frame stacking available confidence intervals with columns
\code{param}, \code{component}, \code{est}, \code{se}, \code{lower},
\code{upper}, and \code{level}. Returns \code{NULL} if no intervals are available.
}
\description{
Summary method for inad_ci objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_homogeneity_cat.R
\name{test_timeinvariance_cat}
\alias{test_timeinvariance_cat}
\title{Likelihood ratio test for time-invariance (categorical data)}
\usage{
test_timeinvariance_cat(
  y,
  order = 1,
  blocks = NULL,
  homogeneous = TRUE,
  n_categories = NULL,
  test = c("lrt", "score", "mlrt")
)
}
\arguments{
\item{y}{Integer matrix with n_subjects rows and n_time columns. Each entry
should be a category code from 1 to c.}

\item{order}{Antedependence order p. Default is 1.}

\item{blocks}{Optional integer vector of length n_subjects specifying group
membership.}

\item{homogeneous}{Logical. If TRUE (default), parameters are shared across
all groups.}

\item{n_categories}{Number of categories. If NULL, inferred from data.}

\item{test}{Type of test statistic. One of \code{"lrt"} (default),
\code{"score"}, or \code{"mlrt"}.}
}
\value{
A list of class \code{"cat_lrt"} containing:
\describe{
\item{method}{Inference method used: one of \code{"lrt"}, \code{"score"},
\code{"mlrt"}, or \code{"wald"}.}
\item{lrt_stat}{Likelihood ratio test statistic}
\item{df}{Degrees of freedom}
\item{p_value}{P-value from chi-square distribution}
\item{fit_null}{Fitted time-invariant model (H0)}
\item{fit_alt}{Fitted time-varying model (H1)}
\item{table}{Summary data frame}
}
}
\description{
Tests whether transition probabilities are constant over time in a
categorical antedependence model.
}
\details{
The null hypothesis is that all transition probabilities (for k > p) are
equal across time:
\deqn{H_0: \pi_{y_k | y_{k-p}, \ldots, y_{k-1}} \text{ is constant for } k = p+1, \ldots, n}

This reduces (n-p) separate transition matrices/arrays to a single one.

The degrees of freedom are:
\deqn{df = (c-1) \times c^p \times (n - p - 1)}

This function currently supports complete data only. If \code{y} contains
missing values, use model-fitting functions (for example \code{fit_cat})
directly with missing-data handling instead of this test wrapper.
}
\examples{
\donttest{
# Simulate data with time-invariant transitions
set.seed(123)
y <- simulate_cat(200, 6, order = 1, n_categories = 2)

# Test time-invariance
test <- test_timeinvariance_cat(y, order = 1)
print(test)
}

}
\references{
Xie, Y. and Zimmerman, D. L. (2013). Antedependence models for nonstationary
categorical longitudinal data with ignorable missingness: likelihood-based
inference. \emph{Statistics in Medicine}, 32, 3274-3289.
}
\seealso{
\code{\link{fit_cat}}, \code{\link{test_order_cat}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gau_utils.R
\name{.innovation_var_gau}
\alias{.innovation_var_gau}
\title{Compute innovation variance estimates under AD(p)}
\usage{
.innovation_var_gau(y, p, mu = NULL)
}
\arguments{
\item{y}{Numeric matrix n_subjects by n_time.}

\item{p}{Antedependence order.}

\item{mu}{Optional mean vector for centering.}
}
\value{
Numeric vector of innovation variance estimates.
}
\description{
The MLE of the innovation variance at time i is RSS_i(p) / N.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_mean_gau.R
\name{test_contrast_gau}
\alias{test_contrast_gau}
\title{Test linear hypotheses on the mean under antedependence}
\usage{
test_contrast_gau(y, C, c = NULL, p = 1L)
}
\arguments{
\item{y}{Numeric matrix with n_subjects rows and n_time columns.}

\item{C}{Contrast matrix with c rows and n_time columns, where c is the
number of contrasts being tested. Rows must be linearly independent.}

\item{c}{Right-hand side vector of length equal to nrow(C). Default is
a vector of zeros.}

\item{p}{Antedependence order of the covariance structure. This is the same
order parameter named \code{order} in \code{\link{fit_gau}}.}
}
\value{
A list with class \code{gau_contrast_test} containing:
\describe{
\item{method}{Inference method used (\code{"wald"}).}
\item{C}{Contrast matrix}
\item{c}{Right-hand side vector}
\item{mu_hat}{Estimated mean vector}
\item{contrast_est}{Estimated value of C * mu}
\item{statistic}{Wald test statistic}
\item{df}{Degrees of freedom (number of contrasts)}
\item{p_value}{P-value from chi-square distribution}
}
}
\description{
Tests the null hypothesis C * mu = c for a specified contrast matrix C
and vector c, under an AD(p) covariance structure. This implements
Theorem 7.2 of Zimmerman & Núñez-Antón (2009).
}
\details{
The Wald test statistic (Theorem 7.2) is:
\deqn{(C\bar{Y} - c)^T (C \hat{\Sigma} C^T)^{-1} (C\bar{Y} - c)}

where \eqn{\hat{\Sigma}} is the REML estimator of the covariance matrix
under the AD(p) model.

Common examples include:
\itemize{
\item Testing if mean is constant: C is the first-difference matrix
\item Testing for linear trend: C tests deviations from linearity
}
}
\examples{
\donttest{
y <- simulate_gau(n_subjects = 50, n_time = 5, order = 1)

# Test if mean is constant (all differences = 0)
# C is 4x5 matrix of first differences
C <- matrix(0, nrow = 4, ncol = 5)
for (i in 1:4) {
  C[i, i] <- 1
  C[i, i+1] <- -1
}
test <- test_contrast_gau(y, C = C, p = 1)
print(test)
}

}
\references{
Zimmerman, D.L. and Núñez-Antón, V. (2009). Antedependence Models for
Longitudinal Data. Chapman & Hall/CRC. Chapter 7.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loglik_gau.R
\name{logL_gau}
\alias{logL_gau}
\title{Log-likelihood for Gaussian AD models (with missing data support)}
\usage{
logL_gau(
  y,
  order,
  mu = NULL,
  phi = NULL,
  sigma = NULL,
  blocks = NULL,
  tau = 0,
  na_action = c("fail", "complete", "marginalize")
)
}
\arguments{
\item{y}{Numeric matrix with n_subjects rows and n_time columns. May contain NA.}

\item{order}{Antedependence order, one of 0, 1, or 2.}

\item{mu}{Mean vector (length n_time).}

\item{phi}{Dependence coefficient(s). For order 1: vector of length n_time-1.
For order 2: matrix with 2 columns or vector of length 2*(n_time-2).}

\item{sigma}{Innovation standard deviations (length n_time).}

\item{blocks}{Integer vector of block membership (length n_subjects), or NULL.}

\item{tau}{Block effects, first element constrained to zero}

\item{na_action}{How to handle missing values:
\itemize{
\item \code{fail}: Error if any NA is present (default)
\item \code{complete}: Use only complete cases
\item \code{marginalize}: Compute observed-data likelihood
}}
}
\value{
Scalar log-likelihood value.
}
\description{
Computes the log-likelihood for Gaussian antedependence models of order 0, 1, or 2.
Supports missing data under MAR assumption via na_action parameter.
}
\details{
For complete data (no NA), all three na_action options give the same result.

For missing data:
\itemize{
\item marginalize: Uses MVN marginalization to compute P(Y_obs). This is
the correct observed-data likelihood for MAR missing data.
\item complete: Removes subjects with any missing values. May lose information.
\item fail: Stops with error. Useful to ensure no missing data present.
}
}
\examples{
set.seed(1)
y <- simulate_gau(n_subjects = 30, n_time = 5, order = 1, phi = 0.3)
fit <- fit_gau(y, order = 1)
logL_gau(y, order = 1, mu = fit$mu, phi = fit$phi, sigma = fit$sigma)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bic_cat.R
\name{bic_order_cat}
\alias{bic_order_cat}
\title{BIC-based order selection for categorical AD models}
\usage{
bic_order_cat(
  y,
  max_order = 2,
  blocks = NULL,
  homogeneous = TRUE,
  n_categories = NULL,
  criterion = "bic"
)
}
\arguments{
\item{y}{Integer matrix of categorical data (n_subjects x n_time).}

\item{max_order}{Maximum order to consider. Default is 2.}

\item{blocks}{Optional block membership vector.}

\item{homogeneous}{Whether to use homogeneous parameters across blocks.}

\item{n_categories}{Number of categories (inferred if NULL).}

\item{criterion}{Which criterion to use: "bic" (default) or "aic".}
}
\value{
A list containing:
\item{table}{Data frame with order, log_l, n_params, aic, bic}
\item{bic}{Named numeric vector of BIC values by order}
\item{best_order}{Order with lowest criterion value}
\item{criterion}{Criterion used for order selection (\code{"bic"} or \code{"aic"})}
\item{fits}{List of fitted models}
}
\description{
Fits AD models of increasing orders and selects the best by BIC.
}
\examples{
\donttest{
y <- simulate_cat(100, 5, order = 1, n_categories = 2)
result <- bic_order_cat(y, max_order = 2)
print(result$table)
print(result$best_order)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_cat.R
\name{.cat_theta_to_prob}
\alias{.cat_theta_to_prob}
\title{Convert unconstrained logits to probability row}
\usage{
.cat_theta_to_prob(theta_row, c)
}
\arguments{
\item{theta_row}{Numeric vector of length c-1}

\item{c}{Number of categories}
}
\value{
Probability vector of length c
}
\description{
Convert unconstrained logits to probability row
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_gau.R
\name{summary.gau_fit}
\alias{summary.gau_fit}
\title{Summary method for gau_fit objects}
\usage{
\method{summary}{gau_fit}(object, ...)
}
\arguments{
\item{object}{A \code{gau_fit} object.}

\item{...}{Unused.}
}
\value{
\code{object}, invisibly.
}
\description{
Prints a structured table of model-fit statistics and estimated parameters.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loglik_cat.R
\name{logL_cat}
\alias{logL_cat}
\title{Log-likelihood for categorical AD models (with missing data support)}
\usage{
logL_cat(
  y,
  order,
  marginal,
  transition = NULL,
  blocks = NULL,
  homogeneous = TRUE,
  n_categories = NULL,
  na_action = c("fail", "complete", "marginalize")
)
}
\arguments{
\item{y}{Integer matrix with n_subjects rows and n_time columns. Each entry
should be a category code from 1 to c.}

\item{order}{Antedependence order p. Must be 0, 1, or 2.}

\item{marginal}{List of marginal/joint probabilities for initial time points.
Structure depends on order (see Details).}

\item{transition}{List of transition probability arrays for time points
k = p+1 to n. Each element should be an array of dimension c^p x c where
the last dimension corresponds to the current time point.}

\item{blocks}{Optional integer vector of length n_subjects specifying group
membership. Required if homogeneous = FALSE.}

\item{homogeneous}{Logical. If TRUE (default), same parameters used for all
subjects. If FALSE, marginal and transition should be lists indexed by block.}

\item{n_categories}{Number of categories. If NULL, inferred from data.}

\item{na_action}{Handling of missing values in \code{y}. One of
\code{"fail"} (default, error if any missing), \code{"complete"}
(drop subjects with any missing values), or \code{"marginalize"}
(integrate over missing categorical outcomes under the AD model).}
}
\value{
Scalar log-likelihood value.
}
\description{
Evaluates the log-likelihood of an AD(p) model for categorical longitudinal
data at given parameter values.
}
\details{
The log-likelihood for AD(p) decomposes into contributions from initial
time points and transition time points.

For order 0 (independence), the log-likelihood is the sum of log marginal
probabilities at each time point.

Parameter structure for marginal:
\itemize{
\item Order 0: List with elements t1, t2, ..., tn, each a vector of length c
\item Order 1: List with element t1 (vector of length c)
\item Order 2: List with t1 (vector), t2_given_1to1 (c x c matrix)
}

Parameter structure for transition:
\itemize{
\item Order 0: Not used (NULL or empty list)
\item Order 1: List with elements t2, t3, ..., tn, each c x c matrix
\item Order 2: List with elements t3, t4, ..., tn, each c x c x c array
}
}
\examples{
set.seed(1)
y <- simulate_cat(n_subjects = 40, n_time = 5, order = 1, n_categories = 3)
fit <- fit_cat(y, order = 1, n_categories = 3)
logL_cat(
  y = y,
  order = 1,
  marginal = fit$marginal,
  transition = fit$transition,
  n_categories = 3
)

}
\references{
Xie, Y. and Zimmerman, D. L. (2013). Antedependence models for nonstationary
categorical longitudinal data with ignorable missingness: likelihood-based
inference. \emph{Statistics in Medicine}, 32, 3274-3289.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{summary.partial_corr}
\alias{summary.partial_corr}
\title{Summary Method for Partial Correlation Objects}
\usage{
\method{summary}{partial_corr}(object, ...)
}
\arguments{
\item{object}{A \code{partial_corr} object returned by
\code{\link{partial_corr}}.}

\item{...}{Currently unused.}
}
\value{
Invisibly returns a data frame with one row per lag and columns
\code{lag}, \code{n_pairs}, \code{mean_abs}, \code{min}, \code{max},
and \code{n_signif}.
}
\description{
Prints a lag-by-lag summary of intervenor-adjusted partial correlations,
reporting the mean absolute value and range at each lag together with the
number of pairs that exceed the rough significance threshold
\eqn{2/\sqrt{n}}.
}
\examples{
\donttest{
data("cattle_growth")
pc <- partial_corr(cattle_growth$y_A)
summary(pc)
}
}
\seealso{
\code{\link{partial_corr}}, \code{\link{plot.partial_corr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loglik_inad.R
\name{logL_inad_i}
\alias{logL_inad_i}
\title{INAD log likelihood contribution at time i (no fixed effect)}
\usage{
logL_inad_i(
  y,
  i,
  order = 1,
  thinning = c("binom", "pois", "nbinom"),
  innovation = c("pois", "bell", "nbinom"),
  alpha,
  theta,
  nb_inno_size = NULL
)
}
\arguments{
\item{y}{Integer matrix n_sub by n_time.}

\item{i}{Time index in 1..ncol(y).}

\item{order}{Integer in \{0, 1, 2\}.}

\item{thinning}{One of "binom", "pois", "nbinom".}

\item{innovation}{One of "pois", "bell", "nbinom".}

\item{alpha}{Thinning parameters. For order 1, numeric length 1 or n_time.
For order 2, either a matrix n_time by 2 or a list(alpha1, alpha2).}

\item{theta}{Innovation parameter at time i, or a vector length 1 or n_time.
For Poisson and negative binomial innovations, this is the innovation
mean. For Bell innovations, this is the Bell rate parameter
(mean \eqn{\theta e^\theta}).}

\item{nb_inno_size}{Size parameter for innovation "nbinom". Numeric length 1 or n_time.}
}
\value{
A scalar log likelihood contribution for time i.
}
\description{
Returns the time i contribution, summed over subjects, for the no fixed effect model.
}
\examples{
set.seed(1)
y <- simulate_inad(
  n_subjects = 50,
  n_time = 5,
  order = 1,
  thinning = "binom",
  innovation = "pois",
  alpha = 0.3,
  theta = 2
)
fit <- fit_inad(y, order = 1, thinning = "binom", innovation = "pois", max_iter = 20)
logL_inad_i(
  y = y,
  i = 3,
  order = 1,
  thinning = "binom",
  innovation = "pois",
  alpha = fit$alpha,
  theta = fit$theta
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bell.R
\name{Bz}
\alias{Bz}
\title{Bell numbers}
\usage{
Bz(z)
}
\arguments{
\item{z}{nonnegative integer giving the maximum order.}
}
\value{
numeric vector of length \code{z + 1} containing B_0, ..., B_z.
}
\description{
Compute Bell numbers up to order z using a truncated series representation.
}
\details{
This function is for internal use. The results are stored in \code{.Bell_num}
and used by the Bell distribution functions.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{fitted.gau_fit}
\alias{fitted.gau_fit}
\alias{residuals.gau_fit}
\title{Fitted values and residuals for Gaussian AD fits}
\usage{
\method{fitted}{gau_fit}(object, y, ...)

\method{residuals}{gau_fit}(object, y, ...)
}
\arguments{
\item{object}{A \code{gau_fit} object (complete-data fit only).}

\item{y}{The original data matrix used to produce \code{object}
(n_subjects \eqn{\times} n_time).  Required because the fit object
does not store the raw data.}

\item{...}{Unused.}
}
\value{
A numeric matrix of the same dimensions as \code{y}.
}
\description{
Computes fitted (conditional mean) values and residuals for a
complete-data Gaussian antedependence fit.  The fitted value for subject
\eqn{s} at time \eqn{t} is the conditional mean
\eqn{E[Y_{st} | Y_{s,t-p}, \ldots, Y_{s,t-1}]}.
}
\examples{
\donttest{
set.seed(1)
y <- simulate_gau(n_subjects = 40, n_time = 5, order = 1, phi = 0.4)
fit <- fit_gau(y, order = 1)
yhat <- fitted(fit, y)
resid <- residuals(fit, y)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_order_inad.R
\name{test_order_inad}
\alias{test_order_inad}
\title{Likelihood ratio test for antedependence order (INAD data)}
\usage{
test_order_inad(
  y,
  order_null = 0,
  order_alt = 1,
  thinning = "binom",
  innovation = "pois",
  blocks = NULL,
  use_chibar = TRUE,
  weights = NULL,
  fit_null = NULL,
  fit_alt = NULL,
  ...
)
}
\arguments{
\item{y}{Integer matrix with n_subjects rows and n_time columns.}

\item{order_null}{Order under null hypothesis (0 or 1).}

\item{order_alt}{Order under alternative hypothesis (1 or 2). Must be order_null + 1.}

\item{thinning}{Thinning operator: "binom", "pois", or "nbinom".}

\item{innovation}{Innovation distribution: "pois", "bell", or "nbinom".}

\item{blocks}{Optional integer vector for block effects.}

\item{use_chibar}{Logical; if TRUE, use chi-bar-square for boundary test.}

\item{weights}{Optional weights for chi-bar-square mixture.}

\item{fit_null}{Optional pre-computed null fit.}

\item{fit_alt}{Optional pre-computed alternative fit.}

\item{...}{Additional arguments passed to fit_inad.}
}
\value{
A list with class \code{"test_order_inad"} containing:
\describe{
\item{method}{Inference method used (\code{"lrt"}).}
\item{fit_null}{Fitted model under H0}
\item{fit_alt}{Fitted model under H1}
\item{statistic}{Test statistic value}
\item{lrt_stat}{Likelihood ratio test statistic}
\item{df}{Degrees of freedom}
\item{p_value}{Chi-square p-value}
\item{p_value_chibar}{Chi-bar-square p-value (if \code{use_chibar = TRUE})}
\item{bic_null}{BIC under H0}
\item{bic_alt}{BIC under H1}
\item{bic_selected}{Which model BIC prefers}
\item{table}{Two-row model comparison table}
\item{settings}{Input and derived settings for the test}
}
}
\description{
Performs a likelihood ratio test comparing INAD models of different orders.
}
\details{
The test compares nested INAD models of orders \code{order_null} and
\code{order_alt = order_null + 1} using:
\deqn{\lambda = 2(\ell_{alt} - \ell_{null})}
where \eqn{\ell_{null}} and \eqn{\ell_{alt}} are maximized log-likelihoods
under the null and alternative models.

The default p-value uses the chi-square approximation with degrees of freedom
matching the number of additional dependence parameters introduced under the
higher-order model. When \code{use_chibar = TRUE}, a chi-bar-square mixture
p-value is also reported for boundary-aware inference.

Missing-data inputs are supported through the same \code{na_action} options
available in \code{\link{fit_inad}}. If \code{y} has missing values and
\code{na_action} is not supplied via \code{...}, this function defaults to
\code{na_action = "marginalize"}.
}
\examples{
set.seed(1)
y <- simulate_inad(
  n_subjects = 40,
  n_time = 5,
  order = 1,
  thinning = "binom",
  innovation = "pois",
  alpha = 0.3,
  theta = 2
)
out <- test_order_inad(
  y,
  order_null = 0,
  order_alt = 1,
  thinning = "binom",
  innovation = "pois",
  max_iter = 20
)
out$statistic
}
\references{
Li, C. and Zimmerman, D.L. (2026). Integer-valued antedependence models for
longitudinal count data. \emph{Biostatistics}.
}
\seealso{
\code{\link{fit_inad}}, \code{\link{bic_order_inad}},
\code{\link{test_stationarity_inad}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cat_utils.R
\name{.validate_y_cat}
\alias{.validate_y_cat}
\title{Validate categorical data matrix}
\usage{
.validate_y_cat(y, n_categories = NULL, allow_na = FALSE)
}
\arguments{
\item{y}{Data matrix}

\item{n_categories}{Number of categories (NULL to infer)}

\item{allow_na}{Logical; if TRUE, allow missing values.}
}
\value{
List with validated y and n_categories
}
\description{
Validate categorical data matrix
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ci_inad.R
\name{.ci_alpha_theta_louis_fe}
\alias{.ci_alpha_theta_louis_fe}
\title{Compute alpha and theta CIs using Louis' identity (general version)}
\usage{
.ci_alpha_theta_louis_fe(
  y,
  fit,
  blocks,
  level,
  idx_time,
  ridge,
  thinning,
  innovation
)
}
\description{
Compute alpha and theta CIs using Louis' identity (general version)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ci_cat.R
\name{.compute_transition_ci}
\alias{.compute_transition_ci}
\title{Compute CIs for transition parameters}
\usage{
.compute_transition_ci(transition, cell_counts, p, c, n_time, z, level)
}
\arguments{
\item{transition}{Transition parameter list from fit}

\item{cell_counts}{Cell counts from fit}

\item{p}{Order}

\item{c}{Number of categories}

\item{n_time}{Number of time points}

\item{z}{Z critical value}

\item{level}{Confidence level}
}
\value{
List of data frames with CIs for each time point
}
\description{
Compute CIs for transition parameters
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_gau.R
\name{fit_gau}
\alias{fit_gau}
\title{Fit Gaussian antedependence model by maximum likelihood}
\usage{
fit_gau(
  y,
  order = 1,
  blocks = NULL,
  na_action = c("fail", "complete", "em"),
  estimate_mu = TRUE,
  em_max_iter = 100,
  em_tol = 1e-06,
  em_verbose = FALSE,
  ...
)
}
\arguments{
\item{y}{Numeric matrix (n_subjects x n_time). May contain NA.}

\item{order}{Integer 0, 1, or 2.}

\item{blocks}{Optional vector of block membership (length n_subjects).}

\item{na_action}{One of "fail", "complete", or "em".}

\item{estimate_mu}{Logical, whether to estimate mu (default TRUE).}

\item{em_max_iter}{Maximum EM iterations (only used when na_action = "em").}

\item{em_tol}{EM convergence tolerance (only used when na_action = "em").}

\item{em_verbose}{Logical, print EM progress (only used when na_action = "em").}

\item{...}{Passed through to the EM fitter.}
}
\value{
A list with components including mu, phi, sigma, tau, log_l, n_obs, n_missing.
}
\description{
Fits an AD(0), AD(1), or AD(2) model for Gaussian longitudinal data
by maximum likelihood. Missing values can be handled by complete-case
deletion or by EM (see \code{\link{em_gau}} for an explicit EM wrapper).
}
\details{
For missing data with \code{na_action = "em"}, AD orders 0 and 1 are the
primary production path. AD order 2 is available, but the current EM
implementation uses simplified second-order updates and should be treated as
provisional for high-stakes inference.

For observed-data likelihood evaluation under MAR without fitting, use
\code{\link{logL_gau}} with \code{na_action = "marginalize"}. In contrast,
\code{fit_gau} handles missingness via complete-case fitting
(\code{na_action = "complete"}) or EM (\code{na_action = "em"}).
}
\examples{
set.seed(1)
y <- simulate_gau(n_subjects = 30, n_time = 5, order = 1, phi = 0.3)
fit <- fit_gau(y, order = 1)
fit$log_l

y_miss <- y
y_miss[1, 2] <- NA
fit_em <- fit_gau(y_miss, order = 1, na_action = "em", em_max_iter = 20)
fit_em$settings$na_action

}
\seealso{
\code{\link{em_gau}},
\code{\link{fit_cat}}, \code{\link{fit_inad}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cat_utils.R
\name{.get_combinations_cat}
\alias{.get_combinations_cat}
\title{Get all category combinations of given length}
\usage{
.get_combinations_cat(n_categories, length)
}
\arguments{
\item{n_categories}{Number of categories c}

\item{length}{Number of positions}
}
\value{
Matrix where each row is a combination (c^length rows, length columns)
}
\description{
Get all category combinations of given length
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_cat.R
\name{.extract_conditional_probs}
\alias{.extract_conditional_probs}
\title{Extract conditional probabilities from transition array}
\usage{
.extract_conditional_probs(trans_array, history)
}
\arguments{
\item{trans_array}{Transition array with last dimension being outcome}

\item{history}{Vector of conditioning values}
}
\value{
Vector of probabilities for current time point
}
\description{
Extract conditional probabilities from transition array
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_order_gau.R
\name{print.gau_order_test}
\alias{print.gau_order_test}
\title{Print method for AD order test}
\usage{
\method{print}{gau_order_test}(x, ...)
}
\arguments{
\item{x}{Object of class \code{gau_order_test}.}

\item{...}{Unused.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Print method for AD order test
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ci_inad.R
\name{ci_inad}
\alias{ci_inad}
\title{Confidence intervals for fitted INAD models}
\usage{
ci_inad(
  y,
  fit,
  blocks = NULL,
  level = 0.95,
  idx_time = NULL,
  ridge = 0,
  profile_maxeval = 2500,
  profile_xtol_rel = 1e-06
)
}
\arguments{
\item{y}{Integer matrix with \code{n_subjects} rows and \code{n_time} columns.}

\item{fit}{A fitted model object returned by \code{\link{fit_inad}}.}

\item{blocks}{Optional integer vector of length \code{n_subjects}. Required for
block effect intervals. If provided, should match \code{fit$settings$blocks}.}

\item{level}{Confidence level between 0 and 1.}

\item{idx_time}{Optional integer vector of time indices for which to compute
intervals. Default is all time points.}

\item{ridge}{Nonnegative ridge value added to the observed information matrix
used for Louis based Wald intervals.}

\item{profile_maxeval}{Maximum number of function evaluations used in the
profile likelihood refits.}

\item{profile_xtol_rel}{Relative tolerance used in the profile likelihood refits.}
}
\value{
An object of class \code{inad_ci}, a list with elements
\code{settings}, \code{level}, \code{alpha}, \code{theta}, \code{nb_inno_size},
and \code{tau}. Each non NULL interval element is a data frame with columns
\code{param}, \code{est}, \code{lower}, \code{upper}, and possibly \code{se}
and \code{width}.
}
\description{
Computes confidence intervals for selected parameters from a fitted INAD model.
For the fixed effect case, Wald intervals for time varying alpha and theta are
computed via Louis identity for supported thinning-innovation combinations.
For block effects tau, profile likelihood intervals are computed by fixing one
component of tau and re maximizing the log likelihood over nuisance parameters.
For negative binomial innovations, Wald intervals for the innovation size
parameter are computed using a one dimensional observed information approximation
per time point, holding other parameters fixed at their fitted values.
}
\examples{
\donttest{
data("bolus_inad", package = "antedep")
y <- bolus_inad$y
blocks <- bolus_inad$blocks
fit <- fit_inad(y, order = 1, thinning = "binom", innovation = "bell", blocks = blocks)
ci <- ci_inad(y, fit, blocks = blocks)
ci$alpha
ci$theta
ci$tau
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-labor_force_cat.R
\docType{data}
\name{labor_force_cat}
\alias{labor_force_cat}
\title{Labor force longitudinal categorical data (Table 1)}
\format{
A list with five components:
\describe{
\item{y}{integer matrix of dimension N by 5 containing expanded subject-level sequences}
\item{counts}{data frame with columns Y1, Y2, Y3, Y4, Y5, Count}
\item{n_categories}{number of categories (2)}
\item{time}{integer vector of calendar years (1967:1971)}
\item{status_labels}{character vector c("employed", "unemployed")}
}
}
\source{
Table 1 (labor-force example) from:
Xie, Y. and Zimmerman, D. L. (2013).
Score and Wald tests for antedependence in categorical longitudinal data.
}
\usage{
labor_force_cat
}
\description{
Five-year employment-status sequences reconstructed from Table 1 in the
labor-force example used in Xie and Zimmerman score/Wald antedependence
testing work. Category coding is 1 = employed, 2 = unemployed.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_cat.R
\name{.fit_cat_single_pop_marginalize}
\alias{.fit_cat_single_pop_marginalize}
\title{Fit CAT model with missing data via observed-data likelihood optimization}
\usage{
.fit_cat_single_pop_marginalize(y, p, c, n_time)
}
\arguments{
\item{y}{Data matrix for one population (may contain NA)}

\item{p}{Order}

\item{c}{Number of categories}

\item{n_time}{Number of time points}
}
\value{
List with marginal, transition, cell_counts, log_l, convergence
}
\description{
Fit CAT model with missing data via observed-data likelihood optimization
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inad_utils.R
\name{.thin_vec}
\alias{.thin_vec}
\title{Thinning probability vector}
\usage{
.thin_vec(k_vals, yprev, a, thinning)
}
\arguments{
\item{k_vals}{Integer vector of possible thinned values.}

\item{yprev}{Previous count (size parameter for thinning).}

\item{a}{Thinning parameter alpha.}

\item{thinning}{One of "binom", "pois", "nbinom".}
}
\value{
Numeric vector of probabilities.
}
\description{
Computes the probability mass for thinned counts.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_gau_em.R
\name{.em_m_step_gau}
\alias{.em_m_step_gau}
\title{M-step: Update parameters from sufficient statistics}
\usage{
.em_m_step_gau(suff_stats, order, blocks, estimate_mu, n_subjects, n_time)
}
\description{
M-step: Update parameters from sufficient statistics
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gau_utils.R
\name{.rss_two_sample_gau}
\alias{.rss_two_sample_gau}
\title{Compute RSS for two-sample case (pooled and separate)}
\usage{
.rss_two_sample_gau(y, blocks, i, p, pooled = TRUE)
}
\arguments{
\item{y}{Numeric matrix n_subjects by n_time.}

\item{blocks}{Integer vector indicating group membership (1 or 2).}

\item{i}{Time index.}

\item{p}{Antedependence order.}

\item{pooled}{Logical. If TRUE, compute pooled RSS; if FALSE, within-group RSS.}
}
\value{
RSS value.
}
\description{
Compute RSS for two-sample case (pooled and separate)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_mean_gau.R
\name{.construct_gau_covariance}
\alias{.construct_gau_covariance}
\title{Construct AD covariance matrix from fitted model}
\usage{
.construct_gau_covariance(fit, n_time)
}
\arguments{
\item{fit}{Fitted AD model from fit_gau().}

\item{n_time}{Number of time points.}
}
\value{
Covariance matrix.
}
\description{
Construct AD covariance matrix from fitted model
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/em_inad.R
\name{print.inad_fit}
\alias{print.inad_fit}
\title{Print method for INAD model fits}
\usage{
\method{print}{inad_fit}(x, digits = 4, ...)
}
\arguments{
\item{x}{An object of class \code{inad_fit}.}

\item{digits}{Number of digits to print.}

\item{...}{Unused.}
}
\value{
The input object, invisibly.
}
\description{
Print method for INAD model fits
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gau_utils.R
\name{.mle_mean_gau}
\alias{.mle_mean_gau}
\title{Compute MLE of mean vector under AD(p)}
\usage{
.mle_mean_gau(y)
}
\arguments{
\item{y}{Numeric matrix n_subjects by n_time.}
}
\value{
Numeric vector of mean estimates.
}
\description{
Under AD(p), the MLE of mu is the sample mean vector when there are no
covariates.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loglik_cat.R
\name{.logL_subject_cat_marginalize_p1}
\alias{.logL_subject_cat_marginalize_p1}
\title{Observed-data log-likelihood for order-1 CAT model}
\usage{
.logL_subject_cat_marginalize_p1(y_s, c, marginal, transition)
}
\arguments{
\item{y_s}{Subject row (may contain NA)}

\item{c}{Number of categories}

\item{marginal}{Marginal list}

\item{transition}{Transition list}
}
\value{
Scalar log-likelihood contribution
}
\description{
Observed-data log-likelihood for order-1 CAT model
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/em_gau.R
\name{em_gau}
\alias{em_gau}
\title{EM algorithm for Gaussian AD model estimation}
\usage{
em_gau(
  y,
  order = 1,
  blocks = NULL,
  estimate_mu = TRUE,
  max_iter = 100,
  tol = 1e-06,
  verbose = FALSE,
  ...
)
}
\arguments{
\item{y}{Numeric matrix (n_subjects x n_time), may contain NA.}

\item{order}{Integer 0, 1, or 2.}

\item{blocks}{Optional vector of block membership (length n_subjects).}

\item{estimate_mu}{Logical, whether to estimate mu (default TRUE).}

\item{max_iter}{Maximum EM iterations.}

\item{tol}{EM convergence tolerance.}

\item{verbose}{Logical, print EM progress.}

\item{...}{Additional arguments passed to \code{\link{fit_gau}}.}
}
\value{
An \code{gau_fit} object as returned by \code{\link{fit_gau}}.
}
\description{
Convenience wrapper around \code{\link{fit_gau}} with
\code{na_action = "em"} to provide a parallel entry point to
\code{\link{em_inad}}.
}
\details{
This is an alias-style helper for users who prefer explicit \code{em_*}
entry points across model families.
}
\examples{
set.seed(1)
y <- simulate_gau(n_subjects = 35, n_time = 5, order = 1, phi = 0.3)
y[sample(length(y), 8)] <- NA
fit <- em_gau(y, order = 1, max_iter = 20, tol = 1e-5)
fit$settings$na_action

}
\seealso{
\code{\link{fit_gau}}, \code{\link{em_inad}}, \code{\link{em_cat}},
\code{\link{fit_cat}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ci_inad.R
\name{.ci_wald_i_louis_fe}
\alias{.ci_wald_i_louis_fe}
\title{Wald CI at time i using Louis' identity}
\usage{
.ci_wald_i_louis_fe(
  y,
  i,
  alpha_hat,
  theta_hat,
  tau_hat,
  blocks,
  thinning,
  innovation,
  nb_inno_size = NULL,
  level = 0.95,
  ridge = 0
)
}
\description{
Wald CI at time i using Louis' identity
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loglik_inad.R
\name{.inad_subject_ll_order2}
\alias{.inad_subject_ll_order2}
\title{Subject-level observed-data likelihood for INAD(2)}
\usage{
.inad_subject_ll_order2(y_s, init_pmf, trans_t2, order2_info, K)
}
\description{
Subject-level observed-data likelihood for INAD(2)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ci_gau.R
\name{print.gau_ci}
\alias{print.gau_ci}
\title{Print method for AD confidence intervals}
\usage{
\method{print}{gau_ci}(x, ...)
}
\arguments{
\item{x}{An object of class \code{gau_ci}.}

\item{...}{Unused.}
}
\value{
The input object, invisibly.
}
\description{
Print method for AD confidence intervals
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_homogeneity_inad.R
\name{run_homogeneity_tests_inad}
\alias{run_homogeneity_tests_inad}
\title{Run all homogeneity tests for INAD}
\usage{
run_homogeneity_tests_inad(
  y,
  blocks,
  order = 1,
  thinning = "binom",
  innovation = "pois",
  ...
)
}
\arguments{
\item{y}{Integer matrix with n_subjects rows and n_time columns.}

\item{blocks}{Integer vector of length n_subjects specifying group membership.}

\item{order}{Antedependence order (0, 1, or 2).}

\item{thinning}{Thinning operator: "binom", "pois", or "nbinom".}

\item{innovation}{Innovation distribution: "pois", "bell", or "nbinom".}

\item{...}{Additional arguments passed to \code{fit_inad}.}
}
\value{
A list with class \code{"homogeneity_tests_inad"} containing results
for all three tests and a summary table.
}
\description{
Convenience function to run all three homogeneity tests at once and
return a summary.
}
\examples{
\donttest{
data("bolus_inad")
tests <- run_homogeneity_tests_inad(bolus_inad$y, bolus_inad$blocks,
                                     order = 1, thinning = "nbinom",
                                     innovation = "bell")
print(tests)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loglik_inad.R
\name{.inad_subject_ll_order1}
\alias{.inad_subject_ll_order1}
\title{Subject-level observed-data likelihood for INAD(1)}
\usage{
.inad_subject_ll_order1(y_s, init_pmf, trans1, K)
}
\description{
Subject-level observed-data likelihood for INAD(1)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrt_stationarity_cat.R
\name{run_stationarity_tests_cat}
\alias{run_stationarity_tests_cat}
\title{Run all stationarity-related tests for categorical AD}
\usage{
run_stationarity_tests_cat(
  y,
  order = 1,
  blocks = NULL,
  homogeneous = TRUE,
  n_categories = NULL,
  test = c("lrt", "score", "mlrt")
)
}
\arguments{
\item{y}{Integer matrix with n_subjects rows and n_time columns. Each entry
should be a category code from 1 to c.}

\item{order}{Antedependence order p. Default is 1.}

\item{blocks}{Optional integer vector of length n_subjects specifying group
membership.}

\item{homogeneous}{Logical. If TRUE (default), parameters are shared across
all groups.}

\item{n_categories}{Number of categories. If NULL, inferred from data.}

\item{test}{Type of test statistic. One of \code{"lrt"} (default),
\code{"score"}, or \code{"mlrt"}.}
}
\value{
A list containing:
\describe{
\item{time_invariance}{Result of test_timeinvariance_cat}
\item{stationarity}{Result of test_stationarity_cat}
\item{table}{Summary data frame}
}
}
\description{
Performs tests for time-invariance and stationarity constraints. For
\code{order = 1}, the stationarity test corresponds to strict stationarity;
for \code{order > 1}, it tests marginal-constancy plus time-invariant
transitions.
Currently supports complete data only.
}
\examples{
\donttest{
y <- simulate_cat(200, 6, order = 1, n_categories = 2)
result <- run_stationarity_tests_cat(y, order = 1)
print(result$table)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bic_cat.R, R/bic_gau.R, R/bic_inad.R
\name{bic_cat}
\alias{bic_cat}
\alias{aic_cat}
\alias{bic_gau}
\alias{aic_gau}
\alias{bic_inad}
\alias{aic_inad}
\title{Bayesian and Akaike Information Criteria for Antedependence Fits}
\usage{
bic_cat(fit, n_subjects = NULL)

aic_cat(fit)

bic_gau(fit, n_subjects = NULL)

aic_gau(fit)

bic_inad(fit, n_subjects = NULL)

aic_inad(fit)
}
\arguments{
\item{fit}{A fitted model object returned by \code{\link{fit_gau}},
\code{\link{fit_cat}}, or \code{\link{fit_inad}}.}

\item{n_subjects}{Number of subjects, typically \code{nrow(y)}. If
\code{NULL}, inferred from \code{fit$settings$n_subjects}.}
}
\value{
A numeric scalar BIC or AIC value.
}
\description{
Compute BIC or AIC for a fitted Gaussian, categorical, or INAD
antedependence model. Since \code{logLik()} methods are registered for all three
fit classes, \code{stats::AIC()} and \code{stats::BIC()} generics also work directly
and are preferred.
}
\details{
The BIC is computed as:
\deqn{BIC = -2 \times \ell + k \times \log(N)}
where \eqn{\ell} is the log-likelihood, \eqn{k} is the number of free parameters,
and \eqn{N} is the number of subjects.

The AIC is computed as:
\deqn{AIC = -2 \times \ell + 2k}
}
\examples{
set.seed(1)
y <- simulate_gau(n_subjects = 30, n_time = 5, order = 1, phi = 0.3)
fit <- fit_gau(y, order = 1)
bic_gau(fit, n_subjects = nrow(y))
aic_gau(fit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gau_utils.R
\name{.partial_corr_matrix_gau}
\alias{.partial_corr_matrix_gau}
\title{Compute matrix of intervenor-adjusted partial correlations}
\usage{
.partial_corr_matrix_gau(y, mu = NULL)
}
\arguments{
\item{y}{Numeric matrix n_subjects by n_time.}

\item{mu}{Optional mean vector for centering.}
}
\value{
Matrix where entry (i,j) for i > j is r_(i,j|(j+1:i-1)).
}
\description{
Compute matrix of intervenor-adjusted partial correlations
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bell.R
\name{Bell}
\alias{Bell}
\alias{dbell}
\alias{pbell}
\alias{rbell}
\alias{qbell}
\title{The Bell distribution}
\usage{
dbell(x, theta, log = FALSE)

pbell(x, theta)

rbell(n, theta, max_z = 100L)

qbell(p, theta, max_z = 100L)
}
\arguments{
\item{x}{vector of nonnegative integers (for \code{dbell} and \code{pbell}).}

\item{theta}{scalar nonnegative Bell parameter.}

\item{log}{logical; if TRUE, probabilities p are given as log(p).}

\item{n}{number of observations to generate (for \code{rbell}).}

\item{max_z}{maximum support value used for approximation in
\code{rbell} and \code{qbell}.}

\item{p}{numeric vector of probabilities between 0 and 1 inclusive (for \code{qbell}).}
}
\value{
For \code{dbell}, a numeric vector of probabilities.
For \code{pbell}, a numeric vector of cumulative probabilities.
For \code{qbell}, an integer vector of quantiles.
For \code{rbell}, an integer vector of random values.
}
\description{
Density, distribution function, quantile function and random generation
for the Bell distribution with parameter \code{theta}.
}
\details{
Let \eqn{B_x} denote the xth Bell number. The Bell distribution has
probability mass function
\deqn{
P(X = x) = \theta^x \exp(-\exp(\theta) + 1) \frac{B_x}{x!},
}
for nonnegative integers \eqn{x} and \eqn{\theta \ge 0}.

For \eqn{\theta > 0}, the Bell mean is \eqn{E[X] = \theta e^\theta}.
At \eqn{\theta = 0}, the distribution is degenerate at 0.

The functions follow the standard naming used in base R:
\code{dbell} for the density, \code{pbell} for the distribution function,
\code{qbell} for the quantile function and \code{rbell} for random
generation.
}
\examples{
dbell(0:5, theta = 1)
pbell(0:5, theta = 1)
qbell(c(0.25, 0.5, 0.9), theta = 1)
set.seed(1)
rbell(10, theta = 1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bic_inad.R
\name{bic_inad}
\alias{bic_inad}
\title{Bayesian information criterion for fitted INAD models}
\usage{
bic_inad(fit, n_subjects = NULL)
}
\arguments{
\item{fit}{A fitted model object returned by \code{\link{fit_inad}}.}

\item{n_subjects}{Number of subjects, typically \code{nrow(y)}. If
\code{NULL}, inferred from \code{fit$settings$n_subjects} or legacy
\code{length(fit$settings$blocks)} when available (with a warning).}
}
\value{
A numeric scalar BIC value.
}
\description{
Computes BIC using the fitted log likelihood and a parameter count that
respects structural zeros and identifiability constraints.
}
\details{
The BIC is computed as:
\deqn{BIC = -2 \times \ell + k \times \log(N)}
where \eqn{\ell} is the log-likelihood, \eqn{k} is the number of free parameters,
and \eqn{N} is the number of subjects.
}
\examples{
set.seed(1)
y <- simulate_inad(
  n_subjects = 40,
  n_time = 5,
  order = 1,
  thinning = "binom",
  innovation = "pois",
  alpha = 0.3,
  theta = 2
)
fit <- fit_inad(y, order = 1, thinning = "binom", innovation = "pois", max_iter = 20)
bic_inad(fit, n_subjects = nrow(y))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ci_inad.R
\name{.louis_info_i_fe}
\alias{.louis_info_i_fe}
\title{Louis' identity observed information at time i}
\usage{
.louis_info_i_fe(
  y,
  i,
  alpha_hat,
  theta_hat,
  tau_hat,
  blocks,
  thinning,
  innovation,
  nb_inno_size = NULL
)
}
\description{
Louis' identity observed information at time i
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ci_gau.R
\name{ci_gau}
\alias{ci_gau}
\title{Confidence intervals for fitted Gaussian AD models}
\usage{
ci_gau(fit, level = 0.95, parameters = "all")
}
\arguments{
\item{fit}{A fitted model object returned by \code{\link{fit_gau}}.}

\item{level}{Confidence level between 0 and 1.}

\item{parameters}{Which parameters to include: \code{"all"} (default),
\code{"mu"}, \code{"phi"}, or \code{"sigma"}.}
}
\value{
An object of class \code{gau_ci}, a list with elements
\code{settings}, \code{level}, \code{mu}, \code{phi}, and \code{sigma}.
Each non-NULL element is a data frame with columns \code{param},
\code{est}, \code{se}, \code{lower}, \code{upper}, and \code{level}.
}
\description{
Computes approximate Wald confidence intervals for selected parameters from a
fitted Gaussian AD model.
}
\details{
This helper currently supports complete-data Gaussian AD fits.

Standard errors are based on large-sample approximations:
\itemize{
\item \eqn{SE(\hat{\mu}_t) \approx \hat{\sigma}_t / \sqrt{n}}
\item \eqn{SE(\hat{\sigma}_t) \approx \hat{\sigma}_t / \sqrt{2n}}
\item \eqn{SE(\hat{\phi}) \approx \sqrt{(1-\hat{\phi}^2)/n}} for free \eqn{\phi} entries
}
}
\examples{
\donttest{
y <- simulate_gau(n_subjects = 80, n_time = 6, order = 1, phi = 0.4)
fit <- fit_gau(y, order = 1)
ci <- ci_gau(fit)
ci$mu
ci$phi
ci$sigma
}

}
\seealso{
\code{\link{fit_gau}}, \code{\link{ci_cat}}, \code{\link{ci_inad}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loglik_inad.R
\name{.inad_state_max}
\alias{.inad_state_max}
\title{Heuristic state-space bound for INAD missing-data recursion}
\usage{
.inad_state_max(y, order, alpha1, alpha2, theta, tau, innovation = "pois")
}
\description{
Heuristic state-space bound for INAD missing-data recursion
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_methods.R
\name{plot.gau_fit}
\alias{plot.gau_fit}
\title{Plot estimated parameters of a Gaussian AD fit}
\usage{
\method{plot}{gau_fit}(x, ...)
}
\arguments{
\item{x}{A \code{gau_fit} object from \code{\link{fit_gau}}.}

\item{...}{Unused; present for S3 consistency.}
}
\value{
\code{x}, invisibly.
}
\description{
Produces a multi-panel base-graphics plot showing the estimated mean
(\eqn{\mu}), innovation standard deviation (\eqn{\sigma}), and (for
AD(1) or AD(2)) the antedependence coefficients (\eqn{\phi}) over time.
}
\examples{
\donttest{
set.seed(1)
y <- simulate_gau(n_subjects = 40, n_time = 6, order = 1, phi = 0.4)
fit <- fit_gau(y, order = 1)
plot(fit)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_methods.R
\name{plot.partial_corr}
\alias{plot.partial_corr}
\title{Heatmap plot for a partial_corr object}
\usage{
\method{plot}{partial_corr}(x, ...)
}
\arguments{
\item{x}{A \code{partial_corr} object from \code{\link{partial_corr}}.}

\item{...}{Unused.}
}
\value{
\code{x}, invisibly.
}
\description{
Displays the partial correlation matrix as a colour heatmap using base
graphics.  Upper triangle shows ordinary correlations; lower triangle
shows intervenor-adjusted partial correlations; the diagonal shows
standardised variances.
}
\examples{
\donttest{
data("bolus_inad")
pc <- partial_corr(bolus_inad$y)
plot(pc)
}

}
